// Import required java libraries
import java.io.*;
import java.net.ConnectException;
import java.util.*;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.output.*;
import org.ccnx.ccn.config.ConfigurationException;
import org.ccnx.ccn.io.RepositoryOutputStream;
import org.ccnx.ccn.protocol.ContentName;
import org.ccnx.ccn.protocol.MalformedContentNameStringException;

import org.ccnx.ccn.CCNHandle;
import org.ccnx.ccn.io.RepositoryOutputStream;
import org.ccnx.ccn.protocol.ContentName;

public class UploadServlet extends HttpServlet {

	private String CONET_STORAGE_PATH;
	private String CONET_STORAGE_TMP_PATH;
	private boolean isMultipart;
	private int maxFileSize = 10000000;
	private int maxMemSize = 4 * 1024;
	private File file ;
	private CCNHandle ccn_handle;
	private String ccnPrefix;
	private boolean ccnxOK = true;
	//private String regFile = null;
	
	public void init( )
	{
		//regFile = System.getProperty("user.home") + "/editToRegFile";
		// java.io.tmpdir in Tomcat is set to $CATALINA_BASE/temp. You can change it by setting the $CATALINA_TMPDIR env
		// before running startup.sh
		CONET_STORAGE_PATH = System.getProperty("java.io.tmpdir") + "/CONET/Storage/";
		CONET_STORAGE_TMP_PATH = System.getProperty("java.io.tmpdir") + "/CONET/Storage/tmp";
		new File(CONET_STORAGE_PATH).mkdirs();
		new File(CONET_STORAGE_TMP_PATH).mkdirs();
		
		
		try {
			// change the key-id locator from "/ccnx.org/Users/tomcat/" to "/ccnx.org/Users/sAS3/"
			File node_properties = new File("/tmp/node.properties");
			if(node_properties.exists()){
				Properties node_p = new Properties();
				node_p.load(new FileInputStream(node_properties));
				String key_locator = node_p.getProperty("Mykey");
				String[] key_locator_components = key_locator.split("/");
				String peer_name = key_locator_components[key_locator_components.length -1];
				System.out.println("Changing \"org.ccnx.config.UserName\" to " + peer_name);
				System.setProperty("org.ccnx.config.UserName", peer_name);
			}
			
			ccn_handle = CCNHandle.open();
		} catch (ConfigurationException e) {
			ccnxOK = false;
			System.out.println("Configuration exception during ccn_handle opening");
		} catch (ConnectException e) {
			ccnxOK = false;
			System.out.println("Connection exception during ccn_handle opening");
		} catch (IOException e) {
			ccnxOK = false;
			System.out.println("IO exception during ccn_handle opening");
		}
		
		ccnPrefix = "ccnx:/";
	}

	public void doPost(HttpServletRequest request, 
			HttpServletResponse response)
	throws ServletException, java.io.IOException {
		// Check that we have a file upload request
		isMultipart = ServletFileUpload.isMultipartContent(request);

		response.setContentType("text/html");
		java.io.PrintWriter out = response.getWriter( );

		if( !isMultipart ){
			out.println("<html>");
			out.println("<head>");
			out.println("<title>Servlet upload</title>");  
			out.println("</head>");
			out.println("<body>");
			out.println("<p>No named-data stored</p>"); 
			out.println("</body>");
			out.println("</html>");
			return;
		}

		DiskFileItemFactory factory = new DiskFileItemFactory();
		// maximum size that will be stored in memory
		factory.setSizeThreshold(maxMemSize);
		// Location to save data that is larger than maxMemSize.
		factory.setRepository(new File(CONET_STORAGE_TMP_PATH));

		// Create a new file upload handler
		ServletFileUpload upload = new ServletFileUpload(factory);
		// maximum file size to be uploaded.
		upload.setSizeMax( maxFileSize );

		try{ 
			// Parse the request to get file items.
			List fileItems = upload.parseRequest(request);

			// Process the uploaded file items
			Iterator i = fileItems.iterator();

			out.println("<html>");
			out.println("<head>");
			out.println("<title>Servlet upload</title>");  
			out.println("</head>");
			out.println("<body>");

			String nid = new String();  
			while ( i.hasNext () ) 
			{
				// NID filed is required to be sent before the actual file; indeed NID defines the local file name 
				FileItem fi = (FileItem)i.next();
				if ( fi.isFormField () ) {	
					String fieldName = fi.getFieldName();
					String fieldValue = fi.getString();
					if (fieldName.equals("NID")) {
						nid=fieldValue;
					}	
				} else {

					// Get the uploaded file parameters
					String fieldName = fi.getFieldName();
					String fieldValue = fi.getString();
					String fileName = fi.getName();
					String contentType = fi.getContentType();
					boolean isInMemory = fi.isInMemory();
					long sizeInBytes = fi.getSize();

					// Compute file name from NID
					String localFileName = CONET_STORAGE_PATH + nid2FileName(nid);
					file = new File(localFileName) ;
					fi.write( file );
					System.out.println("Stored named-data: " + fileName + "with NID :"+nid+" to local file: " + localFileName + " <br>");
				
					// Store the file into the ccnx repository, just if opening handle did throw any exceptions
					if(ccnxOK){
						FileInputStream infile = null;
						RepositoryOutputStream  content_file = null;

						try {
							//File file = new File(local_filename);
							infile = new FileInputStream(localFileName);
							ContentName name = null;
							if(nid.toString().startsWith(ccnPrefix))
								name = ContentName.fromURI(nid);
							else
								name = ContentName.fromURI(ccnPrefix + nid);
							
							// RepositoryFileOutputStream writes a VERSIONED content object
							content_file = new RepositoryOutputStream(name, ccn_handle, true);

							byte data[] = new byte[1024];
							int count;
							while ((count = infile.read(data, 0, 1024)) != -1)
							{
								content_file.write(data, 0, count);
							}

						} catch (MalformedContentNameStringException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						finally
						{
							if (infile != null) {
								try {
									infile.close();
								} catch (IOException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}				
							}
							if (content_file != null) {
								try {
									content_file.close();
								} catch (IOException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
							}
						}
						// edit the file to trigger the advertisement of the new route
//						try{
//							PrintWriter regOut = new PrintWriter(regFile);
//							int index = nid.lastIndexOf('/');
//							System.out.println("Advertisement of: " + "+/" + nid.substring(0, index) + "/" );
//							regOut.print("+/" + nid.substring(0, index) + "/");
//							regOut.close();
//						}
//						catch(FileNotFoundException e){
//							System.out.println("File " + regFile + " not present, meaning no advertise todo");
//						}
					}
				
				}
			}
			
			out.println("</body>");
			out.println("</html>");
		}catch(Exception ex) {
			System.out.println(ex);
			response.sendError(HttpServletResponse.SC_SERVICE_UNAVAILABLE );
		}
	}

	public String nid2FileName (String nid) {
		String fileName = new String();
		String principal = new String();
		String label = new String();
		if( nid.lastIndexOf('/') >= 0 ){
			principal=nid.substring(0,nid.indexOf('/')); 
			label=nid.substring(nid.indexOf('/'));
			label=label.replace( '/', '#' );
		}else{
			principal=nid; 
			label="";
		}
		fileName = principal+label;
		return fileName;
	} 

	public void doGet(HttpServletRequest request, 
			HttpServletResponse response)
	throws ServletException, java.io.IOException {

		throw new ServletException("GET method used with " +
				getClass( ).getName( )+": POST method required.");
	} 
	
	public void destroy(){
		ccn_handle.close();
	}
}
