package eu.convergence.middleware.apps.MiddlewareDemo;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.event.management.MXMEvent;
import org.iso.mpeg.mxm.core.event.management.MXMEventListener;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EventTriggerType;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Condition;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Principal;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.RELRight;

import eu.convergence.middleware.event.MatchingPublicationReceivedEvent;
import eu.convergence.middleware.orchestratorTE.ConvergenceOrchestratorTE;
import eu.convergence.middleware.orchestratorTE.schedule.task.OrchestratorTasks;
import eu.convergence.middleware.tool.er.EventReportTool;
import eu.convergence.middleware.tool.metadata.MetadataCreator;
import eu.convergence.middleware.tool.metadata.MetadataTool;
import eu.convergence.middleware.tool.metadata.MetadataWrapper;
import eu.convergence.middleware.tool.rel.LicenseCreator;
import eu.convergence.middleware.tool.rel.LicenseTool;
import eu.convergence.middleware.tool.vdi.PvdiCreator;
import eu.convergence.middleware.tool.vdi.RvdiCreator;
import eu.convergence.middleware.tool.vdi.SvdiCreator;
import eu.convergence.middleware.tool.vdi.VdiTool;


public class MiddlewareDemo implements MXMEventListener
{
	private License license;
	private StructuredData rdfMetadata;
	private String rvdiId;
	
	public MiddlewareDemo() throws MXMException
	{
		initMiddleware();
		
		startListener();
		
		license = createLicense();
		
		System.out.println(license.createObject(String.class.getCanonicalName()).getContent());
		
		rdfMetadata = wrapRdfMetadata();
		
//		rdfMetadata = createRdfMetadata();
		
		System.out.println(rdfMetadata.createObject(String.class.getCanonicalName()).getContent());
		
		createIdentifyAndAdvertiseContent();
		
		publishContent();
		
		subscribeToContent();
	}
	
	public StructuredData wrapRdfMetadata()
	{
		String rdfFilePath = getClass().getClassLoader().getResource("full_description.rdf").getFile();
		
		MetadataWrapper metadataTool = MetadataTool.newMetadataWrapper();
		
		if (metadataTool.parseRDF(new File(rdfFilePath), "http://escom.msh-paris.fr/"))
		{
			return metadataTool.generateStructuredData();
		}
		
		return null;
	}
	
	public StructuredData createRdfMetadata()
	{
		MetadataCreator metadataTool = MetadataTool.newMetadataCreator();
		
		metadataTool.addRDFTriple("http://convergence.eu/test/subject", "http://convergence.eu/test/predicate", "obj");
		
		return metadataTool.generateStructuredData();
	}
	
	public void subscribeToContent()
	{
		ERR err = createERR();
		
		SvdiCreator svdiTool = VdiTool.newSvdiCreator();
		
		Calendar date = Calendar.getInstance();
		date.add(Calendar.MONTH, 1);
		
		svdiTool.setExpiryDate(date.getTime());
		
		svdiTool.addERR(err);
		
		svdiTool.addFieldValueCondition("field", "=", "value");
		
		List<String> keywords = new ArrayList<String>();
		keywords.add("keyword");
		
		svdiTool.addKeywordCondition(keywords);
		
		List<String> tags = new ArrayList<String>();
		tags.add("tag");
		
		svdiTool.addTagCondition(tags);
		
//		svdiTool.addTripleCondition(subject, predicate, object) // TODO add sparql conditions
		
		svdiTool.setStartDate(new Date());
		
		svdiTool.setFractal("aek");
		
		svdiTool.generateSVDI();
		
		svdiTool.subscribe();
	}
	
	public void publishContent()
	{
		PvdiCreator pvdiTool = VdiTool.newPvdiCreator(rvdiId);

		Calendar date = Calendar.getInstance();
		date.add(Calendar.MONTH, 1);
		
		pvdiTool.setExpiryDate(date.getTime());
		
		pvdiTool.setFractal("aek");
		pvdiTool.setStartDate(new Date());
		
		pvdiTool.generatePVDI();

		pvdiTool.publish();
	}
	
	public ERR createERR()
	{
		EventReportTool erTool = EventReportTool.getInstance();
		return erTool.createERR(EventTriggerType.MATCH);
	}
	
	public void createIdentifyAndAdvertiseContent()
	{
		RvdiCreator rvdiTool = VdiTool.newRvdiCreator();
		org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource res 
					= rvdiTool.createResource("REF", "MIME/TYPE", "This is a comment");

//		rvdiTool.addRelationship(relationshipType, vdiId); // TODO add relationship

		rvdiTool.setStartDate(new Date());

		rvdiTool.addKeyword("keyword");
		
		rvdiTool.addFieldValue("field", "value");
		
		rvdiTool.addStructuredData(rdfMetadata);
		
		rvdiTool.addTag("tag");

		List<License> licenseList = new ArrayList<License>();
		licenseList.add(license);
		
		rvdiTool.addAsset(res, licenseList, null);

		rvdiTool.generateRVDI();

		String filePath = System.getProperty("java.io.tmpdir") + File.separator + UUID.randomUUID() + ".vdi";

		rvdiTool.store(filePath);

		rvdiTool.advertise(filePath);
		
		rvdiId = rvdiTool.getIdentifier();
	}
	
	public License createLicense()
	{
		LicenseCreator licenseTool = LicenseTool.newLicenseCreator();
		
		Principal az = licenseTool.createIdentityHolder("AZ");
		
//		Resource digitalItem = licenseTool.createDIReference("digitalItem");
		Resource digitalResource = licenseTool.createDigitalResource("http://digitalResource");
		
		Condition condition = licenseTool.createValidityIntervalCondition(new Date(), null);
		
		licenseTool.setLicenseIdentifier("AEK");
		licenseTool.setIssuer(az);
		licenseTool.setResource(digitalResource);
		
		licenseTool.setPrincipal(az);
		licenseTool.addGrant(RELRight.play, condition);
		
		return licenseTool.generateLicense();
	}
	
	public void initMiddleware()
	{
		try
		{
			InputStream in = getClass().getClassLoader().getResourceAsStream("MXMConfiguration.xml");
			MXM mxm = MXM.getInstance(in);
			mxm.listEnginesAndProperties();
			
			ConvergenceOrchestratorTE orchestrator = (ConvergenceOrchestratorTE) MXM.getInstance().getDefaultEngine(MXMEngineName.Orchestrator.value());
			
			// boot er, overlay, match, receiver
			orchestrator.getEROrchestrator().bootEREngine();
			orchestrator.getOverlayOrchestrator().bootOverlayEngine();
			orchestrator.getMatchOrchestrator().bootMatchEngine(false);
			orchestrator.getReceiverOrchestrator().bootReceiver();
	        
	        // start overlay tasks
			orchestrator.getTimerOrchestrator(OrchestratorTasks.GossipTask).startTaskExecution(10000);
			orchestrator.getTimerOrchestrator(OrchestratorTasks.RegistryAdvertisementTask).startTaskExecution(30000);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	public void startListener()
	{
		try
		{
	        MXM.getInstance().getMXMEventHandler().registerMXMEventListener(MatchingPublicationReceivedEvent.class.getName(), this);
	    } 
		catch (MXMConfigurationException e) 
		{
	        e.printStackTrace();
	    }
	}
	
	public static void main (String[] args) throws MXMException
	{
		new MiddlewareDemo();
	}

	public void mxmEventOccurred(MXMEvent event)
	{
		System.out.println("NOTIFICATION ARRIVED !!!!!!!!");
		
		System.exit(0);
	}
	
}
