package eu.convergence.middleware.apps.RdfAndSparql;

import java.io.File;
import java.io.InputStream;
import java.util.Date;
import java.util.UUID;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.event.management.MXMEvent;
import org.iso.mpeg.mxm.core.event.management.MXMEventListener;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EventTriggerType;

import eu.convergence.middleware.event.MatchingPublicationReceivedEvent;
import eu.convergence.middleware.matchTE.RepositoryCleaner;
import eu.convergence.middleware.orchestratorTE.ConvergenceOrchestratorTE;
import eu.convergence.middleware.orchestratorTE.schedule.task.OrchestratorTasks;
import eu.convergence.middleware.tool.er.EventReportTool;
import eu.convergence.middleware.tool.metadata.MetadataCreator;
import eu.convergence.middleware.tool.metadata.MetadataTool;
import eu.convergence.middleware.tool.vdi.PvdiCreator;
import eu.convergence.middleware.tool.vdi.RvdiCreator;
import eu.convergence.middleware.tool.vdi.SvdiCreator;
import eu.convergence.middleware.tool.vdi.VdiTool;

public class OfferSparqlMatch implements MXMEventListener
{
	private String offerVdiId;
	private StructuredData offerMetadata;
	
	public OfferSparqlMatch() throws MXMException
	{
		initMiddleware();
		
		startListener();
		
		createMetadata();
		
		createOfferVDI();
		
		publishOfferVDI();
		
		subscribeWithSparql();
	}
	
	public void createMetadata()
	{
		MetadataCreator metadataTool = MetadataTool.newMetadataCreator();
		
		String ns = "http://uti.com/";
		
		metadataTool.addRDFTriple(ns + "offerA", ns + "interest_product", ns + "productA");
		metadataTool.addRDFTriple(ns + "offerA", ns + "interest_category", ns + "product_categoryA"); // @UTI : ADD ALSO THE CATEGORY OF THE PRODUCT
		metadataTool.addRDFLiteralTriple(ns + "offerA", ns + "price", 300);
		
		Date date = new Date();
		date.setMinutes(date.getMinutes() + 5);
		
		metadataTool.addRDFLiteralTriple(ns + "offerA", ns + "expiration_date", date);
		
		offerMetadata = metadataTool.generateStructuredData();
	}
	
	public void createOfferVDI()
	{
		/////////////////////CREATE, PARSE, STORE AND ADVERTISE RVDI WITH TOOLS///////////////////////////

		RvdiCreator rvdiTool = VdiTool.newRvdiCreator();

		rvdiTool.setExpiryDate(new Date());

		rvdiTool.addStructuredData(offerMetadata);

		rvdiTool.generateRVDI();

		String filePath = System.getProperty("java.io.tmpdir") + File.separator + UUID.randomUUID() + ".vdi";

		if (rvdiTool.store(filePath))
		{
			rvdiTool.advertise(filePath);
		}
		
		offerVdiId = rvdiTool.getIdentifier();

		//////////////////////////////////////////////////////////////////////////////////////////////////
	}
	
	public void publishOfferVDI()
	{
		/////////////////////CREATE AND PUBLISH A PVDI WITH TOOLS/////////////////////

		PvdiCreator pubVdiTool = VdiTool.newPvdiCreator(offerVdiId);
		pubVdiTool.setExpiryDate(new Date());
		pubVdiTool.setFractal("offer");

		pubVdiTool.generatePVDI();
		pubVdiTool.publish();

		////////////////////////////////////////////////////////////////////////////////
	}
	
	public void subscribeWithSparql()
	{
		/////////////////////CREATE AND PUBLISH A SVDI WITH TOOLS/////////////////////

		ERR err = EventReportTool.getInstance().createERR(EventTriggerType.MATCH);

		SvdiCreator svdiTool = VdiTool.newSvdiCreator();
		svdiTool.setFractal("offer");
		svdiTool.setExpiryDate(new Date());
		
		String ns = "http://uti.com/";
		
		svdiTool.addSparqlReturnVariable("?x");
		svdiTool.addSparqlTripleCondition("?x", ns + "interest_category", ns + "product_categoryA");
		svdiTool.addSparqlTripleCondition("?x", ns + "price", "?y");
		svdiTool.addSparqlFilterCondition("?y", ">", 200);
		svdiTool.addSparqlFilterCondition("?y", "<", 400);
		svdiTool.addSparqlTripleCondition("?x", ns + "expiration_date", "?z");
		svdiTool.addSparqlFilterCondition("?z", ">", new Date());
		
		svdiTool.addERR(err);

		svdiTool.generateSVDI();

		svdiTool.subscribe();

		///////////////////////////////////////////////////////////////////////////////
	}
	
	public void initMiddleware()
	{
		try
		{
			new RepositoryCleaner().cleanAll();
			
			InputStream in = getClass().getClassLoader().getResourceAsStream("MXMConfiguration.xml");
			MXM mxm = MXM.getInstance(in);
			mxm.listEnginesAndProperties();
			
			ConvergenceOrchestratorTE orchestrator = (ConvergenceOrchestratorTE) MXM.getInstance().getDefaultEngine(MXMEngineName.Orchestrator.value());
			
			// boot er, overlay, match, receiver
			orchestrator.getEROrchestrator().bootEREngine();
			orchestrator.getOverlayOrchestrator().bootOverlayEngine();
			orchestrator.getMatchOrchestrator().bootMatchEngine(false);
			orchestrator.getReceiverOrchestrator().bootReceiver();
	        
	        // start overlay tasks
			orchestrator.getTimerOrchestrator(OrchestratorTasks.GossipTask).startTaskExecution(10000);
			orchestrator.getTimerOrchestrator(OrchestratorTasks.RegistryAdvertisementTask).startTaskExecution(30000);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	public void startListener()
	{
		try
		{
	        MXM.getInstance().getMXMEventHandler().registerMXMEventListener(MatchingPublicationReceivedEvent.class.getName(), this);
	    } 
		catch (MXMConfigurationException e) 
		{
	        e.printStackTrace();
	    }
	}
	
	public static void main (String[] args) throws MXMException
	{
		new OfferSparqlMatch();
	}
	
	public void mxmEventOccurred(MXMEvent event)
	{
		System.out.println("NOTIFICATION ARRIVED !!!!!!!!");
		
		System.exit(0);
	}
	
}
