package eu.convergence.middleware.apps.RdfAndSparql;

import java.io.File;
import java.io.InputStream;
import java.util.Date;
import java.util.UUID;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.event.management.MXMEvent;
import org.iso.mpeg.mxm.core.event.management.MXMEventListener;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EventTriggerType;

import eu.convergence.middleware.event.MatchingPublicationReceivedEvent;
import eu.convergence.middleware.matchTE.RepositoryCleaner;
import eu.convergence.middleware.orchestratorTE.ConvergenceOrchestratorTE;
import eu.convergence.middleware.orchestratorTE.schedule.task.OrchestratorTasks;
import eu.convergence.middleware.tool.er.EventReportTool;
import eu.convergence.middleware.tool.metadata.MetadataTool;
import eu.convergence.middleware.tool.metadata.MetadataWrapper;
import eu.convergence.middleware.tool.vdi.PvdiCreator;
import eu.convergence.middleware.tool.vdi.RvdiCreator;
import eu.convergence.middleware.tool.vdi.SvdiCreator;
import eu.convergence.middleware.tool.vdi.VdiTool;

public class RdfAndSparql implements MXMEventListener
{
	private String analysisRvdiId;
	private StructuredData structuredData;
	
	private String subscribeWithMatchingFieldValue;
	private String subscribeWithMatchingSparql;
	private String subscribeWithMatchingFieldValueAndSparql;
	private String subscribeWithBoth_OnlyFieldValueMatches;
	private String subscribeWithBoth_OnlySparqlMatches;
	private String subscribeWithBoth_NothingMatches;
	
	private String results = "";
	
	private int notificationCount = 0;
	
	
	public RdfAndSparql() throws MXMException
	{
		initMiddleware();
		
		startListener();
		
		parseMetadata();
		
		createAnalysisVDI();
		
		publishAnalysisVDI();
		
		subscribeWithMatchingFieldValue();

		subscribeWithMatchingSparql();

		subscribeWithMatchingFieldValueAndSparql();

		subscribeWithBoth_NothingMatches();

		subscribeWithBoth_OnlyFieldValueMatches();

		subscribeWithBoth_OnlySparqlMatches();

		publishAnalysisVDI();
	}
	
	public void parseMetadata()
	{
		File rdfFile = new File("./src/main/resources/full_description.rdf");
		
		MetadataWrapper metadataTool = MetadataTool.newMetadataWrapper();
		
		metadataTool.parseRDF(rdfFile, "http://escom.msh-paris.fr/");
		
		structuredData = metadataTool.generateStructuredData();
	}
	
	public void createAnalysisVDI()
	{
		/////////////////////CREATE, PARSE, STORE AND ADVERTISE RVDI WITH TOOLS///////////////////////////

		RvdiCreator rvdiTool = VdiTool.newRvdiCreator();

		rvdiTool.setExpiryDate(new Date());

		String[] tags = {"ORIGINAL"};

		for (String tag : tags)
		{
			rvdiTool.addKeyword(tag);
		}
		
		rvdiTool.addFieldValue("TEAM", "AEK");
		
		rvdiTool.addStructuredData(structuredData);

		rvdiTool.generateRVDI();

		String filePath = System.getProperty("java.io.tmpdir") + File.separator + UUID.randomUUID() + ".vdi";

		if (rvdiTool.store(filePath))
		{
			rvdiTool.advertise(filePath);
		}
		
		analysisRvdiId = rvdiTool.getIdentifier();

		//////////////////////////////////////////////////////////////////////////////////////////////////
	}
	
	public void publishAnalysisVDI()
	{
		/////////////////////CREATE AND PUBLISH A PVDI WITH TOOLS/////////////////////

		PvdiCreator pubVdiTool = VdiTool.newPvdiCreator(analysisRvdiId);
		pubVdiTool.setExpiryDate(new Date());
		pubVdiTool.setFractal("analysis");

		pubVdiTool.generatePVDI();
		pubVdiTool.publish();

		////////////////////////////////////////////////////////////////////////////////
	}
	
	public void subscribeWithMatchingFieldValue()
	{
		/////////////////////CREATE AND PUBLISH A SVDI WITH TOOLS/////////////////////

		ERR err = EventReportTool.getInstance().createERR(EventTriggerType.MATCH);

		SvdiCreator svdiTool = VdiTool.newSvdiCreator();
		svdiTool.setFractal("analysis");
		svdiTool.setExpiryDate(new Date());
		
		svdiTool.addFieldValueCondition("TEAM", "=", "AEK");

		svdiTool.addERR(err);

		svdiTool.generateSVDI();

		subscribeWithMatchingFieldValue = svdiTool.subscribe();

		///////////////////////////////////////////////////////////////////////////////
	}
	
	public void subscribeWithMatchingSparql()
	{
		/////////////////////CREATE AND PUBLISH A SVDI WITH TOOLS/////////////////////

		ERR err = EventReportTool.getInstance().createERR(EventTriggerType.MATCH);

		SvdiCreator svdiTool = VdiTool.newSvdiCreator();
		svdiTool.setFractal("analysis");
		svdiTool.setExpiryDate(new Date());
		
		svdiTool.addSparqlReturnVariable("?x");
		svdiTool.addSparqlTripleCondition("?x", "http://escom.msh-paris.fr/videodescription.owl#hasReferences", "http://escom.msh-paris.fr/531c1582-5950-4a2a-aab2-f428c2172212");
		
		svdiTool.addERR(err);

		svdiTool.generateSVDI();

		subscribeWithMatchingSparql = svdiTool.subscribe();

		///////////////////////////////////////////////////////////////////////////////
	}
	
	public void subscribeWithMatchingFieldValueAndSparql()
	{
		/////////////////////CREATE AND PUBLISH A SVDI WITH TOOLS/////////////////////

		ERR err = EventReportTool.getInstance().createERR(EventTriggerType.MATCH);

		SvdiCreator svdiTool = VdiTool.newSvdiCreator();
		svdiTool.setFractal("analysis");
		svdiTool.setExpiryDate(new Date());
		
		svdiTool.addFieldValueCondition("TEAM", "=", "AEK");

		svdiTool.addSparqlReturnVariable("?x");
		svdiTool.addSparqlTripleCondition("?x", "http://escom.msh-paris.fr/videodescription.owl#hasReferences", "http://escom.msh-paris.fr/531c1582-5950-4a2a-aab2-f428c2172212");
		
		svdiTool.addERR(err);

		svdiTool.generateSVDI();

		subscribeWithMatchingFieldValueAndSparql = svdiTool.subscribe();

		///////////////////////////////////////////////////////////////////////////////
	}
	
	public void subscribeWithBoth_OnlyFieldValueMatches()
	{
		/////////////////////CREATE AND PUBLISH A SVDI WITH TOOLS/////////////////////

		ERR err = EventReportTool.getInstance().createERR(EventTriggerType.MATCH);

		SvdiCreator svdiTool = VdiTool.newSvdiCreator();
		svdiTool.setFractal("analysis");
		svdiTool.setExpiryDate(new Date());
		
		svdiTool.addFieldValueCondition("TEAM", "=", "AEK");

		svdiTool.addSparqlReturnVariable("?x");
		svdiTool.addSparqlTripleCondition("?x", "http://escom.msh-paris.fr/aaa", "http://escom.msh-paris.fr/bbb");
		
		svdiTool.addERR(err);

		svdiTool.generateSVDI();

		subscribeWithBoth_OnlyFieldValueMatches = svdiTool.subscribe();

		///////////////////////////////////////////////////////////////////////////////
	}
	
	public void subscribeWithBoth_OnlySparqlMatches()
	{
		/////////////////////CREATE AND PUBLISH A SVDI WITH TOOLS/////////////////////

		ERR err = EventReportTool.getInstance().createERR(EventTriggerType.MATCH);

		SvdiCreator svdiTool = VdiTool.newSvdiCreator();
		svdiTool.setFractal("analysis");
		svdiTool.setExpiryDate(new Date());
		
		svdiTool.addFieldValueCondition("FIELD", "=", "VALUE");

		svdiTool.addSparqlReturnVariable("?x");
		svdiTool.addSparqlTripleCondition("?x", "http://escom.msh-paris.fr/videodescription.owl#hasReferences", "http://escom.msh-paris.fr/531c1582-5950-4a2a-aab2-f428c2172212");
		
		svdiTool.addERR(err);

		svdiTool.generateSVDI();

		subscribeWithBoth_OnlySparqlMatches = svdiTool.subscribe();

		///////////////////////////////////////////////////////////////////////////////
	}
	
	public void subscribeWithBoth_NothingMatches()
	{
		/////////////////////CREATE AND PUBLISH A SVDI WITH TOOLS/////////////////////

		ERR err = EventReportTool.getInstance().createERR(EventTriggerType.MATCH);

		SvdiCreator svdiTool = VdiTool.newSvdiCreator();
		svdiTool.setFractal("analysis");
		svdiTool.setExpiryDate(new Date());
		
		svdiTool.addFieldValueCondition("FIELD", "=", "VALUE");

		svdiTool.addSparqlReturnVariable("?x");
		svdiTool.addSparqlTripleCondition("?x", "http://escom.msh-paris.fr/aaa", "http://escom.msh-paris.fr/bbb");
		
		svdiTool.addERR(err);

		svdiTool.generateSVDI();

		subscribeWithBoth_NothingMatches = svdiTool.subscribe();

		///////////////////////////////////////////////////////////////////////////////
	}
	
	public void initMiddleware()
	{
		try
		{
			new RepositoryCleaner().cleanAll();
			
			InputStream in = getClass().getClassLoader().getResourceAsStream("MXMConfiguration.xml");
			MXM mxm = MXM.getInstance(in);
			mxm.listEnginesAndProperties();
			
			ConvergenceOrchestratorTE orchestrator = (ConvergenceOrchestratorTE) MXM.getInstance().getDefaultEngine(MXMEngineName.Orchestrator.value());
			
			// boot er, overlay, match, receiver
			orchestrator.getEROrchestrator().bootEREngine();
			orchestrator.getOverlayOrchestrator().bootOverlayEngine();
			orchestrator.getMatchOrchestrator().bootMatchEngine(false);
			orchestrator.getReceiverOrchestrator().bootReceiver();
	        
	        // start overlay tasks
			orchestrator.getTimerOrchestrator(OrchestratorTasks.GossipTask).startTaskExecution(10000);
			orchestrator.getTimerOrchestrator(OrchestratorTasks.RegistryAdvertisementTask).startTaskExecution(30000);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	public void startListener()
	{
		try
		{
	        MXM.getInstance().getMXMEventHandler().registerMXMEventListener(MatchingPublicationReceivedEvent.class.getName(), this);
	    } 
		catch (MXMConfigurationException e) 
		{
	        e.printStackTrace();
	    }
	}
	
	public static void main (String[] args) throws MXMException
	{
		new RdfAndSparql();
	}
	
	public void mxmEventOccurred(MXMEvent event)
	{
		System.out.println("NOTIFICATION ARRIVED !!!!!!!!");
		
		if (event instanceof MatchingPublicationReceivedEvent)
		{
			MatchingPublicationReceivedEvent match = (MatchingPublicationReceivedEvent) event;
			
			if (match.getSVDI_id().equals(subscribeWithMatchingFieldValue))
			{
				results+= "\r\nsubscribeWithMatchingFieldValue matches PUB";
				notificationCount++;
			}
			else if (match.getSVDI_id().equals(subscribeWithMatchingSparql))
			{
				results+= "\r\nsubscribeWithMatchingSparql matches PUB";
				notificationCount++;
			}
			else if (match.getSVDI_id().equals(subscribeWithMatchingFieldValueAndSparql))
			{
				results+= "\r\nsubscribeWithMatchingFieldValueAndSparql matches PUB";
				notificationCount++;
			}
			else if (match.getSVDI_id().equals(subscribeWithBoth_NothingMatches))
			{
				results+= "\r\nsubscribeWithBoth_NothingMatches matches PUB";
				notificationCount++;
			}
			else if (match.getSVDI_id().equals(subscribeWithBoth_OnlyFieldValueMatches))
			{
				results+= "\r\nsubscribeWithBoth_OnlyFieldValueMatches matches PUB";
				notificationCount++;
			}
			else if (match.getSVDI_id().equals(subscribeWithBoth_OnlySparqlMatches))
			{
				results+= "\r\nsubscribeWithBoth_OnlySparqlMatches matches PUB";
				notificationCount++;
			}
			
//			if (results.contains("subscribeWithMatchingFieldValueAndSparql") && results.contains("subscribeWithBoth_OnlyFieldValueMatches") &&
//				results.contains("subscribeWithMatchingSparql") && results.contains("subscribeWithMatchingFieldValue") &&
//				results.contains("subscribeWithBoth_OnlySparqlMatches"))
//			{
//				System.exit(0);
//			}
			
			if (notificationCount==10)
			{
				System.out.print("=======================================================");
				System.out.println(results);
				System.out.println("=======================================================");
				System.out.println("\r\nSUCCESS");
				System.exit(0);
			}
		}
	}
}
