/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package eu.convergence.middleware.engine.metadataTE.schema;

import java.util.ArrayList;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public final class ConvergenceMetadataEngineContext {
	
	private static ConvergenceMetadataEngineContext packages = null;
	
	protected Logger logger = LoggerFactory.getLogger(this.getClass());

	protected ArrayList<String> packageNames = new ArrayList<String>();

	private static JAXBContext jc = null;

	/**
	 * @return all package names in sequence
	 */
	private String getPackagesInOrder() {
		StringBuffer orderpackages = new StringBuffer();
		for (int i = 0; i < packageNames.size(); i++) {
			orderpackages.append(packageNames.get(i));
			if (i >= (packageNames.size() - 1))
				break;
			orderpackages.append(":");
		}
		return orderpackages.toString();
	}

	/**
	 * @return the JAXB Context all schemas used by the Engine
	 * @throws JAXBException
	 */
	public JAXBContext getJAXBContext() throws JAXBException {
		if (jc == null) {
			logger.debug("Generating Engine JAXB Context...");
			jc = JAXBContext.newInstance(getPackagesInOrder());
			logger.debug("Done! \n");
		}
		return jc;
	}
	
	private ConvergenceMetadataEngineContext()
	{
		//ConvMeta packages
		packageNames.add("eu.convergence.middleware.engine.metadataTE.schema.convmeta");
		
		// DID packages
		packageNames.add("org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl");
		packageNames.add("org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_msx");
		packageNames.add("org.iso.mpeg.mxm.engine.digitalitemTE.schema.dii");

		//IPMP packages
		packageNames.add("org.iso.mpeg.mxm.engine.digitalitemTE.schema.didmodel_mp21");
		packageNames.add("org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpinfo");

		//ER packages
		packageNames.add("org.iso.mpeg.mxm.engine.eventreportTE.schema.er");
		
		//Mpqf packages
		packageNames.add("org.iso.mpeg.mxm.engine.searchTE.schema.mpqf");
	}
	
	public static synchronized ConvergenceMetadataEngineContext getInstance(){
		if(packages==null)
		{
			packages=new ConvergenceMetadataEngineContext();
		}
		return packages;
	}
}
