package eu.convergence.middleware.cdsTE;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.reasoner.ReasonerRegistry;

public class RepositoryManager
{
	private static final String LOCAL_REPOSITORY_PATH_PARAM = "LOCAL_REPOSITORY_PATH";
	
	private HashMap<String,Model> uri2ontologyModel;
	
	
	public RepositoryManager()
	{
		uri2ontologyModel = new HashMap<String,Model>();
	}
	
	public void initRepositoryManager(MXMGenericParameters mxmParameters) // TODO auto na paei knowledge manager kai na kanei edo 
	{																	  // 	  load ontology me url
		String path = mxmParameters.getProperty(LOCAL_REPOSITORY_PATH_PARAM);
		
		File dir = new File(path);
		dir.mkdirs();
		
		System.out.println("************************************************");
		System.out.println("**********STARTING LOADING ONTOLOGIES***********");
		
		for (File file : dir.listFiles())
		{
			if (file.isFile())
			{
				if (file.getName().endsWith(".ttl"))
				{
					try
					{
						if (file.getName().startsWith("wordnet"))
						{
							if (uri2ontologyModel.get("wordnet")==null)
							{
								OntModel wordNet = ModelFactory.createOntologyModel(OntModelSpec.OWL_DL_MEM);
								uri2ontologyModel.put("wordnet", wordNet);
							}
							uri2ontologyModel.get("wordnet").read(new FileInputStream(file),null,"TTL");
						}
						else if (file.getName().startsWith("corest"))
						{
							OntModel corest = ModelFactory.createOntologyModel(OntModelSpec.OWL_DL_MEM);
							corest.read(new FileInputStream(file),null,"TTL");
							uri2ontologyModel.put("corest", corest);
						}
						
//						System.out.println("PREFIXES: " + wordNet.getNsPrefixMap());
//						System.out.println("- Loaded successfully " + file.getName() + " -> " + wordNet.size() + " triples");
						System.out.println("- Loaded successfully " + file.getName());
					}
					catch (FileNotFoundException e) 
					{
						e.printStackTrace();
					}
				}
				else if (file.getName().endsWith(".owl"))
				{
					try
					{
//						OntModel model = ModelFactory.createOntologyModel(OntModelSpec.OWL_DL_MEM_RDFS_INF);
//						model.read(new FileInputStream(file),null);
//						
//						int dotIndex = file.getName().lastIndexOf(".");
//						
//						uri2ontologyModel.put(file.getName().substring(0,dotIndex), model);
//						
//						System.out.println("- Loaded successfully " + file.getName().substring(0,dotIndex) + " -> " 
//											+ model.size() + " triples");
						
						OntModel model = ModelFactory.createOntologyModel(OntModelSpec.OWL_DL_MEM);
						model.read(new FileInputStream(file),null);
						
						Reasoner reasoner = ReasonerRegistry.getRDFSReasoner();
					    InfModel infmodel = ModelFactory.createInfModel(reasoner, model);
						
//						InfModel infmodel = ModelFactory.createRDFSModel(model);
						
						int dotIndex = file.getName().lastIndexOf(".");
						
						uri2ontologyModel.put(file.getName().substring(0,dotIndex), infmodel);
						
						System.out.println("- Loaded successfully " + file.getName().substring(0,dotIndex) + " -> " 
											+ model.size() + "/" + infmodel.size() + " triples");
					}
					catch (Exception ex)
					{
						ex.printStackTrace();
					}
				}
//				else if (file.getName().endsWith(".rdf"))
//				{
//					
//				}
			}
		}
		
		System.out.println("**********FINISHED LOADING ONTOLOGIES***********");
		System.out.println("************************************************");
	}
	
	public List<Statement> getStatementsWithPattern(String subject, String predicate, String object, String ontologyURI) // TODO den einai kalos tropos na rotas
	{
		Model ontology = uri2ontologyModel.get(ontologyURI);
		
		if (ontology!=null)
		{
			Resource subjectNode = null;
			Property propertyNode = null;
			RDFNode objectNode = null;
			
			if (subject!=null)
			{
				subjectNode = ontology.getResource(subject);
			}
			
			if (predicate!=null)
			{
				propertyNode = ontology.getProperty(predicate);
			}
			
			if (object!=null)
			{
				objectNode = ontology.getResource(object);
			}
			
			StmtIterator statements = ontology.listStatements(subjectNode, propertyNode, objectNode);
			
			List<Statement> statementList = statements.toList();
			
//			System.out.println("TriplePattern: " + subjectNode + " " + propertyNode + " " + objectNode + " RESULT: " + statementList.size());
			
			return statementList;
		}
		else
		{
			return new ArrayList<Statement>();
		}	
	}
	
}
