package eu.convergence.middleware.coneTE.ccnx.SendToName;


import java.io.IOException;

import org.ccnx.ccn.CCNContentHandler;
import org.ccnx.ccn.CCNHandle;
import org.ccnx.ccn.CCNInterestHandler;
import org.ccnx.ccn.protocol.ContentName;
import org.ccnx.ccn.protocol.ContentObject;
import org.ccnx.ccn.protocol.Interest;
import org.ccnx.ccn.protocol.MalformedContentNameStringException;

public class SendToNameThreadC extends Thread implements CCNContentHandler, CCNInterestHandler{

	private String peer = null;
	private String service_name = null;
	private String interestID = null;
	private ContentObject msg = null;
	private CCNHandle handle = null;
	private boolean linger = false;
	private String interestToWaitFor = null;
	//this thread has to die if it cannot complete the handshake with the remote peer
	//within a certain MAX amount of time (i.e. the remote peer is dead)
	private int lifetime = 0;
	private int MAX_LINGER_PERIODS = 20; //5 seconds if 250ms sleep
	
	public SendToNameThreadC(String peer, ContentObject msg, CCNHandle handle, String svc_name){
		super("STNClient:" + peer);
		this.peer = peer;
		this.service_name = svc_name;
		this.msg = msg;
		this.handle = handle;
		this.linger = true;
		this.lifetime = 0;
		int index = peer.lastIndexOf('/');
		this.interestID = peer.substring(index+1);
		try {
			interestToWaitFor = svc_name.substring(0, svc_name.lastIndexOf('/')+1);
			System.out.println("Now listen to " + interestToWaitFor);
			this.handle.registerFilter(ContentName.fromURI(interestToWaitFor), this);
		} catch (MalformedContentNameStringException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void run(){
		Interest interest = null;
		try {
			interest = new Interest(peer);
			System.out.println("Interest is leaving for " + interest.name().toURIString());
			handle.expressInterest(interest, this);
		} catch (MalformedContentNameStringException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		while(linger && lifetime < MAX_LINGER_PERIODS){
			try {
				lifetime++;
				Thread.sleep(250);
				//Thread.currentThread().interrupt();				
			} catch (InterruptedException e) {
				e.printStackTrace();
				//Thread.currentThread().interrupt();
			}
		}
		try {
			this.handle.unregisterFilter(ContentName.fromURI(interestToWaitFor), this);
			System.out.println(this.getName() + " outer has unregistered "+interestToWaitFor);
		} catch (MalformedContentNameStringException e) {
			e.printStackTrace();
		}
		if(!linger) {
			System.out.println(this.getName() + " STN CLIENT is expiring because LINGER has changed to false");
		}
		if(lifetime >= MAX_LINGER_PERIODS) {
			System.out.println(this.getName() + " STN CLIENT is expiring because TIMEOUT has expired");
		}
	}

	@Override
	public boolean handleInterest(Interest arg0) {
		String interest_name = arg0.name().toURIString();
		System.out.println(this.getName() + ": Received Interest " + interest_name);
		if(interest_name.contains(interestID)){
			try {
				System.out.println(this.getName() + ": Sending DATA for " + interest_name);
				System.out.println("4-way sendToName COMPLETED");
				handle.put(msg);
				this.completed();
			} catch (IOException e) {
				e.printStackTrace();
			}
			return true;
		}
		return false;
	}

	public void completed(){
		linger= false;
	}
	
	@Override
	public Interest handleContent(ContentObject arg0, Interest arg1) {
		System.out.println("Received Data " + arg0.name().toURIString());
		if(arg0.name().toURIString().contains(interestID)){
			System.out.println("this is Data ACK for" + arg0.name().toURIString());				
		}
		return null;
	}
}