package eu.convergence.middleware.coneTE.ccnx.SendToName;

import java.io.IOException;
import org.ccnx.ccn.CCNContentHandler;
import org.ccnx.ccn.CCNHandle;
import org.ccnx.ccn.protocol.ContentName;
import org.ccnx.ccn.protocol.ContentObject;
import org.ccnx.ccn.protocol.Interest;
import org.ccnx.ccn.protocol.KeyLocator;
import org.ccnx.ccn.protocol.MalformedContentNameStringException;
import org.ccnx.ccn.protocol.PublisherPublicKeyDigest;
import org.ccnx.ccn.protocol.SignedInfo.ContentType;

import eu.convergence.middleware.conetTe.ConetTE;
import eu.convergence.middleware.engine.ConetTe.services.SAPListener;

public class SendToNameThreadS extends Thread implements CCNContentHandler{

	private String contentToRetrieve = null;
	private String interestID = null;
	private CCNHandle handle = null;
	private boolean linger = false;
	private SAPListener eventReports_handler = null, propagations_handler = null;
	private KeyLocator keyLoc = null;
	private PublisherPublicKeyDigest pubkey = null;
	//this thread has to die if it cannot complete the handshake with the remote peer
	//within a certain MAX amount of time (i.e. the remote peer is dead)
	private int lifetime = 0;
	private int MAX_LINGER_PERIODS = 20; //5 seconds if 250ms sleep
	
	public SendToNameThreadS(SAPListener er_handler, SAPListener msg_handler, CCNHandle handle, String received_interest, String service_name){
		super("STNServer");
		this.eventReports_handler = er_handler;
		this.propagations_handler = msg_handler;
		this.handle = handle;		
		this.linger = true;
		this.lifetime = 0;
		this.contentToRetrieve = received_interest;
		generateSendToNameInterestACK(received_interest);
		int index = received_interest.indexOf(service_name);
		//index += (service_name.length() + 1);
		index += (service_name.length());
		this.contentToRetrieve = "ccnx:/" + received_interest.substring(index); 
		int indexID = contentToRetrieve.lastIndexOf('/');
		this.interestID = contentToRetrieve.substring(indexID+1);
	}
		
	
	public void run(){
		Interest interest = null;
		try {
			interest = new Interest(contentToRetrieve);
			System.out.println("Interest is leaving for " + interest.name().toURIString());
			handle.getNetworkManager().expressInterest(null, interest, this);
		} catch (MalformedContentNameStringException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		while(linger && lifetime < MAX_LINGER_PERIODS){
			try {
				lifetime++;
				Thread.sleep(250);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		if(!linger) {
			System.out.println(this.getName() + " STN SERVER is expiring because LINGER has changed to false");
		}
		if(lifetime >= MAX_LINGER_PERIODS) {
			System.out.println(this.getName() + " STN SERVER is expiring because TIMEOUT has expired");
		}
	}
	
	private void generateSendToNameInterestACK(String interestToACK) {
    	String ack = "This is an ACK";
		byte[] dummy_content = ack.getBytes();
		ContentName contentName;
		try {
			contentName = ContentName.fromURI(interestToACK);
			ContentObject dataAck = ContentObject.buildContentObject(contentName, ContentType.DATA, dummy_content, pubkey, keyLoc, null, 0, null);
			System.out.println("Sending ACK for " + interestToACK);
			handle.put(dataAck);
		} catch (MalformedContentNameStringException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    
	public void completed(){
		linger= false;
	}
	
	@Override
	public Interest handleContent(ContentObject arg0, Interest arg1) {
		System.out.println("Received Content for interest " + arg1.toString());
		// wake-up someone to deal with this content, take as example old recvDaemon
		if(arg0.name().toURIString().contains(interestID)  && !arg0.name().toURIString().contains(ConetTE.getStn_service())){
			// signal to the SendToNameThread server that it can shut down ASAP
			completed();
			
			// we must properly route this content, ER or gorest message
			if(arg0.name().toURIString().contains("/ER/")){
				eventReports_handler.indication(null, arg0.content());
			}
			else{
				propagations_handler.indication(null, arg0.content());
			}
		}
		return null;
	}

}