package eu.convergence.middleware.conetTe;

import eu.convergence.middleware.coneTE.ccnx.SendToName.SendToNameThreadC;
import eu.convergence.middleware.coneTE.ccnx.SendToName.SendToNameThreadS;
import eu.convergence.middleware.conetTe.smile.SendToName.generated.SendToNameOneWay;
import eu.convergence.middleware.core.ConvergenceEngineName;
import eu.convergence.middleware.engine.ConetTe.exception.ConetIOException;
import eu.convergence.middleware.engine.ConetTe.services.*;

import it.uniroma2.smile.core.Fault;
import it.uniroma2.smile.core.SendingMessageException;
import it.uniroma2.smile.run.SmileRunUtil;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;
import java.util.UUID;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.impl.client.DefaultHttpClient;
import org.ccnx.ccn.CCNHandle;
import org.ccnx.ccn.CCNInterestHandler;
import org.ccnx.ccn.io.CCNInputStream;
import org.ccnx.ccn.io.CCNVersionedInputStream;
import org.ccnx.ccn.protocol.ContentName;
import org.ccnx.ccn.protocol.ContentObject;
import org.ccnx.ccn.protocol.Interest;
import org.ccnx.ccn.protocol.KeyLocator;
import org.ccnx.ccn.protocol.MalformedContentNameStringException;
import org.ccnx.ccn.protocol.PublisherPublicKeyDigest;
import org.ccnx.ccn.protocol.SignedInfo.ContentType;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMAbstractEngine;
import org.iso.mpeg.mxm.core.MXMInvalidConfigurationException;
import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import conet.dataplane.LCServer;


/** 
 * this engine supports the store / get mechanism and the sendToName
 * it has to be instantiated and initialized in this way:
 * 	   conetTE = (ConetTE) mxm.getDefaultEngine(ConvergenceEngineName.ConetTE.toString());<p>
 * during the initialization its static field servletBaseUrl is initialized
 * with the value read from the MXMconfiguration.xml file<p>
 * when initialized, the support of sendToName can be added 
 * reading SMILE/SIP parameters from a xxx-sip.cfg
 * configuration file
 * @author Stefano Salsano
 *
 */
public class ConetTE extends MXMAbstractEngine implements eu.convergence.middleware.engine.ConetTe.ConetEngine, eu.convergence.middleware.engine.ConetTe.ConetTE, CCNInterestHandler {
	
	ConetTeSmileRunTimeContainer conetTeSmileRunTimeContainer = null;
	
	// in case of ccnx protocol usage we need a single conetTE instance
	private static ConetTE conetTE = null;
	
	private static String servletBaseUrl;
	
	//the servlet url will be like "http://160.80.82.69:8080/conet/(UploadServlet|DownloadServlet|RevokeServlet)";	
	private String uploadServletName = "UploadServlet";
	private String downloadServletName = "DownloadServlet";
    private String revokeServletName = "RevokeServlet";
	
	private String uploadServlet ;
	private String downloadServlet ;
    private String revokeServlet ;
    
    private String ccnPrefix;
    private static String ccnx = null;
    private static String peerID = null;
    private static String stn_service = null;
    private static String registry_service = null;
	private CCNHandle ccn_handleA = null;
	private CCNHandle ccn_handleS = null;
	private String ccnServiceName = null;
	
	// callback objects who listen for incoming messages
	private static SAPListener ers_handler = null;
	private static SAPListener propagations_handler = null;

	// fetching and setting routes behind the wood
	private static LCServer lc = null;
	private String conf_lcServer = null;
	private int nrs_port = 9999;
	private String node_properties = null;

	/**
     * it throws a RunTimeException ("Servlet Base URL not configured") if
     * it can't read the Servlet Base URL from the mxm configuration file 
     */
    @Override
    public void initialise(String engineID, MXMGenericParameters mxmParameters, boolean isDefault)
    {
    	super.initialise(engineID, mxmParameters, isDefault);
    	
    	try {
    		servletBaseUrl = this.getPropertyValue("CONET_SERVLET_BASE_URL");
    		ccnx = this.getPropertyValue("NET_PROTOCOL");
		} catch (NullPointerException e) {
		}
    	if (servletBaseUrl==null||servletBaseUrl.equals("")) {
    		throw new RuntimeException("Servlet Base URL not configured");
    	}
    	uploadServlet = servletBaseUrl+uploadServletName;
    	downloadServlet = servletBaseUrl+downloadServletName;
        revokeServlet = servletBaseUrl+revokeServletName;
        
        if((ccnx != null) && (ccnx.compareTo("ccnx") == 0)){
        	conetTE = this;
        	try {
        		peerID = this.getPropertyValue("PEER_ID");
    			conf_lcServer = this.getPropertyValue("CONETRE.CONF");
    			nrs_port = Integer.parseInt(this.getPropertyValue("NRS_PORT"));
    			stn_service = this.getPropertyValue("STN_SERVICE");
    			registry_service = this.getPropertyValue("REGISTRY_SERVICE");
    			node_properties = this.getPropertyValue("NODE_PROPERTIES");
    			
    			if(stn_service == null)
    				stn_service = "nsap";
    			if(registry_service == null)
    				registry_service = "registry";
    			
    			System.out.println("*********************************CCNX VARIABLES************************************");
    			System.out.println("*********************************PEER_ID: " + peerID + " ************************************");
    			System.out.println("*********************************CONETRE.CONF: " + conf_lcServer + " ************************************");
    			System.out.println("*********************************NRS_PORT: " + nrs_port + " ************************************");
    			System.out.println("*********************************STN_SERVICE: " + stn_service + " ************************************");
    			System.out.println("*********************************REGISTRY_SERVICE: " + registry_service + " ************************************");
    			System.out.println("*********************************NODE_PROPERTIES: " + node_properties + " ************************************");
    			
    			System.setProperty("org.ccnx.config.UserName", peerID);
    			// starting look-up and cache thread
    			if(lc == null){
    				lc = LCServer.getIstance();
    				lc.initialize(conf_lcServer, nrs_port, node_properties);
    				lc.start();
    			}
    			
    			// it's safe waiting a little bit prior of opening ccnx handles
    			try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
    			
    			ccn_handleA = CCNHandle.open();
				ccn_handleS = CCNHandle.open();
    			
				
    			ccnPrefix = "ccnx:/";
    			
    			// registering own service name [ccnx sendToName] to NRS
    			lc.getRh().unregister("/" + peerID + "/");
    			try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
    			lc.getRh().register("/" + peerID + "/");
    			ccnServiceName = ccnPrefix + peerID + "/" + stn_service + "/";
    			
    			// register own sendToName listener
				ccn_handleS.registerFilter(ContentName.fromURI(ccnServiceName), this);                
        	} catch (IOException e) {
    			System.out.println("IOException during ConetTE initialization");
    			ccnx = null;
    			e.printStackTrace();
    		} catch (org.ccnx.ccn.config.ConfigurationException e) {
    			System.out.println("ConfigurationException during ConetTE initialization");
				e.printStackTrace();
				ccnx = null;
			} catch (MalformedContentNameStringException e) {
				e.printStackTrace();
			}
            
        }
    }
    
    @Override
	public int advertiseSap(NID nid, SAPListener callBack) {
		return advertiseSap(nid, callBack, NAMED_DATA_AS_BYTE_ARRAY);
	}

	@Override
	public int advertiseSap(NID nid, SAPListener callBack, int localDeliveryType) {
		if((ccnx != null) && (ccnx.compareTo("ccnx") == 0)){
			System.out.println("Advertising for " + nid.toString());

			// register callback objects
			if(nid.toString().contains("overlay"))
				propagations_handler = callBack;
			else
				ers_handler = callBack;
		}
		else
			conetTeSmileRunTimeContainer.sendToNameImpl.advertiseSap(nid, callBack, localDeliveryType);


		return 0;
	}

	@Override
	public int get(NID nidObj, String localFileName) throws ConetIOException {
		String nidString=nidObj.toString();
		if((ccnx != null) && (ccnx.compareTo("ccnx") == 0)){
			String ccnx_name = null;
			CCNInputStream in = null;
			FileOutputStream fout = null;
			
			try
			{
				if(!nidString.startsWith(ccnPrefix))
					ccnx_name = ccnPrefix + nidString;
				else
					ccnx_name = nidString;
				
				System.out.println("Getting " + ccnx_name);
				long get_timeout = 10000;

				ContentObject ccnObj = ccn_handleA.get(ContentName.fromURI(ccnx_name), get_timeout);
				if(ccnObj == null)
					return -1;
				
				// by now all the content apart from the registry files will be put as unversioned into the serving node repository
				// registry is a versioned content object, so we need a different inputstream to fetch it
				if(nidObj.toString().contains("registry"))
					in= new CCNVersionedInputStream(ContentName.fromURI(ccnx_name), ccn_handleA);
				else
					in= new CCNInputStream(ContentName.fromURI(ccnx_name), ccn_handleA);
				
				fout = new FileOutputStream(localFileName);

				byte data[] = new byte[1024];
				int count;
				while ((count = in.read(data, 0, 1024)) != -1)
				{
					fout.write(data, 0, count);
				}
			} 
			catch (MalformedContentNameStringException e) {
				System.out.println("Malformed content name");
				e.printStackTrace();
				return -1;
			} catch (IOException e) {
				e.printStackTrace();
				return -1;
			}
			finally
			{
				if (in != null) {
					try {
						in.close();
					} catch (IOException e) {
						e.printStackTrace();
					}				
				}
				else{
					System.out.println("Get from the Conet not performed");
					return -1;
				}
				if (fout != null) {
					try {
						fout.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
				else{
					System.out.println("File not saved");
					return -1;
				}
			}
		}
		else{
			BufferedInputStream in = null;
			FileOutputStream fout = null;
			String urlString=downloadServlet+"?NID="+nidString;
			try
			{
				in = new BufferedInputStream(new URL(urlString).openStream());
				fout = new FileOutputStream(localFileName);

				byte data[] = new byte[1024];
				int count;
				while ((count = in.read(data, 0, 1024)) != -1)
				{
					fout.write(data, 0, count);
				}
			} catch (MalformedURLException e) {
				System.out.println("Invalid URL "+urlString+": "+e.getMessage());
				return -1;
			} catch (IOException e) {
				System.out.println("Unable to execute "+urlString+": "+e.getMessage());
				return -1;
			}

			finally
			{
				if (in != null) {
					try {
						in.close();
					} catch (IOException e) {
						e.printStackTrace();
						return -1;
					}
				} else {
					System.out.println("File not saved");
					return -1;
				}
				if (fout != null) {
					try {
						fout.close();
					} catch (IOException e) {
						e.printStackTrace();
						return -1;
					}
				} else {
					System.out.println("File not saved");
					return -1;
				}

			}
		}
		
	  return 0;
	}

	public ConetTE getConetTE(String processConfFile) {
		if(ccnx == null || (ccnx.compareTo("ccnx") != 0)){
			ConetTE newConet = new ConetTE();
			newConet.initialise(this.engineID, this.mxmParameters,this.isDefault());
			newConet.conetTeSmileRunTimeContainer = new ConetTeSmileRunTimeContainer(processConfFile);
			newConet.conetTeSmileRunTimeContainer.init();
			return newConet;
		}
		else{
			if(conetTE == null){
				conetTE = new ConetTE();
				conetTE.initialise(this.engineID, this.mxmParameters,this.isDefault());
			}
			return conetTE;
		}		
	}

	public ConetTE getConetTE(String processConfPath, String processConfFile) {
		if(ccnx == null || (ccnx.compareTo("ccnx") != 0)){				
			ConetTE newConet = new ConetTE();
			newConet.initialise(this.engineID, this.mxmParameters,this.isDefault());
			newConet.conetTeSmileRunTimeContainer = new ConetTeSmileRunTimeContainer(processConfPath, processConfFile);
			newConet.conetTeSmileRunTimeContainer.init();
			return newConet;
		}
		else{
			if(conetTE == null){
				conetTE = new ConetTE();
				conetTE.initialise(this.engineID, this.mxmParameters,this.isDefault());
			}
			return conetTE;
		}			
	}

	@Override
	public ConetTE getConetTE() 
	{
		if( ccnx != null || ccnx.compareTo("ccnx") == 0){
				if(conetTE == null){
					conetTE = getConetTE(null);
					conetTE.initialise(this.engineID, this.mxmParameters,this.isDefault());
				}
				return conetTE;
		}
		else{
			return getConetTE(null);					
		}
	}

	
	@Override
	public String getEngineName() {
		
		return ConvergenceEngineName.ConetTE.toString();
	}

	public static String getPeerID() {
		return peerID;
	}

	public String getCcnx() {
		return ccnx;
	}

	public static String getStn_service() {
		return stn_service;
	}

	public String nid2FileName (String nidString) {
	    String fileName = new String();
	    String principal = new String();
	    String label = new String();
	    if( nidString.lastIndexOf('/') >= 0 ){
	    	principal=nidString.substring(0,nidString.indexOf('/')); 
	    	label=nidString.substring(nidString.indexOf('/'));
	    	label=label.replace( '/', '#' );
	    }else{
	    	principal=nidString; 
	    	label="";
	    }
	    fileName = principal+label;
	    return fileName;
	 }

	@Override
	public int sendToName(NID nid, byte[] unNamedData) {
		if((ccnx != null) && (ccnx.compareTo("ccnx") == 0)){
			int index = nid.toString().indexOf('.');
			String msgID = UUID.randomUUID().toString();
			// now the NID has the following form peerc1AS1.peers.overlay.ict-convergence.eu/cpkc1AS1, we need to extract c1AS1
			String destination_peer = nid.toString().substring(4, index);
			String data_name = null;
			String peerName = null;

			if(nid.toString().contains("overlay")){
				peerName = ccnPrefix + destination_peer + "/" + stn_service + "/" + peerID + "/" + msgID;
				data_name = ccnPrefix + peerID + "/" + msgID;
			}
			else if(nid.getLabel().contains("er")){
				peerName = ccnPrefix + destination_peer + "/" + stn_service + "/" + peerID + "/ER/" + msgID;
				data_name = ccnPrefix + peerID + "/ER/" + msgID;
			}

			// we must express an interest towards ccnx:/destinationpeerId/nsap/ownPeerId/msgId
			ContentName contentName = null;
			ContentObject dataObject = null;
			try {
				contentName = ContentName.fromURI(data_name);
				// if you use Integer.MAX_VALUE as freshness for this contentObject, the daemon ccnd will raise a warning for a too large freshness
				// buildContentObject(ContentName name, ContentType type, byte[] contents, PublisherPublicKeyDigest publisher, KeyLocator locator, KeyManager keymanager, Integer freshnessSeconds, byte[] finalBlockID)
				// if keymanager is null, it uses the KeyManager.getDefaultKeyManager
				// if publisher is null,  it uses signingKey = keyManager.getDefaultSigningKey(); and publisher = keyManager.getPublisherKeyID(signingKey)
				// if locator is null, it uses locator = keyManager.getKeyLocator(signingKey);
				KeyLocator keyLoc = null;
				PublisherPublicKeyDigest pubkey = null;	

				dataObject = ContentObject.buildContentObject(contentName, ContentType.DATA, unNamedData, pubkey, keyLoc, null, new Integer(1000), null);	
			} catch (MXMInvalidConfigurationException e) {
				e.printStackTrace();
				return -1;
			} catch (MalformedContentNameStringException e) {
				e.printStackTrace();
				return -1;
			}
			SendToNameThreadC stnT = new SendToNameThreadC(peerName, dataObject, ccn_handleA,data_name);
			stnT.start();


			return 0;
		}
		else
			return -1;	
	}

	@Override
	public int sendToName(NID nid, String unNamedDataAsString) {
		SendToNameOneWay msg = new SendToNameOneWay();
		msg.setRequest(unNamedDataAsString);
		try {
			if (conetTeSmileRunTimeContainer.sendToNameImpl.sendToNameClient!=null) {
				System.out.println("SENDING MESSAGE to: "+nid);
				conetTeSmileRunTimeContainer.sendToNameImpl.sendToNameClient.sendToName(SmileRunUtil.nid2Pid(nid), msg);
			}
		} catch (SendingMessageException e) {
			e.printStackTrace();
		} catch (Fault e) {
			e.printStackTrace();
		}

		return 0;
	}

	@Override
	public int sendToLocation(LID lid, byte[] unNamedData) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public int sendToLocation(LID lid, String unNamedDataAsString) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public int storeAndAdvertise(NID nidClass, int rType, String fileName) {
		String content_name = null;

		if((ccnx != null) && (ccnx.compareTo("ccnx") == 0)){
			
			// Store the file into the conet
			if(!nidClass.toString().startsWith(ccnPrefix))
				content_name = ccnPrefix + nidClass.toString();
			else
				content_name = nidClass.toString();

			// if this is a register, we have to store it into another peer through a 4-way mechanism in a similar way to what we do with sendToName
			if(nidClass.toString().contains("registry")){
				content_name.replace(":", "/");
				// extract the fractal name, who is the last component of the name
				String[] components = content_name.split("/");
				String registry_name = components[components.length-1];
				
				// we must express an interest towards ccnx:/registry/ownPeerId/registry/fractal_name
				String peerName = ccnPrefix + registry_service + "/" + peerID + "/" + registry_service + "/" + registry_name;
				String data_name = ccnPrefix + peerID + "/" + registry_service + "/" + registry_name;
				ContentName contentName = null;
				ContentObject dataObject = null;
				RandomAccessFile f;
				byte[] unNamedData = null;
				try {
					f = new RandomAccessFile(fileName, "r");
					unNamedData =  new byte[(int)f.length()];
					f.read(unNamedData);
				} catch (FileNotFoundException e1) {
					e1.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
				
				try {
					contentName = ContentName.fromURI(data_name);
					KeyLocator keyLoc = null;
					PublisherPublicKeyDigest pubkey = null;	

					dataObject = ContentObject.buildContentObject(contentName, ContentType.DATA, unNamedData, pubkey, keyLoc, null, new Integer(1000), null);	
				} catch (MXMInvalidConfigurationException e) {
					e.printStackTrace();
					return -1;
				} catch (MalformedContentNameStringException e) {
					e.printStackTrace();
					return -1;
				}
				SendToNameThreadC stnT = new SendToNameThreadC(peerName, dataObject, ccn_handleA,data_name);
				stnT.start();
			}
			else{
				return webStoreAndAdvertise(nidClass, rType, fileName);
			}
		}
		else
			return webStoreAndAdvertise(nidClass, rType, fileName);

		return 0;
	}

	@Override
	public int storeAndAdvertise(NID nidClass, int rType, String fileName,
			NID hostingNid) {
		// Currently hostingNID is not used
		return storeAndAdvertise(nidClass, rType, fileName);
    }

	private int webStoreAndAdvertise(NID nidClass, int rType, String fileName){
		String nidString = nidClass.toString();
		System.out.println("Storing and advertising: " + nidString);
		HttpClient httpclient = new DefaultHttpClient();
        try {
        	File f = new File(fileName);
            if (!f.exists()) return -1;
        	
            System.out.println("uploadServlet URL: "+ uploadServlet);
            HttpPost httppost = new HttpPost(uploadServlet);
            
            	
            FileBody bin = new FileBody(f,"application/octet-stream");
            StringBody nidStringBody;
            nidStringBody = new StringBody(nidString);
            MultipartEntity reqEntity = new MultipartEntity();
            reqEntity.addPart("NID", nidStringBody);
            reqEntity.addPart("bin", bin);
            

            httppost.setEntity(reqEntity);
            HttpResponse response = httpclient.execute(httppost);
            if (response.getStatusLine().getStatusCode()!=200) return -1;
        
        } catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				return -1;
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return -1;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return -1;
		} finally {
            try { httpclient.getConnectionManager().shutdown(); } catch (Exception ignore) {}
        }
        
		return 0;
	}
	@Override
	public int revoke(NID nidClass, int rType) {
		return revoke(nidClass, rType, null);
		
	}

	@Override
	public int revoke(NID nidClass, int rType, NID hostingNid) {
		// Currently hostingNID is not used

		String nidString = nidClass.toString();
		if((ccnx != null) && (ccnx.compareTo("ccnx") == 0)){
			// TODO perform the revoke operation into the ccnx network
		}
		else{
			String urlString=revokeServlet+"?NID="+nidString;
			try
			{
				URL url = new URL(urlString);
				HttpURLConnection c = (HttpURLConnection) url.openConnection();
				int code = c.getResponseCode();
				if (code == 404) {
					return -2; // File not found
				}
				if (code != 200) {
					return -1; // generic error
				}

			} catch (MalformedURLException e) {
				// Print out the exception that occurred
				System.out.println("Invalid URL "+urlString+": "+e.getMessage());
				return -1;
			} catch (IOException e) {
				// Print out the exception that occurred
				System.out.println("Unable to execute "+urlString+": "+e.getMessage());
				return -1;
			}
		}
		
		return 0;
	}

	@Override
	public void setDependencies(List<MXMEngine> engines)
			throws MXMEngineException {
		// TODO Auto-generated method stub
	}

	@Override
	public boolean handleInterest(Interest arg0) {
		System.out.println("Received Interest " + arg0.name().toURIString());
		String received_interest = arg0.name().toURIString();

		if(ers_handler != null && propagations_handler != null){
			System.out.println("Handlers not null");
			if(received_interest.contains(ConetTE.getPeerID() + "/" + stn_service + "/")){	
				SendToNameThreadS stns = new SendToNameThreadS(ers_handler, propagations_handler, ccn_handleS, received_interest,stn_service);
				System.out.println("SendToName starting");
				stns.start();
				return true;
			}
			return false;
		}
		else
			return false;

	}

}