/*
SMILE: a Simple Middleware Independent LayEr 
Author: Giovanni Bartolomeo

Copyright (C) 2007  University of Rome Tor Vergata - Dipartimento di Ingegneria Elettronica


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package it.uniroma2.smile.core;

/**
 * Interface defines the basic functions for any SIP process. 
 * A binding implements this or, more precisely, the extended Interface <i>process</i> 
 * @author Giovanni Bartolomeo
 */

public interface BasicFunctions  {    

	//used to publish process' own service
	public void publish(Descriptor descriptor) throws MalformedDescriptorException;

	/**
	 * Search for processes according to a descriptor 
	 * @param descriptorFiler to select the processes 
	 */    
	public ProcessID[] search(DescriptorFilter descriptorFilter);

	/** 
	 * Deregister (i.e. remove) a service from the directory 
	 *  
	 * @param descriptor 
	 */ 
	public void delete(Descriptor descriptor); 

	/** 
	 * Send a message asynchronously to a number of recipients concurrently 
	 * @param message the message to be send 
	 * @param receivers an array of reciever processIDs 
	 * @throws SendingMessageException 
	 */ 

	public void send(Message message, ProcessID[] receivers) throws SendingMessageException;

	/** 
	 * Send a message to a single recipient 
	 * @param message the message to be send 
	 * @param receivers an array of reciever processIDs 
	 * @throws SendingMessageException 
	 */ 

	public void send(Message message, ProcessID receiver) throws SendingMessageException;

	/** 
	 *  
	 * @param request 
	 * @param receiver 
	 * @return 
	 * @throws Fault 
	 * @throws SendingMessageException 
	 */ 

	public Message doRequest(Message request, ProcessID receiver) throws Fault, SendingMessageException;

	//call-backs
	public void onReceived(ProcessID sender,Message message, ProcessID receiver);    
	public Message onRequestReceived(ProcessID sender,Message request);     

}
