/*
SMILE: a Simple Middleware Independent LayEr 
Author: Giovanni Bartolomeo

Copyright (C) 2007  University of Rome Tor Vergata - Dipartimento di Ingegneria Elettronica


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package it.uniroma2.smile.core;

import java.util.Hashtable;


/**
 * A broker associates the SMILE peers with a (binding specific) process.<br>
 * The broker delivers message among peers
 * @author Giovanni Bartolomeo
 */
public class Broker {
	
	public static Hashtable<Object, Object> processTable;
	private static Broker defaultBroker=new Broker();
	
	private Broker() {
		if (processTable==null) processTable=new Hashtable<Object, Object>();
	};
	
	public static Broker getDefaultBroker() {		
		return defaultBroker;
	}

	//commands - invoked by third party applications	
	public Message doRequest(SMILEPeer bp,Message request, ProcessID receiver) throws Fault,
			SendingMessageException {		
		if (processTable.containsKey(bp.getUUID())) {
			return ((Process)processTable.get(bp.getUUID())).doRequest(request, receiver);
		} else {		
			error(bp.getUUID());
		}
		return null;
	}


	public void send(SMILEPeer bp, Message message, ProcessID[] receivers)
			throws SendingMessageException {		
		if (processTable.containsKey(bp.getUUID())) {
			((Process)processTable.get(bp.getUUID())).send(message, receivers);
		} else {
			error(bp.getUUID());
		}
	}

	
	public void send(SMILEPeer bp, Message message, ProcessID receiver)
			throws SendingMessageException {	
		if (processTable.containsKey(bp.getUUID())) {
			((Process)processTable.get(bp.getUUID())).send(message, receiver);
		} else {
			error(bp.getUUID());
		}
	}

	public void delete(SMILEPeer bp, Descriptor descriptor) {
		if (processTable.containsKey(bp.getUUID())) {
			((Process)processTable.get(bp.getUUID())).delete(descriptor);
		} else {
			error(bp.getUUID());
		}
	}

	public void publish(SMILEPeer bp, Descriptor descriptor)
			throws MalformedDescriptorException {
		if (processTable.containsKey(bp.getUUID())) {
			((Process)processTable.get(bp.getUUID())).publish(descriptor);
		} else {
			error(bp.getUUID());
		}
	}
	

	public ProcessID[] search(SMILEPeer bp, DescriptorFilter descriptorFilter) {
		if (processTable.containsKey(bp.getUUID())) {
			return ((Process)processTable.get(bp.getUUID())).search(descriptorFilter);
		} else {
			error(bp.getUUID());
		}
		return null;
	}	
	
	//callbacks - the bindings invoke these method in order to notify the
	//broker whenever a message has been received
	public void onReceived(Process process, ProcessID sender,
			Message message) {
		if (processTable.containsKey(process)) {
			((SMILEPeer)processTable.get(process)).onReceived(sender, message, process.getProcessID());
		} else {
			errorOnCallback(process.getProcessID());
		}		
	}

	public Message onRequestReceived(Process process,
			ProcessID sender, Message request) {
		if (processTable.containsKey(process)) {
			return ((SMILEPeer)processTable.get(process)).onRequestReceived(sender, request);
		} else {		
			errorOnCallback(process.getProcessID());
		}	
		return null;
	}	

	public void bindProcess(SMILEPeer boundProcess, Process process) {
		//temp code: actually automatically inject
		//one binding for each BoundProcess		

		// MOVED into the constructor
		//		if (processTable==null) processTable=new Hashtable<Object, Object>();

		processTable.put(boundProcess.getUUID(), process);
		processTable.put(process, boundProcess);
		processTable.put(process.getProcessID().toString(), process);
		//System.out.println(process.getProcessID().toString());
	}

	public void unbindProces(SMILEPeer boundProcess) {
		Process process;
		try {
			process=(Process)processTable.get(boundProcess.getUUID());
			processTable.remove(boundProcess.getUUID());
			processTable.remove(process);
		} catch (NullPointerException ex) {		
			error(boundProcess.getUUID());
		}		
	}		
	
	private static void errorOnCallback(ProcessID pid) {
		//TODO: Broker should raise exceptions instead of error message and halt.
		System.out.println("ERROR: cannot find binding " + pid +
				" into process table; ");
//		System.exit(0);		
	}
	
	private static void error(String uuid) {
		//TODO: Broker should raise exceptions instead of error message and halt.
		System.out.println("ERROR: cannot find any binding for " + uuid +
				" into process table; ");
//		System.exit(0);		
	}

	/**
	 * returns the PID of a Smile Peer as registered in the processTable<br>
	 * if the process table does not contain the process it returns null
	 * @param boundProcess
	 * @return
	 */
	public static ProcessID[] getPID(SMILEPeer boundProcess) {
		Process process;
		try {
			process=(Process)processTable.get(boundProcess.getUUID());
		} catch (NullPointerException ex) {
			System.out.println("[Broker] NullPointerException therefore getPID returns null");
			return null;	
		}		
		if (process==null) {
			return null;
		} else {
			return new ProcessID[] {process.getProcessID()};
		}
	}

}
