/*
 * 
 * 
 * FILE: Util.java 
 * DESCRIPTION: 
 * This file contains some utility methods related to SMILE framework
 * RESPONSIBLE: Stefano Salsano (CNIT, stefano.salsano@uniroma2.it) 
 * 
 * ACKNOWLEDGEMENT:
 * The PERIMETER Project (FP7 - 224024) is co-funded by the European Commission under the 7th Framework Programme for RTD.  
 * 
 * http://www.ict-perimeter.eu/
 *  
 */
package it.uniroma2.smile.run;
import java.io.File;
import java.io.FileInputStream;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import it.uniroma2.smile.core.ProcessID;
import it.uniroma2.smile.core.SMILEPeer;
import it.uniroma2.smile.sipbinding.SipProcess;
import it.uniroma2.smile.sipbinding.SipProcessID;

import org.zoolu.sip.address.NameAddress;
import org.zoolu.sip.address.SipURL;

import eu.convergence.middleware.engine.ConetTe.services.NID;



/**
 * This class provides a set of static methods related to the SMILE framework
 * @author CNIT
 *
 */
public class SmileRunUtil {

//	static String SIP_REGISTER_DOMAIN = "ict-convergence.eu";
	
	//	public static Broker broker;
	//	static String CONFIG_PATH = "cfg";
	//	static String PROCESS_CONF_FILE = "sip.cfg";
	//	static int SIP_PROCESS_TIMEOUT = 32000; //(MS)

	static BaseRunTimeContainer myRunExample ;
	
	static int NO_RUN=0;
	static int RUN_NO_WAIT=1;
	static int RUN_WAIT_SETUP=2;
	

	private static String defaultAor=null;
	
	
	public static void setRunTimeContainer(BaseRunTimeContainer myRunEx){
		myRunExample =  myRunEx;	
	}

	public static String configPath() {
		return myRunExample.CONFIG_PATH;
	}
	
	public static String processConfFile() {
		return myRunExample.PROCESS_CONF_FILE;
	}

	public static String pidsConfFile() {
		return myRunExample.PIDS_CONF_FILE;
	}
	
	public static void advertiseNID (SMILEPeer serverPeer, NID nid) {
		startSipProcAndBindPeer(false, null, false, serverPeer, nid2Pid (nid));
	}
	
	/**
	 * it is tailored to the need of convergence DEMO !<br>
	 * it uses the register address as the host part of the SIP address of the NID
	 * @param nid
	 * @return
	 */
	public static ProcessID nid2Pid (NID nid) {
//		return new SipProcessID(nid.getPrincipal()+"@"+SIP_REGISTER_DOMAIN+";ptype=sendToName;pname="+nid.getLabel());
		return new SipProcessID(nid.getPrincipal()+"@"+getDefaultRegistrar()+";ptype=sendToName;pname="+nid.getLabel());
		
	}
	
	/**
	 * it is tailored to the need of convergence DEMO !
	 * @param nid
	 * @return
	 */
	public static ProcessID strNid2Pid (String strNid) {
		return nid2Pid (new NID (strNid));
	}
	

	/**
	 * it is tailored to the need of convergence DEMO !
	 */
	public static String pid2StrNid (ProcessID pid) {
		return pid2Nid(pid).toString();
	}

	/**
	 * it is tailored to the need of convergence DEMO !
	 */
	public static NID pid2Nid (ProcessID pid) {
		SipURL pidURL = ((SipProcessID)pid).getNameAddress().getAddress();
		return new NID (pidURL.getUserName(),pidURL.getParameter("pname")) ;
	}

	
	/**
	 * starts a sip process and binds it with a SMILE server peer
	 * @param runPeer
	 * @param lockDoBusiness
	 * @param synchrSetup
	 * @param serverPeer
	 * @param ptype
	 * @param pname
	 * @return
	 */
	public static SipProcess startSipProcAndBindPeer(boolean runPeer, Object lockDoBusiness,
			boolean synchrSetup, SMILEPeer serverPeer, String ptype, String pname) {
		return startSipProcAndBindPeer (runPeer, lockDoBusiness, synchrSetup, serverPeer, ptype, pname, null); 
	}

	/** 
	 * this method takes a pid instead of pname and ptype
	 * @param b
	 * @param lockDoBusiness
	 * @param synchrSetup
	 * @param mainPeer
	 * @param pid
	 */
	public static SipProcess startSipProcAndBindPeer(boolean runPeer,
			Object lockDoBusiness, boolean synchrSetup, SMILEPeer serverPeer, ProcessID pid) {
		return startSipProcAndBindPeer (runPeer, lockDoBusiness, synchrSetup, serverPeer, null, null, pid); 
		
	}

	public static SipProcess startSipProcAndBindPeer(boolean runPeer, Object lockDoBusiness,
			boolean synchrSetup, SMILEPeer serverPeer, String ptype, String pname, ProcessID pid) {

		SipProcess sipProc= null;
		if (pid==null) {
			String params = (pname==null?"":"pName="+pname);
			if (ptype!=null){
				if (!params.equals("")){
					params = params + ";";
				}
				params = params + "pType="+ptype;
			}
			//params = params + (pname==null?"":"pName="+pname+";"); 
			sipProc=new SipProcess(configPath()+File.separator+processConfFile(),params,myRunExample.SIP_PROCESS_TIMEOUT);
		} else {
			sipProc=new SipProcess(configPath()+File.separator+processConfFile(),(SipProcessID) pid, myRunExample.SIP_PROCESS_TIMEOUT);
		}
		
		Object token2UnlockAfterSetup = null;

		myRunExample.broker.bindProcess(serverPeer, sipProc);
		if (runPeer) {
			
			if (synchrSetup) {
				token2UnlockAfterSetup = new Object();
				synchronized (token2UnlockAfterSetup) {
					new RunProcess(serverPeer, lockDoBusiness, token2UnlockAfterSetup).start();
					try {
						token2UnlockAfterSetup.wait();
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			} else {
				new RunProcess(serverPeer, lockDoBusiness, token2UnlockAfterSetup).start();
			}
			
		}
		return sipProc;
	}
	
	


//	/**
//	 * Creates a SIP PID from a string<br>
//	 * deprecated, as it can be replaced by the constructor or SipProcessID which
//	 * takes a string as argument
//	 * @param str
//	 * @return
//	 */
//	public static SipProcessID str2SipPid (String str) {
//		SipProcessID sPid = new SipProcessID();
//		sPid.setNameAddress(new NameAddress(str));
//		return sPid;
//	}

	/**
	 * Creates a SIP PID from a baseAOR, a ptype and a pname
	 * @param baseAOR
	 * @param ptype
	 * @param pname
	 * @return
	 */
	public static SipProcessID sipPid(String baseAOR, String ptype, String pname) {
		return new SipProcessID (baseAOR+";pName="+pname+";pType="+ptype);
		//return str2SipPid(baseAOR+";pName="+pname+";pType="+ptype);
	}

	/**
	 * Creates a SIP PID for a peer running in the same run-time container (i.e. a "local" peer)
	 * @param ptype
	 * @param pname
	 * @return
	 */
	public static SipProcessID localSipPid( String ptype, String pname) {
		return sipPid(getDefaultAor(),ptype,pname);
	}

// Stefano removed because it was identical to str2SipPid
//	//@eileen added following for Storage to function
//	public static SipProcessID target(String target) {
//		SipProcessID sPid = new SipProcessID();
//		sPid.setNameAddress(new NameAddress(target));
//		return sPid;
//	}
	
	/**
	 * gets the default Address of Record for the run-time container
	 */
	public static String getDefaultAor() {
		if (defaultAor==null) {
			String user = readStrFromPropertyFile("user",processConfFile());
			String registrar = readStrFromPropertyFile("registrar",processConfFile());
			defaultAor = user+"@"+registrar;
		}
		return defaultAor;
	}

	
	public static String getDefaultRegistrar() {
		String aor = getDefaultAor();
		String[] array = aor.split("@");
		if (array.length>0){
			return array[1];
		} else {
			return "";
		}
	}

	
	/**
	 * 
	 * @param ptype
	 * @param pname
	 * @param mainPeer
	 * @param lockDoBusiness if not null, the doBusiness method of the mainPeer and of the active components (if any)
	 *        is launched when the object is notified
	 * @param token2UnlockAfterSetup if (true/ not null) the setup method of the mainPeer and of the active components (if any)  
	 *        is executed in a synchronous way, e.g. the startComponent does not return until the setup methods are
	 *        all finished
	 * @param passiveSmilePeerArray
	 */
	public static void startComponent (String ptype, String pname,  SMILEPeer mainPeer, Object lockDoBusiness, boolean synchrSetup, SMILEPeer[] passiveSmilePeerArray) {
		//		startComponent (ptype, pname, mainPeer, passiveSmilePeerArray, (PeerTypeName[]) null);
		startComponent (ptype, pname, mainPeer, lockDoBusiness, synchrSetup, passiveSmilePeerArray, (Object[][]) null);
	}

	
	
	public static void startComponent (String ptype, String pname,  SMILEPeer mainPeer, SMILEPeer[] passiveSmilePeerArray) {
		//		startComponent (ptype, pname, mainPeer, passiveSmilePeerArray, (PeerTypeName[]) null);
		startComponent (ptype, pname, mainPeer, null, false, passiveSmilePeerArray, (Object[][]) null);
	}

	//	/**
	//	* It starts a component which is made up of a server (mainPeer), of a
	//	* set of passive client peers (in the passiveSmilePeerArray) and a set of active peers that can be server
	//	* or client (in the activeSmilePeerArray)<br>
	//	* It creates a SipProcess taking the base AOR from CONFIG_PATH+PROCESS_CONF_FILE and using
	//	* the provided ptype and pname<br>
	//	* It binds the server to the SipProcess and runs the server thread.<br>
	//	* It binds the clients to the same SipProcess<br>
	//	* Finally it provides to the mainPeer the references to the clients using the setReference2Peers method
	//	* @param ptype
	//	* @param pname
	//	* @param mainPeer
	//	* @param smilePeerArray
	//	*/
	//	public static void startComponent (String ptype, String pname,  SMILEPeer mainPeer, SMILEPeer[] passiveSmilePeerArray, PeerTypeName[] activePeerTypeNameArray) {
	//	//		SipProcess sipProcServer = startSipProcAndBindPeer(true,mainPeer,ptype,pname);
	//	startSipProcAndBindPeer(true,mainPeer,ptype,pname);

	//	for (int i=0; i<passiveSmilePeerArray.length ;i++){
	//	startSipProcAndBindPeer(false,passiveSmilePeerArray[i],"client"+i+"-in-"+ptype,pname);
	//	//			broker.bindProcess(smilePeerArray[i], sipProcServer);
	//	}

	//	SMILEPeer [] activePeerArr=null;
	//	if (activePeerTypeNameArray!=null) {
	//	activePeerArr = new  SMILEPeer [activePeerTypeNameArray.length];
	//	for (int i=0; i<activePeerTypeNameArray.length ;i++){
	//	activePeerArr [i] = activePeerTypeNameArray[i].sp;
	//	startSipProcAndBindPeer(true,activePeerTypeNameArray[i].sp,activePeerTypeNameArray[i].ptype,(activePeerTypeNameArray[i].pname==null?pname:activePeerTypeNameArray[i].pname));
	//	}
	//	}

	//	mainPeer.setReference2Peers(passiveSmilePeerArray);

	//	if (activePeerArr!=null) {
	//	mainPeer.setReference2ActivePeers(activePeerArr);
	//	}
	//	}

	
	/**
	 * It starts a component which is made up of a server (mainPeer), of a
	 * set of passive client peers (in the passiveSmilePeerArray) and a set of active peers that
	 * can be server or client (in the activeSmilePeerArray)<br>
	 * it receives a vector of processIDs which can be empty and uses it to create a set of
	 * SIP processes that receive messages on the given address and invoke the methods on the mainPeer<br>
	 * as for the active clients it creates the SipProcesses taking the base
	 * AOR from CONFIG_PATH+PROCESS_CONF_FILE and using the provided ptype and pname (if provided)<br>
     * it run the server thread only once (and runs it also if the vector is empty<br>
	 * It binds the server peer to the SipProcesses (if the vector is not empty) note that the server 
	 * peer is not bound to any SipProcess if the  vector is empty<br>
	 * It binds the clients to the same SipProcess<br>
	 * Finally it provides to the mainPeer the references to the clients using the setReference2Peers method
	 * @param mainPeer
	 * @param passiveSmilePeerArray
	 * @param activePeersPtypePnameArray
	 * @param lockDoBusiness if not null, the doBusiness method of the mainPeer and of the active components (if any)
	 *        is launched when the object is notified
	 * @param synchrSetup if (true/ not null) the setup method of the mainPeer and of the active components (if any)  
	 *        is executed in a synchronous way, e.g. the startComponent does not return until the setup methods are
	 *        all finished
	 */
	public static void startComponentListOfServerAddr (Vector<ProcessID> sipServerAddresses,  SMILEPeer mainPeer, Object lockDoBusiness, boolean synchrSetup, SMILEPeer[] passiveSmilePeerArray, Object[][] activePeersPtypePnameArray) {
		//		SipProcess sipProcServer = startSipProcAndBindPeer(true,mainPeer,ptype,pname);
		
		
		Object token2UnlockAfterSetup = null;
		if (synchrSetup) {
			token2UnlockAfterSetup = new Object();
			synchronized (token2UnlockAfterSetup) {
				new RunProcess(mainPeer, lockDoBusiness, token2UnlockAfterSetup).start();
				try {
					token2UnlockAfterSetup.wait();
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		} else {
			new RunProcess(mainPeer, lockDoBusiness, token2UnlockAfterSetup).start();
		}
		
		mainPeer.setReference2Peers(passiveSmilePeerArray);
		for (ProcessID pid : sipServerAddresses) {
			startSipProcAndBindPeer(false,lockDoBusiness,synchrSetup,mainPeer, pid);
		}
		
//		for (int i=0; i<passiveSmilePeerArray.length ;i++){
//			startSipProcAndBindPeer(false,null,false,passiveSmilePeerArray[i], "client"+i+"-in-"+ptype, pname);
//			//			broker.bindProcess(smilePeerArray[i], sipProcServer);
//		}

		SMILEPeer [] activePeerArr=null;
		if (activePeersPtypePnameArray!=null) {
			activePeerArr = new  SMILEPeer [activePeersPtypePnameArray.length];
			for (int i=0; i<activePeersPtypePnameArray.length ;i++){
				activePeerArr [i] = (SMILEPeer) activePeersPtypePnameArray[i][0];
			
				startSipProcAndBindPeer(true,
						lockDoBusiness,
						synchrSetup,
						(SMILEPeer) activePeersPtypePnameArray[i][0], (((Object[])activePeersPtypePnameArray[i]).length>1?(String)activePeersPtypePnameArray[i][1]:null), (((Object[])activePeersPtypePnameArray[i]).length>2?(String)activePeersPtypePnameArray[i][2]:null)
				);
			}
		}

		if (activePeerArr!=null) {
			mainPeer.setReference2ActivePeers(activePeerArr);
		}
	}

	/**
	 * It starts a component which is made up of a server (mainPeer), of a
	 * set of passive client peers (in the passiveSmilePeerArray) and a set of active peers that can be server
	 * or client (in the activeSmilePeerArray)<br>
	 * It creates a SipProcess taking the base AOR from CONFIG_PATH+PROCESS_CONF_FILE and using
	 * the provided ptype and pname<br>
	 * It binds the server to the SipProcess and runs the server thread.<br>
	 * It binds the clients to the same SipProcess<br>
	 * Finally it provides to the mainPeer the references to the clients using the setReference2Peers method
	 * @param ptype
	 * @param pname
	 * @param mainPeer
	 * @param passiveSmilePeerArray
	 * @param activePeersPtypePnameArray
	 */
	public static void startComponent (String ptype, String pname,  SMILEPeer mainPeer, Object lockDoBusiness, boolean synchrSetup, SMILEPeer[] passiveSmilePeerArray, Object[][] activePeersPtypePnameArray) {
		//		SipProcess sipProcServer = startSipProcAndBindPeer(true,mainPeer,ptype,pname);
		
		mainPeer.setReference2Peers(passiveSmilePeerArray);
		startSipProcAndBindPeer(true,lockDoBusiness,synchrSetup,mainPeer, ptype, pname);
		
		for (int i=0; i<passiveSmilePeerArray.length ;i++){
			startSipProcAndBindPeer(false,null,false,passiveSmilePeerArray[i], "client"+i+"-in-"+ptype, pname);
			//			broker.bindProcess(smilePeerArray[i], sipProcServer);
		}

		SMILEPeer [] activePeerArr=null;
		if (activePeersPtypePnameArray!=null) {
			activePeerArr = new  SMILEPeer [activePeersPtypePnameArray.length];
			for (int i=0; i<activePeersPtypePnameArray.length ;i++){
				activePeerArr [i] = (SMILEPeer) activePeersPtypePnameArray[i][0];
			
				startSipProcAndBindPeer(true,
						lockDoBusiness,
						synchrSetup,
						(SMILEPeer) activePeersPtypePnameArray[i][0], (((Object[])activePeersPtypePnameArray[i]).length>1?(String)activePeersPtypePnameArray[i][1]:null), (((Object[])activePeersPtypePnameArray[i]).length>2?(String)activePeersPtypePnameArray[i][2]:null)
				);
			}
		}

		if (activePeerArr!=null) {
			mainPeer.setReference2ActivePeers(activePeerArr);
		}
	}

	/**
	 * Reads a property from a given property file, using the default
	 * path for the configuration folder. Note that the default path can be changed
	 * in the constructor of the class that extends BaseRunExample
	 * @param propertyToRead 
	 * @param fileName
	 * @return
	 */
	public static String readStrFromPropertyFile(String propertyToRead, String fileName) {
		return readStrFromPropertyFileFullPath(propertyToRead, configPath()+File.separator+fileName) ;
	}    

	/**
	 * Reads a property from a given property file, which is given
	 * using the full path
	 * @param propertyToRead 
	 * @param fileName
	 * @return
	 */
	public static String readStrFromPropertyFileFullPath(String propertyToRead, String fileName) {
		String retStr = null;	
		FileInputStream fin;
		try {
			fin = new FileInputStream(fileName);
			Properties props = new Properties();
			props.load(fin);
			for (Enumeration e = props.keys() ; e.hasMoreElements() ;) {
				String sdName=(String)e.nextElement();
				if (sdName.startsWith(propertyToRead)) {  
					retStr=(String)props.get(sdName);
					System.out.println("[Util] Found property "+propertyToRead+"="+retStr);
				}	
			}
		} catch (Exception e1) {
			e1.printStackTrace();
		}
		if (retStr == null) { 
			System.out.println("FATAL: Property "+propertyToRead+" not present in config file "+fileName); 
			System.exit(2); 
		}		
		return retStr;
	}    

	
}
