/*
SMILE: a Simple Middleware Independent LayEr 
Author: Giovanni Bartolomeo

Copyright (C) 2007  University of Rome Tor Vergata - Dipartimento di Ingegneria Elettronica


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package it.uniroma2.smile.sipbinding;

import it.uniroma2.smile.core.ProcessID;
import it.uniroma2.smile.sipbinding.fragmentation.SlidingWindow;

import java.util.Hashtable;

import local.ua.UserAgentProfile;

import org.zoolu.sip.address.NameAddress;
import org.zoolu.sip.address.SipURL;
import org.zoolu.sip.call.RegistrationClient;
import org.zoolu.sip.call.RegistrationClientListener;
import org.zoolu.sip.message.Message;
import org.zoolu.sip.message.MessageFactory;
import org.zoolu.sip.message.SipResponses;
import org.zoolu.sip.provider.SipProvider;
import org.zoolu.sip.provider.SipProviderListener;
import org.zoolu.sip.provider.SipStack;
import org.zoolu.sip.transaction.TransactionClient;
import org.zoolu.sip.transaction.TransactionServer;
import org.zoolu.tools.Log;
import org.zoolu.tools.LogLevel;
/**
 * 
 * @author Giovanni Bartolomeo, Francesco Caputo, Stefano Salsano
 */
public class Dispatcher  implements SipProviderListener, RegistrationClientListener {

	/** Event logger. */
	Log log;


	/** RegistrationClient */
	RegistrationClient rc;

//	/** Remote user. */
//	NameAddress remote_user ;
	
	/** 
	 *  user agent profile used to set the base SIP URI
	 */
	UserAgentProfile userAgentProfile;

	//private SipBaseProcess sbp;

	private final boolean DEBUG=true;

	//MyMessageAgent mma;

	/**
	 * reference to the SIP provider: different dispatchers may refer to the same 
	 * SIP provider<br>
	 * for the purpose of CONVERGENCE early demo, we assume that there will be a single
	 * SIP provider, therefore SipProvider becomes static
	 * 
	 */
	private static SipProvider sp = null;	

	/**
	 * associates a SIP base URI to a dispatcher<br>
	 * the string MUST be obtained with NameAddress.toString 
	 * it is static across all dispatcher instances
	 */
	private static Hashtable<String, Dispatcher> configTable=new Hashtable<String, Dispatcher>();

	/**
	 * associates the string representation of a PID to the SipBaseProcess<br>
	 * each dispatcher can handle a set of PIDs<br>
	 * when a message is received, the To: address is used to lookup in this table and find the process
	 * to which the message will be delivered
	 */
	private Hashtable<String, SipBaseProcess> lookUpTable;


	private boolean registered=false;
	private boolean deRegistering=false; 



	/* 09.07.08 GB
	private Dispatcher(String configFileName) {
		this(new SipProvider(configFileName),new UserAgentProfile(configFileName));			
	}
	*/
	

	/** Creates a new instance of Dispatcher, 
	 * each instance of a Dispatcher has its own reference to a SipProvider, 
	 * its own userAgentProfile, its own registration Client <br>
	 * a sip Provider is started if not existing taking configuration from config file<br>
	 * the dispatcher that creates the SIP provider becomes the listener for this sip provider
	 * it has the task to look for a specific receiver 
	 * as a side effect, the contact information in the uap is updated<br>
	 * if the Dispatcher is created only from the configuration file
	 * the uap of this Dispatcher gets the contact address from the information contained in the file
	 * if the Dispatcher is created also providing a sipPID it gets the contact address from the sipPID<br>
	 * the dispatcher starts the SIP registration process for the SIP base URI,
	 * therefore its thread sleeps 3 seconds waiting for registration success 
	 * @param configFileName TODO
	 */
	private Dispatcher(UserAgentProfile uap, String configFileName, SipProcessID sipPID) {
		//before CONVERGENCE
//		this.sp= new SipProvider(configFileName);

		//for CONVERGENCE
		if (sp==null){ 	//if the SIP provider is null the SIP provider is created
						//then this Dispatcher becomes the listener for the SIP provider
						//(note that there will be as many Dispatcher as many different SIP base AOR)
			sp = new SipProvider(configFileName);
			sp.addSipProviderListener(SipProvider.ANY,this);		
		}
		
		if (sipPID==null) { // if the Dispatcher is created only from the configuration file
							// the uap of this Dispatcher gets the contact address from the information contained in the file
			uap.initContactAddress(sp);
		} else { 	// if the Dispatcher is created also providing a sipPID
					// it gets the contact address from the sipPID
					//XXX it has to be checked if it is correct that the full SIP address, including the parameters is used
					// as contact address
			
			String userName = sipPID.getNameAddress().getAddress().getUserName();
			SipURL url=(sp.getPort()!=SipStack.default_port)
			                   ? new SipURL(userName,userName,sp.getPort()) 
			                   : new SipURL(userName,userName);
            if (!sp.getDefaultTransport().equals(SipProvider.PROTO_UDP)) url.addTransport(sp.getDefaultTransport());
			
			uap.setContactAddress(new NameAddress(url));
			uap.setUserURI(sipPID.getNameAddress());
		}
		

		userAgentProfile=uap;
//		userAgentProfile.initContactAddress(sp);    //this looks redundant as it has been initialized few lines above

		log=sp.getLog();
		lookUpTable=new Hashtable<String, SipBaseProcess>();		                    

		if (userAgentProfile.do_register || userAgentProfile.do_unregister || userAgentProfile.do_unregister_all) {
			rc=new RegistrationClient(sp, //SipProvider sip_provider
					
					new SipURL(userAgentProfile.registrar), //SipURL registrar
					userAgentProfile.getUserURI(), 	//NameAddress target_url
					userAgentProfile.getUserURI(), 	//NameAddress from_url
					userAgentProfile.contact_url,  	//NameAddress contact_url
					userAgentProfile.auth_user,    	//String username
					userAgentProfile.auth_realm,   	//String realm
					userAgentProfile.auth_passwd,  	//String passwd
					this 							//RegistrationClientListener listener
					);
			//this.loopRegister(sp, this.user_profile.expires,this.user_profile.expires/2);
			if (rc.isRegistering()) rc.halt();          
			rc.loopRegister(userAgentProfile.expires,userAgentProfile.expires>>1);

			while (!registered) {
				try {
					printLog("waiting for registration..");
					log("waiting for registering "+userAgentProfile.getUserURI()+"..");
					Thread.sleep(3000);		
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			log("exit waiting loop, registered = "+(registered?"TRUE":"FALSE"));
		}

		//mma.receive();              
		
	}

	
	public void killSip(){
		//getMessageAgent().halt();
		rc.halt();
	}

	/**
	 * gets an instance of dispatcher if already running, otherwise instantiates it,
	 * creating also the sip provider using the given configFileName<br>
	 * when a dispatcher is instantiated, it tries to perform the SIP registration procedure,
	 * therefore it sleeps 3 seconds waiting for the registration <br> 
	 * there is a dispatcher for each user base URI
	 * @param uap
	 * @param configFileName TODO
	 * @return
	 */
	public static Dispatcher getInstance(UserAgentProfile uap, String configFileName, SipProcessID sipPID){
		
		Dispatcher dispatcher=configTable.get(sipPID==null?uap.getUserURI().toString():new NameAddress(sipPID.getNameAddress().getAddress().getBaseURL()).toString());
		if (sipPID!=null) log("sipPID base "+new NameAddress(sipPID.getNameAddress().getAddress().getBaseURL()).toString());
		
		if(dispatcher==null){								
			dispatcher=new Dispatcher(uap, configFileName, sipPID);	
			configTable.put(uap.getUserURI().toString(), dispatcher);
			log("created dispatcher for SIP agent "+uap.getUserURI().toString());			
		} else {
			log("found an existing dispatcher for SIP agent "+uap.getUserURI().toString());
		}
		return dispatcher;
	}

	/*
    public CommandLineMA getCommandLineMA(){
        return this.commandLineMA;
    }

    public void setCommandLineMA(CommandLineMA commandLinaMA){
        this.commandLineMA=commandLineMA;
    }
	 */

	/** Sends out a new message. */
	public void send(NameAddress recipient, NameAddress from, String text, SlidingWindow sw) {
		org.zoolu.sip.message.Message msg=MessageFactory.createMessageRequest(sp,recipient,from,null,"application/text",text);
		if (sw != null)
			sw.addFragment(msg);
		TransactionClient tc=new TransactionClient(sp,msg,sw);
		tc.request();
		//log("Message sent: " + msg.getBody());
	}

	/**
	 * returns the base SIP URI, taking it from the UserURI
	 * of the associated userAgentProfile
	 * 
	 */
	public NameAddress getMyNameAddress(){
		return userAgentProfile.getUserURI();
	}
	/*
    public MessageAgent getMessageAgent(){
        return this.mma;
    }
	 */
	
	/**
	 * associates a Pid and a SipBaseProcess to the dispatcher
	 */ 
	public void registerToDispatcher(SipBaseProcess sipBaseProcess, ProcessID processID){
//		if(this.lookUpTable.get(((SipProcessID)processID).getNameAddress().getAddress().toString())==null){
			this.lookUpTable.put(((SipProcessID)processID).getNameAddress().toString(), sipBaseProcess);
//		} else
//			printLog("process registration failure: "+((SipProcessID)processID).getNameAddress().getAddress().toString()+" already in use ");
	}

	public void degisterFromDispatcher(ProcessID processID){
		this.lookUpTable.remove((((SipProcessID)processID).getNameAddress().toString()));
	}    

	public SipBaseProcess lookUp(String address){
		return this.lookUpTable.get(address);
	}

	public void onRegistrationFailure(RegistrationClient regist,
			NameAddress target, NameAddress contact, String result) {
		printLog("Dispatcher cannot register with the SIP infrastructure: "+result,LogLevel.HIGH);

	}


	public void onRegistrationSuccess(RegistrationClient regist,
			NameAddress target, NameAddress contact, String result) {
		printLog("Dispatcher successfully registered with the SIP infrastructure: "+result,LogLevel.HIGH);
		if(deRegistering){ 
			registered=false; 
			deRegistering=false; 
		} 
		else 
			registered=true; 		 
	}


//	**************************** Logs ****************************/
	/** Adds a new string to the default Log */
	private void printLog(String str) {
		//printLog(str,LogLevel.HIGH);
	}

	/** Adds a new string to the default Log */
	private void printLog(String str, int level) {
		if (log!=null) log.println("CommandLineMA: "+str,level+SipStack.LOG_LEVEL_UA);
		//if (DEBUG) System.out.println("[Disp] printLog: "+str);
	}	

	/*
    public class MyMessageAgent extends MessageAgent {        
        public MyMessageAgent(SipProvider sip_provider, UserAgentProfile user_profile, MessageAgentListener listener) {
            super(sip_provider, user_profile, listener);
        }

         public void send(String recipient, String from, String subject, String content_type, String content) {
            NameAddress to_url=new NameAddress(recipient);
            NameAddress from_url=new NameAddress(from);           
            Message req=MessageFactory.createMessageRequest(sp,to_url,from_url,subject,content_type,content);
            TransactionClient t=new TransactionClient(sp,req,this);
            t.request();
        }
    }
	 */    

	public static void log(String s) {
		System.out.println("[Dispatcher] "+s);
	}

	public void onReceivedMessage(SipProvider sip_provider, Message message) {     	
		/*
        if (message.isRequest() && message.isMessage()) {  
        	new InternalDispatchingThread(sip_provider,message,this).start();
        }
		 */      

		//NOTE: processing time should be lesser than SIP retransmission timeout!! 
		if (message.isRequest() && message.isMessage()) { 
			new TransactionServer(sip_provider,message,null).
			respondWith(MessageFactory.createResponse(message,200,SipResponses.reasonOf(200),null)); 
			NameAddress remote_user=message.getFromHeader().getNameAddress();
			NameAddress to=message.getToHeader().getNameAddress();            
			String content=message.getBody();      							
//			log("New message from: "+remote_user.getAddress().toString()+" to: "+to.getAddress().toString()+" " +
//					"\nMessage content: "+content);
			Dispatcher disp = configTable.get(new NameAddress(to.getAddress().getBaseURL()).toString());
			if (disp==null){
				log("[Dispatcher] base url: "+new NameAddress(to.getAddress().getBaseURL()).toString()+" is not associated to any dispatcher\n"); 
			} else {
				SipBaseProcess rec=(SipBaseProcess)disp.lookUp(to.toString());
				if(rec!=null) {
					rec.onMessageReceived(remote_user, content,message.getCallIdHeader().getCallId());     			         
				} else {
					log("[Dispatcher] message receiver: "+to.getAddress().toString()+" is unknown\n"); 
				}
			}
		}
	}
	/*
    private class InternalDispatchingThread extends Thread {
    	SipProvider _sip_provider;
    	Message _message;
    	Dispatcher _disp;

    	public InternalDispatchingThread (SipProvider sip_provider, Message message,Dispatcher disp) {
        	this._message=message;
        	this._sip_provider=sip_provider;
        	this._disp=disp;
    	}

    	public void run() {
    		new TransactionServer(_sip_provider,_message,null).
    		respondWith(MessageFactory.createResponse(_message,200,SipResponses.reasonOf(200),null)); 
                //System.out.println("[Dispatcher] ACK sent for message: " + _message.getBody() + " with Call-ID: " + _message.getCallIdHeader().getCallId());

    		NameAddress remote_user=_message.getFromHeader().getNameAddress();
    		NameAddress to=_message.getToHeader().getNameAddress();            
    		String content=_message.getBody();      	
    		log("[Disp] New message from: "+remote_user.getAddress().toString()+" to: "+to.getAddress().toString()+" " +
    				"\n[Disp] Message content: "+content);             
    		SipBaseProcess rec=(SipBaseProcess)_disp.lookUp(to.getAddress().toString());
    		if(rec!=null) {
    			rec.onMessageReceived(remote_user, content,_message.getCallIdHeader().getCallId());     			         
    		} else {
    			log("[Disp] message receiver: "+to.getAddress().toString()+" is unknown\n"); 
    	    }    		               
    	}
    } 
	 */   
}
