/*
SMILE: a Simple Middleware Independent LayEr 
Author: Giovanni Bartolomeo

Copyright (C) 2007  University of Rome Tor Vergata - Dipartimento di Ingegneria Elettronica


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package it.uniroma2.smile.sipbinding;

import java.io.FileInputStream;
import java.util.Enumeration;
import java.util.Properties;

import it.uniroma2.smile.sipbinding.util.StringTokenizer;
import org.zoolu.sip.address.NameAddress;
import org.zoolu.sip.address.SipURL;
import it.uniroma2.smile.core.*;
import it.uniroma2.smile.sipbinding.sipmessage.MessageEnvelope;
import it.uniroma2.smile.sipbinding.sipmessage.YPDeleteMessage;
import it.uniroma2.smile.sipbinding.sipmessage.YPPublishMessage;
import it.uniroma2.smile.sipbinding.sipmessage.YPSearchRequestMessage;
import it.uniroma2.smile.sipbinding.sipmessage.YPSearchResponseMessage;

/**
 * 
 * @author Giovanni Bartolomeo
 */
public class SipProcess extends SipBaseProcess {

	private SipProcessID sipYP;
	static String YP_PROPERTY_FILENAME="cfg/ypconfig.properties";

	public static void setYPConfigFile(String s) {
		YP_PROPERTY_FILENAME=s;
	}

	static String YP_PROPERTY_REGISTRY_SIPADDRESS="registry-sip-address";
	static String YP_PROPERTY_MAX_EXP_TIME="max-expiration-time";
	static String YP_PROPERTY_EXP_TIME="expiration-time";
	static String YPAddress=null;				  

	/**
	 * a SipProcess is a Sip Base Process which also has Yellow Pages<br>
	 * therefore it can perform publish and search operations on the Yellow Pages
	 * @param configFileName
	 * @param params
	 * @param req_timeout
	 */
	public SipProcess(String configFileName, String params, int req_timeout) {
		super(configFileName, params, req_timeout);   
		//the config file for the YPAddress is set via setYPConfigFile("..")
		setSipYP ();
	}    

	/**
	 * a SipProcess is a Sip Base Process which also has Yellow Pages<br>
	 * therefore it can perform publish and search operations on the Yellow Pages
	 * @param configFileName
	 * @param params
	 * @param req_timeout
	 */
	public SipProcess(String configFileName, SipProcessID sipPID, int req_timeout) {
		super(configFileName, sipPID, req_timeout);   
		//the config file for the YPAddress is set via setYPConfigFile("..")
		setSipYP ();
	}    

	
	private void setSipYP () {
		NameAddress na = new NameAddress(new SipURL(getYPAddress()));        
		sipYP = new SipProcessID();
		sipYP.setNameAddress(na);
	}

	private String getYPAddress() {
		if (YPAddress==null) {
			FileInputStream fin;
			try {
				fin = new FileInputStream(YP_PROPERTY_FILENAME);
				Properties props = new Properties();
				props.load(fin);
				for (Enumeration<Object> e = props.keys() ; e.hasMoreElements() ;) {
					String sdName=(String)e.nextElement();
					if (sdName.startsWith(YP_PROPERTY_REGISTRY_SIPADDRESS)) {  
						YPAddress=(String)props.get(sdName);
						System.out.println("[SipYPconfig] Found property "+YP_PROPERTY_REGISTRY_SIPADDRESS+"="+YPAddress);
					}	
				}
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}
		if (YPAddress == null) { 
			log("FATAL: Property "+YP_PROPERTY_REGISTRY_SIPADDRESS+" not present in config file "+YP_PROPERTY_FILENAME); 
			System.exit(2); 
		}		
		return YPAddress;
	}    

	//used to publish process' own service
	public void publish(Descriptor descriptor) throws MalformedDescriptorException {
		YPPublishMessage m = new YPPublishMessage();
		m.setDescriptor(descriptor);
		try {
			doRequest(m, sipYP);
		} catch(Exception ex) {            
			log(" Exception in publish(): "+ex);
			ex.printStackTrace();
		}
	}

	/*I ProcessID si ricavano da operazioni di search.
         returns a list of processes which have published at least one service matching the service's description
	 *contained int the filter*/
	public synchronized ProcessID[] search(DescriptorFilter descriptorFilter) {
		YPSearchRequestMessage req = new YPSearchRequestMessage();
		req.setFilter(descriptorFilter);
		try {
			YPSearchResponseMessage message=(YPSearchResponseMessage) doRequest(req, sipYP);
			String[] res = message.getResponse();
			if (res!=null) {
				ProcessID[] procID = new SipProcessID[res.length];
				for(int i=0; i<res.length; i++) {    				
					SipProcessID sipProcID = new SipProcessID();
					sipProcID.setNameAddress(new NameAddress(new SipURL(res[i])));
					procID[i] = sipProcID;
				}
				return procID;
			} else return null;
		} catch (Exception e) {
			log(" Exception in search(): "+e);
			e.printStackTrace();
		}
		return null;
	}

	//used to deregister one service
	public void delete(Descriptor descriptor) {
		YPDeleteMessage m = new YPDeleteMessage();
		m.setDescriptor(descriptor);        
		try {
			doRequest(m, sipYP);
		} catch(Exception ex) {
			ex.printStackTrace();
			log(" Exception in delete(): "+ex);
		}
	}        

	public void onReceived(ProcessID sender, Message message, ProcessID receiver) {
		Broker.getDefaultBroker().onReceived(this, sender, message);
	};

	public Message onRequestReceived(ProcessID sender, Message request) {
		return Broker.getDefaultBroker().onRequestReceived(this, sender, request);
	}		 
}
