/*
SMILE: a Simple Middleware Independent LayEr 
Author: Giovanni Bartolomeo

Copyright (C) 2007  University of Rome Tor Vergata - Dipartimento di Ingegneria Elettronica


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package it.uniroma2.smile.sipbinding;

import java.io.FileInputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;

import org.istsms.util.Javabean2JSON;
import org.zoolu.sip.address.NameAddress;
import org.zoolu.sip.address.SipURL;
import it.uniroma2.smile.core.DefaultFaultMessage;
import it.uniroma2.smile.core.Descriptor;
import it.uniroma2.smile.core.DescriptorFilter;
import it.uniroma2.smile.core.MalformedDescriptorException;
import it.uniroma2.smile.core.Message;
import it.uniroma2.smile.core.ProcessID;
import it.uniroma2.smile.sipbinding.sipmessage.YPDeleteMessage;
import it.uniroma2.smile.sipbinding.sipmessage.YPPublishMessage;
import it.uniroma2.smile.sipbinding.sipmessage.YPSearchRequestMessage;
import it.uniroma2.smile.sipbinding.sipmessage.YPSearchResponseMessage;
/**
 * 
 * @author Giovanni Bartolomeo, Francesco Caputo
 */
public class SipYP extends SipBaseProcess {

	public static long EXP_TIME=300000; //5 min 
	public static long MAX_EXP_TIME=900000; //15 min 

	private class ServiceTable {

		private class ServiceEntry {			
			Object provider;
			Object service;

			public ServiceEntry(Object p, Object s) {
				provider=p;
				service=s;
			}

			public boolean equals(Object obj) {				
				if (obj instanceof ServiceEntry) {
					if (((ServiceEntry)obj).provider.equals(provider)&&
							((ServiceEntry)obj).service.equals(service)) {
						return true;
					}
				}
				return false;
			}
		}


		Hashtable providedBy=new Hashtable();
		Hashtable forExpiration=new Hashtable();
		Vector expirationIndex=new Vector();

		public ServiceTable(){};

		public synchronized void addProvider(Object provider, Object value) {
			cleanup();
			//add the new service
			Vector temp=(Vector) providedBy.get(value);
			if (temp==null) providedBy.put(value, temp=(Vector)new Vector());
			int i=temp.indexOf(provider);
			if (i<0) {
				temp.add(provider);
			}			
			//compute expiration time
			Long expiration=System.currentTimeMillis()+EXP_TIME;			
			for (int inc=0;;inc++) {				
				expiration+=inc;									
				if (forExpiration.get(""+expiration)==null) {
					forExpiration.put(""+expiration, new ServiceEntry(provider,value));
					//find index
					boolean inserted=false;
					for (int j=0;j<expirationIndex.size();j++) {
						String exp=(String) expirationIndex.elementAt(j);
						if (exp!=null) {
							//System.out.println(exp);
							if (Long.parseLong(exp)>expiration) {
								expirationIndex.insertElementAt(""+expiration, j);
								inserted=true;
								break;
							}
						}
					}
					if (!inserted) expirationIndex.add(""+expiration);					
					break;
				}
			}	
			log("adding provider "+provider+" service "+value+ " expiration "+new Date(expiration));
		}

		public synchronized boolean removeProvider(Object provider,Object value) {
			cleanup();
			log("removing provider "+provider+" service "+value);
			if (value==null) return false;
			Vector tmp=(Vector) providedBy.get(value);
			if (tmp==null) return false;
			tmp.remove(provider);
			if (tmp.size()==0) providedBy.remove(value);			
			return true;
		}

		public synchronized Enumeration getProviders(Object value) {
			cleanup();
			Vector tmp=(Vector) providedBy.get(value);
			if (tmp!=null) {
				return tmp.elements();
			} else {
				return null;				
			}
		}						

		private synchronized boolean removeAllProviders(Object value) {
			cleanup();
			if (value==null) return false;
			Vector tmp=(Vector) providedBy.get(value);
			if (tmp!=null) {
				tmp.removeAllElements();				
			}
			providedBy.remove(value);
			return true;
		}

		public synchronized Enumeration allServices() {
			cleanup();			
			return providedBy.keys();
		}

		//remove expired services
		public synchronized void cleanup() {
			long time=System.currentTimeMillis();
			log("cleaning up table, time is "+new Date(time));
			for (;expirationIndex.size()>0;) {
				String exp=(String)expirationIndex.elementAt(0);				
				if (exp!=null) {
					if (Long.parseLong(exp)<time) {						
						ServiceEntry se=(ServiceEntry)forExpiration.get(expirationIndex.elementAt(0));
						forExpiration.remove(expirationIndex.elementAt(0));
						expirationIndex.remove(0);
						if (se!=null&&se.provider!=null&&se.service!=null) {
							//check whether that service offered by that provider is referenced
							//by one more expiration							
							//for now just check one by one entries, to be improved in future releases 
							Enumeration e=forExpiration.elements();
							boolean remove=true;
							while (e.hasMoreElements()) {
								if (((ServiceEntry)e.nextElement()).equals(se)) {
									remove=false;
									break;
								}
							}
							if (remove) {
								log("removing provider "+se.provider+" service "+se.service+" expiration "+new Date(Long.parseLong(exp)));
								Vector v=(Vector)providedBy.get(se.service);
								if (v!=null) v.remove(se.provider);
							}
						}

					} else {
						break;
					}
				}

			}
		}	
	}

	ServiceTable listOfServices;

	public SipYP(String configFileName) {
		super(configFileName); 
System.out.println("config done");
		//try to get EXP_TIME from file, otherwise use default
		FileInputStream fin;
		try {
			fin = new FileInputStream(SipProcess.YP_PROPERTY_FILENAME);
			Properties props = new Properties();
			props.load(fin);
			for (Enumeration e = props.keys() ; e.hasMoreElements() ;) {
				String sdName=(String)e.nextElement();
				if (sdName.startsWith(SipProcess.YP_PROPERTY_MAX_EXP_TIME)) {  
					MAX_EXP_TIME=Long.parseLong((String)props.get(sdName))*1000;					
				}	
				if (sdName.startsWith(SipProcess.YP_PROPERTY_EXP_TIME)) {  
					EXP_TIME=Long.parseLong((String)props.get(sdName))*1000;
				}						
			}
		} catch (Exception e1) {
			//e1.printStackTrace();
		}
		log("property "+SipProcess.YP_PROPERTY_EXP_TIME+"="+EXP_TIME);
		log("property "+SipProcess.YP_PROPERTY_MAX_EXP_TIME+"="+MAX_EXP_TIME);			
		if (EXP_TIME>MAX_EXP_TIME) {
			EXP_TIME=MAX_EXP_TIME;
			log("WARNING: setting property "+SipProcess.YP_PROPERTY_EXP_TIME+"="+EXP_TIME);				
		}		
		listOfServices = new ServiceTable();    	       
	}

	public void onReceived(ProcessID sender,Message message, ProcessID receiver) {
		log("received asynchronous message.. not understood!");
	}

	public Message onRequestReceived(ProcessID sender, Message message) {		
		log("received message: "+message.getName()+" from "+((SipProcessID)sender).getNameAddress().getAddress().toString());
		log("message content is: "+Javabean2JSON.toJSONObject(message).toString());
		if(message instanceof YPPublishMessage) {
			publish(((YPPublishMessage)message).getDescriptor().getType(),((SipProcessID)sender).getNameAddress().getAddress().toString());
			return new Message();		
		} else if(message instanceof YPDeleteMessage){
			delete(((YPDeleteMessage)message).getDescriptor().getType(),((SipProcessID)sender).getNameAddress().getAddress().toString());		
			return new Message();
		} else if(message instanceof YPSearchRequestMessage){
			String[] response = search(((YPSearchRequestMessage)message).getFilter().getType(), 
					((YPSearchRequestMessage)message).getFilter().getMatchingPID());
			return new YPSearchResponseMessage(response);	
		}		
		log("request not understood!");
		DefaultFaultMessage fault=new DefaultFaultMessage("request not understood!");
		fault.setPortType(message.getPortType());
		fault.setOperation(message.getOperation());			
		return fault;		
	}

	public void delete(String serviceType, String identifier) {
		log("Process="+identifier+" is deleting a serviceType="+serviceType);
		listOfServices.removeProvider(identifier,serviceType);	
	}

	public void publish(String serviceType, String identifier) {
		log("Process="+identifier+" is publishing a serviceType="+serviceType);		
		listOfServices.addProvider(identifier, serviceType);
	}

	public String[] search(String serviceType, String matchingPID) {
		log("looking for service type "+serviceType+", matching on PID "+matchingPID);	
		Vector listOfProvider=new Vector();
		Enumeration tmp= (Enumeration)listOfServices.getProviders(serviceType);	
		String[] providers;
		if (tmp!=null) {
			while (tmp.hasMoreElements()) {
				Object current=tmp.nextElement();
				if (matchingPID!=null) {
					if (((String)current).indexOf(matchingPID)!=-1) {
						listOfProvider.add(current);
					}
				} else {
					listOfProvider.add(current);
				}
			}
			providers= new String[listOfProvider.size()];	
			for (int i=0; i<listOfProvider.size(); i++) {				
				providers[i]=(String) listOfProvider.get(i);			
			}	
		} else {
			providers=new String[0];
		}
		log("found "+providers.length+" record(s) matching serviceType="+ serviceType+
				((matchingPID==null)?"":"..and matchingPID="+matchingPID));	
		return providers;		
	}

	//Reflective (Self) Interface - actually NOT USED
	//Allows the YP itself to publish, delete and look for services
	public void delete(Descriptor descriptor) {	
		delete(descriptor.getType(),getProcessID().toString());
	}


	public void publish(Descriptor descriptor)
	throws MalformedDescriptorException {
		publish(descriptor.getType(),getProcessID().toString());
	}

	public ProcessID[] search(DescriptorFilter descriptorFilter) {
		String[] res=search(descriptorFilter.getType(),getProcessID().toString());
		ProcessID[] pids=new ProcessID[res.length];
		for (int i=0;i<pids.length; i++) {
			NameAddress na = new NameAddress(new SipURL(res[i]));
			SipProcessID sipProcID = new SipProcessID();
			sipProcID.setNameAddress(na);
			pids[i] = sipProcID;
		}
		return pids;
	}
}
