/*
 * MOVE application (Mobile Open & Very Easy)
 *
 * Department of Electronic Engineering - University of Rome "Tor Vergata"
 * Simple Mobile Services project (http://www.ist-sms.org/)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read LICENSE in the root of this distribution.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU General Public License
 * along with MjSip; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * As per the GPL, removal of this notice is prohibited.
 */

package it.uniroma2.smile.sipbinding.fragmentation;

import it.uniroma2.smile.sipbinding.Dispatcher;
import it.uniroma2.smile.sipbinding.sipmessage.MessageEnvelope;
import org.json.me.*;
import org.zoolu.sip.address.NameAddress;

public class Fragmenter implements SlidingWindowListener {

    private static final int MAXLENGTH = 400;  // characters

	public static boolean DEBUG=false;

    private int maxLength;
    private boolean running = true;
    
    private String id;  // used to track this fragmenter in log messages
    
    public Fragmenter() {
        this(MAXLENGTH);
    }
    
    public Fragmenter(int n) {
        setMaxLength(n);
        id = this.toString().substring(this.toString().indexOf("@")+1);
    }
    
    public int getMaxLength() {
        return maxLength;
    }
    
    public void setMaxLength(int m) {
        maxLength = m;
    }
    
    public void send(Dispatcher dispatcher, NameAddress remoteUser, NameAddress localUser, MessageEnvelope message) {
        
        String[] frags = split(message);
        SlidingWindow slidingWindow = new SlidingWindow();
        slidingWindow.setListener(this);
        
        for (int i=0; i < frags.length; i++)
			if (running == true) {
                    dispatcher.send(remoteUser, localUser, frags[i], slidingWindow);

			} else {
                log("Fragmenter halted.");
                break;
            }
        
    }

    private String[] split(MessageEnvelope message) {
        
        // we need to include these field in all fragments
        int seqNo = message.getSNo();
        int refSeq = message.getRefS();
        boolean isRequest = message.isReq();
        
        String content = message.getFrag();
        int length = content.length();
        
        // count the number of substrings
        int n = length / maxLength;
        if (length % maxLength > 0)
            n++;
        
        String[] splits=new String[n];
        
        // split the original string in n substrings
        for (int i=0; i < n; i++) {
            
            int start = i * maxLength;
            int end = start + maxLength;
            if (end > length)
                end = length;
            
            // create a new envelope...
            MessageEnvelope envelope = new MessageEnvelope();
            envelope.setSNo(seqNo);
            envelope.setRefS(refSeq);
            envelope.setReq(isRequest);
            
            // ...and fill its fragmentation fields
            envelope.setFrag(content.substring(start, end));
            envelope.setFNo(i);
            envelope.setFTot(n-1);
            
            splits[i] = envelope.toString();
            log("Fragmented message: " + splits[i]);
            
        }
        log("Message " + seqNo + " splitted in " + n + " fragments.");
        
        return splits;
    }
    
    public void onSlidingWindowTransmissionFailed() {
        running = false;
    }

    public void onSlidingWindowMessageReceived(MessageEnvelope message) {
    }

    private void log(String message) {
		if (Fragmenter.DEBUG)
			System.out.println("[Fragmenter][" + id + "] " + message);
    }

}