/*
 * Copyright (C) 2005 Luca Veltri - University of Parma - Italy
 * 
 * This file is part of MjSip (http://www.mjsip.org)
 * 
 * MjSip is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * MjSip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with MjSip; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Author(s):
 * Luca Veltri (luca.veltri@unipr.it)
 */

package org.zoolu.sip.dialog;


import org.zoolu.sip.provider.*;
import org.zoolu.sip.address.NameAddress;
import org.zoolu.sip.message.*;
import org.zoolu.sip.header.EventHeader;


/** NotifierDialogWatcher listens for new incoming subscribe requests that create an NotifierDialog.
  */
public class NotifierDialogWatcher implements SipProviderListener
{  
   /** NotifierDialogWatcherListener that captures new notifier dialogs. */
   NotifierDialogWatcherListener listener;

   /** SipProvider that receives incoming subscribe requests. */
   SipProvider sip_provider;


   /** Creates a new NotifierDialogWatcher of type <i>method</i>,
     * and starts listening for incoming subscribe requests. */
   public NotifierDialogWatcher(SipProvider sip_provider, NotifierDialogWatcherListener listener)
   {  this.listener=listener;
      sip_provider.addSipProviderListener(new MethodIdentifier(SipMethods.NOTIFY),this); 
   }


   /** Method derived from interface SipListener. */
   public void onReceivedMessage(SipProvider provider, Message msg)
   {  if (msg.isRequest() && msg.isNotify())
      {  NameAddress target=msg.getToHeader().getNameAddress();
         NameAddress subscriber=msg.getFromHeader().getNameAddress();
         EventHeader eh=msg.getEventHeader();
         String event=null;
         String id=null;
         if (eh!=null)
         {  event=eh.getEvent();
            id=eh.getId();
         }
         if (listener!=null) listener.onNewNotifierDialog(this,new NotifierDialog(sip_provider,msg,listener),target,subscriber,event,id,msg);
      }
   }


   /** Stops listening for incoming subscribe requests. */
   public void halt()
   {  sip_provider.removeSipProviderListener(new MethodIdentifier(SipMethods.NOTIFY));
   }


}
