/**
 * 
 */
package eu.convergence.middleware.createcontentPE.remote;

import java.io.StringReader;
import java.io.StringWriter;
import java.net.URL;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.ServiceFactory;
import javax.xml.transform.stream.StreamSource;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.engine.createcontentPE.schema.CreateContentEngineContext;
import org.iso.mpeg.mxm.engine.createcontentPE.schema.CreateContentRequest;
import org.iso.mpeg.mxm.engine.createcontentPE.schema.createcontentprotocol.CreateContentRequestType;
import org.iso.mpeg.mxm.engine.createcontentPE.schema.createcontentprotocol.CreateContentResponseType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.convergence.middleware.core.ConvergenceEngineName;
import eu.convergence.middleware.engine.createcontentPE.CreateContentEngine;
import eu.convergence.middleware.es.createcontentES.CreateContentProtocolRemote;

/**
 * @author aca
 *
 */
public class CreateContentPE extends CreateContentEngine
{
	private URL serviceURL = null;
	private QName serviceName = null;
	
	public CreateContentPE()
	{
		
	}
	
	public void initServiceParams(URL serviceURL, QName serviceName)
	{
		this.serviceURL = serviceURL;
		this.serviceName = serviceName;
	}
	public MXMObject createContent(MXMObject request)
	{
		Logger logger = LoggerFactory.getLogger(this.getClass());
		if (serviceURL == null)
		{
			logger.warn("Service URL has not been set");
		}
		if (serviceName == null)
		{
			logger.warn("Service Name has not been set");
		}
		ServiceFactory factory;
		try {
			factory = ServiceFactory.newInstance();
			Service service = factory.createService(serviceURL, serviceName);
			
			CreateContentProtocolRemote createContentProtocol = 
					(CreateContentProtocolRemote) service.getPort(CreateContentProtocolRemote.class);

			//AD- here shall we use the mxm-core interface?	
			//AD- maybe not since we need to marshall?
			//CreateContentRequest createContentRequest = (CreateContentRequest) request.getContent();
			
			CreateContentRequestType createContentRequest = (CreateContentRequestType) request.getContent();
			
			StringWriter sw = new StringWriter();
			
			JAXBContext jc = CreateContentEngineContext.getInstance().getJAXBContext();
			Marshaller m = jc.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			m.marshal(createContentRequest, sw);
			
			System.out.println(sw.toString());
			
			String contentResponse = createContentProtocol.createContent(sw.toString());			
			
			Unmarshaller u = jc.createUnmarshaller();
			JAXBElement<CreateContentResponseType> jaxbElement = 
					u.unmarshal(new StreamSource(new StringReader(contentResponse)), CreateContentResponseType.class);
			CreateContentResponseType createContentResponseType = jaxbElement.getValue();
			
			return new MXMAdapter(createContentResponseType);

		} catch (ServiceException e) {
			e.printStackTrace();
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public String getEngineName() 
	{
		return ConvergenceEngineName.CreateContentPE.value();
	}

}
