package eu.convergence.middleware.eventreportTE.message;

import eu.convergence.middleware.event.ERArrivalEvent;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERRKeeper;

/**
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 8/29/11
 * Time: 6:07 PM
 * <p/>
 * UTI Systems, TIC
 */
public class ERMessageConsumer implements Runnable {
    private final String xmlAsString;

    private final EventReportEngine erEngine;

    public ERMessageConsumer(String xmlAsString, EventReportEngine erEngine) {
        this.xmlAsString = xmlAsString;
        this.erEngine = erEngine;
    }

    @Override
    public void run() {
        // A. receives the ER.
        System.out.println("Received ER of " + xmlAsString.length() + " length");

        try {
            ER er = erEngine.getEventReportEngineSchemaHandler().getER();
            er.parse(new MXMAdapter(xmlAsString));
            final String erId = er.getERIdentifier();
            final String errId = er.getERRIdentifier();
            final String publicationVdiId = er.getDIIId();

            System.out.println("Found ER with ID = " + erId);

            // B. stores it, parses it and creates a MXMERArrivalEvent
            ERKeeper erKeeper = erEngine.getEventReportEngineTechHandler().getEventReportKeeper(ERRepositoryType.PVDI.getName());
            MXMEngineResponse response = erKeeper.storeER(er);
            if (MXMEngineResponse.SUCCESS != response) {
                throw new RuntimeException("The ER was not stored successfuly!");
            } else {
                System.out.println("The ER has been stored successfuly!");
            }

            ERRKeeper errKeeper = erEngine.getEventReportEngineTechHandler().getEventReportRequestKeeper(ERRepositoryType.SVDI.getName());
            final String subscriptionVdiId = errKeeper.getDIDLIdOfERR(errId);

            // generate MXMEvent
            ERArrivalEvent erArrivalEvent = new ERArrivalEvent(
                new MXMAdapter(er),
                publicationVdiId,
                subscriptionVdiId,
                erId,
                errId,
                this);

            try {
                MXM.getInstance().getMXMEventHandler().fireMXMEvent(erArrivalEvent);
            } catch (MXMConfigurationException e) {
                System.err.println("Could not fire the MXMEvent of type ERArrivalEvent");
                e.printStackTrace();
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
