package eu.convergence.middleware.identifycontentPE.remote;

import java.net.URL;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.ServiceFactory;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.identifycontentPE.schema.IdentifyContentRequest;
import org.iso.mpeg.mxm.engine.identifycontentPE.schema.IdentifyContentResponse;
import org.iso.mpeg.mxm.es.identifycontentES.IdentifyContentProtocolRemote;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.convergence.middleware.identifycontentPE.ConvIdentifyContentEngineContext;

/**
 * @author Angelo Difino
 *
 */
public class IdentifyContentPE extends org.iso.mpeg.mxm.identifycontentPE.remote.IdentifyContentPE
{
	private URL serviceURL = null;
	private QName serviceName = null;
	
	public void initServiceParams(URL serviceURL, QName serviceName)
	{
		this.serviceURL = serviceURL;
		this.serviceName = serviceName;
	}
	public MXMObject identifyContent(MXMObject request) throws Exception
	{
		Logger logger = LoggerFactory.getLogger(this.getClass());
		if (serviceURL == null)
		{
			logger.warn("Service URL has not been set");
		}
		if (serviceName == null)
		{
			logger.warn("Service Name has not been set");
		}
		ServiceFactory factory;
		try {
			factory = ServiceFactory.newInstance();
			Service service = factory.createService(serviceURL, serviceName);
			
			logger.info("In IDENTIFY CONTENT request / remote side / convergence");
			
			//remote access to the server
			IdentifyContentProtocolRemote identifyContentProtocol = 
					(IdentifyContentProtocolRemote) service.getPort(IdentifyContentProtocolRemote.class);
			
			org.iso.mpeg.mxm.engine.identifycontentPE.IdentifyContentRequest identifyContentRequest = ((org.iso.mpeg.mxm.engine.identifycontentPE.IdentifyContentRequest) request.getContent());

			MXMObject requestStr=identifyContentRequest.createObject("java.lang.String");

			String identifyResponse =(java.lang.String) identifyContentProtocol.identifyContent((java.lang.String) requestStr.getContent());
			
			org.iso.mpeg.mxm.engine.identifycontentPE.IdentifyContentResponse identifyContentResponse = this.getIdentifyContentResponse();

			identifyContentResponse.parse(new MXMAdapter(identifyResponse));
			
			return new MXMAdapter(identifyContentResponse);

		} catch (MXMException e) {
			e.printStackTrace();
			throw e;
		} catch (ServiceException e) {
			e.printStackTrace();
			throw e;
		} 
	}
	

	@Override
	public String getEngineName() 
	{
		return MXMEngineName.IdentifyContentPE.value();
	}
	
	@Override
	public org.iso.mpeg.mxm.engine.identifycontentPE.IdentifyContentRequest getIdentifyContentRequest() throws MXMEngineException {
		IdentifyContentRequest icr= new IdentifyContentRequest();
		try {
			icr.initContext(ConvIdentifyContentEngineContext.getInstance().getJAXBContext());
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return icr;
	}
	@Override
	public org.iso.mpeg.mxm.engine.identifycontentPE.IdentifyContentResponse getIdentifyContentResponse() throws MXMEngineException{
		IdentifyContentResponse icr= new IdentifyContentResponse();
		try{
			icr.initContext(ConvIdentifyContentEngineContext.getInstance().getJAXBContext());
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return icr;
	}

}

