package eu.convergence.middleware.matchTE;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.store.LockObtainFailedException;

import eu.convergence.middleware.matchTE.metadata.FieldValueMetadata;
import eu.convergence.middleware.matchTE.metadata.MetadataObject;
import eu.convergence.middleware.matchTE.metadata.RdfMetadata;
import eu.convergence.middleware.matchTE.query.FieldValueQuery;
import eu.convergence.middleware.matchTE.query.QueryObject;
import eu.convergence.middleware.matchTE.query.SparqlQuery;
import eu.convergence.middleware.matchTE.repository.FieldValueRepository;
import eu.convergence.middleware.matchTE.repository.RdfRepository;

public class RepositoryManager implements  eu.convergence.middleware.engine.matchTE.RepositoryManager
{
	public static final String MATCH_TE_DIR = System.getProperty("java.io.tmpdir") + File.separator +
											   "CONVERGENCE" + File.separator + "MATCH_TE" + File.separator;
	
	private FieldValueRepository fvRepository;
	private RdfRepository rdfRepository;
	
	public RepositoryManager()
	{
		fvRepository = new FieldValueRepository();
		rdfRepository = new RdfRepository();
	}
	
	public void initRepositories(boolean resetRepositories)
	{
		File f = new File(RepositoryManager.MATCH_TE_DIR);
		f.mkdirs();
		
		if (resetRepositories)
		{
			new RepositoryCleaner().cleanAll();
		}
		else
		{
			for (File file : f.listFiles())
			{
				if (file.isDirectory())
				{
					try
					{
						addServingCoReST(file.getName()); // TODO make them MatchTE exceptions
					} 
					catch (CorruptIndexException e)
					{
						e.printStackTrace();
					} 
					catch (LockObtainFailedException e)
					{
						e.printStackTrace();
					} 
					catch (IOException e)
					{
						e.printStackTrace();
					}
				}
			}
		}
	}
	
	private void addServingCoReST(String coReST) throws IOException
	{
		fvRepository.addServingCoReST(coReST);
		rdfRepository.addServingCoReST(coReST);
		
		System.out.println("Added CoRest " + coReST);
	}
	
	public synchronized void storePublication(String CoRest, String PVDI_ID, MetadataObject metadata) throws CorruptIndexException, LockObtainFailedException, IOException
	{																									// TODO change exception
		if (metadata instanceof FieldValueMetadata)
		{
			fvRepository.storePublication(CoRest, PVDI_ID, (FieldValueMetadata) metadata);
		}
		else if (metadata instanceof RdfMetadata)
		{
			rdfRepository.storePublication(CoRest, PVDI_ID, (RdfMetadata) metadata);
		}
	}
	
	public synchronized void storeSubscription(String CoRest, String SVDI_ID, QueryObject query) throws CorruptIndexException, LockObtainFailedException, IOException
	{																								// TODO change exception
		if (query instanceof FieldValueQuery)
		{
			fvRepository.storeSubscription(CoRest, SVDI_ID,(FieldValueQuery) query);
		}
		else if (query instanceof SparqlQuery)
		{
			rdfRepository.storeSubscription(CoRest, SVDI_ID, (SparqlQuery) query);
		}
	}
	
	public List<String> matchSubToPubsWith(String CoRest, QueryObject query) throws CorruptIndexException, IOException // TODO change exception
	{
		if (query instanceof FieldValueQuery)
		{
			return fvRepository.matchSubToPubsWith(CoRest, (FieldValueQuery) query);
		}
		else if (query instanceof SparqlQuery)
		{
			return rdfRepository.matchSubToPubsWith(CoRest, (SparqlQuery) query);
		}
		
		return null;
	}
	
	public List<String> matchPubToSubsWith(String CoRest, MetadataObject metadata) throws CorruptIndexException, IOException // TODO change exception
	{
		
		if (metadata instanceof FieldValueMetadata)
		{
			return fvRepository.matchPubToSubsWith(CoRest, (FieldValueMetadata) metadata);
		}
		else if (metadata instanceof RdfMetadata)
		{
			return rdfRepository.matchPubToSubsWith(CoRest, (RdfMetadata) metadata);
		}
		
		return null;
	}
	
}
