package eu.convergence.middleware.matchTE.repository;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import com.hp.hpl.jena.query.Dataset;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.ReadWrite;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.tdb.TDB;
import com.hp.hpl.jena.tdb.TDBFactory;


import eu.convergence.middleware.matchTE.RepositoryManager;
import eu.convergence.middleware.matchTE.metadata.RdfMetadata;
import eu.convergence.middleware.matchTE.query.SparqlQuery;

public class RdfRepository 
{
	private static final String RDF_PUBS_DIR = "RDF_PUBS";
	private static final String RDF_SUBS_DIR = "RDF_SUBS";
	
	private static final String convergence_baseURI = "http://www.ict-convergence.eu/";
	private static final String has_Sparql = convergence_baseURI + "has_Sparql";
	
	private HashMap<String,Dataset> coRestPubDataset;
	private HashMap<String,Dataset> coRestSubDataset;
	
	public RdfRepository()
	{
		coRestPubDataset = new HashMap<String,Dataset>();
		coRestSubDataset = new HashMap<String,Dataset>();
		
		TDB.getContext().set(TDB.symUnionDefaultGraph, true);
	}
	
	public void addServingCoReST(String coReST) throws IOException
	{
		File subsDir = new File(RepositoryManager.MATCH_TE_DIR + coReST + File.separator + RDF_SUBS_DIR);
		subsDir.mkdirs();
		File pubsDir = new File(RepositoryManager.MATCH_TE_DIR + coReST + File.separator + RDF_PUBS_DIR);
		pubsDir.mkdirs();
		
		Dataset pubDataset = TDBFactory.createDataset(pubsDir.getAbsolutePath());
		Dataset subDataset = TDBFactory.createDataset(subsDir.getAbsolutePath());
		
		coRestPubDataset.put(coReST, pubDataset);
		coRestSubDataset.put(coReST, subDataset);
	}
	
	public synchronized void storePublication(String CoRest, String PVDI_ID, RdfMetadata metadata) throws IOException
	{
		// TODO edo kanonika exei ena add serving corest, isos na ginei kentiki diaxeirisi ...
		// mporei kapoio na min exei rdf kai na min exei dld tous katalogous
		
		if (coRestPubDataset.get(CoRest)==null)
		{
			addServingCoReST(CoRest);
		}
		
//		for (Statement statement : metadata.getStatements())
//		{
//			System.out.println(statement.toString());
//		}
		
		Dataset dataset = coRestPubDataset.get(CoRest);
		dataset.begin(ReadWrite.WRITE);

		try
		{
			Model model = ModelFactory.createDefaultModel();

			model.add(metadata.getStatements());
			
			dataset.addNamedModel(PVDI_ID, model);
			
			dataset.commit();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{
			dataset.end();
		}
	}
	
	public synchronized void storeSubscription(String CoRest, String SVDI_ID, SparqlQuery query) throws IOException
	{
		// TODO edo kanonika exei ena add serving corest, isos na ginei kentiki diaxeirisi ...
		// mporei kapoio na min exei  rdf kai na min exei dld tous katalogous
		
		if (coRestSubDataset.get(CoRest)==null)
		{
			addServingCoReST(CoRest);
		}
		
		// TODO must store the subscription
		
		Dataset dataset = coRestSubDataset.get(CoRest);
		dataset.begin(ReadWrite.WRITE);

		try
		{
			Model model = dataset.getDefaultModel();
			
			Resource res = model.createResource(convergence_baseURI + SVDI_ID);
			Property property = model.createProperty(has_Sparql);
			
			model.add(res, property, query.getSparqlQuery());
			
			dataset.commit();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		finally
		{
			dataset.end();
		}
	}
	
	public List<String> matchSubToPubsWith(String CoRest, SparqlQuery query)
	{
		List<String> matchingPvdiIds = new ArrayList<String>();
		
		Dataset dataset = coRestPubDataset.get(CoRest);
		dataset.begin(ReadWrite.READ);
		
//		Iterator<String> iter = dataset.listNames();
//		while (iter.hasNext())
//		{
//			System.out.println("NAMESSSSS: " + iter.next());
//		}
		
		System.out.println("QUERY: " + query.getSparqlQuery());

//		QueryExecution qExec = QueryExecutionFactory.create("SELECT (count(*) AS ?count) { ?s ?p ?o}", dataset);
		
		int indexOfBracket = query.getSparqlQuery().indexOf("{");
		String experimentalSparql = "select distinct ?src where { GRAPH ?src " + query.getSparqlQuery().substring(indexOfBracket) + " }";
		
		QueryExecution qExec = QueryExecutionFactory.create(experimentalSparql, dataset);
		
		ResultSet rs = qExec.execSelect();

		try
		{
//			ResultSetFormatter.out(rs);

			while (rs.hasNext())
			{
				matchingPvdiIds.add(rs.next().getResource("src").getURI());
			}
		}
		finally
		{
			qExec.close();
		}

//		        // EDO VLEPEI OLA TA GRAPHS KAI APANTAEI
//		//      QueryExecution qExec = QueryExecutionFactory.create(
//		//      		"select ?x where {?x <http://example/hasColor> <http://example/blue> . " +
//		//      		"								?x <http://example/hasColor> <http://example/red>}", 
//		//      		dataSource) ;
//		      
//		      // EDO KANEI TO QUERY ANA GRAPH
//		      QueryExecution qExec = QueryExecutionFactory.create(
//		      		"select distinct ?src where {GRAPH ?src {?x <http://example/hasColor> ?y}}", 
//		      		dataSource) ; 
//		      ResultSet rs = qExec.execSelect() ;
//		      try {
//		      	ResultSetFormatter.out(rs) ;
//		      } finally { qExec.close() ; }

		dataset.end();

		return matchingPvdiIds;
	}
	
	public List<String> matchPubToSubsWith(String CoRest, RdfMetadata metadata)
	{
		// TODO must fetch all sparql queries and test this publication
		
		List<String> matchingSvdiIds = new ArrayList<String>();
		
		Model publicationModel = ModelFactory.createDefaultModel();

		publicationModel.add(metadata.getStatements());
		
		Dataset dataset = coRestSubDataset.get(CoRest);
		dataset.begin(ReadWrite.READ);
		
		Model subscriptionsModel = dataset.getDefaultModel();
		
		Property hasSparql = subscriptionsModel.getProperty(has_Sparql);
		
		StmtIterator statements = subscriptionsModel.listStatements(null, hasSparql, (RDFNode) null);
		
		while (statements.hasNext())
		{
			Statement statement = statements.nextStatement();
			
			QueryExecution qExec = QueryExecutionFactory.create(statement.getString(), publicationModel);
			
			ResultSet rs = qExec.execSelect();

			try
			{
//				ResultSetFormatter.out(rs);

				if (rs.hasNext())
				{
					matchingSvdiIds.add(statement.getSubject().getURI().replace(convergence_baseURI, ""));
				}
			}
			finally
			{
				qExec.close();
			}
		}
		
		return matchingSvdiIds;
	}
	
}
