package eu.convergence.middleware.matchTE;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;

import eu.convergence.middleware.core.ConvergenceEngineName;
import eu.convergence.middleware.engine.matchTE.exception.MatchEngineException;
import eu.convergence.middleware.matchTE.metadata.FieldValueMetadata;
import eu.convergence.middleware.matchTE.query.FieldValueQuery;

public class FieldValueInequalityMatchingTest extends TestCase
{
	private final String MIX_CD_COREST = "TEST MIX CD"; // Need 2 corests because MatchTE is static and the deletion of the corest doesn't
	private final String MIX_TAPE_COREST = "TEST MIX TAPE"; // delete the directories from memory.
	private String[] PUB_FIRST_TEST_RESULT;
	private String[] SUB_FIRST_TEST_RESULT;
	private FieldValueMetadata pub;
	private List<FieldValueQuery> subs;
	
	protected void setUp()
	{
		RepositoryCleaner repositoryCleaner = new RepositoryCleaner(); 
		repositoryCleaner.cleanFolder(MIX_CD_COREST);
		repositoryCleaner.cleanFolder(MIX_TAPE_COREST);
		
		pub = new FieldValueMetadata();
		subs = new ArrayList<FieldValueQuery>();
		
		ArrayList<Object> values = new ArrayList<Object>();
		values.add(15L);
		pub.getEqualMap().put("FIELD", values);
		
		FieldValueQuery subQuery = new FieldValueQuery();
		subQuery.getGreaterThanMap().put("FIELD", 14L);
		subQuery.getLessThanMap().put("FIELD", 16L);
		
		FieldValueQuery subQuery2 = new FieldValueQuery();
		subQuery2.getGreaterThanMap().put("FIELD", 16L);
		subQuery2.getLessThanMap().put("FIELD", 20L);
			
		subs.add(subQuery);
		subs.add(subQuery2);
		
		PUB_FIRST_TEST_RESULT = new String[] {"[P1]", "[]"};
		SUB_FIRST_TEST_RESULT = new String[] {"[S1]"};
	}
	
	protected void tearDown()
	{
		RepositoryCleaner repositoryCleaner = new RepositoryCleaner(); 
		repositoryCleaner.cleanFolder(MIX_CD_COREST);
		repositoryCleaner.cleanFolder(MIX_TAPE_COREST);
	}
	
	public void testSubscriptionsOverPublications() throws MXMConfigurationException, MXMEngineException
	{
		System.out.println("------------testSubscriptionsOverPublications-------------");
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		MatchTE matchTE = (MatchTE)mxm.getDefaultEngine(ConvergenceEngineName.MatchTE.value());
		
		List<String> coRest = new ArrayList<String>();
		coRest.add(MIX_CD_COREST);
		
		pub.setCorests(coRest);
		
		matchTE.getPublicationManager().matchSubsWith("P1", new MXMAdapter(pub), new Date());
		
		String[] pub_first_results = new String[subs.size()];
		
		for (int i=0; i<subs.size(); i++)
		{
			subs.get(i).setCorests(coRest);
			
			List<String> results = matchTE.getSubscriptionManager().matchPubsWith("S" + (i+1), new MXMAdapter(subs.get(i)), new Date());
	
			if (results.size()>0)
			{
				Collections.sort(results);
				System.out.println("Found matches for S" + (i+1) +": " + results);
			}
			
			pub_first_results[i] = results.toString();
		}
		
		for (int i=0; i<subs.size(); i++)
		{
			Assert.assertEquals(PUB_FIRST_TEST_RESULT[i], pub_first_results[i]);
		}
	}
		
	public void testPublicationsOverSubscription() throws MXMConfigurationException, MXMEngineException
	{
		System.out.println("------------testPublicationsOverSubscription-------------");
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		MatchTE matchTE = (MatchTE)mxm.getDefaultEngine(ConvergenceEngineName.MatchTE.value());
		
		List<String> coRest = new ArrayList<String>();
		coRest.add(MIX_TAPE_COREST);
		
		for (int i=0; i<subs.size(); i++)
		{
			subs.get(i).setCorests(coRest);
			
			matchTE.getSubscriptionManager().matchPubsWith("S" + (i+1), new MXMAdapter(subs.get(i)), new Date());
		}
	
		String[] sub_first_results = new String[1];
		
		pub.setCorests(coRest);
		
		List<String> results = matchTE.getPublicationManager().matchSubsWith("P1", new MXMAdapter(pub), new Date());
			
		if (results.size()>0)
		{
			Collections.sort(results);
			System.out.println("Found matches for P1 : " + results);
		}
		
		sub_first_results[0] = results.toString();
		
		Assert.assertEquals(SUB_FIRST_TEST_RESULT[0], sub_first_results[0]);
	}
	
}
