/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * This software is developed within the CONERGENCE fp7 EU project
 * www.ict-convergence.eu
 * 
 * This software extends the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The initial developer of the MXM Project is CEDEO.net
 * The developer of this extension is the CONVERGENCE Consortium
 * Copyright (C) 2010-2011
 * All Rights Reserved.
 *
 * ***** END LICENSE BLOCK ***** */

package eu.convergence.middleware.metadataTE.schemahandler;

import java.io.File;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;

import eu.convergence.middleware.engine.metadataTE.schema.convmeta.ConvergenceMetadataType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.PublicationMetadataType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.ResourceMetadataType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.SubscriptionMetadataType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.SuperConvMetaAbstractLevel1;
import eu.convergence.middleware.engine.mpqfTE.MpqfEngine;
import eu.convergence.middleware.engine.vdiTE.VDIKinds;


public class ConvergenceMetadata implements eu.convergence.middleware.engine.metadataTE.schemahandler.ConvergenceMetadata
{
	private ConvergenceMetadataType convType;
	private DIEngine diEngine;
	private MpqfEngine mpqfEngine;
	
	public ConvergenceMetadata(DIEngine diEngine, MpqfEngine mpqfEngine)
	{
		this(new ConvergenceMetadataType(), diEngine, mpqfEngine);
	}
	
	public ConvergenceMetadata(ConvergenceMetadataType convMetadataType, DIEngine diEngine, MpqfEngine mpqfEngine)
	{
		this.convType = convMetadataType;
		this.diEngine = diEngine;
		this.mpqfEngine = mpqfEngine;
	}
	
	//---------------- Setters -------------------//
	
	public void setVdiKind(VDIKinds vdiKind)
	{
		if (vdiKind!=null)
		{
			convType.setVDIkind(vdiKind.name());
		}
	}
	
	public void setSequenceIdentifier(String seqId)
	{
		if (seqId!=null)
		{
			convType.setSequenceIdentifier(seqId);
		}
	}
	
	public void setStartDate(Date date)
	{
		if (date!=null)
		{
			try
			{
				GregorianCalendar gCal = DatatypeFactory.newInstance().newXMLGregorianCalendar().toGregorianCalendar();
				gCal.setTime(date);
				convType.setStartDate(DatatypeFactory.newInstance().newXMLGregorianCalendar(gCal));
			}
			catch (DatatypeConfigurationException e) 
			{
				e.printStackTrace();
			}
		}
	}
	
	public void setExpiryDate(Date date)
	{
		if (date!=null)
		{
			try
			{
				GregorianCalendar gCal = DatatypeFactory.newInstance().newXMLGregorianCalendar().toGregorianCalendar();
				gCal.setTime(date);
				convType.setExpiryDate(DatatypeFactory.newInstance().newXMLGregorianCalendar(gCal));
			}
			catch (DatatypeConfigurationException e) 
			{
				e.printStackTrace();
			}
		}
	}
	
	public void setResourceMetadata(eu.convergence.middleware.engine.metadataTE.schemahandler.ResourceMetadata resMeta)
	{
		try
		{
			convType.setResourceMetadata((ResourceMetadataType)resMeta.createObject(ResourceMetadataType.class.getCanonicalName()).getContent());
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}
	
	public void setPublicationMetadata(eu.convergence.middleware.engine.metadataTE.schemahandler.PublicationMetadata pubMeta)
	{
		try
		{
			convType.setPublicationMetadata((PublicationMetadataType)pubMeta.createObject(PublicationMetadataType.class.getCanonicalName()).getContent());
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}
	
	public void setSubscriptionMetadata(eu.convergence.middleware.engine.metadataTE.schemahandler.SubscriptionMetadata subMeta)
	{
		try
		{
			convType.setSubscriptionMetadata((SubscriptionMetadataType)subMeta.createObject(SubscriptionMetadataType.class.getCanonicalName()).getContent());
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}
	
	//---------------- Getters -------------------//
	
	public VDIKinds getVdiKind()
	{
		if (convType.getVDIkind()!=null)
		{
			return VDIKinds.valueOf(convType.getVDIkind());
		}
		
		return null;
	}
	
	public String getSequenceIdentifier()
	{
		return convType.getSequenceIdentifier();
	}
	
	public Date getStartDate()
	{
		if (convType.getStartDate()!=null)
		{
			return convType.getStartDate().toGregorianCalendar().getTime();
		}
		
		return null;
	}
	
	public Date getExpiryDate()
	{
		if (convType.getExpiryDate()!=null)
		{
			return convType.getExpiryDate().toGregorianCalendar().getTime();
		}
		
		return null;
	}
	
	public ResourceMetadata getResourceMetadata()
	{
		if (convType.getResourceMetadata()!=null)
		{
			return new ResourceMetadata(convType.getResourceMetadata(), diEngine);
		}
		
		return null;
	}
	
	public PublicationMetadata getPublicationMetadata()
	{
		if (convType.getPublicationMetadata()!=null)
		{
			return new PublicationMetadata(convType.getPublicationMetadata(), diEngine);
		}
		
		return null;
	}
	
	public eu.convergence.middleware.engine.metadataTE.schemahandler.SubscriptionMetadata getSubscriptionMetadata()
	{
		if (convType.getSubscriptionMetadata()!=null)
		{
			return new SubscriptionMetadata(convType.getSubscriptionMetadata(),mpqfEngine);
		}
		
		return null;
	}
	
	// MXM CREATOR AND PARSER METHODS
	
	@Override
	public void createFile(File f) throws MXMException
	{
		new SuperConvMetaAbstractLevel1(convType).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		return new SuperConvMetaAbstractLevel1(convType).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperConvMetaAbstractLevel1 superAbstract = new SuperConvMetaAbstractLevel1(convType);

		MXMEngineResponse repsonse = superAbstract.parse(object);

		convType = (ConvergenceMetadataType) 
				superAbstract.createObject(ConvergenceMetadataType.class.getCanonicalName()).getContent();

		return repsonse;
	}
	
}
