/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * This software is developed within the CONERGENCE fp7 EU project
 * www.ict-convergence.eu
 * 
 * This software extends the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The initial developer of the MXM Project is CEDEO.net
 * The developer of this extension is the CONVERGENCE Consortium
 * Copyright (C) 2010-2011
 * All Rights Reserved.
 *
 * ***** END LICENSE BLOCK ***** */
package eu.convergence.middleware.metadataTE.schemahandler;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;

import eu.convergence.middleware.engine.metadataTE.schema.convmeta.DimensionType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.FractalAlgebraType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.SuperConvMetaAbstractLevel1;

/**
 * @author Az Mousas
 *
 */
public class Dimension implements eu.convergence.middleware.engine.metadataTE.schemahandler.Dimension
{
	private DimensionType dimType;
	
	public Dimension()
	{
		this(new DimensionType());
	}
	
	public Dimension(DimensionType dimension)
	{
		this.dimType = dimension;
	}
	
	//---------------- Setters -------------------//
	
	public void addFractal(String corest) // TODO this should be a FractalAlgebra ...
	{
		if (dimType.getFractalAlgebra()==null)
		{
			dimType.setFractalAlgebra(new FractalAlgebraType());
		}
		
		dimType.getFractalAlgebra().getFractalOrFractalAlgebra().add(corest);
	}
	
	//---------------- Getters -------------------//
	
	public List<String> getFractalList() // TODO this should be a extended ...
	{
		List<String> fractals = new ArrayList<String>();
		for (Object object : dimType.getFractalAlgebra().getFractalOrFractalAlgebra())
		{
			if (object instanceof String)
			{
				fractals.add((String)object);
			}
		}
		
		return fractals;
	}
	
	// MXM CREATOR AND PARSER METHODS
	
	@Override
	public void createFile(File f) throws MXMException
	{
		new SuperConvMetaAbstractLevel1(dimType).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		return new SuperConvMetaAbstractLevel1(dimType).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperConvMetaAbstractLevel1 superAbstract = new SuperConvMetaAbstractLevel1(dimType);

		MXMEngineResponse repsonse = superAbstract.parse(object);

		dimType = (DimensionType) 
				superAbstract.createObject(DimensionType.class.getCanonicalName()).getContent();

		return repsonse;
	}
	
}
