/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * This software is developed within the CONERGENCE fp7 EU project
 * www.ict-convergence.eu
 * 
 * This software extends the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The initial developer of the MXM Project is CEDEO.net
 * The developer of this extension is the CONVERGENCE Consortium
 * Copyright (C) 2010-2011
 * All Rights Reserved.
 *
 * ***** END LICENSE BLOCK ***** */

package eu.convergence.middleware.metadataTE.schemahandler;

import java.io.File;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;

import eu.convergence.middleware.engine.metadataTE.schema.convmeta.FieldValueType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.SuperConvMetaAbstractLevel1;


public class FieldValue implements eu.convergence.middleware.engine.metadataTE.schemahandler.FieldValue
{
	private FieldValueType fieldValueType;
	
	public FieldValue()
	{
		this(new FieldValueType());
	}
	
	public FieldValue(FieldValueType fieldValue)
	{
		this.fieldValueType = fieldValue;
	}
	
	public FieldValue(String field, Object value)
	{
		this.fieldValueType = new FieldValueType();
		this.fieldValueType.setField(field);
		this.fieldValueType.setValue(value);
	}
	
	//---------------- Setters -------------------//
	
	public void setField(String field)
	{
		fieldValueType.setField(field);
	}
	
	public void setValue(Object value)
	{
		fieldValueType.setValue(value);
	}
	
	//---------------- Getters -------------------//
	
	
	public String getField()
	{
		return fieldValueType.getField();
	}
	
	public Object getValue()
	{
		return fieldValueType.getValue();
	}
	
	// MXM CREATOR AND PARSER METHODS
	
	@Override
	public void createFile(File f) throws MXMException
	{
		new SuperConvMetaAbstractLevel1(fieldValueType).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		return new SuperConvMetaAbstractLevel1(fieldValueType).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperConvMetaAbstractLevel1 superAbstract = new SuperConvMetaAbstractLevel1(fieldValueType);

		MXMEngineResponse repsonse = superAbstract.parse(object);

		fieldValueType = (FieldValueType) 
				superAbstract.createObject(FieldValueType.class.getCanonicalName()).getContent();

		return repsonse;
	}
	
}
