/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * This software is developed within the CONERGENCE fp7 EU project
 * www.ict-convergence.eu
 * 
 * This software extends the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The initial developer of the MXM Project is CEDEO.net
 * The developer of this extension is the CONVERGENCE Consortium
 * Copyright (C) 2010-2011
 * All Rights Reserved.
 *
 * ***** END LICENSE BLOCK ***** */

package eu.convergence.middleware.metadataTE.schemahandler;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_msx.StructuredDataType;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;

import eu.convergence.middleware.engine.metadataTE.schema.convmeta.DimensionType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.FieldValueType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.PublicationMetadataType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.SuperConvMetaAbstractLevel1;


public class PublicationMetadata implements eu.convergence.middleware.engine.metadataTE.schemahandler.PublicationMetadata
{
	private PublicationMetadataType pubMetaType;
	private DIEngine diEngine;
	
	public PublicationMetadata(DIEngine diEngine)
	{
		this(new PublicationMetadataType(), diEngine);
	}
	
	public PublicationMetadata(PublicationMetadataType pubMeta, DIEngine diEngine)
	{
		this.pubMetaType = pubMeta;
		this.diEngine = diEngine;
	}
	
	//---------------- Setters -------------------//
	
	public void addDimension(eu.convergence.middleware.engine.metadataTE.schemahandler.Dimension dimension) // TODO refresh PubMetaType with the new schema having list of dimensions
	{
		try
		{
			pubMetaType.setDimension((DimensionType)dimension.createObject(DimensionType.class.getCanonicalName()).getContent());
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	public void setRVDIIdentifier(String rvdiId)
	{
		pubMetaType.setRVDIid(rvdiId);
	}
	
	public void addKeyword(String keyword)
	{
		pubMetaType.getKeyword().add(keyword);
	}
	
	public void addTag(String tag)
	{
		pubMetaType.getTag().add(tag);
	}
	
	public void addFieldValuePair(String field, Object value)
	{
		try
		{
			FieldValue fieldValue = new FieldValue(field, value);
			pubMetaType.getFieldValue().add((FieldValueType)fieldValue.createObject(FieldValueType.class.getCanonicalName()).getContent());
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	public void addStructuredData(StructuredData structuredData)
	{
		try
		{
			pubMetaType.getStructuredData().add((StructuredDataType)structuredData.createObject(StructuredDataType.class.getCanonicalName()).getContent());
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	//---------------- Getters -------------------//
	
	public List<eu.convergence.middleware.engine.metadataTE.schemahandler.Dimension> getDimensionList()
	{
		List<eu.convergence.middleware.engine.metadataTE.schemahandler.Dimension> dimensions =
				new ArrayList<eu.convergence.middleware.engine.metadataTE.schemahandler.Dimension>();
		if (pubMetaType.getDimension()!=null)
		{
			dimensions.add(new Dimension(pubMetaType.getDimension()));
		}
		
		return dimensions;
	}
	
	public String getRVDIIdentifier()
	{
		return pubMetaType.getRVDIid();
	}
	
	public List<String> getKeywordList()
	{
		return pubMetaType.getKeyword();
	}
	
	public List<String> getTagList()
	{
		return pubMetaType.getTag();
	}
	
	public List<eu.convergence.middleware.engine.metadataTE.schemahandler.FieldValue> getFieldValueList()
	{
		List<eu.convergence.middleware.engine.metadataTE.schemahandler.FieldValue> fieldValueList =
				new ArrayList<eu.convergence.middleware.engine.metadataTE.schemahandler.FieldValue>();
		
		for (FieldValueType fieldValueType : pubMetaType.getFieldValue())
		{
			fieldValueList.add(new FieldValue(fieldValueType));
		}
		
		return fieldValueList;
	}
	
	public List<StructuredData> getStructuredDataList()
	{
		List<StructuredData> structuredDataList = new ArrayList<StructuredData>();
		
		for (StructuredDataType structuredDataType : pubMetaType.getStructuredData())
		{
			try
			{
				StructuredData structuredData = diEngine.getDIEngineSchemaHandler().getStructuredData();
				structuredData.parse(new MXMAdapter(structuredDataType));
				structuredDataList.add(structuredData);
			}
			catch(Exception ex)
			{
				ex.printStackTrace();
			}
		}
		
		return structuredDataList;
	}
	
	// MXM CREATOR AND PARSER METHODS
	
	@Override
	public void createFile(File f) throws MXMException
	{
		new SuperConvMetaAbstractLevel1(pubMetaType).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		return new SuperConvMetaAbstractLevel1(pubMetaType).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperConvMetaAbstractLevel1 superAbstract = new SuperConvMetaAbstractLevel1(pubMetaType);

		MXMEngineResponse repsonse = superAbstract.parse(object);

		pubMetaType = (PublicationMetadataType) 
				superAbstract.createObject(PublicationMetadataType.class.getCanonicalName()).getContent();

		return repsonse;
	}
	
}
