/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * This software is developed within the CONERGENCE fp7 EU project
 * www.ict-convergence.eu
 * 
 * This software extends the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The initial developer of the MXM Project is CEDEO.net
 * The developer of this extension is the CONVERGENCE Consortium
 * Copyright (C) 2010-2011
 * All Rights Reserved.
 *
 * ***** END LICENSE BLOCK ***** */

package eu.convergence.middleware.metadataTE.schemahandler;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.searchTE.schema.mpqf.InputQueryType;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.InputQuery;

import eu.convergence.middleware.engine.metadataTE.schema.convmeta.DimensionType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.FieldValueType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.SubscriptionMetadataType;
import eu.convergence.middleware.engine.metadataTE.schema.convmeta.SuperConvMetaAbstractLevel1;
import eu.convergence.middleware.engine.mpqfTE.MpqfEngine;


public class SubscriptionMetadata implements eu.convergence.middleware.engine.metadataTE.schemahandler.SubscriptionMetadata
{
	private SubscriptionMetadataType subMetaType;
	private MpqfEngine mpqfEngine;
	
	public SubscriptionMetadata(MpqfEngine mpqfEngine)
	{
		this(new SubscriptionMetadataType(), mpqfEngine);
	}
	
	public SubscriptionMetadata(SubscriptionMetadataType subMeta, MpqfEngine mpqfEngine)
	{
		this.subMetaType = subMeta;
		this.mpqfEngine = mpqfEngine;
	}
	
	//---------------- Setters -------------------//
	
	public void addDimension(eu.convergence.middleware.engine.metadataTE.schemahandler.Dimension dimension) // TODO refresh SubMetaType with the new schema having list of dimensions
	{
		try
		{
			subMetaType.setDimension((DimensionType)dimension.createObject(DimensionType.class.getCanonicalName()).getContent());
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	public void setQuery(InputQuery inputQuery)
	{
		try
		{
			subMetaType.setQuery((InputQueryType)inputQuery.createObject(InputQueryType.class.getCanonicalName()).getContent());
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	//---------------- Getters -------------------//
	
	public List<eu.convergence.middleware.engine.metadataTE.schemahandler.Dimension> getDimensionList()
	{
		List<eu.convergence.middleware.engine.metadataTE.schemahandler.Dimension> dimensions = 
				new ArrayList<eu.convergence.middleware.engine.metadataTE.schemahandler.Dimension>();
		if (subMetaType.getDimension()!=null)
		{
			dimensions.add(new Dimension(subMetaType.getDimension()));
		}
		
		return dimensions;
	}
	
	public InputQuery getQuery()
	{
		try
		{
			InputQuery iputQuery = mpqfEngine.getSearchEngineSchemaHandler().getInputQuery();
			iputQuery.parse(new MXMAdapter(subMetaType.getQuery()));
		
			return iputQuery;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	// MXM CREATOR AND PARSER METHODS

	@Override
	public void createFile(File f) throws MXMException
	{
		new SuperConvMetaAbstractLevel1(subMetaType).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		return new SuperConvMetaAbstractLevel1(subMetaType).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperConvMetaAbstractLevel1 superAbstract = new SuperConvMetaAbstractLevel1(subMetaType);

		MXMEngineResponse repsonse = superAbstract.parse(object);

		subMetaType = (SubscriptionMetadataType) 
				superAbstract.createObject(FieldValueType.class.getCanonicalName()).getContent();

		return repsonse;
	}
	
}
