package eu.convergence.middleware.metadataTE.technologyhandler;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.ontology.impl.OntModelImpl;
import com.hp.hpl.jena.rdf.arp.JenaReader;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.RDFReader;
import com.hp.hpl.jena.rdf.model.RDFWriter;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;

import eu.convergence.middleware.engine.metadataTE.technologyhandler.MetadataKinds;

public class RdfMetadataParser implements eu.convergence.middleware.engine.metadataTE.technologyhandler.RdfMetadataParser
{
	private OntModel model;

	public RdfMetadataParser()
	{
		model = new OntModelImpl(OntModelSpec.OWL_MEM);
	}
	
	public RdfMetadataParser(Node node)
	{
		model = new OntModelImpl(OntModelSpec.OWL_MEM);
		
		InputStream in = nodeToInputStream(node);
		
		if (in!=null)
		{
			try
			{
				RDFReader rdfReader = new JenaReader();
				rdfReader.read(model, in, null);
				
				in.close();
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
		}
	}
	
	public boolean parseRDF(File rdfFile, String baseURI)
	{
		try
		{
			FileInputStream fis = new FileInputStream(rdfFile.getAbsolutePath());
			
			RDFReader rdfReader = new JenaReader();
			rdfReader.read(model, fis, baseURI);
			
			fis.close();
			
			return true;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return false;
	}
	
	public void addRDFTriple(String subject, String predicate, String object)
	{
		Resource res = model.createResource(subject);
		Property pro = model.createProperty(predicate);
		Resource reo = model.createResource(object);
		
		model.add(res, pro, reo);
	}
	
	public void addRDFLiteralTriple(String subject, String predicate, Object value)
	{
		Resource res = model.createResource(subject);
		Property pro = model.createProperty(predicate);
		
		if (value instanceof Date)
		{
			Calendar cal = Calendar.getInstance();
			cal.setTime((Date) value);
			value = cal;
		}
		
		model.addLiteral(res, pro, value);
	}
	
	
	
	public List<Statement> listStatements()
	{
		return model.listStatements().toList();
	}
	
	public Node writeModel()
	{
		try
		{
			//	m.setNsPrefix("", NS);
	
			RDFWriter w = model.getWriter("RDF/XML");
			w.setProperty("allowBadURIs","true");
	//		w.setProperty("xmlbase", "http://PRISM");
			w.setProperty("showXmlDeclaration", false);
			w.setProperty("tab",6);
			
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			
			model.write(baos);
			
			ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
			
			DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
			Document doc = docBuilder.parse(bais);
			
			return  doc.getFirstChild();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	private InputStream nodeToInputStream(Node node)
	{
		try
		{
	        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
	        Result outputTarget = new StreamResult(outputStream);
	        Transformer t = TransformerFactory.newInstance().newTransformer();
	        t.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
	        t.transform(new DOMSource(node), outputTarget);
	        return new ByteArrayInputStream(outputStream.toByteArray());
		}
		catch(TransformerException ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	public MetadataKinds getMetadataKind() 
	{
		return MetadataKinds.RDF;
	}

}
