package eu.convergence.middleware.mpqfTE.technologyhandler;

import org.iso.mpeg.mxm.tEngine.searchTE.SearchEngine;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.BooleanExpression;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.Equal;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.Field;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.GreaterThan;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.LessThan;

import eu.convergence.middleware.engine.mpqfTE.exception.MpqfEngineException;

public class FieldOperatorValue implements eu.convergence.middleware.engine.mpqfTE.technologyhandler.FieldOperatorValue
{
	private SearchEngine searchEngine;
	private Field field;
	private String operator;
	private Object value;
	
	public FieldOperatorValue(SearchEngine searchEngine)
	{
		this.searchEngine = searchEngine;
	}
	
	public FieldOperatorValue(SearchEngine searchEngine, String field, String operator, Object value)
	{
		try
		{
			this.field = searchEngine.getSearchEngineSchemaHandler().getField();
			this.field.setValue(field);
			this.operator = operator;
			this.value = value;
			this.searchEngine = searchEngine;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	public String getField()
	{
		return field.getValue();
	}
	
	public String getOperator()
	{
		return operator;
	}
	
	public Object getValue()
	{
		return value;
	}
	
	public void parse(BooleanExpression condition) throws MpqfEngineException
	{
		if (condition instanceof Equal)
		{
			Equal equal = (Equal) condition;
			
			field = equal.getField();
			value = equal.getValue();
			operator = "=";
		}
		else if (condition instanceof GreaterThan)
		{
			GreaterThan greaterThan = (GreaterThan) condition;
			
			field = greaterThan.getField();
			value = greaterThan.getValue();
			operator = ">";
		}
		else if (condition instanceof LessThan)
		{
			LessThan lessThan = (LessThan) condition;
			
			field = lessThan.getField();
			value = lessThan.getValue();
			operator = "<";
		}
		else
		{
			throw new MpqfEngineException("Expression not implemented" + condition.getClass().getSimpleName());
		}
	}
	
	public BooleanExpression createBooleanExpression() throws MpqfEngineException
	{
		try
		{
			if ("=".equals(operator))
			{
				Equal equal = searchEngine.getSearchEngineSchemaHandler().getEqual();
				equal.setComparisonExpression(field, value);
				
				return equal;
			}
			else if (">".equals(operator))
			{
				GreaterThan greaterThan = searchEngine.getSearchEngineSchemaHandler().getGreaterThan();
				greaterThan.setComparisonExpression(field, value);
				
				return greaterThan;
			}
			else if ("<".equals(operator))
			{
				LessThan lessThan = searchEngine.getSearchEngineSchemaHandler().getLessThan();
				lessThan.setComparisonExpression(field, value);
				
				return lessThan;
			}
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		throw new MpqfEngineException("Operator not implemented");
	}
}
