package eu.convergence.middleware.mpqfTE.technologyhandler;

import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.searchTE.SearchEngine;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.AND;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.BooleanExpression;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.OR;

public class FieldValueQueryEditor implements eu.convergence.middleware.engine.mpqfTE.technologyhandler.FieldValueQueryEditor
{
	private SearchEngine searchEngine;
	private List<eu.convergence.middleware.engine.mpqfTE.technologyhandler.FieldOperatorValue> fieldOperatorValueList; // TODO depthFirst iterator ???
	
	public FieldValueQueryEditor(SearchEngine searchEngine)
	{
		fieldOperatorValueList = new ArrayList<eu.convergence.middleware.engine.mpqfTE.technologyhandler.FieldOperatorValue>();
		this.searchEngine = searchEngine; 
	}
	
	public void parse(BooleanExpression booleanExpression) throws MXMEngineException
	{
		if (booleanExpression instanceof OR)
		{
			OR or = (OR) booleanExpression;

			for (BooleanExpression condition : or.getConditionList())
			{
				FieldOperatorValue fieldOperatorValue = new FieldOperatorValue(searchEngine);
				fieldOperatorValue.parse(condition);

				fieldOperatorValueList.add(fieldOperatorValue);
			}
		}
		else if (booleanExpression instanceof AND)
		{
			AND and = (AND) booleanExpression;

			for (BooleanExpression condition : and.getConditionList())
			{
				FieldOperatorValue fieldOperatorValue = new FieldOperatorValue(searchEngine);
				fieldOperatorValue.parse(condition);

				fieldOperatorValueList.add(fieldOperatorValue);
			}
		}
		else
		{
			FieldOperatorValue fieldOperatorValue = new FieldOperatorValue(searchEngine);
			fieldOperatorValue.parse(booleanExpression);

			fieldOperatorValueList.add(fieldOperatorValue);
		}
	}
	
	public BooleanExpression generateBooleanExpression() throws MXMEngineException
	{
		if (fieldOperatorValueList.size() > 1)
		{
			AND and = searchEngine.getSearchEngineSchemaHandler().getAND();
			
			for (eu.convergence.middleware.engine.mpqfTE.technologyhandler.FieldOperatorValue fieldOperatorValue : fieldOperatorValueList)
			{
				and.addCondition(fieldOperatorValue.createBooleanExpression());
			}
			
			return and;
		}
		else if (!fieldOperatorValueList.isEmpty())
		{
			return fieldOperatorValueList.get(0).createBooleanExpression();
		}
		
		throw new MXMEngineException("No field/value conditions provided.");
	}
	
	public void addFieldOperatorValue(String field, String operator, Object value)
	{
		fieldOperatorValueList.add(new FieldOperatorValue(searchEngine, field, operator, value));
	}
	
	public List<eu.convergence.middleware.engine.mpqfTE.technologyhandler.FieldOperatorValue> getFieldOperatorValueList()
	{
		return fieldOperatorValueList;
	}
	
}
