package eu.convergence.middleware.mpqfTE.technologyhandler;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.searchTE.SearchEngine;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.BooleanExpression;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.QueryBySPARQL;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.QueryCondition;

import com.hp.hpl.jena.datatypes.xsd.XSDDatatype;
import com.hp.hpl.jena.datatypes.xsd.XSDDateTime;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.sparql.expr.E_Equals;
import com.hp.hpl.jena.sparql.expr.E_GreaterThan;
import com.hp.hpl.jena.sparql.expr.E_LessThan;
import com.hp.hpl.jena.sparql.expr.Expr;
import com.hp.hpl.jena.sparql.expr.ExprVar;
import com.hp.hpl.jena.sparql.expr.nodevalue.NodeValueDateTime;
import com.hp.hpl.jena.sparql.expr.nodevalue.NodeValueInteger;
import com.hp.hpl.jena.sparql.expr.nodevalue.NodeValueString;
import com.hp.hpl.jena.sparql.syntax.ElementFilter;
import com.hp.hpl.jena.sparql.syntax.ElementGroup;

public class SparqlQueryEditor implements eu.convergence.middleware.engine.mpqfTE.technologyhandler.SparqlQueryEditor
{
	private SearchEngine searchEngine;
	
	private List<Node> returnVariables;
	private List<Triple> triples;
	private List<ElementFilter> filters;
	private Long resultLimit;
	
	private String sparqlString;

	public SparqlQueryEditor(SearchEngine searchEngine)
	{
		this.searchEngine = searchEngine;
		triples = new ArrayList<Triple>();
		filters = new ArrayList<ElementFilter>();
	}
	
	public BooleanExpression generateBooleanExpression() throws MXMEngineException
	{
//		inputQuery.addPrefix("AEEEEEEK", "http://AEK_ORIGINAL_21/");
		
//		outputDescription.addSortByField("AEK", true);
//		outputDescription.addSortBySemanticField("AEK", true);
//		outputDescription.setDistinct(true);
//		outputDescription.setMaxItemCount(300L);
//		outputDescription.setResultDescriptionNamespace("http://AEKARA");
//		outputDescription.setShowResultMediaResourceURL(true);
		
		QueryBySPARQL queryBySparql = searchEngine.getSearchEngineSchemaHandler().getQueryBySPARQL();
		queryBySparql.setSPARQL(generateSparqlQuery());
		
//		System.out.println(inputQuery.createObject(String.class.getCanonicalName()).getContent());
		
		return queryBySparql;
	}
	
	public void addTripleCondition(String subject, String predicate, String object)
	{
		Node subjectNode = createNode(subject);
		Node predicateNode = createNode(predicate);
		Node objectNode = createNode(object);
		
		triples.add(new Triple(subjectNode, predicateNode, objectNode));
	}
	
	public void addTripleLiteralCondition(String subject, String predicate, Object object)
	{
		Node subjectNode = createNode(subject);
		Node predicateNode = createNode(predicate);
		Node objectNode = null;
		
		if (object instanceof String)
		{
//			objectNode = Node.createLiteral(object.toString(), null, XSDDatatype.XSDstring); // TODO psaxnei exact i close matches
			objectNode = Node.createLiteral(object.toString());
		}
		else if (object instanceof Integer)
		{
			objectNode = Node.createLiteral(object.toString(), null, XSDDatatype.XSDinteger);
		}
		else if (object instanceof Date)
		{
			Calendar cal = Calendar.getInstance();
			cal.setTime((Date) object);
			XSDDateTime xsdDate = new XSDDateTime(cal);
			objectNode = Node.createLiteral(xsdDate.toString(), null, XSDDatatype.XSDdateTime);
		}
		
		triples.add(new Triple(subjectNode, predicateNode, objectNode));
	}
	
	public void addFilterCondition(String object, String operator, Object value)
	{
		Expr exprVariable;
		if (object.startsWith("?"))
		{
			exprVariable = new ExprVar(object.substring(1));
		}
		else
		{
			exprVariable = new ExprVar(object);
		}
		
		Expr exprValue = null;
		if (value instanceof String)
		{
			exprValue = new NodeValueString((String)value);
		}
		else if (value instanceof Integer)
		{
			exprValue = new NodeValueInteger((Integer)value);
		}
		else if (value instanceof Date)
		{
			Calendar calendar = Calendar.getInstance();
			calendar.setTime((Date)value);
			
			exprValue = new NodeValueDateTime(new XSDDateTime(calendar));
		}
		
		Expr expr = null;
		if (operator.equals("="))
		{
			expr = new E_Equals(exprVariable, exprValue);
		}
		else if (operator.equals("<"))
		{
			expr = new E_LessThan(exprVariable, exprValue);
		}
		else if (operator.equals(">"))
		{
			expr = new E_GreaterThan(exprVariable, exprValue);
		}
		
		filters.add(new ElementFilter(expr));
	}
	
	private Node createNode(String inputStr)
	{
		Node toReturnNode;
		
		if (inputStr!=null)
		{
			if (inputStr.startsWith("?"))
			{
				String variableName = inputStr.substring(1);
				
				toReturnNode = Node.createVariable(variableName);
			}
			else
			{
				toReturnNode = Node.createURI(inputStr);
			}
		}
		else
		{
			toReturnNode = Node.ANY; // TODO na do ti vgainei
		}
		
		return toReturnNode;
	}
	
	public void addReturnVariable(String varName)
	{
		if (returnVariables==null)
		{
			returnVariables = new ArrayList<Node>();
		}
		
		if (varName.startsWith("?"))
		{
			returnVariables.add(Node.createVariable(varName.substring(1)));
		}
		else
		{
			returnVariables.add(Node.createVariable(varName));
		}
	}
	
	private String generateSparqlQuery() throws MXMEngineException // TODO just select query for now
	{
		String errors = "";
		
		Query query = new Query();
		
		query.setQuerySelectType();
		
		if (returnVariables.isEmpty())
		{
			errors += "No SPARQL return variable provided. ";
		}
		
		for (Node variableNode : returnVariables)
		{
			query.addResultVar(variableNode);
		}
		
		ElementGroup elementGroup = new ElementGroup();
		
		for (Triple triple : triples)
		{
			elementGroup.addTriplePattern(triple);
		}
		
		if (triples.isEmpty())
		{
			errors += "No SPARQL triple conditions provided.";
		}
		
		for (ElementFilter filter : filters)
		{
			elementGroup.addElementFilter(filter);
		}
		
		query.setQueryPattern(elementGroup);
		
		if (resultLimit!=null)
		{
			query.setLimit(resultLimit);
		}
		
		if (!errors.equals(""))
		{
			throw new MXMEngineException(errors);
		}
		
		return query.toString();
	}
	
	public void setResultLimit(long limit)
	{
		resultLimit = limit;
	}

	@Override
	public void parse(BooleanExpression booleanExpression) throws MXMEngineException
	{
		if (booleanExpression instanceof QueryBySPARQL)
		{
			QueryBySPARQL sparqlCondition = (QueryBySPARQL) booleanExpression;
			
			sparqlString = sparqlCondition.getSPARQL();
		}
	}
	
	public String getSparqlString()
	{
		return sparqlString;
	}
	
}