package eu.convergence.middleware.orchestratorTE;

import eu.convergence.middleware.conetTe.ConetTE;
import eu.convergence.middleware.core.ConvergenceEngineName;
import eu.convergence.middleware.engine.cdsTE.CdsEngine;
import eu.convergence.middleware.engine.matchTE.MatchEngine;
import eu.convergence.middleware.engine.matchTE.exception.MatchEngineException;
import eu.convergence.middleware.engine.metadataTE.ConvergenceMetadataEngine;
import eu.convergence.middleware.engine.overlayTE.OverlayEngine;
import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.engine.vdiTE.VdiEngine;
import eu.convergence.middleware.eventreportTE.kernel.conet.EventReportTE;
import eu.convergence.middleware.mpqfTE.MpqfTE;
import eu.convergence.middleware.orchestratorTE.build.ConetEngineOrchestrator;
import eu.convergence.middleware.orchestratorTE.build.MetadataOrchestrator;
import eu.convergence.middleware.orchestratorTE.build.OverlayEngineOrchestrator;
import eu.convergence.middleware.orchestratorTE.build.SecurityEngineOrchestrator;
import eu.convergence.middleware.orchestratorTE.build.VdiOrchestrator;
import eu.convergence.middleware.orchestratorTE.handle.EREngineOrchestrator;
import eu.convergence.middleware.orchestratorTE.handle.MatchEngineOrchestrator;
import eu.convergence.middleware.orchestratorTE.handle.ReceiverOrchestrator;
import eu.convergence.middleware.orchestratorTE.schedule.TimerOrchestrator;
import eu.convergence.middleware.orchestratorTE.schedule.task.OrchestratorTasks;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class ConvergenceOrchestratorTE extends org.iso.mpeg.mxm.orchestrator.OrchestratorEngine
{
	private Logger logger = LoggerFactory.getLogger(this.getClass());
	
	private ConvergenceMetadataEngine convMetadataEngine = null;
	private VdiEngine vdiEngine = null;
	private OverlayEngine overlayEngine = null;
	private MatchEngine matchEngine = null;
	private ConetTE conetEngine = null;
	private CdsEngine cdsEngine = null;
	private MpqfTE mpqfEngine = null;
	
	// TODO 4 orchestrator types
	//		mxmevent driven
	//		mxmtask driven
	//		protocol driven
	//		application driven
	
	// TODO name of extended engine for dependencies
	
	public ReceiverOrchestrator getReceiverOrchestrator()
    {
        return ReceiverOrchestrator.getInstance(vdiEngine, convMetadataEngine, getConetOrchestrator());
    }
	
    public TimerOrchestrator getTimerOrchestrator(OrchestratorTasks ot) throws ClassNotFoundException
    {
        return new TimerOrchestrator(ot.getTaskClass());
    }
    
    public MatchEngineOrchestrator getMatchOrchestrator() throws MatchEngineException, MXMException
	{
		return MatchEngineOrchestrator.getInstance(matchEngine, vdiEngine, convMetadataEngine, erEngine, cdsEngine, mpqfEngine);
	}
	
	public EREngineOrchestrator getEROrchestrator()
	{
		return EREngineOrchestrator.getInstance((EventReportTE) erEngine,ConetEngineOrchestrator.getInstance(conetEngine).getAdvertiser(),
			    										 ConetEngineOrchestrator.getInstance(conetEngine).getSender());
	}
	
	public VdiOrchestrator getVdiOrchestrator()
	{
		return new VdiOrchestrator(vdiEngine, getCreateContentEngineOrchestrator(), getIdentifyContentEngineOrchestrator(),
				ConetEngineOrchestrator.getInstance(conetEngine).getAdvertiser());
	}
	
	public MetadataOrchestrator getMetadataOrchestrator()
	{
		return new MetadataOrchestrator(vdiEngine, convMetadataEngine, mpqfEngine);
	}
	
	public ConetEngineOrchestrator getConetOrchestrator()
	{
		return ConetEngineOrchestrator.getInstance(conetEngine);
	}
	
	public OverlayEngineOrchestrator getOverlayOrchestrator() throws OverlayEngineException
	{
		return OverlayEngineOrchestrator.getInstance(overlayEngine, ConetEngineOrchestrator.getInstance(conetEngine).getAdvertiser(),
													ConetEngineOrchestrator.getInstance(conetEngine).getSender(), vdiEngine);
	}
	
	public SecurityEngineOrchestrator getSecurityOrchestrator()
	{
		return SecurityEngineOrchestrator.getInstance(securityEngine);
	}
    
    public void setDependencies(List<MXMEngine> engines)
			throws MXMEngineException
	{
		if (engines != null)
		{
			super.setDependencies(engines);
			
			for (MXMEngine engine : engines)
			{
				String engineName = engine.getEngineName();
				
				// Technology Engines
				if (engineName.equals(ConvergenceEngineName.CDSTE.value()))
				{
					this.cdsEngine = (CdsEngine)engine;
				}
				else if (engineName.equals(ConvergenceEngineName.OverlayTE.value()))
				{
					this.overlayEngine = (OverlayEngine)engine;
				}
				else if (engineName.equals(ConvergenceEngineName.MatchTE.value()))
				{
					this.matchEngine = (MatchEngine)engine;
				}
				else if (engineName.equals(ConvergenceEngineName.ConetTE.value()))
				{
					this.conetEngine = (ConetTE)engine;
				}
				else if (engineName.equals(ConvergenceEngineName.ConvergenceMetadataTE.value()))
				{
					this.convMetadataEngine = (ConvergenceMetadataEngine)engine;
				}
				else if (engineName.equals(ConvergenceEngineName.MpqfTE.value()))
				{
					this.mpqfEngine = (MpqfTE)engine;
				}
				else if (engineName.equals(MXMEngineName.DigitalItemTE.value()))
				{
					this.vdiEngine = (VdiEngine)engine;
				}
			}
		}
	}
	
}
