package eu.convergence.middleware.orchestratorTE.build;

import eu.convergence.middleware.conetTe.ConetTE;
import eu.convergence.middleware.core.ConvergenceEngineName;
import eu.convergence.middleware.engine.ConetTe.exception.ConetIOException;
import eu.convergence.middleware.engine.ConetTe.services.NID;
import eu.convergence.middleware.orchestratorTE.ConvergenceOrchestratorTE;

import java.io.File;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;

/**
 * Created by IntelliJ IDEA.
 * User: aca
 * Date: 8/27/11
 * Time: 9:37 AM
 */
public class ConetEngineOrchestrator
{
	private Logger logger = Logger.getLogger(ConetEngineOrchestrator.class);
	
    private ConetTE advertiser = null;
    private ConetTE sender = null;
    private ConetTE conetEngine = null;

    private static ConetEngineOrchestrator CONET_Engine_ORCHESTRATOR = null;

    protected ConetEngineOrchestrator(ConetTE conetEngine)
    {
		this.conetEngine = conetEngine;
    }

    public static ConetEngineOrchestrator getInstance(ConetTE conetEngine)
    {
        if (CONET_Engine_ORCHESTRATOR == null)
        {
            CONET_Engine_ORCHESTRATOR = new ConetEngineOrchestrator(conetEngine);
        }
        return CONET_Engine_ORCHESTRATOR;
    }

    public ConetTE getAdvertiser()
    {
        if (advertiser == null)
        {
            String advertiserCfgFileName = conetEngine.getPropertyValue("SAP_ADVERTISE_CFG");
            int procConfPathIndex = advertiserCfgFileName.lastIndexOf(File.separator);
            String procConfPath = advertiserCfgFileName.substring(0, procConfPathIndex);
            String procConfFile = advertiserCfgFileName.substring(procConfPathIndex + 1);
            
            advertiser = conetEngine.getConetTE(procConfPath, procConfFile);
            sender = advertiser;
        }

        return advertiser;
    }
    
    public String fetchResourceFromConet(String identifier) throws ConetIOException
    {
    	String temp_file_name = System.getProperty("java.io.tmpdir")+ File.separatorChar + "TEMP_VDI_FETCH" + UUID.randomUUID();
		
    	// TODO confirm the equivalence between NID and vdi_id
    	
    	NID nid = new NID(identifier);
		if (conetEngine.get(nid, temp_file_name)!=0)
		{
			logger.error("Cannot get an instance of CoNet while fetch and save of VDI " + identifier +" . Failure");
			throw new ConetIOException();
		}
		
		return temp_file_name;
    }
    
    public ConetTE getSender()
    {
        if (sender == null)
        {
//        	getAdvertiser();
        	//SS why you need to configure the sender using a different file??
            String senderCfgFileName = conetEngine.getPropertyValue("SAP_SEND_CFG");
            int procConfPathIndex = senderCfgFileName.lastIndexOf(File.separator);
            String procConfPath = senderCfgFileName.substring(0, procConfPathIndex);
            String procConfFile = senderCfgFileName.substring(procConfPathIndex + 1);
            sender = conetEngine.getConetTE(procConfPath, procConfFile);
        	
        	
        }
        return sender;
    }
}
