package eu.convergence.middleware.orchestratorTE.build;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.AND;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.BooleanExpression;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.InputQuery;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.OR;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.OutputDescription;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.QueryCondition;

import eu.convergence.middleware.engine.metadataTE.ConvergenceMetadataEngine;
import eu.convergence.middleware.engine.metadataTE.schemahandler.ConvergenceMetadata;
import eu.convergence.middleware.engine.metadataTE.schemahandler.Dimension;
import eu.convergence.middleware.engine.metadataTE.schemahandler.FieldValue;
import eu.convergence.middleware.engine.metadataTE.schemahandler.PublicationMetadata;
import eu.convergence.middleware.engine.metadataTE.schemahandler.ResourceMetadata;
import eu.convergence.middleware.engine.metadataTE.schemahandler.SubscriptionMetadata;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.ConvergenceMetadataParser;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.MetadataKinds;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.MetadataParser;
import eu.convergence.middleware.engine.mpqfTE.MpqfEngine;
import eu.convergence.middleware.engine.mpqfTE.technologyhandler.FieldValueQueryEditor;
import eu.convergence.middleware.engine.mpqfTE.technologyhandler.SparqlQueryEditor;
import eu.convergence.middleware.engine.vdiTE.VDIKinds;
import eu.convergence.middleware.engine.vdiTE.VdiEngine;

public class MetadataOrchestrator
{
	private ConvergenceMetadataEngine convMetadataEngine;
	private MpqfEngine mpqfEngine;
	private VdiEngine vdiEngine;
	
	public MetadataOrchestrator(VdiEngine vdiEngine, ConvergenceMetadataEngine convergenceMetadataEngine, MpqfEngine mpqfEngine) 
	{
		this.convMetadataEngine = convergenceMetadataEngine;
		this.mpqfEngine = mpqfEngine;
		this.vdiEngine = vdiEngine;
	}
	
//	public StructuredData addRDFTriple(StructuredData structuredData, String subject, String predicate, String object, String ontologyURI) throws MXMEngineException, SAXException, IOException, ParserConfigurationException, TransformerException
//	{
//		MetadataParserInterface metadataParser = genericMetadataEnine.getMetadataParser(new MXMAdapter(structuredData));
//		
//		if (metadataParser instanceof RdfMetadataParser)
//		{
//			RdfMetadataParser rdfMetadataParser = (RdfMetadataParser) metadataParser;
//			rdfMetadataParser.addRDFTriple(subject, predicate, object);
//			
//			Node node = rdfMetadataParser.writeModel();
//			
//			structuredData.setAny(node);
//		}
//		
//		return structuredData;
//	}
	
	public StructuredData generateResourceVdiMetadata(Date startDate, Date expiryDate, List<String> keywords, 
			List<String> tags, Map<String,List<Object>> fieldValues, List<StructuredData> structuredMetadata)
	{
		try
		{
			ConvergenceMetadata convMetadata = convMetadataEngine.getConvergenceMetadataSchemaHandler().getConvergenceMetadata();
			
			convMetadata.setExpiryDate(expiryDate);
			convMetadata.setStartDate(startDate);
			convMetadata.setVdiKind(VDIKinds.RVDI);
			convMetadata.setSequenceIdentifier("urn:sequence_identifier:temp:" + UUID.randomUUID());
	
			ResourceMetadata resMetadata = convMetadataEngine.getConvergenceMetadataSchemaHandler().getResourceMetadata();
	
			if (keywords!=null)
			{
				for (String keyword : keywords)
				{
					resMetadata.addKeyword(keyword);
				}
			}
	
			if (tags!=null)
			{
				for (String tag : tags)
				{
					resMetadata.addTag(tag);
				}
			}
	
			if (fieldValues!=null)
			{
				for (String field : fieldValues.keySet())
				{
					for (Object value : fieldValues.get(field))
					{
						resMetadata.addFieldValuePair(field, value);
					}
				}
			}
	
			if (structuredMetadata!=null)
			{
				for (StructuredData data : structuredMetadata)
				{
					resMetadata.addStructuredData(data);
				}
			}
	
			convMetadata.setResourceMetadata(resMetadata);
			
			StructuredData structuredData = vdiEngine.getDIEngineSchemaHandler().getStructuredData();
			structuredData.setRef(MetadataKinds.CONVERGENCE.getUri());
			structuredData.setAny(convMetadata);
			
			return structuredData;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	public StructuredData generatePublicationVdiMetadata(Date startDate, Date expiryDate, String corest, String rvdi_id,
																List<String> keywords, List<String> tags, Map<String,List<Object>> fieldValues, 
																List<StructuredData> structuredMetadata) throws MXMEngineException 
	{
		try
		{
			ConvergenceMetadata convergenceMetadata = convMetadataEngine.getConvergenceMetadataSchemaHandler().getConvergenceMetadata();
			convergenceMetadata.setExpiryDate(expiryDate);
			convergenceMetadata.setStartDate(startDate);
			convergenceMetadata.setVdiKind(VDIKinds.PVDI);
			convergenceMetadata.setSequenceIdentifier("urn:sequence_identifier:temp:" + UUID.randomUUID());
	
			Dimension dimension = convMetadataEngine.getConvergenceMetadataSchemaHandler().getDimension();
			if (corest!=null)
			{
				dimension.addFractal(corest);
			}
	
			PublicationMetadata pubMetadata = convMetadataEngine.getConvergenceMetadataSchemaHandler().getPublicationMetadata();
			pubMetadata.setRVDIIdentifier(rvdi_id);
			pubMetadata.addDimension(dimension);
	
			if (keywords!=null)
			{
				for (String keyword : keywords)
				{
					pubMetadata.addKeyword(keyword);
				}
			}
	
			if (tags!=null)
			{
				for (String tag : tags)
				{
					pubMetadata.addTag(tag);
				}
			}
	
			if (fieldValues!=null)
			{
				for (String field : fieldValues.keySet())
				{
					for (Object value : fieldValues.get(field))
					{
						pubMetadata.addFieldValuePair(field, value);
					}
				}
			}
	
			if (structuredMetadata!=null)
			{
				for (StructuredData data : structuredMetadata)
				{
					pubMetadata.addStructuredData(data);
				}
			}
	
			convergenceMetadata.setPublicationMetadata(pubMetadata);
	
			StructuredData structuredData = vdiEngine.getDIEngineSchemaHandler().getStructuredData();
			structuredData.setRef(MetadataKinds.CONVERGENCE.getUri());
			structuredData.setAny(convergenceMetadata);
			
			return structuredData;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	public StructuredData generateSubscriptionVdiMetadata(Date startDate, Date expiryDate, String corest, 
			Map<String,List<Object>> fieldValueEqualMap, Map<String,Object> fieldValueGreaterThanMap, 
			Map<String,Object> fieldValueLessThanMap, SparqlQueryEditor sparqlEditor) throws MXMEngineException 
	{
		try
		{
			ConvergenceMetadata convergenceMetadata = convMetadataEngine.getConvergenceMetadataSchemaHandler().getConvergenceMetadata();
			convergenceMetadata.setExpiryDate(expiryDate);
			convergenceMetadata.setStartDate(startDate);
			convergenceMetadata.setVdiKind(VDIKinds.SVDI);
			convergenceMetadata.setSequenceIdentifier("urn:sequence_identifier:temp:" + UUID.randomUUID());
	
			Dimension dimension = convMetadataEngine.getConvergenceMetadataSchemaHandler().getDimension();
			if (corest!=null)
			{
				dimension.addFractal(corest);
			}
	
			SubscriptionMetadata subMetadata = convMetadataEngine.getConvergenceMetadataSchemaHandler().getSubscriptionMetadata();
			subMetadata.addDimension(dimension);
	
			BooleanExpression fieldValueExpression = null;
			BooleanExpression sparqlExpression = null;
			
			FieldValueQueryEditor fieldValueQueryEditor = mpqfEngine.getMpqfTechnologyHandler().getFieldValueQueryEditor();
	
			if (fieldValueEqualMap!=null)
			{
				for (String field : fieldValueEqualMap.keySet())
				{
					for (Object value : fieldValueEqualMap.get(field))
					{
						fieldValueQueryEditor.addFieldOperatorValue(field, "=", value);
					}
				}
			}
	
			if (fieldValueLessThanMap!=null)
			{
				for (String field : fieldValueLessThanMap.keySet())
				{
					fieldValueQueryEditor.addFieldOperatorValue(field, "<", fieldValueLessThanMap.get(field));
				}
			}
	
			if (fieldValueGreaterThanMap!=null)
			{
				for (String field : fieldValueGreaterThanMap.keySet())
				{
					fieldValueQueryEditor.addFieldOperatorValue(field, ">", fieldValueGreaterThanMap.get(field));
				}
			}
			
			InputQuery inputQuery = mpqfEngine.getSearchEngineSchemaHandler().getInputQuery();
			OutputDescription outputDescription = mpqfEngine.getSearchEngineSchemaHandler().getOutputDescription(); // TODO use maxCount
			
			try
			{
				fieldValueExpression = fieldValueQueryEditor.generateBooleanExpression();
			}
			catch(Exception ex)
			{
				System.err.println(ex.getMessage());
			}
			
			try
			{
				sparqlExpression = sparqlEditor.generateBooleanExpression();
			}
			catch(Exception ex)
			{
				System.err.println(ex.getMessage());
			}
			
			if (fieldValueExpression!=null && sparqlExpression!=null)
			{
				OR orCondition = mpqfEngine.getSearchEngineSchemaHandler().getOR();
				
				orCondition.addCondition(fieldValueExpression);
				orCondition.addCondition(sparqlExpression);
				
				QueryCondition queryCondition = mpqfEngine.getSearchEngineSchemaHandler().getQueryCondition();
				queryCondition.setCondition(orCondition);
				
				inputQuery.setQueryCondition(queryCondition);
			}
			else if (fieldValueExpression!=null)
			{
				QueryCondition queryCondition = mpqfEngine.getSearchEngineSchemaHandler().getQueryCondition();
				queryCondition.setCondition(fieldValueExpression);
				
				inputQuery.setQueryCondition(queryCondition);
			}
			else if (sparqlExpression!=null)
			{
				QueryCondition queryCondition = mpqfEngine.getSearchEngineSchemaHandler().getQueryCondition();
				queryCondition.setCondition(sparqlExpression);
				
				inputQuery.setQueryCondition(queryCondition);
			}
			
			if (inputQuery.getQueryCondition()!=null)
			{
				subMetadata.setQuery(inputQuery);
				
				convergenceMetadata.setSubscriptionMetadata(subMetadata);
		
				StructuredData structuredData = vdiEngine.getDIEngineSchemaHandler().getStructuredData();
				structuredData.setRef(MetadataKinds.CONVERGENCE.getUri());
				structuredData.setAny(convergenceMetadata);
				
				return structuredData;
			}
			
			throw new MXMEngineException("Error - no subscription conditions provided.");
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	public void extractConvergenceMatadata(StructuredData structuredData, List<String> keywords, List<String> tags, 
			Map<String,List<Object>> fieldValues, List<StructuredData> structuredMetadata) throws MXMEngineException
	{
		MetadataParser metadataParser = convMetadataEngine.getConvergenceMetadataTechnologyHandler().getMetadataParser(structuredData);

		if(metadataParser.getMetadataKind() == MetadataKinds.CONVERGENCE)
		{
			ConvergenceMetadata convergenceMetadata = ((ConvergenceMetadataParser)metadataParser).getConvergenceMetadata() ;

			if (convergenceMetadata.getResourceMetadata()!=null)
			{
				for (String keyword : convergenceMetadata.getResourceMetadata().getKeywordList())
				{
					if (!keywords.contains(keyword))
					{
						keywords.add(keyword);
					}
				}
				
				for (String tag : convergenceMetadata.getResourceMetadata().getTagList())
				{
					if (!tags.contains(tag))
					{
						tags.add(tag);
					}
				}
				
				for (FieldValue fieldValue : convergenceMetadata.getResourceMetadata().getFieldValueList())
				{
					if (fieldValues.get(fieldValue.getField())==null)
					{
						fieldValues.put(fieldValue.getField(), new ArrayList<Object>());
					}

					if (!fieldValues.get(fieldValue.getField()).contains(fieldValue.getValue()))
					{
						fieldValues.get(fieldValue.getField()).add(fieldValue.getValue());
					}
				}

				structuredMetadata.addAll(convergenceMetadata.getResourceMetadata().getStructuredDataList());
			}
			else if (convergenceMetadata.getPublicationMetadata()!=null)
			{
				for (String keyword : convergenceMetadata.getPublicationMetadata().getKeywordList())
				{
					if (!keywords.contains(keyword))
					{
						keywords.add(keyword);
					}
				}
				
				for (String tag : convergenceMetadata.getPublicationMetadata().getTagList())
				{
					if (!tags.contains(tag))
					{
						tags.add(tag);
					}
				}
				
				for (FieldValue fieldValue : convergenceMetadata.getPublicationMetadata().getFieldValueList())
				{
					if (fieldValues.get(fieldValue.getField())==null)
					{
						fieldValues.put(fieldValue.getField(), new ArrayList<Object>());
					}

					if (!fieldValues.get(fieldValue.getField()).contains(fieldValue.getValue()))
					{
						fieldValues.get(fieldValue.getField()).add(fieldValue.getValue());
					}
				}

				structuredMetadata.addAll(convergenceMetadata.getPublicationMetadata().getStructuredDataList());
			}
		}
	}
	
}
