package eu.convergence.middleware.orchestratorTE.build;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import eu.convergence.middleware.conetTe.ConetTE;
import eu.convergence.middleware.engine.overlayTE.*;
import eu.convergence.middleware.event.PeerRegistrationEvent;
import org.apache.commons.collections.ListUtils;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.engine.vdiTE.VdiEngine;

import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.exceptions.DIEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;

import eu.convergence.middleware.overlayTE.OverlayEnginePaths;
import eu.convergence.middleware.overlayTE.OverlayManager;
import eu.convergence.middleware.overlayTE.schema.GorestMessage;
import eu.convergence.middleware.overlayTE.schema.PeerRegistry;


public class OverlayEngineOrchestrator 
{
	private static OverlayEngineOrchestrator OVERLAY_ENGINE_ORCHESTRATOR = null;
	
	private OverlayEngine overlayEngine;
	private OverlayManager overlayManager;
    private ConetTE advertiser;
    private ConetTE sender;
    private VdiEngine vdiEngine;
	
	protected OverlayEngineOrchestrator(OverlayEngine overlayEngine, ConetTE advertiser, ConetTE sender, VdiEngine vdiEngine)
	{
		this.overlayEngine = overlayEngine;
        this.advertiser = advertiser;
        this.sender = sender;
        this.vdiEngine = vdiEngine;
//        overlayEngine.configureCoNet(advertiser, sender);

        OverlayEnginePaths overlayEnginePaths = OverlayEnginePaths.getInstance();
        
        File file = new File(overlayEngine.getPropertyValue("PEER_PROPERITES_LOCATION"));
        if (!file.exists())
        {
        	file.mkdirs();
        }
        overlayEnginePaths.setPEER_PROPERITES_LOCATION(
                overlayEngine.getPropertyValue("PEER_PROPERITES_LOCATION"));
        
        file = new File(overlayEngine.getPropertyValue("PUBLICATION_MESSAGES_LOCATION"));
        if (!file.exists())
        {
        	file.mkdirs();
        }
        overlayEnginePaths.setPUBLICATION_MESSAGES_LOCATION(
                overlayEngine.getPropertyValue("PUBLICATION_MESSAGES_LOCATION"));
        
        file = new File(overlayEngine.getPropertyValue("READY_PUBLICATION_MESSAGES_LOCATION"));
        if (!file.exists())
        {
        	file.mkdirs();
        }
        overlayEnginePaths.setREADY_PUBLICATION_MESSAGES_LOCATION(
                overlayEngine.getPropertyValue("READY_PUBLICATION_MESSAGES_LOCATION"));
        
        file = new File(overlayEngine.getPropertyValue("REVOCATION_MESSAGES_LOCATION"));
        if (!file.exists())
        {
        	file.mkdirs();
        }
        overlayEnginePaths.setREVOCATION_MESSAGES_LOCATION(
                overlayEngine.getPropertyValue("REVOCATION_MESSAGES_LOCATION"));
        
        file = new File(overlayEngine.getPropertyValue("READY_REVOCATION_MESSAGES_LOCATION"));
        if (!file.exists())
        {
        	file.mkdirs();
        }
        overlayEnginePaths.setREADY_REVOCATION_MESSAGES_LOCATION(
                overlayEngine.getPropertyValue("READY_REVOCATION_MESSAGES_LOCATION"));
        
        file = new File(overlayEngine.getPropertyValue("READY_REGISTRY_LOCATION"));
        if (!file.exists())
        {
        	file.mkdirs();
        }
        overlayEnginePaths.setREADY_REGISTRY_LOCATION(
                overlayEngine.getPropertyValue("READY_REGISTRY_LOCATION"));
        
        file = new File(overlayEngine.getPropertyValue("READY_SUBSCRIPTION_MESSAGES_LOCATION"));
        if (!file.exists())
        {
        	file.mkdirs();
        }
        overlayEnginePaths.setREADY_SUBSCRIPTION_MESSAGES_LOCATION(
                overlayEngine.getPropertyValue("READY_SUBSCRIPTION_MESSAGES_LOCATION"));
        
        file = new File(overlayEngine.getPropertyValue("REGISTRY_LOCATION"));
        if (!file.exists())
        {
        	file.mkdirs();
        }
        overlayEnginePaths.setREGISTRY_LOCATION(
                overlayEngine.getPropertyValue("REGISTRY_LOCATION"));
        
        file = new File(overlayEngine.getPropertyValue("SUBSCRIPTION_MESSAGES_LOCATION"));
        if (!file.exists())
        {
        	file.mkdirs();
        }
        overlayEnginePaths.setSUBSCRIPTION_MESSAGES_LOCATION(
                overlayEngine.getPropertyValue("SUBSCRIPTION_MESSAGES_LOCATION"));

//        System.out.println("MXMConfiguration: ADVERTISE: " + overlayEngine.getPropertyValue("SAP_ADVERTISE_CFG"));
//        System.out.println("MXMConfiguration: SEND: " + overlayEngine.getPropertyValue("SAP_SEND_CFG"));
        this.overlayManager = OverlayManager.getInstance(OverlayEnginePaths.getInstance().getPEER_PROPERITES_LOCATION() + "peer_configuration.xml", overlayEngine.getPropertyValue("PEER_ID"), overlayEngine.getPropertyValue("OVERLAY_SAP"));
        overlayManager.setAdvertiser(advertiser);
        overlayManager.setSender(sender);
    }
	
	public static OverlayEngineOrchestrator getInstance(OverlayEngine overlayEngine, ConetTE advertiser, ConetTE sender, VdiEngine vdiEngine)
	{
		if (OVERLAY_ENGINE_ORCHESTRATOR == null)
		{
			OVERLAY_ENGINE_ORCHESTRATOR = new OverlayEngineOrchestrator(overlayEngine, advertiser, sender, vdiEngine);
		}
		return OVERLAY_ENGINE_ORCHESTRATOR;
	}
	
	public MXMEngineResponse bootOverlayEngine()
	{
        // peer entered the system; trigger the event
        PeerRegistrationEvent peerRegistrationEvent = new PeerRegistrationEvent(overlayManager.getPeerId(), overlayManager.getOverlaySAP(), this);
        try {
            MXM.getInstance().getMXMEventHandler().fireMXMEvent(peerRegistrationEvent);
        } catch (MXMConfigurationException e) {
            e.printStackTrace();
        }
		try {
			overlayEngine.getPropagationMessageHandler().recvPropagationMessageDaemon();
		} catch (OverlayEngineException e) {
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		
		return MXMEngineResponse.SUCCESS;
	}

    public String getOverlayPeerId()
    {
        return overlayManager.getPeerId();
    }
	
	public MXMEngineResponse gossipMessage(MXMObject message)
	{
		List<String> finalPeerList = new ArrayList<String>();
		List<String> finalSAPList = new ArrayList<String>();
		
		// Go through all the ready publication messages and send them to the peers of the fractals
		PropagationMessageHandler overlayPropagationMessageHandler = overlayEngine.getPropagationMessageHandler();
		
		GorestMessage gorestMessage = new GorestMessage(message);
		createPropagationLists(finalPeerList, finalSAPList, gorestMessage);

        int currentPeerIndex = finalPeerList.indexOf(overlayManager.getPeerId());
        if (currentPeerIndex != -1)
        {
            finalPeerList.remove(currentPeerIndex);
            finalSAPList.remove(currentPeerIndex);
        }

        // remove the peers that already exist in the path (already visited)
        List<String> existingPeersInPath = gorestMessage.getPath();
        int existingPeerIndex = 0;
        for (String existingPeer : existingPeersInPath)
        {
            if ((existingPeerIndex = finalPeerList.indexOf(existingPeer)) != -1)
            {
                finalPeerList.remove(existingPeerIndex);
                finalSAPList.remove(existingPeerIndex);
            }
        }
		
		// if the peers in the pool are less than 4 (~ exp + 1) then gossip just to them directly
		int count = 0;
		if (finalPeerList.size() < 4)
		{
			for (String SAP : finalSAPList)
			{
				String targetPeer = finalPeerList.get(count ++);
				gorestMessage.addPeerToPath(targetPeer);
			}
			for (String SAP : finalSAPList)
			{
				try {
					overlayPropagationMessageHandler.sendPropagationMessage(gorestMessage.toMXMObject(), SAP);
				} catch (OverlayEngineException e) {
					e.printStackTrace();
					return MXMEngineResponse.FAILURE;
				}
			}
		}
		else	// select logN peers uniformly at random from the pool and gossip to them
		{
			List<Integer> selectedTargets = new ArrayList<Integer>();
			double nofTargets = Math.log((double)finalPeerList.size());
			int standardNofTargets = Double.valueOf(nofTargets).intValue();
			double probNofTargets = nofTargets - standardNofTargets;
			
			// choose whether another peer should be added to the number of targets
			Random random = new Random();
			double prob = random.nextDouble();
			if (prob < probNofTargets)
			{
				standardNofTargets ++;
			}
			
			int nofSelectedTargets = 0;
			Integer targetPeerIndex;
			
			while (nofSelectedTargets < standardNofTargets)
			{
				do
				{
					targetPeerIndex = new Integer(random.nextInt(finalPeerList.size()));
				} while (selectedTargets.contains(targetPeerIndex));
				
				selectedTargets.add(targetPeerIndex);
				nofSelectedTargets ++;
				String currentTargetPeer = finalPeerList.get(targetPeerIndex.intValue());
				System.out.println("[OverlayEngineOrchestrator] Adding " + currentTargetPeer + " to path...");
				gorestMessage.addPeerToPath(currentTargetPeer);
			}
			
			// now send the message to the selected targets
			for (Integer target : selectedTargets)
			{
				// target is the index of the peer in the pool
				String SAP = finalSAPList.get(target.intValue());
				try {
					overlayPropagationMessageHandler.sendPropagationMessage(gorestMessage.toMXMObject(), SAP);
				} catch (OverlayEngineException e) {
					e.printStackTrace();
					return MXMEngineResponse.FAILURE;
				}
			}
		}
		
						
		return MXMEngineResponse.SUCCESS;
	}
	
	public MXMEngineResponse handlePublicationVDI(DIDL pVDI, Date expirationDate, String... fractals) throws OverlayEngineException
	{
        System.out.println("Handling publication VDI");
		// first register to the fractal (if not already registered)
		overlayManager.addLeaveDate(expirationDate);
		
		for (String fractal : fractals)
		{
			File file = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + fractal + ".xml");

			// the registry does not exist, so create it
			RegistryCreator registryCreator = overlayEngine.getRegistryCreator();
            try
            {
                registryCreator.registerToFractal(fractal, overlayManager.getOverlaySAP(), overlayManager.getLatestLeaveDate());
            }
            catch (MXMConfigurationException e) {
                e.printStackTrace();
            }
            catch (MXMEngineNotFoundException e) {
                e.printStackTrace();
            } 
            catch (MXMEngineException e)
			{
				e.printStackTrace();
			}
        }
		
		// second create and buffer the message
		GorestMessage gorestMessage = new GorestMessage();
		gorestMessage.setFractals(fractals);
		gorestMessage.addPeerToPath(overlayManager.getPeerId());
		
		try {
			//AD- TODO move to the tools 
			DIDL newDIDL = vdiEngine.getVDIEngineSchemaHandler().getDIDL();

			try {
				
//				newDIDL.initContext(VDIEngineContext.getInstance().getJAXBContext());
				
				newDIDL.setMPEGMItem(pVDI.getMPEGMItem());
			} 
			catch (MXMEngineException e)
			{
				e.printStackTrace();
			} 
			catch (MXMException e)
			{
				e.printStackTrace();
			}
			gorestMessage.setPublicationVDI(newDIDL.createObject(Byte.class.getCanonicalName()));
		} 
		catch (DIEngineException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		catch (MXMEngineException e1) 
		{
			e1.printStackTrace();
			return MXMEngineResponse.FAILURE;
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		
		MessageKeeper messageKeeper = overlayEngine.getMessageKeeper();
		List<String> finalPeerList = new ArrayList<String>();
		List<String> finalSAPList = new ArrayList<String>();
		
		createPropagationLists(finalPeerList, finalSAPList, gorestMessage);
		gorestMessage.setTTL(finalPeerList.size() + 1);		// when buffering, we reduce the TTL anyway
		
		messageKeeper.bufferMessage(gorestMessage.toMXMObject());
		
		return MXMEngineResponse.SUCCESS;
	}
	
	public MXMEngineResponse handleSubscriptionVDI(DIDL sVDI, Date expirationDate, String... fractals) throws OverlayEngineException
	{
		// first register to the fractal (if not already registered)
		overlayManager.addLeaveDate(expirationDate);
		
		for (String fractal : fractals)
		{
			File file = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + fractal + ".xml");
//			if (file.exists())	{continue;}
			
			// the registry does not exist, so create it
			RegistryCreator registryCreator = overlayEngine.getRegistryCreator();
            try 
            {
                registryCreator.registerToFractal(fractal, overlayManager.getOverlaySAP(), overlayManager.getLatestLeaveDate());
            }
            catch (MXMConfigurationException e)
            {
                e.printStackTrace();
            } 
            catch (MXMEngineNotFoundException e) 
            {
                e.printStackTrace();
            } 
            catch (MXMEngineException e)
			{
				e.printStackTrace();
			}
        }
		
		// second create and buffer the message
		GorestMessage gorestMessage = new GorestMessage();
		gorestMessage.setFractals(fractals);
		gorestMessage.addPeerToPath(overlayManager.getPeerId());
		
		
		try {
			//AD- TODO move to the tools 
			DIDL newDIDL=vdiEngine.getVDIEngineSchemaHandler().getDIDL();
			newDIDL.setMPEGMItem(sVDI.getMPEGMItem());
			
			//here i am!
			gorestMessage.setSubscriptionVDI(newDIDL.createObject(Byte.class.getCanonicalName()));
		} catch (DIEngineException e) {
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		} catch (MXMEngineException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();			

		} catch (MXMException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		MessageKeeper messageKeeper = overlayEngine.getMessageKeeper();
		List<String> finalPeerList = new ArrayList<String>();
		List<String> finalSAPList = new ArrayList<String>();
		
		createPropagationLists(finalPeerList, finalSAPList, gorestMessage);
		gorestMessage.setTTL(finalPeerList.size() + 1);		// when buffering, we reduce the TTL anyway
		
		messageKeeper.bufferMessage(gorestMessage.toMXMObject());
		
		return MXMEngineResponse.SUCCESS;
	}
	
	public MXMEngineResponse handleRevocationVDI(DIDL rVDI, String... fractals) throws OverlayEngineException
	{
		// first register to the fractal (if not already registered)		
		for (String fractal : fractals)
		{
			File file = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + fractal + ".xml");
//			if (file.exists())	{continue;}
			
			// the registry does not exist, so create it
			RegistryCreator registryCreator = overlayEngine.getRegistryCreator();
            try
            {
                registryCreator.registerToFractal(fractal, overlayManager.getOverlaySAP(), overlayManager.getLatestLeaveDate());
            } 
            catch (MXMConfigurationException e)
            {
                e.printStackTrace();
            }
            catch (MXMEngineNotFoundException e)
            {
                e.printStackTrace();
            } 
            catch (MXMEngineException e)
			{
				e.printStackTrace();
			}
        }
		
		// second create and buffer the message
		GorestMessage gorestMessage = new GorestMessage();
		gorestMessage.setFractals(fractals);
		gorestMessage.addPeerToPath(overlayManager.getPeerId());
		
		
		try 
		{
			gorestMessage.setRevocationVDI(rVDI.createObject(Byte.class.getCanonicalName()));
		} 
		catch (DIEngineException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		} 
		catch (MXMEngineException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		catch (MXMException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		
		MessageKeeper messageKeeper = overlayEngine.getMessageKeeper();
		List<String> finalPeerList = new ArrayList<String>();
		List<String> finalSAPList = new ArrayList<String>();
		
		createPropagationLists(finalPeerList, finalSAPList, gorestMessage);
		gorestMessage.setTTL(finalPeerList.size() + 1);		// when buffering, we reduce the TTL anyway
		
		messageKeeper.bufferMessage(gorestMessage.toMXMObject());
		
		return MXMEngineResponse.SUCCESS;
	}
	
	@SuppressWarnings("unchecked")
	private MXMEngineResponse createPropagationLists(List<String> finalPeerList, List<String> finalSAPList, GorestMessage gorestMessage)
	{
		RegistryParser overlayRegistryParser = overlayEngine.getRegistryParser();
		List<String> fractalId = gorestMessage.getFractals();
		
		File file = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + fractalId.get(0) + ".xml");
		try {
			overlayRegistryParser.parseFractalRegistry(file);
			PeerRegistry registry = new PeerRegistry(overlayRegistryParser.getRegistry());
			
			finalPeerList.addAll(registry.getPeerIds());
			finalSAPList.addAll(registry.getOverlaySAPs());
		} catch (OverlayEngineException e) {
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
	
		if (fractalId.size() > 1)
		{
			for (int i = 1; i < fractalId.size(); i ++)
			{
				String fractal = fractalId.get(i);
				File f = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + fractal + ".xml");
				try {
					overlayRegistryParser.parseFractalRegistry(f);
					PeerRegistry registry = new PeerRegistry(overlayRegistryParser.getRegistry());
					
					List<String> pool = new ArrayList<String>();
					List<String> poolSAP = new ArrayList<String>();
					
					pool.addAll(registry.getPeerIds());
					poolSAP.addAll(registry.getOverlaySAPs());
					
					finalPeerList = ListUtils.intersection(finalPeerList, pool);
					finalSAPList = ListUtils.intersection(finalSAPList, poolSAP);
				} catch (OverlayEngineException e) {
					e.printStackTrace();
					return MXMEngineResponse.FAILURE;
				}
			}
		}
		
		return MXMEngineResponse.SUCCESS;
	}
}
