package eu.convergence.middleware.orchestratorTE.build;

import java.io.File;
import java.io.FileInputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMInvalidConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.orchestratorEngine.createcontent.CreateContentEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.identifycontent.IdentifyContentEngineOrchestrator;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSchemaHandler;
import org.iso.mpeg.mxm.pEngine.createcontentPE.technologyhandler.CreateContentProtocol;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.IdentifyContentPE;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.schemahandler.IdentifyContentRequest;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.schemahandler.IdentifyContentResponse;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.technologyhandler.IdentifyContentProtocol;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Descriptor;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Statement;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.convergence.middleware.engine.ConetTe.ConetEngine;
import eu.convergence.middleware.engine.ConetTe.ConetTE;
import eu.convergence.middleware.engine.ConetTe.services.NID;
import eu.convergence.middleware.engine.vdiTE.VdiEngine;

public class VdiOrchestrator
{
	private Logger logger = LoggerFactory.getLogger(this.getClass());
	
	private CreateContentEngineOrchestrator cco;
	private IdentifyContentEngineOrchestrator ico;
	private VdiEngine vdiEngine;
	private ConetTE conetEngine;
	
	public VdiOrchestrator(VdiEngine vdiEngine, CreateContentEngineOrchestrator createContentEngineOrchestrator, 
						   IdentifyContentEngineOrchestrator identifyContentEngineOrchestrator, ConetTE conetEngine)
	{
		this.vdiEngine = vdiEngine;
		this.cco = createContentEngineOrchestrator;
		this.ico = identifyContentEngineOrchestrator; 
		this.conetEngine = conetEngine;
	}
	
	public String storeVDItoTemp(DIDL didl) // TODO maybe remove ??
	{
		String vdiId = null;
		try
		{
			vdiId = didl.getMPEGMItem().getIdentifier();
			
			String fileName = System.getProperty("java.io.tmpdir") + File.separatorChar + "VDI_SAVE_" + UUID.randomUUID();
			
			DIDL didlWithContext = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
			didlWithContext.parse(didl.createObject(JAXBElement.class.getCanonicalName()));
			
			didlWithContext.createFile(new File(fileName));

			logger.info("The VDI " + vdiId + " was successfully saved on file " + fileName);

			return fileName;
		} 
		catch (MXMException e)
		{
			logger.error("Strange... Saving VDI " + vdiId + " failed while generating file out of it", e);
		}
		
		return null;
	}
	
	public DIDL createVdiRemotely(List<Asset> assets, StructuredData metadata)
	{
		try
		{
			CreateContentRequest ccr = buildCreateContentRequest(assets, metadata);
			
			CreateContentProtocol createContentProtocol = 
					cco.getCreateContentEngine().getCreateContentTechnologyHandler().getRemoteCreateContentProtocol();
	
			try
			{
				URL serviceURL = new URL(cco.getCreateContentEngine().getPropertyValue("SERVICE_URL"));
				QName serviceName = new QName(cco.getCreateContentEngine().getPropertyValue("SERVICE_NAME_NS"), 
						cco.getCreateContentEngine().getPropertyValue("SERVICE_NAME"));
				
				createContentProtocol.initServiceParams(serviceURL, serviceName);
			} 
			catch (MXMInvalidConfigurationException e)
			{
				e.printStackTrace();
			} 
			catch (MalformedURLException e)
			{
				e.printStackTrace();
			}
			
			CreateContentResponse createContentResponse = createContentProtocol.createContent(ccr);
			
//			System.out.println(createContentResponse.getCreateContentSuccess().getContentEntity().getDIDL().createObject(String.class.getCanonicalName()).getContent());
			
			if (createContentResponse.getCreateContentSuccess()!=null)
			{
				DIDL didl = (DIDL) createContentResponse.getCreateContentSuccess().getContentEntity().getDIDL();
				
				logger.info("The VDI was successfully created");
				
				return didl;
			}
		}
		catch (Exception ex)
		{
			logger.error("Error while creating the VDI", ex);
		}
		
		return null;
	}
	
	public DIDL createVdiLocally(List<Asset> assets, StructuredData metadata)
	{
		try
		{
			CreateContentRequest ccr = buildCreateContentRequest(assets, metadata);
	
			if (ccr!=null)
			{
				CreateContentResponse createContentResponse = cco.createContent(ccr);
				
//				System.out.println(createContentResponse.getCreateContentSuccess().getContentEntity().getDIDL().createObject(String.class.getCanonicalName()).getContent());
				
				if (createContentResponse.getCreateContentSuccess()!=null)
				{
					DIDL didl = (DIDL) createContentResponse.getCreateContentSuccess().getContentEntity().getDIDL();
					
					logger.info("The VDI was successfully created");
					
					return didl;
				}
			}
		}
		catch (Exception ex)
		{
			logger.error("Error while creating the VDI", ex);
		}
		
		return null;
	}
	
	public CreateContentRequest buildCreateContentRequest(List<Asset> assets, StructuredData metadata)
	{
		try
		{
			CreateContentSchemaHandler ccsh = cco.getCreateContentEngine().getCreateContentSchemaHandler();
			
			CreateContentRequest ccr = ccsh.getCreateContentRequest();
			
			for (Asset asset : assets)
			{
				ccr.addAsset(asset);
			}
			
			if (metadata!=null)
			{
				Statement statement = vdiEngine.getDIEngineSchemaHandler().getStatement();
				statement.addMetadata(metadata);
				
				Descriptor descriptor = vdiEngine.getDIEngineSchemaHandler().getDescriptor();
				descriptor.setStatementMPEGM(statement);
				
				ccr.setMetadata(descriptor);
			}
		
			return ccr;
		}
		catch (Exception ex)
		{
			logger.error("Error creating CreateContentRequest", ex);
		}
		
		return null;
	}
	
	/**
	 * Get a VDI without identifier and add a proper identifier to it. If this protocol fails the original untouched VDI is returned
	 * @param original_vdi is the original DIDL object containing the VDI to be identified
	 * @return A new DIDL for a VDI with a correct identifier
	 */
	public DIDL identifyVdiLocally(DIDL didl)
	{
		try
		{
			IdentifyContentRequest icr = buildIdentifyContentRequest(didl);
			
			IdentifyContentResponse identifyContentResponse = ico.identifyContent(icr);
			
			if (identifyContentResponse.getIdentifyContentSuccess()!=null)
			{
				// TODO maybe create a new didl and parse the response
				
				DIDL identifiedDidl = (DIDL) identifyContentResponse.getIdentifyContentSuccess().getIdentifiedContent().getDIDL();
				
				logger.info("The VDI was successfully identified with id " + identifiedDidl.getMPEGMItem().getIdentifier());
				
				return identifiedDidl;
			}
		}
		catch (Exception ex)
		{
			logger.error("Error while identifying the VDI", ex);
		}
		
		return null;
	}
	
	/**
	 * Get a VDI without identifier and add a proper identifier to it. If this protocol fails the original untouched VDI is returned
	 * @param original_vdi is the original DIDL object containing the VDI to be identified
	 * @return A new DIDL for a VDI with a correct identifier
	 */
	public DIDL identifyVdiRemotelly(DIDL didl)
	{
		try
		{
			IdentifyContentRequest icr = buildIdentifyContentRequest(didl);
			
			IdentifyContentProtocol icp = ico.getIdentifyContentEngine().getTechnologyHandler().getRemoteIdentifyContentProtocol();
			
			try
			{
				URL serviceURL = new URL(ico.getIdentifyContentEngine().getPropertyValue("SERVICE_URL"));
				QName serviceName = new QName(ico.getIdentifyContentEngine().getPropertyValue("SERVICE_NAME_NS"), 
						ico.getIdentifyContentEngine().getPropertyValue("SERVICE_NAME"));
				
				icp.initServiceParams(serviceURL, serviceName);
			} 
			catch (MXMInvalidConfigurationException e)
			{
				e.printStackTrace();
			} 
			catch (MalformedURLException e)
			{
				e.printStackTrace();
			}
			
			IdentifyContentResponse identifyContentResponse = icp.identifyContent(icr);
			
			if (identifyContentResponse.getIdentifyContentSuccess()!=null)
			{
				// TODO maybe create a new didl and parse the response
				
				DIDL identifiedDidl = (DIDL) identifyContentResponse.getIdentifyContentSuccess().getIdentifiedContent().getDIDL();
				
				logger.info("The VDI was successfully identified with id " + identifiedDidl.getDIDLDocumentId());
				
				return identifiedDidl;
			}
		}
		catch (Exception ex)
		{
			logger.error("Error while identifying the VDI", ex);
		}
		
		return null;
	}
	
	public IdentifyContentRequest buildIdentifyContentRequest(DIDL didl)
	{
		String net_protocol = ((MXMEngine) conetEngine).getPropertyValue("NET_PROTOCOL"); 
		if((net_protocol != null) && (net_protocol.compareTo("ccnx") == 0)){
			try
			{
				Map<String,String> map = new HashMap<String,String>();
				map.put("PRINCIPAL", "ccnx:/" + ((MXMEngine) conetEngine).getPropertyValue("SERVING_NODE"));

				ContentEntity contentEntity = ico.getIdentifyContentEngine().getBaseProtocolSchemaHandler().getContentEntity();
				IdentifyContentRequest icr = ico.getIdentifyContentEngine().getSchemaHandler().getIdentifyContentRequest();


				Iterator<String> i = (Iterator<String>) map.keySet().iterator();
				while(i.hasNext())
				{
					Object o = i.next();
					org.iso.mpeg.mxm.pEngine.basePE.schemahandler.KeyValueData kvd= ico.getIdentifyContentEngine().getBaseProtocolSchemaHandler().getKeyValueData();
					kvd.setKey((String)o);
					kvd.setValue((String)(map.get(o)));
					icr.addEntry(kvd);
				}

				contentEntity.setDIDL(didl);

				icr.setContentEntity(contentEntity);

				return icr;
			}
			catch (Exception ex)
			{
				logger.error("Error creating IdentifyContentRequest", ex);
			}
		}
		else{
			try
			{
				ContentEntity contentEntity = ico.getIdentifyContentEngine().getBaseProtocolSchemaHandler().getContentEntity();
				contentEntity.setDIDL(didl);
				
				IdentifyContentRequest icr = ico.getIdentifyContentEngine().getSchemaHandler().getIdentifyContentRequest();
				icr.setContentEntity(contentEntity);
				
				return icr;
			}
			catch (Exception ex)
			{
				logger.error("Error creating IdentifyContentRequest", ex);
			}
		}
		
		return null;
	}
	
    /**
	 * Uses a CoNetTE instance to store and advertise the VDI into the network
	 * @param vdiFileName is the String of the file name under which the VDI was saved in the local file system
	 * @return boolean SUCCESS or FAILURE
	 */
	public boolean storeAndAdvertiseVDI(String vdiFileName)
	{
		// TODO storeAndAdvertise the VDIs under both the SEQ ID and the VDI ID
		
		try
		{
			DIDL didl = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
		
			logger.info("filename for reading is :"+vdiFileName);
			
			didl.parse(new MXMAdapter(new FileInputStream(vdiFileName)));
//			didl.parse(new MXMAdapter(new File(vdiFileName)));
			
			String dii = didl.getMPEGMItem().getIdentifier();
			
			NID nid = new NID(dii);
			if (conetEngine.storeAndAdvertise(nid, 0, vdiFileName) == 0)
			{
				return true;
			}
		}
		catch (Exception e1)
		{
			logger.error("Problem advertising VDI saved in " + vdiFileName +".", e1);
		}

		return false;
	}
	
}
