package eu.convergence.middleware.orchestratorTE.handle;

import eu.convergence.middleware.conetTe.ConetTE;
import eu.convergence.middleware.event.VDIMatchEvent;
import eu.convergence.middleware.eventreportTE.kernel.conet.EventReportTE;
import eu.convergence.middleware.eventreportTE.message.ERRepositoryType;
import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.event.management.MXMEvent;
import org.iso.mpeg.mxm.core.event.management.MXMEventListener;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.exception.EventReportException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERRKeeper;

/**
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 8/30/11
 * Time: 1:57 PM
 * <p/>
 * UTI Systems, TIC
 */
public class EREngineOrchestrator implements MXMEventListener {
    private static EREngineOrchestrator ER_ENGINE_ORCHESTRATOR = null;

    private EventReportEngine erEngine;

    private ConetTE advertiser;
    private ConetTE sender;

    private static final Logger log = Logger.getLogger(EREngineOrchestrator.class);

    protected EREngineOrchestrator(EventReportTE erEngine, ConetTE advertiser, ConetTE sender) {
        this.erEngine = erEngine;
        this.advertiser = advertiser;
        this.sender = sender;
        erEngine.configureCoNet(advertiser, sender); // TODO must make a erengine extension for the techhandler

        try {
            // register for the Match event
            MXM.getInstance().getMXMEventHandler().registerMXMEventListener(VDIMatchEvent.class.getName(), this);
        } catch (MXMConfigurationException e) {
            e.printStackTrace();
        }

    }

    public static EREngineOrchestrator getInstance(EventReportTE erEngine, ConetTE advertiser, ConetTE sender) {
        if (ER_ENGINE_ORCHESTRATOR == null) {
            ER_ENGINE_ORCHESTRATOR = new EREngineOrchestrator(erEngine, advertiser, sender);
        }
        return ER_ENGINE_ORCHESTRATOR;
    }

    public MXMEngineResponse bootEREngine() {
        try {
            erEngine.getEventReportEngineTechHandler().getERMessageHandler().recvERDaemon();
        } catch (EventReportException e) {
            e.printStackTrace();
            return MXMEngineResponse.FAILURE;
        } catch (MXMEngineException e) {
            e.printStackTrace();
            return MXMEngineResponse.FAILURE;
        }

        return MXMEngineResponse.SUCCESS;
    }


    /**
     * @return
     */
    public void handleIncommingERR(VDIMatchEvent evt) {
        String subscriberSAP = trySendERToSubscriber(evt);
        String publisherSAP = trySendERToPublisher(evt, subscriberSAP);
    }

    /**
     * @param evt
     * @return Return the subscriber's SAP name
     */
    private String trySendERToSubscriber(VDIMatchEvent evt) {
        String subscriberSAP = null;
        try {
            // get the ERR object by S-VDI identifier
            ERRKeeper errKeeperSVDI = erEngine.getEventReportEngineTechHandler().getEventReportRequestKeeper(ERRepositoryType.SVDI.getName());
            ERR err = errKeeperSVDI.getERRByDIDLId(evt.getSubscriptionVdiId());

            if (null != err) {
                subscriberSAP = err.getSAP();

                // B. creates a ER parsing data of the ERR and MXMMatchEvent .
                ER er = erEngine.getEventReportEngineSchemaHandler().getER();
                er.setERRIdentifier(err.getERRIdentifier());
                er.setDIIId(evt.getPublicationVdiId());

                // C. stores the ER in the peer
                ERKeeper erKeeperPVDI = erEngine.getEventReportEngineTechHandler().getEventReportKeeper(ERRepositoryType.SVDI.getName());
                erKeeperPVDI.storeER(er);

                // D. sends the ER with the ERsender to the ERListener
                MXMEngineResponse response = erEngine.getEventReportEngineTechHandler().getERMessageHandler().sendER(er, subscriberSAP);
                log.info("Sending ER to subscriber with status: " + response);
            } else {
                log.info("No ERR found for SubscriptionVdiId: " + evt.getSubscriptionVdiId());
            }
        } catch (EventReportException e) {
            log.info("Error in sending ER to subscriber. Details: " + e.getMessage());
            e.printStackTrace();
        } catch (MXMEngineException e) {
            log.info("MXM engine error. Details: " + e.getMessage());
            e.printStackTrace();
        }

        return subscriberSAP;
    }

    private String trySendERToPublisher(VDIMatchEvent evt, String subscriberSAP) {
        String publisherSAP = null;
        try {
            // get the ERR object by P-VDI identifier
            ERRKeeper errKeeperPVDI = erEngine.getEventReportEngineTechHandler().getEventReportRequestKeeper(ERRepositoryType.PVDI.getName());
            ERR errPVDI = errKeeperPVDI.getERRByDIDLId(evt.getPublicationVdiId());
            if (null != errPVDI) {
                publisherSAP = errPVDI.getSAP();

                if (null != subscriberSAP && !subscriberSAP.equals(publisherSAP)) {
                    // get the ERR object by S-VDI identifier
                    ERRKeeper errKeeperSVDI = erEngine.getEventReportEngineTechHandler().getEventReportRequestKeeper(ERRepositoryType.SVDI.getName());
                    ERR errSVDI = errKeeperSVDI.getERRByDIDLId(evt.getSubscriptionVdiId());

                    // B. creates a ER parsing data of the ERR and MXMMatchEvent .
                    ER er = erEngine.getEventReportEngineSchemaHandler().getER();
                    er.setERRIdentifier(errSVDI.getERRIdentifier());
                    er.setDIIId(evt.getPublicationVdiId());

                    // C. stores the ER in the peer
                    ERKeeper erKeeperPVDI = erEngine.getEventReportEngineTechHandler().getEventReportKeeper(ERRepositoryType.PVDI.getName());
                    erKeeperPVDI.storeER(er);

                    // D. sends the ER with the ERsender to the ERListener
                    MXMEngineResponse response = erEngine.getEventReportEngineTechHandler().getERMessageHandler().sendER(er, publisherSAP);
                    log.info("Sending ER to publisher with status: " + response);
                } else {
                    log.info("The publisher has the same SAP as the subscriber. No ER sent!");
                }
            } else {
                log.info("No ERR found for PublicationVdiId: " + evt.getPublicationVdiId());
            }
        } catch (EventReportException e) {
            log.info("Error in sending ER to publisher. Details: " + e.getMessage());
            e.printStackTrace();
        } catch (MXMEngineException e) {
            log.info("MXM engine error. Details: " + e.getMessage());
            e.printStackTrace();
        }

        return publisherSAP;
    }

    @Override
    public void mxmEventOccurred(MXMEvent event) {
        // EREngineOrchestrator handles the MXMMatchEvent.
        if (event instanceof VDIMatchEvent) {
            VDIMatchEvent evt = (VDIMatchEvent) event;

            // A. finds the matching ERR for the MXMMatchEvent.
            handleIncommingERR(evt);
        }
    }

    public EventReportEngine getEREngine() {
        return erEngine;
    }
}
