package eu.convergence.middleware.orchestratorTE.handle;

import eu.convergence.middleware.cdsTE.CdsTE;
import eu.convergence.middleware.engine.cdsTE.CdsEngine;
import eu.convergence.middleware.engine.matchTE.MatchEngine;
import eu.convergence.middleware.engine.matchTE.PublicationManager;
import eu.convergence.middleware.engine.matchTE.SubscriptionManager;
import eu.convergence.middleware.engine.metadataTE.ConvergenceMetadataEngine;
import eu.convergence.middleware.engine.metadataTE.schemahandler.ConvergenceMetadata;
import eu.convergence.middleware.engine.metadataTE.schemahandler.FieldValue;
import eu.convergence.middleware.engine.metadataTE.schemahandler.PublicationMetadata;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.ConvergenceMetadataParser;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.MetadataParser;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.RdfMetadataParser;
import eu.convergence.middleware.engine.mpqfTE.MpqfEngine;
import eu.convergence.middleware.engine.mpqfTE.technologyhandler.FieldOperatorValue;
import eu.convergence.middleware.engine.mpqfTE.technologyhandler.FieldValueQueryEditor;
import eu.convergence.middleware.engine.mpqfTE.technologyhandler.BooleanExpressionEditor;
import eu.convergence.middleware.engine.mpqfTE.technologyhandler.SparqlQueryEditor;
import eu.convergence.middleware.engine.vdiTE.VdiEngine;
import eu.convergence.middleware.event.VDIArrivalEvent;
import eu.convergence.middleware.event.VDIMatchEvent;
import eu.convergence.middleware.eventreportTE.message.ERRepositoryType;
import eu.convergence.middleware.matchTE.metadata.FieldValueMetadata;
import eu.convergence.middleware.matchTE.metadata.RdfMetadata;
import eu.convergence.middleware.matchTE.query.FieldValueQuery;
import eu.convergence.middleware.matchTE.query.QueryObject;
import eu.convergence.middleware.matchTE.query.SparqlQuery;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.event.management.MXMEvent;
import org.iso.mpeg.mxm.core.event.management.MXMEventListener;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.AND;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.BooleanExpression;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.InputQuery;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.OR;

import com.hp.hpl.jena.rdf.model.Statement;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


public class MatchEngineOrchestrator implements MXMEventListener
{
	private static MatchEngineOrchestrator matchOrchestrator = null;
	
	private MatchEngine matchEngine;
	private VdiEngine vdiEngine;
	private ConvergenceMetadataEngine convergenceMetadataEngine;
	private EventReportEngine erEngine;
	private CdsEngine cdsEngine;
	private MpqfEngine mpqfEngine;
	
	public static MatchEngineOrchestrator getInstance(MatchEngine matchEngine, VdiEngine vdiEngine, ConvergenceMetadataEngine metadataEngine, 
														EventReportEngine erEngine, CdsEngine cdsEngine, MpqfEngine mpqfEngine) 
	{
		if (matchOrchestrator == null)
		{
			matchOrchestrator = new MatchEngineOrchestrator(matchEngine, vdiEngine, metadataEngine, erEngine, cdsEngine, mpqfEngine);
		}

		return matchOrchestrator;
	}
	
	private MatchEngineOrchestrator(MatchEngine matchEngine, VdiEngine vdiEngine, ConvergenceMetadataEngine metadataEngine, 
			EventReportEngine erEngine, CdsEngine cdsEngine, MpqfEngine mpqfEngine)
	{
		this.matchEngine = matchEngine;
		this.vdiEngine = vdiEngine;
		this.convergenceMetadataEngine = metadataEngine;
		this.erEngine = erEngine;
		this.cdsEngine = cdsEngine;
		this.mpqfEngine = mpqfEngine;
		
	}
	
	public void bootMatchEngine(boolean resetRepositories)
	{
		matchEngine.getRepositoryManager().initRepositories(resetRepositories);
		
		try 
		{
			MXM.getInstance().getMXMEventHandler().registerMXMEventListener(VDIArrivalEvent.class.getName(), this);
		} 
		catch (MXMConfigurationException e) 
		{
			e.printStackTrace();
		}
	}
	
	public List<String> handleIncomingSubscription(InputStream S_VDI_IS) throws MXMException
	{
		ArrayList<String> matchingPubs = new ArrayList<String>();
		
		try
		{	
			DIDL didl = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
			didl.parse(new MXMAdapter(S_VDI_IS));
			
			String S_VDI_Id = didl.getMPEGMItem().getIdentifier();
			
			System.out.println("Parsing S-VDI: " + S_VDI_Id);
			
			MetadataParser genericMetadataParser = convergenceMetadataEngine.getConvergenceMetadataTechnologyHandler().
																		getMetadataParser(didl.getMPEGMItem().getMetadata());
			
			if (genericMetadataParser instanceof ConvergenceMetadataParser)
			{
				ConvergenceMetadata convergenceMetadata = ((ConvergenceMetadataParser) genericMetadataParser).getConvergenceMetadata();
				
				BooleanExpression queryBooleanExpression = convergenceMetadata.getSubscriptionMetadata().getQuery().getQueryCondition().getCondition();

				// TODO move FractalAlgebra in gorest namespace
				// TODO extend this to pass to the overlay engine dimension object
				
				// get dimensions 
				List<String> coreSTs = new ArrayList<String>();
				coreSTs.addAll(convergenceMetadata.getSubscriptionMetadata().getDimensionList().get(0).getFractalList());
				
				FieldValueQueryEditor fieldValueQueryEditor = null;
				SparqlQueryEditor sparqlQueryEditor = null;
				
				if (queryBooleanExpression instanceof OR)
				{
					OR or = (OR) queryBooleanExpression;
					
					for (BooleanExpression booleanExpression : or.getConditionList())
					{
						BooleanExpressionEditor queryBooleanExpressionEditor = mpqfEngine.getMpqfTechnologyHandler().getBooleanExpressionEditor(booleanExpression);
						
						if (queryBooleanExpressionEditor instanceof FieldValueQueryEditor)
						{
							fieldValueQueryEditor = (FieldValueQueryEditor) queryBooleanExpressionEditor;
						}
						else if (queryBooleanExpressionEditor instanceof SparqlQueryEditor)
						{
							sparqlQueryEditor = (SparqlQueryEditor) queryBooleanExpressionEditor;
						}
					}
				}
				else
				{
					BooleanExpressionEditor queryBooleanExpressionEditor = mpqfEngine.getMpqfTechnologyHandler().getBooleanExpressionEditor(queryBooleanExpression);
					
					if (queryBooleanExpressionEditor instanceof FieldValueQueryEditor)
					{
						fieldValueQueryEditor = (FieldValueQueryEditor) queryBooleanExpressionEditor;
					}
					else if (queryBooleanExpressionEditor instanceof SparqlQueryEditor)
					{
						sparqlQueryEditor = (SparqlQueryEditor) queryBooleanExpressionEditor;
					}
				}
				
				FieldValueQuery fieldValueQuery = null;
				
				if (fieldValueQueryEditor!=null)
				{
					List<FieldOperatorValue> fieldOperatorValue = fieldValueQueryEditor.getFieldOperatorValueList();

					fieldValueQuery = new FieldValueQuery();
					
					for(FieldOperatorValue fov : fieldOperatorValue)
					{
						if ("<".equals(fov.getOperator()))
						{
							fieldValueQuery.getLessThanMap().put(fov.getField(), fov.getValue());
						}
						else if (">".equals(fov.getOperator()))
						{
							fieldValueQuery.getGreaterThanMap().put(fov.getField(), fov.getValue());
						}
						else if ("=".equals(fov.getOperator()))
						{
							if (fieldValueQuery.getEqualMap().get(fov.getField())==null)
							{
								fieldValueQuery.getEqualMap().put(fov.getField(), new ArrayList<Object>());
							}

							fieldValueQuery.getEqualMap().get(fov.getField()).add(fov.getValue());
						}
					}
					
					fieldValueQuery.setCorests(coreSTs);
				}
				
				SparqlQuery sparqlQuery = null;
				
				if (sparqlQueryEditor!=null)
				{
					sparqlQuery = new SparqlQuery(sparqlQueryEditor.getSparqlString());
					
					sparqlQuery.setCorests(coreSTs);
				}
				
				// get expiry date
				Date expiryDate = convergenceMetadata.getExpiryDate();
				
				for (Item item : didl.getMPEGMItem().getItem())
				{
					if (item.getERR()!=null)
					{
						// call EventReportEngine to store the ERR
						for (ERR err : item.getERR())
						{
							erEngine.getEventReportEngineTechHandler().getEventReportRequestKeeper(ERRepositoryType.SVDI.name()).storeERR(err, S_VDI_Id);
						}
					}
				}
				
				// call subsriptionManager to get any matches
				SubscriptionManager sManager = matchEngine.getSubscriptionManager();
				
				Set<String> matches = new HashSet<String>();
				
				if (fieldValueQuery!=null)
				{
					List<String> fieldValueMatches = sManager.matchPubsWith(S_VDI_Id, new MXMAdapter(fieldValueQuery), expiryDate);
					
					System.out.println("### " + fieldValueMatches.size() + " FIELD/VALUE MATCHES for " + S_VDI_Id);
					
					matches.addAll(fieldValueMatches);
				}
				
				if (sparqlQuery!=null)
				{
					List<String> sparqlMatches = sManager.matchPubsWith(S_VDI_Id, new MXMAdapter(sparqlQuery), expiryDate);
					
					System.out.println("### " + sparqlMatches.size() + " SPARQL MATCHES for " + S_VDI_Id);
					
					matches.addAll(sparqlMatches);
				}
				
				System.out.println("### " + matches.size() + " MATCHES TOTALLY for " + S_VDI_Id);
				
				// generate match events and forward them to the core
				for (String P_VDI_Id : matches)
				{
					VDIMatchEvent matchEvent = new VDIMatchEvent(S_VDI_Id,P_VDI_Id, matchEngine);
					
					try
					{
	                    MXM.getInstance().getMXMEventHandler().fireMXMEvent(matchEvent);
	                }
					catch (MXMConfigurationException e) 
					{
	                    System.err.println("Could not fire the VDIMatchEvent");
	                    e.printStackTrace();
	                }
				}
			}
			else
			{
//				throw UnsupportedMatchingOperation
			}
		} 
		catch (MXMEngineException e)
		{
			e.printStackTrace();
		}
		
		return matchingPubs;
	}
	
	public List<String> handleIncomingPublication(InputStream P_VDI_IS) throws MXMException
	{
		ArrayList<String> matchingSubs = new ArrayList<String>();
		
		try
		{	
			DIDL didl = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
			didl.parse(new MXMAdapter(P_VDI_IS));
			
			String P_VDI_Id = didl.getMPEGMItem().getIdentifier();

			System.out.println("Parsing P-VDI: " + P_VDI_Id);
			
			MetadataParser genericMetadataParser = convergenceMetadataEngine.getConvergenceMetadataTechnologyHandler().
					getMetadataParser(didl.getMPEGMItem().getMetadata());
			
			if (genericMetadataParser instanceof ConvergenceMetadataParser)
			{
				ConvergenceMetadata convergenceMetadata = ((ConvergenceMetadataParser) genericMetadataParser).getConvergenceMetadata();
				
				PublicationMetadata pubMetadata = convergenceMetadata.getPublicationMetadata();
				
				FieldValueMetadata fieldValueMetadata = new FieldValueMetadata();
				RdfMetadata rdfMetadata = new RdfMetadata();
						
				if (pubMetadata.getKeywordList()!=null)
				{
					if (fieldValueMetadata.getEqualMap().get("KEYWORD")==null)
					{
						fieldValueMetadata.getEqualMap().put("KEYWORD", new ArrayList<Object>());
					}
					
					for (String keyword : pubMetadata.getKeywordList())
					{
						fieldValueMetadata.getEqualMap().get("KEYWORD").add(keyword);
					}
				}
				
				if (pubMetadata.getTagList()!=null)
				{
					List<String> expandedKeywords = cdsEngine.getSemanticExpander().expandKeywordQueryByURI(pubMetadata.getTagList());
					
					for (String keyword : expandedKeywords)
					{
						System.out.println("adding expanded keyword : " + keyword);
						fieldValueMetadata.getEqualMap().get("KEYWORD").add(keyword);
					}
					
					for (String tag : pubMetadata.getTagList())
					{
						System.out.println("adding tag : " + tag);
						fieldValueMetadata.getEqualMap().get("KEYWORD").add(tag);
					}
				}
				
				if (pubMetadata.getFieldValueList()!=null)
				{
					for (FieldValue fieldValue : pubMetadata.getFieldValueList())
					{
						if (fieldValueMetadata.getEqualMap().get(fieldValue.getField())==null)
						{
							fieldValueMetadata.getEqualMap().put(fieldValue.getField(), new ArrayList<Object>());
						}
						
						fieldValueMetadata.getEqualMap().get(fieldValue.getField()).add(fieldValue.getValue());
					}
				}
				
				if (pubMetadata.getStructuredDataList()!=null)
				{
					for (StructuredData structuredData : pubMetadata.getStructuredDataList())
					{
						MetadataParser metadataParser = convergenceMetadataEngine.getConvergenceMetadataTechnologyHandler().
								getMetadataParser(structuredData);
						
						if (metadataParser instanceof RdfMetadataParser)
						{
							rdfMetadata.addStatements(((RdfMetadataParser) metadataParser).listStatements()); // TODO replace jena statements with triple patterns
							
							// TODO expand metadata
						}
					}
				}
				
				// TODO move FractalAlgebra in gorest namespace
				// TODO extend this to pass to the overlay engine dimension object
				
				// get dimensions 
				List<String> coreSTs = new ArrayList<String>();
				coreSTs.addAll(convergenceMetadata.getPublicationMetadata().getDimensionList().get(0).getFractalList());
				
				fieldValueMetadata.setCorests(coreSTs);
				rdfMetadata.setCorests(coreSTs);
				
				// get expiry date
				Date expiryDate = convergenceMetadata.getExpiryDate();
				
				//call to EventReportEngine to store the ERR.
				for (Item item : didl.getMPEGMItem().getItem())
				{
					if (item.getERR()!=null)
					{
						// call EventReportEngine to store the ERR
						for (ERR err : item.getERR())
						{
							erEngine.getEventReportEngineTechHandler().getEventReportRequestKeeper(ERRepositoryType.PVDI.name()).storeERR(err, P_VDI_Id);
						}
					}
				}
				
				// call subsriptionManager to get any matches
				PublicationManager pManager = matchEngine.getPublicationManager();
				
				Set<String> matches = new HashSet<String>();
				
				List<String> fieldValueMatches = pManager.matchSubsWith(P_VDI_Id, new MXMAdapter(fieldValueMetadata), expiryDate);
				
				System.out.println("### " + fieldValueMatches.size() + " FIELD/VALUE MATCHES for " + P_VDI_Id);
				
				matches.addAll(fieldValueMatches);
				
				if (rdfMetadata.getStatements().size()>0)
				{
					List<String> rdfMatches = pManager.matchSubsWith(P_VDI_Id, new MXMAdapter(rdfMetadata), expiryDate);
					
					System.out.println("### " + rdfMatches.size() + " RDF MATCHES for " + P_VDI_Id);
					
					matches.addAll(rdfMatches);
				}
				
				System.out.println("### " + matches.size() + " MATCHES TOTALLY for " + P_VDI_Id);
				
				// generate match events and forward them to the core
				for (String S_VDI_Id : matches)
				{
					VDIMatchEvent matchEvent = new VDIMatchEvent(S_VDI_Id,P_VDI_Id, matchEngine);
					
					try
					{
	                    MXM.getInstance().getMXMEventHandler().fireMXMEvent(matchEvent);
	                }
					catch (MXMConfigurationException e) 
					{
	                    System.err.println("Could not fire the VDIMatchEvent");
	                    e.printStackTrace();
	                }
				}
			}
			else
			{
//				throw UnsupportedMatchingOperation
			}
		} 
		catch (MXMEngineException e)
		{
			e.printStackTrace();
		}
		
		return matchingSubs;
	}

	public void mxmEventOccurred(MXMEvent event)
	{
		if (event instanceof VDIArrivalEvent)
		{
			VDIArrivalEvent evt = (VDIArrivalEvent) event;
			
			if (evt.getVdiPayload().getContentType().equals(new byte[0].getClass().getCanonicalName()))
			{
				byte[] byteArray = (byte[]) evt.getVdiPayload().getContent();
				
				ByteArrayInputStream bais = new ByteArrayInputStream(byteArray);
				
				try
				{
					if (evt.getVdiType()==VDIArrivalEvent.PUBLICATION_VDI)
					{
						handleIncomingPublication(bais);
					}
					else if (evt.getVdiType()==VDIArrivalEvent.SUBSCRIPTION_VDI)
					{
						handleIncomingSubscription(bais);
					}
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
				}
			}
		}
	}

}
