package eu.convergence.middleware.orchestratorTE.schedule.task;

import eu.convergence.middleware.conetTe.ConetTE;
import eu.convergence.middleware.core.ConvergenceEngineName;
import eu.convergence.middleware.engine.ConetTe.exception.ConetIOException;
import eu.convergence.middleware.engine.ConetTe.services.NID;
import eu.convergence.middleware.engine.overlayTE.RegistryCreator;
import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.overlayTE.OverlayEnginePaths;
import eu.convergence.middleware.overlayTE.OverlayManager;
import eu.convergence.middleware.overlayTE.kernel.conet.OverlayTE;
import eu.convergence.middleware.overlayTE.message.Opcodes;
import eu.convergence.middleware.overlayTE.message.OverlayMessageCreator;
import eu.convergence.middleware.overlayTE.registry.OverlayRegistryParser;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.HeaderType;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.PayloadType;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.RegistrationMessageType;
import eu.convergence.middleware.overlayTE.schema.registry.ObjectFactory;
import eu.convergence.middleware.overlayTE.schema.registry.RegistryType;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.File;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Random;
import java.util.TimerTask;

/**
 * Created by IntelliJ IDEA.
 * User: aca
 * Date: 7/27/11
 * Time: 10:15 AM
 */
public class RegistryAdvertisementTask extends TimerTask
{
    public void run()
    {
        OverlayTE overlayTE = new OverlayTE();
//        overlayTE.configureCoNet();
        // read all the registries from the file system
		File registryDir = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION());
		File[] registryFiles = registryDir.listFiles();
		String tempRegLocation;

		// read the corresponding advertised registry and update accordingly
		for (File currentRegistry : registryFiles)
		{
            if (currentRegistry.isDirectory())  {continue;}
            if (currentRegistry.getName().contains(".xml"))
            {
            	OverlayRegistryParser overlayRegistryParser = overlayTE.getRegistryParser();
    			try {
    				overlayRegistryParser.parseFractalRegistry(currentRegistry);
    				RegistryType registry = (RegistryType)overlayRegistryParser.getRegistry().getContent();

//    				ConetTE conetTE = new ConetTE();
    				//SS: is there a conetTE already available somewhere ??
//    				File ff = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
    				MXM mxm = MXM.getInstance();
//    				mxm.listEnginesAndProperties();

    				ConetTE conetTE = (ConetTE) mxm.getDefaultEngine(ConvergenceEngineName.ConetTE.toString());
    				
    				NID regNID = new NID(OverlayTE.registryFormat + registry.getFractal());
    				tempRegLocation = OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + "tmp_reg.xml";

                    int fileExists = conetTE.get(regNID, tempRegLocation);

                    if (fileExists == 0)    // this means that the file does exist
                    {
                        File tmp = new File(tempRegLocation);
                        overlayRegistryParser.parseFractalRegistry(tmp);
                        RegistryType advertisedRegistry = (RegistryType)overlayRegistryParser.getRegistry().getContent();

                        // merge the lists of the advertised registry with the ones of the current one
                        List<String> currentRegistryPeers = registry.getPeerIds();

                        List<String> advertisedRegistryPeers = advertisedRegistry.getPeerIds();
                        int index = 0;

                        for (String advertisedPeer : advertisedRegistryPeers)
                        {
                        	// sending REGISTRATION_MESSAGE to all peers in the advertised registry
                        	// to confirm they are active
                        	// set up the gossip registration message
                    		HeaderType header = new HeaderType();
                    		header.getFractals().add(advertisedRegistry.getFractal());
                    		header.setOpcode(Opcodes.REGISTRATION_MESSAGE);
                    		header.getPath().add(OverlayManager.getInstance().getPeerId());
                    		
                    		int fractalMembers = registry.getFractalMembers().intValue();
                    		double nofRounds = Math.log((double)fractalMembers);
                    		int standardNofRounds = Double.valueOf(nofRounds).intValue();
                    		header.setTTL(standardNofRounds);
                    		
                    		PayloadType payload = new PayloadType();
                    		RegistrationMessageType registrationDiscoveryMessage = new RegistrationMessageType();
                    		registrationDiscoveryMessage.setPeerId(OverlayManager.getInstance().getPeerId());
                    		registrationDiscoveryMessage.setOverlaySAP(OverlayManager.getInstance().getOverlaySAP());
                    		
                    		try 
                    		{
                    			if (OverlayManager.getInstance().getLatestLeaveDate()!=null)
                    			{
	                    			DatatypeFactory df = DatatypeFactory.newInstance();
	                    			GregorianCalendar gc = new GregorianCalendar();
	                    			gc.setTime(OverlayManager.getInstance().getLatestLeaveDate());
	                    			registrationDiscoveryMessage.setLeaveDate(df.newXMLGregorianCalendar(gc));
                    			}
                    		} catch (DatatypeConfigurationException e) {
                    			e.printStackTrace();
                    		}
                    		
                    		payload.setDiscoveryMessage(registrationDiscoveryMessage);
                    		
//                    		OverlayTE overlayTE = new OverlayTE();
                    		
                    		OverlayMessageCreator overlayMessageCreator = overlayTE.getMessageCreator();
                    		MXMObject message = overlayMessageCreator.createMessage(new MXMAdapter(header), new MXMAdapter(payload));
                    		
                    		overlayTE.getPropagationMessageHandler().sendPropagationMessage(message, advertisedRegistry.getOverlaySAPs().get(index));
                    		index ++;
                    		
//                            System.out.println("REGISTRY ADVERTISER: Checking peer " + advertisedPeer);
//        //					if (!currentRegistryPeers.contains(advertisedPeer))
//        //					{
//                                int index = advertisedRegistryPeers.indexOf(advertisedPeer);
//                                String advertisedSAP = advertisedRegistry.getOverlaySAPs().get(index);
//                                XMLGregorianCalendar advertisedLeaveDate = advertisedRegistry.getLeaveDates().get(index);
//
//                                int currentPeerCnt = currentRegistryPeers.indexOf(advertisedPeer);
//                                if (currentPeerCnt != -1)
//                                {
//                                    registry.getPeerIds().remove(currentPeerCnt);
//                                    registry.getOverlaySAPs().remove(currentPeerCnt);
//                                    registry.getLeaveDates().remove(currentPeerCnt);
//                                }
//
//                                registry.getPeerIds().add(advertisedPeer);
//                                registry.getOverlaySAPs().add(advertisedSAP);
//                                registry.getLeaveDates().add(advertisedLeaveDate);
//        //					}
//                        }
//
//                        // remove the advertised registry from the file system
//                        tmp.delete();
//
//                        // store the updated registry
////                        overlayRegistryParser.parseFractalRegistry(currentRegistry);
//                        overlayRegistryParser.updateRegistry(registry);
//                        overlayRegistryParser.storeRegistry();
                        }
                    }

    				// advertise a logN part of the new registry if big enough
    				if (registry.getPeerIds().size() < 4)
    				{
    					// advertise registry as is
    					conetTE.storeAndAdvertise(regNID, 0, OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + registry.getFractal() + ".xml");
    				}
    				else
    				{
    					// take the logN part of the registry, store it temporarily locally
    					// and the advertise it
    					tempRegLocation = OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + "reduced_reg.xml";
    					RegistryType tmpRegistry = new RegistryType();
    					tmpRegistry.setFractal(registry.getFractal());
    					// select logN peers uniformly at random
    					List<Integer> selectedTargets = new ArrayList<Integer>();
    					List<String> finalPeerList = registry.getPeerIds();

    					double nofTargets = Math.log((double)finalPeerList.size());
    					int standardNofTargets = Double.valueOf(nofTargets).intValue();
    					double probNofTargets = nofTargets - standardNofTargets;

    					// choose whether another peer should be added to the number of targets
    					Random random = new Random();
    					double prob = random.nextDouble();
    					if (probNofTargets < prob)
    					{
    						standardNofTargets ++;
    					}

    					int nofSelectedTargets = 0;
    					Integer targetPeerIndex;
    					
    					// push the last peers in the registry since they are the last ones
    					// that have communicated
    					int lastRegPeerIndex = registry.getPeerIds().size() - 1;
    					for (int j = lastRegPeerIndex; j > lastRegPeerIndex - 4; j --)
    					{
    						tmpRegistry.getPeerIds().add(registry.getPeerIds().get(j));
    						tmpRegistry.getOverlaySAPs().add(registry.getOverlaySAPs().get(j));
    						tmpRegistry.getLeaveDates().add(registry.getLeaveDates().get(j));
    					}
    					
    					if (!tmpRegistry.getOverlaySAPs().contains(OverlayManager.getInstance().getOverlaySAP()))
    					{
    						tmpRegistry.getPeerIds().add(OverlayManager.getInstance().getPeerId());
    						tmpRegistry.getOverlaySAPs().add(OverlayManager.getInstance().getOverlaySAP());
    						
    						DatatypeFactory df1 = null;
							try {
								df1 = DatatypeFactory.newInstance();
							
	                			GregorianCalendar gc1 = new GregorianCalendar();
	                			gc1.setTime(OverlayManager.getInstance().getLatestLeaveDate());
	    						
	    						tmpRegistry.getLeaveDates().add(df1.newXMLGregorianCalendar(gc1));
							} catch (DatatypeConfigurationException e) {
								e.printStackTrace();
							}
    					}

//    					while (nofSelectedTargets < standardNofTargets)
//    					{
//    						do
//    						{
//    							targetPeerIndex = new Integer(random.nextInt(finalPeerList.size()));
//    						} while (selectedTargets.contains(targetPeerIndex));
//
//    						selectedTargets.add(targetPeerIndex);
//    						nofSelectedTargets ++;
//    					}

//    					for (Integer index : selectedTargets)
//    					{
//    						tmpRegistry.getPeerIds().add(registry.getPeerIds().get(index.intValue()));
//    						tmpRegistry.getOverlaySAPs().add(registry.getOverlaySAPs().get(index.intValue()));
//    						tmpRegistry.getLeaveDates().add(registry.getLeaveDates().get(index.intValue()));
//    					}

    					// save the new registry to the file
    					JAXBContext jc;
    					File f = new File(tempRegLocation);
    					try {
    						jc = JAXBContext.newInstance(new Class[]{eu.convergence.middleware.overlayTE.schema.registry.ObjectFactory.class});
    						Marshaller m = jc.createMarshaller();
    						m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
    						m.marshal(new ObjectFactory().createRegistry(registry), f);
    					} catch (JAXBException e) {
    						e.printStackTrace();
    					}

    					// advertise it in conet
    					RegistryCreator registryCreator = overlayTE.getRegistryCreator();
    					registryCreator.advertiseRegistry(tempRegLocation);

    					f.delete();
    				}

    			} catch (OverlayEngineException e) {
    				e.printStackTrace();
    			} catch (ConetIOException e) {
    				e.printStackTrace();
    			} catch (MXMConfigurationException e) {
    				e.printStackTrace();
    			} catch (MXMEngineNotFoundException e) {
    				e.printStackTrace();
    			} catch (MXMEngineException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
            }
		}
    }
}
