package eu.convergence.middleware.eventreportTE.test;

import eu.convergence.middleware.engine.ConetTe.DummyConeTE;
import eu.convergence.middleware.event.VDIMatchEvent;
import eu.convergence.middleware.eventreportTE.kernel.conet.EventReportTE;
import eu.convergence.middleware.eventreportTE.message.ERRepositoryType;
import eu.convergence.middleware.orchestratorTE.builder.MockReceiverOrchestrator;
import eu.convergence.middleware.orchestratorTE.handle.EREngineOrchestrator;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERRKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EventTriggerType;
import org.junit.Assert;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.io.File;

/**
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 8/31/11
 * Time: 5:17 PM
 * <p/>
 * UTI Systems, TIC
 */
@Test
public class EREngineOrchestratorTest {
    private static final String S_VDI_ID = "dummyIDForSubscriptionVdiID";
    private static final String P_VDI_ID = "dummyIDForPublicationVdiID";


    @BeforeTest
    public void setUp() {
        //Constants.USE_IN_MEMORY_MAPPING = true;
    }

    @DataProvider(name = "erOrchestrator")
    public Object[][] createEROrchestrator() throws Exception {
        return new Object[][]{
                {getEREngineOrchestrator(), getReceiveOrchestrator()}
        };
    }

    public void testScenarioForER() throws Exception {
        EREngineOrchestrator erOrchestrator = getEREngineOrchestrator();
        MockReceiverOrchestrator rcvOrchestrator = getReceiveOrchestrator();
        // A. creates an ERR calling the createERR of the EREngineOrchestrator.
        ERR err = erOrchestrator.getEREngine().getEventReportEngineSchemaHandler().getERR();
        err.setSAP(erOrchestrator.getEREngine().getEventReportEngineTechHandler().getEREngineParams().getSAP());
        err.setOperation(EventTriggerType.MATCH.getOperation());

        // B. stores the ERR in the peer calling the storeERR of the EREngineOrchestrator.
        ERRKeeper errKeeperSVDI = erOrchestrator.getEREngine().getEventReportEngineTechHandler().getEventReportRequestKeeper(ERRepositoryType.SVDI.getName());
        errKeeperSVDI.storeERR(err, S_VDI_ID);

        // ... waiting for a match to occur


        // register the ERArrivalEventCallback to catch when the ERArrivalEvent occur
        ERArrivalEventCallbackImpl callback = new ERArrivalEventCallbackImpl();
        rcvOrchestrator.registerCallback(callback);

        // C. creates and fires an MXMMatchEvent which goes to the EREngineOrchestrator.
        doEmulateMatch(erOrchestrator, S_VDI_ID, P_VDI_ID);

        // wait for ReceiverOrchestrator to process the ERArrivalEvent
        int repetitionCount = 0;
        while (++repetitionCount < 20) {
            Thread.sleep(500);

            if (callback.gotPublicationVdiId) {
                break;
            }
        }

        Assert.assertTrue(callback.gotPublicationVdiId);
        System.out.println("***** The Publication Vdi ID value in test is: " + callback.publicationVdiId);
    }

    private void doEmulateMatch(EREngineOrchestrator erOrchestrator, String subscriptionVdiId, String publicationVdiId) throws Exception {
        ERR err = erOrchestrator.getEREngine().getEventReportEngineSchemaHandler().getERR();

        // the SAP of the publisher
        err.setSAP(erOrchestrator.getEREngine().getEventReportEngineTechHandler().getEREngineParams().getSAP());
        err.setOperation(EventTriggerType.MATCH.getOperation());

        // B. stores the ERR in the peer calling the storeERR of the EREngineOrchestrator.
        ERRKeeper errKeeperPVDI = erOrchestrator.getEREngine().getEventReportEngineTechHandler().getEventReportRequestKeeper(ERRepositoryType.PVDI.getName());
        errKeeperPVDI.storeERR(err, publicationVdiId);

        MXM.getInstance().getMXMEventHandler().fireMXMEvent(
                new VDIMatchEvent(subscriptionVdiId, publicationVdiId, this)
        );
    }

    private static EREngineOrchestrator getEREngineOrchestrator() throws Exception {
        EventReportTE erTE = getEventReportTE();
        DummyConeTE dummyConet = (DummyConeTE) DummyConeTE.getInstance();
        EREngineOrchestrator erOrchestrator = EREngineOrchestrator.getInstance(erTE, dummyConet, dummyConet);
        erOrchestrator.bootEREngine();

        return erOrchestrator;
    }

    private static EventReportTE getEventReportTE() throws Exception {
        File f = new File(EREngineOrchestratorTest.class.getClassLoader().getResource("MXMConfiguration_EventReport.xml").getFile());
        MXM mxm = MXM.getInstance(f);
        mxm.listEnginesAndProperties();

        return (EventReportTE) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());
    }

    private static class ERArrivalEventCallbackImpl implements MockReceiverOrchestrator.ERArrivalEventCallback {
        String publicationVdiId;
        boolean gotPublicationVdiId;

        @Override
        public void notify(String publicationVdiId) {
            gotPublicationVdiId = true;
            this.publicationVdiId = publicationVdiId;
        }
    }

    private static MockReceiverOrchestrator getReceiveOrchestrator() throws Exception {
        return MockReceiverOrchestrator.getInstance();
    }
}
