package eu.convergence.middleware.orchestratorTE.builder;

import eu.convergence.middleware.event.ERArrivalEvent;
import eu.convergence.middleware.orchestratorTE.handle.ReceiverOrchestrator;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.event.management.MXMEvent;
import org.iso.mpeg.mxm.core.event.management.MXMEventListener;

/**
 * A Mock Implementation of the ReceiverOrchestrator that must be notified by the ERROrchestrator that a match publication
 * VDI has been found and the PVDI identifier must be given to it,
 * so that the ReceiverOrchestrator can fetch the resource VDI associated with the PVDI.
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 8/31/11
 * Time: 4:11 PM
 * <p/>
 * UTI Systems, TIC
 */
public class MockReceiverOrchestrator implements MXMEventListener {
    private static MockReceiverOrchestrator RECEIVER_ORCHESTRATOR = null;
  	private Logger logger = Logger.getLogger(ReceiverOrchestrator.class);

    /**
     * Implemented just for the tests
     */
    private ERArrivalEventCallback callback;

    private MockReceiverOrchestrator() {
        try {
            // register for the Match event
            MXM.getInstance().getMXMEventHandler().registerMXMEventListener(ERArrivalEvent.class.getName(), this);
        } catch (MXMConfigurationException e) {
            e.printStackTrace();
        }
    }

    public static MockReceiverOrchestrator getInstance() {
        if (null == RECEIVER_ORCHESTRATOR) {
            RECEIVER_ORCHESTRATOR = new MockReceiverOrchestrator();
        }

        return RECEIVER_ORCHESTRATOR;
    }

    @Override
    public void mxmEventOccurred(MXMEvent event) {
        // ReceiverOrchestrator handles the MXMERArrivalEvent
        if (event instanceof ERArrivalEvent) {
            ERArrivalEvent evt = (ERArrivalEvent) event;
            String publicationVdiId = evt.getPublicationVdiId();

            System.out.println("MXMERArrivalEvent occured: a match publication VDI has been found and the P-VDI is " + publicationVdiId);

            if (null != callback) {
                callback.notify(publicationVdiId);
            }
        }
    }

    public void registerCallback(ERArrivalEventCallback callback) {
        this.callback = callback;
    }

    public static interface ERArrivalEventCallback {
        void notify(String publicationVdiId);
    }
}
