package eu.convergence.middleware.overlayTE;

import eu.convergence.middleware.conetTe.ConetTE;
import eu.convergence.middleware.overlayTE.schema.peerProperties.ObjectFactory;
import eu.convergence.middleware.overlayTE.schema.peerProperties.PeerProperties;

import javax.xml.bind.*;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.File;
import java.util.*;

public class OverlayManager 
{
	private static OverlayManager OVERLAY_MANAGER = null;
	private String filePath;
	
	private String peerId;
	private String overlaySAP;
	private TreeSet<Date> leaveDates = null;

    private ConetTE sender;
    private ConetTE advertiser;

    public ConetTE getSender()
    {
        return sender;
    }

    public void setSender(ConetTE sender)
    {
        this.sender = sender;
    }

    public ConetTE getAdvertiser()
    {
        return advertiser;
    }

    public void setAdvertiser(ConetTE advertiser)
    {
        this.advertiser = advertiser;
    }

    protected OverlayManager(String filePath)
	{
		this.filePath = filePath;
		readSystemProperties();
	}
	
	protected OverlayManager(String filePath, String peerId, String overlaySAP)
	{
		this.filePath = filePath;
		this.peerId = peerId;
		this.overlaySAP = overlaySAP;
        leaveDates = new TreeSet<Date>();
		updatePropertiesFile();
	}
	
	public static OverlayManager getInstance(String filePath) 
	{
		if (OVERLAY_MANAGER == null)
		{
			OVERLAY_MANAGER = new OverlayManager(filePath);
		}
		return OVERLAY_MANAGER;
	}
	
	public static OverlayManager getInstance(String filePath, String peerId, String overlaySAP)
	{
		if (OVERLAY_MANAGER == null)
		{
			OVERLAY_MANAGER = new OverlayManager(filePath, peerId, overlaySAP);
		}
		return OVERLAY_MANAGER;
	}
	
	public static OverlayManager getInstance()
	{
		return OVERLAY_MANAGER;
	}
	
	private void readSystemProperties()
	{
		File propertiesFile = new File(filePath);
		if (propertiesFile.exists())
		{
			JAXBContext jc;
			try {
				jc = JAXBContext.newInstance(new Class[]{eu.convergence.middleware.overlayTE.schema.peerProperties.ObjectFactory.class});
				Unmarshaller u = jc.createUnmarshaller();
				Object obj = u.unmarshal(propertiesFile);
				@SuppressWarnings("unchecked")
				JAXBElement<PeerProperties> element = (JAXBElement<PeerProperties>)obj;
				PeerProperties peerProperties = element.getValue();
				
				peerId = peerProperties.getPeerId();
				overlaySAP = peerProperties.getOverlaySAP();
				
				if ((peerProperties.getLeaveDate() == null) || (peerProperties.getLeaveDate().isEmpty()))
				{
					
				}
				else
				{
					for (XMLGregorianCalendar xmlLeaveDate : peerProperties.getLeaveDate())
					{
						leaveDates.add(xmlLeaveDate.toGregorianCalendar().getTime());
					}
				}
				
			} catch (JAXBException e) {
				e.printStackTrace();
			}
		}
		else
		{
			System.err.println("Peer properties not found");
		}
	}
	
	private void updatePropertiesFile()
	{
		PeerProperties peerProperties = new PeerProperties();
		peerProperties.setPeerId(peerId);
		peerProperties.setOverlaySAP(overlaySAP);
		List<XMLGregorianCalendar> xmlLeaveDates = new ArrayList<XMLGregorianCalendar>();

		for (Date ld : leaveDates)
		{
			try {
				DatatypeFactory df = DatatypeFactory.newInstance();
				GregorianCalendar gc = new GregorianCalendar();
				gc.setTime(ld);
				xmlLeaveDates.add(df.newXMLGregorianCalendar(gc));
			} catch (DatatypeConfigurationException e) {
				e.printStackTrace();
			}
		}
		
		try {
			JAXBContext jc = JAXBContext.newInstance(new Class[]{eu.convergence.middleware.overlayTE.schema.peerProperties.ObjectFactory.class});
			Marshaller m = jc.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			m.marshal(new ObjectFactory().createProperties(peerProperties), new File(filePath));
		} catch (JAXBException e) {
			e.printStackTrace();
		}
		
	}

	public String getPeerId() {
		return peerId;
	}

	public String getOverlaySAP() {
		return overlaySAP;
	}

	public TreeSet<Date> getLeaveDates() {
		return leaveDates;
	}	
	
	public Date getLatestLeaveDate()
	{
		if (leaveDates != null && !leaveDates.isEmpty())
			return leaveDates.last();
		else{
			// if we leave empty, we raise an exception into the overlaypropagationMessageConsumer registration_message_response
			if(leaveDates.isEmpty()){
				leaveDates.add(new Date((System.currentTimeMillis() / 1000L) + 3600));
				return leaveDates.last();
			}
			else
				return null;
		}
	}

	
	public void addLeaveDate(Date anotherLeaveDate)
	{
        if (leaveDates == null)
        {
            leaveDates = new TreeSet<Date>();
        }
		leaveDates.add(anotherLeaveDate);
		updatePropertiesFile();
	}
	
	public void removeLeaveDate(Date toRemove)
	{
		leaveDates.remove(toRemove);
		updatePropertiesFile();
	}
}
