package eu.convergence.middleware.overlayTE.message;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;

import eu.convergence.middleware.engine.overlayTE.MessageKeeper;
import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.event.VDIArrivalEvent;
import eu.convergence.middleware.overlayTE.OverlayEnginePaths;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.GorestMessageType;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory;

import javax.xml.bind.*;
import javax.xml.transform.stream.StreamSource;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

public class OverlayMessageKeeper implements MessageKeeper 
{
	
	public OverlayMessageKeeper()
	{
	}
	
	@Override
	public MXMEngineResponse bufferMessage(MXMObject overlayPropagationMessage)
			throws OverlayEngineException {
		
		System.out.println("About to buffer message");
		GorestMessageType message = (GorestMessageType) overlayPropagationMessage.getContent();
		int ttl = message.getHeader().getTTL();
		message.getHeader().setTTL(ttl - 1);
		
		/*List<String> path = message.getHeader().getPath();
		path.add(currentPeerId);*/
		
		System.out.println("About to store message");
		storeMessage(message);
		
		// fire the event
		
		GorestMessageType gorestMessage = (GorestMessageType) overlayPropagationMessage.getContent();
		List<String> fractalsList = gorestMessage.getHeader().getFractals();
        String[] fractalNames = fractalsList.toArray(new String[fractalsList.size()]);
        int opcode = gorestMessage.getHeader().getOpcode();
        
        VDIArrivalEvent vdiArrivalEvent = null;
        if ((opcode == Opcodes.PUBLICATION_MESSAGE) || 
        		(opcode == Opcodes.SUBSCRIPTION_MESSAGE) ||
        		(opcode == Opcodes.REVOCATION_MESSAGE))
		{
            // fire the VDIArrivalEvent
            if (opcode == Opcodes.PUBLICATION_MESSAGE)
            {
                vdiArrivalEvent = new VDIArrivalEvent(this, new MXMAdapter(gorestMessage.getPayload().getPublicationVdi()), VDIArrivalEvent.PUBLICATION_VDI, fractalNames);
            }
            else if (opcode == Opcodes.SUBSCRIPTION_MESSAGE)
            {
                vdiArrivalEvent = new VDIArrivalEvent(this, new MXMAdapter(gorestMessage.getPayload().getSubscriptionVdi()), VDIArrivalEvent.SUBSCRIPTION_VDI, fractalNames);
            }
            else if (opcode == Opcodes.REVOCATION_MESSAGE)
            {
            	vdiArrivalEvent = new VDIArrivalEvent(this, new MXMAdapter(gorestMessage.getPayload()), VDIArrivalEvent.REVOCATION_VDI, fractalNames);
            }
            try {
                MXM.getInstance().getMXMEventHandler().fireMXMEvent(vdiArrivalEvent);
            } catch (MXMConfigurationException e) {
                System.err.println("Could not fire the MXMArrivalEvent");
                e.printStackTrace();
            }
        }
        
		return null;
	}

	public List<MXMObject> getBufferedPublicationMessages() throws OverlayEngineException {
		List<MXMObject> bufferedMessages = new ArrayList<MXMObject>();
		// retrieve all files from the directory
		File publicationDirectory = new File(OverlayEnginePaths.getInstance().getPUBLICATION_MESSAGES_LOCATION());
		File[] pubMessageFiles = publicationDirectory.listFiles();
		
		if (pubMessageFiles != null)
		{
			for (File file : pubMessageFiles)
			{
				if (file.isDirectory())	{continue;}
				if (file.getName().contains(".vdi"))
				{
					try {
						JAXBContext jc = JAXBContext.newInstance(new Class[] {eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory.class});
						Unmarshaller u = jc.createUnmarshaller();				
						JAXBElement<GorestMessageType> gorestMessageElement = u.unmarshal(new StreamSource(file), GorestMessageType.class);
						GorestMessageType gorestMessage = gorestMessageElement.getValue();
						bufferedMessages.add(new MXMAdapter(gorestMessage));
					} catch (JAXBException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		return bufferedMessages;
	}
	
	public List<MXMObject> getBufferedReadyPublicationMessages() throws OverlayEngineException {
		List<MXMObject> bufferedMessages = new ArrayList<MXMObject>();
		// retrieve all files from the directory
		File publicationDirectory = new File(OverlayEnginePaths.getInstance().getREADY_PUBLICATION_MESSAGES_LOCATION());
		File[] pubMessageFiles = publicationDirectory.listFiles();
		
		if (pubMessageFiles != null)
		{
			for (File file : pubMessageFiles)
			{
				if (file.isDirectory())	{continue;}
				if (file.getName().contains(".vdi"))
				{
					try {
						JAXBContext jc = JAXBContext.newInstance(new Class[] {eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory.class});
						Unmarshaller u = jc.createUnmarshaller();				
						JAXBElement<GorestMessageType> gorestMessageElement = u.unmarshal(new StreamSource(file), GorestMessageType.class);
						GorestMessageType gorestMessage = gorestMessageElement.getValue();
						bufferedMessages.add(new MXMAdapter(gorestMessage));
					} catch (JAXBException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		return bufferedMessages;
	}

    @Override
    public void removeReadyPublicationMessages() throws OverlayEngineException
    {
        File publicationDirectory = new File(OverlayEnginePaths.getInstance().getREADY_PUBLICATION_MESSAGES_LOCATION());
		File[] pubMessageFiles = publicationDirectory.listFiles();

		if (pubMessageFiles != null)
		{
			for (File file : pubMessageFiles)
			{
				if (file.isDirectory())	{continue;}
	            file.delete();
	        }
		}
		
    }

    public List<MXMObject> getBufferedSubscriptionMessages()
			throws OverlayEngineException {
		List<MXMObject> bufferedMessages = new ArrayList<MXMObject>();
		// retrieve all files from the directory
		File subscriptionDirectory = new File(OverlayEnginePaths.getInstance().getSUBSCRIPTION_MESSAGES_LOCATION());
		File[] subMessageFiles = subscriptionDirectory.listFiles();
		
		if (subMessageFiles != null)
		{
			for (File file : subMessageFiles)
			{
				if (file.isDirectory())	{continue;}
				if (file.getName().contains(".vdi"))
				{
					try {
						JAXBContext jc = JAXBContext.newInstance(new Class[] {eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory.class});
						Unmarshaller u = jc.createUnmarshaller();				
						JAXBElement<GorestMessageType> gorestMessageElement = u.unmarshal(new StreamSource(file), GorestMessageType.class);
						GorestMessageType gorestMessage = gorestMessageElement.getValue();
						bufferedMessages.add(new MXMAdapter(gorestMessage));
					} catch (JAXBException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		return bufferedMessages;
	}
	
	public List<MXMObject> getBufferedReadySubscriptionMessages() throws OverlayEngineException {
		List<MXMObject> bufferedMessages = new ArrayList<MXMObject>();
		// retrieve all files from the directory
		File subscriptionDirectory = new File(OverlayEnginePaths.getInstance().getREADY_SUBSCRIPTION_MESSAGES_LOCATION());
		File[] subMessageFiles = subscriptionDirectory.listFiles();
		
		if (subMessageFiles != null)
		{
			for (File file : subMessageFiles)
			{
				if (file.isDirectory())	{continue;}
				if (file.getName().contains(".vdi"))
				{
					try {
						JAXBContext jc = JAXBContext.newInstance(new Class[] {eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory.class});
						Unmarshaller u = jc.createUnmarshaller();				
						JAXBElement<GorestMessageType> gorestMessageElement = u.unmarshal(new StreamSource(file), GorestMessageType.class);
						GorestMessageType gorestMessage = gorestMessageElement.getValue();
						bufferedMessages.add(new MXMAdapter(gorestMessage));
					} catch (JAXBException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		return bufferedMessages;
	}

    @Override
    public void removeReadySubscriptionMessages() throws OverlayEngineException
    {
        File subscriptionDirectory = new File(OverlayEnginePaths.getInstance().getREADY_SUBSCRIPTION_MESSAGES_LOCATION());
        File[] subMessageFiles = subscriptionDirectory.listFiles();

        if (subMessageFiles != null)
        {
        	for (File file : subMessageFiles)
            {
                if (file.isDirectory())	{continue;}
                file.delete();
            }
        }
        
    }

    public List<MXMObject> getBufferedRegistrationMessages() throws OverlayEngineException
	{
		List<MXMObject> bufferedMessages = new ArrayList<MXMObject>();
		// retrieve all files from the directory
		File registryDirectory = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION());
		File[] registryMessageFiles = registryDirectory.listFiles();
		
		if (registryMessageFiles != null)
		{
			for (File file : registryMessageFiles)
			{
				if (file.isDirectory())	{continue;}
				if (file.getName().contains(".xml"))
				{
					try {
						JAXBContext jc = JAXBContext.newInstance(new Class[] {eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory.class});
						Unmarshaller u = jc.createUnmarshaller();				
						JAXBElement<GorestMessageType> gorestMessageElement = u.unmarshal(new StreamSource(file), GorestMessageType.class);
						GorestMessageType gorestMessage = gorestMessageElement.getValue();
						bufferedMessages.add(new MXMAdapter(gorestMessage));
					} catch (JAXBException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		return bufferedMessages;

	}

    @Override
    public void removeReadyRegistrationMessages() throws OverlayEngineException
    {
        File registryDirectory = new File(OverlayEnginePaths.getInstance().getREADY_REGISTRY_LOCATION());
		File[] registryMessageFiles = registryDirectory.listFiles();

		if (registryMessageFiles != null)
		{
			for (File file : registryMessageFiles)
			{
				if (file.isDirectory())	{continue;}
	            file.delete();
	        }
		}
		
    }

    public List<MXMObject> getBufferedReadyRegistrationMessages() throws OverlayEngineException
	{
		List<MXMObject> bufferedMessages = new ArrayList<MXMObject>();
		// retrieve all files from the directory
		File registryDirectory = new File(OverlayEnginePaths.getInstance().getREADY_REGISTRY_LOCATION());
		File[] registryMessageFiles = registryDirectory.listFiles();
		
		if (registryMessageFiles != null)
		{
			for (File file : registryMessageFiles)
			{
				if (file.isDirectory())	{continue;}
				if (file.getName().contains(".xml"))
				{
					try {
						JAXBContext jc = JAXBContext.newInstance(new Class[] {eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory.class});
						Unmarshaller u = jc.createUnmarshaller();				
						JAXBElement<GorestMessageType> gorestMessageElement = u.unmarshal(new StreamSource(file), GorestMessageType.class);
						GorestMessageType gorestMessage = gorestMessageElement.getValue();
						bufferedMessages.add(new MXMAdapter(gorestMessage));
					} catch (JAXBException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		return bufferedMessages;

	}
    
    public List<MXMObject> getBufferedRevocationMessages() throws OverlayEngineException {
		List<MXMObject> bufferedMessages = new ArrayList<MXMObject>();
		// retrieve all files from the directory
		File revocationDirectory = new File(OverlayEnginePaths.getInstance().getREVOCATION_MESSAGES_LOCATION());
		File[] revMessageFiles = revocationDirectory.listFiles();
		
		if (revMessageFiles != null)
		{
			for (File file : revMessageFiles)
			{
				if (file.isDirectory())	{continue;}
				if (file.getName().contains(".vdi"))
				{
					try {
						JAXBContext jc = JAXBContext.newInstance(new Class[] {eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory.class});
						Unmarshaller u = jc.createUnmarshaller();				
						JAXBElement<GorestMessageType> gorestMessageElement = u.unmarshal(new StreamSource(file), GorestMessageType.class);
						GorestMessageType gorestMessage = gorestMessageElement.getValue();
						bufferedMessages.add(new MXMAdapter(gorestMessage));
					} catch (JAXBException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		return bufferedMessages;
	}
	
	public List<MXMObject> getBufferedReadyRevocationMessages() throws OverlayEngineException {
		List<MXMObject> bufferedMessages = new ArrayList<MXMObject>();
		// retrieve all files from the directory
		File revocationDirectory = new File(OverlayEnginePaths.getInstance().getREADY_REVOCATION_MESSAGES_LOCATION());
		File[] revMessageFiles = revocationDirectory.listFiles();
		
		if (revMessageFiles != null)
		{
			for (File file : revMessageFiles)
			{
				if (file.isDirectory())	{continue;}
				if (file.getName().contains(".vdi"))
				{
					try {
						JAXBContext jc = JAXBContext.newInstance(new Class[] {eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory.class});
						Unmarshaller u = jc.createUnmarshaller();				
						JAXBElement<GorestMessageType> gorestMessageElement = u.unmarshal(new StreamSource(file), GorestMessageType.class);
						GorestMessageType gorestMessage = gorestMessageElement.getValue();
						bufferedMessages.add(new MXMAdapter(gorestMessage));
					} catch (JAXBException e) {
						e.printStackTrace();
					}
				}
			}
		}
		
		return bufferedMessages;
	}

    public void removeReadyRevocationMessages() throws OverlayEngineException
    {
        File revocationDirectory = new File(OverlayEnginePaths.getInstance().getREADY_REVOCATION_MESSAGES_LOCATION());
		File[] revMessageFiles = revocationDirectory.listFiles();

		if (revMessageFiles != null)
		{
			for (File file : revMessageFiles)
			{
				if (file.isDirectory())	{continue;}
	            file.delete();
	        }
		}
		
    }

	@Override
	public MXMEngineResponse unBufferMessage(MXMObject overlayPropagationMessage, String id)
			throws OverlayEngineException
    {
	    GorestMessageType gorestMessage = (GorestMessageType)overlayPropagationMessage.getContent();
        OverlayEnginePaths overlayEnginePaths = OverlayEnginePaths.getInstance();
        File file = null;

        switch (gorestMessage.getHeader().getOpcode())
        {
            case Opcodes.PUBLICATION_MESSAGE:
                file = new File(overlayEnginePaths.getPUBLICATION_MESSAGES_LOCATION() + id);
                file.delete();
                file = new File(overlayEnginePaths.getREADY_PUBLICATION_MESSAGES_LOCATION() + id);
                if ((file != null) && (file.exists()))  {file.delete();}
            case Opcodes.SUBSCRIPTION_MESSAGE:
                file = new File(overlayEnginePaths.getSUBSCRIPTION_MESSAGES_LOCATION() + id);
                if ((file != null) && (file.exists()))  {file.delete();}
                file = new File(overlayEnginePaths.getREADY_SUBSCRIPTION_MESSAGES_LOCATION() + id);
                if ((file != null) && (file.exists()))  {file.delete();}
        }

		return MXMEngineResponse.METHOD_NOT_IMPLEMENTED;
	}
	
	private void storeMessage(GorestMessageType message)
	{
		System.out.println("Storing message");
		try {
			Random random = new Random(new Date().getTime());
			String fileName = new String(random.nextInt() + ".vdi");
			File f = null;
			File fready = null;
					
			if (message.getHeader().getOpcode() == Opcodes.REGISTRATION_MESSAGE)
			{
				f = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + message.getHeader().getFractals().get(0) + ".xml");
				if (message.getHeader().getTTL() > 0)
				{
					fready = new File(OverlayEnginePaths.getInstance().getREADY_REGISTRY_LOCATION() + message.getHeader().getFractals().get(0) + ".xml");
				}
			}
			else if (message.getHeader().getOpcode() == Opcodes.SUBSCRIPTION_MESSAGE)
			{
				f = new File(OverlayEnginePaths.getInstance().getSUBSCRIPTION_MESSAGES_LOCATION() + fileName);
				if (message.getHeader().getTTL() > 0)
				{
					fready = new File(OverlayEnginePaths.getInstance().getREADY_SUBSCRIPTION_MESSAGES_LOCATION() + fileName);
				}
			}
			else if (message.getHeader().getOpcode() == Opcodes.PUBLICATION_MESSAGE)
			{
				f = new File(OverlayEnginePaths.getInstance().getPUBLICATION_MESSAGES_LOCATION() + fileName);
				if (message.getHeader().getTTL() > 0)
				{
					fready = new File(OverlayEnginePaths.getInstance().getREADY_PUBLICATION_MESSAGES_LOCATION() + fileName);
				}
			} 
			else if (message.getHeader().getOpcode() == Opcodes.REVOCATION_MESSAGE)
			{
				f = new File(OverlayEnginePaths.getInstance().getREVOCATION_MESSAGES_LOCATION() + fileName);
				if (message.getHeader().getTTL() > 0)
				{
					fready = new File(OverlayEnginePaths.getInstance().getREADY_REVOCATION_MESSAGES_LOCATION() + fileName);
				}
			}

			System.out.println("Marshalling...");
			JAXBContext jc = JAXBContext.newInstance(new Class[] {eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory.class});
			Marshaller m = jc.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			m.marshal(new ObjectFactory().createGorestMessage(message), f);
			if (fready != null)
			{
				m.marshal(message, fready);
			}
			System.out.println("Stored...");
			
			
		} catch (JAXBException e) {
			e.printStackTrace();
		}
	}
}
