package eu.convergence.middleware.overlayTE.message.tcpip;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.event.VDIArrivalEvent;
import eu.convergence.middleware.overlayTE.OverlayManager;
import eu.convergence.middleware.overlayTE.kernel.tcpip.OverlayTE;
import eu.convergence.middleware.overlayTE.message.Opcodes;
import eu.convergence.middleware.overlayTE.message.OverlayMessageCreator;
import eu.convergence.middleware.overlayTE.message.OverlayMessageKeeper;
import eu.convergence.middleware.overlayTE.registry.OverlayRegistryParser;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.GorestMessageType;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.HeaderType;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.PayloadType;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.RegistrationMessageType;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.transform.stream.StreamSource;
import java.io.IOException;
import java.io.InputStream;
import java.net.Socket;
import java.util.GregorianCalendar;
import java.util.List;

public class OverlayPropagationMessageConsumer implements Runnable 
{
	private Socket socket;
	private String currentPeerId;
	
	public OverlayPropagationMessageConsumer(Socket socket, String currentPeerId)
	{
		this.socket = socket;
		this.currentPeerId = currentPeerId;
	}

	@Override
	public void run() 
	{
		try {
			InputStream in = socket.getInputStream();
			JAXBContext jc = JAXBContext.newInstance(new Class[] {eu.convergence.middleware.overlayTE.schema.gorestMessage.ObjectFactory.class});
			Unmarshaller u = jc.createUnmarshaller();
			
			JAXBElement<GorestMessageType> jaxbElement = u.unmarshal(new StreamSource(in), GorestMessageType.class);
			GorestMessageType gorestMessage = jaxbElement.getValue();
			
			int opcode = gorestMessage.getHeader().getOpcode();
            List<String> fractalsList = gorestMessage.getHeader().getFractals();
            String[] fractalNames = fractalsList.toArray(new String[fractalsList.size()]);

            System.out.println("Received message " + opcode + " from Peer: " + socket.getRemoteSocketAddress().toString());
			
			if ((opcode == Opcodes.PUBLICATION_MESSAGE) || (opcode == Opcodes.SUBSCRIPTION_MESSAGE))
			{
				OverlayMessageKeeper overlayMessageKeeper = new OverlayMessageKeeper();
				overlayMessageKeeper.bufferMessage(new MXMAdapter(gorestMessage));

                // fire the VDIArrivalEvent
//                VDIArrivalEvent vdiArrivalEvent = null;
//                if (opcode == Opcodes.PUBLICATION_MESSAGE)
//                {
//                    vdiArrivalEvent = new VDIArrivalEvent(this, new MXMAdapter(gorestMessage.getPayload().getPublicationVdi()), VDIArrivalEvent.PUBLICATION_VDI, fractalNames);
//                }
//                else
//                {
//                    vdiArrivalEvent = new VDIArrivalEvent(this, new MXMAdapter(gorestMessage.getPayload().getSubscriptionVdi()), VDIArrivalEvent.SUBSCRIPTION_VDI, fractalNames);
//                }
//
//                try {
//                    MXM.getInstance().getMXMEventHandler().fireMXMEvent(vdiArrivalEvent);
//                } catch (MXMConfigurationException e) {
//                    System.err.println("Could not fire the MXMArrivalEvent");
//                    e.printStackTrace();
//                }
			}
			else if (opcode == Opcodes.REGISTRATION_MESSAGE)
			{
                System.out.println("Received REGISTRATION message...");
                // trigger the event
//                VDIArrivalEvent vdiArrivalEvent = new VDIArrivalEvent(this, new MXMAdapter(gorestMessage.getPayload().getDiscoveryMessage()), VDIArrivalEvent.REGISTRATION_MSG, fractalNames);
//                try {
//                    MXM.getInstance().getMXMEventHandler().fireMXMEvent(vdiArrivalEvent);
//                } catch (MXMConfigurationException e) {
//                    e.printStackTrace();
//                }
				String fractalId = gorestMessage.getHeader().getFractals().get(0);
				OverlayRegistryParser overlayRegistryParser = new OverlayRegistryParser(true, fractalId);
				RegistrationMessageType registrationMessageType = gorestMessage.getPayload().getDiscoveryMessage();
				System.out.println("Inserting peer to registry...");
				overlayRegistryParser.insertPeerToRegistry(registrationMessageType.getPeerId(), 
						registrationMessageType.getOverlaySAP(),
						registrationMessageType.getLeaveDate().toGregorianCalendar().getTime());
                System.out.println("Storing registry...");
				overlayRegistryParser.storeRegistry();
				
				// create the response with the part of the registry to be stored in the remote peer
				HeaderType header = new HeaderType();
				header.getFractals().add(fractalId);
				header.setOpcode(Opcodes.REGISTRATION_MESSAGE_RESPONSE);
				header.getPath().add(currentPeerId);
				header.setTTL(1);
				
				PayloadType payload = new PayloadType();
				RegistrationMessageType registrationResponseMessage = new RegistrationMessageType();
				registrationResponseMessage.setPeerId(currentPeerId);
				registrationResponseMessage.setOverlaySAP(OverlayManager.getInstance().getOverlaySAP());
				
				try {
					DatatypeFactory df = DatatypeFactory.newInstance();
					GregorianCalendar gc = new GregorianCalendar();
					gc.setTime(OverlayManager.getInstance().getLatestLeaveDate());
					registrationMessageType.setLeaveDate(df.newXMLGregorianCalendar(gc));
				} catch (DatatypeConfigurationException e) {
					e.printStackTrace();
				}
				
				payload.setDiscoveryMessage(registrationResponseMessage);
				
				OverlayTE overlayTE = new OverlayTE();
				
				GorestMessageType responseMessage = new GorestMessageType();
				OverlayMessageCreator overlayMessageCreator = overlayTE.getMessageCreator();
				responseMessage = (GorestMessageType)overlayMessageCreator.createMessage(new MXMAdapter(header), new MXMAdapter(payload));
				OverlayPropagationMessageHandler overlayPropagationMessageHandler = overlayTE.getPropagationMessageHandler();
				overlayPropagationMessageHandler.sendPropagationMessage(new MXMAdapter(responseMessage), registrationMessageType.getOverlaySAP());
				
				// store the registry in the ready directory
				overlayTE.getMessageKeeper().bufferMessage(new MXMAdapter(gorestMessage));
			}
			else if (opcode == Opcodes.REGISTRATION_MESSAGE_RESPONSE)
			{
				// insert the remote peer in the registry
				String fractalId = gorestMessage.getHeader().getFractals().get(0);
				OverlayRegistryParser overlayRegistryParser = new OverlayRegistryParser(true, fractalId);
				RegistrationMessageType registrationMessageType = gorestMessage.getPayload().getDiscoveryMessage();
				
				overlayRegistryParser.insertPeerToRegistry(registrationMessageType.getPeerId(), 
						registrationMessageType.getOverlaySAP(),
						registrationMessageType.getLeaveDate().toGregorianCalendar().getTime());
				overlayRegistryParser.storeRegistry();
			}
			
			socket.close();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (JAXBException e) {
			e.printStackTrace();
		} catch (OverlayEngineException e) {
			e.printStackTrace();
		}
	}

}
