package eu.convergence.middleware.overlayTE.registry;

import eu.convergence.middleware.core.ConvergenceEngineName;
import eu.convergence.middleware.event.PeerToFractalRegistrationEvent;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMObject;

import eu.convergence.middleware.engine.ConetTe.ConetTE;
import eu.convergence.middleware.engine.ConetTe.exception.ConetIOException;
import eu.convergence.middleware.engine.ConetTe.services.NID;
import eu.convergence.middleware.engine.overlayTE.RegistryParser;
import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.overlayTE.OverlayEnginePaths;
import eu.convergence.middleware.overlayTE.kernel.conet.OverlayTE;
import eu.convergence.middleware.overlayTE.schema.registry.RegistryType;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;

import javax.xml.bind.*;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.transform.stream.StreamSource;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.*;

public class OverlayRegistryParser implements RegistryParser 
{
	private RegistryType registry;
	
	public OverlayRegistryParser()
	{
		
	}
	
	public OverlayRegistryParser(RegistryType registry)
	{
		this.registry = registry;
	}
	
	public OverlayRegistryParser(boolean useFileSystemRegistry, String fractalId)
	{
		File file = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + fractalId + ".xml");
		if(!file.exists())
		{
			// if the peer doesn't have a local copy of the registry, it must get one from the CoNet
			try{
				MXM mxm = MXM.getInstance();
				ConetTE conetTE = (ConetTE) mxm.getDefaultEngine(ConvergenceEngineName.ConetTE.toString());
				NID nid = new NID(OverlayTE.registryFormat + fractalId);

				int found = conetTE.get(nid, OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + fractalId + ".xml");
				if (found != -1)
	                System.out.println("Registry has been taken from the CoNet...");
				else
					System.out.println("Not able to get the registry from the CoNet...");
			}catch (MXMConfigurationException e1) {
				e1.printStackTrace();
			} catch (MXMEngineNotFoundException e1) {
				e1.printStackTrace();
			}
			catch (ConetIOException e) {
				e.printStackTrace();
			}
		}
		try {
			this.parseFractalRegistry(file);
		} catch (OverlayEngineException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public MXMEngineResponse parseFractalRegistry(InputStream in) throws OverlayEngineException 
	{
		try {
			JAXBContext jc = JAXBContext.newInstance(new Class[]{eu.convergence.middleware.overlayTE.schema.registry.ObjectFactory.class});
			Unmarshaller u = jc.createUnmarshaller();
            JAXBElement<RegistryType> registryTypeElement = u.unmarshal(new StreamSource(in), RegistryType.class);
			this.registry = registryTypeElement.getValue();
		} catch (JAXBException e) {
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		return MXMEngineResponse.SUCCESS;
	}

	@Override
	public List<String> getOverlayServiceEndpoints() {
		List<String> SAPs = new ArrayList<String>(); 
		SAPs.addAll(registry.getOverlaySAPs());
		
		return SAPs;
	}

	@Override
	public String getPeerOverlayServiceEndpoint(String peerId) {
		String SAP = new String();
		int count = 0;
		
		for (String peer : registry.getPeerIds())
		{
			if (!peer.equals(peerId))
			{
				count ++;
			}
		}
		
		SAP = (String)registry.getOverlaySAPs().get(count);
				
		return SAP;
	}

	@Override
	public MXMEngineResponse insertPeerToRegistry(String peerId, String overlayServiceEndpoint, Date leaveDate)
			throws OverlayEngineException 
	{
        if (registry.getPeerIds().contains(peerId))
		{
            // check whether this is consistent with the current properties
            System.out.println("Removing peer " + peerId + " from registry");
            removePeerFromRegistry(peerId, overlayServiceEndpoint);
		}
        else
        {
            // just trigger the event that the peer registered to the fractal
            PeerToFractalRegistrationEvent peerToFractalRegistrationEvent =
                    new PeerToFractalRegistrationEvent(peerId, registry.getOntologyLocation(), registry.getFractal(), this);
            try {
                MXM.getInstance().getMXMEventHandler().fireMXMEvent(peerToFractalRegistrationEvent);
            } catch (MXMConfigurationException e) {
                e.printStackTrace();
            }
        }
		registry.getPeerIds().add(peerId);
		registry.getOverlaySAPs().add(overlayServiceEndpoint);

        System.out.println("Adding peer " + peerId + " with SAP " + overlayServiceEndpoint + " to registry");
		
		try {
			DatatypeFactory df = DatatypeFactory.newInstance();
			Calendar cal = Calendar.getInstance();
			cal.setTime(leaveDate);
			GregorianCalendar gregorianCalendar = new GregorianCalendar(
					cal.get(Calendar.YEAR), 
					cal.get(Calendar.MONTH), 
					cal.get(Calendar.DAY_OF_MONTH), 
					cal.get(Calendar.HOUR_OF_DAY), 
					cal.get(Calendar.MINUTE), 
					cal.get(Calendar.SECOND));
			gregorianCalendar.setTimeInMillis(leaveDate.getTime());
			XMLGregorianCalendar xmlLeaveDate = df.newXMLGregorianCalendar(gregorianCalendar);
			registry.getLeaveDates().add(xmlLeaveDate);
			// increase the number of members in the fractal
			Integer nofMembers = registry.getFractalMembers();
			if (nofMembers == null)
			{
				nofMembers = new Integer(0);
			}
			int updatedNofMembers = nofMembers.intValue() + 1;
			registry.setFractalMembers(new Integer(updatedNofMembers));
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		
		return MXMEngineResponse.SUCCESS;
	}

	@Override
	public MXMEngineResponse removePeerFromRegistry(String peerId, String overlayServiceEndpoint) throws OverlayEngineException 
	{
		int count = 0;
		if (peerId != null)
		{
            count = registry.getPeerIds().indexOf(peerId);
		}
		else if (overlayServiceEndpoint != null)
		{
			count = registry.getOverlaySAPs().indexOf(overlayServiceEndpoint);
		}
		
		registry.getLeaveDates().remove(count);
		registry.getOverlaySAPs().remove(count);
		registry.getPeerIds().remove(count);
		
		// decrease the number of members in the fractal
		Integer nofMembers = registry.getFractalMembers();
		int updatedNofMembers = nofMembers.intValue() - 1;
		registry.setFractalMembers(new Integer(updatedNofMembers));
		
		return MXMEngineResponse.SUCCESS;
	}
	
	public MXMEngineResponse storeRegistry()
	{
		try {
			JAXBContext jc = JAXBContext.newInstance(new Class[]{eu.convergence.middleware.overlayTE.schema.registry.ObjectFactory.class});
			Marshaller m = jc.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			File f = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION()
					+ registry.getFractal() + ".xml");
			m.marshal(registry, f);
		} catch (JAXBException e) {
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		return MXMEngineResponse.SUCCESS;
	}
	
	public MXMObject getRegistry()
	{
		return new MXMAdapter(registry);
	}

    public void updateRegistry(RegistryType registry)
    {
        this.registry = registry;
    }

	@Override
	public MXMEngineResponse parseFractalRegistry(File f) throws OverlayEngineException 
	{
		try {
			FileInputStream fin = new FileInputStream(f);
			return parseFractalRegistry(fin);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		
		return MXMEngineResponse.FAILURE;
	}
}
