package eu.convergence.middleware.overlayTE.registry.tcpip;

import eu.convergence.middleware.conetTe.ConetTE;
import eu.convergence.middleware.core.ConvergenceEngineName;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;

import eu.convergence.middleware.engine.ConetTe.exception.ConetIOException;
import eu.convergence.middleware.engine.ConetTe.services.NID;
import eu.convergence.middleware.engine.overlayTE.RegistryCreator;
import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.overlayTE.OverlayEnginePaths;
import eu.convergence.middleware.overlayTE.OverlayManager;
import eu.convergence.middleware.overlayTE.kernel.tcpip.OverlayTE;
import eu.convergence.middleware.overlayTE.message.Opcodes;
import eu.convergence.middleware.overlayTE.message.OverlayMessageCreator;
import eu.convergence.middleware.overlayTE.registry.OverlayRegistryParser;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.HeaderType;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.PayloadType;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.RegistrationMessageType;
import eu.convergence.middleware.overlayTE.schema.registry.RegistryType;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import java.io.File;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

public class OverlayRegistryCreator implements RegistryCreator 
{
	private RegistryType registry;
	private String fractalId;
	
	@Override
	public MXMEngineResponse registerToFractal(String fractalId, String overlaySAP, Date leaveDate) throws MXMConfigurationException, MXMEngineException 
	{
		int found = 0;
		this.fractalId = fractalId;
		
		// first get the registry from the fractal and store it locally
//		ConetTE conetTE = new ConetTE();
		//SS: is there a conetTE instance somewhere ?
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();

		ConetTE conetTE = (ConetTE) mxm.getDefaultEngine(ConvergenceEngineName.ConetTE.toString());
		NID nid = new NID(OverlayTE.registryFormat + fractalId);
		try {
			found = conetTE.get(nid, OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + fractalId + ".xml");
			if (found != -1)
			{
                System.out.println("Registry has been found in CoNet...");
				OverlayRegistryParser overlayRegistryParser = new OverlayTE().getRegistryParser();
				overlayRegistryParser.parseFractalRegistry(new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + fractalId + ".xml"));
				overlayRegistryParser.insertPeerToRegistry(OverlayManager.getInstance().getPeerId(), OverlayManager.getInstance().getOverlaySAP(), OverlayManager.getInstance().getLatestLeaveDate());
                overlayRegistryParser.storeRegistry();
				this.registry = (RegistryType)overlayRegistryParser.getRegistry().getContent();
                notifyNeighbors();
			}
			
		} catch (ConetIOException e) {
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		
		if (found == -1) // registry has not been found in the network
		{
            System.out.println("Registry was not found...");
			// create the registry locally
			registry = new RegistryType();
			registry.setFractal(fractalId);
			OverlayRegistryParser overlayRegistryParser = new OverlayRegistryParser(registry);
			overlayRegistryParser.insertPeerToRegistry(OverlayManager.getInstance().getPeerId(), overlaySAP, leaveDate);
			
			// store the registry
			overlayRegistryParser.storeRegistry();
		}
		
		return MXMEngineResponse.SUCCESS;
	}

	@Override
	public MXMEngineResponse notifyNeighbors() throws OverlayEngineException 
	{
		// set up the gossip registration message
		HeaderType header = new HeaderType();
		header.getFractals().add(fractalId);
		header.setOpcode(Opcodes.REGISTRATION_MESSAGE_RESPONSE);
		header.getPath().add(OverlayManager.getInstance().getPeerId());
		
		int fractalMembers = registry.getFractalMembers().intValue();
		double nofRounds = Math.log((double)fractalMembers);
		int standardNofRounds = Double.valueOf(nofRounds).intValue();
		header.setTTL(standardNofRounds);
		
		PayloadType payload = new PayloadType();
		RegistrationMessageType registrationDiscoveryMessage = new RegistrationMessageType();
		registrationDiscoveryMessage.setPeerId(OverlayManager.getInstance().getPeerId());
		registrationDiscoveryMessage.setOverlaySAP(OverlayManager.getInstance().getOverlaySAP());
		
		try {
			DatatypeFactory df = DatatypeFactory.newInstance();
			GregorianCalendar gc = new GregorianCalendar();
			gc.setTime(OverlayManager.getInstance().getLatestLeaveDate());
			registrationDiscoveryMessage.setLeaveDate(df.newXMLGregorianCalendar(gc));
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}
		
		payload.setDiscoveryMessage(registrationDiscoveryMessage);
		
		OverlayTE overlayTE = new OverlayTE();
		
		OverlayMessageCreator overlayMessageCreator = overlayTE.getMessageCreator();
		MXMObject message = overlayMessageCreator.createMessage(new MXMAdapter(header), new MXMAdapter(payload));
		
		// now gossip to all the peers of the registry
		List<String> registrySAPs = registry.getOverlaySAPs();
		for (String SAP : registrySAPs)
		{
            if (SAP.equals(OverlayManager.getInstance().getOverlaySAP()))
            {
                continue;
            }
			overlayTE.getPropagationMessageHandler().sendPropagationMessage(message, SAP);
		}
		
		return MXMEngineResponse.SUCCESS;
	}

	@Override
	public MXMEngineResponse advertiseRegistry(String regPath) throws MXMConfigurationException, MXMEngineException {
//		ConetTE conetTE = new ConetTE();
		//SS: is there a conetTE instance somewhere ?
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();

		ConetTE conetTE = (ConetTE) mxm.getDefaultEngine(ConvergenceEngineName.ConetTE.toString());
		
		
		NID nid = new NID(OverlayTE.registryFormat + fractalId);
		conetTE.storeAndAdvertise(nid, 0, regPath);
		
		return MXMEngineResponse.SUCCESS;
	}

}
