package eu.convergence.middleware.overlayTE.schema;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMObject;
import eu.convergence.middleware.overlayTE.message.Opcodes;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.GorestMessageType;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.HeaderType;
import eu.convergence.middleware.overlayTE.schema.gorestMessage.PayloadType;

import java.util.List;

public class GorestMessage 
{
	private GorestMessageType gorestMessageType = null;
	private HeaderType gorestMessageHeader = null;
	private PayloadType gorestMessagePayload = null;
	
	public GorestMessage()
	{
		gorestMessageType = new GorestMessageType();
		gorestMessageHeader = new HeaderType();
		gorestMessagePayload = new PayloadType();
	}
	
	public GorestMessage(GorestMessageType gorestMessageType)
	{
		this.gorestMessageType = gorestMessageType;
		this.gorestMessageHeader = this.gorestMessageType.getHeader();
		this.gorestMessagePayload = this.gorestMessageType.getPayload();
	}
	
	public GorestMessage(MXMObject message)
	{
		this.gorestMessageType = (GorestMessageType)message.getContent();
		this.gorestMessageHeader = this.gorestMessageType.getHeader();
		this.gorestMessagePayload = this.gorestMessageType.getPayload();
	}
	
	public List<String> getFractals()
	{
		return gorestMessageHeader.getFractals();
	}

    public List<String> getPath()
    {
        return gorestMessageHeader.getPath();
    }
	
	public void addPeerToPath(String peerId)
	{
		gorestMessageHeader.getPath().add(peerId);
	}
	
	public MXMObject toMXMObject()
	{
        gorestMessageType.setHeader(gorestMessageHeader);
        gorestMessageType.setPayload(gorestMessagePayload);
		return new MXMAdapter(gorestMessageType);
	}
	
	public void setPublicationVDI(MXMObject VDI)
	{
		// convert VDI to binary and set the payload
		byte[] payloadVDI = (byte[])VDI.getContent();
		gorestMessagePayload.setPublicationVdi(payloadVDI);
		gorestMessageHeader.setOpcode(Opcodes.PUBLICATION_MESSAGE);
	}
	
	public void setSubscriptionVDI(MXMObject VDI)
	{
		// convert VDI to binary and set the payload
		byte[] payloadVDI = (byte[])VDI.getContent();
		gorestMessagePayload.setSubscriptionVdi(payloadVDI);
		gorestMessageHeader.setOpcode(Opcodes.SUBSCRIPTION_MESSAGE);
	}
	
	public void setRevocationVDI(MXMObject VDI)
	{
		// convert VDI to binary and set the payload
		byte[] payloadVDI = (byte[])VDI.getContent();
		gorestMessagePayload.setRevocationVdi(payloadVDI);
		gorestMessageHeader.setOpcode(Opcodes.REVOCATION_MESSAGE);
	}
	
	public void setFractals(String... fractals)
	{
		for (String fractal : fractals)
		{
			gorestMessageHeader.getFractals().add(fractal);
		}
	}
	
	public void setTTL(int TTL)
	{
		gorestMessageHeader.setTTL(TTL);
	}
}
