/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package convergence_pkgcontent.boxes;
import java.io.File;
import java.io.FileInputStream;

import convergence_pkgcontent.constants.BoxType;
import convergence_pkgcontent.utils.ArrayUtils;


/**
 * This class provides user methods to perform operations on BXMLBox. 
 * It is extended from FullBox
 * 
 * @author Xueqing Liu
 *	
 */
public class BXMLBox extends FullBox
{
	/**
	 * Reports the box type.
	 */
	public final String type = BoxType.BXML;
	
	/**
	 * A byte array which contains the BXMLData.
	 */
    public byte[] BXMLData;
	    
    //*************************************************************
       
    /**
     * Caculates the Box size in bytes.
     */
    public int GetBoxSize()
    {
    	 boxSize = 4 + //size
		   		   type.length() +
		   		   1 + //version
		   		   3 + //flag
		           BXMLData.length;

    	return boxSize;
    }
    
    /**
     * Get BXML data from a BXML file
     * @param fileName the file name of the BXML file which contains the BXML data
     */
    public void GetDataFromBXMLFile(String fileName)
    {
    	try
    	{
    		File file =new File(fileName);
    		FileInputStream fis=new FileInputStream(file);
    		BXMLData=new byte[(int)file.length() ];
    		fis.read(BXMLData,0,(int)file.length());
    		fis.close();
    	}
    	catch (Exception ex)
    	{
    		System.out.println("BXMLBox1:" + ex.toString() );
    	}
    }
    
    /**
     * Put all data of this box into a byte array
     * @param bytes byte array which to contain the data
     * @return the box size
     */
    public int Serialize(byte[] bytes)
    {
    	int n=0;
        
        try
        {
        	//size
        	byte[] temp=new byte[4];
        	ArrayUtils.IntToByteArray(size, temp);
        	for (int i = 0; i < temp.length ; i++)
            {
        		bytes[n] = (byte)temp[i];
        		n++;
            }

        	//type
        	for (int i = 0; i < type.length() ; i++)
        	{
        		bytes[n] = (byte)type.charAt(i);
        		n++;
        	}
        	
        	//version
        	bytes[n]=version;
        	n++;
        	
        	//flag
        	for(int i=0; i<3; i++)
        	{
        		bytes[n]=flags[i];
        		n++;
        	}
        	//bxml data
        	for (int i = 0; i < BXMLData.length; i++)
            {
                bytes[n]=BXMLData[i];
                n++;
            }
        }
        catch (Exception ex)
        {
        	System.out.println("BXMLBox" + ex.toString() );
        }
        
        return n;  	     
    }
    
    /**
     * Get box type
     * @return Box type
     */
    public String GetBoxType()
    {
    	return this.type;
    }
}
