/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package convergence_pkgcontent.boxes;

/**
 * This class specifies the attributes of the ExtentInfo.
 * This class is used by the class ItemInfo.
 * 
 * @author Xueqing Liu
 *
 */
public class ExtentInfo 
{
	/**
	 * Offset size
	 */
	protected int offsetSize;
	
	/**
	 * Length size
	 */
	protected int lengthSize;
	
	/**
	 * The extent offset.
	 */
	public long extentOffset;
	
	/**
	 * The extent length.
	 */
	public long extentLength;
	
	
	public ExtentInfo(int offsetSize, int lengthSize)
	{
		this.offsetSize=offsetSize;
		this.lengthSize=lengthSize;
	}
	/**
	 * Get extent info size
	 * @return extent info size
	 */
	public int GetExtentInfoSize()
	{
		int size=0;
		
		size= this.offsetSize + 
			  this.lengthSize;
		
		return size;
	}
	
	/**
	 * Put all data into a byte array
	 * @return extent info size
	 */
	public int Serialize(byte[] bytes)
	{
		int n=0;
		
		try
		{
			for(int i=0; i<offsetSize; i++)
			{
				bytes[n]=(byte)(extentOffset >>(offsetSize-1-i)*8);
				n++;
			}

			for(int i=0; i<lengthSize; i++)
			{
				bytes[n]=(byte)(extentLength >>(lengthSize-1-i)*8);
				n++;
			}
		}
		catch(Exception ex)
		{
			System.out.println("ExtentInfo:" + ex.toString());
		}
		
		return n;
	}
}
