/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package convergence_pkgcontent.boxes;
import convergence_pkgcontent.constants.BoxType;
import convergence_pkgcontent.utils.ArrayUtils;

/**
 * This class provides user methods to perform operations on HandlerBox. 
 * It is extended from Box
 * 
 * @author Xueqing Liu
 *	
 */
public class HandlerBox extends FullBox
{
	/**
	 * Reports the box type.
	 */
	public final String  type = BoxType.Handler;

	/**
	 * predefined
	 */
	public int preDefined=0;
	
	/**
	 * Specifies the handler type.
	 */
	public String handlerType ="dmpf";
    
	/**
	 * Specifies the reserved data in an array
	 */
	public final int[] reserved ={0,0,0};
	
	/**
	 * Specifies name
	 */
	public String name="dmpf";
	
    //*************************************************************
	
	/**
	 * Calculates the box's size in byte
	 */
    public int GetBoxSize()
    {
     	 boxSize = 4 + //size
 		   		   type.length() +
 		   		   1 + //version
 		   		   3 + //flag
 		   		   4 + //preDefined
 		           4 + //handlerType.length() +
 		           12 + //reserved
 		           name.length();
     	
     	return boxSize;
    }
    
   /**
    * put all bytes of the box into a byte array
    * 
    */
    public int Serialize(byte[] bytes)
    {
    	int n=0;

    	try
    	{
    		//size
    		byte[] temp=new byte[4];
        	ArrayUtils.IntToByteArray(size, temp);
        	for (int i = 0; i < temp.length ; i++)
            {
        		bytes[n] = (byte)temp[i];
        		n++;
            }

        	//type
    		for (int i = 0; i < type.length() ; i++)
    		{
    			bytes[n] = (byte)type.charAt(i);
    			n++;
    		}

    		//version
        	bytes[n]=version;
        	n++;
        	
        	//flag
        	for(int i=0; i<3; i++)
        	{
        		bytes[n]=flags[i];
        		n++;
        	}
    		
    		//predefinded
        	temp=new byte[4];
        	for(int i=0; i<4; i++)
        	{
        		bytes[n]=(byte)(preDefined>>(3-i)*8);
        		n++;
        	}
        	
        	//handler type
        	for(int i=0; i<4; i++)
        	{
        		bytes[n]=(byte)handlerType.charAt(i);
        		n++;
        	}
    		//reserved
    		temp=new byte[4];
    		for(int i=0; i<3; i++)
    		{
    			for(int j=0; j<4; j++)
    			{
    				bytes[n]=(byte)(reserved[i]>>(3-i)*8);
            		n++;
    			}
    		}
    		
    		//name
    		for(int i=0; i<name.length(); i++)
    		{
    			bytes[n]=(byte)name.charAt(i);
    			n++;
    		}
    	}
    	catch(Exception ex)
    	{
    		System.out.println("HandlerBox:" + ex.toString());
    	}
    	
    	return n;
    }
    
    /**
     * Get box type
     * @return Box type
     */
    public String GetBoxType()
    {
    	return this.type;
    }
}
