/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package convergence_pkgcontent.boxes;
import java.io.File;
import java.io.FileInputStream;

import convergence_pkgcontent.constants.BoxType;
import convergence_pkgcontent.utils.ArrayUtils;

/**
 * This class provides user methods to perform operations on LicenseBox. 
 * It is extended from Box
 * 
 * @author Xueqing Liu
 *	
 */
public class LicenseBox extends FullBox
{
	/**
	 * Reports the box type.
	 */
	public final String type =BoxType.License ;
	
	/**
	 * A byte array which contains the license data.
	 */
    public byte[] licenseData;
    
    //*************************************************************
    
    /**
     * Get license data from a license file
     * @param fileName the path of the signature file which contains the license data
     */
     public void GetLicenseDataFromFile(String fileName)
     {
    	 try
     	{
     		File file =new File(fileName);
     		FileInputStream fis=new FileInputStream(fileName);
     		licenseData=new byte[(int)file.length()];
     		fis.read(licenseData,0,(int)file.length());
     		fis.close();
     	}
     	catch (Exception ex)
     	{
     		System.out.println("LicenseBox1:" + ex.toString());
     	}
     }

    /**
	 * Calculates the box's size in byte
	 */
    public int GetBoxSize()
    {
    	 boxSize = 4 + //size
		   		   type.length() +
		   		   1 + //version
		   		   3 + //flag
		           licenseData.length ;
    	
    	return boxSize;
    }
    
    /**
     * 
     */
    public int Serialize(byte[] bytes)
    {
    	int n=0;
        
        try
        {
        	//size
        	byte[] temp=new byte[4];
        	ArrayUtils.IntToByteArray(size, temp);
        	for (int i = 0; i < temp.length ; i++)
            {
        		bytes[n] = (byte)temp[i];
        		n++;
            }

        	//type
        	for (int i = 0; i < 4; i++)
        	{
        		bytes[n] = (byte)type.charAt(i);
        		n++;
        	}

        	//version
        	bytes[n]=version;
        	n++;
        	
        	//flag
        	for(int i=0; i<3; i++)
        	{
        		bytes[n]=flags[i];
        		n++;
        	}
        	
        	//license data
        	for(int i=0; i<licenseData.length; i++)
        	{
        		bytes[n]=licenseData[i];
        		n++;
        	}
        	
        }
        catch (Exception ex)
        {
        	System.out.println("LicenseBox:" + ex.toString() );
        }

        return n;
    }
    
    /**
     * Get box type
     * @return Box type
     */
    public String GetBoxType()
    {
    	return this.type;
    }
}
