/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package convergence_pkgcontent.boxes;
import java.util.ArrayList;

import convergence_pkgcontent.constants.BoxType;
import convergence_pkgcontent.utils.ArrayUtils;

/**
 * This class provides user methods to perform operations on LicenseContainerBox. 
 * It is extended from FullBox
 * 
 * @author Xueqing Liu
 *	
 */
public class LicenseContainerBox extends FullBox
{
	/**
	 * Reports the box type.
	 */
	public final String type = BoxType.LicenseContainer;
	
	/**
	 * Specifies the item count in byte array.
	 */
    public int itemCount=0;
    
    /**
     * license array
     */
    public ArrayList<License> licenses=new ArrayList<License>();
    
    //*************************************************************
    
    /**
	 * Caculates the full size of this box in bytes
	 */    
    public int GetBoxSize()
    {
    	boxSize = 4 + //size 
    			  type.length() + 
    			  1 + //version
		   		  3 + //flag
    			  2 ; //itemCount
    	 
    	int n=0;
    	for(int i=0; i<itemCount; i++)
    	{
    		License license = licenses.get(i);
    		n=n+license.GetLicenseSize();
    	}
    	boxSize=boxSize+n;
    	 
    	return boxSize;
    }
    
    /**
	 * Put all data of this box into a byte array
	 */
    public int Serialize(byte[] bytes)
    {
    	int n=0;
        
        try
        {
        	//size
        	byte[] temp=new byte[4];
        	ArrayUtils.IntToByteArray(size, temp);
        	for (int i = 0; i < temp.length ; i++)
            {
        		bytes[n] = (byte)temp[i];
        		n++;
            }

        	//type
        	for (int i = 0; i < 4; i++)
        	{
        		bytes[n] = (byte)type.charAt(i);
        		n++;
        	}

        	//version
        	bytes[n]=version;
        	n++;
        	
        	//flag
        	for(int i=0; i<3; i++)
        	{
        		bytes[n]=flags[i];
        		n++;
        	}
        	
        	//itemCount
        	for(int i=0; i<2; i++)
        	{
        		bytes[n]=(byte)(itemCount>>(1-i)*8);
        		n++;
        	}
        	
        	//license array
        	for(int i=0; i<itemCount; i++)
        	{
        		License license=(License)licenses.get(i);
        		temp=new byte[license.GetLicenseSize()];
        		license.Serialize(temp);
        		for(int j=0; j<license.GetLicenseSize(); j++)
        		{
        			bytes[n]=temp[j];
        			n++;
        		}
        	}
        }
        catch (Exception ex)
        {
        	System.out.println("LicenseContainerBox:" + ex.toString() );
        }
    	
        return n;
    }
    
    /**
     * Get box type
     * @return Box type
     */
    public String GetBoxType()
    {
    	return this.type;
    }
}
