/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package convergence_pkgcontent.boxes;
import convergence_pkgcontent.constants.BoxType;
import convergence_pkgcontent.utils.ArrayUtils;

/**
 * This class provides user methods to perform operations on MetaBox. 
 * It is extended from Box
 * 
 * @author Xueqing Liu
 *	
 */

public class MetaBox extends FullBox
{
	/**
	 * Reports the box type
	 */
	public final String type = BoxType.Meta;
	
    /**
	 * Specifies a HandlerBox.
	 */
    public HandlerBox handlerBox;
    
    /**
     * Specifies a ItemLocationBox.
     */
    public ItemLocationBox itemLocationBox;
    
//    /**
//     * Specifies a BXMLBox.
//     */
//    public BXMLBox bxmlBox;
    
	/**
     * Specifies an XMLBox.
     */
    public XMLBox xmlBox ;
    
        
    //*************************************************************
    
    /**
	 * Calculates the box's size in byte
	 */
    public int GetBoxSize()
    {
    	 boxSize = 4 + //size
		   		   type.length() + 
		   		   1 + //version
		   		   3 + //flag
		           handlerBox.GetBoxSize() +
		           itemLocationBox.GetBoxSize() +
		           //bxmlBox.GetBoxSize() +
		           xmlBox.GetBoxSize();

    	 return boxSize;
    }
    
    /**
     * Put all data of this box into a byte array
     * 
     */
    public int Serialize(byte[] bytes)
    {
    	int n=0;
        
        try
        {
        	//size
        	byte[] temp=new byte[4];
        	ArrayUtils.IntToByteArray(size, temp);
        	for (int i = 0; i < temp.length ; i++)
            {
        		bytes[n] = (byte)temp[i];
        		n++;
            }

        	//type
        	for (int i = 0; i < type.length() ; i++)
        	{
        		bytes[n] = (byte)type.charAt(i);
        		n++;
        	}

        	//version
        	bytes[n]=version;
        	n++;
        	
        	//flag
        	for(int i=0; i<3; i++)
        	{
        		bytes[n]=flags[i];
        		n++;
        	}
        	
        	//handler box
        	byte[] temps=new byte[handlerBox.GetBoxSize()];
        	handlerBox.Serialize(temps);
        	for(int i=0; i<handlerBox.GetBoxSize(); i++)
        	{
        		bytes[n]=temps[i];
        		n++;
        	}
        	
        	//iloc box
        	temps=new byte[itemLocationBox.GetBoxSize()];
        	itemLocationBox.Serialize(temps);
        	for(int i=0; i<itemLocationBox.GetBoxSize(); i++)
        	{
        		bytes[n]=temps[i];
        		n++;
        	}
        	
//        	//bxml box
//        	temps=new byte[bxmlBox.GetBoxSize()];
//        	bxmlBox.Serialize(temps);
//        	for(int i=0; i<bxmlBox.GetBoxSize(); i++)
//        	{
//        		bytes[n]=temps[i];
//        		n++;
//        	}

        	//xml box
        	temps=new byte[xmlBox.GetBoxSize()];
        	xmlBox.Serialize(temps);
        	for(int i=0; i<xmlBox.GetBoxSize(); i++)
        	{
        		bytes[n]=temps[i];
        		n++;
        	}
        }
        catch (Exception ex)
        {
        	System.out.println("MetaBox:" + ex.toString() );
        }
        
        return n;
    }
    
    /**
     * Get box type
     * @return Box type
     */
    public String GetBoxType()
    {
    	return this.type;
    }
}
