/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package convergence_pkgcontent.boxes;
import convergence_pkgcontent.constants.BoxType;
import convergence_pkgcontent.utils.ArrayUtils;


/**
 * This class provides user methods to perform operations on SignatureBox. 
 * It is extended from Box
 * 
 * @author Xueqing Liu
 *	
 */
public class SignatureBox extends Box
{
	/**
	 * Reports the box type.
	 */
	public final String  type = BoxType.Signature;
	
	/**
	 * A byte array which contains the signature data.
	 */
    public byte[] signatureData;
    
    
    /**
    * Get signature data from a signature file
    * @param fileName the path of the signature file which contains the signature data
    */
    public void GetSignatureData(byte[] bytes)
    {
    	try
    	{
    		signatureData=new byte[bytes.length];
    		for(int i=0; i<bytes.length; i++)
    		{
    			signatureData[i]=bytes[i];
    		}
    	}
    	catch (Exception ex)
    	{
    		System.out.println("Signature1:" + ex.toString() );
    	}
    }
    
    /**
	 * Calculates the box's size in byte
	 */
    public int GetBoxSize()
    {
    	 boxSize = 4 + //size
		   		   type.length() + 
		           signatureData.length;
    	
    	return boxSize;
    }
    
    /**
     * Put all data of this box into a byte array
     */
    public int Serialize(byte[] bytes)
    {
    	int n=0;
        
        try
        {
        	//size
        	byte[] temp=new byte[4];
        	ArrayUtils.IntToByteArray(size, temp);
        	for (int i = 0; i < temp.length ; i++)
            {
        		bytes[n] = (byte)temp[i];
        		n++;
            }

        	//type
        	for (int i = 0; i < 4; i++)
        	{
        		bytes[n] = (byte)type.charAt(i);
        		n++;
        	}

        	//signature data
        	for (int i=0; i<signatureData.length; i++)
        	{
        		bytes[n]=signatureData[i];
        		n++;
        	}
        }
        catch (Exception ex)
        {
        	System.out.println("SignatureBox:" + ex.toString() );
        }
        
        return n;
    }
    
    /**
     * Get box type
     * @return Box type
     */
    public String GetBoxType()
    {
    	return this.type;
    }
}
