/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package eu.convergence.middleware.relTE.authorisation;

import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.engine.relTE.Issuer;
import org.iso.mpeg.mxm.engine.relTE.authorisation.AuthorisationResult;

public class AuthorisationResponse {

	private Issuer issuerToBeVerified = null;
	
	private List<EvaluatedGrant> evaluatedGrants = null;
	
	private EvaluatedGrant succesfulGrant = null;

	private AuthorisationResult result = AuthorisationResult.GENERIC_AUTHORISATION_FAILURE;
	
	private String displayString = null;
	
	private String licenseID = null;
	
	public AuthorisationResponse() {
		evaluatedGrants = new ArrayList<EvaluatedGrant>();
	}
	
	public AuthorisationResult getResult() {
		return result;
	}

	public EvaluatedGrant getSuccessfulGrant() {
		return succesfulGrant;
	}

	public Issuer getIssuerToBeVerified() {
		return issuerToBeVerified;
	}

	public void setIssuerToBeVerified(Issuer issuerToBeVerified) {
		this.issuerToBeVerified = issuerToBeVerified;
	}

	public List<EvaluatedGrant> getEvaluatedGrants() {
		return evaluatedGrants;
	}

	public void setEvaluatedGrants(List<EvaluatedGrant> evaluatedGrants) {
		this.evaluatedGrants = evaluatedGrants;
		
		for (EvaluatedGrant evaluatedGrant : evaluatedGrants)
			recalculateScore(evaluatedGrant);			
	}

	public void addEvaluatedGrant(EvaluatedGrant evaluatedGrant) {
		this.evaluatedGrants.add(evaluatedGrant);
		
		recalculateScore(evaluatedGrant);
	}
	
	/**
	 * This method calculates whether a new added condition is "more far away from being met"
	 * than the previous "most far away from being met" condition result stored in the 
	 * <code>result</code> member variable.
	 * 
	 * @param newConditionType
	 */
	private void recalculateScore(EvaluatedGrant evaluatedGrant) {
		if (evaluatedGrant.getAuthorisationResult().compareTo(result) < 0 ) {
			result = evaluatedGrant.getAuthorisationResult();
			succesfulGrant = evaluatedGrant;
		}
	}

	public String getDisplayString() {
		return displayString;
	}

	public void setDisplayString(String displayString) {
		this.displayString = displayString;
	}

	public String getLicenseID() {
		return licenseID;
	}

	public void setLicenseID(String licenseID) {
		this.licenseID = licenseID;
	}
	
}
