/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2007
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package eu.convergence.middleware.relTE.authorisation;

import java.util.GregorianCalendar;
import java.util.List;

import javax.xml.bind.JAXBElement;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;

import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.engine.relTE.exception.RELAuthorisationException;
import org.iso.mpeg.mxm.engine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.engine.relTE.schema.relm2x.SecuritySystem;
import org.iso.mpeg.mxm.engine.relTE.schema.relm3x.CopyrightNotice;
import org.iso.mpeg.mxm.engine.relTE.schema.relm3x.NonCommercialUse;
import org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition;
import org.iso.mpeg.mxm.engine.relTE.schema.relr.ValidityInterval;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.ExerciseLimit;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.FeeFlat;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.FeeMetered;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.FeePerInterval;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.FeePerUse;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.FeePerUsePrePay;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.Territory;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.ValidityTimeMetered;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.Territory.Domain;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.Territory.Location;

public class ConditionManager {
	
	private Logger logger = Logger.getLogger(ConditionManager.class);
	
	public ConditionManager() {
	}
	
	public ConditionResult verifyConditions (List<MXMObject> conditions, QName currentCountry, QName currentRegion, boolean isCommercial) throws SecurityEngineException, RELAuthorisationException{
		
		//prepare the result to be returned by this method
		ConditionResult conditionResult = new ConditionResult();		
		
		if (conditions == null) {
			// if there are no conditions, the default value is: SUCCESSFULLY_MET
			return conditionResult;
		}

		//FIXME-security			
//		for (MXMObject obj : conditions) {
//			
//			if (! obj.getContentType().equals(JAXBElement.class.getCanonicalName()) )
//				throw new RELAuthorisationException("Unable to handle conditions of type: " + obj.getContentType());
//				
//			JAXBElement<? extends Condition> conditionElement = (JAXBElement<? extends Condition>) obj.getContent();

//			EvaluatedCondition evalCondition = new EvaluatedCondition(conditionElement);
//			
//			//VALIDITY INTERVAL
//			if (conditionElement.getValue() instanceof ValidityInterval) {
//				ValidityInterval validityInterval = (ValidityInterval)conditionElement.getValue();
//				if (this.verifyValidityInterval(validityInterval)) 
//					evalCondition.setConditionType(ConditionResultType.SUCCESSFULLY_MET);
//				else 
//					evalCondition.setConditionType(ConditionResultType.NOT_MET);
//			}
//
//			//EXERCISE_LIMIT
//			else if (conditionElement.getValue() instanceof ExerciseLimit) {
//				ExerciseLimit exerciseLimit = (ExerciseLimit)conditionElement.getValue();
//				if (exerciseLimit.getCount().intValue() >= 0)
//					evalCondition.setConditionType(ConditionResultType.SUCCESSFULLY_MET);
//				else
//					evalCondition.setConditionType(ConditionResultType.NOT_MET);
//			}
//
//			//TERRITORY
//			else if (conditionElement.getValue() instanceof Territory) {
//				Territory territory = (Territory)conditionElement.getValue();
//				List<Object> contents = territory.getLocationOrDomain();
//				if (contents.get(0) instanceof Location) {
//					Territory.Location location = (Territory.Location)contents.get(0);
//					evalCondition.setConditionType(this.verifyLocation(location, currentCountry, currentRegion));
//				}
//				else if (contents.get(0) instanceof Domain){
//					Territory.Domain domain = (Territory.Domain)contents.get(0);
//					evalCondition.setConditionType(ConditionResultType.UNSUPPORTED);
//				}
//			}
//
//			//VALIDITY_TIME_METERED
//			else if (conditionElement.getValue() instanceof ValidityTimeMetered) {
//				evalCondition.setConditionType(ConditionResultType.UNSUPPORTED);
//			}
//
//			//SECURITY_SYSTEM
//			else if (conditionElement.getValue() instanceof SecuritySystem) {
//				evalCondition.setConditionType(ConditionResultType.UNSUPPORTED);
//			}
//
//			//ORC
//			else if (conditionElement.getValue() instanceof NonCommercialUse) {
//				if (isCommercial)
//					evalCondition.setConditionType(ConditionResultType.UNSUPPORTED);
//				else
//					evalCondition.setConditionType(ConditionResultType.SUCCESSFULLY_MET);
//			}
//			else if (conditionElement.getValue() instanceof CopyrightNotice) {
//				evalCondition.setConditionType(ConditionResultType.COPYRIGHT_NOTICE);
//			}
//
//			//PAYMENT
//			else if (conditionElement.getValue() instanceof FeeFlat ||
//					conditionElement.getValue() instanceof FeeMetered ||
//					conditionElement.getValue() instanceof FeePerInterval ||
//					conditionElement.getValue() instanceof FeePerUse ||
//					conditionElement.getValue() instanceof FeePerUsePrePay) {
//				evalCondition.setConditionType(ConditionResultType.PAYMENT);
//			}
//			
//			//all other conditions are not supported
//			else {
//				evalCondition.setConditionType(ConditionResultType.UNSUPPORTED);				
//			}
//			
//			conditionResult.addEvaluatedCondition(evalCondition);
//		}

		return conditionResult;
	}
	
	private boolean verifyValidityInterval(ValidityInterval validityInterval) throws RELAuthorisationException {	
		DatatypeFactory dateTypeFactory;
		try {
			dateTypeFactory = DatatypeFactory.newInstance();
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
			String message = "A DataTypeConfigurationException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message);
			throw new RELAuthorisationException(message);
		}
		XMLGregorianCalendar now = dateTypeFactory.newXMLGregorianCalendar(new GregorianCalendar());
		return verifyNotAfter(validityInterval.getNotAfter(), now) && verifyNotBefore(validityInterval.getNotBefore(), now);
	}
	
	private boolean verifyNotAfter(XMLGregorianCalendar notAfter, XMLGregorianCalendar now) {
		logger.debug("Not After: " + notAfter.toString());
		logger.debug("Now: " + now.toString());
		if (notAfter == null) 
			return true;
		else {
			boolean value = now.compare(notAfter) == DatatypeConstants.LESSER;
			return value;
		}
	}

	private boolean verifyNotBefore(XMLGregorianCalendar notBefore, XMLGregorianCalendar now) {
		logger.debug("Not Before: " + notBefore.toString());
		logger.debug("Now: " + now.toString());
		if (notBefore == null) 
			return true;
		else {
			boolean value = now.compare(notBefore) == DatatypeConstants.GREATER;
			return value;
		}
	}

	private ConditionResultType verifyLocation (Location licenseLocation, QName currentCountry, QName currentRegion) throws RELAuthorisationException {
		QName destCountry = licenseLocation.getCountry();
		QName destRegion = licenseLocation.getRegion();
		if (destCountry != null) {
			if (currentCountry != null) {
				if(currentCountry.equals(destCountry)) {
					if (destRegion != null) {
						if (currentRegion != null) {					
							if(currentRegion.equals(destRegion))
								return ConditionResultType.SUCCESSFULLY_MET;
							else
								return ConditionResultType.NOT_MET;
						}
						else
							return ConditionResultType.UNDETERMINED;
					}
					else
						return ConditionResultType.SUCCESSFULLY_MET;
				}
				else
					return ConditionResultType.NOT_MET;
			}
			else
				return ConditionResultType.UNDETERMINED;
		}
		else
			return ConditionResultType.SUCCESSFULLY_MET;
	}

}
