/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package eu.convergence.middleware.relTE.authorisation;

import org.iso.mpeg.mxm.engine.relTE.Grant;
import org.iso.mpeg.mxm.engine.relTE.authorisation.AuthorisationResult;



/**
 * This enum list all possible Evaluation conditions
 * that may occur when evaluating whether a Condition is met.
 * The order in which they are listed is important: they are in 
 * order from "the most far away from being met" to the 
 * closest to be fully met. Payment requires the End-User to 
 * perform a transaction in order to be met. ORC requires
 * the explicit agreement to Creative Commons License terms
 * in order to be met. 
 * 
 * @author filippo
 *
 */
public class EvaluatedGrant {
	
	private AuthorisationResult authorisationResult = null;
	
	private ConditionResult conditionEvaluationResult = null;
	
	private String displayString = null;

	private byte[] encryptedMasterKey = null;
	
	private Grant grant = null;

	public EvaluatedGrant(Grant grant) {
		this.grant = grant;
	}

	public AuthorisationResult getAuthorisationResult() {
		return authorisationResult;
	}

	public void setAuthorisationResult(AuthorisationResult authorisationResult) {
		this.authorisationResult = authorisationResult;
	}

	public Grant getGrant() {
		return grant;
	}

	public ConditionResult getConditionEvaluationResult() {
		return conditionEvaluationResult;
	}

	public void setConditionEvaluationResult(ConditionResult conditionEvaluationResult) {
		this.conditionEvaluationResult = conditionEvaluationResult;
	}

	public byte[] getEncryptedMasterKey() {
		return encryptedMasterKey;
	}

	public void setEncryptedMasterKey(byte[] encryptedMasterKey) {
		this.encryptedMasterKey = encryptedMasterKey;
	}

	public String getDisplayString() {
		return displayString;
	}

	public void setDisplayString(String displayString) {
		this.displayString = displayString;
	}

}
