/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package eu.convergence.middleware.revokePE.utils;

public class ArrayUtils 
{
	/**
	 * Convert a byte array to a string 
	 * @param bytes bytes to be converted to string
	 * @return a string
	 */
	public static String ByteArrayToString(byte[] bytes)
	{
		char[]  chars=new char[bytes.length];
		for(int i=0; i<bytes.length; i++)
		{
			chars[i]=(char)bytes[i];
		}
		
		return String.copyValueOf(chars);
		
	}

	/**
	 * Convert a byte array to an int type
	 * @param bytes a byte array which to be converted to int
	 * @return int result
	 */
	public static int ByteArrayToInt(byte[] bytes)
	{
		int n=0;
		for(int i=bytes.length-1; i>=0; i--)
		{
			int temp=bytes[i] & 0x000000ff;
			temp =(temp << 8*(bytes.length -1-i)) ;
			n=n | temp;
		}
		return n;
		
	}
	
	/**
	 * Convert a byte array to a long type
	 * @param bytes a byte array which to be converted to long
	 * @return long result
	 */
	public static long ByteArrayToLong(byte[] bytes)
	{
		long n=0;
		for(int i=bytes.length-1; i>=0; i--)
		{
			int temp=bytes[i] & 0x000000ff;
			temp =(temp << 8*(bytes.length -1-i)) ;
			n=n | temp;
		}
		return n;
		
	}
	/**
	 * Convert an int value into a byte array
	 * @param value int value which to be converted 
	 * @param bytes converted result
	 */
	public static void IntToByteArray(int value, byte[] bytes)
	{
		int temp=value;
		
		for(int i=0; i<4; i++)
		{
			byte bytetemp=(byte) (temp >> (8*i) & 0xff);
			bytes[4-i-1] = bytetemp; 
		}
	}
}
