/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package eu.convergence.middleware.revokePE.utils;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.exceptions.DIEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;

import eu.convergence.middleware.engine.overlayTE.MessageKeeper;
import eu.convergence.middleware.engine.overlayTE.OverlayEngine;
import eu.convergence.middleware.engine.overlayTE.RegistryCreator;
import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.overlayTE.OverlayEnginePaths;
import eu.convergence.middleware.overlayTE.OverlayManager;
import eu.convergence.middleware.overlayTE.schema.GorestMessage;

/**
 * <HL>
 * This class offer methods to handle the revocation of a VDI.
 */
public class HandleRevocationVDI 
{
	private OverlayManager overlayManager;
	private OverlayEngine overlayEngine;
	
	/**
	 * Convert a byte array to a string 
	 * @param bytes bytes to be converted to string
	 * @return a string
	 */
	public static String ByteArrayToString(byte[] bytes)
	{
		char[]  chars=new char[bytes.length];
		for(int i=0; i<bytes.length; i++)
		{
			chars[i]=(char)bytes[i];
		}
		
		return String.copyValueOf(chars);
		
	}

	/**
	 * Convert a byte array to an int type
	 * @param bytes a byte array which to be converted to int
	 * @return int result
	 */
	public static int ByteArrayToInt(byte[] bytes)
	{
		int n=0;
		for(int i=bytes.length-1; i>=0; i--)
		{
			int temp=bytes[i] & 0x000000ff;
			temp =(temp << 8*(bytes.length -1-i)) ;
			n=n | temp;
		}
		return n;
		
	}
	
	/**
	 * Convert a byte array to a long type
	 * @param bytes a byte array which to be converted to long
	 * @return long result
	 */
	public static long ByteArrayToLong(byte[] bytes)
	{
		long n=0;
		for(int i=bytes.length-1; i>=0; i--)
		{
			int temp=bytes[i] & 0x000000ff;
			temp =(temp << 8*(bytes.length -1-i)) ;
			n=n | temp;
		}
		return n;
		
	}
	/**
	 * Convert an int value into a byte array
	 * @param value int value which to be converted 
	 * @param bytes converted result
	 */
	public static void IntToByteArray(int value, byte[] bytes)
	{
		int temp=value;
		
		for(int i=0; i<4; i++)
		{
			byte bytetemp=(byte) (temp >> (8*i) & 0xff);
			bytes[4-i-1] = bytetemp; 
		}
	}
	
	
	/**
	 * <HL>
	 * Creation of a buffer message to be delivered by the fractals
	 * @param rVDI The resource VDI file
	 * @param fractals The list of fractals
	 * 
	 */
	public MXMEngineResponse revokeComidVDI (DIDL rVDI, String... fractals)
	{
		for (String fractal : fractals)
		{
			System.out.println ("Fractals: " + fractal);
		}
		
		// second create and buffer the message
		GorestMessage gorestMessage = new GorestMessage();
		System.out.println("PINT1 - Point 1.");
		gorestMessage.setFractals(fractals);
		System.out.println("PINT2 - Point 2.");
		//gorestMessage.addPeerToPath(overlayManager.getPeerId());
		
		try 
		{
			gorestMessage.setRevocationVDI(rVDI.createObject(Byte.class.getCanonicalName()));
		} 
		catch (DIEngineException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		} 
		catch (MXMEngineException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		catch (MXMException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		
		
		//MessageKeeper messageKeeper = overlayEngine.getMessageKeeper();
		List<String> finalPeerList = new ArrayList<String>();
		List<String> finalSAPList = new ArrayList<String>();
		
		
		//createPropagationLists(finalPeerList, finalSAPList, gorestMessage);
		gorestMessage.setTTL(finalPeerList.size() + 1);		// when buffering, we reduce the TTL anyway
		
		//messageKeeper.bufferMessage(gorestMessage.toMXMObject());
		
		
		
		return MXMEngineResponse.SUCCESS;
	}
	
	
	/*
	public MXMEngineResponse handleRevocationVDI(DIDL rVDI, String... fractals) throws OverlayEngineException
	{
		// first register to the fractal (if not already registered)		
		for (String fractal : fractals)
		{
			File file = new File(OverlayEnginePaths.getInstance().getREGISTRY_LOCATION() + fractal + ".xml");
//			if (file.exists())	{continue;}
			
			// the registry does not exist, so create it
			RegistryCreator registryCreator = overlayEngine.getRegistryCreator();
            try
            {
                registryCreator.registerToFractal(fractal, overlayManager.getOverlaySAP(), overlayManager.getLatestLeaveDate());
            } 
            catch (MXMConfigurationException e)
            {
                e.printStackTrace();
            }
            catch (MXMEngineNotFoundException e)
            {
                e.printStackTrace();
            } 
            catch (MXMEngineException e)
			{
				e.printStackTrace();
			}
        }
		
		// second create and buffer the message
		GorestMessage gorestMessage = new GorestMessage();
		gorestMessage.setFractals(fractals);
		gorestMessage.addPeerToPath(overlayManager.getPeerId());
		
		
		try 
		{
			gorestMessage.setRevocationVDI(rVDI.createObject(Byte.class.getCanonicalName()));
		} 
		catch (DIEngineException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		} 
		catch (MXMEngineException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		catch (MXMException e)
		{
			e.printStackTrace();
			return MXMEngineResponse.FAILURE;
		}
		
		MessageKeeper messageKeeper = overlayEngine.getMessageKeeper();
		List<String> finalPeerList = new ArrayList<String>();
		List<String> finalSAPList = new ArrayList<String>();
		
		createPropagationLists(finalPeerList, finalSAPList, gorestMessage);
		gorestMessage.setTTL(finalPeerList.size() + 1);		// when buffering, we reduce the TTL anyway
		
		messageKeeper.bufferMessage(gorestMessage.toMXMObject());
		
		return MXMEngineResponse.SUCCESS;
	} */

	
}
