/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package eu.convergence.middleware.vdiengine.test;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMAbstractEngine;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.ApplicationSpecificData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.w3c.dom.Document;

import eu.convergence.middleware.engine.vdiTE.VdiEngine;
import eu.convergence.middleware.vdiTE.Relationship;

public class DIDEngineTest extends TestCase {

	private Logger logger = Logger.getLogger(DIDEngineTest.class);
	
	private final String  TEMP_FOLDER = System.getProperty("java.io.tmpdir") + File.separator;
	
	private final String  OUTPUT_DCI_FILE_NAME = TEMP_FOLDER + "mytestDI.xml";

	private final String  SOURCE_DCI_FILE_NAME = "testDI.xml";	
		
	public void testDIParsing() throws FileNotFoundException, JAXBException, MXMException {
		logger.info("\n\n---------- testDIParsing ----------");
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		VdiEngine vdiEngine = (VdiEngine) mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
		EventReportEngine erEngine = (EventReportEngine) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());
		IPMPEngine ipmp = (IPMPEngine) mxm.getDefaultEngine(MXMEngineName.IPMPTE.toString());
		SecurityEngine secEngine = (SecurityEngine)mxm.getDefaultEngine(MXMEngineName.SecurityTE.toString());
		RELEngine relEngine = (RELEngine)mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
		MetadataEngine meta = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
		
		List<MXMEngine> dependencies=new ArrayList();
		dependencies.add(vdiEngine);
		dependencies.add(erEngine);
		dependencies.add(ipmp);
		dependencies.add(secEngine);
		dependencies.add(meta);
		vdiEngine.setDependencies(dependencies);
		
		DIDL di = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
		
		MXMEngineResponse response = di.parse(new MXMAdapter(new File(getClass().getClassLoader().getResource(SOURCE_DCI_FILE_NAME).getFile())));
		if (response.equals(MXMEngineResponse.SUCCESS)) {
			try {
				logger.info("Content ID: " + di.getMPEGMItem().getIdentifier());

	
				logger.info("Top-level Item ID: " + di.getMPEGMItem().getId());
				
				// Content Relationship
				List<ApplicationSpecificData> relationships = di.getMPEGMItem().getApplicationSpecificData();
				if(relationships != null && (!relationships.isEmpty())){
					for (ApplicationSpecificData rel : relationships) {
						
	//					if(((JAXBElement<?>)rel.getContent()).getValue().getClass().getCanonicalName().equals(eu.convergence.middleware.relationships.schema.convRelation.Relationship.class.getCanonicalName())){
	//						logger.info("Convergence relationship found:");
	//						eu.convergence.middleware.relationships.schema.convRelation.Relationship r = (eu.convergence.middleware.relationships.schema.convRelation.Relationship) ((JAXBElement<?>)rel.getContent()).getValue();
	//						logger.info("		SubjectVDI: " + r.getSubjectVDI());
	//						logger.info("		Predicate: " + r.getPredicate());
	//						logger.info("		ObjectVDI: " + r.getObjectVDI());
	//					}
	//					else
	//						logger.info("The Relationship was found being of type: " + ((JAXBElement<?>)rel.getContent()).getValue().getClass().getCanonicalName() + " which is not supported by this engine.");
	//				}
						
						
						
						Relationship o=new Relationship();
						
						//Document d=(Document)rel.getAny();
						
						o.parse(new MXMAdapter((eu.convergence.middleware.engine.vdiTE.schema.convrelation.Relationship) ((JAXBElement<?>)rel.getAny()).getValue()));
						logger.info("		SubjectVDI: " + o.getSubject());
						logger.info("		Predicate: " + o.getPredicate());
						logger.info("		ObjectVDI: " + o.getObject());
					}
				}
				else
					logger.info("No relationship content was found");
			
			} catch (MXMException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
						
		}
		else
			throw new MXMEngineException("The DIDEngine was unable to parse the Digital Item: " + SOURCE_DCI_FILE_NAME);
		
		logger.info("The DCI file was successfully parsed");
	}

	public void testDICreation() throws MXMException {
		logger.info("\n\n---------- testDICreation ----------");
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		VdiEngine vdiEngine = (VdiEngine)mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
		EventReportEngine erEngine = (EventReportEngine) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());
		IPMPEngine ipmp = (IPMPEngine) mxm.getDefaultEngine(MXMEngineName.IPMPTE.toString());
		RELEngine relEngine = (RELEngine) mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
		SecurityEngine secEngine = (SecurityEngine) mxm.getDefaultEngine(MXMEngineName.SecurityTE.toString());	
		MetadataEngine meta = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
		
		
		List<MXMEngine> dependencies=new ArrayList();
		dependencies.add(vdiEngine);
		dependencies.add(erEngine);
		dependencies.add(ipmp);
		dependencies.add(secEngine);
		dependencies.add(meta);
		vdiEngine.setDependencies(dependencies);

		DIDL di=vdiEngine.getVDIEngineSchemaHandler().getDIDL();
		Item toplevelitem=vdiEngine.getDIEngineSchemaHandler().getItem();
		ApplicationSpecificData asd=vdiEngine.getDIEngineSchemaHandler().getApplicationSpecificData();

		toplevelitem.setId("ANGELO");
	
		try{
			//for testing, here we simply add an 
			//List<JAXBElement<?>> objs = new ArrayList<JAXBElement<?>>();
			eu.convergence.middleware.engine.vdiTE.schema.convrelation.ObjectFactory fact= new eu.convergence.middleware.engine.vdiTE.schema.convrelation.ObjectFactory();
	        //objs.add(fact.createRelationship());

			asd.setAny(fact.createRelationship("ontologyURI","subjectID","RelationshipType","objectID"));
			
			toplevelitem.addApplicationSpecificData(asd);
			di.setMPEGMItem(toplevelitem);
			
			di.createFile(new File(OUTPUT_DCI_FILE_NAME));
			
			logger.info("The DCI file was successfully created in file: " + OUTPUT_DCI_FILE_NAME);
			
		}
		catch(Exception e)
		{
			logger.info("The Digital Item was found being of type which is not supported by this engine.");						

			
		}
	
	}	
}