package eu.convergence.middleware.tool.metadata;

import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;

/**
 * This interface specifies the methods that are provided to a developer for creating RDF/XML metadata and
 * wrapping it in a StructuredData object, suitable for adding it in a VDI.
 * 
 * An example of creating metadata :
 * <pre>
 * MetadataCreator metadataTool = MetadataTool.newMetadataCreator();
 * metadataTool.addRDFTriple("http://convergence.eu/test/subject", "http://convergence.eu/test/predicate", "obj");
 * metadataTool.generateStructuredData();
 * </pre>
 */
public interface MetadataCreator
{
	/**
     * Adds a RDF triple to the description.
     * The user passes a subject, a relationship and an object and the tool transforms it 
     * to a RDF statement. Note that subject, predicate and object muxt be valid URIs.
     * 
     * @param subject The subject of the triple
     * @param predicate The predicate of the triple
     * @param object The object of the triple
     */
	public void addRDFTriple(String subject, String predicate, String object);
	
	/**
    * Adds a RDF triple to the description.
    * The user passes a subject, a relationship and an object and the tool transforms it 
    * to a RDF statement. Both subject and predicate must be valid URIs. Notice
    * that the object is of anyType, allowing the user to submit also non-String values for
    * datatype properties.
    * 
    * @param subject The subject of the triple
    * @param predicate The predicate of the triple
    * @param value The value of the property
    */
	public void addRDFLiteralTriple(String subject, String predicate, Object value);
	
	/**
     * The method returns a structuredMetadata object containing the full
     * description submitted by the user.
     * 
     * @return The structured data object
	 * @throws MXMEngineException 
     */
	public StructuredData generateStructuredData();
}
