package eu.convergence.middleware.tool.metadata;

import java.util.List;
import java.util.Map;

/**
 * This interface specifies the methods that are provided to a developer for requesting entities of an 
 * ontology model, based on their labels.
 * 
 * An example of requesting metadata :
 * <pre>
 * MetadataRequester metadataTool = MetadataTool.newMetadataRequester();
 * metadataTool.requestMetadata(labelPrefix, type, ontologyURI);
 * </pre>
 */
public interface MetadataRequester
{
	/**
     * Returns a list of metadata entities that satisfy the parameters passed by the user.
     * The modelURI parameter specifies the model where the entity will be searched. The
     * request parameter specifies the prefix of the entity label. The type parameter is
     * used in case the model is ontology, and the requested entity should be of a specific
     * type.
     * 
     * @param label The request string.
     * @param type The type of the entity.
     * @param modelURI The namespace of the model to search.
     * @return A list of structuredMetadata.
     */
	public List<Map<String,List<String>>> requestMetadata(String label, String type, String modelURI);
}
