package eu.convergence.middleware.tool.metadata;

import java.io.File;

import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;

/**
 * This interface specifies the methods that are provided to a developer for parsing RDF/XML metadata and
 * wrapping it in a StructuredData object, suitable for adding it in a VDI.
 * 
 * An example of wrapping a RDF :
 * <pre>
 * String rdfFilePath = "rdf_description.rdf");
 * MetadataWrapper metadataTool = MetadataTool.newMetadataWrapper();
 * metadataTool.parseRDF(new File(rdfFilePath))
 * metadataTool.generateStructuredData();
 * </pre>
 */
public interface MetadataWrapper
{
	/**
     * This method enables the parsing of existing rdf/xml files.
     * 
     * @param rdfFile The rdf/xml file to be parsed.
     * @param baseURI The baseURI of the rdf file.
     * @return true if parsing was successful otherwise false.
     */
	public boolean parseRDF(File rdfFile, String baseURI);
	
	/**
     * The method returns a structuredMetadata object containing the full
     * description submitted by the user.
     * 
     * @return The structured data object
	 * @throws MXMEngineException 
     */
	public StructuredData generateStructuredData();
}
