package eu.convergence.middleware.tool.rel;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.UUID;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
import org.iso.mpeg.mxm.pEngine.createlicensePE.CreateLicenseEngine;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseResponse;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseSchemaHandler;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.RightGroup;
import org.iso.mpeg.mxm.pEngine.createlicensePE.technologyhandler.CreateLicenseProtocol;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngineSchemaHandler;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.UniqueID;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngineSchemaHandler;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.ConditionBuilder;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm1x.IdentityHolder;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm1x.ProtectedResource;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relmx.DiReference;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.AllPrincipals;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.AnXmlPatternAbstract;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Condition;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.DigitalResource;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.ForAll;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Grant;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Issuer;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.KeyHolder;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.NonSecureReference;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Principal;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.PropertyAbstract;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.PropertyPossessor;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.PropertyUri;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.RELRight;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.CipherData;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey;

/**
 * The LicenseTool allows the creation and parsing of licenses associated with the resources of specific VDIs.
 * Licenses are expressed in the Rights Expression Language (REL) and are serialized with the help of the REL TE.
 * In addition, the tool provides methods for the verification of a license validity as well as to check the 
 * rights of a principal on a resource.
 */
public class LicenseTool implements LicenseCreator, LicenseParser // TODO mipos na iparxei enas license object creator ???
{	
	private CreateLicenseEngine createLicenseEngine;
	private RELEngine relEngine;
	private MetadataEngine metadataEngine;
	private SecurityEngine securityEngine;
	
	private String identifier;
	private Issuer issuer;
	private List<Principal> principals;
	private List<RightGroup> grants;
	private ForAll forAll;
	private Resource resource;
	
	private License license;
	
	public static LicenseCreator newLicenseCreator()
	{
		return new LicenseTool();
	}
	
	public static LicenseParser newLicenseParser()
	{
		return new LicenseTool();
	}
	
	private LicenseTool()
	{
		try
		{
			createLicenseEngine = (CreateLicenseEngine) MXM.getInstance().getDefaultEngine(MXMEngineName.CreateLicensePE.toString());
			relEngine = (RELEngine) MXM.getInstance().getDefaultEngine(MXMEngineName.RELTE.toString());
			metadataEngine = (MetadataEngine) MXM.getInstance().getDefaultEngine(MXMEngineName.MetadataTE.toString());
			securityEngine = (SecurityEngine) MXM.getInstance().getDefaultEngine(MXMEngineName.SecurityTE.toString());
		}
		catch (Exception e) 
		{
			e.printStackTrace();
		} 
	}
	
	public License parseLicenseFile(File licenseFile)
	{
		return parseLicense(licenseFile);
	}
	
	public License parseLicenseString(String licenseStr)
	{
		return parseLicense(licenseStr);
	}
	
	private License parseLicense(Object licenseObject)
	{
		try
		{
			license = relEngine.getRELEngineSchemaHandler().getLicense();
			license.parse(new MXMAdapter(licenseObject));
			
			// TODO implement parsing to be able to get grants
			
			return license;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	public boolean storeLicense(String filePath)
	{
		try
		{
			license.createFile(new File(filePath));
			
			return true;
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
		
		return false;
	}
	
	public PropertyPossessor createPropertyPossessor(String propertyURI)
	{
		try
		{
			PropertyPossessor propertyPossessor = relEngine.getRELEngineSchemaHandler().getPropertyPossessor();
			
			PropertyUri propertyUri = relEngine.getRELEngineSchemaHandler().getPropertyUri();
			propertyUri.setDefinition(propertyURI);
			
			propertyPossessor.setPropertyAbstract(propertyUri);
			
			return propertyPossessor;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	public void setForAll(PropertyPossessor propertypossessor)
	{
		try
		{
			ForAll forAllAll = relEngine.getRELEngineSchemaHandler().getForAll();
			
			forAllAll.addAnXmlPatternAbstract(propertypossessor);
			
			this.forAll = forAllAll;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
     * This method creates a new key holder every time a user wishes to define a principal for a grant.
     * A KeyHolder principal is identified by a X509Data certificate data.
     * 
     * @param x509data Certificate of the principal.
     * @return A Principal object representing the principal.
     */
	public Principal createKeyHolder(X509Data x509data)
	{
		try
		{
			KeyInfo keyInfo = securityEngine.getSecurityEngineSchemaHandler().getKeyInfo();
			keyInfo.setX509Data(x509data);
			
			KeyHolder keyHolder = relEngine.getRELEngineSchemaHandler().getKeyHolder();
			keyHolder.setKeyInfo(keyInfo);

			return keyHolder;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	/**
     * This method enables the creation of principals that are identified by their identity. E.g.
     * defining a principal by his/her username or e-mail.
     * 
     * @param principalIdentifier The identifier of a principal.
     */
	public Principal createIdentityHolder(String principalIdentifier)
	{
		try
		{
			IdentityHolder identityHolder = relEngine.getRELEngineSchemaHandler().getIdentityHolder();
			identityHolder.addIdValue(principalIdentifier);

			return identityHolder;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	/**
     * Sets identifier of the license. If not provided a random identifier will be generated.
     * 
     * @param identifier The license identifier.
     */
	public void setLicenseIdentifier(String identifier)
	{
		this.identifier = identifier;
	}

	/**
     * This method creates a Digital Item Reference that is the Resource that is protected by the license.
     * 
     * @param diIdentifier The digital item identifier.
     */
	public Resource createDIReference(String diIdentifier)
	{
		try
		{
			DiReference diReference = relEngine.getRELEngineSchemaHandler().getDiReference();
			diReference.setIdentifier(diIdentifier);
			
			return diReference;
		} 
		catch (MXMEngineException e)
		{
			e.printStackTrace();
		}
		
		return null;
	}

	// TODO add more resources
	
	/**
     * This method creates a Digital Resource reference that is the Resource that is protected by the license.
     * 
     * @param url The url to access the resource.
     */
	public Resource createDigitalResource(String url)
	{
		try
		{
			NonSecureReference nonSecureReference = relEngine.getRELEngineSchemaHandler().getNonSecureReference();
			nonSecureReference.setURI(url);
			
			DigitalResource digitalResource = relEngine.getRELEngineSchemaHandler().getDigitalResource();
			digitalResource.setNonSecureIndirect(nonSecureReference);
			
			return digitalResource;
		} 
		catch (MXMEngineException e)
		{
			e.printStackTrace();
		}
		
		return null;
	}
	
	/**
     * This method creates a Digital Resource reference that is the Resource that is protected by the license.
     * 
     * @param url The url to access the resource.
     */
	public Resource createProtectedResource(String url, byte[] key)
	{
		try
		{
			CipherData cipherData = securityEngine.getSecurityEngineSchemaHandler().getCipherData();
			cipherData.setCipherValue(key);
			
			EncryptedKey encryptedKey = securityEngine.getSecurityEngineSchemaHandler().getEncryptedKey();
			encryptedKey.setCipherData(cipherData);
			
			// TODO kanonika mpainei tou au to keyinfo sto encryptedkey
			
			ProtectedResource protectedResource = relEngine.getRELEngineSchemaHandler().getProtectedResource();
			protectedResource.setResourceLocator(url);
			protectedResource.addEncryptedKey(encryptedKey);
			
			return protectedResource;
		} 
		catch (MXMEngineException e)
		{
			e.printStackTrace();
		}
		
		return null;
	}

	/**
	 * With this method the user may create all types of conditions, including territory and temporal conditions.
	 * The parameters for each type of condition are passed in the form of a parameter value map.
	 * 
	 * @param notBeforeDate The type of the condition.
	 * @return A Condition object representing the license condition.
	 */
	public Condition createValidityIntervalCondition(Date notBeforeDate, Date notAfterDate)
	{
		try
		{
			ConditionBuilder conditionBuilder = relEngine.getRELEngineSchemaHandler().getConditionBuilder();
			
			// TODO add Fulfiller
			// TODO add ExcersieLimit
			// TODO add FeePerUse
			// TODO add Territory
			
			DatatypeFactory dateTypeFactory = DatatypeFactory.newInstance();

			XMLGregorianCalendar notBefore = null;
			XMLGregorianCalendar notAfter = null;

			if (notBeforeDate!=null)
			{
				GregorianCalendar calendar = new GregorianCalendar();
				calendar.setTime(notBeforeDate);

				notBefore = dateTypeFactory.newXMLGregorianCalendar(calendar);
			}

			if (notAfterDate!=null)
			{
				GregorianCalendar calendar = new GregorianCalendar();
				calendar.setTime(notAfterDate);

				notAfter = dateTypeFactory.newXMLGregorianCalendar(calendar);
			}

			return conditionBuilder.buildValidityInterval(notBefore, notAfter);
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}

	/**
     * Sets the issuer of a license in case this is needed before the actual issue of the license.
     * 
     * @param issuer The issuer of the license.
     */
	public void setIssuer(Principal issuer)
	{
		try
		{
			Issuer issuerToAdd = relEngine.getRELEngineSchemaHandler().getIssuer();
			issuerToAdd.setPrincipal(issuer);
			
			this.issuer = issuerToAdd;
		} 
		catch (MXMEngineException e)
		{
			e.printStackTrace();
		}
	}

	/**
     * Adds a principal to the grants of the license.
     * 
     * @param principal The principal to add.
     */
	public void setPrincipal(Principal principal)
	{
		if (principals ==null)
		{
			principals = new ArrayList<Principal>();
		}
		
		principals.add(principal);
	}
	
	/**
     * With this method the user may add a new Grant in the license.
     * Each Grant refers to a Principal, a property possessor or a keyholder,
     * granting him a Right on the specified resource under certain conditions.
     * The list of Rights that a user may specify in the license are in agreement
     * with the REL specification and include rights to adapt, copy or play a resource.
     * 
     * @param right The right of the grant.
     * @param condition The conditions that should be met.
     */
	public void addGrant(RELRight right, Condition condition)
	{
		if (grants==null)
		{
			grants = new ArrayList<RightGroup>();
		}
		
		try
		{
			RightGroup rightGroup = createLicenseEngine.getCreateLicenseSchemaHandler().getRightGroup();
			rightGroup.setRELRight(right);
			rightGroup.setCondition(condition);
			
			grants.add(rightGroup);
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}

//	public void setForAll(List<String> properties) // TODO reenable
//	{
//		try
//		{
//			PropertyPossessor propertyPossessor = relEngine.getRELEngineSchemaHandler().getPropertyPossessor();
//			propertyPossessor.setPropertyAbstract(propertyAbstract);
//
//			ForAll forAll = relEngine.getRELEngineSchemaHandler().getForAll();
//			forAll.addAnXmlPatternAbstract(propertyPossessor);
//		}
//		catch(Exception ex)
//		{
//			ex.printStackTrace();
//		}
//	}

	/**
     * This method sets the Resource that is to be protected by the license. A Resource object must
     * be first created using the relevant methods.
     * 
     * @param resource The resource.
     */
	public void setResource(Resource resource)
	{
		this.resource = resource;
	}
	
	/**
     * This method generates a new license that contains all submitted data.
     * 
     * @return The REL license object.
     */
	public License generateLicense() // TODO move to the orchestrator
	{
		try
		{
			// get schema handlers
			CreateLicenseSchemaHandler clsh = createLicenseEngine.getCreateLicenseSchemaHandler();
			RELEngineSchemaHandler resh = relEngine.getRELEngineSchemaHandler();
			MetadataEngineSchemaHandler mesh = metadataEngine.getMetadataEngineSchemaHandler();
			SecurityEngineSchemaHandler sesh = securityEngine.getSecurityEngineSchemaHandler();
	
			CustomLicense customLicense = clsh.getCustomLicense();
	
			if (identifier==null)
			{
				// add a custom license identifier
				UniqueID uniqueId = mesh.getUniqueID();
				uniqueId.setValue(UUID.randomUUID().toString());
				customLicense.setLicenseIdentifier(uniqueId);
			}
			else
			{
				UniqueID uniqueId = mesh.getUniqueID();
				uniqueId.setValue(identifier);
				customLicense.setLicenseIdentifier(uniqueId);
			}
	
			if (forAll!=null)
			{
				customLicense.setForAll(forAll);
			}
			else
			{
				if (principals.size()>1)
				{
					AllPrincipals allPrincipals = resh.getAllPrincipals();
					
					for (Principal principal : principals)
					{
						allPrincipals.addPrincipal(principal);
					}
				}
				else
				{
					customLicense.setPrincipal(principals.get(0));
				}
			}
	
			// set the resource
			customLicense.setResource(resource);
	
			// set the rights
			for (RightGroup rightGroup : grants)
			{
				customLicense.addRightGroup(rightGroup);
			}
	
			// set the issuer
			if (issuer!=null)
			{
				customLicense.setIssuer(issuer);
			}
	
			// create the license request
			CreateLicenseRequest clr = clsh.getCreateLicenseRequest();
			clr.setCustomLicense(customLicense);
	
//			CreateLicenseProtocol clp = createLicenseEngine.getCreateLicenseTechnologyHandler().getRemoteCreateLicenseProtocol(); // TODO
			CreateLicenseProtocol clp = createLicenseEngine.getCreateLicenseTechnologyHandler().getLocalCreateLicenseProtocol();
			
//			try
//			{
//				URL serviceURL = new URL(createLicenseEngine.getPropertyValue("SERVICE_URL"));
//				QName serviceName = new QName(createLicenseEngine.getPropertyValue("SERVICE_NAME_NS"), 
//						createLicenseEngine.getPropertyValue("SERVICE_NAME"));
//				
//				clp.initServiceParams(serviceURL, serviceName);
//			} 
//			catch (MXMInvalidConfigurationException e)
//			{
//				e.printStackTrace();
//			} catch (MalformedURLException e)
//			{
//				e.printStackTrace();
//			}
	
			CreateLicenseResponse clResponse = clp.createLicense(clr);
	
//			System.out.println(clResponse.createObject(String.class.getCanonicalName()).getContent());
			
			// if success return the license
			if (clResponse.getCreateLicenseSuccess()!=null)
			{
				license = clResponse.getCreateLicenseSuccess().getLicenseEntity().getLicense();
				
				return license;
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}

		return null;
	}

	@Override
	public String getResourceUrl()
	{
		Resource resource = license.getGrants().get(0).getResource();
		
		if (resource instanceof ProtectedResource)
		{
			ProtectedResource proRes = (ProtectedResource) resource;
			
			return proRes.getResourceLocator();
		}
		
		return null;
	}

	@Override
	public byte[] getEncryptedKey()
	{
		Resource resource = license.getGrants().get(0).getResource();
		
		if (resource instanceof ProtectedResource)
		{
			ProtectedResource proRes = (ProtectedResource) resource;
			
			return proRes.getEncryptedKeyList().get(0).getCipherData().getCipherValue();
		}
		
		return null;
	}

	@Override
	public List<String> getPropertyPossessorRights(String propertyURI)
	{
		List<String> propertyPossessorRights = new ArrayList<String>();
		
		for (Grant grant : license.getGrants())
		{
			boolean checkGrant = false;
			
			for (AnXmlPatternAbstract patternAbstract : grant.getForAll().get(0).getAnXmlPatternAbstract())
			{
				if (patternAbstract instanceof PropertyPossessor)
				{
					PropertyPossessor propertyPossessor = (PropertyPossessor) patternAbstract;
					
					PropertyAbstract propertyAbstract = propertyPossessor.getPropertyAbstract();
					
					if (propertyAbstract instanceof PropertyUri)
					{
						if (((PropertyUri) propertyAbstract).getDefinition().equals(propertyURI))
						{
							checkGrant = true;
						}
					}
				}
			}
			
			if (checkGrant)
			{
				propertyPossessorRights.add(grant.getRELRight().toString());
			}
		}
		
		return propertyPossessorRights;
	}

	@Override
	public List<String> getPropertyPossessors()
	{
		List<String> propertyPossessors = new ArrayList<String>();
		
		for (Grant grant : license.getGrants())
		{
			for (AnXmlPatternAbstract patternAbstract : grant.getForAll().get(0).getAnXmlPatternAbstract())
			{
				if (patternAbstract instanceof PropertyPossessor)
				{
					PropertyPossessor propertyPossessor = (PropertyPossessor) patternAbstract;
					
					PropertyAbstract propertyAbstract = propertyPossessor.getPropertyAbstract();
					
					if (propertyAbstract instanceof PropertyUri)
					{
						propertyPossessors.add(((PropertyUri) propertyAbstract).getDefinition());
					}
				}
			}
		}
		
		return propertyPossessors;
	}
	
//	/**
//     * This method processes the license and checks the privileges of a principal on a certain resource.
//     * For this, the user provides a principal, a right and a resourceRef. Then the tool, employing the
//     * AuthorizationManager of the REL TE, checks the rights that are granted to the principal, evaluating
//     * also real-time conditions, such as territory, and temporal conditions. The tool returns an
//     * AuthorizationResponse to the user.
//     * 
//     * @return The result of the evaluation of the license.
//     */
//	public boolean checkLicense(KeyHolder keyholder, RELRight right, String resourceId) // TODO
//	{
//		return false;
//	}
	
//	/**
//  * This method verifies the validity of the license.
//  * The tool employs REL TE to process the license and check its integrity and REL validity.
//  * 
//  * @return The digital signature of the license.
//  */
//	public boolean verifyLicense()
//	{
//		return false;
//	}
	
	
//	/**
//     * This method returns the issuer of a license.
//     * 
//     * @return The issuer of the license.
//     */
//	public Issuer getIssuer()
//	{
//		return null;
//	}
//	
//	/**
//     * Returns a list with all the grants stated in the license.
//     * 
//     * @return The grants stated in the license.
//     */
//	public List<Grant> getGrantList()
//	{
//		return null;
//	}
//	
//	/**
//     * Returns the signature of a license.
//     * The user may later check the signature of the license by calling the SecurityTE.
//     * 
//     * @return The digital signature of the license.
//     */
//	public Signature getSignature()
//	{
//		
//		return null;
//	}

}
