package eu.convergence.middleware.tool.revoke;

import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
//import org.iso.mpeg.mxm.engine.digitalitemTE.schema.Metadata;
//import org.iso.mpeg.mxm.engine.digitalitemTE.schema.StructuredData;
//import org.iso.mpeg.mxm.engine.metadataTE.ContentEntity;
//import org.iso.mpeg.mxm.engine.metadataTE.TextAnnotation;
//import org.iso.mpeg.mxm.engine.metadataTE.Textual;
//import org.iso.mpeg.mxm.engine.metadataTE.schema.DescriptionMetadata;
//import org.iso.mpeg.mxm.engine.metadataTE.schema.Mpeg7;
import org.iso.mpeg.mxm.ipmpTE.IPMPTE;

import eu.convergence.middleware.core.ConvergenceEngineName;
//import eu.convergence.middleware.metadataTE.GenericMetadataTE;
//import eu.convergence.middleware.metadataTE.MetadataKinds;
//import eu.convergence.middleware.metadataTE.MetadataTE;
import eu.convergence.middleware.orchestratorTE.ConvergenceOrchestratorTE;
//import eu.convergence.middleware.orchestratorTE.builder.MetadataOrchestrator;
//import eu.convergence.middleware.tool.vdi.ResourceVdiTool;
import eu.convergence.middleware.vdiTE.VDITE;

//import eu.convergence.middleware.revokePE.RevokePETest;
import eu.convergence.middleware.revokePE.methods.revokeFromConet;
import eu.convergence.middleware.revokePE.methods.unpublishFromComid;
import eu.convergence.middleware.revokePE.utils.*;
import eu.convergence.middleware.revokePE.workflow.*;

/**
 * The RevokeTool allows the user to revoke previous content from the CONET and COMID.
 * It implements the digital forgetting mechanisms. 
 */
public class RevokeTool 
{
	private static RevokeTool REVOKE_TOOL;
	
	private revokeFromConet REVOKE_PE = null;
	private unpublishFromComid Unpublish_PE = null;
	
	private Logger logger = Logger.getLogger(RevokeTool.class);
	private final String TEMP_FOLDER = System.getProperty("java.io.tmpdir");
	private final String SOURCE_DCI_FILE_NAME = TEMP_FOLDER + "testDCI.xml";
	
	private final String TEMP_FOLDER2 = getClass().getClassLoader().getResource(".").getPath();
	private final String OUTPUT_MXM_FILE_NAME = TEMP_FOLDER + "MXMConfiguration.xml";

	private final String SOURCE_DCI_FILE_NAME2 = TEMP_FOLDER2 + "testDCI.xml";
	
	/** Get a new instance of revokeTool */
	public static RevokeTool getInstance()
	{
		if (REVOKE_TOOL==null)
		{
			REVOKE_TOOL= new RevokeTool();
		}
		
		return REVOKE_TOOL;
	}
	
		
	/**
     * Returns the result of the revoke operation from Conet. The associated Content VDI will be eliminated from
     * the Convergence system. 
     * 
     * List of dependencies
     * CONET TE -> in the mxmconfiguration.xml file the url of CONET shall be provided:
     * <EngineParameters>
        <entry key="CONET_SERVLET_BASE_URL">http://160.80.82.69:8080/conet/</entry>
       </EngineParameters> 
	 *
     * @param dciFilePathName The path of the DCI file.
     * @param vdiID The ID of the VDI.
     * @param nid The NID object to be revoked.
     * @return indication of success or failure
     */
	public boolean revokeFromConetTool(String dciFilePathName, String vdiID, String nid) throws Exception {
	{		
		logger.info("\n\n---------- Calling revokeFromConet Methods ----------");
		revokeFromConet rvk = new revokeFromConet();
		
		String tempFilePathName = TEMP_FOLDER2;
		System.out.println("Temp path is: " + tempFilePathName);
				
		//Calling the revoke from CONET operation
		if (rvk.revokeFromConetProcess(nid)==true) {
			logger.info("\n\n---------- Revoke from CONET succeeded ----------");
			return true;}
		else { 
			logger.info("\n\n---------- Revoke from CONET failed ----------");
			return false;}}
	}
	
	
	/**
     * Returns the result of the unpublish from Comid operation. The vdiID is sent to the COMID framework. 
	 *
	 * List of dependencies
	 * VDI TE -> to manipulate and extract Digital Item
	 * Overlay Engine -> to gossip the unpublish VDI through the fractals
	 *
     * @param vdiID The ID of the VDI.
     * @return indication of success or failure
     */
	public boolean unpublishFromComidTool(String vdiID) throws Exception {
	{		
		logger.info("\n\n---------- Calling unpublishFromComid Methods ----------");
		unpublishFromComid upb = new unpublishFromComid();
				
		//Calling the revoke from CONET operation
		if (upb.unpublishFromComid(vdiID) == true) {
			logger.info("\n\n---------- Unpublish from Comid succeeded ----------");
			return true;}
		else { 
			logger.info("\n\n---------- Unpublish from Comid failed ----------");
			return false;}
	}
	
	
	}
	
}
