package eu.convergence.middleware.tools.monitoring;

import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.engine.vdiTE.VdiEngine;
import eu.convergence.middleware.event.VDIArrivalEvent;
import eu.convergence.middleware.orchestratorTE.ConvergenceOrchestratorTE;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
import org.iso.mpeg.mxm.core.event.management.MXMEvent;
import org.iso.mpeg.mxm.core.event.management.MXMEventHandler;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.exceptions.DIEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

/**
 * Created by IntelliJ IDEA.
 * User: aca
 * Date: 9/3/11
 * Time: 1:19 PM
 */
public class MonitorVDIArrivals implements org.iso.mpeg.mxm.core.event.management.MXMEventListener
{
    private String monitoringAppURL;

    public MonitorVDIArrivals(String monitoringAppURL)
    {
        this.monitoringAppURL = monitoringAppURL;
        try {
            MXM.getInstance().getMXMEventHandler().registerMXMEventListener(VDIArrivalEvent.class.getName(), this, MXMEventHandler.HIGH_PRIORITY);
        } catch (MXMConfigurationException e) {
            e.printStackTrace();
        }
    }
    @Override
    public void mxmEventOccurred(MXMEvent mxmEvent)
    {
        if (mxmEvent instanceof VDIArrivalEvent)
        {
            VDIArrivalEvent vdiArrivalEvent = (VDIArrivalEvent) mxmEvent;
            try {
            	ConvergenceOrchestratorTE orchestrator = (ConvergenceOrchestratorTE) MXM.getInstance().getDefaultEngine(MXMEngineName.Orchestrator.value());
            	VdiEngine vdiEngine = (VdiEngine) MXM.getInstance().getDefaultEngine(MXMEngineName.DigitalItemTE.value());
            	
            	
                String overlayPeerId = orchestrator.getOverlayOrchestrator().getOverlayPeerId();
                if (vdiArrivalEvent.getVdiPayload().getContentType().equals(new byte[0].getClass().getCanonicalName()))
                {
                    byte[] byteArray = (byte[]) vdiArrivalEvent.getVdiPayload().getContent();
                    
                    ByteArrayInputStream bais = new ByteArrayInputStream(byteArray);
                    String vdiId = new String();
                    try {
                    	DIDL didl = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
                    	didl.parse(new MXMAdapter(bais));
                    	
                        vdiId = didl.getMPEGMItem().getIdentifier();
                    } catch (MXMException e) {
                        e.printStackTrace();
                    }

                    String fractalNames = new String();
                    String[] eventFractalNames = vdiArrivalEvent.getFractalName();
                    for (String str : eventFractalNames)
                    {
                        fractalNames += str + " ";
                    }
                    // remove the last whitespace
                    String finalFractalNamesStr = fractalNames.substring(0, fractalNames.lastIndexOf(" "));
                    System.out.println("Notifying monitoring application for VDI Arrival");

                    URL url = null;
                    if (vdiArrivalEvent.getVdiType() == VDIArrivalEvent.PUBLICATION_VDI)
                    {
                        url = new URL(monitoringAppURL + "/MessageArrivedServlet" +
                                "?messageType=PUBLICATION_MESSAGE" +
                                "&vdiID=" + vdiId +
                                "&overlayPeerId=" + overlayPeerId +
                                "&fractalName=" + finalFractalNamesStr
                        );
                    }
                    else if (vdiArrivalEvent.getVdiType() == VDIArrivalEvent.SUBSCRIPTION_VDI)
                    {
                        url = new URL(monitoringAppURL + "/MessageArrivedServlet" +
                                "?messageType=SUBSCRIPTION_MESSAGE" +
                                "&vdiID=" + vdiId +
                                "&overlayPeerId=" + overlayPeerId +
                                "&fractalName=" + finalFractalNamesStr
                        );
                    }

                    if (url == null)
                    {
                        System.err.println("VDIArrivalMonitoringListener: Cannot determine message type...");
                    }
                    else
                    {
                    	System.out.println("Contacting: " + url.getPath());
                        URLConnection urlConnection = url.openConnection();
                        BufferedReader reader = new BufferedReader(
                                new InputStreamReader(urlConnection.getInputStream())
                        );
                        String response;

                        while ((response = reader.readLine()) != null)
                        {
                            System.out.println(response);
                        }

                        reader.close();
                    }
                }
            } catch (OverlayEngineException e) {
                e.printStackTrace();
            } catch (MalformedURLException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            catch (MXMException e)
			{
				e.printStackTrace();
			}
        }
    }
}
