package eu.convergence.middleware.tools.monitoring;

import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.event.VDIMatchEvent;
import eu.convergence.middleware.orchestratorTE.ConvergenceOrchestratorTE;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
import org.iso.mpeg.mxm.core.event.management.MXMEvent;
import org.iso.mpeg.mxm.core.event.management.MXMEventHandler;
import org.iso.mpeg.mxm.core.event.management.MXMEventListener;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Date;

/**
 * Created by IntelliJ IDEA.
 * User: aca
 * Date: 9/3/11
 * Time: 2:16 PM
 */
public class MonitorVDIMatches implements MXMEventListener
{
    private String monitoringAppURL;

    public MonitorVDIMatches(String monitoringAppURL)
    {
        this.monitoringAppURL = monitoringAppURL;

        try {
            MXM.getInstance().getMXMEventHandler().registerMXMEventListener(VDIMatchEvent.class.getName(), this, MXMEventHandler.HIGH_PRIORITY);
        } catch (MXMConfigurationException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void mxmEventOccurred(MXMEvent mxmEvent)
    {
        if (mxmEvent instanceof VDIMatchEvent)
        {
            VDIMatchEvent vdiMatchEvent = (VDIMatchEvent) mxmEvent;
            String pubVDIId = vdiMatchEvent.getPublicationVdiId();
            String subVDIId = vdiMatchEvent.getSubscriptionVdiId();
            try {
            	ConvergenceOrchestratorTE orchestrator = (ConvergenceOrchestratorTE) MXM.getInstance().getDefaultEngine(MXMEngineName.Orchestrator.value());
            	
            	String overlayPeerId = orchestrator.getOverlayOrchestrator().getOverlayPeerId();
                Date now = new Date();

                URL url = new URL(monitoringAppURL + "/MessagesMatchedServlet" +
                        "?matchTime=" + now.getTime() +
                        "&pubVDIId=" + pubVDIId +
                        "&subVDIId=" + subVDIId +
                        "&overlayPeerId=" + overlayPeerId
                );

                URLConnection urlConnection = url.openConnection();
                BufferedReader reader = new BufferedReader(
                        new InputStreamReader(urlConnection.getInputStream())
                );
                String response;
                System.out.println("Contacting: " + url.getPath());
                while ((response = reader.readLine()) != null)
                {
                    System.out.println(response);
                }

                reader.close();
            } catch (MalformedURLException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (OverlayEngineException e) {
                e.printStackTrace();
            }
            catch (MXMException e)
			{
				e.printStackTrace();
			}
        }

    }
}
