package eu.convergence.middleware.tool.er.test;

import eu.convergence.middleware.conetTe.ConetTE;
import eu.convergence.middleware.engine.ConetTe.DummyConeTE;
import eu.convergence.middleware.eventreportTE.kernel.conet.EventReportTE;
import eu.convergence.middleware.tool.er.EventReportTool;
import junit.framework.TestCase;
import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EventTriggerType;

import java.io.File;

/**
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 11/11/11
 * Time: 9:34 PM
 * <p/>
 * UTI Systems, TIC
 */
public class EventReportToolTest extends TestCase {
    private Logger logger = Logger.getLogger(EventReportToolTest.class);

    private EventReportTool eventReportTool;

    private EventReportEngine erEngine;

    @Override
    public void setUp() throws Exception {
        logger.info("\n--- Setup MXMConfiguration ---\n");
        File f = new File(getClass().getClassLoader().getResource("MXMConfiguration_EventReport.xml").getFile());
        MXM mxm = MXM.getInstance(f);
        mxm.listEnginesAndProperties();

        EventReportTE erTE = (EventReportTE) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());
        ConetTE dummyConeTE = DummyConeTE.getInstance();
        erTE.configureCoNet(dummyConeTE, dummyConeTE);
        erEngine = erTE;

        logger.info("\n--- Init EventReportEngine - Done! (peer=" +
                erEngine.getEventReportEngineTechHandler().getEREngineParams().getPeerId() + ") ---\n");


        eventReportTool = EventReportTool.getInstance();
    }

    public void testERRCreation() throws Exception {
        logger.info("\n\n---------- test ERR creation ----------");
        ERR err = eventReportTool.createERR(EventTriggerType.MATCH);
        logger.info("create ERR with id = " + err.getERRIdentifier());


        logger.info("\n\n---------- test ERR parsing ----------");
        String errIdentifier = eventReportTool.getERRIdentifier(err);
        logger.info("parse ERR Identifier: " + errIdentifier);


        logger.info("\n\n---------- ER creation (when a match publication event occurs) ----------");
        ER er = erEngine.getEventReportEngineSchemaHandler().getER();
        er.setERRIdentifier(errIdentifier);
        er.setDIIId("didl-" + System.currentTimeMillis());

        logger.info("\n\n---------- test ER parsing ----------");
        errIdentifier = eventReportTool.getERRIdentifier(er);
        logger.info("found ERR Identifier: " + errIdentifier);
        String P_VDI_Id = eventReportTool.getDIDLIId(er);
        logger.info("found P-VDI Id: " + P_VDI_Id);
    }


}
