package eu.convergence.middleware.tool.revoke.test;

import java.io.File;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.orchestratorEngine.unpublishcontent.UnpublishContentEngineOrchestrator;
import org.iso.mpeg.mxm.pEngine.unpublishcontentPE.UnpublishContentPE;
import org.iso.mpeg.mxm.pEngine.unpublishcontentPE.schemahandler.UnpublishContentRequest;
import org.iso.mpeg.mxm.pEngine.unpublishcontentPE.schemahandler.UnpublishContentResponse;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;

import eu.convergence.middleware.tool.revoke.RevokeTool;


/** Class responsible for the implemention of tests in Revoke Tool */
public class RevokeToolTest extends TestCase
{

	private Logger logger = Logger.getLogger(RevokeToolTest.class);
	
	private final String TEMP_FOLDER = System.getProperty("java.io.tmpdir") + File.separator;
	//private final String SOURCE_DCI_FILE_NAME = TEMP_FOLDER + "testDCI.xml";
	
	private final String TEMP_FOLDER2 = getClass().getClassLoader().getResource(".").getPath();
	//private final String OUTPUT_MXM_FILE_NAME = TEMP_FOLDER + "MXMConfiguration.xml";

	private final String SOURCE_DCI_FILE_NAME = TEMP_FOLDER2 + "testDCI.xml";
	
	
	/** Main method to test Revoke Tool */
	public void testRevokeTool()
	{
		try
		{
			logger.info("\n\n---------- testRevokeFromConet ----------");
			
			File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
			MXM mxm = MXM.getInstance(f);
			mxm.listEnginesAndProperties();
			
			String dciFilePathName = SOURCE_DCI_FILE_NAME;
			System.out.println("dciFilePathName is: " + dciFilePathName);
			String vdiID = "urn:eu:convergence:server1/66096a69-3b80-45e8-b340-9b7b7b7d3f63";
			//String nid = "cubo.jpg";
			String nid = "file:/var/folders/jq/jqBGHUw4G0C1IlAPh04Nrk+++TI/-Tmp-/upload-00fe53b0-b561-4143-aee1-2a9a0b628610-video-3994725604289010107.mp4";
			
			logger.info("\n\n---------- Calling Revoke Methods ----------");
			RevokeTool rvkTool = new RevokeTool();
						
			//Calling the revoke from CONET operation
			if (rvkTool.revokeFromConetTool(dciFilePathName, vdiID, nid)==true)
				logger.info("\n\n---------- Revoke from CONET succeeded ----------");
			else 
				logger.info("\n\n---------- Revoke from CONET failed ----------");
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		
		try
		{
			logger.info("\n\n---------- testUnpublishFromComid ----------");
			
			File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
			MXM mxm = MXM.getInstance(f);
			mxm.listEnginesAndProperties();
			
			
			System.out.println("PG2 - Creation of a new MXM Engine");
			MXMEngine mxmEngine = mxm.getDefaultEngine(MXMEngineName.UnpublishContentPE.toString());
			System.out.println("PG3 - Creation of a new UnpublishContent PE");
			UnpublishContentPE ccpEngine = (UnpublishContentPE) mxm.getDefaultEngine(MXMEngineName.UnpublishContentPE.toString());
			System.out.println("PG4 - Creation of a new DI Engine");
			DIEngine diEngine = (DIEngine) mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());

			System.out.println("PG5 - Creation of a new UnpublishContentequest");
			UnpublishContentRequest ccpR=ccpEngine.getSchemaHandler().getUnpublishContentRequest();
			
			//ccpR.parse(new MXMAdapter(new File(getClass().getClassLoader().getResource("testPKG.xml").getFile())));
				
			//PackageContentEngineOrchestrator cceo=new org.iso.mpeg.mxm.orchestrator.packagecontent.PackageContentEngineOrchestrator(ccpEngine, diEngine);
			System.out.println("PG6 - Creation of a new UnpublishContent Engine Orchestrator");
			UnpublishContentEngineOrchestrator cceo=new org.iso.mpeg.mxm.orchestrator.unpublishcontent.UnpublishContentEngineOrchestrator(ccpEngine, diEngine);
								
			System.out.println("PG7 - Creation of a new Unpublish Content Response");
			UnpublishContentResponse pcr=cceo.unpublishContent(ccpR);
			
			//pcr.getPackageContentSuccess().addSuccessCode("1");
			pcr.getUnpublishContentSuccess();
					
			//ccpR2.getCreateContentSuccess().getIdentifiedContent().getDIDL();		
			System.out.println(ccpR.createObject((String.class.getCanonicalName())).getContent());
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			
			/*
			String vdiID = "001";
			
			logger.info("\n\n---------- Calling Unpublish Methods ----------");
			RevokeTool rvkTool = new RevokeTool();
						
			//Calling the revoke from CONET operation
			if (rvkTool.unpublishFromComidTool(vdiID)==true)
				logger.info("\n\n---------- Unpublish from Comid succeeded ----------");
			else 
				logger.info("\n\n---------- Unpublish from Comid failed ----------");
			*/
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}	
		
	}
	
	/** Main default method */
	public static void main(String[] args) throws MXMConfigurationException, MXMEngineException
	{
		new RevokeToolTest();
	}
	
}
