package eu.convergence.middleware.tool.security.test;




import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.KeyPair;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.util.Date;
import java.util.Random;

import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.security.cert.CertificateException;
import javax.security.cert.X509Certificate;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;
import morpho.util.Converter;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.securityTE.schema.SecurityEngineContext;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.iso.mpeg.mxm.securityTE.technologyhandler.obj.EncryptedDataObj;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager;
import org.xml.sax.SAXException;

import eu.convergence.middleware.tool.security.SecurityTool;


public class SecurityToolTest extends TestCase
{
    private final String     authReference   = "CN=Issuer,L=London,C=GB,O=Organization";
    private EncryptedDataObj encObject       = null;
    private final int        expiryDays      = 10;
    private final String     holderReference = "CN=Holder,L=London,C=GB,O=Organization";
    private JAXBContext      jc              = null;
    private final byte       keyIdentifier   = 0;
    private final int        keyLength       = 2048;
    private static final String APP_ALIAS1 = "1";
    // private static final String KEY_IDENTIFIER_SIG = "sig";
    // private static final String PIN_GAKP = "12345678";
    // private static final String PIN_SIG = "123456";
    private final Logger     logger          = Logger.getLogger(SecurityToolTest.class);
    private Marshaller       m               = null;
    private final BigInteger PubExp          = new BigInteger("10001", 16);
    private SecurityEngine   securityTE      = null;
    private SecurityTool     securityTool    = null;
    private final String     SignOID         = "SHA256withRSAEncryption";

    @Override
    public void setUp() throws Exception
    {
        super.setUp();
        // Initialize JAXB Context.
        this.jc = SecurityEngineContext.getInstance().getJAXBContext();
        this.m = this.jc.createMarshaller();
        this.m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
        this.m.setProperty("com.sun.xml.bind.namespacePrefixMapper",
            SchemaNamespacePrefixMapperImpl.getInstance());
        // Initialize MXM.
        this.logger.info("\n--- Setup MXMConfiguration ---");
        final File f = new File(this.getClass().getClassLoader()
            .getResource("MXMConfiguration.xml").getFile());
        final MXM mxm = org.iso.mpeg.mxm.core.MXM.getInstance(f);
        mxm.listEnginesAndProperties();
        this.logger.info("\n--- Init SecurityEngine ---");
        this.securityTE = (SecurityEngine)mxm.getDefaultEngine(MXMEngineName.SecurityTE
            .toString());
        this.securityTE.setProperty("conf", "223456");
        this.securityTE.setProperty("auth", "323456");
        this.securityTE.setProperty("sig", "123456");
        this.securityTE.setProperty("gakp", "12345678");
        this.securityTE.setProperty("appAlias", APP_ALIAS1);
        this.logger.info("\n--- Done! ---");
        this.logger.info("\n--- Init SecurityTool ---");
        this.securityTool = SecurityTool.getInstance();
        this.logger.info("\n--- Done! ---");
        
    }

    public void testSecurityToolParsing() throws MXMConfigurationException,
        MXMEngineException
    {
        SecretKey secretKetObj = SecurityTool.getInstance().generateSymmetricKey(128);
        String dataToEnc = "This is data to be encrypted";
       
        System.out.println("DATA TO ENCRYPT:"+dataToEnc);
        
        System.out.println("-------------------------------------------------------------------------");
        System.out.println("GENERATING KEY PAIR");
        System.out.println("-------------------------------------------------------------------------");
        System.out.println("generating auth key");
        PublicKey publickEY = SecurityTool.getInstance().generateKeyPair("auth",1024);
        System.out.println("generating sig key");
        PublicKey publickEY1 = SecurityTool.getInstance().generateKeyPair("sig",1024);
        System.out.println("generating conf key");
        PublicKey publickEY2 = SecurityTool.getInstance().generateKeyPair("conf",1024);
        
        
       
      
        System.out.println("-------------------------------------------------------------------------");
        System.out.println("TESTING AES ENCRYPTION");
        System.out.println("-------------------------------------------------------------------------");
        byte[] aesenc=null;
        aesenc = SecurityTool.getInstance().symmetricEncrypt(dataToEnc.getBytes(), secretKetObj);
        System.out.println("\nAES ENCRYPTION::"+new String(aesenc));
        byte[] aesDec = null;
        aesDec = SecurityTool.getInstance().symmetricDecrypt(aesenc,secretKetObj);
        System.out.println("\nAES DECRYPTION:"+new String(aesDec));
        System.out.println("-------------------------------------------------------------------------");
        System.out.println("TESTING Wrapping");
        System.out.println("-------------------------------------------------------------------------");
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrappedKey = SecurityTool.getInstance().wrap(secretKetObj);
        System.out.println("-------------------------------------------------------------------------");
        System.out.println("TESTING UnWrapping");
        System.out.println("-------------------------------------------------------------------------");
        SecretKey unwrapped = SecurityTool.getInstance().unWrap(wrappedKey);
        System.out.println("-------------------------------------------------------------------------");
        System.out.println("TESTING decrypting with unwrapped key");
        System.out.println("-------------------------------------------------------------------------");
        byte[] aesDec1 = SecurityTool.getInstance().symmetricDecrypt(aesenc,unwrapped);
        System.out.println("\nAES DECRYPTION:"+new String(aesDec));
        System.out.println("Checking original secket key decryption and unwrapped key decryption");
        assertEquals(new String(aesDec), new String(aesDec1));
        System.out.println("verified");
        
        
        System.out.println("\n--- signing and verifying  xml document ---");
        final File f = new File(this.getClass().getClassLoader()
          .getResource("testSec.xml").getFile());
        DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = null;
        org.w3c.dom.Document doc = null; 
            try
            {
                docBuilder = dbfac.newDocumentBuilder();
                doc = docBuilder.parse(f.getAbsoluteFile());
            }
            catch (ParserConfigurationException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            catch (SAXException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            catch (IOException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
           
            Signature sign = SecurityTool.getInstance().generateSignature(doc);
            boolean verified = SecurityTool.getInstance().verifySignature(doc, sign);
            assertTrue(verified);
            System.out.println("document verified: "+verified);
            String str = Math.random()*100000 + "";
            try
            {
                sign = SecurityTool.getInstance().generateSignature(str.getBytes());
                verified = SecurityTool.getInstance().verifySignature(str.getBytes(), sign);
            }
            catch (Exception e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            
            assertTrue(verified);
            System.out.println("message verified: "+verified);
//        byte[] wrappedKey=null;
//            System.out.println("wrapping secret key with publickey to get wrapped key");
//            wrappedKey = SecurityTool.getInstance().hybridEncrypt(secretKetObj.getEncoded());
//        System.out.println("DECRYPTION SAME THING WITH WRAPPED KEY");
//        byte[] wrappedDec=null;
//        
//            byte []tmp = SecurityTool.getInstance().hybridDecrypt(wrappedKey);
//            System.out.println("Wraped key decrypted with private key as:"+new String(tmp));
//            //wrappedDec = enc.decrypt(aesenc, wrappedKey);
//        
//        
//        System.out.println("Encrypted data decrypted as:"+new String(wrappedDec));
    }
}