package eu.convergence.middleware.env.workflow;

import java.io.File;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import org.drools.KnowledgeBase;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.definition.KnowledgePackage;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;

public class WorkflowExecutor {
	private Collection<ProcessInstance> loadedInstances;
	
	public Map<String, Object> executeWorkflow(Map<String, Object> parameters, String mainProcessId, File...bpmnFileResources)
	{
		KnowledgeBase kbase = readKnowledgeBase(bpmnFileResources);
		StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
		KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory.newFileLogger(ksession, "workflow.log");
		
		ksession.startProcess(mainProcessId, parameters);
		
		loadedInstances = ksession.getProcessInstances();
		
		logger.close();
		
		return parameters;
	}
	
	private static KnowledgeBase readKnowledgeBase(File...bpmnResources)
	{
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		for (File bpmnResource : bpmnResources)
		{
			kbuilder.add(ResourceFactory.newFileResource(bpmnResource), ResourceType.BPMN2);
		}
		Collection<KnowledgePackage> knowledgePackages = kbuilder.getKnowledgePackages();
		Iterator<KnowledgePackage> iterator = knowledgePackages.iterator();
		while (iterator.hasNext())
		{
			KnowledgePackage knowledgePackage = iterator.next();
			System.out.println("Have process: " + knowledgePackage.getName() + ": " + knowledgePackage.toString());
		}
		return kbuilder.newKnowledgeBase();
	}
	
	public Collection<ProcessInstance> getLoadedProcessInstances()
	{
		return loadedInstances;
	}
}
