/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/securityengine/CertificateManager.java $
 * 
 * Last submit:
 *  $Revision: #2 $
 *  $Change: 365799 $
 *  $DateTime: 2012/06/01 14:17:13 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.securityengine;


import java.math.BigInteger;
import java.security.InvalidAlgorithmParameterException;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAKeyGenParameterSpec;
import java.security.spec.RSAPublicKeySpec;
import java.util.Date;

import morpho.cryptomodule.convergence.CommandDispatcher;
import morpho.cryptomodule.convergence.NoCardInUseException;
import morpho.securityengine.exceptions.SecurityEngineException;

import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.cert.X509CertificateHolder;
import org.bouncycastle.cert.X509v3CertificateBuilder;
import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
import org.bouncycastle.cert.jcajce.JcaX509v3CertificateBuilder;
import org.bouncycastle.operator.ContentSigner;
import org.bouncycastle.operator.OperatorCreationException;
import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;


public class CertificateManager
{
    private final CommandDispatcher cryptoModuleManager;

    public CertificateManager(final CommandDispatcher cryptoModuleManager)
    {
        this.cryptoModuleManager = cryptoModuleManager;
    }

    public X509Certificate generateCertificate(final BigInteger serialNumber,
        final String holderReference, final String issuerReference, final int expiryDays,
        final PublicKey pubKey, final PrivateKey privKey, final String SignOID,
        final String globalID) throws SecurityEngineException
    {
        X509Certificate resultCertificate = null;
        ContentSigner contentSigner = null;
        try
        {
            // TODO check that globalID is of length 32 bytes.
            final X500Name issuer = new X500Name(issuerReference);
            final X500Name subject = new X500Name(holderReference + ",UID=" + globalID);
            final Date currentTime = new Date(System.currentTimeMillis());
            final Date expiryTime = new Date(currentTime.getTime()
                + (1000L * 60 * 60 * 24 * expiryDays));
            final X509v3CertificateBuilder v3CertGen = new JcaX509v3CertificateBuilder(
                issuer, serialNumber, currentTime, expiryTime, subject, pubKey);
            // TODO user id in reference packen
            // Add the CONVERGENCE global ID to the certificate as extension.
            // X509Extension subjectKeyIdentifier = new X509Extension(false,
            // ASN1OctetString.getInstance(globalID));
            // SubjectKeyIdentifierStructure subjectKeyIdentifier = new
            // SubjectKeyIdentifierStructure();
            // SubjectAlternativeName subjectAlternativeName
            // v3CertGen = v3CertGen.addExtension(X509Extension.subjectKeyIdentifier,
            // false,
            // subjectKeyIdentifier);
            contentSigner = new JcaContentSignerBuilder("SHA256withRSAEncryption")
                .setProvider("BC").build(privKey);
            final X509CertificateHolder x509CertificateHolder = v3CertGen
                .build(contentSigner);
            resultCertificate = new JcaX509CertificateConverter().setProvider("BC")
                .getCertificate(x509CertificateHolder);
        }
        catch (final OperatorCreationException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final CertificateException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return resultCertificate;
    }

    // public X509Certificate generateCertificate(final String holderReference,
    // final String issuerReference, final int expiryDays, final PublicKey pubKey,
    // final PrivateKey privKey, final byte keyIdentifier, final String SignOID)
    // throws SecurityEngineException
    // {
    // // TODO use the card to do the job using the keyIdentifier
    // X509Certificate resultCertificate = null;
    // //
    // // issuer
    // //
    // final X500Name issuer = new X500Name(issuerReference);
    // //
    // // subject
    // //
    // final X500Name subject = new X500Name(holderReference);
    // //
    // // create the certificate - version 3
    // //
    // final Date currentTime = new Date(System.currentTimeMillis());
    // final Date expiryTime = new Date(currentTime.getTime()
    // + (1000L * 60 * 60 * 24 * expiryDays));
    // final X509v3CertificateBuilder v3CertGen = new JcaX509v3CertificateBuilder(
    // issuer, BigInteger.valueOf(20), currentTime, expiryTime, subject, pubKey);
    // //
    // // add extensions
    // //
    // // v3CertGen.addExtension(null, false, null);
    // ContentSigner contentSigner;
    // try
    // {
    // contentSigner = new JcaContentSignerBuilder(SignOID).setProvider("BC").build(
    // privKey);
    // resultCertificate = new JcaX509CertificateConverter().setProvider("BC")
    // .getCertificate(v3CertGen.build(contentSigner));
    // }
    // catch (final OperatorCreationException e)
    // {
    // throw new SecurityEngineException(e.getMessage());
    // }
    // catch (final CertificateException e)
    // {
    // throw new SecurityEngineException(e.getMessage());
    // }
    // return resultCertificate;
    // }
    public X509Certificate generateCVCertificate(final String holderReference,
        final String issuerReference, final int expiryDays, final PublicKey pubKey,
        final PrivateKey privKey, final String SignOID) throws SecurityEngineException
    {
        X509Certificate resultCertificate = null;
        //
        // issuer
        //
        final X500Name issuer = new X500Name(issuerReference);
        //
        // subject
        //
        final X500Name subject = new X500Name(holderReference);
        //
        // create the certificate - version 3
        //
        final Date currentTime = new Date(System.currentTimeMillis());
        final Date expiryTime = new Date(currentTime.getTime()
            + (1000L * 60 * 60 * 24 * expiryDays));
        final X509v3CertificateBuilder v3CertGen = new JcaX509v3CertificateBuilder(
            issuer, BigInteger.valueOf(20), currentTime, expiryTime, subject, pubKey);
        //
        // add extensions
        //
        // v3CertGen.addExtension(null, false, null);
        ContentSigner contentSigner;
        try
        {
            contentSigner = new JcaContentSignerBuilder(SignOID).setProvider("BC").build(
                privKey);
            resultCertificate = new JcaX509CertificateConverter().setProvider("BC")
                .getCertificate(v3CertGen.build(contentSigner));
        }
        catch (final OperatorCreationException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final CertificateException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return resultCertificate;
    }

    public KeyPair generateRSAKeyPair(final int keyLength, final BigInteger PubExp)
        throws SecurityEngineException
    {
        RSAKeyGenParameterSpec rsaParams;
        KeyPairGenerator pairGen;
        KeyPair pair;
        try
        {
            rsaParams = new RSAKeyGenParameterSpec(keyLength, PubExp);
            pairGen = KeyPairGenerator.getInstance("RSA", "BC");
            final SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
            pairGen.initialize(rsaParams, random);
            pair = pairGen.generateKeyPair();
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidAlgorithmParameterException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return pair;
    }

    // public KeyPair generateRSAKeyPairCard(final byte keyIdentifier, final int
    // keyLength,
    // final BigInteger PubExp) throws SecurityEngineException
    // {
    // RSAKeyGenParameterSpec rsaParams;
    // KeyPairGenerator pairGen;
    // KeyPair pair = null;
    // try
    // {
    // // TODO use card here instead of the PC version using the keyIdentifier
    // // this.cryptoModuleManager.doNothingForNow();
    // rsaParams = new RSAKeyGenParameterSpec(keyLength, PubExp);
    // pairGen = KeyPairGenerator.getInstance("RSA", "BC");
    // final SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
    // pairGen.initialize(rsaParams, random);
    // pair = pairGen.generateKeyPair();
    // }
    // catch (final NoSuchAlgorithmException e)
    // {
    // throw new SecurityEngineException(e.getMessage());
    // }
    // catch (final InvalidAlgorithmParameterException e)
    // {
    // throw new SecurityEngineException(e.getMessage());
    // }
    // catch (final NoSuchProviderException e)
    // {
    // throw new SecurityEngineException(e.getMessage());
    // }
    // return pair;
    // }
    public RSAPublicKey generateRSAKeyPairCard(final String keyIdentifier,
        final String appAlias, final int keyLength, final String PIN)
        throws SecurityEngineException
    {
        RSAPublicKey rsaPublicKey = null;
        try
        {
            final byte[] cardModulus = this.cryptoModuleManager.generateRSAKeyPair(
                keyIdentifier, appAlias, keyLength, PIN);
            final RSAPublicKeySpec pubKeySpec = new RSAPublicKeySpec(new BigInteger(
                cardModulus), new BigInteger("10001", 16));
            KeyFactory keyFactory;
            keyFactory = KeyFactory.getInstance("RSA", "BC");
            rsaPublicKey = (RSAPublicKey)keyFactory.generatePublic(pubKeySpec);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoCardInUseException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeySpecException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return rsaPublicKey;
    }
}
