/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/securityengine/KeyManager.java $
 * 
 * Last submit:
 *  $Revision: #2 $
 *  $Change: 365799 $
 *  $DateTime: 2012/06/01 14:17:13 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.securityengine;


import java.math.BigInteger;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.SignatureException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;
import java.security.spec.AlgorithmParameterSpec;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import morpho.cryptomodule.convergence.CommandDispatcher;
import morpho.cryptomodule.convergence.NoCardInUseException;
import morpho.securityengine.exceptions.SecurityEngineException;


public class KeyManager
{
    public static String toHex(final byte[] bytes)
    {
        final BigInteger bi = new BigInteger(1, bytes);
        return String.format("%0" + (bytes.length << 1) + "X", bi);
    }

    CommandDispatcher cryptoModuleManager = null;

    public KeyManager(final CommandDispatcher cryptoModuleManager)
    {
        this.cryptoModuleManager = cryptoModuleManager;
    }

    public byte[] decrypt(final byte[] data, final SecretKey decryptionKey,
        final String algorithm) throws SecurityEngineException
    {
        byte[] result = null;
        Cipher cipher = null;
        try
        {
            // Using zero IV.
            final byte[] iv = new byte[] {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
            final AlgorithmParameterSpec paramSpec = new IvParameterSpec(iv);
            cipher = Cipher.getInstance(algorithm, "BC");
            cipher.init(Cipher.DECRYPT_MODE, decryptionKey, paramSpec);
            result = cipher.doFinal(data);
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IllegalBlockSizeException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final BadPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidAlgorithmParameterException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    public byte[] encrypt(final byte[] data, final SecretKey encryptionKey,
        final String algorithm) throws SecurityEngineException
    {
        byte[] result = null;
        Cipher cipher = null;
        AlgorithmParameterSpec paramSpec;
        try
        {
            // Using zero IV.
            final byte[] iv = new byte[] {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
            paramSpec = new IvParameterSpec(iv);
            cipher = Cipher.getInstance(algorithm, "BC");
            cipher.init(Cipher.ENCRYPT_MODE, encryptionKey, paramSpec);
            result = cipher.doFinal(data);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IllegalBlockSizeException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final BadPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidAlgorithmParameterException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    public SecretKey generateRandomKey(final String algorithm, final int keyLength)
        throws SecurityEngineException
    {
        SecretKey result = null;
        try
        {
            final KeyGenerator kgen = KeyGenerator.getInstance(algorithm, "BC");
            kgen.init(keyLength);
            result = kgen.generateKey();
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    public void initStreamUnwrap(final String appAlias, final String PIN)
        throws SecurityEngineException
    {
        try
        {
            this.cryptoModuleManager.initStreamUnwrap(null, appAlias, PIN);
        }
        catch (final NoCardInUseException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
    }

    public SecretKey streamUnwrapAfterInit(final byte[] wrappedSymmKey)
        throws SecurityEngineException
    {
        byte[] plainTextKey = null;
        SecretKeySpec unWrappedKey = null;
        try
        {
            plainTextKey = this.cryptoModuleManager.streamUnwrapAfterInit(wrappedSymmKey);
            unWrappedKey = new SecretKeySpec(plainTextKey, "AES");
        }
        catch (final NoCardInUseException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return unWrappedKey;
    }

    // public SecretKey unwrap(final byte[] wrappedsymmKey, final String appAlias,
    // final String PIN) throws SecurityEngineException
    // {
    // String plainTextKey = "";
    // SecretKeySpec unWrappedKey = null;
    // try
    // {
    // plainTextKey = this.cryptoModuleManager.unwrapKey(wrappedsymmKey, null,
    // appAlias, PIN);
    // unWrappedKey = new SecretKeySpec(Converter.toByteArray(plainTextKey), "AES");
    // }
    // catch (final CardException e)
    // {
    // throw new SecurityEngineException(e.getMessage());
    // }
    // catch (final NoCardInUseException e)
    // {
    // throw new SecurityEngineException(e.getMessage());
    // }
    // return unWrappedKey;
    // }
    public SecretKey unwrap(final byte[] wrappedSymmKey, final String appAlias,
        final String PIN) throws SecurityEngineException
    {
        byte[] plainTextKey = null;
        SecretKeySpec unWrappedKey = null;
        try
        {
            plainTextKey = this.cryptoModuleManager.unwrapKeyOnce(wrappedSymmKey, null,
                appAlias, PIN);
            unWrappedKey = new SecretKeySpec(plainTextKey, "AES");
        }
        catch (final NoCardInUseException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return unWrappedKey;
    }

    public byte[] wrap(final Key symmKey, final String algorithm, final String keyAlias,
        final PublicKey pubKey) throws SecurityEngineException
    {
        byte[] result = null;
        try
        {
            final Cipher c = Cipher.getInstance(algorithm, "BC");
            c.init(Cipher.WRAP_MODE, pubKey);
            result = c.wrap(symmKey);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IllegalBlockSizeException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    public byte[] wrap(final SecretKey symmKey, final String algorithm,
        final X509Certificate userCert, final X509Certificate issuerCert)
        throws SecurityEngineException
    {
        byte[] result = null;
        try
        {
            userCert.checkValidity();
            userCert.verify(issuerCert.getPublicKey(), "BC");
            final Cipher c = Cipher.getInstance("RSA/NONE/OAEPWithSHA256AndMGF1Padding",
                "BC");
            c.init(Cipher.WRAP_MODE, userCert.getPublicKey());
            result = c.wrap(symmKey);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IllegalBlockSizeException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final CertificateExpiredException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final CertificateNotYetValidException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final CertificateException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final SignatureException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }
}
