/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/securityengine/obj/EncryptedDataObj.java $
 * 
 * Last submit:
 *  $Revision: #1 $
 *  $Change: 365542 $
 *  $DateTime: 2012/05/31 20:23:21 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.securityengine.obj;


import java.io.Serializable;
import java.util.HashMap;


public class EncryptedDataObj implements Serializable
{
    private static final long             serialVersionUID = 4548510636201260505L;
    private final byte[]                  cipherText;
    // HashMap which associated userIds to wrappedKeys
    private final HashMap<String, byte[]> wrappedKeys;

    /**
     * This constructor initialize the Hashmap for the wrapped keys and add the ciphertext
     * 
     * @param cipherText
     */
    public EncryptedDataObj(final byte[] cipherText)
    {
        super();
        this.cipherText = cipherText;
        this.wrappedKeys = new HashMap<String, byte[]>(); // <UserId, wrappedKey>
    }

    /**
     * This method returns the CipherText of the object
     * 
     * @return
     */
    public byte[] getCipherText()
    {
        return this.cipherText;
    }

    /**
     * Returns the wrappedkey if it exists.
     * 
     * @param userID
     * @return The wrappedKey to the associated userID
     */
    public byte[] getWrappedKey(final String userID)
    {
        return this.wrappedKeys.get(userID);
    }

    /**
     * Inserts a wrappedKey to the object.
     * 
     * @param id
     * @param wrappedKey
     */
    public void insertWrappedKey(final String userID, final byte[] wrappedKey)
    {
        this.wrappedKeys.put(userID, wrappedKey);
    }
}
