/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/securitylib/SecurityLib.java $
 * 
 * Last submit:
 *  $Revision: #2 $
 *  $Change: 365799 $
 *  $DateTime: 2012/06/01 14:17:13 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.securitylib;


import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import morpho.cryptomodule.CryptoModule;
import morpho.securitylib.exceptions.SecurityLibException;


public class SecurityLib
{
    // Currently up to two active cryptoModules are allowed at maximun.
    private static final String  ACTIVE_CRYPTOMODULE_NAMES = "convergenceCard";
    private static final boolean ENABLE_CRYPTOMODULES      = true;
    // private static final boolean ENABLE_SECURITYENGINE = false;
    // private static final boolean ENABLE_WEBSERVICES = false;
    // private static final String ACTIVE_WEBSERVICE_NAMES =
    // "convergenceIP;convergenceSP";
    // private static final boolean ENABLE_DATABASES = false;
    // private static final String ACTIVE_DATABASE_NAMES = "convergenceMySQL";
    private CryptoModule         cryptoModule1             = null;
    private CryptoModule         cryptoModule2             = null;

    public SecurityLib() throws SecurityLibException
    {
        // TODO optimize configuration of library components (String tokenizer - switch
        // case - and so on...)
        if (ENABLE_CRYPTOMODULES)
        {
            if (ACTIVE_CRYPTOMODULE_NAMES.contains("convergenceCard"))
            {
                try
                {
                    final String cardProtocol = "T=1";
                    final Properties properties = new Properties();
                    final BufferedInputStream bis = new BufferedInputStream(
                        new FileInputStream("./config/ConvergenceCard.properties"));
                    properties.load(bis);
                    bis.close();
                    final String cardTerminalName = properties
                        .getProperty("cardTerminalName");
                    if (this.cryptoModule1 != null)
                    {
                        this.cryptoModule1 = new morpho.cryptomodule.convergence.CommandDispatcher(
                            cardProtocol, cardTerminalName);
                    }
                    else if (this.cryptoModule2 != null)
                    {
                        this.cryptoModule2 = new morpho.cryptomodule.convergence.CommandDispatcher(
                            cardProtocol, cardTerminalName);
                    }
                    else
                    {
                        throw new SecurityLibException(
                            "There are already two active crypto modules. Maximum allowed active crypto modules is two at the moment.");
                    }
                }
                catch (final IOException e)
                {
                    throw new SecurityLibException(
                        "IOException while reading .properties file: " + e.getMessage());
                }
            }
        }
        else
        {
            this.cryptoModule1 = null;
            this.cryptoModule2 = null;
        }
    }

    public CryptoModule getCryptoModule1()
    {
        return this.cryptoModule1;
    }

    public CryptoModule getCryptoModule2()
    {
        return this.cryptoModule2;
    }

    public void setCryptoModule1(final CryptoModule cryptoModule1)
    {
        this.cryptoModule1 = cryptoModule1;
    }

    public void setCryptoModule2(final CryptoModule cryptoModule2)
    {
        this.cryptoModule2 = cryptoModule2;
    }
}
