/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/securitylib/demo/GenerateSignature.java $
 * 
 * Last submit:
 *  $Revision: #2 $
 *  $Change: 365799 $
 *  $DateTime: 2012/06/01 14:17:13 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.securitylib.demo;


import morpho.securityengine.SecurityTool;
import morpho.securityengine.exceptions.SecurityEngineException;
import morpho.securitylib.demo.gui.Controlpanel;
import morpho.util.Converter;
import morpho.util.FileAndObjectHandler;


public class GenerateSignature
{
    private static final String       APP_ALIAS1         = "1";
    private static final Controlpanel guiReference       = Controlpanel
                                                             .getControlPanelInstance();
    private static final String       KEY_IDENTIFIER_SIG = "sig";
    private static String             modulus_filepath   = guiReference.getFileDir()
                                                             .getAbsolutePath();
    private static final String       PIN_GAKP           = "12345678";
    private static final String       PIN_SIG            = "123456";

    /**
     * @param args the file name of the VDI
     */
    public static void main(final String[] args)
    {
        if (args.length == 0)
        {
            guiReference.getLogArea().append(
                "ERROR: Please provide an input file name!" + "\n");
            return;
        }
        else if ((args[0] == null) || (args[0] == ""))
        {
            guiReference.getLogArea().append(
                "ERROR: Please provide an input file name!" + "\n");
            return;
        }
        byte[] example_VDI = null;
        guiReference.getLogArea().append("*************************\n");
        guiReference.getLogArea().append("GENERATE A VDI SIGNATURE:\n");
        guiReference.getLogArea().append("*************************\n");
        guiReference.getLogArea().append("\n");
        byte[] cardSignature = null;
        String cardModulus = "\n";
        try
        {
            final SecurityTool securityTool = new SecurityTool();
            // Read the VDI from hard disk.
            guiReference.getLogArea().append(
                "Read the file '" + args[0] + "' from hard disk:\n");
            example_VDI = FileAndObjectHandler.readBytesFromFile1(guiReference
                .getWorkingFile().getAbsolutePath());
            guiReference.getLogArea().append("done\n");
            guiReference.getLogArea().append("");
            // Generate signature of VDI on card.
            guiReference.getLogArea().append(
                "Generate signature of '" + args[0] + "' on card:\n");
            cardSignature = securityTool.generateCardSignature(example_VDI, APP_ALIAS1,
                PIN_SIG);
            // cardSignature = securityTool.generateCardSignature(example_VDI, args[1],
            // args[4]);
            guiReference.getLogArea().append(
                "Resulting Signature: " + Converter.toHex(cardSignature) + "\n");
            guiReference.getLogArea().append("\n");
            // Write signature to hard disk.
            guiReference.getLogArea().append("Write 'signature.bin' to hard disk:\n");
            FileAndObjectHandler.writeBytesToFile(cardSignature, guiReference
                .getFileDir().getAbsolutePath() + "/signature.bin");
            guiReference.getLogArea().append("done\n");
            guiReference.getLogArea().append("\n");
            // Get the used public key from the card.
            guiReference.getLogArea().append("Get the used public key from the card:\n");
            cardModulus = securityTool.getCardModulus(KEY_IDENTIFIER_SIG, APP_ALIAS1,
                PIN_GAKP);
            // cardModulus = securityTool.getCardModulus(args[2], args[1], args[3]);
            guiReference.getLogArea().append("Resulting Modulus: " + cardModulus + "\n");
            guiReference.getLogArea().append("\n");
            // Write modulus to hard disk.
            guiReference.getLogArea().append("Write 'modulus_sig.bin' to hard disk:\n");
            FileAndObjectHandler.writeBytesToFile(Converter.toByteArray(cardModulus),
                modulus_filepath + "/modulus_sig.bin");
        }
        catch (final SecurityEngineException e)
        {
            guiReference.getLogArea().append("ERROR: " + e.getMessage());
            return;
        }
        guiReference.getLogArea().append("done\n");
        guiReference.getLogArea().append("\n");
    }
}
