/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/securitylib/demo/VerifySignature.java $
 * 
 * Last submit:
 *  $Revision: #2 $
 *  $Change: 365799 $
 *  $DateTime: 2012/06/01 14:17:13 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.securitylib.demo;


import morpho.securityengine.SecurityTool;
import morpho.securityengine.exceptions.SecurityEngineException;
import morpho.securitylib.demo.gui.Controlpanel;
import morpho.util.Converter;
import morpho.util.FileAndObjectHandler;


public class VerifySignature
{
    private static final Controlpanel guiReference = Controlpanel
                                                       .getControlPanelInstance();

    public static void main(final String[] args)
    {
        if (args.length == 0)
        {
            guiReference.getLogArea().append(
                "ERROR: Please provide an input file name!\n");
            return;
        }
        else if ((args[0] == null) || (args[0] == ""))
        {
            guiReference.getLogArea().append(
                "ERROR: Please provide an input file name!\n");
            return;
        }
        byte[] example_VDI = null;
        guiReference.getLogArea().append("*********************\n");
        guiReference.getLogArea().append("VERIFY VDI SIGNATURE:\n");
        guiReference.getLogArea().append("*********************\n");
        guiReference.getLogArea().append("\n");
        byte[] cardSignature = null;
        String cardModulus = "";
        try
        {
            final SecurityTool securityTool = new SecurityTool();
            // Read the VDI from hard disk.
            guiReference.getLogArea().append(
                "Read the file '" + args[0] + "' from hard disk:\n");
            example_VDI = FileAndObjectHandler.readBytesFromFile1(guiReference
                .getWorkingFile().getAbsolutePath());
            guiReference.getLogArea().append("done\n");
            guiReference.getLogArea().append("\n");
            // Read the signature from hard disk.
            guiReference.getLogArea().append("Read the signature from hard disk:\n");
            cardSignature = FileAndObjectHandler.readBytesFromFile(guiReference
                .getFileDir().getAbsolutePath() + "/signature.bin");
            guiReference.getLogArea().append("done\n");
            guiReference.getLogArea().append(
                "Resulting Signature: " + Converter.toHex(cardSignature) + "\n");
            guiReference.getLogArea().append("\n");
            // Read the modulus from hard disk.
            guiReference.getLogArea().append("Read the modulus from hard disk:\n");
            cardModulus = Converter.toHex(FileAndObjectHandler
                .readBytesFromFile(guiReference.getFileDir().getAbsolutePath()
                    + "/modulus_sig.bin"));
            guiReference.getLogArea().append("done\n");
            guiReference.getLogArea().append("Resulting Modulus: " + cardModulus + "\n");
            guiReference.getLogArea().append("\n");
            // Verify the signature.
            guiReference.getLogArea().append("Verify the signature:\n");
            guiReference.getLogArea().append("\n");
            if (securityTool.verifySignature(example_VDI, cardSignature, cardModulus))
            {
                guiReference
                    .getLogArea()
                    .append(
                        "             >>>> THE SIGNATURE OF THE VDI WAS SUCCESSFULLY VERIFIED <<<<\n");
            }
            else
            {
                guiReference.getLogArea().append(
                    "             >>>> THE SIGNATURE OF THE VDI IS INVALID <<<<\n");
            }
        }
        catch (final SecurityEngineException e)
        {
            guiReference.getLogArea().append("ERROR: " + e.getMessage() + "\n");
            return;
        }
    }
}
