/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/securitylib/demo/gui/ButtonListener.java $
 * 
 * Last submit:
 *  $Revision: #1 $
 *  $Change: 365799 $
 *  $DateTime: 2012/06/01 14:17:13 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.securitylib.demo.gui;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.math.BigInteger;
import java.security.PublicKey;
import java.security.cert.X509Certificate;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPasswordField;

import morpho.securityengine.exceptions.SecurityEngineException;
import morpho.securityengine.obj.EncryptedDataObj;
import morpho.securitylib.demo.GenerateSignatureThread;
import morpho.securitylib.demo.VerifySignatureThread;
import morpho.securitylib.exceptions.SecurityLibException;
import morpho.securitylib.interfaces.JavaSecurityModule;
import morpho.util.Converter;
import morpho.util.FileAndObjectHandler;


public class ButtonListener implements ActionListener
{
    private static ButtonListener instance = null;

    public static ButtonListener getButtonListenerInstance()
    {
        if (instance == null)
        {
            instance = new ButtonListener();
        }
        return instance;
    }

    private JavaSecurityModule module;
    private EncryptedDataObj   result;

    private ButtonListener()
    {
        this.module = null;
        try
        {
            this.module = new JavaSecurityModule();
        }
        catch (final SecurityLibException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @Override
    public void actionPerformed(final ActionEvent e)
    {
        final Controlpanel guiReference = Controlpanel.getControlPanelInstance();
        final String actionCommand = e.getActionCommand();
        if ("genKeyOnCard".equals(actionCommand))
        {
            final JPasswordField pinField = new JPasswordField();
            pinField.addAncestorListener(new RequestFocusListener());
            final JComponent[] inputs = new JComponent[] {
                new JLabel("Please enter your PIN first"), pinField,};
            JOptionPane.showMessageDialog(null, inputs, "Authentication",
                JOptionPane.PLAIN_MESSAGE);
            final String insertedPin = new String(pinField.getPassword());
            guiReference.setEnteredPIN(insertedPin);
            if ("12345678".equals(guiReference.getEnteredPIN()))
            {
                final PublicKey key = this.module.generateRSAKeyPairCard("12345678");
                guiReference.getLogArea().append(key.toString());
            }
            else
            {
                JOptionPane.showMessageDialog(guiReference, "Wrong PIN");
            }
        }
        if ("encrypt".equals(actionCommand))
        {
            final String plaintextRaw = JOptionPane.showInputDialog(null,
                "Please enter the text you want to encrypt", "Encryption", 1);
            if (!("".equals(plaintextRaw)) && (plaintextRaw != null))
            {
                final byte[] plaintext = Converter
                    .toByteArray(plaintextRaw.toCharArray());
                X509Certificate myIssuerCert = null;
                try
                {
                    myIssuerCert = (X509Certificate)FileAndObjectHandler
                        .readObjectFromFile("./certificates/SecurityLib_root.cert");
                }
                catch (final SecurityEngineException e1)
                {
                    e1.printStackTrace();
                }
                final PublicKey pubKey = this.module.getCardPublicKey("12345678");
                final BigInteger userSerialNumber = new BigInteger("3");
                final String userID = "345";
                final String userReference = "C=DE,O=Company";
                final int expiryDays = 23;
                final X509Certificate userCert = this.module.generateCertificate(userID,
                    userSerialNumber, userReference, expiryDays, pubKey);
                guiReference.getLogArea().append(userCert.toString());
                final X509Certificate[] userCerts = new X509Certificate[1];
                userCerts[0] = userCert;
                guiReference.getLogArea().append(myIssuerCert.toString());
                guiReference.getLogArea().append(userCert.toString());
                guiReference.getLogArea().append(
                    "Plaintext: " + Converter.toHex(plaintext) + "\n");
                this.result = this.module.encrypt(plaintext, userCerts, myIssuerCert);
                JavaSecurityModule.writeObjectToFile(
                    "./secDemoFiles/plainDataObject.obj", plaintext);
                JavaSecurityModule.writeObjectToFile(
                    "./SecDemoFiles/encryptedDataObject.obj", this.result);
            }
        }
        if ("decrypt".equals(actionCommand))
        {
            final JPasswordField pinField = new JPasswordField();
            pinField.addAncestorListener(new RequestFocusListener());
            final JComponent[] inputs = new JComponent[] {
                new JLabel("Please enter your PIN first"), pinField,};
            JOptionPane.showMessageDialog(null, inputs, "Authentication",
                JOptionPane.PLAIN_MESSAGE);
            final String insertedPin = new String(pinField.getPassword());
            guiReference.setEnteredPIN(insertedPin);
            if ("223456".equals(guiReference.getEnteredPIN()))
            {
                this.module.initCardStreamDecrypt("223456");
                final byte[] decryptedResult = this.module.cardStreamDecrypt("345",
                    this.result);
                guiReference.getLogArea().append(
                    "Decrypted Plaintext: " + Converter.toHex(decryptedResult) + "\n");
                JavaSecurityModule.writeObjectToFile(
                    "./SecDemoFiles/decryptedDataObject.obj", decryptedResult);
            }
            else
            {
                JOptionPane.showMessageDialog(guiReference, "Wrong PIN");
            }
        }
        if ("chooseFile".equals(e.getActionCommand()))
        {
            final int returnVal = guiReference.getFileChooser().showOpenDialog(
                guiReference);
            if (returnVal == JFileChooser.APPROVE_OPTION)
            {
                guiReference.setWorkingFile(guiReference.getFileChooser()
                    .getSelectedFile());
                guiReference.getLogArea().append(
                    "Selected File " + guiReference.getWorkingFile().getName() + "."
                        + "\n");
            }
        }
        if (guiReference.getWorkingFile() != null)
        {
            if ("genSignature".equals(actionCommand))
            {
                final JPasswordField pinField = new JPasswordField();
                pinField.addAncestorListener(new RequestFocusListener());
                final JComponent[] inputs = new JComponent[] {
                    new JLabel("Please enter your PIN first"), pinField,};
                JOptionPane.showMessageDialog(null, inputs, "Authentication",
                    JOptionPane.PLAIN_MESSAGE);
                final String insertedPin = new String(pinField.getPassword());
                guiReference.setEnteredPIN(insertedPin);
                if ("123456".equals(guiReference.getEnteredPIN()))
                {
                    final Thread genSignatureThread = new Thread(
                        new GenerateSignatureThread(), "genSignatureThread");
                    genSignatureThread.start();
                }
                else
                {
                    JOptionPane.showMessageDialog(guiReference, "Wrong PIN");
                }
            }
            if ("verifySig".equals(actionCommand))
            {
                final Thread verifySignatureThread = new Thread(
                    new VerifySignatureThread(), "verifySigThread");
                verifySignatureThread.start();
            }
        }
        else if ((guiReference.getWorkingFile() == null)
            && ("genSignature".equals(actionCommand) || "verifySig".equals(actionCommand))
            && !("chooseFile".equals(actionCommand)))
        {
            guiReference.getLogArea().append("Please select a file first" + "\n");
        }
    }
}
