/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/securitylib/demo/gui/RequestFocusListener.java $
 * 
 * Last submit:
 *  $Revision: #1 $
 *  $Change: 365799 $
 *  $DateTime: 2012/06/01 14:17:13 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.securitylib.demo.gui;


import javax.swing.JComponent;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;


/**
 * Convenience class to request focus on a component. When the component is added to a
 * realized Window then component will request focus immediately, since the ancestorAdded
 * event is fired immediately. When the component is added to a non realized Window, then
 * the focus request will be made once the window is realized, since the ancestorAdded
 * event will not be fired until then. Using the default constructor will cause the
 * listener to be removed from the component once the AncestorEvent is generated. A second
 * constructor allows you to specify a boolean value of false to prevent the
 * AncestorListener from being removed when the event is generated. This will allow you to
 * reuse the listener each time the event is generated.
 */
public class RequestFocusListener implements AncestorListener
{
    // Note: This approach seems not to work on jre6 and Linux
    private final boolean removeListener;

    /*
     * Convenience constructor. The listener is only used once and then it is removed from
     * the component.
     */
    public RequestFocusListener()
    {
        this(true);
    }

    /*
     * Constructor that controls whether this listen can be used once or multiple times.
     * @param removeListener when true this listener is only invoked once otherwise it can
     * be invoked multiple times.
     */
    public RequestFocusListener(final boolean removeListener)
    {
        this.removeListener = removeListener;
    }

    @Override
    public void ancestorAdded(final AncestorEvent e)
    {
        final JComponent component = e.getComponent();
        component.requestFocusInWindow();
        if (this.removeListener)
        {
            component.removeAncestorListener(this);
        }
    }

    @Override
    public void ancestorMoved(final AncestorEvent e)
    {
    }

    @Override
    public void ancestorRemoved(final AncestorEvent e)
    {
    }
}
