/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/securitylib/interfaces/JavaSecurityModule.java $
 * 
 * Last submit:
 *  $Revision: #1 $
 *  $Change: 365799 $
 *  $DateTime: 2012/06/01 14:17:13 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.securitylib.interfaces;


import java.math.BigInteger;

import morpho.securityengine.SecurityTool;
import morpho.securityengine.exceptions.SecurityEngineException;
import morpho.securityengine.obj.EncryptedDataObj;
import morpho.securitylib.exceptions.SecurityLibException;
import morpho.util.FileAndObjectHandler;


/**
 * Provides card security functions to the user
 */
public class JavaSecurityModule
{
    public static Object readObjectFromFile(final String filePath)
    {
        Object result = null;
        try
        {
            result = FileAndObjectHandler.readObjectFromFile(filePath);
        }
        catch (final SecurityEngineException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return result;
    }

    public static void writeObjectToFile(final String filePath, final Object object)
    {
        try
        {
            FileAndObjectHandler.writeObjectToFile(filePath, object);
        }
        catch (final SecurityEngineException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    private SecurityTool securityTool = null;

    public JavaSecurityModule() throws SecurityLibException
    {
        // TODO instantiate at some later point with modular SecurityLib class
        this.securityTool = new SecurityTool();
    }

    public byte[] cardStreamDecrypt(final String globalID,
        final EncryptedDataObj encryptedDataObject)
    {
        return this.securityTool.streamUnwrapAfterInit(globalID, encryptedDataObject);
    }

    // public void generateKeyOnCard(String keyIdentifier, String appAlias, String pin){
    // String [] args = {keyIdentifier, appAlias, pin};
    // GenerateKeyOnCard.main(args);
    // }
    //
    // public void generateSignature(String filepath, String appAlias, String
    // keyIdentifier,
    // String pin_GAKP, String pinSig){
    // String [] args = {filepath, appAlias, keyIdentifier, pin_GAKP, pinSig};
    // GenerateSignature.main(args);
    // }
    //
    // public void getCardModulusForDecrypt(String appAlias, String keyIdentifier,
    // String pin_GAKP){
    // String [] args = {appAlias, keyIdentifier, pin_GAKP};
    // security.demo.GetCardModulusForDecrypt.main(args);
    // }
    //
    // public void hybridEncrypt(String filepath){
    // String [] args = {filepath};
    // HybridEncryption.main(args);
    // }
    //
    // public void hybridDecrypt(String appAlias, String pin_conf){
    // String [] args = {appAlias, pin_conf};
    // HybridDecryption.main(args);
    // }
    //
    // public void verifySignature(String filepath){
    // String [] args = {filepath};
    // VerifySignature.main(args);
    // }
    public EncryptedDataObj encrypt(final byte[] plaintext,
        final java.security.cert.X509Certificate[] userCerts,
        final java.security.cert.X509Certificate issuerCert)
    {
        return this.securityTool.hybridEncrypt(plaintext, userCerts, issuerCert);
    }

    public java.security.cert.X509Certificate generateCertificate(final String globalID,
        final BigInteger serialNumber, final String holderReference,
        final int expiryDays, final java.security.PublicKey pubKey)
    {
        return this.securityTool.generateCertificate(globalID, serialNumber,
            holderReference, expiryDays, pubKey);
    }

    public java.security.PublicKey generateRSAKeyPairCard(final String PIN)
    {
        return this.securityTool.generateRSAKeyPairCard("conf", "1", 1024, PIN);
    }

    public java.security.PublicKey getCardPublicKey(final String PIN)
    {
        return this.securityTool.getCardKey("conf", "1", PIN);
    }

    public void initCardStreamDecrypt(final String PIN)
    {
        this.securityTool.initCardStreamDecrypt(PIN);
    }
}
