/*
 * Copyright (c) 2011 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/util/Converter.java $
 * 
 * Last submit:
 *  $Revision: #11 $
 *  $Change: 365542 $
 *  $DateTime: 2012/05/31 20:23:21 $
 *  $Author: akohlos $
 *
 * 
 * Created on 25.10.2011
 *
 */
package morpho.util;


import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;


public class Converter
{
    static final byte[] HEX_CHAR_TABLE = {(byte)'0', (byte)'1', (byte)'2', (byte)'3',
        (byte)'4', (byte)'5', (byte)'6', (byte)'7', (byte)'8', (byte)'9', (byte)'a',
        (byte)'b', (byte)'c', (byte)'d', (byte)'e', (byte)'f'};

    // Precondition: d is a nonnegative integer.
    public static String decimalToHex(int decimal)
    {
        final String digits = "0123456789ABCDEF";
        if (decimal == 0)
        {
            return "0";
        }
        String hex = "";
        while (decimal > 0)
        {
            final int digit = decimal % 16; // Rightmost digit
            hex = digits.charAt(digit) + hex; // String concatenation
            decimal = decimal / 16;
        }
        return hex;
    }

    public static String getHexString(final byte[] raw)
        throws UnsupportedEncodingException
    {
        final byte[] hex = new byte[2 * raw.length];
        int index = 0;
        for (final byte b : raw)
        {
            final int v = b & 0xFF;
            hex[index++] = HEX_CHAR_TABLE[v >>> 4];
            hex[index++] = HEX_CHAR_TABLE[v & 0xF];
        }
        return new String(hex, "ASCII");
    }

    // TODO fill in java doc here
    /**
     * @param dataBytes
     * @return
     */
    public static byte[] getLcBytes(final byte[] dataBytes)
    {
        final byte[] lcBytesBuffer = toByteArray(Integer.toHexString(dataBytes.length));
        byte[] lcBytes = null;
        if (lcBytesBuffer.length == 1)
        {
            lcBytes = new byte[1];
            lcBytes[0] = lcBytesBuffer[0];
        }
        else if (lcBytesBuffer.length == 2)
        {
            lcBytes = new byte[3];
            lcBytes[0] = (byte)0x00;
            lcBytes[1] = lcBytesBuffer[0];
            lcBytes[2] = lcBytesBuffer[1];
        }
        else if (lcBytesBuffer.length == 3)
        {
            lcBytes = new byte[3];
            lcBytes[0] = lcBytesBuffer[0];
            lcBytes[1] = lcBytesBuffer[1];
            lcBytes[2] = lcBytesBuffer[2];
        }
        else
        {
            System.err
                .println("Error while trying to calculate the Lc bytes of the command APDU");
        }
        return lcBytes;
    }

    // TODO fill in java doc here
    /**
     * @param dataBytes
     * @return
     */
    public static byte[] getLeBytes(final byte[] dataBytes)
    {
        final byte[] leBytesBuffer = toByteArray(Integer.toHexString(dataBytes.length));
        byte[] leBytes = null;
        if (leBytesBuffer.length == 1)
        {
            leBytes = new byte[1];
            leBytes[0] = leBytesBuffer[0];
        }
        else if (leBytesBuffer.length == 2)
        {
            leBytes = new byte[2];
            leBytes[0] = leBytesBuffer[0];
            leBytes[1] = leBytesBuffer[1];
        }
        else
        {
            System.err.println("Wrong command APDU Le bytes");
        }
        return leBytes;
    }

    public static long hexToDecimal(String hexString)
    {
        final String digits = "0123456789ABCDEF";
        hexString = hexString.toUpperCase();
        long val = 0;
        for (int i = 0; i < hexString.length(); i++)
        {
            final char c = hexString.charAt(i);
            final int d = digits.indexOf(c);
            val = (16 * val) + d;
        }
        return val;
    }

    // TODO fill in java doc here
    /**
     * @param integer
     * @return
     */
    public static byte[] intToTwoBytes(final int integer)
    {
        final byte[] result = new byte[2];
        result[0] = (byte)(integer >>> 8);
        result[1] = (byte)integer;
        return result;
    }

    public static byte[] toByteArray(final char[] array)
    {
        return toByteArray(array, Charset.defaultCharset());
    }

    /**
     * Converts a String into a byte array.
     * 
     * @author abotter
     */
    public static byte[] toByteArray(final char[] array, final Charset charset)
    {
        final CharBuffer cbuf = CharBuffer.wrap(array);
        final ByteBuffer bbuf = charset.encode(cbuf);
        final byte[] resultArray = bbuf.array();
        for (int i = 0; i < resultArray.length; i++)
        {
            resultArray[i] = (byte)(resultArray[i] - 48);
        }
        return resultArray;
    }

    // TODO implement for odd numbers of bytes also
    /**
     * Converts a String with an even number of digits into a byte array. The String is
     * supposed to be in hexadecimal format.
     * 
     * @param string the string in hexadecimal format to convert
     * @return The converted byte array.
     */
    public static byte[] toByteArray(final String string)
    {
        final int length = string.length();
        byte[] data = null;
        if (length == 1)
        {
            data = new byte[1];
            data[0] = (byte)((0 << 4) + Character.digit(string.charAt(0), 16));
        }
        else if (length == 3)
        {
            data = new byte[2];
            for (int i = 0; i < length; i += 2)
            {
                if (i == 2)
                {
                    data[i / 2] = (byte)((Character.digit(string.charAt(i - 1), 16) << 4) + (Character
                        .digit(string.charAt(i), 16)));
                }
                else
                {
                    data[i / 2] = (byte)(Character.digit(string.charAt(i), 16));
                }
            }
        }
        else if ((length % 2) == 0)
        {
            data = new byte[length / 2];
            for (int i = 0; i < length; i += 2)
            {
                data[i / 2] = (byte)((Character.digit(string.charAt(i), 16) << 4) + Character
                    .digit(string.charAt(i + 1), 16));
            }
        }
        else
        {
            System.err.println("Error during HexString to ByteArray conversion");
        }
        return data;
    }

    /**
     * Converts a byte array with an even number of bytes into a String of hexadecimal
     * format.
     * 
     * @param bytes the byte array to convert
     * @return The String representation of the byte array in hexadecimal format.
     */
    public static String toHex(final byte[] bytes)
    {
        final BigInteger bi = new BigInteger(1, bytes);
        return String.format("%0" + (bytes.length << 1) + "X", bi);
    }

    // TODO fill in java doc here
    /**
     * @param b
     * @return
     */
    public static int twoByteArrayToInt(final byte[] bytes)
    {
        final int offset = 0;
        int value = 0;
        for (int i = 0; i < 2; i++)
        {
            final int shift = (2 - 1 - i) * 8;
            value += (bytes[i + offset] & 0x00FF) << shift;
        }
        return value;
    }
}
