/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/util/FileAndObjectHandler.java $
 * 
 * Last submit:
 *  $Revision: #1 $
 *  $Change: 365542 $
 *  $DateTime: 2012/05/31 20:23:21 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.util;


import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import morpho.securityengine.exceptions.SecurityEngineException;


public class FileAndObjectHandler
{
    public static Object blobToObject(final byte[] blob) throws SecurityEngineException
    {
        Object result = null;
        final ByteArrayInputStream bais = new ByteArrayInputStream(blob);
        ObjectInputStream ois;
        try
        {
            ois = new ObjectInputStream(bais);
            result = ois.readObject();
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final ClassNotFoundException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    public static void deleteFile(final String filePath) throws SecurityEngineException
    {
        final File file = new File(filePath);
        // Make sure the file or directory exists and isn't write protected.
        if (!file.exists())
        {
            throw new SecurityEngineException("Delete: no such file or directory");
        }
        if (!file.canWrite())
        {
            throw new SecurityEngineException("Delete: write protected");
        }
        // If it is a directory, make sure it is empty
        if (file.isDirectory())
        {
            final String[] files = file.list();
            if (files.length > 0)
            {
                throw new SecurityEngineException("Delete: directory not empty");
            }
        }
        // Attempt to delete it.
        final boolean success = file.delete();
        if (!success)
        {
            throw new SecurityEngineException("Delete: deletion failed");
        }
    }

    public static byte[] objectToBlob(final Object object) throws SecurityEngineException
    {
        byte[] result = null;
        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ObjectOutputStream oos = null;
        try
        {
            oos = new ObjectOutputStream(baos);
            oos.writeObject(object);
            oos.flush();
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        finally
        {
            if (oos != null)
            {
                try
                {
                    oos.close();
                }
                catch (final IOException e)
                {
                    throw new SecurityEngineException(e.getMessage());
                }
            }
            result = baos.toByteArray();
            if (baos != null)
            {
                try
                {
                    baos.close();
                }
                catch (final IOException e)
                {
                    throw new SecurityEngineException(e.getMessage());
                }
            }
        }
        return result;
    }

    public static byte[] readBinaryFile(final String filePath) throws Exception
    {
        final File file = new File(filePath);
        try
        {
            if (file.exists())
            {
                final FileInputStream in = new FileInputStream(file);
                final byte[] result = new byte[in.available()];
                in.read(result);
                in.close();
                return result;
            }
            else
            {
                throw new Exception("File: '" + filePath + "' - not found");
            }
        }
        catch (final Exception e)
        {
            throw new Exception("Error while reading the file: '" + filePath + "'.");
        }
    }

    public static byte[] readBytesFromFile(final String filePath)
        throws SecurityEngineException
    {
        final File file = new File(filePath);
        try
        {
            if (file.exists())
            {
                final FileInputStream in = new FileInputStream(file);
                final byte[] result = new byte[in.available()];
                in.read(result);
                in.close();
                return result;
            }
            else
            {
                throw new SecurityEngineException("File: '" + filePath + "' - not found");
            }
        }
        catch (final FileNotFoundException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
    }

    public static byte[] readBytesFromFile1(final String filePath)
    {
        final File file = new File(filePath);
        byte[] result = null;
        FileInputStream fis = null;
        BufferedInputStream bis = null;
        try
        {
            fis = new FileInputStream(file);
            bis = new BufferedInputStream(fis);
            result = new byte[bis.available()];
            fis.close();
            bis.close();
        }
        catch (final FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (final IOException e)
        {
            e.printStackTrace();
        }
        return result;
    }

    public static Object readObjectFromFile(final String filePath)
        throws SecurityEngineException
    {
        Object result = null;
        ObjectInputStream ois = null;
        FileInputStream fis = null;
        final File file = new File(filePath);
        if (file.exists())
        {
            try
            {
                fis = new FileInputStream(file);
                ois = new ObjectInputStream(fis);
                result = ois.readObject();
            }
            catch (final IOException e)
            {
                throw new SecurityEngineException("Error while reading the file: '"
                    + filePath + "'.");
            }
            catch (final ClassNotFoundException e)
            {
                throw new SecurityEngineException("Error while reading the file: '"
                    + filePath + "'.");
            }
            finally
            {
                if (ois != null)
                {
                    try
                    {
                        ois.close();
                    }
                    catch (final IOException e)
                    {
                        throw new SecurityEngineException(e.getMessage());
                    }
                }
                if (fis != null)
                {
                    try
                    {
                        fis.close();
                    }
                    catch (final IOException e)
                    {
                        throw new SecurityEngineException(e.getMessage());
                    }
                }
            }
        }
        else
        {
            throw new SecurityEngineException("File: '" + filePath + "' - not found");
        }
        return result;
    }

    // TODO fill in java doc here
    /**
     * @param filePath
     * @param data
     * @throws Exception
     */
    public static void writeBinaryFile(final String filePath, final byte[] data)
        throws Exception
    {
        final File file = new File(filePath);
        try
        {
            if (!file.exists())
            {
                final FileOutputStream out = new FileOutputStream(file);
                out.write(data);
                out.flush();
                out.close();
            }
            else
            {
                System.err.println("File: '" + filePath
                    + "' - already exists. Cannot write here.");
            }
        }
        catch (final Exception e)
        {
            throw new Exception("Error while writing the file: '" + filePath + "'.");
        }
    }

    public static void writeBytesToFile(final byte[] data, final String filePath)
        throws SecurityEngineException
    {
        final File file = new File(filePath);
        if (file.exists())
        {
            FileAndObjectHandler.deleteFile(filePath);
        }
        FileOutputStream out = null;
        try
        {
            out = new FileOutputStream(file);
            out.write(data);
            out.flush();
            out.close();
        }
        catch (final FileNotFoundException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
    }

    public static void writeBytesToFile1(final byte[] data, final String filePath)
        throws SecurityEngineException
    {
        final File file = new File(filePath);
        if (file.exists())
        {
            FileAndObjectHandler.deleteFile(filePath);
        }
        FileOutputStream fos = null;
        BufferedOutputStream bos = null;
        try
        {
            fos = new FileOutputStream(file);
            bos = new BufferedOutputStream(fos);
            bos.write(data);
            bos.flush();
            bos.close();
            fos.close();
        }
        catch (final FileNotFoundException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
    }

    public static void writeObjectToFile(final String filePath, final Object object)
        throws SecurityEngineException
    {
        FileOutputStream fos = null;
        ObjectOutputStream oos = null;
        final File file = new File(filePath);
        if (file.exists())
        {
            FileAndObjectHandler.deleteFile(filePath);
        }
        try
        {
            fos = new FileOutputStream(file);
            oos = new ObjectOutputStream(fos);
            oos.writeObject(object);
            oos.flush();
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        finally
        {
            if (oos != null)
            {
                try
                {
                    oos.close();
                }
                catch (final IOException e)
                {
                    throw new SecurityEngineException(e.getMessage());
                }
            }
            if (fos != null)
            {
                try
                {
                    fos.close();
                }
                catch (final IOException e)
                {
                    throw new SecurityEngineException(e.getMessage());
                }
            }
        }
    }
}
