/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/util/X509NameService.java $
 * 
 * Last submit:
 *  $Revision: #1 $
 *  $Change: 365542 $
 *  $DateTime: 2012/05/31 20:23:21 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.util;


import java.io.IOException;

import javax.security.auth.x500.X500Principal;

import morpho.securityengine.exceptions.SecurityEngineException;

import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.asn1.x500.X500NameBuilder;
import org.bouncycastle.asn1.x500.style.BCStyle;


public class X509NameService
{
    public static X500Principal create(final String country, final String state,
        final String locality, final String organization,
        final String organizationalUnit, final String commonName, final String street,
        final String domainComponent, final String userID) throws SecurityEngineException
    {
        try
        {
            return new X500Principal(createX509(country, state, locality, organization,
                organizationalUnit, commonName, street, domainComponent, userID)
                .getEncoded());
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
    }

    private static X500Name createX509(final String country, final String state,
        final String locality, final String organization,
        final String organizationalUnit, final String commonName, final String street,
        final String domainComponent, final String userID)
    {
        final X500NameBuilder builder = new X500NameBuilder(BCStyle.INSTANCE);
        if (country != null)
        {
            builder.addRDN(BCStyle.C, country);
        }
        if (state != null)
        {
            builder.addRDN(BCStyle.ST, state);
        }
        if (locality != null)
        {
            builder.addRDN(BCStyle.L, locality);
        }
        if (organization != null)
        {
            builder.addRDN(BCStyle.O, organization);
        }
        if (organizationalUnit != null)
        {
            builder.addRDN(BCStyle.OU, organizationalUnit);
        }
        if (commonName != null)
        {
            builder.addRDN(BCStyle.CN, commonName);
        }
        if (street != null)
        {
            builder.addRDN(BCStyle.STREET, street);
        }
        if (domainComponent != null)
        {
            builder.addRDN(BCStyle.DC, domainComponent);
        }
        if (userID != null)
        {
            builder.addRDN(BCStyle.UID, userID);
        }
        return builder.build();
    }

    public static X500Name get(final X500Principal principal)
    {
        return new X500Name(principal.getName());
    }
}
