package DIMaker;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMInvalidConfigurationException;
import org.iso.mpeg.mxm.orchestratorTE.OrchestratorTE;
import org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse;
import org.iso.mpeg.mxm.pEngine.createcontentPE.technologyhandler.CreateContentProtocol;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.exceptions.IdentifyContentEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.exceptions.DIEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.mp21ffTE.MPEG21FileCreator;

public class DIMaker {

	private static String DEFAULT_MIMETYPE = "video/mov";
	private static String DEFAULT_RESOURCE_URL = "http://research.cedeo.net/angelo/trailer_720p.mov";
	
	private static String DEFAULT_TITLE="Lebowsky";
	private static String DEFAULT_ACTOR_GIVENNAME="Jeff";
	private static String DEFAULT_ACTOR_FAMILYNAME="Bridges";
	
	private static final String TEMP_FOLDER = System.getProperty("java.io.tmpdir") + File.separator;
	private static String DEFAULT_OUTPUT_DI_FILE_NAME = TEMP_FOLDER + "first_test.di";
	//on windows7 point to C:\Users\<username>\AppData\Local\Temp\first_test.di
	
	//private static final String SOURCE_MEDIA_RESOURCE_FILE_NAME="c:\\jfp.png";
	//private static final String MEDIA_RESOURCE_ITEM_ID = "B0";
	private static final String DEFAULT_OUTPUT_DCF_FILE_NAME = TEMP_FOLDER +  "testDCF.mp21";
	
	/**
	 * @param RESOURCE_URL (optional)
	 * @param MIMETYPE (optional)
	 * @param TITLE (optional)
	 * @param ACTOR_GIVENNAME (optional)
	 * @param ACTOR_FAMILYNAME (optional)
	 * @param OUTPUT_DI_FILE_NAME (optional)
	 */
	public static void main(String[] args) {
		
		DIDL didl=null;
		
		Map maps=new HashMap();
		maps.put("PRINCIPAL", "PeerID");
		try {
			
			if (args.length>=2)
			{
				DEFAULT_RESOURCE_URL=args[0];
				DEFAULT_MIMETYPE=args[1];
			}
			
			if (args.length>=5)
			{
				DEFAULT_TITLE=args[2];
				DEFAULT_ACTOR_GIVENNAME=args[3];
				DEFAULT_ACTOR_FAMILYNAME=args[4];
			}
			
			if (args.length>=6)
			{
				DEFAULT_OUTPUT_DI_FILE_NAME=args[5];
			}
			
			//generate VDI locally
			//			didl = OrchestratorTE.dealWithDIDEngine().buildDI(DEFAULT_MIMETYPE, DEFAULT_RESOURCE_URL, DEFAULT_TITLE, DEFAULT_ACTOR_GIVENNAME, DEFAULT_ACTOR_FAMILYNAME);
						
			//generate VDI remotly
			{
							CreateContentEngine createContentEngine = (CreateContentEngine) OrchestratorTE.getCreateContentEngine();
							CreateContentProtocol createContentProtocol = createContentEngine.getCreateContentTechnologyHandler().getRemoteCreateContentProtocol();
							URL serviceURL = null;
							QName serviceName = null;
							
							try
							{
								serviceURL = new URL(createContentEngine.getPropertyValue("SERVICE_URL"));
								serviceName = new QName(createContentEngine.getPropertyValue("SERVICE_NAME_NS"), 
										createContentEngine.getPropertyValue("SERVICE_NAME"));
							} 
							catch (MXMInvalidConfigurationException e)
							{
								e.printStackTrace();
							} 
							catch (MalformedURLException e)
							{
								e.printStackTrace();
							}
							
							createContentProtocol.initServiceParams(serviceURL, serviceName);
							
							CreateContentRequest createContentRequest = createContentEngine.getCreateContentSchemaHandler().getCreateContentRequest();
							createContentRequest.parse(new MXMAdapter(new File(".\\target\\classes\\testCCR.xml")));//get request from file
							createContentRequest.setTransactionIdentifier("TID");
							
							CreateContentResponse createContentResponse = createContentProtocol.createContent(createContentRequest);
							
							System.out.println("trans id is>"+createContentResponse.getTransactionIdentifier());
							
							didl= (org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL) createContentResponse.getCreateContentSuccess().getContentEntity().getDIDL();
			}
			
			//identify VDI
			didl=OrchestratorTE.dealWithIdentifyContentEngine().identifyDI(didl,maps);//use maps
			//didl=OrchestratorTE.dealWithIdentifyContentEngine().identifyDI(didl);//not using maps

		}
		catch (IdentifyContentEngineException e) {
			//e.printStackTrace();
			System.out.println("Using TEMPORARY id");
		} 
		catch (DIEngineException e) 
		{
			e.printStackTrace();
		} 	
		catch (MXMException e) 
		{
			e.printStackTrace();
		} 		
		try
		{
			//generate VDI locally
			didl.createFile(new File(DEFAULT_OUTPUT_DI_FILE_NAME));
			
			System.out.println("The VDI file was successfully saved on "+DEFAULT_OUTPUT_DI_FILE_NAME);
		}
		catch (MXMException e) {
			e.printStackTrace();
		}

		//create an mp21 with resource and di
		MPEG21FileCreator dcfCreator;
		try {
			dcfCreator = OrchestratorTE.getMP21FileEngine().getMPEG21FileCreator();

			dcfCreator.setDIPathName(DEFAULT_OUTPUT_DI_FILE_NAME);
			//dcfCreator.addResource(SOURCE_MEDIA_RESOURCE_FILE_NAME, MEDIA_RESOURCE_ITEM_ID);
			dcfCreator.createMPEG21File(DEFAULT_OUTPUT_DCF_FILE_NAME);
			
			System.out.println("The DCF file was successfully created in file: " + DEFAULT_OUTPUT_DCF_FILE_NAME);
		} catch (MXMException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
