/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD User, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this user.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package DMAGTests;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

//import org.iso.mpeg.mxm.basePE.LicenseEntity;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMObject;
//import org.iso.mpeg.mxm.engine.relTE.schema.relmx.Play;
//import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.KeyInfoType;
import org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.AuthorizeUserEngine;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.schemahandler.AuthorizeUserRequest;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.schemahandler.AuthorizeUserResponse;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.technologyhandler.AuthorizeUserProtocol;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
//import org.iso.mpeg.mxm.relTE.RELEngine;
//import org.iso.mpeg.mxm.relTE.schemahandler.common.NonSecureReference;
//import org.iso.mpeg.mxm.relTE.schemahandler.resource.DigitalResource;
//import org.iso.mpeg.mxm.relTE.schemahandler.resource.principal.KeyHolder;
//import org.iso.mpeg.mxm.relTE.schemahandler.right.GovernedCopy;
//import org.iso.mpeg.mxm.securityTE.schemahandler.dsig.KeyInfo;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm1x.GovernedCopy;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.DigitalResource;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.KeyHolder;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.NonSecureReference;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Right;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.RELRight;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo;

/**
 * @author Jonathan Florido, DMAG-UPC <jflorido@ac.upc.edu>
 *
 */
public class AuthorizeUserTest
{	
	public static void main(String[] args)
	{
		URL serviceURL = null;
		QName serviceName = null;
		try
		{
			InputStream is = AuthorizeUserTest.class.getClassLoader().getResourceAsStream("MXMConfiguration.xml");
			MXM mxm = MXM.getInstance(is);
			
			AuthorizeUserEngine authorizeUserEngine = 
					(AuthorizeUserEngine)mxm.getDefaultEngine(MXMEngineName.AuthorizeUserPE.value());

			RELEngine relEngine = (RELEngine)mxm.getDefaultEngine(MXMEngineName.RELTE.value());
			MetadataEngine metadataEngine = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.value());
			
			List<MXMEngine> deps = new ArrayList<MXMEngine>();
			deps.add(relEngine);
			deps.add(metadataEngine);
			authorizeUserEngine.setDependencies(deps);
			
			AuthorizeUserRequest authorizeUserRequest = 
					authorizeUserEngine.getAuthorizeUserSchemaHandler().getAuthorizeUserRequest();
			authorizeUserRequest.setTransactionIdentifier("Authorize User Test");
			
			// Parse and set the license to AuthorizeUserRequest
			License licenseParser = relEngine.getRELEngineSchemaHandler().getLicense();
			File licenseFile = new File(AuthorizeUserTest.class.getClassLoader().getResource("licenseSpec.xml").getFile());
			MXMObject licenseObject = new MXMAdapter(licenseFile);
			licenseParser.parse(licenseObject);			
			//LicenseEntity licenseEntity = new LicenseEntity(relEngine, metadataEngine);
			LicenseEntity licenseEntity = authorizeUserRequest.getLicenseEntity();

			licenseEntity.setLicense(licenseParser);
			authorizeUserRequest.setLicenseEntity(licenseEntity);
			
			// Set the principal
			KeyHolder keyHolder=relEngine.getRELEngineSchemaHandler().getKeyHolder();
			//KeyHolder keyHolder = new KeyHolder();
			//KeyInfoType keyInfoType = new KeyInfoType();
			//keyInfoType.setId("Test User");
			//KeyInfo keyInfo = new KeyInfo(keyInfoType);
			KeyInfo keyInfo=relEngine.getRELEngineSchemaHandler().getKeyHolder().getKeyInfo();
			keyHolder.setKeyInfo(keyInfo);
			authorizeUserRequest.setPrincipal(keyHolder);
			
			// Set the right which the user wants to be authorized
			//GovernedCopy governedCopy = new GovernedCopy();
			GovernedCopy governedCopy=relEngine.getRELEngineSchemaHandler().getGovernedCopy();
			authorizeUserRequest.addRight(governedCopy);
			
			// Set the resources
			//DigitalResource digitalResource = new DigitalResource();
			DigitalResource digitalResource=relEngine.getRELEngineSchemaHandler().getDigitalResource();
			//NonSecureReference nonSecureReference = new NonSecureReference();
			NonSecureReference nonSecureReference=relEngine.getRELEngineSchemaHandler().getNonSecureReference();
			nonSecureReference.setURI("TEST");
			digitalResource.setNonSecureIndirect(nonSecureReference);
			authorizeUserRequest.addResource(digitalResource);

			System.out.println("Sending TID in Request: " + authorizeUserRequest.getTransactionIdentifier());
			
			OrchestratorEngine orchestratorEngine = 
					(OrchestratorEngine)mxm.getDefaultEngine(MXMEngineName.Orchestrator.value());
			deps.add(authorizeUserEngine);
			orchestratorEngine.setDependencies(deps);
		
			// To be performed correctly
			/*AuthorizeUserProtocol authorizeUserProtocol = new org.iso.mpeg.mxm.authorizeuserPE.technology.local.AuthorizeUserProtocol(orchestratorEngine);
			AuthorizeUserResponse authorizeUserResponse = authorizeUserProtocol.authorizeUser(authorizeUserRequest);
			System.out.println("Received TID in Response: " + authorizeUserResponse.getTransactionIdentifier());*/
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
}
