/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package DMAGTests;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

//import org.iso.mpeg.mxm.basePE.LicenseEntity;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
import org.iso.mpeg.mxm.pEngine.createcontentPE.technologyhandler.CreateContentProtocol;
import org.iso.mpeg.mxm.pEngine.revokelicensePE.RevokeLicenseEngine;
import org.iso.mpeg.mxm.pEngine.revokelicensePE.schemahandler.RevokeLicenseRequest;
import org.iso.mpeg.mxm.pEngine.revokelicensePE.schemahandler.RevokeLicenseResponse;
import org.iso.mpeg.mxm.pEngine.revokelicensePE.technologyhandler.RevokeLicenseProtocol;
//import org.iso.mpeg.mxm.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;

/**
 * @author Jonathan Florido, DMAG-UPC <jflorido@ac.upc.edu>
 *
 */
public class RevokeLicenseTest
{	
	public static void main(String[] args)
	{
		URL serviceURL = null;
		QName serviceName = null;
		try
		{
			InputStream is = RevokeLicenseTest.class.getClassLoader().getResourceAsStream("MXMConfiguration.xml");
			MXM mxm = MXM.getInstance(is);
			
			RevokeLicenseEngine revokeLicenseEngine = 
					(RevokeLicenseEngine)mxm.getDefaultEngine(MXMEngineName.RevokeLicensePE.value());

			RELEngine relEngine = (RELEngine)mxm.getDefaultEngine(MXMEngineName.RELTE.value());
			MetadataEngine metadataEngine = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.value());
			
			List<MXMEngine> deps1 = new ArrayList<MXMEngine>();
			deps1.add(relEngine);
			deps1.add(metadataEngine);
			revokeLicenseEngine.setDependencies(deps1);
			
			RevokeLicenseRequest revokeLicenseRequest = 
					revokeLicenseEngine.getRevokeLicenseSchemaHandler().getRevokeLicenseRequest();
			revokeLicenseRequest.setTransactionIdentifier("Revoke License Test");

			License licenseParser = relEngine.getRELEngineSchemaHandler().getLicense();
			File licenseFile = new File(RevokeLicenseTest.class.getClassLoader().getResource("licenseSpec.xml").getFile());
			MXMObject licenseObject = new MXMAdapter(licenseFile);
			licenseParser.parse(licenseObject);
			
			//LicenseEntity licenseEntity = new LicenseEntity(relEngine, metadataEngine);
			LicenseEntity licenseEntity = revokeLicenseRequest.getRevokedLicense();
			licenseEntity.setLicense(licenseParser);
			revokeLicenseRequest.setRevokedLicense(licenseEntity);
			revokeLicenseRequest.getRevocationReason().add("Revocation Test!");
			GregorianCalendar gc = new GregorianCalendar();
			DatatypeFactory dtf = DatatypeFactory.newInstance();
			XMLGregorianCalendar xgc = dtf.newXMLGregorianCalendar(gc);
			revokeLicenseRequest.setRevocationDate(xgc);

			System.out.println("Sending TID in Request: " + revokeLicenseRequest.getTransactionIdentifier());
			
			OrchestratorEngine orchestratorEngine = 
					(OrchestratorEngine)mxm.getDefaultEngine(MXMEngineName.Orchestrator.value());
			List<MXMEngine> deps = new ArrayList<MXMEngine>();
			deps.add(revokeLicenseEngine);
			orchestratorEngine.setDependencies(deps);
		
			//RevokeLicenseProtocol revokeLicenseProtocol = new org.iso.mpeg.mxm.revokelicensePE.technology.local.RevokeLicenseProtocol(orchestratorEngine);
			//is it remote or local????
			RevokeLicenseProtocol revokeLicenseProtocol = revokeLicenseEngine.getRevokeLicenseTechnologyHandler().getRemoteRevokeLicenseProtocol();
			RevokeLicenseResponse revokeLicenseResponse = revokeLicenseProtocol.revokeLicense(revokeLicenseRequest);
			System.out.println("Received TID in Response: " + revokeLicenseResponse.getTransactionIdentifier());
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
}
