/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package DMAGTests;

import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine;
import org.iso.mpeg.mxm.pEngine.searchlicensePE.SearchLicenseEngine;
import org.iso.mpeg.mxm.pEngine.searchlicensePE.schemahandler.SearchLicenseRequest;
import org.iso.mpeg.mxm.pEngine.searchlicensePE.schemahandler.SearchLicenseResponse;
import org.iso.mpeg.mxm.pEngine.searchlicensePE.technologyhandler.SearchLicenseProtocol;
//import org.iso.mpeg.mxm.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;

/**
 * @author Jonathan Florido, DMAG-UPC <jflorido@ac.upc.edu>
 *
 */
public class SearchLicenseTest
{	
	public static void main(String[] args)
	{
		URL serviceURL = null;
		QName serviceName = null;
		try
		{
			InputStream is = SearchLicenseTest.class.getClassLoader().getResourceAsStream("MXMConfiguration.xml");
			MXM mxm = MXM.getInstance(is);
			
			SearchLicenseEngine searchLicenseEngine = 
					(SearchLicenseEngine)mxm.getDefaultEngine(MXMEngineName.SearchLicensePE.value());	
			
			SearchLicenseRequest searchLicenseRequest = 
					searchLicenseEngine.getSearchLicenseSchemaHandler().getSearchLicenseRequest();
			searchLicenseRequest.setTransactionIdentifier("Search License Test");

			System.out.println("Sending TID in Request: " + searchLicenseRequest.getTransactionIdentifier());
			
			OrchestratorEngine orchestratorEngine = 
					(OrchestratorEngine)mxm.getDefaultEngine(MXMEngineName.Orchestrator.value());
			
			RELEngine relEngine = (RELEngine)mxm.getDefaultEngine(MXMEngineName.RELTE.value());
			MetadataEngine metadataEngine = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.value());
			List<MXMEngine> deps = new ArrayList<MXMEngine>();
			deps.add(relEngine);
			deps.add(metadataEngine);
			searchLicenseEngine.setDependencies(deps);
			deps.add(searchLicenseEngine);
			orchestratorEngine.setDependencies(deps);
		
			//SearchLicenseProtocol searchLicenseProtocol = new org.iso.mpeg.mxm.searchlicensePE.technology.local.SearchLicenseProtocol(orchestratorEngine);
			//is it remote or local????
			SearchLicenseProtocol searchLicenseProtocol = searchLicenseEngine.getSearchLicenseTechnologyHandler().getRemoteSearchLicenseProtocol();
			SearchLicenseResponse searchLicenseResponse = searchLicenseProtocol.searchLicense(searchLicenseRequest);
			System.out.println("Received TID in Response: " + searchLicenseResponse.getTransactionIdentifier());
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
}
