/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package DMAGTests;

import java.io.File;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

//import org.iso.mpeg.mxm.basePE.LicenseEntity;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMInvalidConfigurationException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine;
import org.iso.mpeg.mxm.orchestratorTE.OrchestratorTE;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
import org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine;
import org.iso.mpeg.mxm.pEngine.createlicensePE.CreateLicenseEngine;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest;
import org.iso.mpeg.mxm.pEngine.revokelicensePE.RevokeLicenseEngine;
import org.iso.mpeg.mxm.pEngine.revokelicensePE.schemahandler.RevokeLicenseRequest;
import org.iso.mpeg.mxm.pEngine.revokelicensePE.technologyhandler.RevokeLicenseProtocol;
import org.iso.mpeg.mxm.pEngine.storelicensePE.StoreLicenseEngine;
import org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseRequest;
import org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseResponse;
import org.iso.mpeg.mxm.pEngine.storelicensePE.technologyhandler.StoreLicenseProtocol;
//import org.iso.mpeg.mxm.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.exception.RELEngineException;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
//import org.iso.mpeg.es.StoreLicenseES.StoreLicenseES;

/**
 * @author Jonathan Florido, DMAG-UPC <jflorido@ac.upc.edu>
 *
 */
public class StoreLicenseTest
{	
	public static void main(String[] args)
	{
		URL serviceURL = null;
		QName serviceName = null;
		try
		{
			InputStream is = StoreLicenseTest.class.getClassLoader().getResourceAsStream("MXMConfiguration.xml");
			MXM mxm = MXM.getInstance(is);
			
			StoreLicenseEngine storeLicenseEngine = 
					(StoreLicenseEngine)mxm.getDefaultEngine(MXMEngineName.StoreLicensePE.value());

			RELEngine relEngine = (RELEngine)mxm.getDefaultEngine(MXMEngineName.RELTE.value());
			MetadataEngine metadataEngine = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.value());
			
			List<MXMEngine> deps1 = new ArrayList<MXMEngine>();
			deps1.add(relEngine);
			deps1.add(metadataEngine);
			storeLicenseEngine.setDependencies(deps1);
			
			StoreLicenseRequest storeLicenseRequest = 
					storeLicenseEngine.getStoreLicenseSchemaHandler().getStoreLicenseRequest();
			storeLicenseRequest.setTransactionIdentifier("Store License Test");

			License licenseParser = relEngine.getRELEngineSchemaHandler().getLicense();
			File licenseFile = new File(StoreLicenseTest.class.getClassLoader().getResource("licenseSpec.xml").getFile());
			MXMObject licenseObject = new MXMAdapter(licenseFile);
			licenseParser.parse(licenseObject);
			
			//LicenseEntity licenseEntity = new LicenseEntity(relEngine, metadataEngine);
			LicenseEntity licenseEntity = storeLicenseRequest.getLicenseEntity();
			licenseEntity.setLicense(licenseParser);
			storeLicenseRequest.setLicenseEntity(licenseEntity);

			System.out.println("Sending TID in Request: " + storeLicenseRequest.getTransactionIdentifier());
			
			OrchestratorEngine orchestratorEngine = 
					(OrchestratorEngine)mxm.getDefaultEngine(MXMEngineName.Orchestrator.value());
			List<MXMEngine> deps = new ArrayList<MXMEngine>();
			deps.add(storeLicenseEngine);
			orchestratorEngine.setDependencies(deps);
			//StoreLicenseProtocol storeLicenseProtocol = new org.iso.mpeg.mxm.storelicensePE.technology.local.StoreLicenseProtocol(orchestratorEngine);
			//is it remote or local????
			StoreLicenseProtocol storeLicenseProtocol = storeLicenseEngine.getStoreLicenseTechnologyHandler().getRemoteStoreLicenseProtocol();
			StoreLicenseResponse storeLicenseResponse = storeLicenseProtocol.storeLicense(storeLicenseRequest);
			
			/*StoreLicenseES storeES = new StoreLicenseES();
			String request = (String)storeLicenseRequest.createObject(String.class.getCanonicalName()).getContent();
			String response = storeES.StoreLicense(request);*/
			
			System.out.println("Received TID in Response: " + storeLicenseResponse.getTransactionIdentifier());
			//System.out.println(response);
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
}
