/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package UserAuthenticator;

import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.namespace.QName;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMInvalidConfigurationException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.AuthenticateUserEngine;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserRequest;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserResponse;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.technologyhandler.AuthenticateUserProtocol;

/**
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 *
 */
public class RemoteUserAuthenticator
{	
	public static void main(String[] args)
	{
		URL serviceURL = null;
		QName serviceName = null;
		try
		{
			InputStream is = RemoteUserAuthenticator.class.getClassLoader().getResourceAsStream("MXMConfiguration.xml");
			MXM mxm = MXM.getInstance(is);
			
			AuthenticateUserEngine authenticateUserEngine = 
					(AuthenticateUserEngine)mxm.getDefaultEngine(MXMEngineName.AuthenticateUserPE.value());
			
			AuthenticateUserRequest authenticateUserRequest = 
					authenticateUserEngine.getAuthenticateUserSchemaHandler().getAuthenticateUserRequest();
			authenticateUserRequest.setTransactionIdentifier("AUTH USER TID");
			
			System.out.println("Sending TID in Request: " + authenticateUserRequest.getTransactionIdentifier());
			
			serviceURL = new URL(authenticateUserEngine.getPropertyValue("SERVICE_URL"));
			serviceName = new QName(authenticateUserEngine.getPropertyValue("SERVICE_NAME_NS"), 
					authenticateUserEngine.getPropertyValue("SERVICE_NAME"));
			
			AuthenticateUserProtocol authenticateUserProtocol = authenticateUserEngine.getAuthenticateUserTechnologyHandler()
					.getRemoteAuthenticateUserProtocol();
			authenticateUserProtocol.initServiceParams(serviceURL, serviceName);
			
			AuthenticateUserResponse authenticateUserResponse = 
					authenticateUserProtocol.authenticateUser(authenticateUserRequest);
			
			System.out.println("Received TID in Response: " + authenticateUserResponse.getTransactionIdentifier());
			
		} catch (MXMConfigurationException e)
		{
			e.printStackTrace();
		} catch (MXMEngineException e)
		{
			e.printStackTrace();
		} catch (MXMInvalidConfigurationException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MalformedURLException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
}
