/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino <angelo@cedeo.net>
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;

/**
 * This is the abstract class at a highest level of an MPEGMEngine. Every MPEGM Engines extend this abstract class.
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 * @author Angelo Difino <angelo@cedeo.net>
 * @author Az Mousas <azmousas@icbnet.ece.ntua.gr>
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 */
public abstract class MXMAbstractEngine implements MXMEngine
{
	protected MXMGenericParameters mxmParameters = null;
	
	protected String engineID = null;
	private boolean isTheDefaultEngine = true;
	
	private Logger logger = Logger.getLogger(MXMAbstractEngine.class);
	
	public void initialise(String engineID, MXMGenericParameters mxmParameters) {
		this.engineID = engineID;
		this.mxmParameters = mxmParameters;
	}
	
	public void initialise(String engineID, MXMGenericParameters mxmParameters, boolean isTheDefaultEngine) {
		this.engineID = engineID;
		this.mxmParameters = mxmParameters;
		this.isTheDefaultEngine = isTheDefaultEngine;
	}

	public void printInfo() {
		logger.info("\n-------------- MXM Engine --------------");
		logger.info("-- Engine name: " + this.getEngineName().toString() + " --");
		logger.info("-- Engine ID: " + engineID);
	}

	public void printProperties() {
		if (mxmParameters != null)
			mxmParameters.printProperties();
		else
			logger.warn("No MXM Parameters were set.");
	}

	public String getEngineID() {
		return this.engineID;
	}

	public String getPropertyValue(String key) throws MXMInvalidConfigurationException {
        if (mxmParameters!=null) {
            return mxmParameters.getProperty(key);
        } else {
            throw new MXMInvalidConfigurationException("Engine mxmParameters ("+key+") not initialized");
        }
    }

	public void setProperty(String key, String value) {
		mxmParameters.setProperty(key, value);
	}
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMEngine#isDefault()
	 */
	@Override
	public boolean isDefault()
	{
		return isTheDefaultEngine;
	}
}
