/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino <angelo@cedeo.net>
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core;

import java.io.File;

/**
 * This interface specifies common methods to all classes that may be 
 * employed to create a MPEGM-conformant data
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 * @author Angelo Difino <angelo@cedeo.net>
 */

public interface MXMCreator {

	/**
	 * This method is used to add an element enclosed in an MXMObject to an existing MXMCreator
	 * @param object the object to be added to the MXMParser
	 * 
	 * @return MXMEngineResponse.SUCCESS if the object was successfully added
	 */
	//public MXMEngineResponse addObject(MXMObject object);

	/**
	 * This method is employed to store the result of the creation in a file given in input
	 * 
	 * @param f the File where the content shall be stored
	 * @throws MXMException 
	 * 
	 */
	public void createFile(File f) throws MXMException ;

	/**
	 * This method is employed to create an MXMObject containing the value of the object 
	 * implementing this interface.
	 * 
	 * @param objectType the type of object to be returned
	 * @return an MXMObject wrapping the object that was created
	 * @throws MXMException 
	 * 
	 */
	public MXMObject createObject(String objectType) throws MXMException ;


}
