package org.iso.mpeg.mxm.core;

import java.util.List;

import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;


/**
 * This is the interface at a highest level of an MPEGMEngine. Every MPEGM Engines extend this interface.
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 * @author Angelo Difino <angelo@cedeo.net>
 * @author Az Mousas <azmousas@icbnet.ece.ntua.gr>
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 */
public interface MXMEngine
{
	public void initialise(String engineID, MXMGenericParameters mxmParameters);
	
	public void initialise(String engineID, MXMGenericParameters mxmParameters, boolean isTheDefaultEngine);
	
	public void printInfo();
	
	public void printProperties();
	
	public String getEngineID();
	
	public String getPropertyValue(String key) throws MXMInvalidConfigurationException;
	
	public void setProperty(String key, String value);
	
	/**
	 * This method is employed to retrieve the specific type of an MXMEngine, whose list in specified
	 * by the <code>MXMEngineName</code> enumeration.
	 * 
	 * @return the name (or type) of an MXMEngine among those defined in <code>MXMEngineName</code>.
	 */
	public String getEngineName() ;
	
	public void setDependencies(List<MXMEngine> engines) throws MXMEngineException;
	
	public boolean isDefault();
}
