/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is the Turin Chillout task force ;-)  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: the Turin Chillout task force ;-)  
 * Contributor(s): Angelo Difino <angelo@cedeo.net>
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core;

/**
 * This is the basic interface for most of the MXM classes. An MXMObject is a wrapper of specific objects
 * that can therefore be exchanged by MXM Engines. 
 * It defines the version of MXM and it provides a number of fundamental methods to know if an MXMObject is 
 * actually wrapping an object, what is the class name of the wrapped object and a getter method for 
 * obtaining it.
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 * @author Walter Allasia <allasia@eurix.it>
 * @author Francesco Gallo <gallo@eurix.it>
 * @author Angelo Difino <angelo@cedeo.net>
 */

public interface MXMObject {

	public String mxmVersion = "0.0.1";
	
	/**
	 * This method is used to obtain the current version of MXM.
	 * 
	 * @return the current version of MXM.
	 */
	public String getMxmVersion() ;

	/**
	 * This method is used to determine whether the MXMObject is actually wrapping an object.
	 * 
	 * @return true if the MXMObject is actually wrapping an instance of a class.
	 */
	public boolean hasContent() ;

	/**
	 * This method is used to obtain the object wrapped by an MXMObject.
	 * 
	 * @return the object wrapped by the MXMObject, if any, or null otherwise.
	 */
	public Object getContent() ;

	/**
	 * This method is employed to get the type of the object, i.e. the canonical name of its class.
	 * 
	 * @return the canonical name of the class of the object wrapped by the MXMObject.
	 */
	public String getContentType() ;
		
}
