/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino <angelo@cedeo.net>
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core;

import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;

/**
 * This interface specifies methods common to all classes that may be 
 * employed to parse MXM-conformant data
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 * @author Angelo Difino <angelo@cedeo.net>
 */

public interface MXMParser {

	/**
	 * This method is employed to parse an object in order to obtain the information contained within.
	 * 
	 * @param object an MXMObject wrapping an object that the specific MXMParser shall be able to parse
	 * @return MXMEngineResponse.SUCCESS if the parsing was successful.
	 * @throws RELParsingException 
	 */
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException ;

	/**
	 * This method is employed to determine whether an MXMParser is equals to another.
	 * 
	 *  @param another another MXMParser to be compared with.
	 *  @return MXMEngineResponse.SUCCESS if the two MXMParsers are equals, MXMEngineResponse.FAILURE if different,
	 *  MXMEngineResponse.METHOD_NOT_IMPLEMENTED if the method has not been implemented
	 * 
	 */
	//public MXMEngineResponse equals(MXMParser another) throws MXMEngineException ;

	
}
