/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core.configuration;

import java.util.List;

import org.apache.log4j.Logger;

import org.iso.mpeg.mxm.configuration.schema.MXMParameterType;

/**
 * MPEGMParameters
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 * @author Filippo Chiariglione
 */

public class MXMParameters extends MXMGenericParameters{

	private Logger logger = Logger.getLogger(MXMParameters.class);

	private MXMParameterType mxmParametersType = null;
	
	public MXMParameters (MXMParameterType mxmParametersType) throws MXMConfigurationException {
		super.initialise(mxmParametersType);
		this.mxmParametersType = mxmParametersType;
	}

	public MXMParameters () {
		super();
	}

	public List<String> getMPEGMEnginesFolders() {
		return this.mxmParametersType.getMXMEnginesFolder();
	}

	public void addMPEGMEnginesFolders(List<String> mxmEnginesFolders) {
		this.mxmParametersType.getMXMEnginesFolder().addAll(mxmEnginesFolders);
	}

	public void addMPEGMEnginesFolder(String mxmEnginesFolder) {
		this.mxmParametersType.getMXMEnginesFolder().add(mxmEnginesFolder);
	}

	public void printProperties() {
		super.printProperties();
		logger.info("\n-- Specific Properties:");
		for (String s : getMPEGMEnginesFolders())
			logger.info("---- MXM Engine Folder: " + s);
	}

}