/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core.engine.management;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;

/**
 * MPEGMEngineList
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 * @author Filippo Chiariglione
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 */

public class MXMEngineList extends MXMAdapter {

	private String engineListName = null;
	private List<MXMEngine> engineList = null;
	
	private Logger logger = Logger.getLogger(MXMEngineList.class);
	
	public MXMEngineList(String engineListName) {
		this.engineListName = engineListName;
		engineList = new ArrayList<MXMEngine>();
	}

	/**
	 * Each EngineList has the same name as the MXM engines contained.
	 * e.g. the EngineList containing 'MediaFrameworkEngine' items 
	 * will return 'MediaFrameworkEngine'.
	 * @return
	 */
	public String getEngineListName() {
		return engineListName;
	}

	public List<MXMEngine> getEngineList() {
		return engineList;
	}

	public void setEngineList(List<MXMEngine> engineList) {
		this.engineList = engineList;
	}
	
	public void addEngine(MXMEngine engine) {
		engineList.add(engine);
	}

	public MXMEngine getEngine(String engineID) {
		for (MXMEngine engine : engineList) {
			if (engine.getEngineID().equals(engineID))
				return engine;
		}
		logger.error("An engine with ID: " + engineID + " is not present in the " + engineListName + " list.");
		return null;
	}
	
	public MXMEngine getDefaultEngine() {
		if (engineList.size() == 0) {
			logger.error("There are no " + engineListName + "s currently configured.");			
		}
		for (MXMEngine mxmEngine : engineList)
		{
			if (mxmEngine.isDefault())
			{
//				logger.info("The default engine has Id: " + mxmEngine.getEngineID());
				return mxmEngine;
			}
		}
		logger.error("Could not find a MXMEngine set as Default; selecting the first one...");
		return engineList.get(0);
	}
	
	public void listEngines() {
		for (MXMEngine engine : engineList)
			engine.printInfo();
	}

	public void printInfoAndProperties() {
		for (MXMEngine engine : engineList) {
			engine.printInfo();
			engine.printProperties();
		}
	}
	
	public boolean contains(String engineID) {
		for (MXMEngine engine : engineList) {
			if (engine.getEngineID().equals(engineID))
				return true;
		}
		return false;
	}

}
