/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core.engine.management;

import java.util.List;



import org.iso.mpeg.mxm.configuration.schema.MXMEngineType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;

import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;

/**
 * MPEGMEngineManager
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 * @author Filippo Chiariglione
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 */

public class MXMEngineManager extends MXMAdapter {

	private static MXMEngineManager engineManager = null;
	
	private static MXMEngineRegistry engineRegistry = null; 
	
	private MXMEngineManager() {
		engineRegistry = new MXMEngineRegistry();
	}
	
	public static MXMEngineManager getInstance() {
		if (engineManager == null) {
			engineManager = new MXMEngineManager();
		}
		return engineManager;
	}

	public static MXMEngineManager getInstance(List<MXMEngineType> engineElements) throws MXMConfigurationException, MXMEngineException {
		if (engineManager == null) {
			engineManager = new MXMEngineManager();
			initialise(engineElements);
		}
		return engineManager;
	}
	
    public static void killInstance()
    {
        engineRegistry = null;
        engineManager = null;
    }
    
    private static void initialise(List<MXMEngineType> engineElements) throws MXMConfigurationException, MXMEngineException
    {
    	MXMEngineLoader mxmEngineLoader = new MXMEngineLoader();
    	List<MXMEngine> enginesLoaded = mxmEngineLoader.load(engineElements);
    	for (MXMEngine engineLoaded : enginesLoaded)
    	{
    		engineRegistry.addEngine(engineLoaded);
    	}
    }
	
//	private static void initialise(List<MXMEngineType> engineElements) throws MXMConfigurationException {
//		
//		for (MXMEngineType engineElement : engineElements) {
//			MXMEngineLoader mxmEngineLoader = new MXMEngineLoader();
//			engineRegistry.addEngine(mxmEngineLoader.load(engineElement));
//		}
//	}
	
	public void setEngineElements(List<MXMEngineType> engineElements) throws MXMConfigurationException, MXMEngineException {
		initialise(engineElements);
	}

	public MXMEngine getEngine(String engineName, String engineID) {
		return engineRegistry.getEngine(engineName, engineID);
	}
	
	public MXMEngine getDefaultEngine(String engineName) {
		return engineRegistry.getDefaultEngine(engineName);
	}

	public void listEngines() {
		engineRegistry.listEngines();
	}

	public void printInfoAndProperties() {
		engineRegistry.printInfoAndProperties();
	}
	
	public MXMEngineRegistry getMxmEngineRegistry()
	{
		return engineRegistry;
	}

}
