/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core.engine.management;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;

/**
 * MPEGMEngineRegistry
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 * @author Filippo Chiariglione
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 */

public class MXMEngineRegistry extends MXMAdapter {
	
	private List<MXMEngineList> engineLists = null;
	
	private Logger logger = Logger.getLogger(MXMEngineRegistry.class);
	
	public MXMEngineRegistry() {
		engineLists = new ArrayList<MXMEngineList>();
	}

	public void addEngine(MXMEngine engine) throws MXMConfigurationException {
		String engineName = engine.getEngineName();
		
		//Each Engine must have an Engine ID. If an MXM engine comes without
		//an ID, a random ID will be given by default.
		String engineID = engine.getEngineID();

		//let's see if there is already a list of MXM engines of the same type
		boolean found = false;
		for (MXMEngineList engineList : engineLists) {
			if (engineList.getEngineListName().equals(engineName)) {
				
				if (engineList.contains(engineID)) {
					throw new MXMConfigurationException("There are two " + engineName.toString() + " having EngineID: " + engineID);
				}
				
				engineList.addEngine(engine);
				found = true;
				break;
			}
		}
		
		if (!found) {
			//We need to add a new list of MXM engines of this new type
			MXMEngineList engineList = new MXMEngineList(engineName);
			engineList.addEngine(engine);
			engineLists.add(engineList);
		}
	}

	public MXMEngineList getEngineList(String engineName) {
		for (MXMEngineList engineList : engineLists) {
			if (engineList.getEngineListName().equals(engineName))
				return engineList;
		}
		logger.warn("There are no " + engineName.toString());
		return null;
	}

	public MXMEngine getEngine(String engineName, String engineID) {
		for (MXMEngineList engineList : engineLists) {
			if (engineList.getEngineListName().equals(engineName)) {
				return engineList.getEngine(engineID);
			}
		}
		logger.warn("There are no " + engineName.toString() + "\nhaving Engine ID: " + engineID);
		return null;
	}

	public MXMEngine getDefaultEngine(String engineName) {
		MXMEngineList engineList = getEngineList(engineName);
		return engineList.getDefaultEngine();
	}

	public void listEngines() {
		for (MXMEngineList engineList : engineLists)
			engineList.listEngines();
	}

	public void printInfoAndProperties() {
		for (MXMEngineList engineList : engineLists)
			engineList.printInfoAndProperties();
	}
}
