package org.iso.mpeg.mxm.core.event.management;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: aca
 * Date: 8/22/11
 * Time: 6:37 PM
 */
public class MXMEventHandler
{
	public static final int HIGH_PRIORITY = 2;
	public static final int MEDIUM_PRIORITY = 1;
	public static final int LOW_PRIORITY = 0;
	
	private int maxPriority = 0;
	
    Map<String, List<MXMEventListener>> listeners;
    
    Map<String, Map<Integer, List<MXMEventListener>>> prioritizedListeners;

    public MXMEventHandler()
    {
        listeners = new HashMap<String, List<MXMEventListener>>();
        prioritizedListeners = new HashMap<String, Map<Integer,List<MXMEventListener>>>();
    }

//    public void registerMXMEventListener(String eventType, MXMEventListener listener)
//    {
//        List<MXMEventListener> eventListenerList = listeners.get(eventType);
//        
//        if (eventListenerList==null)
//        {
//        	eventListenerList = new ArrayList<MXMEventListener>();
//        }
//        
//        eventListenerList.add(listener);
//
//        listeners.put(eventType, eventListenerList);
//        System.out.println("[MXMEvent] Class " + listener.getClass().getName() + " registered as " + eventType + " Listener");
//    }
    
    public void registerMXMEventListener(String eventType, MXMEventListener listener)
    {
    	registerMXMEventListener(eventType, listener, MEDIUM_PRIORITY);
    }
    
    public void registerMXMEventListener(String eventType, MXMEventListener listener, int priority)
    {
    	Map<Integer, List<MXMEventListener>> prioritizedEventListenerMap = prioritizedListeners.get(eventType);
    	
    	if (prioritizedEventListenerMap == null)
    	{
    		prioritizedEventListenerMap = new HashMap<Integer, List<MXMEventListener>>();
    	}
    	
    	List<MXMEventListener> eventListenerList = prioritizedEventListenerMap.get(new Integer(priority));
        
        if (eventListenerList == null)
        {
        	eventListenerList = new ArrayList<MXMEventListener>();
        }
        
        eventListenerList.add(listener);
        prioritizedEventListenerMap.put(new Integer(priority), eventListenerList);
        
        prioritizedListeners.put(eventType, prioritizedEventListenerMap);
        
        if (priority > maxPriority)	
        {
        	maxPriority = priority;
        }

        System.out.println("[MXMEvent] Class " + listener.getClass().getName() + " registered as " + eventType + " Listener");
    }

    public void unregisterMXMEventListener(String eventType, MXMEventListener listener)
    {
        List<MXMEventListener> eventListenerList = listeners.get(eventType);
        eventListenerList.remove(listener);

        listeners.put(eventType, eventListenerList);
    }

//    public void fireMXMEvent(MXMEvent event)
//    {
//        String eventClassName = event.getClass().getName();
//
//        List<MXMEventListener> eventListenerList = listeners.get(eventClassName);
//
//        if (eventListenerList != null)
//        {
//            for (MXMEventListener listener : eventListenerList)
//            {
//                listener.mxmEventOccurred(event);
//            }
//        }
//        else
//        {
//            System.out.println("[MXMEvent] No Listeners Registered for Event: " + eventClassName);
//        }
//    }
    
    public void fireMXMEvent(MXMEvent event)
    {
    	String eventClassName = event.getClass().getName();
    	
    	Map<Integer, List<MXMEventListener>> prioritizedEventListenerMap = prioritizedListeners.get(eventClassName);
    	
    	if (prioritizedEventListenerMap != null)
    	{
        	// invoke listeners according to priorities
        	for (int i = maxPriority; i >= 0; i --)
        	{
        		List<MXMEventListener> eventListeners = prioritizedEventListenerMap.get(new Integer(i));
        		if (eventListeners != null)
        		{
        			for (MXMEventListener eventListener : eventListeners)
        			{
        				eventListener.mxmEventOccurred(event);
        			}
        		}
        	}
    	}
    }
}
