/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler;

import java.util.List;

import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didmodel_mp21.Descriptor;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;

/**
 * This interface provides the methods to create and parse a Create Content Request and follows
 * the specification of the Create Content Protocol in ISO/IEC 23006-4: 
 * Information technology — Multimedia service platform technologies — Part 4: Elementary services.
 * 
 * @author Angelos-Christos Anadiotis
 * @version 1.0.0
 */
public interface CreateContentRequest extends org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolRequest
{
	/**
	 * Get the Event Report Request ({@link ERR}) associated with the content to be created.
	 * 
	 * @return The {@link ERR} associated with the digital item to be created.
	 * @throws MXMEngineException 
	 */
	public ERR getERR() throws MXMEngineException;

	/**
	 * Pass an Event Report Request ({@link ERR}) to the content that is going to be created.
	 * 
	 * @param eRR The {@link ERR} to be inserted in the digital item.
	 * @throws MXMException 
	 */
	public void setERR(ERR eRR) throws MXMException;

	/**
	 * Get the metadata describing the content.
	 * 
	 * @return A {@link Descriptor} with the metadata describing the content.
	 * @throws MXMEngineException 
	 */
	public Descriptor getMetadata() throws MXMEngineException;

	/**
	 * Set the metadata describing the content to be created.
	 * 
	 * @param descriptor A {@link Descriptor} with the metadata describing the content to be created.
	 * @throws MXMException 
	 */
	public void setMetadata(Descriptor descriptor) throws MXMException;

	/**
	 * A list of one or more {@link Asset}s that should generate the content.
	 * 
	 * @return A reference to a list of one or more {@link Asset}s that should generate the content.
	 * @throws MXMEngineException 
	 */
	public List<Asset> getAsset() throws MXMEngineException;
	
	/**
	 * Add an {@link Asset} that should generate the content.
	 * 
	 * @param asset The {@link Asset} that should be added among the ones to generate the content.
	 * @throws MXMException 
	 */
	public void addAsset(Asset asset) throws MXMException;
}