/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.pEngine.createcontentPE.technologyhandler;

import java.net.URL;

import javax.xml.namespace.QName;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.pEngine.createcontentPE.exceptions.CreateContentEngineException;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse;

/**
 * This protocol is used by the Create Content Elementary Service, which enables a User to create a Digital Item. 
 * In order to achieve that, several data should be provided, specified in the {@link CreateContentRequest}.
 * This protocol may be called either locally, in case the technology engines need to create a Digital Item 
 * are implemented in the same device, or remotely, in which case we make a service call to the 
 * Create Content Elementary Service.
 * 
 * This protocol is based on the Create Content Elementary Service Protocol, specified in ISO/IEC 23006-4: 
 * Information technology — Multimedia service platform technologies — Part 4: Elementary services.
 * 
 * @author Angelos-Christos Anadiotis
 * @version 1.0.0
 */
public interface CreateContentProtocol 
{
	/**
	 * This method is used only in case we want to make a remote call to the Create Content Elementary Service.
	 * Otherwise, a {@link MXMEngineResponse} METHOD_NOT_IMPLEMENTED must be returned.
	 * 
	 * @param serviceURL The URL of the Create Content Elementary Service. This URL should point to the WSDL location.
	 * @param serviceName The Create Content Elementary Service {@link QName}.
	 * @return {@link MXMEngineResponse} indicating whether the method was successful or not, or event if it has been
	 * implemented at all.
	 */
	public MXMEngineResponse initServiceParams(URL serviceURL, QName serviceName);
	
	/**
	 * This is the fundamental operation of the Create Content protocol. When the caller's device has the necessary
	 * technology to create the digital item, it just makes a call to the technology engines needed to create the digital item. 
	 * Note that this method is also called by the Create Content Elementary Service on the server part, as that part
	 * handles the technology to create the digital item. In case this method is implemented on the client side, then
	 * it makes the call to the Create Content Elementary Service and returns the response back to the client. 
	 * 
	 * @param request The {@link CreateContentRequest} that contains the request parameters.
	 * @return The {@link CreateContentResponse} that contains the response parameters.
	 * @throws MXMException 
	 */
	public CreateContentResponse createContent(CreateContentRequest request) throws CreateContentEngineException;
}
