/**
 * The copyright in this software is being made available under the BSD License,
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 * 
 * Copyright (c) 2012, CEDEO
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this 
 *     list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *     - Neither the name of the CEDEO nor the names of its contributors may be used 
 *     to endorse or promote products derived from this software without specific 
 *     prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS �AS IS� AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
*/

package org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl;

import java.util.List;

import org.iso.mpeg.mxm.core.MXMCreatorAndParser;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_mp21.Declarations;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;

/**
 * Profiled version of the standard MP21 DIDL structure, defined in the progress of MPEG-M (and MSAF)
 * It extends the standard one, available on didl_mp21 package, and:
 * -add the DI Signature in DIDLInfo
 * -restrict some Item/Statement information
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 */
public interface DIDL extends MXMCreatorAndParser, org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_mp21.DIDL
{

	/** 
	 * Get specific list of DIDLInfo XML fragment, mainly containing the Signature of the whole DI
	 * (MPEG-M profile) 
	 * @return  <code>List<DIDLInfo></code> with all the information contained in the XML fragment
	 * @throws MXMEngineException 
	 */
	public List<DIDLInfo> getMPEGMDIDLInfo() throws MXMEngineException;
	
	/** 
	 * Set specific list of DIDLInfo XML fragment, mainly containing the Signature of the whole DI
	 * (MPEG-M profile) 
	 * @return  <code>List<DIDLInfo></code> with all the information contained in the XML fragment
	 * @throws MXMException 
	 */
	public void setMPEGMDIDLInfo(List<DIDLInfo> infos) throws MXMException;
	
	/** 
	 * Get specific Declarations XML fragment, refers to the official mp21 PROFILE
	 * @return  <code>Declarations</code> with all the information contained in the XML fragment
	 */
	public Declarations getDeclarations();

	/** 
	 * Set specific Declarations XML fragment, refers to the official mp21 PROFILE
	 * @param  <code>Declarations</code> with all the information contained in the XML fragment
	 */
	public void setDeclarations(Declarations declarations);

	/** 
	 * Set specific Container XML fragment (MPEG-M profile)
	 * @param  <code>Container</code> with all the information contained in the XML fragment
	 */
	public void setMPEGMContainer(Container container);

	/** 
	 * Get specific Container XML fragment (MPEG-M profile)
	 * @return  <code>Container</code> with all the information contained in the XML fragment
	 */	
	public Container getMPEGMContainer();
	
	/** 
	 * Get specific DocumentId
	 * @return  <code>String</code> with specific DocumentId
	 */		
	public String getDIDLDocumentId();

	/** 
	 * Set specific DocumentId
	 * @param  <code>String</code> with specific DocumentId
	 */		
	public void setDIDLDocumentId(String string);

	/** 
	 * Get specific Item XML fragment (MPEG-M profile)
	 * @return  <code>Item</code> with all the information contained in the XML fragment
	 * @throws MXMEngineException 
	 */	
	public Item getMPEGMItem() throws MXMEngineException;

	/** 
	 * Set specific Item XML fragment (MPEG-M profile)
	 * @param  <code>Item</code> with all the information contained in the XML fragment
	 * @throws MXMException 
	 */	
	public void setMPEGMItem(Item item) throws MXMException;
	
	
	
	/*********************************************************************************************************************************************SHORTCUT METHODS*/
	
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Set specific Signature of the DIDL
	 * This is a shortcut method for access the first DIDLInfo contains Signature
	 * @return  <code>Signature</code> with all the information contained in the XML fragment
	 * @throws MXMEngineException 
	 */		
	public Signature getSignature() throws MXMEngineException;
	
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Set specific Signature of the DIDL
	 * This is a shortcut method to set the first DIDLInfo and the passed Signature 
	 * @param  <code>Signature</code> with all the information contained in the XML fragment
	 * @throws MXMEngineException 
	 * @throws MXMException 
	 */		
	public void setSignature(Signature signature) throws MXMEngineException, MXMException;
}