/**
 * The copyright in this software is being made available under the BSD License,
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 * 
 * Copyright (c) 2012, CEDEO
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this 
 *     list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *     - Neither the name of the CEDEO nor the names of its contributors may be used 
 *     to endorse or promote products derived from this software without specific 
 *     prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
*/

package org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl;

import java.util.List;


import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;

import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didmodel_mp21.DIDBase;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.dii.RelatedIdentifier;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.IPMPGeneralInfoDescriptor;
//import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Mpeg7;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
/**
 * Profiled version of the standard MP21 Item structure, defined in the progress of MPEG-M (and MSAF)
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 */
public interface Item extends org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didmodel_mp21.Item
{
	/**
	 * Set specific ID of the item
	 */
	public void setId(String string);

	/**
	 * Get specific ID of the item
	 */
	public String getId();
	
	/**
	 * Get ItemOrComponent inside of the XML fragment
	 */
	public List<DIDBase> getItemOrComponent();

	/**
	 * Get getDescriptor inside of the XML fragment
	 * @throws MXMException 
	 */
	public List<Descriptor> getDescriptor() throws MXMEngineException, MXMException;


	
	/*********************************************************************************************************************************************PARTICULAR METHODS*/

	//shortcut from getItemOrComponent
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Get specific inner List<Item> of the Item (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getItemOrComponent) the Item as managed as Object, instead of using MPEGM PROFILED Item
	 * @return  <code>List<Item></code> with all the information contained in the XML fragment
	 * @throws MXMException 
	 */			
	public List<Item> getItem()  throws MXMEngineException, MXMException;
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner Item  (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getItemOrComponent) the Item as managed as Object, instead of using MPEGM PROFILED Item
	 * @param  <code>Item</code> with all the information contained in the XML fragment
	 */			
	public void addItem(Item i)  throws MXMException;
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner List<Component>  (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getItemOrComponent) the Component as managed as Object, instead of using MPEGM PROFILED Component
	 * @return  <code>List<Component></code> with all the information contained in the XML fragment
	 * @throws MXMException 
	 */				
	public List<Component> getComponent() throws MXMEngineException, MXMException;
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner Component  (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getItemOrComponent) the Component as managed as Object, instead of using MPEGM PROFILED Component
	 * @param  <code>Component</code> with all the information contained in the XML fragment
	 */			
	public void addComponent(Component i) throws MXMException;
	
	/*********************************************************************************************************************************************SHORTCUT METHODS*/

	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Get specific Identifier of the Item
	 * This is a shortcut method for access Descriptor->Statement->Identifier
	 * @return  <code>String</code> with all the information contained in the XML fragment
	 */		
	public String getIdentifier() throws MXMException ;
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Set specific Identifier of the Item
	 * This is a shortcut method for access Descriptor->Statement->Identifier
	 * @param  <code>String</code> with all the information contained in the XML fragment
	 */		
	public void setIdentifier(String value) throws MXMException ;
	
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Get specific List<ERR> of the Item
	 * This is a shortcut method for access Descriptor->Statement->ERR
	 * @return  <code>List<ERR></code> with all the information contained in the XML fragment
	 */		
	public List<ERR> getERR() throws MXMException ;
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Set specific ERR of the Item
	 * This is a shortcut method for access Descriptor->Statement->ERR
	 * @param  <code>ERR</code> with all the information contained in the XML fragment
	 */		
	public void addERR(ERR err) throws MXMException ;
	
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Get specific List<RelatedIdentifier> of the Item
	 * This is a shortcut method for access Descriptor->Statement->RelatedIdentifier
	 * @return  <code>List<RelatedIdentifier></code> with all the information contained in the XML fragment
	 */		
	public List<RelatedIdentifier> getRelatedIdentifier() throws MXMEngineException, MXMException;
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Set specific RelatedIdentifier of the Item
	 * This is a shortcut method for access Descriptor->Statement->RelatedIdentifier
	 * @param  <code>RelatedIdentifier</code> with all the information contained in the XML fragment
	 */		
	public void addRelatedIdentifier(RelatedIdentifier id) throws MXMException;
	
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Get specific StructuredData of the Item
	 * This is a shortcut method for access Descriptor->Statement->StructuredData
	 * @return  <code>StructuredData</code> with all the information contained in the XML fragment
	 * @throws MXMException 
	 */		
	public StructuredData getMetadata() throws MXMException;
	
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Set specific StructuredData of the Item
	 * This is a shortcut method for access Descriptor->Statement->StructuredData
	 * @param  <code>Mpeg7</code> with all the information contained in the XML fragment
	 */		
	public void addMetadata(StructuredData sd) throws MXMException;
	
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Get specific IPMPGeneralInfoDescriptor of the Item
	 * This is a shortcut method for access Descriptor->Statement->IPMPGeneralInfoDescriptor
	 * @return  <code>IPMPGeneralInfoDescriptor</code> with all the information contained in the XML fragment
	 */		
	public IPMPGeneralInfoDescriptor getIPMPGeneralInfoDescriptor() throws MXMEngineException, MXMException;
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Set specific IPMPGeneralInfoDescriptor of the Item
	 * This is a shortcut method for access Descriptor->Statement->IPMPGeneralInfoDescriptor
	 * @param  <code>IPMPGeneralInfoDescriptor</code> with all the information contained in the XML fragment
	 */		
	public void addIPMPGeneralInfoDescriptor(IPMPGeneralInfoDescriptor i) throws MXMException;		
	
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Get specific Signature of the Item
	 * This is a shortcut method for access Descriptor->Statement->Signature
	 * @return  <code>Signature</code> with all the information contained in the XML fragment
	 */		
	public Signature getContentElementSignature() throws MXMEngineException, MXMException;
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Set specific Signature of the Item
	 * This is a shortcut method for access Descriptor->Statement->Signature
	 * @param  <code>Signature</code> with all the information contained in the XML fragment
	 */		
	public void addContentElementSignature(Signature c) throws MXMException;
	
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Get specific List<LaserObj> of the Item
	 * This is a shortcut method for access Descriptor->Statement->LaserObj
	 * @return  <code>List<LaserObj></code> with all the information contained in the XML fragment
	 */		
	public byte[] getLaserObj() throws MXMEngineException;
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Set specific LaserObj of the Item
	 * This is a shortcut method for access Descriptor->Statement->LaserObj
	 * @param  <code>LaserObj</code> with all the information contained in the XML fragment
	 */			
	public void addLaserObj(byte[] l) throws MXMException;	
	
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Get specific List<ApplicationSpecificData> of the Item
	 * This is a shortcut method for access Descriptor->Statement->ApplicationSpecificData
	 * @return  <code>List<ApplicationSpecificData></code> with all the information contained in the XML fragment
	 */		
	public List<ApplicationSpecificData> getApplicationSpecificData() throws MXMEngineException, MXMException;
	/** 
	 * <b>***SHORTCUT METOD</b>
	 * Set specific ApplicationSpecificData of the Item
	 * This is a shortcut method for access Descriptor->Statement->ApplicationSpecificData
	 * @param  <code>ApplicationSpecificData</code> with all the information contained in the XML fragment
	 */			
	public void addApplicationSpecificData(ApplicationSpecificData l) throws MXMException;	
	
	
	public void addMetadataDescriptor(org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didmodel_mp21.Descriptor d) throws MXMException;
	public org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didmodel_mp21.Descriptor getMetadataDescriptor() throws MXMException;
	
}