/**
 * The copyright in this software is being made available under the BSD License,
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 * 
 * Copyright (c) 2012, CEDEO
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this 
 *     list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *     - Neither the name of the CEDEO nor the names of its contributors may be used 
 *     to endorse or promote products derived from this software without specific 
 *     prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
*/

package org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl;

import org.iso.mpeg.mxm.core.MXMCreatorAndParser;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
//import org.iso.mpeg.mxm.digitalitemTE.schemahandler.StructuredData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.dii.RelatedIdentifier;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.IPMPGeneralInfoDescriptor;
//import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Mpeg7;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;

import java.util.List;

/**
 * Profiled version of the standard MP21 Statement structure, defined in the progress of MPEG-M (and MSAF)
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 */
public interface Statement extends MXMCreatorAndParser
{
	public void setRef(String string);

	public void setMime(String string);

	public String getRef();

	public List<Object> getContent();

	public String getMime();
	
	/*********************************************************************************************************************************************PARTICULAR METHODS*/

	//schortcut for getContent:  can be
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Get specific inner Identifier of the Statement (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the Item as managed as Object, instead of using MPEGM PROFILED Identifier
	 * 
	 * was in previously: contentID
	 * 
	 * @return  <code>List<Item></code> with all the information contained in the XML fragment
	 */			
	public String getIdentifier();
	
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner Identifier  (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the Identifier as managed as Object, instead of using MPEGM PROFILED Identifier
	 * @param  <code>Identifier</code> with all the information contained in the XML fragment
	 */			
	public void setIdentifier(String value);
	
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Get specific inner List<ERR> of the Statement (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the ERR as managed as Object, instead of using MPEGM PROFILED ERR
	 * @return  <code>List<ERR></code> with all the information contained in the XML fragment
	 */			
	public List<ERR> getERR() throws MXMEngineException;
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner ERR  (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the ERR as managed as Object, instead of using MPEGM PROFILED ERR
	 * @param  <code>ERR</code> with all the information contained in the XML fragment
	 */			
	public void addERR(ERR err) throws MXMException;
	
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Get specific inner List<RelatedIdentifier> of the Statement (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the RelatedIdentifier as managed as Object, instead of using MPEGM PROFILED RelatedIdentifier
	 * @return  <code>List<RelatedIdentifier></code> with all the information contained in the XML fragment
	 */			
	public List<RelatedIdentifier> getRelatedIdentifier() throws MXMEngineException;
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner RelatedIdentifier  (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the RelatedIdentifier as managed as Object, instead of using MPEGM PROFILED RelatedIdentifier
	 * @param  <code>RelatedIdentifier</code> with all the information contained in the XML fragment
	 */		
	public void addRelatedIdentifier(RelatedIdentifier id) throws MXMException;
	
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Get specific inner StructuredData of the Statement (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the StructuredData as managed as Object, instead of using MPEGM PROFILED Mpeg7
	 * @return  <code>StructuredData</code> with all the information contained in the XML fragment
	 */			
	public StructuredData getMetadata() throws MXMException;
	
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner StructuredData  (MPEG/M PROFILE or MP7)
	 * This particular method avoid to cast (from getContent) the Mpeg7 as managed as Object, instead of using MPEGM PROFILED Mpeg7
	 * @param  <code>StructuredData</code> with all the information contained in the XML fragment
	 */		
	public void addMetadata(StructuredData sd) throws MXMException;

	
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Get specific inner IPMPGeneralInfoDescriptor of the Statement (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the IPMPGeneralInfoDescriptor as managed as Object, instead of using MPEGM PROFILED IPMPGeneralInfoDescriptor
	 * @return  <code>IPMPGeneralInfoDescriptor</code> with all the information contained in the XML fragment
	 */			
	public IPMPGeneralInfoDescriptor getIPMPGeneralInfoDescriptor() throws MXMEngineException;
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner IPMPGeneralInfoDescriptor  (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the IPMPGeneralInfoDescriptor as managed as Object, instead of using MPEGM PROFILED IPMPGeneralInfoDescriptor
	 * @param  <code>IPMPGeneralInfoDescriptor</code> with all the information contained in the XML fragment
	 */		
	public void addIPMPGeneralInfoDescriptor(IPMPGeneralInfoDescriptor i) throws MXMException;		
	
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Get specific inner Signature of the Statement (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the Signature as managed as Object, instead of using MPEGM PROFILED Signature
	 * @return  <code>Signature</code> with all the information contained in the XML fragment
	 */			
	public Signature getContentElementSignature() throws MXMEngineException;
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner Signature  (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the Signature as managed as Object, instead of using MPEGM PROFILED Signature
	 * @param  <code>Signature</code> with all the information contained in the XML fragment
	 */		
	public void addContentElementSignature(Signature c) throws MXMException;
	
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Get specific inner LaserObj> of the Statement (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the LaserObj as managed as Object, instead of using MPEGM PROFILED LaserObj
	 * @return  <code>LaserObj</code> with all the information contained in the XML fragment
	 */			
	public byte[] getLaserObj() throws MXMEngineException;
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner LaserObj  (MPWEG/M PROFILE)
	 * This particular method avoid to cast (from getContent) the LaserObj as managed as Object, instead of using MPEGM PROFILED LaserObj
	 * @param  <code>LaserObj</code> with all the information contained in the XML fragment
	 */		
	public void addLaserObj(byte[] l) throws MXMException;	
	
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Get specific inner List<ApplicationSpecificData> of the Statement (MPWEG/M PROFILE)
 	 * This method is used to get from a didl:Item a list of custom XML fragments.  
	 * This particular method avoid to cast (from getContent) the ApplicationSpecificData as managed as Object, instead of using MPEGM PROFILED ApplicationSpecificData
	 * @return  <code>List<ApplicationSpecificData></code> with all the information contained in the XML fragment
	 */			
	public List<ApplicationSpecificData> getApplicationSpecificData() throws MXMEngineException;
	/** 
	 * <b>***PARTICULAR METOD</b>
	 * Set specific inner ApplicationSpecificData  (MPWEG/M PROFILE)
 	 * This method is used to insert into a didl:Item a list of custom XML fragments, by putting them into a specific Statement inside a dedicated Descriptor. 
	 * This particular method avoid to cast (from getContent) the ApplicationSpecificData as managed as Object, instead of using MPEGM PROFILED ApplicationSpecificData
	 * @param  <code>ApplicationSpecificData</code> with all the information contained in the XML fragment
	 */		
	public void addApplicationSpecificData(ApplicationSpecificData l) throws MXMException;

		
}