/**
 * The copyright in this software is being made available under the BSD License,
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, UTI Systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *     - Redistributions of source code must retain the above copyright notice, this 
 *     list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *     - Neither the name of the UTI Systems nor the names of its contributors may be used 
 *     to endorse or promote products derived from this software without specific 
 *     prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 */

package org.iso.mpeg.mxm.tEngine.eventreportTE;

import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;

/**
 * Classes implementing this interface are entry point classes for operating on <tt>ISO/IEC 21000-15 Part 15:
 * Event Reporting</tt>, Event Report (ER) and Event Report Request (ERR) data structures
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 8/29/11
 * Time: 4:14 PM
 * <p/>
 * UTI Systems, TIC
 */
public interface EventReportEngine extends MXMEngine {
    /**
     * This method returns an instance of class <code>EventReportEngineSchemaHandler</code> which is
     * needed to parse/create a ER/ERR xml fragment
     *
     * @return an object of class <code>EventReportEngineSchemaHandler</code> or null if this method
     *         has not been implemented by the specific EventReportEngine.
     * @throws MXMEngineException In case of any internal exception occur.
     */
    public EventReportEngineSchemaHandler getEventReportEngineSchemaHandler() throws MXMEngineException;

    /**
     * This method returns an instance of class <code>EventReportEngineTechHandler</code> which is
     * needed to act technological methods on ER/ERR object
     *
     * @return an object of class <code>EventReportEngineTechHandler</code> or null if this method
     *         has not been implemented by the specific EventReportEngine.
     * @throws MXMEngineException In case of any internal exception occur.
     */
    public EventReportEngineTechnologyHandler getEventReportEngineTechHandler() throws MXMEngineException;

}
