/**
 * The copyright in this software is being made available under the BSD License,
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, UTI Systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *     - Redistributions of source code must retain the above copyright notice, this 
 *     list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *     - Neither the name of the UTI Systems nor the names of its contributors may be used 
 *     to endorse or promote products derived from this software without specific 
 *     prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 */

package org.iso.mpeg.mxm.tEngine.eventreportTE;

import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EREngineParams;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERMessageHandler;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERRKeeper;

import java.util.List;

/**
 * This class provides the Event Report Tehnology Handlers.
 * It relies on the available functionalities the Event Report Engine can employ on the data he manages.
 *
 * @author <a href="mailto:bogdan.ardeleanu@uti.ro">Bogdan Ardeleanu</a>, UTI Systems
 */
public interface EventReportEngineTechnologyHandler {

    /**
     * The ERMessageHandler has responsabilities to send event reports to
     * the specified recipient and to receive event reports using the specific mechanism.
     *
     * @return An implementation of the ERMessageHandler.
     */
    public abstract ERMessageHandler getERMessageHandler();

    /**
     * Retreives the structural parameters that the engine uses.
     *
     * @return EREngineParams object that provided the structural parameters that the engine uses
     */
    public abstract EREngineParams getEREngineParams();

    /**
     * The <tt>ERRKeeper</tt> is responsible for storing/loading Event Report Requests to local repository
     * identify by <tt>repositoryKey</tt>.
     * <br/>
     * The ERR repository location is defined under the engine parameter: <tt>rootERRStorageLocation</tt>
     *
     * @param repositoryKey The ERR repository key to identify a particular ERR repository.
     * @return <tt>ERRKeeper</tt> object responsible to store/load Event Report Requests
     * @see org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EREngineParams
     */
    public abstract ERRKeeper getEventReportRequestKeeper(String repositoryKey);

    /**
     * Method to return all ERR keerpers responsible for storing/loading Event Report Requests
     * to local repositories they represent
     * <br/>
     * The ERR repository location is defined under the engine parameter: <tt>rootERRStorageLocation</tt>
     *
     * @returna list of ERRKeeper objects register until the invocation
     * @see org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EREngineParams
     */
    public abstract List<ERRKeeper> getEventReportRequestKeepers();

    /**
     * * The <tt>ERKeeper</tt> is responsible for storing/loading Event Reports to local repository
     * identify by <tt>repositoryKey</tt>.
     * <br/>
     * The ER repository location is defined under the engine parameter: <tt>rootERStorageLocation</tt>
     *
     * @param repositoryKey The ER repository key to identify a particular ER repository.
     * @return <tt>ERKeeper</tt> object responsible to store/load Event Reports
     * @see org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EREngineParams
     */
    public abstract ERKeeper getEventReportKeeper(String repositoryKey);

    /**
     * Method to return all ER keerpers responsible for storing/loading Event Reports
     * to local repositories they represent
     * <br/>
     * The ER repository location is defined under the engine parameter: <tt>rootERStorageLocation</tt>
     *
     * @return a list of ERKeeper objects register until the invocation
     * @see org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EREngineParams
     */
    public abstract List<ERKeeper> getEventReportKeepers();

}
