/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 * Contributor(s): Bogdan Ardeleanu
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.tEngine.eventreportTE.exception.EventReportException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;

import java.util.List;

/**
 * The <tt>ERKeeper</tt> class is responsible for storing/loading Event Reports to local repository
 * identify by <tt>repositoryKey</tt>.
 * <br/>
 * The ER repository location is defined under the engine parameter: <tt>rootERStorageLocation</tt>
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro">Bogdan Ardeleanu</a>
 * Date: 8/29/11
 * Time: 4:45 PM
 * <p/>
 * UTI Systems, TIC
 */
public interface ERKeeper {

    /**
     * Method that stores an event report (ER) to current local repository.
     *
     * @param er The event report request to store
     * @return The operation status using a <tt>MXMEngineResponse</tt> object.
     * @throws EventReportException In case of internal exception occur
     */
    public MXMEngineResponse storeER(ER er) throws EventReportException;

    /**
     * Method that loads an ER object from current ERR repository using the ER identifier.
     *
     * @param erId The ERR identifier
     * @return Null if the ER object not found
     * @throws EventReportException In case of internal exception occur
     */
    public ER getER(String erId) throws EventReportException;

    /**
     * Method that load the a list of ER (Event Report) objects using the related Event Report Request identifier.
     *
     * @param errId The related ERR identifier
     * @return A list of <tt>ER</tt> objects.
     * @throws EventReportException In case of internal exception occur
     */
    public List<String> getERIdsByERRId(String errId) throws EventReportException;

    /**
     * Method that load the a list of ER (Event Report) objects using the digital item identifier of the related Event Report Request.
     * It uses a err keerper call to find out the err identifier using the digital item identifier.
     * After that that <tt>getERIdsByERRId</tt> method will be call using the ERR identifier already found.
     *
     * @param DIDLid The digital item identifier of the related Event Report Request
     * @return A list of <tt>ER</tt> objects
     * @throws EventReportException In case of internal exception occur
     */
    public List<String> getERIdsByDIDLId(String DIDLid) throws EventReportException;


}
