/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 * Contributor(s): Bogdan Ardeleanu
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.exception.EventReportException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;

/**
 * The event report message handler interface whose responsabilities are to send event reports to
 * the specified recipient and to receive event reports using the specific mechanism.
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 8/29/11
 * Time: 4:37 PM
 * <p/>
 * UTI Systems, TIC
 */
public interface ERMessageHandler {
    /**
     * Method using to send event reports to the specified recipient using the specific mechanism.
     *
     * @param er  The <tt>EventReport</tt> object to be sent.
     * @param SAP Identifies the name of the Service Access Point where the ER will be send taken
     *            from the event report engine parameters.
     * @return The operation status using a <tt>MXMEngineResponse</tt> object.
     * @throws EventReportException
     */
    public MXMEngineResponse sendER(ER er, String SAP) throws EventReportException;

    /**
     * Method for start ER deamon that receives event reports.
     *
     * @return The operation status using a <tt>MXMEngineResponse</tt> object.
     * @throws EventReportException
     * @throws MXMEngineException
     */
    public MXMEngineResponse recvERDaemon() throws EventReportException, MXMEngineException;
}
