/**
 * The copyright in this software is being made available under the BSD License,
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 * 
 * Copyright (c) 2012, CEDEO
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this 
 *     list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *     - Neither the name of the CEDEO nor the names of its contributors may be used 
 *     to endorse or promote products derived from this software without specific 
 *     prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
*/

package org.iso.mpeg.mxm.tEngine.ipmpTE;

import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;

/**
 * Classes extending this abstract class are entry point classes for operating on ISO/IEC 21000-4 
 * Intellectual Property management and Protection (IPMP) data structures.  
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 * @author Victor Rodriguez Doncel <victorr@ac.upc.edu>
 * @author Eva Rodriguez <evar@ac.upc.edu>
 * @author Angelo Difino <angelo@cedeo.net>
 */
public interface IPMPEngine extends MXMEngine
{
	/**
	 * This method returns an instance of class <code>IPMPEngineSchemaHandler</code> which is 
	 * needed to parse/create a IPMP xml fragment 
	 * @return an object of class <code>IPMPEngineSchemaHandler</code> or null if this method 
	 * has not been implemented by the specific EventReportEngine.
	 * @throws MXMEngineException 
	 */	
	public IPMPEngineSchemaHandler getIPMPEngineSchemaHandler() throws MXMEngineException;
	
	/**
	 * This method returns an instance of class <code>IPMPEngineTechHandler</code> which is 
	 * needed to act technological methods on IPMP object
	 * @return an object of class <code>IPMPEngineTechHandler</code> or null if this method 
	 * has not been implemented by the specific EventReportEngine.
	 */	
	//public abstract IPMPEngineTechHandler getIPMPEngineTechHandler();

}
