/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s):  Angelo Difino
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.tEngine.mediaframeworkTE.access;

import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.tEngine.mediaframeworkTE.exception.MediaFrameworkEngineException;

/**
 * @author Angelo Difino <angelo@cedeo.net>
 *
 */
public interface AccessMedia {

	/**
		* This method is used to prepare a media to be played
		*
		* @param media \ref MXMObject containing the resource
		* @throws MediaFrameworkEngineException 
	*/
	public void open( MXMObject media ) throws MediaFrameworkEngineException;

	/**
		This method is used to play/render a media 

	 * @throws MediaFrameworkEngineException 
	*/
	public void play( ) throws MediaFrameworkEngineException;

	/**
		This method is used to stop a media 

	 * @throws MediaFrameworkEngineException 
	*/
	public void stop( ) throws MediaFrameworkEngineException;
	
	/**
	This method is used to pause a media

 	* @throws MediaFrameworkEngineException 
	*/
	public void pause( ) throws MediaFrameworkEngineException;	

	/**
	This method is used to release any resource 

 	* @throws MediaFrameworkEngineException 
	*/
	public void release() throws MediaFrameworkEngineException;

	/**
	This method is used to set the volume of the media

 	* @throws MediaFrameworkEngineException 
	*/
	public void setVolume(int volume) throws MediaFrameworkEngineException;
}
