/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): 
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.tEngine.mp21ffTE;

import org.iso.mpeg.mxm.tEngine.mp21ffTE.exceptions.MPEG21FileCreationException;

/**
 * Classes implementing this interface are used to create ISO/IEC 21000-9 files
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 *
 */
public interface MPEG21FileCreator {


	/**
	 * Adds the filename of a resource which shall be inserted in a MediaDataBox when the DCF is created
	 * @param mediaResourceFileName the resource filename
	 * @param mediaResourceItemID the didl:item ID of the item in the DCI corresponding to this resource. 
	 * @throws MPEG21FileCreationException
	 */
	public void addResource(String mediaResourceFileName, String mediaResourceItemID) throws MPEG21FileCreationException ;

	/**
	 * Sets the DCI which shall be inserted in the XML box of the DCF when this is created.
	 * @param diPathName the path to the file containing the DCI as XML
	 * @throws MPEG21FileCreationException
	 */
	public void setDIPathName(String diPathName) throws MPEG21FileCreationException ;
	
	/**
	 * Sets the binary DCI which shall be inserted in the BXML box of the DCF when this is created.
	 * @param diBytes the binary DCI
	 * @throws MPEG21FileCreationException
	 */
	public void setDIBytes(byte[] diBytes) throws MPEG21FileCreationException ;
	
	/**
	 * This method is used to create a DCF after the DCI and a number of resources have been set.
	 * @param mpeg21PathName the path to the file where the DCF will be stored.
	 * @throws MPEG21FileCreationException
	 */
	public void createMPEG21File(String mpeg21PathName) throws MPEG21FileCreationException ;

}
