/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): 
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.tEngine.mp21ffTE;

import org.iso.mpeg.mxm.core.MXMEngine;


/**
 * An interface defining the methods for operating over ISO/IEC 21000-9 MPEG-21 File Format files. 
 * From classes implementing the MPEG21FileEngine interface it is possible to obtain instances of 
 * classes for:
 * <ul>
 * 	<li>create ISO/IEC 21000-9 files</li>
 * 	<li>access data contained in ISO/IEC 21000-9 files</li>
 * </ul>
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 */
public interface MPEG21FileEngine extends MXMEngine {

	/**
	 * The MPEG21FileParser can be employed to extract data (the Digital Item and the referenced resources)
	 * from files conforming to ISO/IEC 21000-9
	 * @return
	 */
	public MPEG21FileParser getMPEG21FileParser() ;

	/**
	 * The MPEG21FileParser can be employed to create files conforming to ISO/IEC 21000-9
	 * @return
	 */
	public MPEG21FileCreator getMPEG21FileCreator() ;
	
}
