/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): 
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.tEngine.mp21ffTE;

import org.iso.mpeg.mxm.tEngine.mp21ffTE.exceptions.MPEG21FileParsingException;

/**
 * Classes implementing this interface are employed to parse ISO/IEC 21000-9 files.
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 *
 */
public interface MPEG21FileParser {

	/**
	 * <HL>
	 * Returns the DI extracted from the XML box of the DCF.
	 * @param mpeg21FilePathName the path name of the DCF to be parsed
	 * @param diPathName the pathname of the file where the DCI shall be saved as XML
	 * @throws MPEG21FileParsingException 
	 */
	public void getDCI(String mpeg21FilePathName, String diPathName) throws MPEG21FileParsingException ;

	/**
	 * <HL>
	 * Returns the binary DCI extracted from the BXML box of the DCF.
	 * @param mpeg21FilePathName the path name of the DCF to be parsed
	 * @return The binary DCI as a byte array
	 * @throws MPEG21FileParsingException 
	 */
	public byte[] getBinaryDI(String mpeg21FilePathName) throws MPEG21FileParsingException;

	/**
	 * <HL>
	 * Returns the media data related to an item having a specific item ID
	 * @param itemID the Item ID specified in the DCI whose corresponding Media Resource is sought
	 * @param  mediaDataFileName the path to the file where the media data extracted from the DCF has to be stored
	 * @throws MPEG21FileParsingException 
	 */
	public void getMediaData(String itemID, String mediaDataFileName) throws MPEG21FileParsingException ;

}
