package org.iso.mpeg.mxm.tEngine.relTE;

import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.ConditionBuilder;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm1x.*;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm2x.*;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm3x.*;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relmx.*;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.*;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.*;

public interface RELEngineSchemaHandler 
{
	public ConditionBuilder getConditionBuilder();
	
	// RELM1X
	public Delist getDelist() throws MXMEngineException;
	public DerivationConstraint getDerivationConstraint() throws MXMEngineException;
	public DerivationConstraintIsPartOf getDerivationConstraintIsPartOf() throws MXMEngineException;
	public DerivationConstraintResourceExclusionList getDerivationConstraintResourceExclusionList() throws MXMEngineException;
	public DerivationConstraintResourceInclusionList getDerivationConstraintResourceInclusionList() throws MXMEngineException;
	public DerivationConstraintResourceReplacementList getDerivationConstraintResourceReplacementList() throws MXMEngineException;
	public DrmSystem getDrmSystem() throws MXMEngineException;
	public Enlist getEnlist() throws MXMEngineException;
	public GovernedCopy getGovernedCopy() throws MXMEngineException;
	public GovernedMove getGovernedMove() throws MXMEngineException;
	public IdentityHolder getIdentityHolder() throws MXMEngineException;
	public OutputRegulation getOutputRegulation() throws MXMEngineException;
	public OutputRegulationRegulation getOutputRegulationRegulation() throws MXMEngineException;
	public ProtectedResource getProtectedResource() throws MXMEngineException;
	public SeekPermission getSeekPermission() throws MXMEngineException;
	public SeekPermissionCacheable getSeekPermissionCacheable() throws MXMEngineException;
	public ServiceLocation getServiceLocation() throws MXMEngineException;
	public StartCondition getStartCondition() throws MXMEngineException;

	// RELM2X
	public DestinationCondition getDestinationCondition() throws MXMEngineException;
	public DestinationPrincipal getDestinationPrincipal() throws MXMEngineException;
	public Export getExport() throws MXMEngineException;
	public ExtendRights getExtendRights() throws MXMEngineException;
	public NoSkipConstraint getNoSkipConstraint() throws MXMEngineException;
	public Proximity getProximity() throws MXMEngineException;
	public RelTimeDuration getRelTimeDuration() throws MXMEngineException;
	public Scrambling getScrambling() throws MXMEngineException;
	public SimultaneousAccess getSimultaneousAccess() throws MXMEngineException;
	public SimultaneousAccessIsPartOf getSimultaneousAccessIsPartOf() throws MXMEngineException;
	public TimedExerciseLimit getTimedExerciseLimit() throws MXMEngineException;
	public TimeShiftDuration getTimeShiftDuration() throws MXMEngineException;

	// RELM3X
	public Aggregate getAggregate() throws MXMEngineException;
	public Embed getEmbed() throws MXMEngineException;
	public GovernedAdapt getGovernedAdapt() throws MXMEngineException;
	public GovernedAggregate getGovernedAggregate() throws MXMEngineException;
	public GovernedEmbed getGovernedEmbed() throws MXMEngineException;
	public SourceCode getSourceCode() throws MXMEngineException;
	
	// RELMX
	public Adapt getAdapt() throws MXMEngineException;
	public Complement getComplement() throws MXMEngineException;
	public Derive getDerive() throws MXMEngineException;
	public DiCriteria getDiCriteria() throws MXMEngineException;
	public Diminish getDiminish() throws MXMEngineException;
	public DiPartOf getDiPartOf() throws MXMEngineException;
	public DiReference getDiReference() throws MXMEngineException;
	public Enhance getEnhance() throws MXMEngineException;
	public Execute getExecute() throws MXMEngineException;
	public Intersection getIntersection() throws MXMEngineException;
	public Modify getModify() throws MXMEngineException;
	public Play getPlay() throws MXMEngineException;
	public Print getPrint() throws MXMEngineException;
	public ProhibitedAttributeChanges getProhibitedAttributeChanges() throws MXMEngineException;
	public Set getSet() throws MXMEngineException;
	public Union getUnion() throws MXMEngineException;

	// RELR
	public AllPrincipals getAllPrincipals() throws MXMEngineException;
	public AnXmlExpression getAnXmlExpression() throws MXMEngineException;
	public AnXmlPatternAbstract getAnXmlPatternAbstract() throws MXMEngineException;
	public Condition getCondition() throws MXMEngineException;
	public ConditionIncremental getConditionIncremental() throws MXMEngineException;
	public ConditionPattern getConditionPattern() throws MXMEngineException;
	public ConditionPatternAbstract getConditionPatternAbstract() throws MXMEngineException;
	public ConditionUnchanged getConditionUnchanged() throws MXMEngineException;
	public Datum getDatum() throws MXMEngineException;
	public DcConstraint getDcConstraint() throws MXMEngineException;
	public DelegationControl getDelegationControl() throws MXMEngineException;
	public DepthConstraint getDepthConstraint() throws MXMEngineException;
	public DigitalResource getDigitalResource() throws MXMEngineException;
	public DigitalResourceAnXml getDigitalResourceAnXml() throws MXMEngineException;
	public EncryptedContent getEncryptedContent() throws MXMEngineException;
	public ExerciseMechanism getExerciseMechanism() throws MXMEngineException;
	public ExerciseMechanismExerciseService getExerciseMechanismExerciseService() throws MXMEngineException;
	public ExistsRight getExistsRight() throws MXMEngineException;
	public ForAll getForAll() throws MXMEngineException;
	public Fulfiller getFulfiller() throws MXMEngineException;
	public Grant getGrant() throws MXMEngineException;
	public GrantGroup getGrantGroup() throws MXMEngineException;
	public GrantGroupPattern getGrantGroupPattern() throws MXMEngineException;
	public GrantPattern getGrantPattern() throws MXMEngineException;
	public Inventory getInventory() throws MXMEngineException;
	public Issue getIssue() throws MXMEngineException;
	public Issuer getIssuer() throws MXMEngineException;
	public IssuerDetails getIssuerDetails() throws MXMEngineException;
	public KeyHolder getKeyHolder() throws MXMEngineException;
	public License getLicense() throws MXMEngineException;
	public LicenseGroup getLicenseGroup() throws MXMEngineException;
	public LicenseOtherInfo getLicenseOtherInfo() throws MXMEngineException;
	public LicensePart getLicensePart() throws MXMEngineException;
	public LinguisticString getLinguisticString() throws MXMEngineException;
	public NonSecureReference getNonSecureReference() throws MXMEngineException;
	public Obtain getObtain() throws MXMEngineException;
	public PatternFromLicensePart getPatternFromLicensePart() throws MXMEngineException;
	public PossessProperty getPossessProperty() throws MXMEngineException;
	public PrerequisiteRight getPrerequisiteRight() throws MXMEngineException;
	public Principal getPrincipal() throws MXMEngineException;
	public PrincipalPattern getPrincipalPattern() throws MXMEngineException;
	public PrincipalPatternAbstract getPrincipalPatternAbstract() throws MXMEngineException;
	public PropertyAbstract getPropertyAbstract() throws MXMEngineException;
	public PropertyPossessor getPropertyPossessor() throws MXMEngineException;
	public Resource getResource() throws MXMEngineException;
	public ResourcePattern getResourcePattern() throws MXMEngineException;
	public ResourcePatternAbstract getResourcePatternAbstract() throws MXMEngineException;
	public Revocable getRevocable() throws MXMEngineException;
	public RevocationFreshness getRevocationFreshness() throws MXMEngineException;
	public RevocationMechanism getRevocationMechanism() throws MXMEngineException;
	public RevocationMechanismRevocationService getRevocationMechanismRevocationService() throws MXMEngineException;
	public Revoke getRevoke() throws MXMEngineException;
	public org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Right getRight() throws MXMEngineException;
	public RightPattern getRightPattern() throws MXMEngineException;
	public RightPatternAbstract getRightPatternAbstract() throws MXMEngineException;
	public ServiceDescription getServiceDescription() throws MXMEngineException;
	public ServiceReference getServiceReference() throws MXMEngineException;
	public ServiceReferenceServiceParameters getServiceReferenceServiceParameters() throws MXMEngineException;
	public ToConstraint getToConstraint() throws MXMEngineException;
	public TrustedRootGrants getTrustedRootGrants() throws MXMEngineException;
	public TrustedRootIssuers getTrustedRootIssuers() throws MXMEngineException;
	public TrustRoot getTrustRoot() throws MXMEngineException;
	public ValidityInterval getValidityInterval() throws MXMEngineException;

	// RELSX
	public AccountPayable getAccountPayable() throws MXMEngineException;
	public AccountPayableAba getAccountPayableAba() throws MXMEngineException;
	public AccountPayablePaymentService getAccountPayablePaymentService() throws MXMEngineException;
	public AnonymousStateService getAnonymousStateService() throws MXMEngineException;
	public CallForCondition getCallForCondition() throws MXMEngineException;
	public CommonName getCommonName() throws MXMEngineException;
	public DnsName getDnsName() throws MXMEngineException;
	public EmailName getEmailName() throws MXMEngineException;
	public ExerciseLimit getExerciseLimit() throws MXMEngineException;
	public LicenseIdPattern getLicenseIdPattern() throws MXMEngineException;
	public Name getName() throws MXMEngineException;
	public PropertyUri getPropertyUri() throws MXMEngineException;
	public Rate getRate() throws MXMEngineException;
	public RightUri getRightUri() throws MXMEngineException;
	public SeekApproval getSeekApproval() throws MXMEngineException;
	public StateDistinguisher getStateDistinguisher() throws MXMEngineException;
	public StatefulCondition getStatefulCondition() throws MXMEngineException;
	public StateReferenceValuePattern getStateReferenceValuePattern() throws MXMEngineException;
	public TrackQuery getTrackQuery() throws MXMEngineException;
	public TrackReport getTrackReport() throws MXMEngineException;
	public TransferControl getTransferControl() throws MXMEngineException;
	public Uddi getUddi() throws MXMEngineException;
	public UddiKey getUddiKey() throws MXMEngineException;
	public ValidityIntervalDurationPattern getValidityIntervalDurationPattern() throws MXMEngineException;
	public ValidityIntervalFloating getValidityIntervalFloating() throws MXMEngineException;
	public ValidityIntervalStartsNow getValidityIntervalStartsNow() throws MXMEngineException;
	public ValidityTimePeriodic getValidityTimePeriodic() throws MXMEngineException;
	public WsdlAddress getWsdlAddress() throws MXMEngineException;
	public WsdlAddressAddress getWsdlAddressAddress() throws MXMEngineException;
	public WsdlAddressKind getWsdlAddressKind() throws MXMEngineException;
	public WsdlComplete getWsdlComplete() throws MXMEngineException;
	public X509SubjectName getX509SubjectName() throws MXMEngineException;
	public X509SubjectNamePattern getX509SubjectNamePattern() throws MXMEngineException;
}
