/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.tEngine.relTE.technologyhandler;

/**
 * This enum list all possible Evaluation conditions
 * that may occur when evaluating whether a Condition is met.
 * The order in which they are listed is important: they are in 
 * order from "the most far away from being met" to the 
 * closest to be fully met. Payment requires the End-User to 
 * perform a transaction in order to be met. ORC requires
 * the explicit agreement to Creative Commons License terms
 * in order to be met. 
 * 
 * @author filippo
 *
 */
public enum EvaluatedConditionResult {
	
	// definitely not met
	NOT_MET,
	
	// failure: the specified condition is not currently supported
	UNSUPPORTED,
	
	// failure: the specified condition could not be verified
	UNDETERMINED,

	// a successful result may be obtained if a payment is made
	PAYMENT,
	
	// a successful result may be obtained if the End-User
	// agrees to the CreativeCommons conditions
	COPYRIGHT_NOTICE,
	
	// definitely met
	SUCCESSFULLY_MET;

}
