/**
 * The copyright in this software is being made available under the BSD License,
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 * 
 * Copyright (c) 2012, CEDEO
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this 
 *     list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *     - Neither the name of the CEDEO nor the names of its contributors may be used 
 *     to endorse or promote products derived from this software without specific 
 *     prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS �AS IS� AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 */
package org.iso.mpeg.mxm.tEngine.securityTE;


import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.CanonicalizationMethod;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.DSAKeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.DigestMethod;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Manifest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.PGPData;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RetrievalMethod;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SPKIData;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignatureMethod;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignatureProperties;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignatureProperty;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignatureValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignedInfo;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Transform;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509IssuerSerial;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Action;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Advice;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Assertion;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Attribute;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AttributeStatement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AudienceRestriction;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnContext;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnStatement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthzDecisionStatement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.BaseIDAbstract;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.ConditionAbstract;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Conditions;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Decision;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.EncryptedElement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Evidence;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.KeyInfoConfirmationData;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.NameID;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.OneTimeUse;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.ProxyRestriction;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.StatementAbstract;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Subject;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.SubjectConfirmation;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.SubjectConfirmationData;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.SubjectLocality;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.ArtifactResolve;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.ArtifactResponse;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AssertionIDRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AttributeQuery;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AuthnContextComparison;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AuthnQuery;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AuthnRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AuthzDecisionQuery;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.Extensions;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.IDPEntry;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.IDPList;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.LogoutRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.ManageNameIDRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.NameIDMappingRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.NameIDMappingResponse;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.NameIDPolicy;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.RequestAbstract;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.RequestedAuthnContext;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.Response;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.Scoping;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.Status;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.StatusCode;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.StatusDetail;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.StatusResponse;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.SubjectQueryAbstract;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.Terminate;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.AgreementMethod;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.CipherData;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.CipherReference;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.Encrypted;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedData;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptionMethod;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptionProperties;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptionProperty;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.ReferenceList;


public interface SecurityEngineSchemaHandler
{
    // SAML
    public Action getAction() throws MXMEngineException;

    public Advice getAdvice() throws MXMEngineException;

    // XENC
    public AgreementMethod getAgreementMethod() throws MXMEngineException;

    public ArtifactResolve getArtifactResolve() throws MXMEngineException;

    public ArtifactResponse getArtifactResponse() throws MXMEngineException;

    public Assertion getAssertion() throws MXMEngineException;

    public AssertionIDRequest getAssertionIDRequest() throws MXMEngineException;

    public Attribute getAttribute() throws MXMEngineException;

    public AttributeQuery getAttributeQuery() throws MXMEngineException;

    public AttributeStatement getAttributeStatement() throws MXMEngineException;

    public AudienceRestriction getAudienceRestriction() throws MXMEngineException;

    public AuthnContext getAuthnContext() throws MXMEngineException;

    public AuthnContextComparison getAuthnContextComparison() throws MXMEngineException;

    public AuthnQuery getAuthnQuery() throws MXMEngineException;

    public AuthnRequest getAuthnRequest() throws MXMEngineException;

    public AuthnStatement getAuthnStatement() throws MXMEngineException;

    public AuthzDecisionQuery getAuthzDecisionQuery() throws MXMEngineException;

    public AuthzDecisionStatement getAuthzDecisionStatement() throws MXMEngineException;

    public BaseIDAbstract getBaseIDAbstract() throws MXMEngineException;

    // DSIG
    public CanonicalizationMethod getCanonicalizationMethod() throws MXMEngineException;

    public CipherData getCipherData() throws MXMEngineException;

    public CipherReference getCipherReference() throws MXMEngineException;

    public ConditionAbstract getConditionAbstract() throws MXMEngineException;

    public Conditions getConditions() throws MXMEngineException;

    public Decision getDecision() throws MXMEngineException;

    public DigestMethod getDigestMethod() throws MXMEngineException;

    public DSAKeyValue getDSAKeyValue() throws MXMEngineException;

    public Encrypted getEncrypted() throws MXMEngineException;

    public EncryptedData getEncryptedData() throws MXMEngineException;

    public EncryptedElement getEncryptedElement() throws MXMEngineException;

    public EncryptedKey getEncryptedKey() throws MXMEngineException;

    public EncryptionMethod getEncryptionMethod() throws MXMEngineException;

    public EncryptionProperties getEncryptionProperties() throws MXMEngineException;

    public EncryptionProperty getEncryptionProperty() throws MXMEngineException;

    public Evidence getEvidence() throws MXMEngineException;

    public Extensions getExtensions() throws MXMEngineException;

    public IDPEntry getIDPEntry() throws MXMEngineException;

    public IDPList getIDPList() throws MXMEngineException;

    public KeyInfo getKeyInfo() throws MXMEngineException;

    public KeyInfoConfirmationData getKeyInfoConfirmationData() throws MXMEngineException;

    public KeyValue getKeyValue() throws MXMEngineException;

    public LogoutRequest getLogoutRequest() throws MXMEngineException;

    public ManageNameIDRequest getManageNameIDRequest() throws MXMEngineException;

    public Manifest getManifest() throws MXMEngineException;

    public NameID getNameID() throws MXMEngineException;

    public NameIDMappingRequest getNameIDMappingRequest() throws MXMEngineException;

    public NameIDMappingResponse getNameIDMappingResponse() throws MXMEngineException;

    public NameIDPolicy getNameIDPolicy() throws MXMEngineException;

    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Object getObject()
        throws MXMEngineException;

    public OneTimeUse getOneTimeUse() throws MXMEngineException;

    public PGPData getPGPData() throws MXMEngineException;

    public ProxyRestriction getProxyRestriction() throws MXMEngineException;

    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Reference getReferenceDSig()
        throws MXMEngineException;

    public ReferenceList getReferenceList() throws MXMEngineException;

    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.Reference getReferenceXEnc()
        throws MXMEngineException;

    public RequestAbstract getRequestAbstract() throws MXMEngineException;

    public RequestedAuthnContext getRequestedAuthnContext() throws MXMEngineException;

    public Response getResponse() throws MXMEngineException;

    public RetrievalMethod getRetrievalMethod() throws MXMEngineException;

    public RSAKeyValue getRSAKeyValue() throws MXMEngineException;

    public Scoping getScoping() throws MXMEngineException;

    public Signature getSignature() throws MXMEngineException;

    public SignatureMethod getSignatureMethod() throws MXMEngineException;

    public SignatureProperties getSignatureProperties() throws MXMEngineException;

    public SignatureProperty getSignatureProperty() throws MXMEngineException;

    public SignatureValue getSignatureValue() throws MXMEngineException;

    public SignedInfo getSignedInfo() throws MXMEngineException;

    public SPKIData getSPKIData() throws MXMEngineException;

    public StatementAbstract getStatementAbstract() throws MXMEngineException;

    public Status getStatus() throws MXMEngineException;

    public StatusCode getStatusCode() throws MXMEngineException;

    public StatusDetail getStatusDetail() throws MXMEngineException;

    public StatusResponse getStatusResponse() throws MXMEngineException;

    public Subject getSubject() throws MXMEngineException;

    public SubjectConfirmation getSubjectConfirmation() throws MXMEngineException;

    public SubjectConfirmationData getSubjectConfirmationData() throws MXMEngineException;

    public SubjectLocality getSubjectLocality() throws MXMEngineException;

    public SubjectQueryAbstract getSubjectQueryAbstract() throws MXMEngineException;

    public Terminate getTerminate() throws MXMEngineException;

    public Transform getTransform() throws MXMEngineException;

    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Transforms getTransformsDSig()
        throws MXMEngineException;

    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.Transforms getTransformsXEnc()
        throws MXMEngineException;

    public X509Data getX509Data() throws MXMEngineException;

    public X509IssuerSerial getX509IssuerSerial() throws MXMEngineException;
}
