/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2007
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s): Edoardo Radica
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.tEngine.utils;

import java.util.HashMap;
import java.util.Map;

import com.sun.xml.bind.marshaller.NamespacePrefixMapper;

public class SchemaNamespacePrefixMapperImpl extends NamespacePrefixMapper {

	private Map<String, String> prefixs = new HashMap<String, String>();

	private static SchemaNamespacePrefixMapperImpl snpm = null;
	
	public static SchemaNamespacePrefixMapperImpl getInstance(){
		if(snpm == null)
			snpm = new SchemaNamespacePrefixMapperImpl();
		return snpm; 
	}
	
	private SchemaNamespacePrefixMapperImpl(){
		setDefaultUri();
	}
	
	private void setDefaultUri(){
						
		prefixs.put("http://www.w3.org/2001/XMLSchema-instance", "xsi");
		
		prefixs.put("urn:mpeg:mpeg21:2007:01-BBL-NS", "bbl");
		prefixs.put("urn:mpeg:mpeg21:2003:01-DIA-NS", "dia");
		
		prefixs.put("urn:mpeg:mpegM:schema:06-didl-NS:2012", "mpegm-didl");
		prefixs.put("urn:mpeg:mpeg21:2002:02-DIDL-NS", "didl_mp21");
		prefixs.put("urn:mpeg:maf:schema:mediastreaming:DIDLextensions", "didl_msx");
		prefixs.put("urn:mpeg:mpeg21:2002:02-DIDMODEL-NS", "didmodel");      
		prefixs.put("urn:mpeg:mpeg21:2002:01-DII-NS", "dii");
		
		prefixs.put("urn:mpeg:mpeg21:2005:01-ERL-NS", "er");
		
		prefixs.put("urn:mpeg:mpeg21:2004:01-IPMPDIDL-NS", "ipmpdidl");
		prefixs.put("urn:mpeg:mpeg21:2004:01-IPMPINFO-NS", "ipmpinfo");
		prefixs.put("urn:mpeg:maf:Schema:mediastreaming:IPMPINFOextensions:2007", "ipmpinfo_msx");		
		prefixs.put("urn:mpeg:mpegB:schema:IPMP-XML-MESSAGES:2007", "ipmpmsg");      
		prefixs.put("urn:mpeg:mpeg4:IPMPSchema:2002", "mpeg4ipmp");
		prefixs.put("urn:mpeg:mpegM:schema:08-IPMPDIDL-NS:2012", "mpegmipmpdidl");
		prefixs.put("urn:mpeg:mpegM:schema:07-IPMPINFO-NS:2012", "mpegmipmpinfo");
		
		
		prefixs.put("urn:mpeg:mpeg7:schema:2001", "mpeg7");
		prefixs.put("urn:mpeg:mpeg7:smp:schema:2001", "mpeg7smp");
		prefixs.put("urn:mpeg:mpeg7:schema:2004", "mpeg7_v3");
		prefixs.put("urn:mpeg:mpegB:schema:FragmentRequestUnits:2007", "fru");

		prefixs.put("urn:mpeg:mpeg21:2005:01-REL-M1X-NS", "rel-m1x");
		prefixs.put("urn:mpeg:mpeg21:2006:01-REL-M2X-NS", "rel-m2x");      
		prefixs.put("urn:mpeg:mpeg21:2006:01-REL-M3X-NS", "rel-m3x");
		prefixs.put("urn:mpeg:mpeg21:2003:01-REL-MX-NS", "rel-mx");
		prefixs.put("urn:mpeg:mpeg21:2003:01-REL-R-NS", "rel-r");
		prefixs.put("urn:mpeg:mpeg21:2003:01-REL-SX-NS", "rel-sx");

		prefixs.put("http://www.w3.org/2000/09/xmldsig#", "dsig");
		prefixs.put("http://www.w3.org/2001/04/xmlenc#", "xenc");
		
		prefixs.put("org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion", "samla");
		prefixs.put("org.iso.mpeg.mxm.engine.securityTE.schema.saml.protocol", "samlp");	
	}

	public String getPreferredPrefix(String namespaceUri, String suggestion, boolean requirePrefix) {
		
		if(prefixs.containsKey(namespaceUri)){
			return prefixs.get(namespaceUri);
		}
		// otherwise I don't care. Just use the default suggestion, whatever it may be.
		return suggestion;
	}

	public void addNamespaceUri(String namespaceUri, String prefix){
		if( !prefixs.containsKey(namespaceUri) )
			prefixs.put(namespaceUri, prefix);
	}
}
