/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.baseprotocol;

import java.util.ArrayList;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;

public class BaseProtocolEngineContext {
	
	private Logger logger = Logger.getLogger(this.getClass());
	
	protected ArrayList<Class> packageNames = new ArrayList<Class>();
	
	private static BaseProtocolEngineContext packages = null;

	private static JAXBContext jc = null;
	
	protected BaseProtocolEngineContext() {
//		packageNames.add("org.iso.mpeg.mxm.baseprotocol.schema");
		packageNames.add(org.iso.mpeg.mxm.baseprotocol.schema.ObjectFactory.class);
		
		// DID packages
		packageNames.add(org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_mp21.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_msx.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.digitalitemTE.schema.didmodel_mp21.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.digitalitemTE.schema.dii.ObjectFactory.class);
		
		// ER packages
		packageNames.add(org.iso.mpeg.mxm.engine.eventreportTE.schema.er.ObjectFactory.class);
		
		// Metadata packages
		packageNames.add(org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.ObjectFactory.class);
		
		// IPMP packages
		//packageNames.add(org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpdidl.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpinfo.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpinfo_msx.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpmsg.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.ipmpTE.schema.mpeg4ipmp.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.ipmpTE.schema.mpegmipmpdidl.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.ipmpTE.schema.mpegmipmpinfo.ObjectFactory.class);
		
		//REL packages
		packageNames.add(org.iso.mpeg.mxm.engine.relTE.schema.relr.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.relTE.schema.relsx.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.relTE.schema.relmx.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.relTE.schema.relm1x.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.relTE.schema.relm2x.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.relTE.schema.relm3x.ObjectFactory.class);

		//Security packages
		packageNames.add(org.iso.mpeg.mxm.engine.securityTE.schema.dsig.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.securityTE.schema.xenc.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.ObjectFactory.class);
		packageNames.add(org.iso.mpeg.mxm.engine.securityTE.schema.saml.protocol.ObjectFactory.class);
	}
	
	public static synchronized BaseProtocolEngineContext getInstance(){
		if(packages==null)
			packages=new BaseProtocolEngineContext();
		return packages;
	}
	
	/**
	 * @return all package names in sequence
	 */
	private String getPackagesInOrder(){
		StringBuffer orderpackages=new StringBuffer();
		for(int i=0;i<packageNames.size();i++){
			orderpackages.append(packageNames.get(i));
			if(i>=(packageNames.size()-1))
				break;
			orderpackages.append(":");
		}	
		return orderpackages.toString();
	}
	
	/**
	 * @return the JAXB Context all schemas used by the DIDEngine
	 * @throws JAXBException
	 */
	public JAXBContext getJAXBContext() throws JAXBException{
		if (jc == null) {			
			logger.debug("Generating Base Protocol JAXB Context...");
			Class[] packages = new Class[packageNames.size()];
			jc = JAXBContext.newInstance(packageNames.toArray(packages));
			logger.debug("Done!");	
		}
		return jc;
	}
	
//	public void addPackage(String packageName) {
//		packageNames.add(packageName);
//	}
	
	public void addPackage(Class packageClass)
	{
		packageNames.add(packageClass);
	}
}
