/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.engine.relTE.schema;

import java.util.ArrayList;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;

public final class RELEngineContext {
	
	private Logger logger = Logger.getLogger(RELEngineContext.class);
	
	private ArrayList<String> packageNames = new ArrayList<String>();
	
	private static RELEngineContext packages = null;

	private static JAXBContext jc = null;
	
	private RELEngineContext() {
		
		// DI packages
				packageNames.add("org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl");
				packageNames.add("org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_mp21");
				packageNames.add("org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_msx");
				packageNames.add("org.iso.mpeg.mxm.engine.digitalitemTE.schema.dii");
				packageNames.add("org.iso.mpeg.mxm.engine.digitalitemTE.schema.didmodel_mp21");
				
				// ER packages
				packageNames.add("org.iso.mpeg.mxm.engine.eventreportTE.schema.er");
				
				// IPMP packages
				packageNames.add("org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpinfo");
				packageNames.add("org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpinfo_msx");		
				packageNames.add("org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpmsg");
				packageNames.add("org.iso.mpeg.mxm.engine.ipmpTE.schema.mpeg4ipmp");
				packageNames.add("org.iso.mpeg.mxm.engine.ipmpTE.schema.mpegmipmpdidl");
				packageNames.add("org.iso.mpeg.mxm.engine.ipmpTE.schema.mpegmipmpinfo");

				// MPEG7 packages
				packageNames.add("org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3");
				packageNames.add("org.iso.mpeg.mxm.engine.metadataTE.schema.fru");
				
				//REL packages
				packageNames.add("org.iso.mpeg.mxm.engine.relTE.schema.relr");
				packageNames.add("org.iso.mpeg.mxm.engine.relTE.schema.relsx");
				packageNames.add("org.iso.mpeg.mxm.engine.relTE.schema.relmx");
				packageNames.add("org.iso.mpeg.mxm.engine.relTE.schema.relm1x");
				packageNames.add("org.iso.mpeg.mxm.engine.relTE.schema.relm2x");
				packageNames.add("org.iso.mpeg.mxm.engine.relTE.schema.relm3x");
				
				// Security packages
				packageNames.add("org.iso.mpeg.mxm.engine.securityTE.schema.dsig");
				packageNames.add("org.iso.mpeg.mxm.engine.securityTE.schema.xenc");
				packageNames.add("org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion");
				packageNames.add("org.iso.mpeg.mxm.engine.securityTE.schema.saml.protocol");

	}
	
	public static synchronized RELEngineContext getInstance(){
		if(packages==null)
			packages=new RELEngineContext();
		return packages;
	}
	
	/**
	 * @return all package names in sequence
	 */
	private String getPackagesInOrder(){
		StringBuffer orderpackages=new StringBuffer();
		for(int i=0;i<packageNames.size();i++){
			orderpackages.append(packageNames.get(i));
			if(i>=(packageNames.size()-1))
				break;
			orderpackages.append(":");
		}	
		return orderpackages.toString();
	}
	
	/**
	 * @return the JAXB Context all schemas used by the DIDEngine
	 * @throws JAXBException
	 */
	public JAXBContext getJAXBContext() throws JAXBException{
		if (jc == null) {			
			logger.debug("Generating RELEngine JAXB Context...");
			jc = JAXBContext.newInstance(getPackagesInOrder());
			logger.debug("Done!");	
		}
		return jc;
	}
	
	public void addPackage(String packageName) {
		packageNames.add(packageName);
	}
}
