import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.security.PublicKey;

import junit.framework.TestCase;

import org.apache.xerces.impl.dv.util.Base64;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.iso.mpeg.mxm.basePE.ContentEntity;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.pEngine.authenticatecontentPE.AuthenticateContentEngine;
import org.iso.mpeg.mxm.pEngine.authenticatecontentPE.schemahandler.AuthenticateContentRequest;
//import org.iso.mpeg.mxm.pEngine.authenticatecontentPE.schemahandler.AuthenticateContentResponse;
import org.iso.mpeg.mxm.pEngine.authenticateentityPE.schemahandler.AuthenticateEntityResponse;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.ApplicationSpecificData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.dii.RelatedIdentifier;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.IPMPGeneralInfoDescriptor;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.LicenseCollection;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.RightsDescriptor;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Assertion;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager;
import org.w3c.dom.Document;


public class AuthenticateContentProtocolTest extends TestCase
{
    public static boolean             initialized = false;
    private final String              APP_ALIAS   = "1";
    private AuthenticateContentEngine authenticateContentEngine;
    private CertificateManager        certMngr    = null;
    private DIEngine                  diEngine;
    private MXM                       mxm;
    private SecurityEngine            securityEngine;

    public AuthenticateContentProtocolTest() throws Exception
    {
//        final File f = new File(this.getClass().getClassLoader()
//            .getResource("MXMConfiguration1.xml").getFile());
//        if (initialized)
//        {
//            return;
//        }
//        initialized = true;
//        this.mxm = MXM.getInstance(f);
//        this.mxm.listEnginesAndProperties();
    }

    public void authenticate() throws Exception
    {
        
        System.out.println("TESTING CONTENT AUTHENTICATION");
        final AuthenticateContentRequest request = this.authenticateContentEngine
            .getAuthenticateContentSchemaHandler().getAuthenticateContentRequest();
        final Assertion assertion = this.securityEngine.getSecurityEngineSchemaHandler()
            .getAssertion();
        org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity entity = this.authenticateContentEngine
            .getBaseProtocolSchemaHandler().getContentEntity();
        request.setAuthenticationContent(entity);
        DIDL didl = this.createDI("test");
        entity.setDIDL(didl);
        Document doc = (Document)didl.createObject(Document.class.getCanonicalName())
            .getContent();
        Signature sign = this.securityEngine.getSecurityEngineTechnologyHandler()
            .getSignatureManager().generateSignature(doc, APP_ALIAS);
        request.setContentEntitySignature(sign);
        AuthenticateEntityResponse response = this.authenticateContentEngine
            .getAuthenticateContentTechnologyHandler()
            .getLocalAuthenticateContentProtocol().authenticateContent(request);
        assertNotNull(response);
        assertNotNull(response.getAuthenticateEntitySuccess());
        DIDL didlNew = this.createDI("test1");
        entity.setDIDL(didlNew);
        AuthenticateEntityResponse responseNew = this.authenticateContentEngine
            .getAuthenticateContentTechnologyHandler()
            .getLocalAuthenticateContentProtocol().authenticateContent(request);
        assertNull(responseNew.getAuthenticateEntitySuccess());
        assertNotNull(responseNew.getAuthenticateEntityFailure());
    }

    /**
     * create a dummy didl with given id
     * 
     * @param id
     * @return
     * @throws Exception
     */
    private DIDL createDI(String id) throws Exception
    {
        DIEngine didEngine = (DIEngine)mxm.getDefaultEngine(MXMEngineName.DigitalItemTE
            .toString());
        EventReportEngine erEngine = (EventReportEngine)mxm
            .getDefaultEngine(MXMEngineName.EventReportTE.toString());
        IPMPEngine ipmp = (IPMPEngine)mxm.getDefaultEngine(MXMEngineName.IPMPTE
            .toString());
        RELEngine relEngine = (RELEngine)mxm.getDefaultEngine(MXMEngineName.RELTE
            .toString());
        DIDL di = didEngine.getDIEngineSchemaHandler().getDIDL();
        Item toplevelitem = didEngine.getDIEngineSchemaHandler().getItem();
        ApplicationSpecificData asd = didEngine.getDIEngineSchemaHandler()
            .getApplicationSpecificData();
        ERR err = erEngine.getEventReportEngineSchemaHandler().getERR();
        Item innerItem1 = didEngine.getDIEngineSchemaHandler().getItem();
        Item innerItem2 = didEngine.getDIEngineSchemaHandler().getItem();
        toplevelitem.setId(id);
        IPMPGeneralInfoDescriptor gen_desc = ipmp.getIPMPEngineSchemaHandler()
            .getIPMPGeneralInfoDescriptor();
        LicenseCollection lc = ipmp.getIPMPEngineSchemaHandler().getLicenseCollection();
        RightsDescriptor rd = ipmp.getIPMPEngineSchemaHandler().getRightsDescriptor();
        org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.License ipmp_l = ipmp
            .getIPMPEngineSchemaHandler().getIPMPLicense();
        License l = relEngine.getRELEngineSchemaHandler().getLicense();
        l.setLicenseId("1234");
        ipmp_l.addLicense(l);
        rd.setLicense(ipmp_l);
        lc.setRightsDescriptor(rd);
        gen_desc.setLicenseCollection(lc);
        Component c = didEngine.getDIEngineSchemaHandler().getComponent();
        Resource r = didEngine.getDIEngineSchemaHandler().getResource();
        r.setEncoding("encoding--resource");
        r.setMime("mime--resource");
        r.setRef("href--resource");
        r.getContentEncoding().add("content--encoding--resource");
        c.addResource(r);
        innerItem1.addComponent(c);
        RelatedIdentifier ri = didEngine.getDIEngineSchemaHandler()
            .getRelatedIdentifier();
        ri.setRelationship("relation");
        ri.setValue("value relat");
        di.setMPEGMItem(toplevelitem);
        return di;
    }


    public void testAutenticateContent() throws Exception
    {
//        System.out.println("\n--- testAutnenticateContent ---");
//        // IdentifyUserEngine identifyUserEngine = (IdentifyUserEngine)mxm
//        // .getDefaultEngine(MXMEngineName.IdentifyUserPE.toString());
//        this.securityEngine = (SecurityEngine)this.mxm
//            .getDefaultEngine(MXMEngineName.SecurityTE.toString());
//        this.securityEngine.setProperty("conf", "123456");
//        this.securityEngine.setProperty("auth", "323456");
//        this.securityEngine.setProperty("sig", "123456");
//        this.securityEngine.setProperty("gakp", "12345678");
//        this.securityEngine.setProperty("appAlias", APP_ALIAS);
//      
//        this.authenticateContentEngine = (AuthenticateContentEngine)this.mxm
//            .getDefaultEngine(MXMEngineName.AuthenticateContentPE.toString());
//        this.diEngine = (DIEngine)this.mxm.getDefaultEngine(MXMEngineName.DigitalItemTE
//            .toString());
//        this.certMngr = this.securityEngine.getSecurityEngineTechnologyHandler()
//            .getCertificateManager();
//        this.authenticate();
    }
}
