/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD Entity, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this entity.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.authenticateentityPE.schema;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolResponse;
import org.iso.mpeg.mxm.baseprotocol.schema.ProtocolFailureType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.authenticatecontentPE.schema.authenticatecontentprotocol.AuthenticateContentRequestType;
import org.iso.mpeg.mxm.engine.protocol.authenticateentityPE.schema.authenticateentityprotocol.AuthenticateEntityResponseType;
import org.iso.mpeg.mxm.engine.protocol.authenticateentityPE.schema.authenticateentityprotocol.AuthenticateEntitySuccessType;
import org.iso.mpeg.mxm.engine.protocol.searchentityPE.schema.searchentityprotocol.SearchEntityResponseType;
import org.iso.mpeg.mxm.engine.protocol.searchentityPE.schema.searchentityprotocol.SearchEntitySuccessType;
import org.iso.mpeg.mxm.pEngine.authenticateentityPE.schemahandler.AuthenticateEntitySuccess;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.pEngine.searchentityPE.schemahandler.SearchEntitySuccess;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelo Difino, Cedeo.net <angelo@cedeo.net>
 *
 */
public class AuthenticateEntityResponse extends ProtocolResponse implements org.iso.mpeg.mxm.pEngine.authenticateentityPE.schemahandler.AuthenticateEntityResponse 
{
	BaseProtocolEngine baseProtocolEngine;
	SecurityEngine securityEngine;
	
	public AuthenticateEntityResponse(BaseProtocolEngine baseProtocolEngine, SecurityEngine securityEngine)
	{
		super(new AuthenticateEntityResponseType(), securityEngine);
		this.baseProtocolEngine=baseProtocolEngine;
		this.securityEngine=securityEngine;
	}
	
	public AuthenticateEntityResponse(AuthenticateEntityResponseType authenticateContentResponseType, BaseProtocolEngine baseProtocolEngine, SecurityEngine securityEngine, RELEngine relEngine, MetadataEngine metadataEngine)
	{
		super(authenticateContentResponseType, securityEngine);
		this.baseProtocolEngine=baseProtocolEngine;
		this.securityEngine=securityEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#searchFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((AuthenticateEntityResponseType)protocolBaseType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#searchObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((AuthenticateEntityResponseType)protocolBaseType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((AuthenticateEntityResponseType)protocolBaseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolBaseType = 
					(AuthenticateEntityResponseType)(level1.createObject(AuthenticateEntityResponseType.class.getName()).getContent());
		}
		return response;
	}

	@Override
	public AuthenticateEntitySuccess getAuthenticateEntitySuccess() throws MXMEngineException {
		AuthenticateEntitySuccess authenticateEntitySuccess = new org.iso.mpeg.mxm.authenticateentityPE.schema.AuthenticateEntitySuccess(securityEngine);
		if(((AuthenticateEntityResponseType)protocolBaseType).getAuthenticateEntitySuccess() == null) {
		    return null;
		}
		authenticateEntitySuccess.parse(new MXMAdapter(((AuthenticateEntityResponseType)protocolBaseType).getAuthenticateEntitySuccess()));
		return authenticateEntitySuccess;
	}

	@Override
	public void setAuthenticateEntitySuccess(
			AuthenticateEntitySuccess authenticateEntitySuccess) throws MXMException {
		AuthenticateEntitySuccessType authenticateEntitySuccessType = (AuthenticateEntitySuccessType)	(authenticateEntitySuccess.createObject(AuthenticateEntitySuccessType.class.getName()).getContent());
		((AuthenticateEntityResponseType)protocolBaseType).setAuthenticateEntitySuccess(authenticateEntitySuccessType);	
	}

	@Override
	public ProtocolFailure getAuthenticateEntityFailure() throws MXMEngineException {
		ProtocolFailure protocolFailure = baseProtocolEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
		protocolFailure.parse(new MXMAdapter(((AuthenticateEntityResponseType)protocolBaseType).getAuthenticateEntityFailure()));
		return protocolFailure;
	}

	@Override
	public void setAuthenticateEntityFailure(ProtocolFailure protocolFailure) throws MXMException {
		ProtocolFailureType protocolFailureType = (ProtocolFailureType)
				(protocolFailure.createObject(ProtocolFailureType.class.getName()).getContent());
		((AuthenticateEntityResponseType)protocolBaseType).setAuthenticateEntityFailure(protocolFailureType);
	}
}
