/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The entitys of this file are subject to the Mozilla  Public Entity Version
 * 1.1 (the "Entity"); you may not use this file except in compliance with
 * the Entity. You may obtain a copy of the Entity at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the Entity is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the Entity
 * for the specific language governing rights and  limitations under the
 * Entity.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Angelo Difino
 * Contributor(s): 
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.authenticateentityPE.schema;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolSuccess;
import org.iso.mpeg.mxm.baseprotocol.schema.SuperBaseProtocolAbstractLevel1;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.authenticateentityPE.schema.authenticateentityprotocol.AuthenticateEntitySuccessType;
import org.iso.mpeg.mxm.engine.protocol.searchentityPE.schema.searchentityprotocol.SearchEntityRequestType;
import org.iso.mpeg.mxm.engine.protocol.searchentityPE.schema.searchentityprotocol.SearchEntityResponseType;
import org.iso.mpeg.mxm.engine.protocol.searchentityPE.schema.searchentityprotocol.SearchEntitySuccessType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureType;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.searchTE.schemahandler.mpqf.OutputQuery;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;

/**
 * @author Angelo Difino, Cedeo.net <angelo@cedeo.net>
 *
 */
public class AuthenticateEntitySuccess extends ProtocolSuccess implements org.iso.mpeg.mxm.pEngine.authenticateentityPE.schemahandler.AuthenticateEntitySuccess
{
	SecurityEngine securityEngine;
	public AuthenticateEntitySuccess(SecurityEngine securityEngine) 
	{
		super(new AuthenticateEntitySuccessType());
		this.securityEngine=securityEngine;
	}
	
	public AuthenticateEntitySuccess(AuthenticateEntitySuccessType aest, SecurityEngine securityEngine) 
	{
		super(aest);
		this.securityEngine=securityEngine;		
	}

	@Override
	public void createFile(File f) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1((AuthenticateEntitySuccessType)protocolSuccessType);
		level1.createFile(f);
	}
	
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1((AuthenticateEntitySuccessType)protocolSuccessType);
		return level1.createObject(objectType);
	}
	
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((AuthenticateEntitySuccessType)protocolSuccessType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolSuccessType = 
					(AuthenticateEntitySuccessType)(level1.createObject(AuthenticateEntitySuccessType.class.getName()).getContent());
		}
		return response;
	}

	@Override
	public Signature getEntitySignature() throws MXMEngineException {
		SignatureType signatureType = ((AuthenticateEntitySuccessType)protocolSuccessType).getEntitySignature();
		Signature signature = securityEngine.getSecurityEngineSchemaHandler().getSignature();
		signature.parse(new MXMAdapter(signatureType));
		
		return signature;
	}

	@Override
	public void setEntitySignature(Signature signature) throws MXMException {
		SignatureType signatureType = (SignatureType)(signature.createObject(SignatureType.class.getName()).getContent());
		((AuthenticateEntitySuccessType)protocolSuccessType).setEntitySignature(signatureType);
	}
}
