/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.authenticateuserPE.technology.remote;


import java.net.URL;
import java.security.PublicKey;

import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.ServiceFactory;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.es.authenticateuserES.AuthenticateUserProtocolRemote;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserRequest;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserResponse;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 */
public class AuthenticateUserProtocol
    implements
    org.iso.mpeg.mxm.pEngine.authenticateuserPE.technologyhandler.AuthenticateUserProtocol
{
    private URL            serviceURL;
    private QName          serviceName;
    private SecurityEngine securityEngine;
    private Logger         logger;

    public AuthenticateUserProtocol(SecurityEngine securityEngine)
    {
        this.securityEngine = securityEngine;
        logger = LoggerFactory.getLogger(this.getClass());
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.pEngine.authenticateuserPE.technologyhandler.AuthenticateUserProtocol
     * #initServiceParams(java.net.URL, javax.xml.namespace.QName)
     */
    @Override
    public MXMEngineResponse initServiceParams(URL serviceURL, QName serviceName)
    {
        this.serviceURL = serviceURL;
        this.serviceName = serviceName;
        return MXMEngineResponse.SUCCESS;
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.pEngine.authenticateuserPE.technologyhandler.AuthenticateUserProtocol
     * #authenticateUser(org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.
     * AuthenticateUserRequest)
     */
    @Override
    public AuthenticateUserResponse authenticateUser(AuthenticateUserRequest request)
    {
        // TODO Auto-generated method stub
        if (serviceURL == null)
        {
            logger.warn("Service URL has not been set");
        }
        if (serviceName == null)
        {
            logger.warn("Service Name has not been set");
        }
        ServiceFactory serviceFactory = null;
        try
        {
            serviceFactory = ServiceFactory.newInstance();
            Service service = serviceFactory.createService(serviceURL, serviceName);
            String serviceRequest = (String)(request.createObject(String.class
                .getCanonicalName()).getContent());
            AuthenticateUserProtocolRemote authenticateUserProtocolRemote = (AuthenticateUserProtocolRemote)service
                .getPort(AuthenticateUserProtocolRemote.class);
            String serviceResponse = authenticateUserProtocolRemote
                .authenticateUser(serviceRequest);
            AuthenticateUserResponse authenticateUserResponse = new org.iso.mpeg.mxm.authenticateuserPE.schema.AuthenticateUserResponse(
                securityEngine);
            authenticateUserResponse.parse(new MXMAdapter(serviceResponse));
            return authenticateUserResponse;
        }
        catch (ServiceException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (MXMException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public String getChallangeFor(PublicKey key)
    {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean isAuthorised(AuthenticateUserResponse response) throws MXMException
    {
        // TODO Auto-generated method stub
        return false;
    }
}
