import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.net.PasswordAuthentication;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.xml.bind.JAXBElement;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.xerces.impl.dv.util.Base64;
import org.iso.mpeg.mxm.baseprotocol.schema.ContentEntityType;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_mp21.DIDLInfoType;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_mp21.DIDLType;
import org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpmsg.AuthCodesType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.AgentObjectType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.AgentType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.ClassificationPreferencesType.Subject;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.ContentCollectionType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.PersonType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.PlaceType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.PlaceType.GeographicPosition;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.TextualType;
import org.iso.mpeg.mxm.engine.relTE.schema.relr.DigitalResource;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.CanonicalizationMethodType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.KeyInfoType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureMethodType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureValueType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignedInfoType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AdviceType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AssertionType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AuthnContextType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AuthnStatementType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.ConditionsType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.NameIDType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.StatementAbstractType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.SubjectConfirmationType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.SubjectType;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.AuthenticateUserEngine;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserRequest;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserResponse;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.AuthorizeUserEngine;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.schemahandler.AuthorizeUserRequest;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ApplicationSpecificData;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.UserEntity;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.IdentifyUserEngine;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserResponse;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Agent;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.NameComponent;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Person;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.PersonName;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Place;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.PlaceGeographicPosition;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Assertion;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnContext;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnStatement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.ConditionAbstract;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Conditions;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.NameID;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.SubjectConfirmation;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AuthnRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.Response;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.UserDataBaseManager;

import junit.framework.TestCase;


public class AuthenticateUserTest extends TestCase
{
    private MXM                    mxm;
    private CertificateManager     certMngr    = null;
    private SecurityEngine         securityEngine;
    private AuthenticateUserEngine authenticateUserUseEngine;
    private DIEngine               diEngine;
    public static boolean          initialized = false;
    private String                 APP_ALIAS   = "1";

    public AuthenticateUserTest() throws Exception
    {
        File f = new File(getClass().getClassLoader()
            .getResource("MXMConfiguration1.xml").getFile());
        if (initialized) return;
        initialized = true;
        mxm = MXM.getInstance(f);
        mxm.listEnginesAndProperties();
    }

    private boolean checkConf()
    {
        if (securityEngine.getPropertyValue("skipAuthentication").equals("true"))
        {
            System.out.println("skipAuthentication = true");
            return false;
        }
        return true;
    }

    public void testAuthenticateUserRequest() throws Exception
    {
        System.out.println("\n--- testAuthenticateUserRequest ---");
        // IdentifyUserEngine identifyUserEngine = (IdentifyUserEngine)mxm
        // .getDefaultEngine(MXMEngineName.IdentifyUserPE.toString());
        securityEngine = (SecurityEngine)mxm.getDefaultEngine(MXMEngineName.SecurityTE
            .toString());
        securityEngine.setProperty("conf", "123456");
        securityEngine.setProperty("auth", "323456");
        securityEngine.setProperty("sig", "123456");
        securityEngine.setProperty("gakp", "12345678");
        securityEngine.setProperty("appAlias", "1");
        if (!checkConf())
        {
            return;
        }
        authenticateUserUseEngine = (AuthenticateUserEngine)mxm
            .getDefaultEngine(MXMEngineName.AuthenticateUserPE.toString());
        diEngine = (DIEngine)mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
        this.certMngr = securityEngine.getSecurityEngineTechnologyHandler()
            .getCertificateManager();
        authenticate();
    }

    public void authenticate() throws MXMEngineException
    {
        if (!checkConf())
        {
            return;
        }
        System.out.println("TESTING USER AUTHENTICATION");
        Assertion assertion = securityEngine.getSecurityEngineSchemaHandler()
                .getAssertion();
        AuthnRequest request = this.securityEngine.getSecurityEngineSchemaHandler().getAuthnRequest();
        PublicKey storedKey = this.securityEngine
                .getSecurityEngineTechnologyHandler().getAuthenticationManager()
                .getPublicKey(APP_ALIAS, "auth", "");
            String hashForPublicKey = Base64.encode(storedKey.getEncoded());
            assertion.setID(hashForPublicKey.hashCode() + "");
            
        // todo: generate test signature form the user private key stored in his card.
        // String hashCode = publicKey.hashCode() + "";
        try
        {
            
            System.out.println("1: Acess resource");
            System.out.println("2: redirect to authrequest to identify provider");
            request.setID(hashForPublicKey.hashCode() + "");
            Response idProviderResponse = this.securityEngine.getSecurityEngineTechnologyHandler().getUserDataBaseManager().authenticateUser(request);
            
            System.out.println("user id:"+idProviderResponse.getID());
            assertTrue((idProviderResponse.getID()!= "-1"));
            assertTrue((idProviderResponse.getAssertionOrEncryptedAssertion().size() > 0));
            
            System.out.println("3: SAML response recived from identify provider, which conatins signed assertion");
            
            System.out.println("4: Send request for a resource with this assertioin as proof of authentication");
            
            AuthenticateUserRequest request1 = authenticateUserUseEngine
                    .getAuthenticateUserSchemaHandler().getAuthenticateUserRequest();
            request1.setAssertion(extractAssertion(idProviderResponse));
            
            AuthenticateUserResponse serviceProviderResponse = authenticateUserUseEngine
            .getAuthenticateUserTechnologyHandler()
            .getLocalAuthenticateUserProtocol().authenticateUser(request1);
         
            System.out.println("5: service provider returned with response for request");
            assertTrue(serviceProviderResponse.getApplicationSpecificData().size()>0);
            if(serviceProviderResponse.getApplicationSpecificData().size()>0){
                System.out.println("6: Resource granted");
                Object object = serviceProviderResponse.getApplicationSpecificData().get(0);
                if(object.getClass().getCanonicalName().equals(org.iso.mpeg.mxm.basePE.ApplicationSpecificData.class.getCanonicalName())) {
                    System.out.println("Recived data for request is: "+ ((ApplicationSpecificData)object).getAny().toString());
                }else {
                    System.out.println("Recived data for request is: "+ object.getClass().getCanonicalName());
                }
            }
            else {
                System.out.println("6: Resource not granted");
            }
            // / System.out.println(response.createObject(String.class.getCanonicalName())
            // .getContent());
        }
        catch (MXMException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
    
    
    private Assertion extractAssertion(Response idProviderResponse) {
        for(int i = 0;i< idProviderResponse.getAssertionOrEncryptedAssertion().size();i++) {
            Object object = idProviderResponse.getAssertionOrEncryptedAssertion().get(i);
            if(object.getClass().getCanonicalName().equals(org.iso.mpeg.mxm.securityTE.schemahandler.saml.Assertion.class.getCanonicalName()) ) {
                Assertion assertion = (Assertion)object;
                if(assertion != null) {
                    return assertion;
                }
            }
        }
        return null;
    }
}
