/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.basePE;

import java.util.List;

import org.iso.mpeg.mxm.core.MXMAbstractEngine;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.BaseProtocolSchemaHandler;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public abstract class BaseProtocolEngine extends MXMAbstractEngine implements org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine
{
	protected MetadataEngine metadataEngine;
	protected DIEngine diEngine;
	protected RELEngine relEngine;
	protected SecurityEngine securityEngine;
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolEngine#getSchemaHandler()
	 */
	@Override
	public BaseProtocolSchemaHandler getBaseProtocolSchemaHandler()
	{
		return new org.iso.mpeg.mxm.basePE.BaseProtocolSchemaHandler(metadataEngine, diEngine, relEngine, securityEngine);
	}

	public void setDependencies(List<MXMEngine> engines)
			throws MXMEngineException
	{
		if (engines != null)
		{
			for (MXMEngine engine : engines)
			{
				String engineName = engine.getEngineName();
				if (engineName.equals(MXMEngineName.MetadataTE.value()))
				{
					this.metadataEngine = (MetadataEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.DigitalItemTE.value()))
				{
					this.diEngine = (DIEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.RELTE.value()))
				{
					this.relEngine = (RELEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.SecurityTE.value()))
				{
					this.securityEngine = (SecurityEngine)engine;
				}
//				else 
//				{
//					throw new MXMEngineException("There is no requirement for Engine: " + engineName);
//				}
			}
		}
//		if (metadataEngine==null || diEngine==null || relEngine==null || securityEngine==null)
//			throw new MXMEngineException("dependencies not setted correctly");
	}

}
