/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.basePE;

import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ApplicationSpecificData;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContractEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.KeyValueData;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolSuccess;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.Service;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ServiceEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.SupportedMetadataSchema;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.SupportedTransferProtocol;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.UserEntity;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class BaseProtocolSchemaHandler extends org.iso.mpeg.mxm.pEngine.basePE.schemahandler.BaseProtocolSchemaHandler
{
	private DIEngine diEngine;
	private MetadataEngine metadataEngine;
	private RELEngine relEngine;
	private SecurityEngine securityEngine;
	
	public BaseProtocolSchemaHandler(MetadataEngine metadataEngine, DIEngine diEngine, RELEngine relEngine,SecurityEngine securityEngine)
	{
		this.metadataEngine = metadataEngine;
		this.diEngine = diEngine;
		this.relEngine = relEngine;
		this.securityEngine = securityEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getApplicationSpecificData()
	 */
	@Override
	public ApplicationSpecificData getApplicationSpecificData()
	{
		return new org.iso.mpeg.mxm.basePE.ApplicationSpecificData();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getContentEntity()
	 */
	@Override
	public ContentEntity getContentEntity()
	{
		return new org.iso.mpeg.mxm.basePE.ContentEntity(diEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getContractEntity()
	 */
	@Override
	public ContractEntity getContractEntity()
	{
		return new org.iso.mpeg.mxm.basePE.ContractEntity(metadataEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getKeyValueData()
	 */
	@Override
	public KeyValueData getKeyValueData()
	{
		return new org.iso.mpeg.mxm.basePE.KeyValyueData();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getLicenseEntity()
	 */
	@Override
	public LicenseEntity getLicenseEntity()
	{
		return new org.iso.mpeg.mxm.basePE.LicenseEntity(relEngine, metadataEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getProtocolFailure()
	 */
	@Override
	public ProtocolFailure getProtocolFailure()
	{
		return new org.iso.mpeg.mxm.basePE.ProtocolFailure();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getProtocolSuccess()
	 */
	@Override
	public ProtocolSuccess getProtocolSuccess()
	{
		return new org.iso.mpeg.mxm.basePE.ProtocolSuccess();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getService(org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine)
	 */
	@Override
	public Service getService()
	{
		return new org.iso.mpeg.mxm.basePE.Service(metadataEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getServiceEntity(org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine)
	 */
	@Override
	public ServiceEntity getServiceEntity()
	{
		return new org.iso.mpeg.mxm.basePE.ServiceEntity(metadataEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getSupportedMetadataSchema()
	 */
	@Override
	public SupportedMetadataSchema getSupportedMetadataSchema()
	{
		return new org.iso.mpeg.mxm.basePE.SupportedMetadataSchema();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getSupportedTransferProtocol()
	 */
	@Override
	public SupportedTransferProtocol getSupportedTransferProtocol()
	{
		return new org.iso.mpeg.mxm.basePE.SupportedTransferProtocol(metadataEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.BaseProtocolSchemaHandler#getUserEntity()
	 */
	@Override
	public UserEntity getUserEntity()
	{
		return new org.iso.mpeg.mxm.basePE.UserEntity(metadataEngine);
	}
}
