/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.basePE;

import java.io.File;
import java.util.List;

import org.iso.mpeg.mxm.baseprotocol.schema.ContentEntityType;
import org.iso.mpeg.mxm.baseprotocol.schema.SuperBaseProtocolAbstractLevel1;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.DIDLType;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;

/**
 * @author Angelos-Christos Anadiotis
 * @author Angelo Difino
 *
 */
public class ContentEntity extends EntityBase implements org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity
{
	private ContentEntityType contentEntityType;
	private DIEngine diEngine;
	
	public ContentEntity(DIEngine diEngine)
	{
		this.contentEntityType = new ContentEntityType();
		this.diEngine = diEngine;
	}
		
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1(contentEntityType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1(contentEntityType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1(contentEntityType);
		MXMEngineResponse response=level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			this.contentEntityType=(ContentEntityType)(level1.createObject(ContentEntityType.class.getCanonicalName())).getContent();
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ContentEntity#getDIDL()
	 */
	@Override
	public org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_mp21.DIDL getDIDL() throws MXMException
	{
		//TODO to test 
		
		DIDLType didlType ;
		
		if ( ! (contentEntityType.getDIDL() instanceof org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.DIDLType))
			throw new MXMEngineException("contentEntityType contains a DIDLType not in line with mpegm profile");
		else
			 didlType = (org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.DIDLType) contentEntityType.getDIDL();
		
		org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL didl=diEngine.getDIEngineSchemaHandler().getDIDL();
		didl.parse(new MXMAdapter(didlType));

		return didl;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ContentEntity#setDIDL(org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_mp21.DIDL)
	 */
	@Override
	public void setDIDL(org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_mp21.DIDL dIDL) throws MXMException
	{
		
		//TODO to test 
		
		DIDLType didltype;
		if ( ! (dIDL instanceof org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL))
			throw new MXMEngineException("contentEntityType contains a DIDLType not in line with mpegm profile");
		else
			didltype=(DIDLType) dIDL.createObject(DIDLType.class.getName()).getContent();
		
		contentEntityType.setDIDL(didltype);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ContentEntity#getContentRef()
	 */
	@Override
	public List<String> getContentRef()
	{
		return contentEntityType.getContentRef();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ContentEntity#getIdentifier()
	 */
	@Override
	public List<String> getIdentifier()
	{
		return contentEntityType.getIdentifier();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ContentEntity#addContentRef(java.lang.String)
	 */
	@Override
	public void addContentRef(String contentRef)
	{
		contentEntityType.getContentRef().add(contentRef);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ContentEntity#addIdentifier(java.lang.String)
	 */
	@Override
	public void addIdentifier(String identifier)
	{
		contentEntityType.getIdentifier().add(identifier);
	}	
}