/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.basePE;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.iso.mpeg.mxm.basePE.KeyValyueData;
import org.iso.mpeg.mxm.baseprotocol.schema.ApplicationSpecificDataType;
import org.iso.mpeg.mxm.baseprotocol.schema.KeyValueDataType;
import org.iso.mpeg.mxm.baseprotocol.schema.ProtocolBaseType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureType;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ApplicationSpecificData;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.KeyValueData;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public abstract class ProtocolBase implements org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolBase
{
	protected ProtocolBaseType protocolBaseType;
	protected SecurityEngine securityEngine;
	
	protected ProtocolBase()
	{
		
	}
	
	protected ProtocolBase(ProtocolBaseType protocolBaseType, SecurityEngine securityEngine)
	{
		this.protocolBaseType = protocolBaseType;
		this.securityEngine = securityEngine;
	}
		
	protected void init(ProtocolBaseType protocolBaseType, SecurityEngine securityEngine)
	{
		this.protocolBaseType = protocolBaseType;
		this.securityEngine = securityEngine;
	}
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(protocolBaseType);
		level1.createFile(f);
	}
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(protocolBaseType);
		return level1.createObject(objectType);
	}
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(protocolBaseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolBaseType = (ProtocolBaseType)(level1.createObject(ProtocolBaseType.class.getCanonicalName()).getContent());
		}
		return response;
	}
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ProtocolBase#setSignature(org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature)
	 */
	@Override
	public void setSignature(Signature signature) throws MXMException
	{
		SignatureType signatureType = (SignatureType)(signature.createObject(SignatureType.class.getName()).getContent());
		protocolBaseType.setSignature(signatureType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ProtocolBase#getTransactionIdentifier()
	 */
	@Override
	public String getTransactionIdentifier()
	{
		return protocolBaseType.getTransactionIdentifier();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ProtocolBase#setTransactionIdentifier(java.lang.String)
	 */
	@Override
	public void setTransactionIdentifier(String string)
	{
		protocolBaseType.setTransactionIdentifier(string);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ProtocolBase#getApplicationSpecificData()
	 */
	@Override
	public List<ApplicationSpecificData> getApplicationSpecificData() throws MXMEngineException
	{	
		List<ApplicationSpecificDataType> applicationSpecificDataTypes = protocolBaseType.getApplicationSpecificData();
		if (applicationSpecificDataTypes != null)
		{
			List<ApplicationSpecificData> applicationSpecificDatas = new ArrayList<ApplicationSpecificData>();
			for (ApplicationSpecificDataType applicationSpecificDataType : applicationSpecificDataTypes)
			{
				org.iso.mpeg.mxm.basePE.ApplicationSpecificData applicationSpecificData = new org.iso.mpeg.mxm.basePE.ApplicationSpecificData();
				applicationSpecificData.parse(new MXMAdapter(applicationSpecificDataType));
				applicationSpecificDatas.add(applicationSpecificData);
			}
			
			return applicationSpecificDatas;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ProtocolBase#getSignature()
	 */
	@Override
	public Signature getSignature() throws MXMEngineException
	{
		Signature signature = securityEngine.getSecurityEngineSchemaHandler().getSignature();
		if (signature.parse(new MXMAdapter(protocolBaseType.getSignature())) == MXMEngineResponse.SUCCESS)
		{
			return signature;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ProtocolBase#getEntry()
	 */
	@Override
	public List<KeyValueData> getEntry() throws MXMEngineException
	{
		List<KeyValueDataType> keyValueDataTypes = protocolBaseType.getEntry();
		if (keyValueDataTypes != null)
		{
			System.out.println("KEYKEY is not null");
			List<KeyValueData> keyValueDatas = new ArrayList<KeyValueData>();
			for (KeyValueDataType keyValueDataType : keyValueDataTypes)
			{
				KeyValyueData keyValyueData = new KeyValyueData();
				keyValyueData.parse(new MXMAdapter(keyValueDataType));
				System.out.println("KEYKEY adding "+keyValyueData.getKey()+ " "+keyValyueData.getValue());
				keyValueDatas.add(keyValyueData);
			}
			return keyValueDatas;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ProtocolBase#setTimestamp(javax.xml.datatype.XMLGregorianCalendar)
	 */
	@Override
	public void setTimestamp(Date timestamp)
	{
		GregorianCalendar c = new GregorianCalendar();
		c.setTime(timestamp);
		XMLGregorianCalendar xmlGregorianCalendar = null;
		try {
			xmlGregorianCalendar = DatatypeFactory.newInstance().newXMLGregorianCalendar(c);
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}
		
		protocolBaseType.setTimestamp(xmlGregorianCalendar);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ProtocolBase#getTimestamp()
	 */
	@Override
	public Date getTimestamp()
	{		
		return protocolBaseType.getTimestamp().toGregorianCalendar().getTime();
	}
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ProtocolBase#addApplicationSpecificData(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ApplicationSpecificData)
	 */
	@Override
	public void addApplicationSpecificData(
			ApplicationSpecificData applicationSpecificData) throws MXMException
	{
		ApplicationSpecificDataType applicationSpecificDataType = 
				(ApplicationSpecificDataType)(applicationSpecificData.createObject(ApplicationSpecificDataType.class.getName())).getContent();
		protocolBaseType.getApplicationSpecificData().add(applicationSpecificDataType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ProtocolBase#addEntry(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.KeyValueData)
	 */
	@Override
	public void addEntry(KeyValueData entry) throws MXMException
	{
		KeyValueDataType keyValueDataType = (KeyValueDataType)(entry.createObject(KeyValueDataType.class.getName())).getContent();
		protocolBaseType.getEntry().add(keyValueDataType);
	}
}
