/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.basePE;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.baseprotocol.schema.KeyValueDataType;
import org.iso.mpeg.mxm.baseprotocol.schema.SuperBaseProtocolAbstractLevel1;
import org.iso.mpeg.mxm.baseprotocol.schema.SupportedTransferProtocolType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.ControlledTermUseType;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.KeyValueData;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.ControlledTermUse;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class SupportedTransferProtocol implements org.iso.mpeg.mxm.pEngine.basePE.schemahandler.SupportedTransferProtocol
{
	private SupportedTransferProtocolType supportedTransferProtocolType;
	
	private MetadataEngine metadataEngine;
	
	public SupportedTransferProtocol(MetadataEngine metadataEngine)
	{
		supportedTransferProtocolType = new SupportedTransferProtocolType();
		
		this.metadataEngine = metadataEngine;
	}
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1(supportedTransferProtocolType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1(supportedTransferProtocolType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1(supportedTransferProtocolType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			supportedTransferProtocolType = 
					(SupportedTransferProtocolType)(level1.createObject(SupportedTransferProtocolType.class.getName()).getContent());
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.SupportedTransferProtocol#getPref()
	 */
	@Override
	public Float getPref()
	{
		return supportedTransferProtocolType.getPref();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.SupportedTransferProtocol#setPref(java.lang.Float)
	 */
	@Override
	public void setPref(Float f)
	{
		supportedTransferProtocolType.setPref(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.SupportedTransferProtocol#getTransferProtocol()
	 */
	@Override
	public ControlledTermUse getTransferProtocol() throws MXMEngineException
	{
		ControlledTermUse controlledTermUse = metadataEngine.getMetadataEngineSchemaHandler().getControlledTermUse();
		controlledTermUse.parse(new MXMAdapter(supportedTransferProtocolType.getTransferProtocol()));
		return controlledTermUse;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.SupportedTransferProtocol#setTransferProtocol(org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.ControlledTermUse)
	 */
	@Override
	public void setTransferProtocol(ControlledTermUse controlledTermUse) throws MXMException
	{
		ControlledTermUseType controlledTermUseType = (ControlledTermUseType)(controlledTermUse.createObject(ControlledTermUseType.class.getName())).getContent();
		supportedTransferProtocolType.setTransferProtocol(controlledTermUseType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.SupportedTransferProtocol#getOption()
	 */
	@Override
	public List<KeyValueData> getOption() throws MXMEngineException
	{
		List<KeyValueDataType> keyValueDataTypes = supportedTransferProtocolType.getOption();
		if (keyValueDataTypes != null)
		{
			List<KeyValueData> keyValueDatas = new ArrayList<KeyValueData>();
			for (KeyValueDataType keyValueDataType : keyValueDataTypes)
			{
				KeyValueData keyValueData = new KeyValyueData();
				keyValueData.parse(new MXMAdapter(keyValueDataType));
				keyValueDatas.add(keyValueData);
			}
			
			return keyValueDatas;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.SupportedTransferProtocol#addOption(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.KeyValueData)
	 */
	@Override
	public void addOption(KeyValueData option) throws MXMException
	{
		KeyValueDataType keyValueDataType = (KeyValueDataType)(option.createObject(KeyValueDataType.class.getName())).getContent();
		supportedTransferProtocolType.getOption().add(keyValueDataType);
	}

}
