/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.basePE;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.baseprotocol.schema.ContractEntityType;
import org.iso.mpeg.mxm.baseprotocol.schema.SuperBaseProtocolAbstractLevel1;
import org.iso.mpeg.mxm.baseprotocol.schema.UserEntityType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.AgentType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.UniqueIDType;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Agent;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.UniqueID;



/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class UserEntity extends EntityBase implements org.iso.mpeg.mxm.pEngine.basePE.schemahandler.UserEntity
{
	private UserEntityType userEntityType;
	
	private MetadataEngine metadataEngine;
	
	public UserEntity(MetadataEngine metadataEngine)
	{
		userEntityType = new UserEntityType();
		
		this.metadataEngine = metadataEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.UserEntity#getUser()
	 */
	@Override
	public Agent getUser() throws MXMEngineException
	{
		Agent agent = (Agent) metadataEngine.getMetadataEngineSchemaHandler().getPerson();
		agent.parse(new MXMAdapter(userEntityType.getUser()));
		return agent;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.UserEntity#setUser(org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Agent)
	 */
	@Override
	public void setUser(Agent agent) throws MXMException
	{
		AgentType agentType = (AgentType)(agent.createObject(AgentType.class.getName())).getContent();
		userEntityType.setUser(agentType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.UserEntity#getUserRef()
	 */
	@Override
	public List<String> getUserRef()
	{
		return userEntityType.getUserRef();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.UserEntity#getUserIdentifier()
	 */
	@Override
	public List<UniqueID> getUserIdentifier() throws MXMEngineException
	{
		List<UniqueIDType> uniqueIDTypes = userEntityType.getUserIdentifier();
		List<UniqueID> uniqueIDs = new ArrayList<UniqueID>();
		if (uniqueIDTypes != null)
		{
			for (UniqueIDType uniqueIDType : uniqueIDTypes)
			{
				UniqueID uniqueID = metadataEngine.getMetadataEngineSchemaHandler().getUniqueID();
				MXMEngineResponse response = uniqueID.parse(new MXMAdapter(uniqueIDType));
				if (!response.equals(MXMEngineResponse.SUCCESS))
				{
					throw new MXMEngineException("Error parsing UniqueID: " + response.value());
				}
				uniqueIDs.add(uniqueID);
			}
		}
		return uniqueIDs;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.UserEntity#addUserRef(java.lang.String)
	 */
	@Override
	public void addUserRef(String ref)
	{
		userEntityType.getUserRef().add(ref);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.UserEntity#addUniqueID(org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.UniqueID)
	 */
	@Override
	public void addUniqueID(UniqueID uniqueID) throws MXMException
	{
		UniqueIDType uniqueIDType = (UniqueIDType)(uniqueID.createObject(UniqueIDType.class.getName())).getContent();
		userEntityType.getUserIdentifier().add(uniqueIDType);
	}

	// MXM CREATOR AND PARSER METHODS

	@Override
	public void createFile(File f) throws MXMException
	{
		new SuperBaseProtocolAbstractLevel1(userEntityType).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		return new SuperBaseProtocolAbstractLevel1(userEntityType).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperBaseProtocolAbstractLevel1 superAbstract = new SuperBaseProtocolAbstractLevel1(userEntityType);

		MXMEngineResponse repsonse = superAbstract.parse(object);

		userEntityType = (UserEntityType) 
				superAbstract.createObject(UserEntityType.class.getCanonicalName()).getContent();

		return repsonse;
	}
}