/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.checkwithlicensePE.schema;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.basePE.ProtocolRequest;
import org.iso.mpeg.mxm.baseprotocol.schema.LicenseEntityType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.checkwithlicensePE.schema.protocol.CheckWithLicenseRequestType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Condition;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Principal;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Right;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class CheckWithLicenseRequest extends ProtocolRequest
						implements org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest
{	
	private BaseProtocolEngine baseProtocolEngine;
	private RELEngine relEngine;
	
	public CheckWithLicenseRequest(BaseProtocolEngine baseProtocolEngine, RELEngine relEngine, SecurityEngine securityEngine)
	{
		super(new CheckWithLicenseRequestType(), securityEngine);
		this.baseProtocolEngine = baseProtocolEngine;
		this.relEngine = relEngine;		
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CheckWithLicenseRequestType)protocolBaseType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CheckWithLicenseRequestType)protocolBaseType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CheckWithLicenseRequestType)protocolBaseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolBaseType = 
					(CheckWithLicenseRequestType)(level1.createObject(CheckWithLicenseRequestType.class.getName()).getContent());
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest#setLicenseEntity(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity)
	 */
	@Override
	public void setLicenseEntity(LicenseEntity licenseEntity) throws MXMException
	{
		LicenseEntityType licenseEntityType = 
				(LicenseEntityType)(licenseEntity.createObject(LicenseEntityType.class.getName()).getContent());
		((CheckWithLicenseRequestType)protocolBaseType).setLicenseEntity(licenseEntityType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest#getLicenseEntity()
	 */
	@Override
	public LicenseEntity getLicenseEntity() throws MXMEngineException
	{
		LicenseEntityType licenseEntityType = ((CheckWithLicenseRequestType)protocolBaseType).getLicenseEntity();
		LicenseEntity licenseEntity = baseProtocolEngine.getBaseProtocolSchemaHandler().getLicenseEntity();
		licenseEntity.parse(new MXMAdapter(licenseEntityType));
		return licenseEntity;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest#setPrincipal(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Principal)
	 */
	@Override
	public void setPrincipal(Principal principal) throws MXMException
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.Principal principalType = 
				(org.iso.mpeg.mxm.engine.relTE.schema.relr.Principal)
					(principal.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.Principal.class.getName()).getContent());
		((CheckWithLicenseRequestType)protocolBaseType).setPrincipal(principalType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest#getPrincipal()
	 */
	@Override
	public Principal getPrincipal() throws MXMEngineException
	{
		Principal principal = relEngine.getRELEngineSchemaHandler().getPrincipal();
		principal.parse(new MXMAdapter(((CheckWithLicenseRequestType)protocolBaseType).getPrincipal()));
		return principal;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest#addRight(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Right)
	 */
	@Override
	public void addRight(Right right) throws MXMException
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.Right rightType = 
				(org.iso.mpeg.mxm.engine.relTE.schema.relr.Right)
					(right.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.Right.class.getName()).getContent());
		((CheckWithLicenseRequestType)protocolBaseType).getRight().add(rightType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest#getRights()
	 */
	@Override
	public List<Right> getRights() throws MXMEngineException
	{
		List<org.iso.mpeg.mxm.engine.relTE.schema.relr.Right> rightsType = ((CheckWithLicenseRequestType)protocolBaseType).getRight();
		List<Right> rights = new ArrayList<Right>();
		
		if (rightsType != null)
		{
			for (org.iso.mpeg.mxm.engine.relTE.schema.relr.Right rightType : rightsType)
			{
				Right right = relEngine.getRELEngineSchemaHandler().getRight();
				MXMEngineResponse response = right.parse(new MXMAdapter(rightType));
				if (response.equals(MXMEngineResponse.SUCCESS))
				{
					rights.add(right);
				}
			}
		}
		
		return rights;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest#addResource(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource)
	 */
	@Override
	public void addResource(Resource resource) throws MXMException
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource resourceType = 
				(org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource)
					(resource.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource.class.getName()).getContent());
		((CheckWithLicenseRequestType)protocolBaseType).getResource().add(resourceType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest#getResources()
	 */
	@Override
	public List<Resource> getResources() throws MXMEngineException
	{
		List<org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource> resourcesType = ((CheckWithLicenseRequestType)protocolBaseType).getResource();
		List<Resource> resources = new ArrayList<Resource>();
		
		if (resourcesType != null)
		{
			for (org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource resourceType : resourcesType)
			{
				Resource resource = relEngine.getRELEngineSchemaHandler().getResource();
				MXMEngineResponse response = resource.parse(new MXMAdapter(resourceType));
				if (response.equals(MXMEngineResponse.SUCCESS))
				{
					resources.add(resource);
				}
			}
		}
		
		return resources;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest#addCondition(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Condition)
	 */
	@Override
	public void addCondition(Condition condition) throws MXMException
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition conditionType = 
				(org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition)
					(condition.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition.class.getName()).getContent());
		((CheckWithLicenseRequestType)protocolBaseType).getCondition().add(conditionType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.checkwithlicensePE.schemahandler.CheckWithLicenseRequest#getConditions()
	 */
	@Override
	public List<Condition> getConditions() throws MXMEngineException
	{
		List<org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition> conditionsType = ((CheckWithLicenseRequestType)protocolBaseType).getCondition();
		List<Condition> conditions = new ArrayList<Condition>();
		
		if (conditionsType != null)
		{
			for (org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition conditionType : conditionsType)
			{
				Condition condition = relEngine.getRELEngineSchemaHandler().getCondition();
				MXMEngineResponse response = condition.parse(new MXMAdapter(conditionType));
				if (response.equals(MXMEngineResponse.SUCCESS))
				{
					conditions.add(condition);
				}
			}
		}
		
		return conditions;
	}

}
