/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createcontentPE;

import java.util.List;

import org.iso.mpeg.mxm.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSchemaHandler;
import org.iso.mpeg.mxm.pEngine.createcontentPE.technologyhandler.CreateContentTechnologyHandler;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class CreateContentPE extends BaseProtocolEngine implements CreateContentEngine
{
	private IPMPEngine ipmpEngine;
	private EventReportEngine eventReportEngine;
	private OrchestratorEngine orchestratorEngine;
	public  DIEngine diEngine;

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine#getSchemaHandler()
	 */
	@Override
	public CreateContentSchemaHandler getCreateContentSchemaHandler()
	{
		return new org.iso.mpeg.mxm.createcontentPE.schemahandler.CreateContentSchemaHandler(this, ipmpEngine, 
															eventReportEngine, diEngine, securityEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine#getTechnologyHandler()
	 */
	@Override
	public CreateContentTechnologyHandler getCreateContentTechnologyHandler()
	{
		return new org.iso.mpeg.mxm.createcontentPE.technologyhandler.CreateContentTechnologyHandler(this, 
															orchestratorEngine, securityEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMEngine#getEngineName()
	 */
	@Override
	public String getEngineName()
	{
		return MXMEngineName.CreateContentPE.value();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMEngine#setDependencies(java.util.List)
	 */
	@Override
	public void setDependencies(List<MXMEngine> engines)
			throws MXMEngineException
	{
		// set basePE dependencies
		super.setDependencies(engines);
		
		if (engines != null)
		{
			for (MXMEngine engine : engines)
			{
				String engineName = engine.getEngineName();
				if (engineName.equals(MXMEngineName.EventReportTE.value()))
				{
					this.eventReportEngine = (EventReportEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.Orchestrator.value()))			
				{
					this.orchestratorEngine = (OrchestratorEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.IPMPTE.value()))
				{
					this.ipmpEngine = (IPMPEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.DigitalItemTE.value()))
				{
					this.diEngine = (DIEngine)engine;
				}
				
			}
		}
//		if (diEngine==null ||eventReportEngine==null || ipmpEngine==null )//orchestrator can be null for local side
//			throw new MXMEngineException("dependencies not setted correctly");
	}

}
