/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createcontentPE.schemahandler;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.baseprotocol.schema.LicenseEntityType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.ResourceType;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.DescriptorType;
import org.iso.mpeg.mxm.engine.eventreportTE.schema.er.ERRType;
import org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpinfo.IPMPGeneralInfoDescriptorType;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.createcontentPE.schema.createcontentprotocol.AssetType;
import org.iso.mpeg.mxm.engine.protocol.createcontentPE.schema.createcontentprotocol.IPMPToolInfoType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.IPMPToolInfo;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didmodel_mp21.Descriptor;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didmodel_mp21.Resource;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.IPMPGeneralInfoDescriptor;

/**
 * @author Angelos-Christos Anadiotis
 * @author Angelo Difino
 *
 */
public class Asset implements org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset
{
	private AssetType assetType;
	
	private BaseProtocolEngine baseProtocolEngine; 
	private IPMPEngine ipmpEngine;
	private DIEngine diEngine;
	private EventReportEngine errengine;
	
	public Asset(BaseProtocolEngine baseProtocolEngine, IPMPEngine ipmpEngine, DIEngine diEngine, EventReportEngine errengine)
	{
		this.assetType = new AssetType();
		
		this.baseProtocolEngine = baseProtocolEngine;
		this.ipmpEngine = ipmpEngine;
		this.diEngine = diEngine;
		this.errengine = errengine;
	}
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(assetType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(assetType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(assetType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			assetType = (AssetType)(level1.createObject(AssetType.class.getName()).getContent());
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset#getTool()
	 */
	@Override
	public IPMPGeneralInfoDescriptor getIPMPGeneralInfoDescriptor() throws MXMEngineException
	{
		if (assetType.getIPMPGeneralInfoDescriptor()!=null)
		{
			IPMPGeneralInfoDescriptorType iType = assetType.getIPMPGeneralInfoDescriptor();
			IPMPGeneralInfoDescriptor i=ipmpEngine.getIPMPEngineSchemaHandler().getIPMPGeneralInfoDescriptor();
			i.parse(new MXMAdapter(iType));
			
			return i;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset#setTool(org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.Tool)
	 */
	@Override
	public void setIPMPGeneralInfoDescriptor(IPMPGeneralInfoDescriptor i) throws MXMException
	{
		assetType.setIPMPGeneralInfoDescriptor((IPMPGeneralInfoDescriptorType)(i.createObject(IPMPGeneralInfoDescriptorType.class.getCanonicalName())).getContent());
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset#getLicenseEntity()
	 */
	@Override
	public LicenseEntity getLicenseEntity() throws MXMEngineException
	{
		LicenseEntityType licenseEntityType = assetType.getLicenseEntity();
		LicenseEntity licenseEntity = baseProtocolEngine.getBaseProtocolSchemaHandler().getLicenseEntity();
		licenseEntity.parse(new MXMAdapter(licenseEntityType));
		
		return licenseEntity;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset#setLicenseEntity(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.LicenseEntity)
	 */
	@Override
	public void setLicenseEntity(LicenseEntity licenseEntity) throws MXMException
	{
		assetType.setLicenseEntity((LicenseEntityType)licenseEntity.createObject(LicenseEntityType.class.getName()).getContent());
	}

	@Override
	public IPMPToolInfo getIPMPToolInfo() throws MXMEngineException {
		IPMPToolInfo ipmpToolInfo = new org.iso.mpeg.mxm.createcontentPE.schemahandler.IPMPToolInfo(ipmpEngine);
		ipmpToolInfo.parse(new MXMAdapter(assetType.getIPMPToolInfo()));
		return ipmpToolInfo;
	}

	@Override
	public void setIPMPToolInfo(IPMPToolInfo iPMPToolInfo) throws MXMException {
		IPMPToolInfoType ipmpToolInfoType = 
				(IPMPToolInfoType)(iPMPToolInfo.createObject(IPMPToolInfoType.class.getName()).getContent());
		assetType.setIPMPToolInfo(ipmpToolInfoType);
	}

	@Override
	public Descriptor getMetadata() throws MXMEngineException {
		
		if (assetType.getMetadata()!=null)
		{
			//TOTEST
			DescriptorType descriptorType;
			
			if ( ! (assetType.getMetadata() instanceof org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.DescriptorType))
				throw new MXMEngineException("descriptorType contains a DescriptorType not in line with mpegm profile");
			else
				descriptorType = (org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.DescriptorType) assetType.getMetadata();
			
			Descriptor descriptor = diEngine.getDIEngineSchemaHandler().getDescriptor();
			descriptor.parse(new MXMAdapter(descriptorType));
			return descriptor;
		}
		
		return null;
	}

	@Override
	public void setMetadata(Descriptor descriptor) throws MXMException {
		
		//TODO to test
		DescriptorType descriptorType;
		if ( ! (descriptor instanceof org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Descriptor))
			throw new MXMEngineException("descriptor contains a DescriptorType not in line with mpegm profile");
		else 
			descriptorType= (DescriptorType)(descriptor.createObject(DescriptorType.class.getName()).getContent());
		
		assetType.setMetadata(descriptorType);
	}

	@Override
	public void setResource(Resource resource) throws MXMEngineException, MXMException
	{
		//TODO to test
		ResourceType resourceType;
		
		if ( ! (resource instanceof org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource))
			throw new MXMEngineException("resource contains a ResourceType not in line with mpegm profile");
		else
			resourceType= (ResourceType)resource.createObject(ResourceType.class.getName()).getContent();
		
		assetType.setResource(resourceType);
	}
	
	@Override
	public Resource getResource() throws MXMEngineException 
	{
		// TODO to test
		ResourceType resType ;
		
		if ( ! (assetType.getResource() instanceof org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.ResourceType))
			throw new MXMEngineException("resourceType contains a Resource not in line with mpegm profile");
		else
			resType = (org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.ResourceType) assetType.getResource();
		
		org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource resource=diEngine.getDIEngineSchemaHandler().getResource();
		resource.parse(new MXMAdapter(resType));

		return resource;
	}

	@Override
	public List<ERR> getERRs() throws MXMEngineException {
		List<ERR> toreturn=new ArrayList();
		for (ERRType err:assetType.getERR()) {
			ERR new_err= errengine.getEventReportEngineSchemaHandler().getERR();
			new_err.parse(new MXMAdapter(err));
			toreturn.add(new_err);
		}
		return toreturn;	
	}

	@Override
	public void addERR(ERR err) throws MXMException {
		assetType.getERR().add((ERRType)err.createObject(ERRType.class.getName()).getContent());
	}
}
