/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createcontentPE.schemahandler;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.basePE.ProtocolRequest;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didmodel_mp21.DescriptorType;
import org.iso.mpeg.mxm.engine.eventreportTE.schema.er.ERRType;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.createcontentPE.schema.createcontentprotocol.AssetType;
import org.iso.mpeg.mxm.engine.protocol.createcontentPE.schema.createcontentprotocol.CreateContentRequestType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didmodel_mp21.Descriptor;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class CreateContentRequest extends ProtocolRequest 
					implements org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest
{	
	private EventReportEngine eventReportEngine;
	private DIEngine diEngine;
	private BaseProtocolEngine baseProtocolEngine;
	private IPMPEngine ipmpEngine;
	
	public CreateContentRequest(BaseProtocolEngine baseProtocolEngine, IPMPEngine ipmpEngine, 
								EventReportEngine eventReportEngine, DIEngine diEngine, SecurityEngine securityEngine)
	{
		super(new CreateContentRequestType(), securityEngine);
		this.eventReportEngine = eventReportEngine;
		this.diEngine = diEngine;
		this.baseProtocolEngine = baseProtocolEngine;
		this.ipmpEngine = ipmpEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CreateContentRequestType)protocolBaseType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CreateContentRequestType)protocolBaseType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CreateContentRequestType)protocolBaseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolBaseType = 
					(CreateContentRequestType)(level1.createObject(CreateContentRequestType.class.getName()).getContent());			
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest#getERR()
	 */
	@Override
	public ERR getERR() throws MXMEngineException
	{
		ERR err = eventReportEngine.getEventReportEngineSchemaHandler().getERR();
		err.parse(new MXMAdapter(((CreateContentRequestType)protocolBaseType).getERR()));
		return err;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest#setERR(org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR)
	 */
	@Override
	public void setERR(ERR eRR) throws MXMException
	{
		ERRType errType = (ERRType)(eRR.createObject(ERRType.class.getName()).getContent());
		((CreateContentRequestType)protocolBaseType).setERR(errType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest#getMetadata()
	 */
	@Override
	public Descriptor getMetadata() throws MXMEngineException
	{
		DescriptorType descriptorType = ((CreateContentRequestType)protocolBaseType).getMetadata();
		
		if (descriptorType!=null)
		{
			Descriptor descriptor = diEngine.getDIEngineSchemaHandler().getDescriptor();
			descriptor.parse(new MXMAdapter(descriptorType));
			return descriptor;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest#setMetadata(org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didmodel_mp21.Descriptor)
	 */
	@Override
	public void setMetadata(Descriptor descriptor) throws MXMException
	{
		DescriptorType descriptorType = (DescriptorType)(descriptor.createObject(DescriptorType.class.getName()).getContent());
		((CreateContentRequestType)protocolBaseType).setMetadata(descriptorType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest#getAsset()
	 */
	@Override
	public List<Asset> getAsset() throws MXMEngineException
	{
		List<AssetType> assetTypes = ((CreateContentRequestType)protocolBaseType).getAsset();
		List<Asset> assets = new ArrayList<Asset>();
		if (assetTypes != null)
		{
			for (AssetType assetType : assetTypes)
			{
				Asset asset = new org.iso.mpeg.mxm.createcontentPE.schemahandler.Asset(baseProtocolEngine, ipmpEngine, diEngine, eventReportEngine);
				MXMEngineResponse response = asset.parse(new MXMAdapter(assetType));
				if (!response.equals(MXMEngineResponse.SUCCESS))
				{
					throw new MXMEngineException("Error parsing Asset: " + response);
				}
				assets.add(asset);
			}
		}
		return assets;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest#addAsset(org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset)
	 */
	@Override
	public void addAsset(Asset asset) throws MXMException
	{
		AssetType assetType = (AssetType)(asset.createObject(AssetType.class.getName()).getContent());
		((CreateContentRequestType)protocolBaseType).getAsset().add(assetType);
	}

}
