/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createcontentPE.schemahandler;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolResponse;
import org.iso.mpeg.mxm.baseprotocol.schema.ProtocolFailureType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.createcontentPE.schema.createcontentprotocol.CreateContentResponseType;
import org.iso.mpeg.mxm.engine.protocol.createcontentPE.schema.createcontentprotocol.CreateContentSuccessType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSuccess;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;


/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class CreateContentResponse extends ProtocolResponse 
							implements org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse
{	
	private BaseProtocolEngine baseProtocolEngine;
	
	public CreateContentResponse(BaseProtocolEngine baseProtocolEngine, SecurityEngine securityEngine)
	{
		super(new CreateContentResponseType(), securityEngine);		
		this.baseProtocolEngine = baseProtocolEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CreateContentResponseType)protocolBaseType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CreateContentResponseType)protocolBaseType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CreateContentResponseType)protocolBaseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolBaseType = 
					(CreateContentResponseType)(level1.createObject(CreateContentResponseType.class.getName()).getContent());
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse#getCreateContentSuccess()
	 */
	@Override
	public CreateContentSuccess getCreateContentSuccess() throws MXMEngineException
	{
		CreateContentSuccessType createContentSuccessType = ((CreateContentResponseType)protocolBaseType).getCreateContentSuccess();
		CreateContentSuccess createContentSuccess = 
				new org.iso.mpeg.mxm.createcontentPE.schemahandler.CreateContentSuccess(baseProtocolEngine);
		createContentSuccess.parse(new MXMAdapter(createContentSuccessType));
		return createContentSuccess;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse#setCreateContentSuccess(org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSuccess)
	 */
	@Override
	public void setCreateContentSuccess(
			CreateContentSuccess createContentSuccess) throws MXMException
	{
		CreateContentSuccessType createContentSuccessType = 
				(CreateContentSuccessType)(createContentSuccess.createObject(CreateContentSuccessType.class.getName()).getContent());
		((CreateContentResponseType)protocolBaseType).setCreateContentSuccess(createContentSuccessType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse#getCreateContentFailure()
	 */
	@Override
	public ProtocolFailure getCreateContentFailure() throws MXMEngineException
	{
		ProtocolFailure protocolFailure = baseProtocolEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
		ProtocolFailureType protocolFailureType = ((CreateContentResponseType)protocolBaseType).getCreateContentFailure();
		protocolFailure.parse(new MXMAdapter(protocolFailureType));
		return protocolFailure;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse#setCreateContentFailure(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure)
	 */
	@Override
	public void setCreateContentFailure(ProtocolFailure protocolFailure) throws MXMException
	{
		ProtocolFailureType protocolFailureType = 
				(ProtocolFailureType)(protocolFailure.createObject(ProtocolFailureType.class.getName()).getContent());
		((CreateContentResponseType)protocolBaseType).setCreateContentFailure(protocolFailureType);
	}

}
