/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createcontentPE.schemahandler;

import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSuccess;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.IPMPToolInfo;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class CreateContentSchemaHandler extends org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSchemaHandler
{
	private BaseProtocolEngine baseProtocolEngine;
	private IPMPEngine ipmpEngine;
	private EventReportEngine eventReportEngine;
	private DIEngine diEngine;
	private SecurityEngine securityEngine;
	
	public CreateContentSchemaHandler(BaseProtocolEngine baseProtocolEngine, IPMPEngine ipmpEngine, EventReportEngine eventReportEngine,
													DIEngine diEngine, SecurityEngine securityEngine)
	{
		this.baseProtocolEngine = baseProtocolEngine;
		this.ipmpEngine = ipmpEngine;
		this.eventReportEngine = eventReportEngine;
		this.diEngine = diEngine;
		this.securityEngine = securityEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSchemaHandler#getCreateContentRequest()
	 */
	@Override
	public CreateContentRequest getCreateContentRequest()
	{
		return new org.iso.mpeg.mxm.createcontentPE.schemahandler.CreateContentRequest(baseProtocolEngine, ipmpEngine,
																			eventReportEngine, diEngine, securityEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSchemaHandler#getCreateContentResponse()
	 */
	@Override
	public CreateContentResponse getCreateContentResponse()
	{
		return new org.iso.mpeg.mxm.createcontentPE.schemahandler.CreateContentResponse(baseProtocolEngine, securityEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSchemaHandler#getAsset()
	 */
	@Override
	public Asset getAsset()
	{
		return new org.iso.mpeg.mxm.createcontentPE.schemahandler.Asset(baseProtocolEngine, ipmpEngine, diEngine, eventReportEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSchemaHandler#getCreateContentSuccess()
	 */
	@Override
	public CreateContentSuccess getCreateContentSuccess()
	{
		return new org.iso.mpeg.mxm.createcontentPE.schemahandler.CreateContentSuccess(baseProtocolEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSchemaHandler#getIpmpToolInfo()
	 */
	@Override
	public IPMPToolInfo getIpmpToolInfo()
	{
		return new org.iso.mpeg.mxm.createcontentPE.schemahandler.IPMPToolInfo(ipmpEngine);
	}

}
