/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createcontentPE.schemahandler;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolSuccess;
import org.iso.mpeg.mxm.baseprotocol.schema.ContentEntityType;
import org.iso.mpeg.mxm.baseprotocol.schema.SuperBaseProtocolAbstractLevel1;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.createcontentPE.schema.createcontentprotocol.CreateContentSuccessType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class CreateContentSuccess extends ProtocolSuccess 
						implements org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSuccess
{	
	private BaseProtocolEngine baseProtocolEngine;
	
	public CreateContentSuccess(BaseProtocolEngine baseProtocolEngine)
	{
		super(new CreateContentSuccessType());
		this.baseProtocolEngine = baseProtocolEngine;
	}

	@Override
	public void createFile(File f) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1((CreateContentSuccessType)protocolSuccessType);
		level1.createFile(f);
	}
	
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1((CreateContentSuccessType)protocolSuccessType);
		return level1.createObject(objectType);
	}
	
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1((CreateContentSuccessType)protocolSuccessType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolSuccessType = 
					(CreateContentSuccessType)(level1.createObject(CreateContentSuccessType.class.getName()).getContent());			
		}
		return response;
	}	
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSuccess#getContentEntity()
	 */
	@Override
	public ContentEntity getContentEntity() throws MXMEngineException
	{
		ContentEntity contentEntity = baseProtocolEngine.getBaseProtocolSchemaHandler().getContentEntity();
		contentEntity.parse(new MXMAdapter(((CreateContentSuccessType)protocolSuccessType).getContentEntity()));
		return contentEntity;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSuccess#setContentEntity(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity)
	 */
	@Override
	public void setContentEntity(ContentEntity contentEntity) throws MXMException
	{
		ContentEntityType contentEntityType = 
				(ContentEntityType)(contentEntity.createObject(ContentEntityType.class.getName()).getContent());
		((CreateContentSuccessType)protocolSuccessType).setContentEntity(contentEntityType);
	}

}
