/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


//usefull import
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import junit.framework.TestCase;

import org.apache.log4j.Logger;

//import org.iso.mpeg.mxm.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMAbstractEngine;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;



import org.iso.mpeg.mxm.engine.digitalitemTE.schema.DIDEngineContext;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;


import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.ApplicationSpecificData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Descriptor;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Statement;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.dii.RelatedIdentifier;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.IPMPGeneralInfoDescriptor;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.LicenseCollection;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.RightsDescriptor;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Mpeg7;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;



public class createContentPETest extends TestCase {

	private final String MPEG7_METADATA_URI = "urn:mpeg:mpeg7:schema:2004";
	
	private Logger logger = Logger.getLogger(createContentPETest.class);
	
	private final String TEMP_FOLDER = System.getProperty("java.io.tmpdir") + File.separator;

	private final String SOURCE_DI_FILE_NAME = "testCreateContentRequest.xml";
	
	private final String OUTPUT_DI_FILE_NAME =TEMP_FOLDER + "mytestCreateContentRequest.xml";
	
	public void testCreateContentParsing() throws MXMConfigurationException, FileNotFoundException, JAXBException, MXMEngineException {
	logger.info("\n\n---------- testCreateContentParsing ----------");
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		DIEngine diEngine = (DIEngine)mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
		EventReportEngine erEngine = (EventReportEngine) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());
		IPMPEngine ipmp = (IPMPEngine) mxm.getDefaultEngine(MXMEngineName.IPMPTE.toString());
		RELEngine relEngine = (RELEngine) mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
		SecurityEngine secEngine = (SecurityEngine) mxm.getDefaultEngine(MXMEngineName.SecurityTE.toString());	
		MetadataEngine meta = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
		CreateContentEngine cc = (CreateContentEngine) mxm.getDefaultEngine(MXMEngineName.CreateContentPE.toString());
	
//		List<MXMEngine> dependencies=new ArrayList();
//		dependencies.add(diEngine);
//		dependencies.add(erEngine);
//		dependencies.add(ipmp);
//		dependencies.add(relEngine);
//		dependencies.add(secEngine);
//		dependencies.add(meta);
//		
//		ipmp.setDependencies(dependencies);
//		diEngine.setDependencies(dependencies);
//		cc.setDependencies(dependencies);
		
		List<MXMEngine> engines=new ArrayList();
		engines.add(relEngine);
		
		ipmp.setDependencies(engines);
		
		CreateContentRequest ccr=cc.getCreateContentSchemaHandler().getCreateContentRequest();
		ccr.parse(new MXMAdapter(new File(getClass().getClassLoader().getResource(SOURCE_DI_FILE_NAME).getFile())));

		try {

			for (Asset a:ccr.getAsset())
			{
				Resource r= (org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource) a.getResource();
				
				logger.info("resource mime:"+r.getMime());
				logger.info("resource ref:"+r.getRef());
				logger.info("resource contencoding:"+r.getContentEncoding());
				logger.info("resource encoding:"+r.getEncoding());
				
				Descriptor d=(org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Descriptor)a.getMetadata();
				Statement s = (Statement) d.getStatementMPEGM();
				
				Mpeg7 m7=meta.getMetadataEngineSchemaHandler().getMpeg7();
				
				if (s.getMetadata()!=null)
				{
					logger.info("there are some metdata for inner item");
					logger.info("WITH NS:"+s.getMetadata().getRef());
					
					if (s.getMetadata().getRef().equals(MPEG7_METADATA_URI))
					{
						Object o= s.getMetadata().getAny();
						m7.parse(new MXMAdapter(o));
						logger.info("time unit is:"+m7.getTimeUnit());
					}
				}
				
				for (RightsDescriptor rd : a.getIPMPGeneralInfoDescriptor().getLicenseCollection().getRightsDescriptor())
				{
					if ((rd.getLicense()!=null)&&(!rd.getLicense().getLicense().isEmpty()))
						for (License l: rd.getLicense().getLicense()) {
							logger.info("LICENSE id is: "+l.getLicenseId());
						}
				}
				
				if (!a.getERRs().isEmpty())
				{
					logger.info("there are some ERRs for inner item");
					
					for (ERR err:a.getERRs()){
						logger.info("id:"+err.getERRIdentifier());
					}
				}
			}
			
			//ADD SIGNATURE
			Signature signa=ccr.getSignature();
			logger.info("signa id is:"+signa.getId());

			//ADD ERR
			ERR err=ccr.getERR();
			logger.info("topl level err id is:"+err.getERRIdentifier());
			
		} catch (MXMException e) 
		{
			e.printStackTrace();
		}	
			
		logger.info("The CreateContent file was successfully parsed");
	}

	public void testCreateContentCreation() throws MXMConfigurationException, MXMEngineException {
		logger.info("\n\n---------- testCreateContentCreation ----------");
	
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
			
		DIEngine diEngine = (DIEngine)mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
		EventReportEngine erEngine = (EventReportEngine) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());
		IPMPEngine ipmp = (IPMPEngine) mxm.getDefaultEngine(MXMEngineName.IPMPTE.toString());
		RELEngine relEngine = (RELEngine) mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
		SecurityEngine secEngine = (SecurityEngine) mxm.getDefaultEngine(MXMEngineName.SecurityTE.toString());	
		MetadataEngine meta = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
		CreateContentEngine cc = (CreateContentEngine) mxm.getDefaultEngine(MXMEngineName.CreateContentPE.toString());
		
//		List<MXMEngine> dependencies=new ArrayList();
//		dependencies.add(diEngine);
//		dependencies.add(erEngine);
//		dependencies.add(ipmp);
//		dependencies.add(relEngine);
//		dependencies.add(secEngine);
//		dependencies.add(meta);
//		diEngine.setDependencies(dependencies);
//		
//		cc.setDependencies(dependencies);
		
		List<MXMEngine> engines=new ArrayList();
		engines.add(relEngine);
		
		ipmp.setDependencies(engines);
				
		//err creation
		ERR err=erEngine.getEventReportEngineSchemaHandler().getERR();
		err.setERRIdentifier("my_identifier");
		
		//signa creation
		Signature signa=secEngine.getSecurityEngineSchemaHandler().getSignature();
		signa.setId("your_id_signa");
		
		//license creation
		IPMPGeneralInfoDescriptor gen_desc=ipmp.getIPMPEngineSchemaHandler().getIPMPGeneralInfoDescriptor();
		LicenseCollection lc=ipmp.getIPMPEngineSchemaHandler().getLicenseCollection();
		RightsDescriptor rd=ipmp.getIPMPEngineSchemaHandler().getRightsDescriptor();
		org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.License ipmp_l =ipmp.getIPMPEngineSchemaHandler().getIPMPLicense();
		License l =relEngine.getRELEngineSchemaHandler().getLicense();
		l.setLicenseId("1234");	
		
		//metadata creation
		Mpeg7 mp7=meta.getMetadataEngineSchemaHandler().getMpeg7();
		mp7.setTimeUnit("12345");
		StructuredData sd=diEngine.getDIEngineSchemaHandler().getStructuredData();
		sd.setRef(MPEG7_METADATA_URI);
		
		//resource creation
		Resource r=diEngine.getDIEngineSchemaHandler().getResource();
		r.setRef("prova");
		
		//asset creation
		Asset a=cc.getCreateContentSchemaHandler().getAsset();
		
		//request creation
		CreateContentRequest ccr=cc.getCreateContentSchemaHandler().getCreateContentRequest();
		
		try {
			
			//set resource
			a.setResource(r);
			
			//set license
			ipmp_l.addLicense(l);
			rd.setLicense(ipmp_l);
			lc.setRightsDescriptor(rd);
			gen_desc.setLicenseCollection(lc);
			a.setIPMPGeneralInfoDescriptor(gen_desc);
			
			//set meta
			Descriptor descriptor=diEngine.getDIEngineSchemaHandler().getDescriptor();
			Statement s=diEngine.getDIEngineSchemaHandler().getStatement();	
			sd.setAny(mp7);
			s.addMetadata(sd);
			descriptor.setStatementMPEGM(s);						
			a.setMetadata(descriptor);
			
			//set err for asset
			a.addERR(err);
			
			//set asset
			ccr.addAsset(a);
			
			//set signa
			ccr.setSignature(signa);

			//set err for top level item
			ccr.setERR(err);
			
			ccr.createFile(new File(OUTPUT_DI_FILE_NAME));
				
			logger.info("The ContentEntity was successfully created in file: " + OUTPUT_DI_FILE_NAME);
					
		} catch (MXMException e) {
			e.printStackTrace();
		}	
	}	
}