/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createlicensePE.schema;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolRequest;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.createlicensePE.schema.createlicenseprotocol.CreateLicenseRequestType;
import org.iso.mpeg.mxm.engine.protocol.createlicensePE.schema.createlicenseprotocol.CustomLicenseType;
import org.iso.mpeg.mxm.engine.protocol.createlicensePE.schema.createlicenseprotocol.TemplateLicenseType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.TemplateLicense;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class CreateLicenseRequest extends ProtocolRequest 
					implements org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest
{	
	private BaseProtocolEngine baseProtocolEngine;
	private RELEngine relEngine;
	private MetadataEngine metadataEngine;
	
	public CreateLicenseRequest(BaseProtocolEngine baseProtocolEngine, RELEngine relEngine, MetadataEngine metadataEngine, 
			SecurityEngine securityEngine)
	{
		super(new CreateLicenseRequestType(), securityEngine);
		this.baseProtocolEngine = baseProtocolEngine;
		this.relEngine = relEngine;
		this.metadataEngine = metadataEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CreateLicenseRequestType)protocolBaseType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CreateLicenseRequestType)protocolBaseType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((CreateLicenseRequestType)protocolBaseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolBaseType = 
					(CreateLicenseRequestType)(level1.createObject(CreateLicenseRequestType.class.getName()).getContent());
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest#getCustomLicense()
	 */
	@Override
	public CustomLicense getCustomLicense() throws MXMEngineException
	{
		CustomLicenseType customLicenseType = ((CreateLicenseRequestType)protocolBaseType).getCustomLicense();
		CustomLicense customLicense = new org.iso.mpeg.mxm.createlicensePE.schema.CustomLicense(relEngine, metadataEngine);
		customLicense.parse(new MXMAdapter(customLicenseType));
		return customLicense;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest#setCustomLicense(org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense)
	 */
	@Override
	public void setCustomLicense(CustomLicense customLicense) throws MXMException
	{
		CustomLicenseType customLicenseType = (CustomLicenseType)(customLicense.createObject(CustomLicenseType.class.getName()).getContent());
		((CreateLicenseRequestType)protocolBaseType).setCustomLicense(customLicenseType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest#getTemplateLicense()
	 */
	@Override
	public TemplateLicense getTemplateLicense() throws MXMEngineException
	{
		TemplateLicenseType templateLicenseType = ((CreateLicenseRequestType)protocolBaseType).getTemplateLicense();
		TemplateLicense templateLicense = new org.iso.mpeg.mxm.createlicensePE.schema.TemplateLicense(baseProtocolEngine, relEngine);
		templateLicense.parse(new MXMAdapter(templateLicenseType));
		return templateLicense;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest#setTemplateLicense(org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.TemplateLicense)
	 */
	@Override
	public void setTemplateLicense(TemplateLicense templateLicense) throws MXMException
	{
		TemplateLicenseType templateLicenseType = (TemplateLicenseType)(templateLicense.createObject(TemplateLicenseType.class.getName()).getContent());
		((CreateLicenseRequestType)protocolBaseType).setTemplateLicense(templateLicenseType);
	}

}
