/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createlicensePE.schema;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolSuccess;
import org.iso.mpeg.mxm.baseprotocol.schema.LicenseEntityType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.createlicensePE.schema.createlicenseprotocol.CreateLicenseSuccessType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class CreateLicenseSuccess extends ProtocolSuccess implements org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseSuccess
{	
	private BaseProtocolEngine baseProtocolEngine;
	
	public CreateLicenseSuccess(BaseProtocolEngine baseProtocolEngine)
	{
		super(new CreateLicenseSuccessType());
		this.baseProtocolEngine = baseProtocolEngine;		
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseSuccess#getLicenseEntity()
	 */
	@Override
	public LicenseEntity getLicenseEntity() throws MXMEngineException
	{
		if (((CreateLicenseSuccessType)protocolSuccessType).getLicenseEntity()!=null)
		{
			LicenseEntityType licenseEntityType = ((CreateLicenseSuccessType)protocolSuccessType).getLicenseEntity();
			LicenseEntity licenseEntity = baseProtocolEngine.getBaseProtocolSchemaHandler().getLicenseEntity();
			licenseEntity.parse(new MXMAdapter(licenseEntityType));
			return licenseEntity;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseSuccess#setLicenseEntity(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.LicenseEntity)
	 */
	@Override
	public void setLicenseEntity(LicenseEntity licenseEntity) throws MXMException
	{
		LicenseEntityType licenseEntityType = (LicenseEntityType)(licenseEntity.createObject(LicenseEntityType.class.getName()).getContent());
		((CreateLicenseSuccessType)protocolSuccessType).setLicenseEntity(licenseEntityType);
	}
	
	@Override
	public void createFile(File f) throws MXMException
	{
		new SuperProtocolEngineAbstractLevel1(((CreateLicenseSuccessType)protocolSuccessType)).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		return new SuperProtocolEngineAbstractLevel1(((CreateLicenseSuccessType)protocolSuccessType)).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 superAbstract = new SuperProtocolEngineAbstractLevel1(((CreateLicenseSuccessType)protocolSuccessType));
		MXMEngineResponse repsonse = superAbstract.parse(object);
		protocolSuccessType = (CreateLicenseSuccessType) 
				superAbstract.createObject(CreateLicenseSuccessType.class.getCanonicalName()).getContent();
		
		return repsonse;
	}

}
