/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createlicensePE.schema;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBElement;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.UniqueIDType;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.createlicensePE.schema.createlicenseprotocol.CustomLicenseType;
import org.iso.mpeg.mxm.engine.protocol.createlicensePE.schema.createlicenseprotocol.RightGroupType;
import org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.RightGroup;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.UniqueID;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.ForAll;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Issuer;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Principal;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class CustomLicense implements org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense
{
	private CustomLicenseType customLicenseType;
	
	private RELEngine relEngine;
	private MetadataEngine metadataEngine;
	
	public CustomLicense(RELEngine relEngine, MetadataEngine metadataEngine)
	{
		this.relEngine = relEngine;
		this.metadataEngine = metadataEngine;
		
		this.customLicenseType = new CustomLicenseType();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(customLicenseType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(customLicenseType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(customLicenseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.value().equals(MXMEngineResponse.SUCCESS.value()))
		{
			customLicenseType = (CustomLicenseType)(level1.createObject(CustomLicenseType.class.getName()).getContent());
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#getLicenseIdentifier()
	 */
	@Override
	public UniqueID getLicenseIdentifier() throws MXMEngineException
	{
		UniqueIDType uniqueIDType = customLicenseType.getLicenseIdentifier();
		UniqueID uniqueID = metadataEngine.getMetadataEngineSchemaHandler().getUniqueID();
		uniqueID.parse(new MXMAdapter(uniqueIDType));
		return uniqueID;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#setResource(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void setResource(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource resource) throws MXMException
	{
		JAXBElement<? extends Resource> resourceElement = (JAXBElement<? extends Resource>)
							(resource.createObject(JAXBElement.class.getCanonicalName()).getContent());
		customLicenseType.setResource(resourceElement);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#getIssuer()
	 */
	@Override
	public Issuer getIssuer() throws MXMEngineException
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.Issuer issuerType = customLicenseType.getIssuer();
		
		if (issuerType!=null)
		{
			Issuer issuer = relEngine.getRELEngineSchemaHandler().getIssuer();
			issuer.parse(new MXMAdapter(issuerType));
			return issuer;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#getForAll()
	 */
	@Override
	public ForAll getForAll() throws MXMEngineException
	{
		if (customLicenseType.getForAll()!=null)
		{
			org.iso.mpeg.mxm.engine.relTE.schema.relr.ForAll forAllType = customLicenseType.getForAll();
			ForAll forAll = relEngine.getRELEngineSchemaHandler().getForAll();
			forAll.parse(new MXMAdapter(forAllType));
			return forAll;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#getPrincipal()
	 */
	@Override
	public Principal getPrincipal() throws MXMEngineException
	{
		if (customLicenseType.getPrincipal()!=null)
		{
			org.iso.mpeg.mxm.engine.relTE.schema.relr.Principal principalType = customLicenseType.getPrincipal();
			
			Principal principal = relEngine.getRELEngineSchemaHandler().getPrincipal();
			principal.parse(new MXMAdapter(principalType));
			return principal;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#setPrincipal(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Principal)
	 */
	@Override
	public void setPrincipal(Principal principal) throws MXMException
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.Principal principalType = (org.iso.mpeg.mxm.engine.relTE.schema.relr.Principal)
				(principal.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.Principal.class.getName()).getContent());
		customLicenseType.setPrincipal(principalType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#setIssuer(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Issuer)
	 */
	@Override
	public void setIssuer(Issuer issuer) throws MXMException
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.Issuer issuerType = (org.iso.mpeg.mxm.engine.relTE.schema.relr.Issuer)
				(issuer.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.Issuer.class.getName()).getContent());
		customLicenseType.setIssuer(issuerType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#setLicenseIdentifier(org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.UniqueID)
	 */
	@Override
	public void setLicenseIdentifier(UniqueID uniqueID) throws MXMException
	{
		UniqueIDType uniqueIDType = (UniqueIDType)(uniqueID.createObject(UniqueIDType.class.getName()).getContent());
		customLicenseType.setLicenseIdentifier(uniqueIDType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#setForAll(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.ForAll)
	 */
	@Override
	public void setForAll(ForAll forAll) throws MXMException
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.ForAll forAllType = (org.iso.mpeg.mxm.engine.relTE.schema.relr.ForAll)
				(forAll.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.ForAll.class.getName()).getContent());
		customLicenseType.setForAll(forAllType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#getRightGroup()
	 */
	@Override
	public List<RightGroup> getRightGroup() throws MXMEngineException
	{
		List<RightGroupType> rightGroupTypes = customLicenseType.getRightGroup();
		if (rightGroupTypes != null)
		{
			List<RightGroup> rightGroups = new ArrayList<RightGroup>();
			for (RightGroupType rightGroupType : rightGroupTypes)
			{
				RightGroup rightGroup = new org.iso.mpeg.mxm.createlicensePE.schema.RightGroup(relEngine);
				rightGroup.parse(new MXMAdapter(rightGroupType));
				rightGroups.add(rightGroup);
			}
			
			return rightGroups;
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#getResource()
	 */
	@Override
	public org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource getResource() throws MXMEngineException
	{
		JAXBElement<? extends Resource> resourceElement = customLicenseType.getResource();
		org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource resource = relEngine.getRELEngineSchemaHandler().getResource();
		resource.parse(new MXMAdapter(resourceElement));
		return resource;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense#addRightGroup(org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.RightGroup)
	 */
	@Override
	public void addRightGroup(RightGroup rightGroup) throws MXMException
	{
		RightGroupType rightGroupType = (RightGroupType)(rightGroup.createObject(RightGroupType.class.getName()).getContent());
		customLicenseType.getRightGroup().add(rightGroupType);
	}

}
