/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createlicensePE.schema;

import java.io.File;

import javax.xml.bind.JAXBElement;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.createlicensePE.schema.createlicenseprotocol.RightGroupType;
import org.iso.mpeg.mxm.engine.relTE.schema.SuperRelAbstractLevel1;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Condition;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Right;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.RELRight;


/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class RightGroup implements org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.RightGroup
{
	private RightGroupType rightGroupType;
	
	private RELEngine relEngine;
	
	public RightGroup(RELEngine relEngine)
	{
		this.relEngine = relEngine;
		
		this.rightGroupType = new RightGroupType();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(rightGroupType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(rightGroupType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1(rightGroupType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			rightGroupType = (RightGroupType)(level1.createObject(RightGroupType.class.getName()).getContent());
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.RightGroup#getCondition()
	 */
	@Override
	public Condition getCondition() throws MXMEngineException
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition conditionType = rightGroupType.getCondition();
		
		if (conditionType!=null)
		{
			Condition condition = relEngine.getRELEngineSchemaHandler().getCondition();
			condition.parse(new MXMAdapter(conditionType));
			return condition;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.RightGroup#setCondition(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Condition)
	 */
	@Override
	public void setCondition(Condition condition) throws MXMException
	{
		if (condition!=null)
		{
			org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition conditionType = (org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition)
					(condition.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition.class.getCanonicalName()).getContent());
			rightGroupType.setCondition(conditionType);
		}
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.RightGroup#getRight()
	 */
	@Override
	public Right getRight() throws MXMEngineException
	{
		JAXBElement rightElement = rightGroupType.getRight();
		Right right = relEngine.getRELEngineSchemaHandler().getRight();
		right.parse(new MXMAdapter(rightElement));
		return right;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.RightGroup#setRight(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Right)
	 */
	@Override
	public void setRight(Right right) throws MXMException
	{
		JAXBElement rightElement = (JAXBElement)(right.createObject(JAXBElement.class.getCanonicalName()).getContent());
		rightGroupType.setRight(rightElement);
	}

	@Override
	public void setRELRight(RELRight right)
	{
		try
		{
			if (right.equals(RELRight.adapt))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relmx.Adapt());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.aggregate))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm3x.Aggregate());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
	//		else if (right.equals(RELRight.delete))
	//			return RELRight.delete;
			else if (right.equals(RELRight.delist))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm1x.Delist());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.diminish))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relmx.Diminish());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.embed))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm3x.Embed());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.enhance))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relmx.Enhance());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
	//		else if (right.equals(RELRight.Enlarge))
	//			return RELRight.enlarge;
			else if (right.equals(RELRight.enlist))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm1x.Enlist());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.execute))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relmx.Execute());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.export))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm2x.Export());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.extendRights))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm2x.ExtendRights());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.governedAdapt))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm3x.GovernedAdapt());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.governedAggregate))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm3x.GovernedAggregate());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.governedCopy))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm1x.GovernedCopy());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.governedEmbed))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm3x.GovernedEmbed());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.governedMove))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relm1x.GovernedMove());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
	//		else if (right.equals(RELRight.install))
	//			return RELRight.install;
			else if (right.equals(RELRight.issue))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relr.Issue());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.modify))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relmx.Modify());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
	//		else if (right.equals(RELRight.move))
	//			return RELRight.move;
			else if (right.equals(RELRight.obtain))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relr.Obtain());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.play))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relmx.Play());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.possessProperty))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relr.PossessProperty());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
			else if (right.equals(RELRight.print))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relmx.Print());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
	//		else if (right.equals(RELRight.reduce))
	//			return RELRight.reduce;
			else if (right.equals(RELRight.revoke))
			{
				SuperRelAbstractLevel1 abstractLevel1 = new SuperRelAbstractLevel1(new org.iso.mpeg.mxm.engine.relTE.schema.relr.Revoke());
				rightGroupType.setRight((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Right>)
						abstractLevel1.createObject(JAXBElement.class.getCanonicalName()).getContent());
			}
	//		else if (right.equals(RELRight.uninstall))
	//			return RELRight.uninstall;
	//		else
	//			throw new RELEException("Unsupported right found.");
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}

	@Override
	public RELRight getRELRight()
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.Right right = rightGroupType.getRight().getValue();
		
		if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relmx.Adapt)
			return RELRight.adapt;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm3x.Aggregate)
			return RELRight.aggregate;
//		else if (right.getClass().getCanonicalName().equalsIgnoreCase("eu.convergence.middleware.relTE.schema.rel_mx.Delete"))
//			return RELRight.delete;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm1x.Delist)
			return RELRight.delist;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relmx.Diminish)
			return RELRight.diminish;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm3x.Embed)
			return RELRight.embed;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relmx.Enhance)
			return RELRight.enhance;
//		else if (right.getClass().getCanonicalName().equalsIgnoreCase("eu.convergence.middleware.relTE.schema.rel_mx.Enlarge"))
//			return RELRight.enlarge;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm1x.Enlist)
			return RELRight.enlist;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relmx.Execute)
			return RELRight.execute;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm2x.Export)
			return RELRight.export;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm2x.ExtendRights)
			return RELRight.extendRights;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm3x.GovernedAdapt)
			return RELRight.governedAdapt;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm3x.GovernedAggregate)
			return RELRight.governedAggregate;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm1x.GovernedCopy)
			return RELRight.governedCopy;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm3x.GovernedEmbed)
			return RELRight.governedEmbed;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm1x.GovernedMove)
			return RELRight.governedMove;
//		else if (right.getClass().getCanonicalName().equalsIgnoreCase("eu.convergence.middleware.relTE.schema.rel_mx.Install"))
//			return RELRight.install;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.Issue)
			return RELRight.issue;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relmx.Modify)
			return RELRight.modify;
//		else if (right.getClass().getCanonicalName().equalsIgnoreCase("eu.convergence.middleware.relTE.schema.rel_mx.Move"))
//			return RELRight.move;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.Obtain)
			return RELRight.obtain;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relmx.Play)
			return RELRight.play;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.PossessProperty)
			return RELRight.possessProperty;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relmx.Print)
			return RELRight.print;
//		else if (right.getClass().getCanonicalName().equalsIgnoreCase("eu.convergence.middleware.relTE.schema.rel_mx.Reduce"))
//			return RELRight.reduce;
		else if (right instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.Revoke)
			return RELRight.revoke;
//		else if (right.getClass().getCanonicalName().equalsIgnoreCase("eu.convergence.middleware.relTE.schema.rel_mx.Uninstall"))
//			return RELRight.uninstall;
//		else
//			throw new RELEException("Unsupported right found.");
		
		return null;
	}

}
