/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.createlicensePE.technology.remote;

import java.net.URL;

import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.ServiceFactory;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.es.createlicenseES.CreateLicenseProtocolRemote;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.createlicensePE.exception.CreateLicenseEngineException;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseResponse;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class CreateLicenseProtocol implements org.iso.mpeg.mxm.pEngine.createlicensePE.technologyhandler.CreateLicenseProtocol
{
	private URL serviceURL;
	private QName serviceName;
	
	private Logger logger;
	
	private BaseProtocolEngine baseProtocolEngine;
	private RELEngine relEngine;
	private SecurityEngine securityEngine;
	
	public CreateLicenseProtocol(BaseProtocolEngine baseProtocolEngine, RELEngine relEngine, SecurityEngine securityEngine)
	{
		this.baseProtocolEngine = baseProtocolEngine;
		this.relEngine = relEngine;
		this.securityEngine = securityEngine;
		
		logger = LoggerFactory.getLogger(this.getClass());
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.technologyhandler.CreateLicenseProtocol#initServiceParams(java.net.URL, javax.xml.namespace.QName)
	 */
	@Override
	public MXMEngineResponse initServiceParams(URL serviceURL, QName serviceName)
	{
		this.serviceName = serviceName;
		this.serviceURL = serviceURL;
		return MXMEngineResponse.SUCCESS;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.createlicensePE.technologyhandler.CreateLicenseProtocol#createLicense(org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest)
	 */
	@Override
	public CreateLicenseResponse createLicense(CreateLicenseRequest request) throws CreateLicenseEngineException
	{
		if (serviceURL == null)
		{
			logger.warn("Service URL has not been set");
		}
		if (serviceName == null)
		{
			logger.warn("Service Name has not been set");
		}
		ServiceFactory serviceFactory = null;
		
		try
		{
			serviceFactory = ServiceFactory.newInstance();
			Service service = serviceFactory.createService(serviceURL, serviceName);
			
			String serviceRequest = (String)(request.createObject(String.class.getCanonicalName()).getContent());
			CreateLicenseProtocolRemote createLicenseProtocolRemote = (CreateLicenseProtocolRemote)
					service.getPort(CreateLicenseProtocolRemote.class);
			String serviceResponse = createLicenseProtocolRemote.createLicense(serviceRequest);
			CreateLicenseResponse createLicenseResponse = 
					new org.iso.mpeg.mxm.createlicensePE.schema.CreateLicenseResponse(baseProtocolEngine, relEngine, securityEngine);
			createLicenseResponse.parse(new MXMAdapter(serviceResponse));
			
			return createLicenseResponse;
		} catch (ServiceException e)
		{
			throw new CreateLicenseEngineException(e.getMessage());
		} catch (MXMException e)
		{
			throw new CreateLicenseEngineException(e.getMessage());
		}
	}

}
