/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.describecontentPE.schema;

import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityRequest;
import org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityResponse;
import org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntitySuccess;
import org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.SetDescribeEntityRequest;
import org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.SetDescribeEntityResponse;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;


/**
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 *
 */
public class DescribeContentSchemaHandler 
				extends org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.DescribeContentSchemaHandler
{
	private BaseProtocolEngine baseProtocolEngine;
	private MetadataEngine metadataEngine;
	private SecurityEngine securityEngine;
	
	public DescribeContentSchemaHandler(BaseProtocolEngine baseProtocolEngine, MetadataEngine metadataEngine,
			SecurityEngine securityEngine)
	{
		this.baseProtocolEngine = baseProtocolEngine;
		this.metadataEngine = metadataEngine;
		this.securityEngine = securityEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.DescribeEntitySchemaHandler#getGetDescribeEntityRequest()
	 */
	@Override
	public GetDescribeEntityRequest getGetDescribeEntityRequest()
	{
		return new GetDescribeContentRequest(baseProtocolEngine, metadataEngine, securityEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.DescribeEntitySchemaHandler#getGetDescribeEntityResponse()
	 */
	@Override
	public GetDescribeEntityResponse getGetDescribeEntityResponse()
	{
		return new GetDescribeContentResponse(baseProtocolEngine, metadataEngine, securityEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.DescribeEntitySchemaHandler#getGetDescribeEntitySuccess()
	 */
	@Override
	public GetDescribeEntitySuccess getGetDescribeEntitySuccess()
	{
		return new org.iso.mpeg.mxm.describeentityPE.schema.GetDescribeEntitySuccess(metadataEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.DescribeEntitySchemaHandler#getSetDescribeEntityRequest()
	 */
	@Override
	public SetDescribeEntityRequest getSetDescribeEntityRequest()
	{
		return new SetDescribeContentRequest(metadataEngine, securityEngine);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.DescribeEntitySchemaHandler#getSetDescribeEntityResponse()
	 */
	@Override
	public SetDescribeEntityResponse getSetDescribeEntityResponse()
	{
		return new SetDescribeContentResponse(baseProtocolEngine, securityEngine);
	}

	@Override
	public org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.GetDescribeContentRequest getGetDescribeContentRequest()
	{
		return new GetDescribeContentRequest(baseProtocolEngine, metadataEngine, securityEngine);
	}

	@Override
	public org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.GetDescribeContentResponse getGetDescribeContentResponse()
	{
		return new GetDescribeContentResponse(baseProtocolEngine, metadataEngine, securityEngine);
	}

	@Override
	public org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.SetDescribeContentRequest getSetDescribeContentRequest()
	{
		return new SetDescribeContentRequest(metadataEngine, securityEngine);
	}

	@Override
	public org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.SetDescribeContentResponse getSetDescribeContentResponse()
	{
		return new SetDescribeContentResponse(baseProtocolEngine, securityEngine);
	}

}
