package org.iso.mpeg.mxm.engine.describecontentPE.test;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.pEngine.describecontentPE.DescribeContentEngine;
import org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.GetDescribeContentRequest;
import org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.GetDescribeContentResponse;
import org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.SetDescribeContentRequest;
import org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.SetDescribeContentResponse;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.fru.FRU;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.fru.Src;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.AccessUnit;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.FragmentUpdateContext;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.FragmentUpdatePayload;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.FragmentUpdateUnit;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class DescribeContentLocalTest extends TestCase
{
	private static final String contentId = "TEST_ENTITY";
	
	private Logger logger = Logger.getLogger(DescribeContentLocalTest.class);
	private DescribeContentEngine dce;
	private MetadataEngine metadataEngine;
	
	public DescribeContentLocalTest() throws Exception
	{
		logger.info("\n\n---------- Test Describe Content Orchestrators and Protocols ----------");
		
		File mxmConf = new File(getClass().getClassLoader().getResource("DescribeContentMxmConfiguration.xml").getFile());
		
		MXM mxm = MXM.getInstance(mxmConf);
		mxm.listEnginesAndProperties();
		
		metadataEngine = (MetadataEngine) mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
		
//		dce = dco.getDescribeContentEngine(); // XXX THIS DOESN'T WORK. DON'T USE THIS
		dce = (DescribeContentEngine) mxm.getDefaultEngine(MXMEngineName.DescribeContentPE.toString());
	}
	
	public void setUp() throws Exception
	{
		metadataEngine.getMetadataEngineTechnologyHandler().getMetadataRepository().initRepository(true);
	}
	
	public void testDescribeContent() throws Exception
	{
		// get Description
		
		step1_GET();
		
		// addNode / -> <a><h2>Awesome</h2><h2>Oh Yeah!</h2></a>
		
		step2_ADD();
		
		// addNode /a -> <h2>Bitchin</h2>
		
		step3_ADD();
		
		// replaceNode /a*[2] = <h2>Oh Yeah!</h2> -> <h2>Awesome 2</h2>
		
		step4_REPLACE();
		
		// deleteNode /a*[2]
		
		step5_DELETE();
		
		// get Description
		
		step6_GET();
		
		// reset Description
		
		step7_RESET();
		
		// get Description
		
		step8_GET();
	}
	
	public void step1_GET() throws Exception
	{
		Src src = metadataEngine.getMetadataEngineSchemaHandler().getSrc();
		src.setValue("//h2/text()");
		
		FRU fru = metadataEngine.getMetadataEngineSchemaHandler().getFRU();
		fru.setSrc(src);
		
		GetDescribeContentRequest gdcReq = dce.getDescribeContentSchemaHandler().getGetDescribeContentRequest();
		gdcReq.setEntityIdentifier(contentId);
		gdcReq.setFRU(fru);
		
		GetDescribeContentResponse gdcResp = dce.getDescribeContentTechnologyHandler().getLocalGetDescribeContentProtocol().getDescribeContent(gdcReq);
		System.out.println("GET SUCCESS : " + (gdcResp.getGetDescribeEntitySuccess()!=null));
	}
	
	public void step2_ADD() throws Exception
	{
		SetDescribeContentRequest sdcReq = dce.getDescribeContentSchemaHandler().getSetDescribeContentRequest();
		
		DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		
		Document bufferDoc = docBuilder.parse(new ByteArrayInputStream("<a><h2>Awesome</h2><h2>Oh Yeah!</h2></a>".getBytes()));
		
		Element xmlfragment = bufferDoc.getDocumentElement();
				
		FragmentUpdatePayload fup = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdatePayload();
		fup.setAny(xmlfragment);
		
		FragmentUpdateContext fuc = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateContext();
		fuc.setValue("/");
		
		FragmentUpdateUnit fuu = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateUnit();
		fuu.setFUCommand("addNode");
		fuu.setFUPayload(fup);
		fuu.setFUContext(fuc);
		
		AccessUnit accessUnit = metadataEngine.getMetadataEngineSchemaHandler().getAccessUnit();
		accessUnit.addFragmentUpdateUnit(fuu);
		
		sdcReq.setEntityIdentifier(contentId);
		sdcReq.setAccessUnit(accessUnit);
		
		SetDescribeContentResponse sdcResp = dce.getDescribeContentTechnologyHandler().getLocalSetDescribeContentProtocol().setDescribeContent(sdcReq);
		System.out.println("ADD SUCCESS : " + (sdcResp.getSetDescribeEntitySuccess()!=null));
	}
	
	public void step3_ADD() throws Exception
	{
		SetDescribeContentRequest sdcReq = dce.getDescribeContentSchemaHandler().getSetDescribeContentRequest();
		
		DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		
		Document bufferDoc = docBuilder.parse(new ByteArrayInputStream("<h2>Bitchin</h2>".getBytes()));
		
		Element xmlfragment = bufferDoc.getDocumentElement();
				
		FragmentUpdatePayload fup = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdatePayload();
		fup.setAny(xmlfragment);
		
		FragmentUpdateContext fuc = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateContext();
		fuc.setValue("/a");
		
		FragmentUpdateUnit fuu = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateUnit();
		fuu.setFUCommand("addNode");
		fuu.setFUPayload(fup);
		fuu.setFUContext(fuc);
		
		AccessUnit accessUnit = metadataEngine.getMetadataEngineSchemaHandler().getAccessUnit();
		accessUnit.addFragmentUpdateUnit(fuu);
		
		sdcReq.setEntityIdentifier(contentId);
		sdcReq.setAccessUnit(accessUnit);
		
		SetDescribeContentResponse sdcResp = dce.getDescribeContentTechnologyHandler().getLocalSetDescribeContentProtocol().setDescribeContent(sdcReq);
		System.out.println("ADD SUCCESS : " + (sdcResp.getSetDescribeEntitySuccess()!=null));
	}
	
	public void step4_REPLACE() throws Exception
	{
		SetDescribeContentRequest sdcReq = dce.getDescribeContentSchemaHandler().getSetDescribeContentRequest();
		
		DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		
		Document bufferDoc = docBuilder.parse(new ByteArrayInputStream("<h2>Awesome 2</h2>".getBytes()));
		
		Element xmlfragment = bufferDoc.getDocumentElement();
				
		FragmentUpdatePayload fup = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdatePayload();
		fup.setAny(xmlfragment);
		
		FragmentUpdateContext fuc = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateContext();
		fuc.setValue("/a/*[last()]");
		
		FragmentUpdateUnit fuu = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateUnit();
		fuu.setFUCommand("replaceNode");
		fuu.setFUPayload(fup);
		fuu.setFUContext(fuc);
		
		AccessUnit accessUnit = metadataEngine.getMetadataEngineSchemaHandler().getAccessUnit();
		accessUnit.addFragmentUpdateUnit(fuu);
		
		sdcReq.setEntityIdentifier(contentId);
		sdcReq.setAccessUnit(accessUnit);
		
		SetDescribeContentResponse sdcResp = dce.getDescribeContentTechnologyHandler().getLocalSetDescribeContentProtocol().setDescribeContent(sdcReq);
		System.out.println("REPLACE SUCCESS : " + (sdcResp.getSetDescribeEntitySuccess()!=null));
	}
	
	public void step5_DELETE() throws Exception
	{
		SetDescribeContentRequest sdcReq = dce.getDescribeContentSchemaHandler().getSetDescribeContentRequest();
		
		FragmentUpdateContext fuc = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateContext();
		fuc.setValue("/a/*[2]");
		
		FragmentUpdateUnit fuu = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateUnit();
		fuu.setFUCommand("deleteNode");
		fuu.setFUContext(fuc);
		
		AccessUnit accessUnit = metadataEngine.getMetadataEngineSchemaHandler().getAccessUnit();
		accessUnit.addFragmentUpdateUnit(fuu);
		
		sdcReq.setEntityIdentifier(contentId);
		sdcReq.setAccessUnit(accessUnit);
		
		SetDescribeContentResponse sdcResp = dce.getDescribeContentTechnologyHandler().getLocalSetDescribeContentProtocol().setDescribeContent(sdcReq);
		System.out.println("DELETE SUCCESS : " + (sdcResp.getSetDescribeEntitySuccess()!=null));
	}
	
	public void step6_GET() throws Exception
	{
		Src src = metadataEngine.getMetadataEngineSchemaHandler().getSrc();
		src.setValue("/");
		
		FRU fru = metadataEngine.getMetadataEngineSchemaHandler().getFRU();
		fru.setSrc(src);
		
		GetDescribeContentRequest gdcReq = dce.getDescribeContentSchemaHandler().getGetDescribeContentRequest();
		gdcReq.setEntityIdentifier(contentId);
		gdcReq.setFRU(fru);
		
		GetDescribeContentResponse gdcResp = dce.getDescribeContentTechnologyHandler().getLocalGetDescribeContentProtocol().getDescribeContent(gdcReq);
		
		String position = gdcResp.getGetDescribeEntitySuccess().getAccessUnit().getFragmentUpdateUnitList().get(0).getFUContext().getValue();
		Element result = gdcResp.getGetDescribeEntitySuccess().getAccessUnit().getFragmentUpdateUnitList().get(0).getFUPayload().getAny();
		
		DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		Document bufferDoc = docBuilder.newDocument();
		bufferDoc.adoptNode(result);
		bufferDoc.appendChild(result);
		
		StringWriter out = new StringWriter();
		OutputFormat ouputFormat = new OutputFormat(bufferDoc);
		ouputFormat.setOmitXMLDeclaration(true);
		XMLSerializer serializer = new XMLSerializer(out, ouputFormat);
		serializer.serialize(bufferDoc);
		
		String description = out.toString();
		
		System.out.println("GET SUCCESS : " + (description.contains("<h2>Awesome</h2>") && description.contains("<h2>Awesome 2</h2>") && position.equals("/")));
	}
	
	public void step7_RESET() throws Exception
	{
		SetDescribeContentRequest sdcReq = dce.getDescribeContentSchemaHandler().getSetDescribeContentRequest();
		
		FragmentUpdateUnit fuu = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateUnit();
		fuu.setFUCommand("reset");
		
		AccessUnit accessUnit = metadataEngine.getMetadataEngineSchemaHandler().getAccessUnit();
		accessUnit.addFragmentUpdateUnit(fuu);
		
		sdcReq.setEntityIdentifier(contentId);
		sdcReq.setAccessUnit(accessUnit);
		
		SetDescribeContentResponse sdcResp = dce.getDescribeContentTechnologyHandler().getLocalSetDescribeContentProtocol().setDescribeContent(sdcReq);
		System.out.println("RESET SUCCESS : " + (sdcResp.getSetDescribeEntitySuccess()!=null));
	}
	
	public void step8_GET() throws Exception
	{
		Src src = metadataEngine.getMetadataEngineSchemaHandler().getSrc();
		src.setValue("/");
		
		FRU fru = metadataEngine.getMetadataEngineSchemaHandler().getFRU();
		fru.setSrc(src);
		
		GetDescribeContentRequest gdcReq = dce.getDescribeContentSchemaHandler().getGetDescribeContentRequest();
		gdcReq.setEntityIdentifier(contentId);
		gdcReq.setFRU(fru);
		
		GetDescribeContentResponse gdcResp = dce.getDescribeContentTechnologyHandler().getLocalGetDescribeContentProtocol().getDescribeContent(gdcReq);
		System.out.println("GET SUCCESS : " + (gdcResp.getGetDescribeEntitySuccess()!=null));
	}
	
	public void tearDown() throws Exception
	{
		metadataEngine.getMetadataEngineTechnologyHandler().getMetadataRepository().initRepository(true);
	}
	
}
