/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.engine.describecontentPE.test;


import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.namespace.QName;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMInvalidConfigurationException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.pEngine.describecontentPE.DescribeContentEngine;
import org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.GetDescribeContentRequest;
import org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.GetDescribeContentResponse;
import org.iso.mpeg.mxm.pEngine.describecontentPE.technologyhandler.GetDescribeContentProtocol;
import org.junit.Test;

import junit.framework.TestCase;

/**
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 *
 */
public class DescribeContentRemoteTest extends TestCase
{

	@Test
	public void test()
	{
		MXM mxm;
		try
		{
			mxm = MXM.getInstance(this);
			mxm.listEnginesAndProperties();
			
			DescribeContentEngine describeContentEngine = 
					(DescribeContentEngine)mxm.getDefaultEngine(MXMEngineName.DescribeContentPE.value());
			
			GetDescribeContentRequest getDescribeContentRequest = (GetDescribeContentRequest)
					describeContentEngine.getDescribeContentSchemaHandler().getGetDescribeEntityRequest();
			
			getDescribeContentRequest.setTransactionIdentifier("DescribeContentTID");
			
			GetDescribeContentProtocol getDescribeContentProtocol = describeContentEngine
					.getDescribeContentTechnologyHandler().getRemoteGetDescribeContentProtocol();
			
			URL serviceURL = new URL(describeContentEngine.getPropertyValue("SERVICE_URL"));
			QName serviceName = new QName(describeContentEngine.getPropertyValue("SERVICE_NAME_NS"), 
					describeContentEngine.getPropertyValue("SERVICE_NAME"));
			
			getDescribeContentProtocol.initServiceParams(serviceURL, serviceName);
			
			GetDescribeContentResponse getDescribeContentResponse = 
					getDescribeContentProtocol.getDescribeContent(getDescribeContentRequest);
			
			
			if (getDescribeContentResponse!=null)
			{
				System.out.println("Sent TID: " + getDescribeContentRequest.getTransactionIdentifier());
				System.out.println("Received TID: " + getDescribeContentResponse.getTransactionIdentifier());
			}
			else
			{
				System.out.println("Service not available");
			}
			
		} catch (MXMConfigurationException e)
		{
			e.printStackTrace();
		} catch (MXMEngineException e)
		{
			e.printStackTrace();
		} catch (MXMInvalidConfigurationException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MalformedURLException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
