/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.describeentityPE.schema;

import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.basePE.ProtocolRequest;
import org.iso.mpeg.mxm.baseprotocol.schema.SupportedMetadataSchemaType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.describeentityPE.schema.describeentityprotocol.GetDescribeEntityRequestType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.SupportedMetadataSchema;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.fru.FRU;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public abstract class GetDescribeEntityRequest extends ProtocolRequest
						implements org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityRequest
{	
	private BaseProtocolEngine baseProtocolEngine;
	private MetadataEngine metadataEngine;
	
	protected GetDescribeEntityRequest(GetDescribeEntityRequestType getDescribeEntityRequestType, 
			BaseProtocolEngine baseProtocolEngine, MetadataEngine metadataEngine, SecurityEngine securityEngine)
	{
		super(getDescribeEntityRequestType, securityEngine);
		
		this.baseProtocolEngine = baseProtocolEngine;
		this.metadataEngine = metadataEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityRequest#getEntityIdentifier()
	 */
	@Override
	public String getEntityIdentifier()
	{
		return ((GetDescribeEntityRequestType)protocolBaseType).getEntityIdentifier();
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityRequest#setEntityIdentifier(java.lang.String)
	 */
	@Override
	public void setEntityIdentifier(String string)
	{
		((GetDescribeEntityRequestType)protocolBaseType).setEntityIdentifier(string);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityRequest#getSupportedMetadataSchema()
	 */
	@Override
	public List<SupportedMetadataSchema> getSupportedMetadataSchema() throws MXMEngineException
	{
		List<SupportedMetadataSchemaType> supportedMetadataSchemaTypes = 
				((GetDescribeEntityRequestType)protocolBaseType).getSupportedMetadataSchema();
		List<SupportedMetadataSchema> supportedMetadataSchemas = new ArrayList<SupportedMetadataSchema>();
		
		if (supportedMetadataSchemaTypes != null)
		{
			for (SupportedMetadataSchemaType supportedMetadataSchemaType : supportedMetadataSchemaTypes)
			{
				SupportedMetadataSchema supportedMetadataSchema = 
						baseProtocolEngine.getBaseProtocolSchemaHandler().getSupportedMetadataSchema();
				supportedMetadataSchema.parse(new MXMAdapter(supportedMetadataSchemaType));
				supportedMetadataSchemas.add(supportedMetadataSchema);
			}
		}
		
		return supportedMetadataSchemas;
	}
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityRequest#addSupportedMetadataSchema(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.SupportedMetadataSchema)
	 */
	@Override
	public void addSupportedMetadataSchema(SupportedMetadataSchema supportedMetadataSchema) throws MXMException
	{
		SupportedMetadataSchemaType supportedMetadataSchemaType = 
				(SupportedMetadataSchemaType)(supportedMetadataSchema.createObject(SupportedMetadataSchemaType.class.getName()).getContent());
		((GetDescribeEntityRequestType)protocolBaseType).getSupportedMetadataSchema().add(supportedMetadataSchemaType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityRequest#getFRU()
	 */
	@Override
	public FRU getFRU() throws MXMEngineException
	{
		org.iso.mpeg.mxm.engine.metadataTE.schema.fru.FRU fruType = ((GetDescribeEntityRequestType)protocolBaseType).getFRU();
		FRU fru = metadataEngine.getMetadataEngineSchemaHandler().getFRU();
		fru.parse(new MXMAdapter(fruType));
		return fru;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityRequest#setFRU(org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.fru.FRU)
	 */
	@Override
	public void setFRU(FRU fRU) throws MXMException
	{
		org.iso.mpeg.mxm.engine.metadataTE.schema.fru.FRU fruType = 
				(org.iso.mpeg.mxm.engine.metadataTE.schema.fru.FRU)
					(fRU.createObject(org.iso.mpeg.mxm.engine.metadataTE.schema.fru.FRU.class.getName()).getContent());
		((GetDescribeEntityRequestType)protocolBaseType).setFRU(fruType);
	}

}
