/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.describeentityPE.schema;

import org.iso.mpeg.mxm.basePE.ProtocolResponse;
import org.iso.mpeg.mxm.baseprotocol.schema.ProtocolFailureType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.describeentityPE.schema.describeentityprotocol.GetDescribeEntityResponseType;
import org.iso.mpeg.mxm.engine.protocol.describeentityPE.schema.describeentityprotocol.GetDescribeEntitySuccessType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntitySuccess;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public abstract class GetDescribeEntityResponse extends ProtocolResponse
						implements org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityResponse
{
	private BaseProtocolEngine baseProtocolEngine;
	private MetadataEngine metadataEngine;
	
	public GetDescribeEntityResponse(GetDescribeEntityResponseType getDescribeEntityResponseType, 
									BaseProtocolEngine baseProtocolEngine, MetadataEngine metadataEngine, 
									SecurityEngine securityEngine)
	{
		super(getDescribeEntityResponseType, securityEngine);
		this.baseProtocolEngine = baseProtocolEngine;
		this.metadataEngine = metadataEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityResponse#getGetDescribeEntitySuccess()
	 */
	@Override
	public GetDescribeEntitySuccess getGetDescribeEntitySuccess()
	{
		GetDescribeEntitySuccess getDescribeEntitySuccess = 
				new org.iso.mpeg.mxm.describeentityPE.schema.GetDescribeEntitySuccess(metadataEngine,((GetDescribeEntityResponseType)protocolBaseType).getGetDescribeEntitySuccess());
		return getDescribeEntitySuccess;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityResponse#setGetDescribeEntitySuccess(org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntitySuccess)
	 */
	@Override
	public void setGetDescribeEntitySuccess(GetDescribeEntitySuccess getDescribeEntitySuccess) throws MXMException
	{
		GetDescribeEntitySuccessType getDescribeEntitySuccessType = 
				(GetDescribeEntitySuccessType)
					(getDescribeEntitySuccess.createObject(GetDescribeEntitySuccessType.class.getName()).getContent());
		((GetDescribeEntityResponseType)protocolBaseType).setGetDescribeEntitySuccess(getDescribeEntitySuccessType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityResponse#getGetDescribeEntityFailure()
	 */
	@Override
	public ProtocolFailure getGetDescribeEntityFailure() throws MXMEngineException
	{
		ProtocolFailure protocolFailure = baseProtocolEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
		protocolFailure.parse(new MXMAdapter(((GetDescribeEntityResponseType)protocolBaseType).getGetDescribeEntityFailure()));
		return protocolFailure;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntityResponse#setGetDescribeEntityFailure(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure)
	 */
	@Override
	public void setGetDescribeEntityFailure(ProtocolFailure protocolFailure) throws MXMException
	{
		ProtocolFailureType protocolFailureType = 
				(ProtocolFailureType)(protocolFailure.createObject(ProtocolFailureType.class.getName()).getContent());
		((GetDescribeEntityResponseType)protocolBaseType).setGetDescribeEntityFailure(protocolFailureType);
	}

}
