/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.describeentityPE.schema;

import org.iso.mpeg.mxm.basePE.ProtocolSuccess;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.AccessUnitType;
import org.iso.mpeg.mxm.engine.protocol.describeentityPE.schema.describeentityprotocol.GetDescribeEntitySuccessType;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.AccessUnit;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class GetDescribeEntitySuccess extends ProtocolSuccess
						implements org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntitySuccess
{
	private MetadataEngine metadataEngine;
	
	public GetDescribeEntitySuccess(MetadataEngine metadataEngine)
	{
		super(new GetDescribeEntitySuccessType());
		this.metadataEngine = metadataEngine;
	}
	
	public GetDescribeEntitySuccess(MetadataEngine metadataEngine, GetDescribeEntitySuccessType getDescribeEntitySuccessType)
	{
		super(getDescribeEntitySuccessType);
		this.metadataEngine = metadataEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntitySuccess#getAccessUnit()
	 */
	@Override
	public AccessUnit getAccessUnit() throws MXMEngineException
	{
		AccessUnitType accessUnitType = ((GetDescribeEntitySuccessType)protocolSuccessType).getAccessUnit();
		
		if (accessUnitType!=null)
		{
			AccessUnit accessUnit = metadataEngine.getMetadataEngineSchemaHandler().getAccessUnit();
			accessUnit.parse(new MXMAdapter(accessUnitType));
			return accessUnit;
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntitySuccess#setAccessUnit(org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.AccessUnit)
	 */
	@Override
	public void setAccessUnit(AccessUnit accessUnit) throws MXMException
	{
		AccessUnitType accessUnitType = (AccessUnitType)(accessUnit.createObject(AccessUnitType.class.getName()).getContent());
		((GetDescribeEntitySuccessType)protocolSuccessType).setAccessUnit(accessUnitType);
	}

}
