package org.iso.mpeg.mxm.digitalitemTE.schemahandler;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.digitalitemTE.DigitalitemTE;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.DIDLInfoType;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.DIDLType;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.ItemType;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.SuperDidlAbstractLevel1;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Container;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDLInfo;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_mp21.Declarations;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;

public class DIDL extends DIDL_mp21 implements org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL {

	protected DIDLType didltype;

	DigitalitemTE DIengine;
//	MXMEngine SECengine;
	
	public DIDL(DigitalitemTE DIengine) throws MXMEngineException{
		this.DIengine = DIengine;


		didltype=new DIDLType();
	}
	
	@Override
	public void createFile(File f) throws MXMException {
		(new SuperDidlAbstractLevel1(didltype)).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException {
		return (new SuperDidlAbstractLevel1(didltype)).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException {
		SuperDidlAbstractLevel1 level1=new SuperDidlAbstractLevel1(didltype);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			this.didltype=(DIDLType)(level1.createObject(DIDLType.class.getCanonicalName())).getContent();
		}
		return response;
	}


	@Override
	public Item getMPEGMItem() throws MXMEngineException {
		
		org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.ItemType itemtype_mpegm;
		
		JAXBElement<? extends org.iso.mpeg.mxm.engine.digitalitemTE.schema.didmodel_mp21.ItemType> itemtype = didltype.getItem();
		if (! (itemtype.getValue() instanceof org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.ItemType))
			throw new MXMEngineException("didl contains a ItemType not in line with mpegm profile");
		else
			itemtype_mpegm=(org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.ItemType)itemtype.getValue();
			
		Item item=DIengine.getDIEngineSchemaHandler().getItem();
		item.parse(new MXMAdapter(itemtype_mpegm));
		return item;
	}

	@Override
	public void setMPEGMItem(Item item) throws MXMException {
		
		//TODO test it
		
		didltype.setItem((JAXBElement<ItemType>)(item.createObject(JAXBElement.class.getCanonicalName())).getContent());
	}



	@Override
	public void setMPEGMContainer(Container container) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public Container getMPEGMContainer() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Signature getSignature() throws MXMEngineException {
		for (DIDLInfo dinfo:getMPEGMDIDLInfo())
		{
			return dinfo.getSignature();
		}
		return null;
		
	}

	@Override
	public void setSignature(Signature signature) throws MXMException {
		DIDLInfo d=DIengine.getDIEngineSchemaHandler().getDIDLInfo();
		d.setSignature(signature);
		List<DIDLInfo> infos=new ArrayList();
		infos.add(d);
		setMPEGMDIDLInfo(infos);
	}

	@Override
	public void setMPEGMDIDLInfo(List<DIDLInfo> infos) throws MXMException {
		for (DIDLInfo dinf:infos){
				//didltype.getDIDLInfo().add((JAXBElement<? extends org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_mp21.DIDLInfoType>)dinf.createObject(DIDLInfoType.class.getCanonicalName()).getContent());
			didltype.getDIDLInfo().add((JAXBElement<? extends org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_mp21.DIDLInfoType>)dinf.createObject(JAXBElement.class.getCanonicalName()).getContent());
		}
		
		
	}
	
	@Override
	public List<DIDLInfo> getMPEGMDIDLInfo() throws MXMEngineException {
		List<DIDLInfo> toreturn=new ArrayList();
		for (JAXBElement<? extends org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_mp21.DIDLInfoType> dinf: didltype.getDIDLInfo())
		{
			DIDLInfo d=DIengine.getDIEngineSchemaHandler().getDIDLInfo();
			d.parse(new MXMAdapter(dinf));
			toreturn.add(d);
			
		}
		return toreturn;
	}


}
