/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.digitalitemTE.test;

//usefull import
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.DIDEngineContext;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.ApplicationSpecificData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.dii.RelatedIdentifier;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.IPMPGeneralInfoDescriptor;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.LicenseCollection;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.RightsDescriptor;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Mpeg7;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;



public class DigitalitemEngineTest extends TestCase {

	private final String MPEG7_METADATA_URI = "urn:mpeg:mpeg7:schema:2004";
	
	private JAXBContext jc = null;
	private Marshaller m = null;	
	
	//private final String SOURCE_MEDIA_RESOURCE_FILE_NAME = "icons_small.jpg";

	private final String COPYRIGHT = "\u00a9";
	//private final String REGISTERED = "\u00ae";
	
//	@Override
//	protected void setUp() throws Exception {
//		super.setUp();
//		jc = DIDEngineContext.getInstance().getJAXBContext();
//		m = jc.createMarshaller();
//		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);	
//		m.setProperty("com.sun.xml.bind.namespacePrefixMapper", SchemaNamespacePrefixMapperImpl.getInstance());
//	}

	private Logger logger = Logger.getLogger(DigitalitemEngineTest.class);
	
	private final String TEMP_FOLDER = System.getProperty("java.io.tmpdir") + File.separator;
	private final String OUTPUT_DCI_FILE_NAME = TEMP_FOLDER + "mytestDCI.xml";


	private final String SOURCE_DI_FILE_NAME = "testDI.xml";
	
	private final String OUTPUT_DI_FILE_NAME =TEMP_FOLDER + "myDI.xml";
	
	public void testDIParsing() throws MXMConfigurationException, FileNotFoundException, JAXBException, MXMEngineException {
		logger.info("\n\n---------- testDIParsing ----------");
		
		//File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(this);
		mxm.listEnginesAndProperties();
		
		DIEngine didEngine = (DIEngine) mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
		EventReportEngine erEngine = (EventReportEngine) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());
		IPMPEngine ipmp = (IPMPEngine) mxm.getDefaultEngine(MXMEngineName.IPMPTE.toString());
		SecurityEngine secEngine = (SecurityEngine)mxm.getDefaultEngine(MXMEngineName.SecurityTE.toString());
		RELEngine relEngine = (RELEngine)mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
		MetadataEngine meta = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
		
		List<MXMEngine> engines=new ArrayList();
		engines.add(relEngine);
		
		ipmp.setDependencies(engines);
		
		DIDL di = null;
        try
        {
            di = didEngine.getDIEngineSchemaHandler().getDIDL();
        }
        catch (MXMException e1)
        {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
		
		Mpeg7 mp7=meta.getMetadataEngineSchemaHandler().getMpeg7();
		
		di.parse(new MXMAdapter(new File(getClass().getClassLoader().getResource(SOURCE_DI_FILE_NAME).getFile())));
		
		try {	
			
			Item topLevelItem=(Item)di.getMPEGMItem();
		
			logger.info("ID is:"+topLevelItem.getId());

			if (di.getSignature()!=null){
				logger.info("there is a signature");
			}
			
			logger.info("Identifier is:"+topLevelItem.getIdentifier());

			if (topLevelItem.getMetadata()!=null)
			{
				logger.info("there are some metdata for top level item");
				logger.info("WITH NS:"+topLevelItem.getMetadata().getRef());
				
				if (topLevelItem.getMetadata().getRef().equals(MPEG7_METADATA_URI))
				{
					Object o= topLevelItem.getMetadata().getAny();
					mp7.parse(new MXMAdapter(o));
					logger.info("time unit is:"+mp7.getTimeUnit());
				}
			}
			
			for (ERR err: topLevelItem.getERR()){
				logger.info("''event");
				logger.info("The event triggered is: " + err.getOperation());
				logger.info("The SAP is: " + err.getSAP());
				logger.info("The ERR identifier is: " + err.getERRIdentifier());
			}
			
			for (Component c:topLevelItem.getComponent()){
				for (Resource r:c.getResource()){
					logger.info("resource mime>"+r.getMime());
					logger.info("resource ref>"+r.getRef());
					logger.info("resource contencoding>"+r.getContentEncoding());
					logger.info("resource encoding>"+r.getEncoding());
				}	
			}
	
			List<ApplicationSpecificData> relationships = topLevelItem.getApplicationSpecificData();
			if(relationships != null && (!relationships.isEmpty()))
			{
				for (ApplicationSpecificData rel : relationships) 
				{
						logger.info("The Relationship was found");
				}
			}
			else
				logger.info("No relationship content was found");
			
			if (topLevelItem.getContentElementSignature()!=null){
				logger.info("there are some signature...");		
			}
		
		for (Item i:topLevelItem.getItem())
		{
			logger.info("(inner)ID is:"+i.getId());
			logger.info("(inner)Identifier is:"+i.getIdentifier());
			
			if (topLevelItem.getMetadata()!=null)
			{
				logger.info("there are some metdata for inneritem");
				logger.info("WITH NS:"+topLevelItem.getMetadata().getRef());
			}
			
			for (Component c:i.getComponent()){
				for (Resource r:c.getResource()){
					logger.info("resource mime>"+r.getMime());
					logger.info("resource ref>"+r.getRef());
					logger.info("resource contencoding>"+r.getContentEncoding());
					logger.info("resource encoding>"+r.getEncoding());
				}	
			}
			
			if (i.getRelatedIdentifier()!=null)
				for (RelatedIdentifier ri:i.getRelatedIdentifier()){
					logger.info("Relationship type: " + ri.getRelationship());
					logger.info("RelatedIdentifier: " + ri.getValue());
					
				}
			
			if (i.getIPMPGeneralInfoDescriptor()!=null){
				if (i.getIPMPGeneralInfoDescriptor().getLicenseCollection()!=null){
					if (!i.getIPMPGeneralInfoDescriptor().getLicenseCollection().getRightsDescriptor().isEmpty()){
						for (RightsDescriptor rd:i.getIPMPGeneralInfoDescriptor().getLicenseCollection().getRightsDescriptor()){
							if ((rd.getLicense()!=null)&&(!rd.getLicense().getLicense().isEmpty()))
								for (License l: rd.getLicense().getLicense()) {
									logger.info("LICENSE id is: "+l.getLicenseId());
								}
						}
					}
				}
			}
		}
		
	
		} catch (MXMException e) {
			e.printStackTrace();
		}
			
		logger.info("The DI file was successfully parsed");
	}

			public void testDCICreation() throws MXMConfigurationException, MXMEngineException {
				logger.info("\n\n---------- testDICreation ----------");
				
				//File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
				MXM mxm = MXM.getInstance(this);
				mxm.listEnginesAndProperties();
				
				DIEngine didEngine = (DIEngine)mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
				EventReportEngine erEngine = (EventReportEngine) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());
				IPMPEngine ipmp = (IPMPEngine) mxm.getDefaultEngine(MXMEngineName.IPMPTE.toString());
				RELEngine relEngine = (RELEngine) mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
				SecurityEngine secEngine = (SecurityEngine) mxm.getDefaultEngine(MXMEngineName.SecurityTE.toString());	
				MetadataEngine meta = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
				
				List<MXMEngine> engines=new ArrayList();
				engines.add(relEngine);
				
				ipmp.setDependencies(engines);
				
				org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Mpeg7 m7=meta.getMetadataEngineSchemaHandler().getMpeg7();
				
				Signature signa=secEngine.getSecurityEngineSchemaHandler().getSignature();
				
				DIDL di = null;
                try
                {
                    di = didEngine.getDIEngineSchemaHandler().getDIDL();
                }
                catch (MXMException e1)
                {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                }
				
				m7.setTimeUnit("323232");
				
				StructuredData sd=didEngine.getDIEngineSchemaHandler().getStructuredData();
				sd.setRef(MPEG7_METADATA_URI);
				
				Item toplevelitem=didEngine.getDIEngineSchemaHandler().getItem();
				
				ApplicationSpecificData asd=didEngine.getDIEngineSchemaHandler().getApplicationSpecificData();
				
				ERR err = erEngine.getEventReportEngineSchemaHandler().getERR();
				
				Item innerItem1=didEngine.getDIEngineSchemaHandler().getItem();
				Item innerItem2=didEngine.getDIEngineSchemaHandler().getItem();
				toplevelitem.setId("ANGELO");
				
				IPMPGeneralInfoDescriptor gen_desc=ipmp.getIPMPEngineSchemaHandler().getIPMPGeneralInfoDescriptor();
				LicenseCollection lc=ipmp.getIPMPEngineSchemaHandler().getLicenseCollection();
				RightsDescriptor rd=ipmp.getIPMPEngineSchemaHandler().getRightsDescriptor();
				org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.License ipmp_l =ipmp.getIPMPEngineSchemaHandler().getIPMPLicense();
				
				License l =relEngine.getRELEngineSchemaHandler().getLicense();

				l.setLicenseId("1234");	

				try {
					ipmp_l.addLicense(l);
				
					rd.setLicense(ipmp_l);
				
					lc.setRightsDescriptor(rd);
					
					gen_desc.setLicenseCollection(lc);
					
					toplevelitem.setIdentifier("temp:id:123456:ANGELO");
			        
					toplevelitem.addApplicationSpecificData(asd);
					
					err.setOperation("match");

			        err.setSAP("localhost://");
			        err.setERRIdentifier("ANGE");
					
					innerItem1.setId("B0");
					innerItem1.setIdentifier("urn:angelo");
					innerItem2.setId("B1");
					innerItem2.setIdentifier("unr:babbo");
					
					sd.setAny(m7);
					
					innerItem2.addMetadata(sd);
					
					Component c=didEngine.getDIEngineSchemaHandler().getComponent();
					Resource r=didEngine.getDIEngineSchemaHandler().getResource();
					
					r.setEncoding("encoding--resource");
					r.setMime("mime--resource");
					r.setRef("href--resource");
					r.getContentEncoding().add("content--encoding--resource");
					
					c.addResource(r);

					innerItem1.addComponent(c);
			        
					innerItem1.addIPMPGeneralInfoDescriptor(gen_desc);
					
					Signature sig = secEngine.getSecurityEngineSchemaHandler().getSignature();
					
					toplevelitem.addComponent(c);
				
					toplevelitem.addItem(innerItem1);
		
					toplevelitem.addItem(innerItem2);
					
					toplevelitem.addContentElementSignature(sig);
					
					toplevelitem.addERR(err);
					
					RelatedIdentifier ri=didEngine.getDIEngineSchemaHandler().getRelatedIdentifier();
					
					ri.setRelationship("relation");
					ri.setValue("value relat");

					toplevelitem.addRelatedIdentifier(ri);
					
					di.setMPEGMItem(toplevelitem);

					di.setSignature(signa);
					
					di.createFile(new File(OUTPUT_DI_FILE_NAME));
						
					logger.info("The DCI file was successfully created in file: " + OUTPUT_DI_FILE_NAME);
					
				} catch (MXMException e) {
					e.printStackTrace();
				}
		}
}