package org.iso.mpeg.mxm.eventreportTE;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.keeper.DefaultERKeeper;
import org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.keeper.DefaultERRKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.exception.EventReportException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EREngineParams;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERMessageHandler;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERRKeeper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


public class EREngineTechnologyHandler implements org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngineTechnologyHandler {
    protected EventReportTE erEngine;

    protected Map<String, ERKeeper> erKeeperMap = new HashMap<String, ERKeeper>();
    protected Map<String, ERRKeeper> errKeeperMap = new HashMap<String, ERRKeeper>();

    protected ERMessageHandler erMessageHandler;

    protected EREngineParams erEngineParams = null;

    public EREngineTechnologyHandler(EventReportTE erEngine) throws MXMEngineException {
        this.erEngine = erEngine;
    }

    @Override
    public ERMessageHandler getERMessageHandler() {
        // default ERMessageHandler
        if (null == erMessageHandler) {
            erMessageHandler = new ERMessageHandler() {

                @Override
                public MXMEngineResponse sendER(org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER er, String SAP) throws EventReportException {
                    return MXMEngineResponse.METHOD_NOT_IMPLEMENTED;
                }

                @Override
                public MXMEngineResponse recvERDaemon() throws EventReportException, MXMEngineException {
                    return MXMEngineResponse.METHOD_NOT_IMPLEMENTED;
                }
            };
        }
        return erMessageHandler;
    }

    @Override
    public EREngineParams getEREngineParams() {
        if (null == erEngineParams) {
            erEngineParams = new EREngineParams() {
                @Override
                public String getPeerId() {
                    return erEngine.getPropertyValue("PEER_ID");
                }

                @Override
                public String getSAP() {
                    return erEngine.getPropertyValue("ER_SAP");
                }

                @Override
                public String getRootERRStorageLocation() {
                    return erEngine.getPropertyValue("ROOT_ERR_STORAGE_LOCATION");
                }

                @Override
                public String getRootERStorageLocation() {
                    return erEngine.getPropertyValue("ROOT_ER_STORAGE_LOCATION");
                }
            };
        }

        return erEngineParams;
    }

    @Override
    public ERRKeeper getEventReportRequestKeeper(String repositoryKey) {
        ERRKeeper errKeeper = errKeeperMap.get(repositoryKey);
        if (null == errKeeper) {
            errKeeper = new DefaultERRKeeper(
                repositoryKey,
                getEREngineParams().getRootERRStorageLocation()
            );
            errKeeperMap.put(repositoryKey, errKeeper);
        }

        return errKeeper;
    }

    @Override
    public List<ERRKeeper> getEventReportRequestKeepers() {
        return new ArrayList<ERRKeeper>(errKeeperMap.values());
    }

    @Override
    public ERKeeper getEventReportKeeper(String repositoryKey) {
        ERKeeper erKeeper = erKeeperMap.get(repositoryKey);
        if (null == erKeeper) {
            erKeeper = new DefaultERKeeper(
                repositoryKey,
                getEREngineParams().getRootERStorageLocation(),
                getEventReportRequestKeeper(repositoryKey)
            );
            erKeeperMap.put(repositoryKey, erKeeper);
        }

        return erKeeper;
    }

    @Override
    public List<ERKeeper> getEventReportKeepers() {
        return new ArrayList<ERKeeper>(erKeeperMap.values());
    }


}
