package org.iso.mpeg.mxm.eventreportTE;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.eventreportTE.schema.er.*;
import org.iso.mpeg.mxm.engine.relTE.schema.relr.ServiceReference;
import org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.keeper.IDGenerator;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERRDescriptor;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERSpecification;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.EventConditionDescriptor;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm1x.ServiceLocation;

import java.io.File;
import java.util.List;

public class ERR implements org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR {

    ERRType errtype;

    RELEngine RELengine;

    public ERR(RELEngine RELengine) throws MXMEngineException {
        this.RELengine = RELengine;

        errtype = new ERRType();
        setERRIdentifier(IDGenerator.generateUsingUUID(IDGenerator.ERR_IDENTIFIER_PREFIX));
    }

    @Override
    public void createFile(File f) throws MXMException {
        (new SuperErAbstractLevel1(errtype)).createFile(f);
    }

    @Override
    public MXMObject createObject(String objectType) throws MXMException {
        return (new SuperErAbstractLevel1(errtype)).createObject(objectType);
    }

    @Override
    public MXMEngineResponse parse(MXMObject object) throws MXMEngineException {
        SuperErAbstractLevel1 level1 = new SuperErAbstractLevel1(errtype);
        MXMEngineResponse response = level1.parse(object);
        this.errtype = (ERRType) (level1.createObject(ERRType.class.getCanonicalName())).getContent();
        return response;
    }

    @Override
    public ERRDescriptor getERRDescriptor() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public void setERRDescriptor(ERRDescriptor eRRDescriptor) {
        // TODO Auto-generated method stub
    }

    @Override
    public ERSpecification getERSpecification() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public void setERSpecification(ERSpecification eRSpecification) {
        // TODO Auto-generated method stub
    }

    @Override
    public EventConditionDescriptor getEventConditionDescriptor() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public void setEventConditionDescriptor(
        EventConditionDescriptor eventConditionDescriptor) {
        // TODO Auto-generated method stub
    }

    @Override
    /**
     * return "" if not found
     */
    public String getSAP() throws MXMEngineException {
        ERSpecificationType est = errtype.getERSpecification();

        List<ERDeliverySpecificationType> sdsts = est.getERDeliverySpecification();

        for (ERDeliverySpecificationType sdst : sdsts) {
            DITransportServiceType dtst = sdst.getDITransportService();

            ServiceReference serviceReferenceType = dtst.getServiceReference();

            org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.ServiceReference serviceReference =
                RELengine.getRELEngineSchemaHandler().getServiceReference();
            MXMEngineResponse response = serviceReference.parse(new MXMAdapter(serviceReferenceType));
            if (MXMEngineResponse.SUCCESS == response) {
                if (serviceReference.getServiceDescription() instanceof ServiceLocation) {
                    ServiceLocation serviceLocation = (ServiceLocation) serviceReference.getServiceDescription();
                    return serviceLocation.getUrl();
                }
                return serviceReference.getServiceDescription().getVarRef();
            }
        }

        return "";
    }

    @Override
    /**
     * return "" if not found
     */
    public String getOperation() {
        EventConditionDescriptorType ecd = errtype.getEventConditionDescriptor();
        List<Object> objs = ecd.getOperatorOrTimeConditionOrDIOperationCondition();//.getEventConditionGroup();
        for (Object obj : objs) {
            if (obj instanceof DIOperationConditionType) {
                DIOperationConditionType dio = (DIOperationConditionType) obj;
                for (Object ocg : dio.getDIOperationEventOrOperator()) {
                    if (ocg instanceof DIOperationEventType)
                        return ((DIOperationEventType) ocg).getOperation();

                }

            }
        }
        //TODO fixme thrown an exception
        return "";
    }

    @Override
    public String getERRIdentifier() {
        ERSpecificationType est = errtype.getERSpecification();
        EmbeddedERRType eet = est.getEmbeddedERR().get(0);
        Object errIdentifier = eet.getERROrERRReference().get(0);

        return (String) errIdentifier;
    }

    @Override
    public void setERRIdentifier(String errIdentifier) {
        ERSpecificationType est;

        if ((est = errtype.getERSpecification()) == null) {
            est = new ERSpecificationType();
        }

        if (est.getEmbeddedERR().isEmpty()) {
            est.getEmbeddedERR().add(new EmbeddedERRType());
        }
        EmbeddedERRType ee = est.getEmbeddedERR().get(0);
        if (ee.getERROrERRReference().isEmpty()) {
            ee.getERROrERRReference().add(new Object());
        }
        ee.getERROrERRReference().set(0, errIdentifier);

        errtype.setERSpecification(est);

    }

    @Override
    public void setSAP(String SAP) throws MXMException {
        ServiceLocation serviceLocation = RELengine.getRELEngineSchemaHandler().getServiceLocation();
//        org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.ServiceDescription serviceDescription = RELengine.getRELEngineSchemaHandler().getServiceDescription();
        org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.ServiceReference serviceReference = RELengine.getRELEngineSchemaHandler().getServiceReference();

        serviceLocation.setUrl(SAP);
        serviceReference.setServiceDescription(serviceLocation);

        ServiceReference serviceReferenceType = (ServiceReference) (serviceReference.createObject(ServiceReference.class.getName()).getContent());

        DITransportServiceType dtst = new DITransportServiceType();
        dtst.setServiceReference(serviceReferenceType);

        ERDeliverySpecificationType edst = new ERDeliverySpecificationType();
        edst.setDITransportService(dtst);

        ERSpecificationType est;

        if ((est = errtype.getERSpecification()) == null)
            est = new ERSpecificationType();

        est.getERDeliverySpecification().add(edst);

        errtype.setERSpecification(est);

    }

    @Override
    public void setOperation(String operation) {
        DIOperationEventType dot = new DIOperationEventType();
        dot.setOperation(operation);

        DIOperationConditionType doc = new DIOperationConditionType();
        doc.getDIOperationEventOrOperator().add(dot);

        EventConditionDescriptorType ecd = new EventConditionDescriptorType();
        ecd.getOperatorOrTimeConditionOrDIOperationCondition().add(doc);

        errtype.setEventConditionDescriptor(ecd);
    }

}
