package org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.keeper;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.exception.EventReportException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERRKeeper;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

/**
 * The default implementation of the <tt>ERKeeper</tt> interface.
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 8/31/11
 * Time: 6:23 PM
 * <p/>
 * UTI Systems, TIC
 */
public class DefaultERKeeper implements ERKeeper {
    private static final String XML_EXT = ".xml";


    /**
     * Mapping the ERR id to a list ERR ids.
     */
    private final GenericKeeper<String, Collection<String>> keeperERR;

    private final Logger logger = Logger.getLogger(DefaultERKeeper.class);

    private final ERRKeeper errKeeper;
    private final String rootERStorageLocation;

    public DefaultERKeeper(String repositoryKey, String rootERStorageLocation, ERRKeeper errKeeper) {
        this.errKeeper = errKeeper;

        if (null == repositoryKey || repositoryKey.isEmpty()) {
            throw new IllegalArgumentException("repositoryKey cannot be null or empty!");
        }

        this.rootERStorageLocation = rootERStorageLocation;

        try {
            File f = new File(rootERStorageLocation);
            if (!f.exists()) {
                FileUtils.forceMkdir(f);
                logger.info("rootERStorageLocation created to: " + rootERStorageLocation);
            }
        } catch (IOException e) {
            throw new RuntimeException("Can not create rootERStorageLocation to: " + rootERStorageLocation);
        }

        keeperERR = new GenericKeeper<String, Collection<String>>(
            rootERStorageLocation + File.separatorChar + "mapping_" + repositoryKey + XML_EXT
        );
    }

    @Override
    public MXMEngineResponse storeER(ER er) throws EventReportException {
        //ER er = (ER) erMXMObject.getContent();

        String erIdentifier = er.getERIdentifier();
        String errIdentifier = er.getERRIdentifier();

        Collection<String> erIds = keeperERR.get(errIdentifier);
        if (null == erIds) {
            erIds = new HashSet<String>();
        }
        erIds.add(erIdentifier);
        keeperERR.put(errIdentifier, erIds);

        try {
            er.createFile(new File(buildFilePath(erIdentifier)));
        } catch (MXMException e) {

            throw new EventReportException("Trouble storeER", e);
        }

        return MXMEngineResponse.SUCCESS;
    }

    @Override
    public ER getER(String erId) throws EventReportException {
        String absFilePath = buildFilePath(erId);

        try {
            ER er = new org.iso.mpeg.mxm.eventreportTE.ER();
            er.parse(new MXMAdapter(new File(absFilePath)));
            return er;
        } catch (MXMEngineException e) {
            throw new EventReportException("Cannot get ER object from location: " + absFilePath, e);
        }
    }

    @Override
    public List<String> getERIdsByERRId(String errId) throws EventReportException {
        Collection<String> erIds = keeperERR.get(errId);
        return (null != erIds) ? new ArrayList<String>(erIds) : new ArrayList<String>();
    }

    @Override
    public List<String> getERIdsByDIDLId(String DIDLId) throws EventReportException {
        String errId = errKeeper.getERRIdByDIDLId(DIDLId);
        if (null != errId) {
            return new ArrayList<String>(getERIdsByERRId(errId));
        } else {
            return new ArrayList<String>();
        }
    }

    private String buildFilePath(String fName) {
        fName = fName.replaceFirst(IDGenerator.ER_IDENTIFIER_PREFIX, "");
        return rootERStorageLocation + File.separatorChar + fName + XML_EXT;
    }
}
