package org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.keeper;

import java.util.HashMap;
import java.util.Map;

import org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.Constants;
import org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.util.GenericMappingSerializer;

/**
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 9/5/11
 * Time: 5:40 PM
 * <p/>
 * UTI Systems, TIC
 */
public class GenericKeeper<K, V> {
    private Map<K, V> mapping;

    private String absFilePath;

    private GenericMappingSerializer<K, V> serializer;

    public GenericKeeper(String absFilePath) {
        if (Constants.USE_IN_MEMORY_MAPPING) {
            mapping = new HashMap<K, V>();
        } else {
            this.serializer = new GenericMappingSerializer<K, V>(absFilePath);
        }
    }

    public void put(K key, V value) {
        if (Constants.USE_IN_MEMORY_MAPPING) {
            mapping.put(key, value);
        } else {
            Map<K, V> m = serializer.unmarshall();
            m.put(key, value);
            serializer.marshall(m);
        }
    }

    public V get(K key) {
        if (Constants.USE_IN_MEMORY_MAPPING) {
            return mapping.get(key);
        } else {
            Map<K, V> m = serializer.unmarshall();
            return m.get(key);
        }
    }

    public Map<K, V> getMapping() {
        if (Constants.USE_IN_MEMORY_MAPPING) {
            return new HashMap<K, V>(mapping);
        } else {
            return serializer.unmarshall();
        }
    }
}
