/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006
 * All Rights Reserved.
 *
 * The Initial Developer: Nicola Christian Barbieri
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package eu.convergence.middleware.eventreportTE;

import junit.framework.TestCase;
import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;

import java.io.File;

public class ERBuilderTest extends TestCase {

    private final String TEMP_FOLDER = System.getProperty("java.io.tmpdir") + File.separator;
    private final String OUTPUT_ER_FILE_NAME = TEMP_FOLDER + "mytestER.xml";

    private final String SOURCE_ER_FILE_NAME = "mytestER.xml";

    private Logger logger = Logger.getLogger(ERBuilderTest.class);

    private EventReportEngine erEngine;

    public ERBuilderTest(String arg0) {
        super(arg0);
    }

    protected void setUp() throws Exception {
        super.setUp();

        File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
        MXM mxm = MXM.getInstance(f);
        mxm.listEnginesAndProperties();

        erEngine = (EventReportEngine) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testERGeneration() throws Exception {
        logger.info("\n\n---------- testERGenerator ----------");

        ER er = erEngine.getEventReportEngineSchemaHandler().getER();

        er.setERRIdentifier("errId");
        er.setDIIId("DIDLId");

        er.createFile(new File(OUTPUT_ER_FILE_NAME));

        logger.info("Generated ER on : " + OUTPUT_ER_FILE_NAME);
    }

    public void testERParsing() throws Exception {
        logger.info("\n\n---------- testDIParsing ----------");

        ER er = erEngine.getEventReportEngineSchemaHandler().getER();

        er.parse(new MXMAdapter(new File(getClass().getClassLoader().getResource(SOURCE_ER_FILE_NAME).getFile())));

        logger.info("The ERR Identifier is: " + er.getERRIdentifier());
        logger.info("The DIDL id is: " + er.getDIIId());
        logger.info("The ER Identifier is: " + er.getERIdentifier());
    }
}
