package eu.convergence.middleware.eventreportTE;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.Constants;
import org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.keeper.DefaultERKeeper;
import org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.keeper.DefaultERRKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERRKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EventTriggerType;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Test;

import java.io.File;
import java.util.List;

/**
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 9/6/11
 * Time: 3:48 PM
 * <p/>
 * UTI Systems, TIC
 */
@Test
public class ERKeeperTest {
    private static final String S_VDI_ID = "S_VDI_ID";
    private static final String P_VDI_ID = "S_VDI_ID";

    private static final String ROOT_ERR_STORAGE_LOCATION = "C:\\Temp\\ERR";
    private static final String ROOT_ER_STORAGE_LOCATION = "C:\\Temp\\ER";

    private static final String ERR_REPOSITORY_KEY = "ERR";
    private static final String ER_REPOSITORY_KEY = "ER";

    private EventReportEngine erEngine;

    @BeforeTest
    public void setUp() throws Exception {
        Constants.USE_IN_MEMORY_MAPPING = false;

        File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
        MXM mxm = MXM.getInstance(f);
        mxm.listEnginesAndProperties();

        erEngine = (EventReportEngine) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());
    }

    public void testGetMultipleERsByERRId() throws Exception {
        ERRKeeper errKeeper = new DefaultERRKeeper(ERR_REPOSITORY_KEY, ROOT_ERR_STORAGE_LOCATION);
        ERKeeper erKeeper = new DefaultERKeeper(ER_REPOSITORY_KEY, ROOT_ER_STORAGE_LOCATION, errKeeper);

        ERR err = erEngine.getEventReportEngineSchemaHandler().getERR();

        // create and store ERR
        err.setOperation(EventTriggerType.MATCH.toString());
        err.setSAP("SAP");
        errKeeper.storeERR(err, S_VDI_ID);
        //err.parse(err);
        String errId = err.getERRIdentifier();

        // create and store ER1
        ER er = erEngine.getEventReportEngineSchemaHandler().getER();
        er.setERRIdentifier(errId);
        er.setDIIId(P_VDI_ID);
        erKeeper.storeER(er);

        // create and store ER2
        er = erEngine.getEventReportEngineSchemaHandler().getER();
        er.setERRIdentifier(errId);
        er.setDIIId(P_VDI_ID);
        erKeeper.storeER(er);

        // create and store ER1
        er = erEngine.getEventReportEngineSchemaHandler().getER();
        er.setERRIdentifier(errId);
        er.setDIIId(P_VDI_ID);
        erKeeper.storeER(er);


        // get list of ERIds by ERRId
        List<String> erIds = erKeeper.getERIdsByERRId(errId);
        System.out.println("erKeeper.getERIdsByERRId = " + erIds);

        // get list od ERIds by S-VDI-ID
        erIds = erKeeper.getERIdsByDIDLId(S_VDI_ID);
        System.out.println("erKeeper.getERIdsByDIDLId = " + erIds);
    }
}
