/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Angelo Difino
 * Contributor(s): 
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.identifycontentPE.schema;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolSuccess;
import org.iso.mpeg.mxm.baseprotocol.schema.ContentEntityType;
import org.iso.mpeg.mxm.baseprotocol.schema.SuperBaseProtocolAbstractLevel1;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.identifycontentPE.schema.identifycontentprotocol.IdentifyContentSuccessType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;


/**
 * @author Angelo Difino
 *
 */
public class IdentifyContentSuccess extends ProtocolSuccess 
									implements org.iso.mpeg.mxm.pEngine.identifycontentPE.schemahandler.IdentifyContentSuccess
{
	private BaseProtocolEngine baseProtocolEngine;
	
	public IdentifyContentSuccess(BaseProtocolEngine baseProtocolEngine) {
		super(new IdentifyContentSuccessType());
		this.baseProtocolEngine=baseProtocolEngine;
	}

	@Override
	public void createFile(File f) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1((IdentifyContentSuccessType)protocolSuccessType);
		level1.createFile(f);
	}
	
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1((IdentifyContentSuccessType)protocolSuccessType);
		return level1.createObject(objectType);
	}
	
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1((IdentifyContentSuccessType)protocolSuccessType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolSuccessType = 
					(IdentifyContentSuccessType)(level1.createObject(IdentifyContentSuccessType.class.getName()).getContent());
		}
		return response;
	}
	
	@Override
	public ContentEntity getIdentifiedContent() throws MXMEngineException {
		ContentEntity ce= baseProtocolEngine.getBaseProtocolSchemaHandler().getContentEntity();
		ce.parse(new MXMAdapter(((IdentifyContentSuccessType)protocolSuccessType).getIdentifiedContent()));
		return ce;
	}

	@Override
	public void setIdentifiedContent(ContentEntity contentEntity) throws MXMException {
		ContentEntityType didlType = (ContentEntityType)(contentEntity.createObject(ContentEntityType.class.getName()).getContent());
		((IdentifyContentSuccessType)protocolSuccessType).setIdentifiedContent(didlType);
	}
}
