/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Angelo Difino
 * Contributor(s): 
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.identifycontentPE.technology.remote;

import java.net.URL;

import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.ServiceFactory;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.es.identifycontentES.IdentifyContentProtocolRemote;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.exceptions.IdentifyContentEngineException;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.schemahandler.IdentifyContentRequest;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.schemahandler.IdentifyContentResponse;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Angelo Difino <angelo@cedeo.net>
 *
 */
public class IdentifyContentProtocol implements org.iso.mpeg.mxm.pEngine.identifycontentPE.technologyhandler.IdentifyContentProtocol
{
	Logger logger = LoggerFactory.getLogger(this.getClass());
	
	private URL serviceURL = null;
	private QName serviceName = null;
	
	private BaseProtocolEngine baseProtocolEngine;
	private SecurityEngine securityEngine;

	public IdentifyContentProtocol(BaseProtocolEngine baseProtocolEngine,SecurityEngine securityEngine)
	{
		this.baseProtocolEngine = baseProtocolEngine;
		this.securityEngine=securityEngine;
	}
	
	@Override
	public MXMEngineResponse initServiceParams(URL serviceURL, QName serviceName)
	{
		this.serviceURL = serviceURL;
		this.serviceName = serviceName;
		
		return MXMEngineResponse.SUCCESS;
	}
	
	@Override
	public IdentifyContentResponse identifyContent(IdentifyContentRequest request) throws IdentifyContentEngineException
	{
		if (serviceURL == null)
		{
			logger.warn("Service URL has not been set");
		}
		if (serviceName == null)
		{
			logger.warn("Service Name has not been set");
		}

		try 
		{
			ServiceFactory factory = ServiceFactory.newInstance();
			Service service = factory.createService(serviceURL, serviceName);
		
			IdentifyContentProtocolRemote identifyContentProtocolRemote = 
								(IdentifyContentProtocolRemote) service.getPort(IdentifyContentProtocolRemote.class);

			// Create the request
			String serviceRequest = (String)((request.createObject(String.class.getName())).getContent());
			
			// Create the response
			String serviceResponse = identifyContentProtocolRemote.identifyContent(serviceRequest);
			
			// Parse the response into the standard API
			IdentifyContentResponse identifyContentResponse = new org.iso.mpeg.mxm.identifycontentPE.schema.IdentifyContentResponse(baseProtocolEngine, securityEngine);//getIdentifyContentResponse();
			identifyContentResponse.parse(new MXMAdapter(serviceResponse));			
			
			return identifyContentResponse;
		} 
		catch (MXMException e) 
		{
			throw new IdentifyContentEngineException(e.getMessage(), e);
		} 
		catch (ServiceException e) 
		{
			throw new IdentifyContentEngineException(e.getMessage(), e);
		} 
	}
}