/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.identifydevicePE.schemahandler;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolResponse;
import org.iso.mpeg.mxm.baseprotocol.schema.ProtocolFailureType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.identifydevicePE.schema.identifydeviceprotocol.IdentifyDeviceResponseType;
import org.iso.mpeg.mxm.engine.protocol.identifydevicePE.schema.identifydeviceprotocol.IdentifyDeviceSuccessType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.pEngine.identifydevicePE.schemahandler.IdentifyDeviceSuccess;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class IdentifyDeviceResponse extends ProtocolResponse 
					implements org.iso.mpeg.mxm.pEngine.identifydevicePE.schemahandler.IdentifyDeviceResponse
{	
	private BaseProtocolEngine baseProtocolEngine;
	
	public IdentifyDeviceResponse(BaseProtocolEngine baseProtocolEngine, SecurityEngine securityEngine)
	{
		super(new IdentifyDeviceResponseType(), securityEngine);
		this.baseProtocolEngine = baseProtocolEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((IdentifyDeviceResponseType)protocolBaseType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((IdentifyDeviceResponseType)protocolBaseType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((IdentifyDeviceResponseType)protocolBaseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolBaseType = 
					(IdentifyDeviceResponseType)(level1.createObject(IdentifyDeviceResponseType.class.getName()).getContent());
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifydevicePE.schemahandler.IdentifyDeviceResponse#getIdentifyDeviceSuccess()
	 */
	@Override
	public IdentifyDeviceSuccess getIdentifyDeviceSuccess() throws MXMEngineException
	{
		IdentifyDeviceSuccessType identifyDeviceSuccessType = ((IdentifyDeviceResponseType)protocolBaseType).getIdentifyDeviceSuccess();
		IdentifyDeviceSuccess identifyDeviceSuccess =
				new org.iso.mpeg.mxm.identifydevicePE.schemahandler.IdentifyDeviceSuccess(securityEngine);
		MXMEngineResponse response = identifyDeviceSuccess.parse(new MXMAdapter(identifyDeviceSuccessType));
		if (!response.equals(MXMEngineResponse.SUCCESS))
		{
			return null;
		}
		return identifyDeviceSuccess;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifydevicePE.schemahandler.IdentifyDeviceResponse#setIdentifyDeviceSuccess(org.iso.mpeg.mxm.pEngine.identifydevicePE.schemahandler.IdentifyDeviceSuccess)
	 */
	@Override
	public void setIdentifyDeviceSuccess(IdentifyDeviceSuccess identifyDeviceSuccess) throws MXMException
	{
		IdentifyDeviceSuccessType identifyDeviceSuccessType = 
				(IdentifyDeviceSuccessType)(identifyDeviceSuccess.createObject(IdentifyDeviceSuccessType.class.getName()).getContent());
		((IdentifyDeviceResponseType)protocolBaseType).setIdentifyDeviceSuccess(identifyDeviceSuccessType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifydevicePE.schemahandler.IdentifyDeviceResponse#getIdentifyDeviceFailure()
	 */
	@Override
	public ProtocolFailure getIdentifyDeviceFailure() throws MXMEngineException
	{
		ProtocolFailureType protocolFailureType = ((IdentifyDeviceResponseType)protocolBaseType).getIdentifyDeviceFailure();
		ProtocolFailure protocolFailure = baseProtocolEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
		protocolFailure.parse(new MXMAdapter(protocolFailureType));
		return protocolFailure;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifydevicePE.schemahandler.IdentifyDeviceResponse#setIdentifyDeviceFailure(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure)
	 */
	@Override
	public void setIdentifyDeviceFailure(ProtocolFailure protocolFailure) throws MXMException
	{
		ProtocolFailureType protocolFailureType = 
				(ProtocolFailureType)(protocolFailure.createObject(ProtocolFailureType.class.getName()).getContent());
		((IdentifyDeviceResponseType)protocolBaseType).setIdentifyDeviceFailure(protocolFailureType);
	}

}
