/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.identifydevicePE.schemahandler;

import org.iso.mpeg.mxm.basePE.ProtocolSuccess;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.identifydevicePE.schema.identifydeviceprotocol.IdentifyDeviceSuccessType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.KeyInfoType;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class IdentifyDeviceSuccess extends ProtocolSuccess 
						implements org.iso.mpeg.mxm.pEngine.identifydevicePE.schemahandler.IdentifyDeviceSuccess
{
	private SecurityEngine securityEngine;
	
	public IdentifyDeviceSuccess(SecurityEngine securityEngine)
	{
		super(new IdentifyDeviceSuccessType());
		this.securityEngine = securityEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifydevicePE.schemahandler.IdentifyDeviceSuccess#getDeviceKey()
	 */
	@Override
	public KeyInfo getDeviceKey() throws MXMEngineException
	{
		KeyInfoType keyInfoType = ((IdentifyDeviceSuccessType)protocolSuccessType).getDeviceKey();
		KeyInfo keyInfo = securityEngine.getSecurityEngineSchemaHandler().getKeyInfo();
		keyInfo.parse(new MXMAdapter(keyInfoType));
		return keyInfo;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifydevicePE.schemahandler.IdentifyDeviceSuccess#setDeviceKey(org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo)
	 */
	@Override
	public void setDeviceKey(KeyInfo keyInfo) throws MXMException
	{
		KeyInfoType keyInfoType = (KeyInfoType)(keyInfo.createObject(KeyInfoType.class.getName()).getContent());
		((IdentifyDeviceSuccessType)protocolSuccessType).setDeviceKey(keyInfoType);
	}

}
