/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.identifyuserPE.schemahandler;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolRequest;
import org.iso.mpeg.mxm.baseprotocol.schema.ContentEntityType;
import org.iso.mpeg.mxm.baseprotocol.schema.UserEntityType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.identifyuserPE.schema.identifyuserprotocol.IdentifyUserRequestType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AssertionType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.UserEntity;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Assertion;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class IdentifyUserRequest extends ProtocolRequest
					implements org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest
{
	private BaseProtocolEngine baseProtocolEngine;
	
	public IdentifyUserRequest(BaseProtocolEngine baseProtocolEngine, SecurityEngine securityEngine)
	{
		super(new IdentifyUserRequestType(), securityEngine);
		this.baseProtocolEngine = baseProtocolEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((IdentifyUserRequestType)protocolBaseType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((IdentifyUserRequestType)protocolBaseType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((IdentifyUserRequestType)protocolBaseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolBaseType = 
					(IdentifyUserRequestType)(level1.createObject(IdentifyUserRequestType.class.getName()).getContent());
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest#getContentEntity()
	 */
	@Override
	public ContentEntity getContentEntity() throws MXMEngineException
	{
		ContentEntityType contentEntityType = ((IdentifyUserRequestType)protocolBaseType).getIdentifyingMaterial();
		ContentEntity contentEntity = baseProtocolEngine.getBaseProtocolSchemaHandler().getContentEntity();
		contentEntity.parse(new MXMAdapter(contentEntityType));
		return contentEntity;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest#setContentEntity(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity)
	 */
	@Override
	public void setContentEntity(ContentEntity contentEntity) throws MXMException
	{
		ContentEntityType contentEntityType = 
				(ContentEntityType)(contentEntity.createObject(ContentEntityType.class.getName()).getContent());
		((IdentifyUserRequestType)protocolBaseType).setIdentifyingMaterial(contentEntityType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest#getUserIdentificationAssertion()
	 */
	@Override
	public Assertion getUserIdentificationAssertion() throws MXMEngineException
	{
		AssertionType assertionType = ((IdentifyUserRequestType)protocolBaseType).getAssertion();
		Assertion assertion = securityEngine.getSecurityEngineSchemaHandler().getAssertion();
		assertion.parse(new MXMAdapter(assertionType));
		return assertion;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest#setUserIdentificationAssertion(org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Assertion)
	 */
	@Override
	public void setUserIdentificationAssertion(Assertion assertion) throws MXMException
	{
		AssertionType assertionType = (AssertionType)(assertion.createObject(AssertionType.class.getName()).getContent());
		((IdentifyUserRequestType)protocolBaseType).setAssertion(assertionType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest#getUserEntity()
	 */
	@Override
	public UserEntity getUserEntity() throws MXMEngineException
	{
		UserEntityType userEntityType = ((IdentifyUserRequestType)protocolBaseType).getIdentificationUserEntity();
		UserEntity userEntity = baseProtocolEngine.getBaseProtocolSchemaHandler().getUserEntity();
		userEntity.parse(new MXMAdapter(userEntityType));
		return userEntity;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest#setUserEntity(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.UserEntity)
	 */
	@Override
	public void setUserEntity(UserEntity userEntity) throws MXMException
	{
		UserEntityType userEntityType = (UserEntityType)(userEntity.createObject(UserEntityType.class.getName()).getContent());
		((IdentifyUserRequestType)protocolBaseType).setIdentificationUserEntity(userEntityType);
	}

}
