import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.net.PasswordAuthentication;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.xml.bind.JAXBElement;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.xerces.impl.dv.util.Base64;
import org.iso.mpeg.mxm.baseprotocol.schema.ContentEntityType;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.digitalitemTE.DigitalitemTE;
import org.iso.mpeg.mxm.digitalitemTE.schemahandler.DIDL;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_mp21.DIDLInfoType;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl_mp21.DIDLType;
import org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpmsg.AuthCodesType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.AgentObjectType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.AgentType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.ClassificationPreferencesType.Subject;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.ContentCollectionType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.PersonType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.PlaceType;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.PlaceType.GeographicPosition;
import org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.TextualType;
import org.iso.mpeg.mxm.engine.relTE.schema.relr.DigitalResource;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.CanonicalizationMethodType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.KeyInfoType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureMethodType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureValueType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignedInfoType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AdviceType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AssertionType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AuthnContextType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AuthnStatementType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.ConditionsType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.NameIDType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.StatementAbstractType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.SubjectConfirmationType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.SubjectType;
import org.iso.mpeg.mxm.orchestrator.OrchestratorEngine;
import org.iso.mpeg.mxm.orchestrator.identifyuser.IdentifyUserEngineOrchestrator;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.AuthorizeUserEngine;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.schemahandler.AuthorizeUserRequest;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.UserEntity;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.IdentifyUserEngine;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserResponse;
import org.iso.mpeg.mxm.securityTE.technologyhandler.util.FileAndObjectHandler;
import org.iso.mpeg.mxm.securityTE.technologyhandler.util.FileSystemStorage;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Agent;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.NameComponent;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Person;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.PersonName;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Place;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.PlaceGeographicPosition;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Assertion;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnContext;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnStatement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.ConditionAbstract;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Conditions;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.NameID;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.SubjectConfirmation;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AuthnRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager;

import junit.framework.TestCase;


public class IdentifyUserTest extends TestCase
{
    private MXM                 mxm;
    private CertificateManager  certMngr          = null;
    private OrchestratorEngine  orchestratorEngine;
    private SecurityEngine      securityEngine;
    private AuthorizeUserEngine authorizeUseEngine;
    private DIEngine            diEngine;
    private String              messageToBeSigned = "messageToBeSigned";
    private String              APP_ALIAS         = "1";

    public IdentifyUserTest() throws Exception
    {
        File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml")
            .getFile());
        mxm = MXM.getInstance(f);
        mxm.listEnginesAndProperties();
    }

    private boolean checkConf()
    {
        if (securityEngine.getPropertyValue("skipIdentification").equals("true"))
        {
            System.out.println("skipIdentification = true");
            return false;
        }
        return true;
    }

    public void testIdentifyUserRequestCreation() throws Exception
    {
        System.out.println("\n--- testIdentifyUserRequestCreation ---");
        IdentifyUserEngine identifyUserEngine = (IdentifyUserEngine)mxm
            .getDefaultEngine(MXMEngineName.IdentifyUserPE.toString());
        securityEngine = (SecurityEngine)mxm.getDefaultEngine(MXMEngineName.SecurityTE
            .toString());
        securityEngine.setProperty("conf", "123456");
        securityEngine.setProperty("auth", "123456");
        securityEngine.setProperty("sig", "12345678");
        securityEngine.setProperty("gakp", "12345678");
        if (!checkConf())
        {
            return;
        }
        this.certMngr = securityEngine.getSecurityEngineTechnologyHandler()
            .getCertificateManager();
        diEngine = (DIEngine)mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
        MXMGenericParameters parm = mxm.getMxmParameters();
        // securityEngine.initialise(MXMEngineName.SecurityTE.toString(),mxm.getMxmParameters());
        MetadataEngine metadataEngine = (MetadataEngine)mxm
            .getDefaultEngine(MXMEngineName.MetadataTE.toString());
        IdentifyUserRequest identifyUserRequest = identifyUserEngine
            .getIdentifyUserSchemaHandler().getIdentifyUserRequest();
        UserEntity userEntity = identifyUserEngine.getBaseProtocolSchemaHandler()
            .getUserEntity();
        DigitalitemTE digitalItemTE = (DigitalitemTE)mxm
            .getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
        orchestratorEngine = (OrchestratorEngine)mxm
            .getDefaultEngine(MXMEngineName.Orchestrator.toString());
        BaseProtocolEngine baseProtocolEngine = null;// mxm.getDefaultEngine(MXMEngineName);
                                                     // // * TODO: how to get base
                                                     // protocol engine
        // /final KeyPair cardPair = this.certMngr.generateRSAKeyPair(2048, new
        // BigInteger(
        // "10001", 16));
        // this.secRepoManager.setKeyPairs(
        // this.certMngr.toPublicRSAKeyValue(cardPair.getPublic()),
        // this.certMngr.toPrivateRSAKeyValue(cardPair.getPrivate()), null);
        // this.certificate
        // TODO create a simple UserEntity object that holds information of the user
        // - add inside it an Agent of type Person with name, address, nationality
        userEntity.setUser(getAgent("uname","Lastname" ,null, "Germany"));// TODO: get a Geographical
                                                               // address
        // TODO create a simple assertion that the identifyUserSP would take as an input
        // to determine that the user is authenticated somewhere
        // SAMPLE GOOGLE ASSERTION THAT WE SHOULD BE ABLE TO CREATE
        // <Assertion ID="dojnoaponicbieffopfdecilinaepodfimmkpjij"
        // IssueInstant="2003-04-17T00:46:02Z" Version="2.0">
        // <Issuer>https://www.opensaml.org/IDP </Issuer>
        // <Subject>
        // <NameID Format="urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress">
        // demouser </NameID>
        // <SubjectConfirmation Method="urn:oasis:names:tc:SAML:2.0:cm:bearer" />
        // </Subject>
        // <Conditions NotBefore="2003-04-17T00:46:02Z"
        // NotOnOrAfter="2008-04-17T00:51:02Z"> </Conditions>
        // <AuthnStatement AuthnInstant="2006-08-17T10:05:29Z">
        // <AuthnContext>
        // <AuthnContextClassRef> urn:oasis:names:tc:SAML:2.0:ac:classes:Password
        // </AuthnContextClassRef>
        // </AuthnContext>
        // </AuthnStatement>
        // </Assertion>
        // TODO implement Assertion methods
        // TODO create your custom ContentEntity object that holds the identification
        // information of the user
        // this could have inside it a digital representation of a fingerprint, or a
        // certificate
        // - add inside it an Digital Item which has as resources the identification info
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data certificate = null;
        PublicKey key = null;
        RSAKeyValue publicKey = null;
        Item item = diEngine.getDIEngineSchemaHandler().getItem();
        Component c = diEngine.getDIEngineSchemaHandler().getComponent();
        Resource r = diEngine.getDIEngineSchemaHandler().getResource();
        // Maybe addressource after getcontent?
        c.addResource(r);
        // TODO use Stringishing here as before
        if (securityEngine.getPropertyValue("enableSmartCard").equals("false"))
        {
            key = certMngr.generateRSAKeyPair(APP_ALIAS, "auth", 2048);
            PublicKey storedKey = this.securityEngine
                .getSecurityEngineTechnologyHandler().getAuthenticationManager()
                .getPublicKey(APP_ALIAS, "auth", "");
            certificate = getCertificate(securityEngine);
            r.getContent().add(
                (JAXBElement)certificate.createObject(
                    JAXBElement.class.getCanonicalName()).getContent());
        }
        else
        {
            key = getPublicKeyFromCard(securityEngine);
            publicKey = this.certMngr.toPublicRSAKeyValue(key);
            r.getContent().add(
                (JAXBElement)publicKey.createObject(JAXBElement.class.getCanonicalName())
                    .getContent());
        }
        item.addComponent(c);
        ContentEntity contentEntity = identifyUserEngine.getBaseProtocolSchemaHandler()
            .getContentEntity();
        contentEntity.setDIDL(getDIDL(diEngine, item));// store
                                                       // certificate
        identifyUserRequest.setContentEntity(contentEntity);
        identifyUserRequest.setUserEntity(userEntity);
        // identifyUserRequest.setUserIdentificationAssertion(assertion);
        // TODO printout the identifyUserRequest ( with createObject -> String)
        System.out.println(identifyUserRequest.createObject(
            String.class.getCanonicalName()).getContent());
        int id = testUserIdentification(identifyUserRequest);
        String idFromKey = this.securityEngine.getSecurityEngineTechnologyHandler()
            .getUserDataBaseManager().getUserId(key);
        assertEquals(id + "", idFromKey);
    }

    private int testUserIdentification(IdentifyUserRequest identifyUserRequest)
    {
        if (!checkConf())
        {
            return -1;
        }
        IdentifyUserResponse identifyUserResponse;
        try
        {
            org.iso.mpeg.mxm.orchestratorEngine.identifyuser.IdentifyUserEngineOrchestrator orchestrator = orchestratorEngine
                .getIdentifyUserEngineOrchestrator();
            identifyUserResponse = orchestrator.identifyUser(identifyUserRequest);
            // System.out.println(identifyUserResponse.createObject(String.class.getCanonicalName()).getContent());
            UserEntity userEntity = identifyUserRequest.getUserEntity();
            String userId = userEntity.getUser().getId();
            System.out.println("User id:" + userId);
            assertNotNull(userId);
            return Integer.parseInt(userId);
        }
        catch (MXMEngineException e)
        {
            e.printStackTrace();
        }
        catch (MXMException e)
        {
            e.printStackTrace();
        }
        return -1;
    }

    private X509Data getCertificate(SecurityEngine securityEngine)
        throws SecurityEngineException,MXMEngineException
    {
        if (!checkConf())
        {
            return null;
        }
        String holderReference = "CN=Holder,L=London,C=GB,O=Organization";
        String authReference = "CN=Issuer,L=London,C=GB,O=Organization";
        int expiryDays = 10;
        String SignOID = "SHA256withRSAEncryption";
        BigInteger serialNumber = new BigInteger("1");
        BigInteger PubExp = new BigInteger("10001", 16);
        int keyLength = 2048;
        org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager certMngr = securityEngine
            .getSecurityEngineTechnologyHandler().getCertificateManager();
        // TODO: in generate RSA key pair there should be parameter for exp date,
        // holder,serial number because this.keypair.private key is needed to make
        // certificate , so it is not possible to generate certiifcate from outside
        X509Data cert = certMngr.getUserCertificate("auth", APP_ALIAS);
        return cert;
    }

    private RSAPublicKey getPublicKeyFromCard(SecurityEngine securityEngine)
        throws SecurityEngineException,MXMEngineException
    {
        String holderReference = "CN=Holder,L=London,C=GB,O=Organization";
        String authReference = "CN=Issuer,L=London,C=GB,O=Organization";
        int expiryDays = 10;
        String SignOID = "SHA256withRSAEncryption";
        BigInteger serialNumber = new BigInteger("1");
        BigInteger PubExp = new BigInteger("10001", 16);
        int keyLength = 2048;
        PublicKey publicKey1 = certMngr.generateRSAKeyPair(APP_ALIAS, "auth", keyLength);
        RSAPublicKey publicKey = securityEngine.getSecurityEngineTechnologyHandler()
            .getAuthenticationManager().getPublicKey(APP_ALIAS, "auth", "12345678");
        try
        {
            String keyRepn1 = this.securityEngine.getSecurityEngineTechnologyHandler()
                .getFileAndObjectHandler().getString(publicKey1);
            String keyRepn2 = this.securityEngine.getSecurityEngineTechnologyHandler()
                .getFileAndObjectHandler().getString(publicKey);
            //assertEquals(keyRepn1, keyRepn2);
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return publicKey;
    }

    private Agent getAgent(String name,String lastName, PlaceGeographicPosition address,
        String nationality) throws Exception
    {
        MetadataEngine metadataEngine = (MetadataEngine)mxm
            .getDefaultEngine(MXMEngineName.MetadataTE.toString());
        Place place = metadataEngine.getMetadataEngineSchemaHandler().getPlace();
        place.setGeographicPosition(address);
        NameComponent nameComponent = metadataEngine.getMetadataEngineSchemaHandler()
            .getNameComponent();
        NameComponent familyName = metadataEngine.getMetadataEngineSchemaHandler().getNameComponent();
        nameComponent.setValue(name);
        familyName.setValue(lastName);
        PersonName personName = metadataEngine.getMetadataEngineSchemaHandler()
            .getPersonName();
        
        personName.addGivenName(nameComponent);
        personName.addFamilyName(familyName);
        Person person = metadataEngine.getMetadataEngineSchemaHandler().getPerson();
        person.setNationality(nationality);
        person.setAddress(place);
        person.addName(personName);
        
        return person;
    }

    private org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_mp21.DIDL getDIDL(
        DIEngine diEngine, Item item) throws MXMException
    {
        org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL didl = diEngine
            .getDIEngineSchemaHandler().getDIDL();
        didl.setMPEGMItem(item);
        return didl;
    }
}
