/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.ipmpTE.test;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBException;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.IPMPGeneralInfoDescriptor;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.LicenseCollection;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.RightsDescriptor;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Issuer;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;

public class IPMPEngineTest extends TestCase {

	private Logger logger = Logger.getLogger(IPMPEngineTest.class);
	
	private final String TEMP_FOLDER = System.getProperty("java.io.tmpdir") + File.separator;
	private final String OUTPUTIPMP_FILE_NAME = TEMP_FOLDER + "myIPMP.xml";
	private final String SOURCEIPMP_FILE_NAME = "testIPMP.xml";

	public void testIPMPParsing() throws MXMConfigurationException, FileNotFoundException, JAXBException, MXMEngineException {
		logger.info("\n\n---------- testIPMPParsing ----------");
		
		MXM mxm = MXM.getInstance(this);
		
		mxm.listEnginesAndProperties();
		
		IPMPEngine ipmp = (IPMPEngine) mxm.getDefaultEngine(MXMEngineName.IPMPTE.toString());	
		RELEngine rel = (RELEngine) mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
		
		List<MXMEngine> engines=new ArrayList();
		engines.add(rel);
		
		ipmp.setDependencies(engines);
		
		IPMPGeneralInfoDescriptor gen_desc=ipmp.getIPMPEngineSchemaHandler().getIPMPGeneralInfoDescriptor();
		
		gen_desc.parse(new MXMAdapter(new File(getClass().getClassLoader().getResource(SOURCEIPMP_FILE_NAME).getFile())));
			
		List<RightsDescriptor> rds = gen_desc.getLicenseCollection().getRightsDescriptor();
		
		for (RightsDescriptor rd:rds){
			if (rd.getLicense()!=null)
			for (License l:rd.getLicense().getLicense()){
				System.out.println("id of the embedded license is:"+l.getLicenseId());
				for (Issuer i:l.getIssuers()){
					Signature s=i.getSignature();
					//TODO support signature here
					//System.out.println("id of the embedded signature is:"+s.getId());
				}
			}
		}		
	}

	public void testIPMPGeneration() throws MXMConfigurationException, MXMEngineException {
		logger.info("\n\n---------- testIPMPGeneration ----------");
		
		MXM mxm = MXM.getInstance(this);
		mxm.listEnginesAndProperties();
		
		IPMPEngine ipmp = (IPMPEngine) mxm.getDefaultEngine(MXMEngineName.IPMPTE.toString());
		RELEngine relEngine = (RELEngine) mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
		//SecurityEngine secEngine = (SecurityEngine) mxm.getDefaultEngine(MXMEngineName.SecurityTE.toString());	
		
		List<MXMEngine> engines=new ArrayList();
		engines.add(relEngine);
		
		ipmp.setDependencies(engines);
		
		IPMPGeneralInfoDescriptor gen_desc=ipmp.getIPMPEngineSchemaHandler().getIPMPGeneralInfoDescriptor();
		LicenseCollection lc=ipmp.getIPMPEngineSchemaHandler().getLicenseCollection();
		RightsDescriptor rd=ipmp.getIPMPEngineSchemaHandler().getRightsDescriptor();
		org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.License ipmp_l =ipmp.getIPMPEngineSchemaHandler().getIPMPLicense();
		
		License l =relEngine.getRELEngineSchemaHandler().getLicense();
		Issuer i =relEngine.getRELEngineSchemaHandler().getIssuer();
		
		//Signature s=secEngine.getSecurityEngineSchemaHandler().getSignature();
		//s.setId("my_id");
		
		l.setLicenseId("1234");	
		//i.setSignature(s);
		//todo support signature here
		//l.addIssuer(i);
		
		try {
			
			ipmp_l.addLicense(l);
		
			rd.setLicense(ipmp_l);
		
			lc.setRightsDescriptor(rd);
			
			gen_desc.setLicenseCollection(lc);
				
			gen_desc.createFile(new File(OUTPUTIPMP_FILE_NAME));
			
			logger.info("The IPMP file was successfully created in file: " + OUTPUTIPMP_FILE_NAME);
			
		} catch (MXMException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}	
}