package org.iso.mpeg.mxm.mediaframeworkTE;

import java.awt.Canvas;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;
import org.gstreamer.Bus;
import org.gstreamer.BusSyncReply;
import org.gstreamer.Element;
import org.gstreamer.ElementFactory;
import org.gstreamer.Gst;
import org.gstreamer.Message;
import org.gstreamer.State;
import org.gstreamer.Structure;
import org.gstreamer.elements.PlayBin;
import org.gstreamer.event.BusSyncHandler;
import org.gstreamer.interfaces.XOverlay;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.tEngine.mediaframeworkTE.access.AccessMediaFramework;
import org.iso.mpeg.mxm.tEngine.mediaframeworkTE.exception.MediaFrameworkEngineException;

import com.sun.jna.Platform;

/**
 * @author Angelo Difino <angelo@cedeo.net>
 *
 */
public class GSTImplementation implements AccessMediaFramework {

	private static Bus bus;
	private Logger logger = Logger.getLogger(MXM.class);
	static final String name = "GSTEngineImplementation";
    String[] arguments = Gst.init("VideoPlayer", new String[]{});
    final PlayBin playbin = new PlayBin("VideoPlayer");
    //static JComponent canvas;
	
	public GSTImplementation(String[] args, Canvas canvas) 
	{
		logger.info("GST path is:"+args[1]);
		
		//TODO: use the passed parameter and load the libraries needed as in VLC scenario
		
		//NativeLibrary.addSearchPath(args[1]);
		//Native.loadLibrary(RuntimeUtil.getLibVlcLibraryName(), LibVlc.class);

		//this.canvas=canvas;

		_init(new String[]{args[0]}, canvas);
		
		logger.info("GST MediaFramework arg: " + args[0]);
	}



	@Override
	public void open(MXMObject media) throws MediaFrameworkEngineException {
        String uri=((String)media.getContent());
        logger.info("open called on:"+uri);
		if (uri.startsWith("file://"))
		{
			playbin.setInputFile(new File(uri.substring(7)));
		}
		else
			try {
				playbin.setURI(new URI((String)media.getContent()));
			} catch (URISyntaxException e) {
				throw new MediaFrameworkEngineException(e.getMessage(),e);
			}
	}
	
	@Override
	public void play() throws MediaFrameworkEngineException{
		logger.debug("play called");		
		SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                playbin.setState(State.PLAYING);
            }
        });
	}
	
	@Override
	public void stop()throws MediaFrameworkEngineException {
		logger.debug("stop called");	
		SwingUtilities.invokeLater(new Runnable() {
            public void run() {
            	playbin.stop();
            }
        });
	}

	@Override
	public void pause() throws MediaFrameworkEngineException {
		SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                playbin.setState(State.PAUSED);
            }
        });
	}

	@Override
	public void release() throws MediaFrameworkEngineException {
		SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                playbin.setState(State.NULL);
            }
        });
	}

	@Override
	public void setVolume(int volume) throws MediaFrameworkEngineException {
		final long newvolume=volume/200; //gstream accept value between 0 and 1
		
		if ((newvolume<0)||(newvolume>200)){
			logger.error("Volume not valid:"+newvolume+". Accepted values are between 0 and 200");
				throw new MediaFrameworkEngineException("Volume not valid:"+newvolume+". Accepted values are between 0 and 200");
		}
		
		SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                playbin.setVolume(newvolume);
            }
        });
	}
	
	private void _init(String args[], Canvas c) {
		
		final Canvas newc;
		
		if (c==null){
			newc = new Canvas();
			newc.setBackground(java.awt.Color.black);
			newc.setVisible(true);
			logger.debug("passed jcomponent is null. Creating an ad-hoc JFRAME");
			JFrame frame = new JFrame("dummy jframe");
			frame.add(newc);
			frame.setLocation(100, 100);
			frame.setSize(1050, 600);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			frame.setVisible(true);
			frame.pack();
		}
		else newc=c;
		
		
		
		bus = playbin.getBus();
		final String overlayFactory = Platform.isWindows() ? "directdrawsink" : "xvimagesink";
        //final VideoComponent videoComponent = new VideoComponent();
        //playbin.setVideoSink(videoComponent.getElement());
		final Element videoSink = ElementFactory.make(overlayFactory, "overlay video sink");
		playbin.setVideoSink(videoSink);
		if (!Platform.isWindows()) { 
            bus.setSyncHandler(new BusSyncHandler() {

                public BusSyncReply syncMessage(Message msg) {
                    Structure s = msg.getStructure();
                    if (s == null || !s.hasName("prepare-xwindow-id")) {
                        return BusSyncReply.PASS;
                    }
                    XOverlay.wrap(videoSink).setWindowID(newc);
                    
                    return BusSyncReply.DROP;
                }

		
            });
        } else {
            XOverlay.wrap(videoSink).setWindowID(c);
        } 
//        if (c==null)
//        {
//			logger.debug("passed jcomponent is null. Creating an ad-hoc JFRAME");
//        	JFrame frame = new JFrame("VideoPlayer");
//        	frame.getContentPane().add(videoComponent, BorderLayout.CENTER);
//        	frame.setPreferredSize(new Dimension(640, 480));
//        	frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
//        	frame.pack();
//        	frame.setVisible(true);
//        }
//        else 
//        {
//        	c.add(videoComponent);
//        }
	}



	@Override
	public boolean consumeResource(List<MXMObject> protectedTools,
			InputStream is, String mediaType, OutputStream os, MXMObject useType)
			throws MediaFrameworkEngineException {
		// TODO Auto-generated method stub
		return false;
	}
}