package org.iso.mpeg.mxm.metadataTE.technologyhandler;

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.basex.core.BaseXException;
import org.basex.core.Context;
import org.basex.core.cmd.Add;
import org.basex.core.cmd.Check;
import org.basex.core.cmd.CreateDB;
import org.basex.core.cmd.Delete;
import org.basex.core.cmd.DropDB;
import org.basex.core.cmd.InfoDB;
import org.basex.core.cmd.Open;
import org.basex.core.cmd.RepoInstall;
import org.basex.core.cmd.Set;
import org.basex.core.cmd.XQuery;
import org.basex.io.IO;
import org.basex.io.serial.Serializer;
import org.basex.query.QueryException;
import org.basex.query.QueryProcessor;
import org.basex.query.iter.Iter;
import org.basex.query.value.item.Item;
import org.basex.util.InputInfo;
import org.basex.util.InputParser;


public class MetadataRepository implements org.iso.mpeg.mxm.tEngine.metadataTE.technologyhandler.MetadataRepository
{
	public static final String DESCRIBE_CONTENT_XML_DATABASE_NAME = "DescribeContentXmlDb";  
	
	// TODO normally you should have a db for each metadata schema describe content ES supports. 
	
	private Context xmlDbContext;
	

	public MetadataRepository()
	{
		
	}
	
	public Map<String,String> getXmlDescription(String contentId, String xPath)
	{
		try
		{
			String query =  "import module namespace functx = 'http://www.functx.com'; " +
							"let $doc := collection('" + DESCRIBE_CONTENT_XML_DATABASE_NAME + "/" + contentId + "') " +
//							"return ($doc"+xPath+"/string-join(ancestor-or-self::*/name(.), '/') , $doc"+xPath +")";
//							"return (functx:path-to-node-with-pos($doc"+xPath+") , $doc"+xPath +")";
							"for $y in $doc"+xPath + " " +
							"return (functx:path-to-node-with-pos($y), $y)";
			System.out.println("QUERY: " + query);
			
			QueryProcessor proc = new QueryProcessor(query, xmlDbContext);
		    
//			System.out.println(proc.execute());
			Iter iter = proc.iter();
		    
		    List<String> results = new ArrayList<String>();
		    
		    for(Item item; (item = iter.next()) != null;) 
		    {
		    	ByteArrayOutputStream baos = new ByteArrayOutputStream();
		    	Serializer ser = proc.getSerializer(baos);
		    	item.serialize(ser);
		    	ser.close();
		    	
		    	results.add(baos.toString());
		    }
			
		    proc.close();
		    
		    System.out.println("RESULTS (POS,XML): " + results);
		    
		    Map<String,String> xPath2node = new HashMap<String,String>();
		    
		    for (int i=0; i<results.size()-1; i = i+2)
		    {
		    	xPath2node.put("/" + results.get(i), results.get(i+1));
		    }
		    
		    return xPath2node;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	@Override
	public boolean replaceXmlDescriptionNode(String contentId, String xPath,
			String inputNode)
	{
		try
		{
		   	if (documentExists(contentId))
		   	{
		   		String query = "let $x := collection('" + DESCRIBE_CONTENT_XML_DATABASE_NAME + "/" + contentId + "') " +
		   				"return replace node $x" + xPath + " with " + inputNode;

		   		System.out.println("QUERY: " + query);

		   		new XQuery(query).execute(xmlDbContext);

		   		// new XQuery("replace node collection('" + DESCRIBE_CONTENT_XML_DATABASE_NAME + "/" + contentId + "')" + xPath + " with " + inputNode).execute(xmlDbContext);
		   	}
		   	else
		   	{
		   		System.err.println("Document " + contentId + " does not exist.");
		   	}
		   	
		    return true;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return false;
	}

	@Override
	public boolean addXmlDescriptionNode(String contentId, String xPath,
			String inputNode)
	{
		try
		{
		   	if (documentExists(contentId))
		   	{
		   		String query = "let $x := collection('" + DESCRIBE_CONTENT_XML_DATABASE_NAME + "/" + contentId + "') " +
		   				"return insert node " + inputNode + "into " + xPath;

		   		System.out.println("QUERY: " + query);

		   		new XQuery(query).execute(xmlDbContext);
		   	}
		   	else // create doc
		   	{
		   		System.out.println("Adding document " + contentId + " " + inputNode);
		   			
		   		new Add(contentId,inputNode).execute(xmlDbContext);
		   	}
		   	
		    return true;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return false;
	}

	@Override
	public boolean removeXmlDescriptionNode(String contentId, String xPath)
	{
		try
		{
		   	if (documentExists(contentId))
		   	{
		   		String query = "let $x := collection('" + DESCRIBE_CONTENT_XML_DATABASE_NAME + "/" + contentId + "') " +
		   				"return delete node $x" + xPath;

		   		System.out.println("QUERY: " + query);

		   		new XQuery(query).execute(xmlDbContext);
		   	}
		   	else // create doc
		   	{
		   		System.err.println("Document " + contentId + " does not exist.");
		   	}
		   	
		    return true;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return false;
	}

	@Override
	public boolean deleteXmlDescription(String contentId)
	{
		try
		{
		   	if (documentExists(contentId))
		   	{
		   		System.out.println("Deleting document " + contentId);

		   		new Delete(contentId).execute(xmlDbContext);
		   	}
		   	else // create doc
		   	{
		   		System.err.println("Document " + contentId + " does not exist.");
		   	}
		   	
		    return true;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return false;
	}
	
	private boolean documentExists(String contentId) throws QueryException
	{
		String query = "let $x := db:is-xml('" + DESCRIBE_CONTENT_XML_DATABASE_NAME + "', '" + contentId + "') " + 
				   "return $x ";
	
		QueryProcessor proc = new QueryProcessor(query, xmlDbContext);

		boolean docExists = "true".equals(proc.execute().toString());

		proc.close();
		
		return docExists;
	}
	
	public void initRepository(boolean cleanUp)
	{
		try
		{
			if (xmlDbContext!=null)
			{
				xmlDbContext.close();
			}
			
			xmlDbContext = new Context();
			
			new Set("intparse", true).execute(xmlDbContext);
			
			if (cleanUp)
			{
				try
				{
					new DropDB(DESCRIBE_CONTENT_XML_DATABASE_NAME).execute(xmlDbContext);
				} 
			    catch (BaseXException e)
				{
			    	System.out.println(DESCRIBE_CONTENT_XML_DATABASE_NAME + " database does not exist. Trying to create it...");
				}
				
				try
				{
					new CreateDB(DESCRIBE_CONTENT_XML_DATABASE_NAME).execute(xmlDbContext);
				} 
			    catch (BaseXException e)
				{
					e.printStackTrace();
				}
			}
			
			try
			{
				new Check(DESCRIBE_CONTENT_XML_DATABASE_NAME).execute(xmlDbContext);
			}
			catch(Exception ex)
			{
				new CreateDB(DESCRIBE_CONTENT_XML_DATABASE_NAME).execute(xmlDbContext);
				new Open(DESCRIBE_CONTENT_XML_DATABASE_NAME).execute(xmlDbContext);
			}
			
			System.out.print(new org.basex.core.cmd.List().execute(xmlDbContext));
			
//			System.out.print(new InfoDB().execute(xmlDbContext));
			
			boolean moduleExists = new XQuery("repo:list()").execute(xmlDbContext).contains("http://www.functx.com");

			if (!moduleExists)
			{
				InputStream is = getClass().getClassLoader().getResourceAsStream("functx.xq");
				
				String tmpFile = System.getProperty("java.io.tmpdir") + "/functx.xq";
				
				FileOutputStream fos = new FileOutputStream(tmpFile);
				
				while (is.available()>0)
				{
					fos.write(is.read());
				}
				
				is.close();
				fos.close();
				
				System.out.print(new RepoInstall(tmpFile, null).execute(xmlDbContext));
				
//				System.out.print(new RepoInstall("jar:C:/Work/mxm-metadataTE/0.3.1/mxm-metadataTE-0.3.1.jar!/functx.xq", null).execute(xmlDbContext));
//				System.out.print(new RepoInstall(getClass().getClassLoader().getResource("functx-1.0.xar").getFile(),null).execute(xmlDbContext));
//				System.out.print(new XQuery("repo:install('" + getClass().getClassLoader().getResource("functx-1.0.xar").getFile() + "')").execute(xmlDbContext));
			}

//		    System.out.println(new XQuery("for $doc in collection() return base-uri($doc)").execute(xmlDbContext));
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	public List<String> queryXMLDatabase(String xQuery)
	{
		return null;
	}
	
//	public void storeRDFFragmentToDatabase()
//	{
//		
//	}
	
//	public void queryRDFDatabase(String sparql)
//	{
//		
//	}
	
	// search for documents with description = XQuery(return base-uri(doc) that match description)
	
//	public static void main (String[] args) throws Exception
//	{
//		MetadataRepository rep = new MetadataRepository();
//		rep.getXmlDescription("aaa", "//h2/text()");
////		rep.setXmlDescription("aaa", ADD_NODE, "", "<a><h2>Awesome</h2><h2>Oh Yeah!</h2></a>");
////		rep.setXmlDescription("aaa", ADD_NODE, "/a/h2[last()]", "<h2>Sweet</h2>");
////		rep.setXmlDescription("aaa", ADD_NODE, "/a", "<h2>Bitchin</h2>");
////		rep.setXmlDescription("aaa", REPLACE_NODE, "/a/*[last()]", "<h2>Awesome 2</h2>");
////		rep.setXmlDescription("aaa", DELETE_NODE, "/a/*[last()]", "");
////		rep.setXmlDescription("aaa", RESET_DESCRIPTION, "", "");
////		rep.getXmlDescription("aaa", "/");
//	}

}
