/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.metadataTE.util;



public enum MIMETypeMap {

	// -----------------
	// ---- CONTENT ---- 
	// -----------------
	dcf("application/mp21"), 
	dci("application/mp21"), 
	xml("application/mp21"), 

	// -----------------
	// ----- VIDEO ----- 
	// -----------------
	mp4("video/mp4"), 

	mov("video/quicktime"), 
	qt("video/quicktime"), 
	
	mpg("video/mpeg"), 
	mpeg("video/mpeg"), 
	mp2("video/mpeg"), 
	mpa("video/mpeg"), 
	
	avi("video/avi"), 
	
	flv("video/x-flv"),
	
	// -----------------
	// ----- AUDIO ----- 
	// -----------------
	mp3("audio/mpeg"), 

	ogg("application/ogg"), 
	
	wav("audio/wav"),
		
	// -----------------
	// ----- IMAGES ---- 
	// -----------------
	jpg("image/jpeg"),
	jpeg("image/jpeg"),
	jpe("image/jpeg"),
	
	bmp("image/bmp"),
	
	gif("image/gif"),

	png("image/png");
	
	// ----------------
	
	private final String mimeType;

	private MIMETypeMap(String mimeType) {
		this.mimeType = mimeType;
	}

	public String getValue() {
		return this.mimeType;
	}

	public static String getMimeType(String filename) {
		String fileExtension = filename.substring(filename.lastIndexOf('.') + 1, filename.length());
		MIMETypeMap[] types = MIMETypeMap.values();
		for (int i = 0; i < types.length; i ++) {
			if (types[i].toString().equalsIgnoreCase(fileExtension))
					return types[i].getValue();
		}
		return null;
	}
	
	public static String getFileExtension(String mimeType) {
		MIMETypeMap[] types = MIMETypeMap.values();
		for (int i = 0; i < types.length; i ++) {
			if (types[i].getValue().equalsIgnoreCase(mimeType))
					return types[i].toString();
		}
		return null;
	}

}
