package org.iso.mpeg.mxm.metadataTE.test;

import java.io.File;
import java.io.IOException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;

import org.apache.log4j.Logger;

import org.iso.mpeg.mxm.engine.metadataTE.schema.MetadataEngineContext;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngineSchemaHandler;
import org.iso.mpeg.mxm.tEngine.metadataTE.exceptions.MetadataEngineException;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.ClassificationScheme;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.ClassificationSchemeDescription;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.CompleteDescription;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Mpeg7;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.TermDefinition;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.TermDefinitionTerm;


import junit.framework.TestCase;

public class MPEG7ClassificationSchemaTest extends TestCase {

	private JAXBContext jc = null;
	private Marshaller m = null;	

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		jc = MetadataEngineContext.getInstance().getJAXBContext();
		m = jc.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);		
		m.setProperty("com.sun.xml.bind.namespacePrefixMapper", SchemaNamespacePrefixMapperImpl.getInstance());
	}

	private Logger logger = Logger.getLogger(MPEG7ClassificationSchemaTest.class);
	private final String SOURCE_MPEG7_FILE_NAME = "ContentCommercialCS.xml";
	private final String TEMP_FOLDER = System.getProperty("java.io.tmpdir") + File.separator;
	private final String OUTPUT_MPEG7_FILE_NAME = "target" + File.separator + "mpeg7CS.xml";
	private final String OUTPUT_MPEG7_CONTENT_FILE_NAME = TEMP_FOLDER + "mpeg7Content.xml";


	public void testMetadataCreation() throws IOException, MXMException {
		logger.info("\n\n---------- testMPEG7Creation ----------");
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		MetadataEngine metadataEngine = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
		MetadataEngineSchemaHandler metadataEngineSchemaHandler = metadataEngine.getMetadataEngineSchemaHandler();	
		
		/*
		<Term termID="3.50">
			<Name xml:lang="en">Commercial/Products</Name>
			<Term termID="3.50.1">
				<Name xml:lang="en">Agriculture, forestry and fishery products</Name>
				<Term termID="3.50.1.1">
					<Name xml:lang="en">Products of agriculture, horticulture and market gardening</Name>
				</Term>
			</Term>
		</Term>
		*/
		
		TermDefinitionTerm term35011 = metadataEngineSchemaHandler.getTermDefinitionTerm();
		term35011.setTermID("3.50.1.1");
		org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.TermDefinitionBaseName name = metadataEngineSchemaHandler.getTermDefinitionBaseName();
		name.setPreferred(true);
		name.setValue("Products of agriculture, horticulture and market gardening");
		term35011.getName().add(name);
		
		TermDefinitionTerm term3501 = metadataEngineSchemaHandler.getTermDefinitionTerm();
		term3501.setTermID("3.50.1");
		name = metadataEngineSchemaHandler.getTermDefinitionBaseName();
		name.setPreferred(true);
		name.setValue("Agriculture, forestry and fishery products");
		term3501.getName().add(name);
		term3501.getTerm().add(term35011);
		
		TermDefinitionTerm term350 = metadataEngineSchemaHandler.getTermDefinitionTerm();
		term350.setTermID("3.50");
		name = metadataEngineSchemaHandler.getTermDefinitionBaseName();
		name.setPreferred(true);
		name.setValue("Commercial/Products");
		term350.getName().add(name);
		term350.getTerm().add(term3501);
				
		ClassificationScheme classificationScheme = metadataEngineSchemaHandler.getClassificationScheme();
		classificationScheme.getTerm().add(term350);
		classificationScheme.setUri("urn:tva:metadata:cs:ContentCommercialCS:2005-03");
		
		ClassificationSchemeDescription classificationSchemeDescription = metadataEngineSchemaHandler.getClassificationSchemeDescription();
		
		classificationSchemeDescription.getClassificationScheme().add(classificationScheme);

		Mpeg7 mpeg7smp = metadataEngineSchemaHandler.getMpeg7();
		
		mpeg7smp.getDescription().add(classificationSchemeDescription);
		
		File outputMpeg7 = new File(OUTPUT_MPEG7_FILE_NAME);
		
		mpeg7smp.createFile(outputMpeg7);
		
	}
	

	public void testMpeg7Parsing() throws Exception {
		logger.info("\n\n---------- testMpeg7Parsing ----------");

		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();

		MetadataEngine metadataEngine = (MetadataEngine)mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
		MetadataEngineSchemaHandler metadataEngineSchemaHandler = metadataEngine.getMetadataEngineSchemaHandler();	
		
		Mpeg7 mpeg7smp = metadataEngineSchemaHandler.getMpeg7();
		
		MXMEngineResponse response = mpeg7smp.parse(new MXMAdapter(new File(getClass().getClassLoader().getResource(SOURCE_MPEG7_FILE_NAME).getFile())));
		
		assertTrue(response.equals(MXMEngineResponse.SUCCESS));
		
		for (CompleteDescription completeDescription : mpeg7smp.getDescription()) {
			if( completeDescription instanceof ClassificationSchemeDescription ){
				ClassificationSchemeDescription classificationSchemeDescription = (ClassificationSchemeDescription) completeDescription;
				for (ClassificationScheme classificationScheme : classificationSchemeDescription.getClassificationScheme()) {
					for (TermDefinition termDefinition : classificationScheme.getTerm()) {
						logger.info("Term   id: " + termDefinition.getTermID());
						for (org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.TermDefinitionBaseName name : termDefinition.getName()) {
							logger.info("Term name: " + name.getValue());
						}
						for (TermDefinition term : termDefinition.getTerm()) {
							printTerm(term,"\t");
						}
					}
				}
			}
		}
		logger.info("\n\nThe MPEG7 file: " + SOURCE_MPEG7_FILE_NAME + " was successfully parsed.");
	}

	private void printTerm(TermDefinition term, String offset) throws MetadataEngineException {
		
		logger.info(offset + "Term   id: " + term.getTermID());
		for (org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.TermDefinitionBaseName name : term.getName()) {
			logger.info(offset +"Term name: " + name.getValue());
		}
		for (TermDefinition innerTerm : term.getTerm()) {
			printTerm(innerTerm,offset + "\t");
		}
	}

}
