/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.metadataTE.test;

//usefull import
import java.io.File;
import java.io.IOException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.engine.metadataTE.schema.MetadataEngineContext;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngineSchemaHandler;
import org.iso.mpeg.mxm.tEngine.metadataTE.exceptions.MetadataParsingException;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.CompleteDescription;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Creation;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.CreationDescription;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.CreationInformation;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Mpeg7;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.TextAnnotation;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Textual;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Title;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;

public class MetadataEngineTest extends TestCase {

	private JAXBContext jc = null;
	private Marshaller m = null;

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		jc = MetadataEngineContext.getInstance().getJAXBContext();
		m = jc.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		m.setProperty("com.sun.xml.bind.namespacePrefixMapper", SchemaNamespacePrefixMapperImpl.getInstance());
	}

	private Logger logger = Logger.getLogger(MetadataEngineTest.class);

	private final String OUTPUT_MPEG7_FILE_NAME = "target" + File.separator + "mpeg7.xml";
	private final String SOURCE_MPEG7_FILE_NAME = "mpeg7-simple.xml"; //"testMpeg7.xml";
	private final String SOURCE_MEDIA_RESOURCE_FILE_NAME = "chillout_small.jpg";
	private final String COPYRIGHT = "\u00a9";
	private final String REGISTERED = "\u00ae";

	public void testMpeg7Parsing() throws Exception {
		
		logger.info("\n\n---------- test Mpeg7 Parsing ----------");
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		MetadataEngine contentMetadataEngine = (MetadataEngine) mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
		Mpeg7 mpeg7Parser = contentMetadataEngine.getMetadataEngineSchemaHandler().getMpeg7();
		
		File file = new File(getClass().getClassLoader().getResource(SOURCE_MPEG7_FILE_NAME).getFile());
		
		MXMEngineResponse response = mpeg7Parser.parse(new MXMAdapter(file));
		
		if (! response.equals(MXMEngineResponse.SUCCESS)) {
			throw new MetadataParsingException("Unable to parse the Mpeg7 file: " + SOURCE_MPEG7_FILE_NAME);				
		}
		else {
			for (CompleteDescription completeDescription : mpeg7Parser.getDescription())
			{			
				if (completeDescription != null) {
					if( completeDescription instanceof CreationDescription ){
						CreationDescription cd = (CreationDescription) completeDescription;
						
							//Titles
							for (CreationInformation ci:cd.getCreationInformation())
							{
						
								for (Title title : ci.getCreation().getTitle()) {
									logger.info("Title: " + title.getValue());
								}
								
								for (TextAnnotation ta : ci.getCreation().getAbstract()) {
									for (Textual t:ta.getFreeTextAnnotation()){
										logger.info("ABS: " + t.getValue());
									}
								}
							}
//							
//							//TitleMedia
//							TitleMedia titleMedia = video.getVideo().getCreationInformation().getCreation().getTitleMedia();
//							String uri = titleMedia.getTitleImage().getMediaUri();
//							logger.info("A TitleImage uri: " + uri);		
//							
//							//Abstracts
//							for (TextAnnotation textAnnotation : video.getVideo().getCreationInformation().getCreation().getAbstract()) {
//								logger.info("Got a new Abstract!");
//								for (Textual textual : textAnnotation.getFreeTextAnnotation()) {
//									logger.info("-- FreeTextAnnotation: " + textual.getValue());
//									//									logger.info("-- FreeTextAnnotation language: " + textual.getLang());
//								}
//								for (StructuredAnnotation structuredAnnotation : textAnnotation.getStructuredAnnotation()) {
//									//									logger.info("-- StructuredAnnotation -- Language: " + structuredAnnotation.getLang());					
//								}
//							}
//							
//							//Creators
//							for (Creator creator : video.getVideo().getCreationInformation().getCreation().getCreator()) {
//								logger.info("Got a new Actor!");
//								if( creator.getAgent() instanceof Person){
//									Person person = (Person) creator.getAgent();
//									for (PersonName personName : person.getName()) {
//										logger.info("-- First name: " + personName.getGivenName().get(0).getValue());
//										logger.info("-- Last name: " + personName.getFamilyName().get(0).getValue());
//									}
//									logger.info("-- Role: " + creator.getRole().getHref());
//								}
//							}
//							
//							//CreationCoordinates
//							//							for (CreationCoordinates creationCoordinate : video.getVideo().getCreationInformation().getCreation().getCreationCoordinates()) {
//							//								logger.info("Got a new Creation Coordinates!");					
//							//								logger.info("-- Time point: " + creationCoordinate.getDate().getTimePoint());
//							//								
//							//								Place location = creationCoordinate.getLocation();
//							//								if (location.getGeographicPosition() != null) {
//							//									logger.info("-- Place datum: " + location.getGeographicPosition().getDatum());
//							//									logger.info("-- Place latitude: " + location.getGeographicPosition().getPoint().getLatitude());
//							//									logger.info("-- Place longitude: " + location.getGeographicPosition().getPoint().getLongitude());
//							//									logger.info("-- Place altitude: " + location.getGeographicPosition().getPoint().getAltitude());
//							//								}	
//							//								logger.info("-- Place Id: " + location.getId());
//							//								logger.info("-- Place lang: " + location.getLang());
//							//								
//							//								if (location.getName() != null) {
//							//									for (Textual name : location.getName()) {
//							//										logger.info("Location name: " + name.getValue());
//							//										logger.info("Location lang: " + name.getLang());
//							//									}
//							//								}
//							//								
//							//								if (location.getRegion() != null) {
//							//									for (String region : location.getRegion())
//							//										logger.info("Region: " + region);
//							//								}
//							//								
//							//								if (location.getRole() != null) {
//							//									logger.info("Role hRef: " + location.getRole().getHref());
//							//									if (location.getRole().getName() != null) {
//							//										for (org.iso.mpeg.mxm.engine.metadataTE.Name name : location.getRole().getName()) {
//							//											logger.info("Role name: " + name.getValue());
//							//											logger.info("Role lang: " + name.getLang());
//							//										}
//							//									}
//							//									if (location.getRole().getDefinition() != null) {
//							//										for (Textual name : location.getRole().getDefinition()) {
//							//											logger.info("Role definition name: " + name.getValue());
//							//											logger.info("Role definition lang: " + name.getLang());
//							//										}
//							//									}
//							//								}
//							//							}	
//							
//							//CopyrightString
//							for (Textual textual : video.getVideo().getCreationInformation().getCreation().getCopyrightString()) {
//								logger.info("Got a new copyrightString!");
//								logger.info("CopyrightString: " + textual.getValue());					
//									//								logger.info("CopyrightString language: " + textual.getLang());
//							}
//							
//							//Genres
//							for (ClassificationGenre genre : video.getVideo().getCreationInformation().getClassification().getGenre()) {
//								logger.info("Got a genre!");
//								for (InlineTermDefinitionName name : genre.getName()) {
//									logger.info("genre name: " + name.getValue());					
//										//									logger.info("genre language: " + name.getLang());	
//									logger.info("genre isPreferred: " + (name.isPreferred() ? "true" : "false"));					
//								}
//								logger.info("genre href: " + genre.getHref());					
//								logger.info("genre type: " + genre.getType());	
//							}
//								
//							//Parental guidance
//							for (ParentalGuidance parentalGuidance : video.getVideo().getCreationInformation().getClassification().getParentalGuidance()) {
//								logger.info("Got a ParentalGuidance!");
//								if (parentalGuidance.getParentalRating() != null) {
//									ControlledTermUse parentalRating = parentalGuidance.getParentalRating();
//									for (InlineTermDefinitionName object : parentalGuidance.getParentalRating().getName()) {
//										logger.info("parentalGuidance name: " + object.getValue());					
//											//										logger.info("parentalGuidance language: " + object.getLang());	
//									}
//									for (Textual object : parentalGuidance.getParentalRating().getDefinition()) {
//										logger.info("parentalGuidance definition name: " + object.getValue());					
//										//										logger.info("parentalGuidance definition language: " + object.getLang());	
//									}
//									logger.info("parentalGuidance href: " + parentalGuidance.getParentalRating().getHref());
//								}	
//							}
//						}
					}
				}
			}	
		}
		logger.info("\n\nThe MPEG7 file: " + SOURCE_MPEG7_FILE_NAME + " was successfully parsed.");
	}

	public void testMetadataCreation() throws IOException, MXMException {
		logger.info("\n\n---------- testMPEG7Creation ----------");

		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();

		MetadataEngine metadataEngine = (MetadataEngine) mxm.getDefaultEngine(MXMEngineName.MetadataTE.toString());
		MetadataEngineSchemaHandler metadataEngineSchemaHandler = metadataEngine.getMetadataEngineSchemaHandler();
		
		CreationDescription cd= metadataEngineSchemaHandler.getCreationDescription();
		
		CreationInformation creationInformation = metadataEngineSchemaHandler.getCreationInformation();
		
		Creation creation = metadataEngineSchemaHandler.getCreation();
		
		TextAnnotation ta = metadataEngineSchemaHandler.getTextAnnotation();

		// Create titles
		Title title = metadataEngineSchemaHandler.getTitle();
		title.setValue("The BIG Lebowsky");
		creation.addTitle(title);
		
		Textual abs = metadataEngineSchemaHandler.getTextual();
		abs.setValue("desciption of work");
		
		ta.addFreeTextAnnotation(abs);
		
		creation.addAbstract(ta);
		
		creationInformation.setCreation(creation);
	
//		creationCreator.getCopyrightString().add(copyrightString);
		
//		Title italianTitle = metadataEngineSchemaHandler.getTitle();
//		//		italianTitle.setLang("it");
//		italianTitle.setValue("Il grande Lebowsky");
//		italianTitle.setType("popular");
//		creationCreaTitle> getTitle() {
//		List<Title> titleList = new ArrayList<Title>();
//		
//		for (TitleType titleType: creationType.getTitle())
//		{
//			if (titleType instanceof org.iso.mpeg.mxm.engine.metadataTE.schema.mpeg7_v3.TitleType)
//			{
//				titleList.add(new org.iso.mpeg.mxm.metadataTE.schemahandler.Title((TitleType) titleType));
//			}
//			//ADDMORE
//		}
//		return titletor.getTitle().add(italianTitle);
//
//		// Create a titlemedia
//		TitleMedia titleMediaCreator = metadataEngineSchemaHandler.getTitleMedia();
//		ImageLocator imageLocatorCreator = metadataEngineSchemaHandler.getImageLocator();
//		imageLocatorCreator.setMediaUri(getClass().getClassLoader().getResource(SOURCE_MEDIA_RESOURCE_FILE_NAME).getFile());
//		titleMediaCreator.setTitleImage(imageLocatorCreator);
//		creationCreator.setTitleMedia(titleMediaCreator);
//		
//		// Create an abstract
//		TextAnnotation abst = metadataEngineSchemaHandler.getTextAnnotation();
//		Textual textualCreator = metadataEngineSchemaHandler.getTextual();
//		//		textualCreator.setLang("en");
//		textualCreator.setValue("Her life was in their hands. Now her toe is in the mail.");
//		abst.addFreeTextAnnotation(textualCreator);
//		creationCreator.getAbstract().add(abst);
//
		// Create some creators
//		Creator creator1 = metadataEngineSchemaHandler.getCreator();
//		
//		Person person = metadataEngineSchemaHandler.getPerson();
//		
//		PersonName personName = metadataEngineSchemaHandler.getPersonName();
//		
//		NameComponent name = metadataEngineSchemaHandler.getNameComponent();
//		name.setValue("Jeff");
//		personName.addGivenName(name);
//		
//		name = metadataEngineSchemaHandler.getNameComponent();
//		name.setValue("Bridges");
//		personName.addFamilyName(name);
//		
//		person.addName(personName);
//		
//		creator1.setAgent(person);
//		
//		ControlledTermUse role = metadataEngineSchemaHandler.getControlledTermUse();
//		role.setHref(CreatorRole.actor.getValue());
//		
//		creator1.setRole(role);
//		
//		creationCreator.addCreator(creator1);
		
//		creator1 = metadataEngineSchemaHandler.getCreator();
//		personName = metadataEngineSchemaHandler.getPersonName();
//		
//		name = metadataEngineSchemaHandler.getNameComponent();
//		name.setValue("John");
//		personName.addGivenName(name);
//		
//		name = metadataEngineSchemaHandler.getNameComponent();
//		name.setValue("Goodman");
//		personName.addFamilyName(name);
//		
//		person.addName(personName);
//		
//		creator1.setAgent(person);
//		
//		role = metadataEngineSchemaHandler.getControlledTermUse();
//		role.setHref(CreatorRole.actor.getValue());
//		
//		creator1.setRole(role);
//		
//		creationCreator.addCreator(creator1);
//		
//		creator1 = metadataEngineSchemaHandler.getCreator();
//		
//		personName = metadataEngineSchemaHandler.getPersonName();
//		
//		name = metadataEngineSchemaHandler.getNameComponent();
//		name.setValue("Joel");
//		personName.addGivenName(name);
//		
//		name = metadataEngineSchemaHandler.getNameComponent();
//		name.setValue("Coen");
//		personName.addFamilyName(name);
//		
//		person.addName(personName);
//		
//		creator1.setAgent(person);
//		
//		role = metadataEngineSchemaHandler.getControlledTermUse();
//		role.setHref(CreatorRole.director.getValue());
//		
//		creator1.setRole(role);
//		
//		creationCreator.addCreator(creator1);
//
//		// Create creationCoordinates
//		//		CreationCoordinates creationCoordinates = metadataEngine.getCreationCoordinatesCreator();
//		//		Time timeCreator = metadataEngine.getTimeCreator();
//		//		timeCreator.setTimePoint("1998-05-01T00:00:00");
//		//		creationCoordinates.setDate(timeCreator);
//		//
//		//		Place place = metadataEngine.getPlaceCreator();
//		//		Textual shootingPlace = metadataEngine.getTextualCreator();
//		//		shootingPlace.setLang("en");
//		//		shootingPlace.setValue("Beverly Hills");
//		//		place.addName(shootingPlace);
//		//		place.addRegion("US-CA");
//		//
//		//		TermUse placeRole = metadataEngine.getTermUseCreator();
//		//		org.iso.mpeg.mxm.engine.metadataTE.Name roleName = metadataEngine.getNameCreator(true, "real location", "en");
//		//		placeRole.addName(roleName);
//		//		place.setRole(placeRole);
//		//
//		//		creationCoordinates.setLocation(place);
//		//		creationCreator.addCreationCoordinates(creationCoordinates);
//
//		// Create a copyrightString
//		Textual copyrightString = metadataEngineSchemaHandler.getTextual();
//		//		copyrightString.setLang("en");
//		copyrightString.setValue(COPYRIGHT + " 1997 PolyGram Filmed Enterteinment Distribution, Inc.");
//		creationCreator.getCopyrightString().add(copyrightString);
//		
//		creationInformationCreator.setCreation(creationCreator);
//		
//		// Create a Genre
//		Classification classificationCreator = metadataEngineSchemaHandler.getClassification();
//		ClassificationGenre genre = metadataEngineSchemaHandler.getClassificationGenre();
//		genre.setHref("urn:mpeg:mpeg7:cs:GenreCS:2001:2.1.2");
//
//		InlineTermDefinitionName name1 = metadataEngineSchemaHandler.getInlineTermDefinitionName();  
//		name1.setPreferred(true);
//		name1.setValue("Comedy");
//		genre.getName().add(name1);
//
//		InlineTermDefinitionName name2 = metadataEngineSchemaHandler.getInlineTermDefinitionName();  
//		name1.setPreferred(true);
//		name1.setValue("Crime");
//		genre.getName().add(name2);
//		
//		classificationCreator.getGenre().add(genre);
//
//		// Create the ParentalGuidance
//		ParentalGuidance parental = metadataEngineSchemaHandler.getParentalGuidance();
//		
//		ControlledTermUse parentalRating = metadataEngineSchemaHandler.getControlledTermUse();
//		parentalRating.setHref(MPAAParentalRating.R.getValue());
//		
//		parental.setParentalRating(parentalRating);
//		
//		classificationCreator.getParentalGuidance().add(parental);
//		
//		creationInformationCreator.setClassification(classificationCreator);
//		
//		VideoSegment videoSegmentCreator = metadataEngineSchemaHandler.getVideoSegment();
//		
//		videoSegmentCreator.setCreationInformation(creationInformationCreator);
//		
//		Video video = metadataEngineSchemaHandler.getVideo();
//		
//		video.setVideo(videoSegmentCreator);
//		
//		ContentEntity contentEntityCreator = metadataEngineSchemaHandler.getContentEntity();
//		
//		contentEntityCreator.getMultimediaContent().add(video);
		
		cd.addCreationInformation(creationInformation);
		
		// Create the Mpeg7 element
		Mpeg7 mpeg7smp = metadataEngineSchemaHandler.getMpeg7();
		mpeg7smp.addDescription(cd);
		
		File outputMpeg7 = new File(OUTPUT_MPEG7_FILE_NAME);
		
		mpeg7smp.createFile(outputMpeg7);
	}
}