/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s): Filippo Chiariglione
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.mpeg21ffTE;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.tEngine.mp21ffTE.exceptions.MPEG21FileCreationException;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.FileTypeBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.HandlerBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.IBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.ItemInfo;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.ItemLocationBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.MediaDataBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.MediaDataInfo;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.MetaBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.XMLBox;
import org.iso.mpeg.mxm.mpeg21ffTE.constants.BoxType;
import org.iso.mpeg.mxm.mpeg21ffTE.dcf.DCF;

/**
 * This class provides user methods to create a dmp content file
 * 
 * @author Xueqing Liu
 * 
 */
public class DCFCreator implements org.iso.mpeg.mxm.tEngine.mp21ffTE.MPEG21FileCreator {
	/**
	 * DCF is a locgical container who will hold various boxes.
	 * The DCFMaker will create out a dmp content file from this DCF.
	 */
	public DCF dcf = null;
	private ArrayList<MediaDataInfo> mediaDataInfos=new ArrayList<MediaDataInfo>();
	private XMLBox xmlBox=new XMLBox();
//	private BXMLBox bxmlBox=new BXMLBox();
	
	private Logger logger = Logger.getLogger(DCFCreator.class);

	public DCFCreator() 
	{
		this.dcf = new DCF();
	}
	
	/**
	 * <HL>
	 * Add media data resource to a DCFMaker
	 * @param mediaResourceFileName Specifies the media file path name
	 * @param mediaResourceItemID Specifies the item ID of media data above
	 * @throws DCFGenerationException 
	 */
	public void addResource(String mediaResourceFileName, String mediaResourceItemID) throws MPEG21FileCreationException 
	{
		//Create MediaDataBox
		MediaDataBox mediaDataBox=new MediaDataBox();
		//mediaDataBox.GetMediaDataFromFile(mediaResourceFileName);
		mediaDataBox.setMediaDataFileName(mediaResourceFileName);
		mediaDataBox.size=mediaDataBox.GetBoxFullSize();
		
		
		//Store this paire of media data box and item ID into an array list
		MediaDataInfo mediaDataInfo=new MediaDataInfo();
		mediaDataInfo.mediaDataBox=mediaDataBox;
		mediaDataInfo.itemID=mediaResourceItemID;
		mediaDataInfos.add(mediaDataInfo);
	}
	
	/**
	 * <HL>
	 * Add DCI data into DCFMaker
	 * @param dciFilePathName Filename of the DCI XML file
	 */
	public void setDIPathName(String dciFilePathName) throws MPEG21FileCreationException {
		//Create XMLBox, add DCI data into XMLBox
		xmlBox.GetDataFromXMLFile(dciFilePathName);
		xmlBox.size=xmlBox.GetBoxSize();
		
//		bxmlBox.GetDataFromBXMLFile(dciFilePathName);
//		bxmlBox.size=bxmlBox.GetBoxSize();
	}
	
	/**
	 * <HL>
	 * Add DCI data into DCFMaker
	 * @param dciData A byte array of DCI data 
	 */
	public void setDIBytes(byte[] dciData) throws MPEG21FileCreationException {
		//Create XMLBox, add DCI data into XMLBox
		XMLBox xmlBox=new XMLBox();
		xmlBox.XMLData=dciData;
		xmlBox.size=xmlBox.GetBoxSize();
		
//		BXMLBox bxmlBox=new BXMLBox();
//		bxmlBox.BXMLData=dciData;
//		bxmlBox.size=bxmlBox.GetBoxSize();
	}
	
	/**
	 * Create a dmp content file into a specified location
	 * @param dcfPathName The dmp content file name
	 */
	public void createMPEG21File(String dcfPathName) throws MPEG21FileCreationException {
		//Create FileTypeBox 
		FileTypeBox fileTypeBox=new FileTypeBox();
		fileTypeBox.compatibleBrands.add("dmpf");
		fileTypeBox.size=fileTypeBox.GetBoxSize();
	
		//Create ItemLocationBox
		ItemLocationBox ilocBox=new ItemLocationBox();
		ilocBox.offsetSize =4;
		ilocBox.lengthSize =4;
		ilocBox.baseOffsetSize =4;
		ilocBox.itemCount =mediaDataInfos.size();
	
		int mediaOffset=0;
		for(int i=0; i<ilocBox.itemCount; i++)
		{
			ItemInfo itemInfo=new ItemInfo(ilocBox.offsetSize, ilocBox.lengthSize, ilocBox.baseOffsetSize);
			MediaDataInfo mediaDataInfo=(MediaDataInfo)mediaDataInfos.get(i);
			itemInfo.itemID =mediaDataInfo.itemID;
			itemInfo.baseOffset =fileTypeBox.GetBoxSize() + mediaOffset;
			itemInfo.extentCount=0;
			mediaOffset=mediaOffset+mediaDataInfo.mediaDataBox.GetBoxFullSize();

			ilocBox.itemInfos.add(itemInfo);
		}
		ilocBox.size=ilocBox.GetBoxSize();
	
		//HandlerBox
		HandlerBox handlerBox=new HandlerBox();
		handlerBox.size=handlerBox.GetBoxSize();
		
		//Create MetaBox
		MetaBox metaBox=new MetaBox();
		metaBox.handlerBox=handlerBox;
		metaBox.itemLocationBox=ilocBox;
//			metaBox.bxmlBox=bxmlBox;
		metaBox.xmlBox=xmlBox;
		metaBox.size=metaBox.GetBoxSize();
	
		//Add various boxes into DCF(a box container)
		
		//add file type box
		dcf.AddBox(fileTypeBox);
		
		//add media data boxes
		for(int i=0; i<mediaDataInfos.size(); i++)
		{
			MediaDataInfo mediaDataInfo=(MediaDataInfo)mediaDataInfos.get(i);
			dcf.AddBox(mediaDataInfo.mediaDataBox);
		}
		
		//add meta box
		dcf.AddBox(metaBox);
		
		//make dmp content file
		try {
			this.writeFile(dcfPathName);
		} catch (IOException e) {
			String message = "An exception occurred while creating the MPEG-21 file";
			logger.error(message, e);
			throw new MPEG21FileCreationException(message + ": " + e.getMessage());
		}   		
	}
	
    /**
     * Create a dmp content file into a specified location
     * @param fileName The dmp content file name
     * @throws IOException 
     */
    private boolean writeFile(String fileName) throws IOException {
		File dcfFile=new File(fileName);
		FileOutputStream fos=new FileOutputStream(dcfFile);
	
		//write box objects into file
		for (int i = 0; i < this.dcf.boxes.size(); i++)
		{
			IBox box=(IBox)this.dcf.boxes.get(i);
			byte[] bytes=new byte[box.GetBoxSize()];
			box.Serialize(bytes);
			fos.write(bytes);
			
			//if this box is a media data box,
			//we will read media data from the source media data file,
			//because the media data size could be possibly very large,
			//it's hard to put them into a byte buffer 
			//due to the limited memory size
			if(box.GetBoxType().equals((Object)BoxType.MediaData))
			{
				MediaDataBox mediaDataBox=(MediaDataBox)box;
				File mediaFile=new File(mediaDataBox.getMediaDataFileName());
				FileInputStream fis=new FileInputStream(mediaFile);
				int blockSize=1024;
				long n=mediaFile.length()/blockSize;
				byte[] mediaData=new byte[1024];
				for(int j=0; j<n; j++)
				{
					fis.read(mediaData);
					fos.write(mediaData);
				}
				if (mediaFile.length()>n*blockSize)
	    		{
	    			mediaData=new byte[(int)(mediaFile.length()-n*blockSize)];
	    			fis.read(mediaData);
	    			fos.write(mediaData);
	    		}
				fis.close();
			}
		}
		fos.close();
		
		return true;
    }
    
    /**
     * Write the dcf meta box(including dci and iloc information) 
     * into an existing iso format file to create a dmp content file,
     * the precondition is user knows the item location infos of the iso file in advance.
     * @param ISOFileName filename of source iso format file   
     * @param metaBox meta box which to be added into the DCF
     * @param DCFFileName filename of dcf which to be created
     */
    private void CreateFileFromISO(String ISOFileName, MetaBox metaBox, String DCFFileName)
    {
    	try
    	{
    		//Copy iso file to dcf file
    		File isoFile=new File(ISOFileName);
    		FileInputStream isoFis=new FileInputStream(isoFile);
    		
    		File dcfFile=new File(DCFFileName);
    		FileOutputStream dcfFos=new FileOutputStream(dcfFile);
    		
    		int blockSize=1024;
    		int n=(int)isoFile.length()/blockSize;
    		byte[] temparray=new  byte[blockSize];
    		for(int i=0; i<n; i++)
    		{
    			isoFis.read( temparray);
    			dcfFos.write(temparray);
    		}
    		if (isoFile.length()>n*blockSize)
    		{
    			temparray=new byte[(int)(isoFile.length()-n*blockSize)];
    			isoFis.read(temparray);
    			dcfFos.write(temparray);
    		}
    		isoFis.close();
    		dcfFos.close();
    		   		
    		//Get meta box data 
    		n=metaBox.GetBoxSize();
    		byte[] bytes=new byte[n];
    		metaBox.Serialize(bytes);
    		
    		//write meta box into the dcf file
    		RandomAccessFile dcfRaf =new RandomAccessFile(DCFFileName,"rw");
    		dcfRaf.seek(dcfRaf.length());
    		dcfRaf.write(bytes);
    		dcfRaf.close();
    	}
    	catch(Exception ex)
    	{
    		System.out.println("CreateFileFromISO:" + ex.toString() );
    	}
    }
}
