/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s): Filippo Chiariglione
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.mpeg21ffTE;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.tEngine.mp21ffTE.exceptions.MPEG21FileParsingException;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.BoxLocationInfo;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.ExtentInfo;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.FreeBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.HandlerBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.ItemInfo;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.ItemLocationBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.License;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.LicenseBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.LicenseContainerBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.MediaDataBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.MetaBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.SignatureBox;
import org.iso.mpeg.mxm.mpeg21ffTE.boxes.XMLBox;
import org.iso.mpeg.mxm.mpeg21ffTE.constants.BoxType;
import org.iso.mpeg.mxm.mpeg21ffTE.utils.ArrayUtils;
import org.iso.mpeg.mxm.mpeg21ffTE.utils.TypeUtils;

/**
 * This class provides methods to get various boxes or data
 * from a dmp content file
 * 
 * @author Xueqing Liu
 */
public class DCFParser implements org.iso.mpeg.mxm.tEngine.mp21ffTE.MPEG21FileParser {	

	private String dcfPathName = null;
	
	private Logger logger = Logger.getLogger(DCFParser.class);


	/**
	 * <HL>
	 * Get DCI in XML file format from a dmp content file.It's an overlaoded method.
	 * @param dcfPathName full path name of DCF file to be parsed from
	 * @param dciPathName full path name of DCI file to be created
	 * @throws IOException 
	 * @throws MPEG21FileParsingException 
	 */
	public void getDCI(String dcfPathName, String dciPathName) throws MPEG21FileParsingException  {
		this.dcfPathName = dcfPathName;
		
		byte[] XMLData=null;
		MetaBox metaBox=this.GetMetaBox(dcfPathName);
		XMLBox xmlBox=metaBox.xmlBox;
		XMLData=xmlBox.XMLData;
		
		try {
			File file=new File(dciPathName);
			FileOutputStream fos=new FileOutputStream(file);
	    	fos.write(XMLData);
	    	fos.close();
	    	
	    	if (file.length() == 0)
	    		throw new MPEG21FileParsingException("The DCI extracted fromthe DCF is null.");
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			logger.error("A FileNotFoundException occurred.", e);
			throw new MPEG21FileParsingException("The file: " + dcfPathName + " was not found."); 
		} catch (IOException e) {
			e.printStackTrace();
			logger.error("A IOException occurred.", e);
			throw new MPEG21FileParsingException("An IOException ocurred while parsing the file: " + dcfPathName); 
		}
	}
	
	/**
	 * <HL>
	 * Get DCI in a byte array from a dmp content file. It's an overloaded method.
	 * @param dcfPathName full path name of DCF to be parsed
	 * @return a byte array of XML Data
	 * @throws MPEG21FileParsingException 
	 */
	public byte[] getBinaryDI(String dcfPathName) throws MPEG21FileParsingException {
		this.dcfPathName = dcfPathName;
		
		byte[] XMLData=null;
		MetaBox metaBox=this.GetMetaBox(dcfPathName);
		XMLBox xmlBox=metaBox.xmlBox;
		XMLData=xmlBox.XMLData;

		if (XMLData.length == 0)
    		throw new MPEG21FileParsingException("The DCI extracted fromthe DCF is null.");
 
		return XMLData;
	}
	
	/**
	 * <HL>
	 * Get media data requested through item ID
	 * @param itemID the Item ID of the required Media Resource
	 * @param  mediaDataFileName Filename of the output media data file
	 * @return true if succeed, false if fail
	 * @throws IOException 
	 * @throws MPEG21FileParsingException 
	 * @throws DCFGenerationException 
	 */
	public void getMediaData(String itemID, String mediaDataFileName) throws MPEG21FileParsingException	{
		long baseOffset=0;
		//Get meta box
		MetaBox metaBox=this.GetMetaBox(this.dcfPathName);
	
		//Get item location box
		ItemLocationBox ilocBox=metaBox.itemLocationBox;
	
		//Get item infos
		ArrayList<ItemInfo> itemInfos=ilocBox.itemInfos;
	
		//Get a single item info whose item ID is equal to the input item ID
		for(int i=0; i<ilocBox.itemCount; i++)
		{
			ItemInfo itemInfo = itemInfos.get(i);
			if (itemInfo.itemID.equals((Object)itemID))
			{
				baseOffset=itemInfo.baseOffset;
				break;
			}
		}

		try {
			//Get media data box with the known base offset
			MediaDataBox mediaDataBox=this.GetMediaDataBox(this.dcfPathName, baseOffset, mediaDataFileName);
		} catch (IOException e) {
			e.printStackTrace();
			logger.error("A IOException occurred.", e);
			throw new MPEG21FileParsingException("An IOException ocurred while parsing the file: " + dcfPathName); 
		}
	}

	/*
	public boolean writeMediaDataToFile (File out, String itemID) {

		long baseOffset=0;
		
		//Get meta box
		MetaBox metaBox=this.GetMetaBox(this.dcfPathName);
		
		//Get item location box
		ItemLocationBox ilocBox=metaBox.itemLocationBox;
		
		//Get item infos
		ArrayList itemInfos=ilocBox.itemInfos;
		
		//Get a single item info whose item ID is equal to the input item ID
		for(int i=0; i<ilocBox.itemCount; i++)
		{
			ItemInfo itemInfo=(ItemInfo)itemInfos.get(i);
			if (itemInfo.itemID.equalsIgnoreCase(itemID))
			{
				baseOffset=itemInfo.baseOffset;
				break;
			}
		}
		
		//Get media data box with the known base offset
		MediaDataBox mediaDataBox=this.GetMediaDataBox(this.dcfPathName,baseOffset);
		int mediaDataSize = mediaDataBox.mediaData.length;
		
		try{
			FileOutputStream fos = new FileOutputStream(out);
			fos.write(mediaDataBox.mediaData, 0, mediaDataSize);
			fos.flush();
			fos.close();
			return true;
		}
		catch(Exception ex) {
			ex.printStackTrace();
			return false;
		}	
	}
	*/
	
	/**
	 * Get meta box from dcf
	 * @param dcfPathName DCF full path name
	 * @return meta box
	 */
	private MetaBox GetMetaBox(String dcfPathName)
	{
		MetaBox metaBox=new MetaBox();
		
		try
		{
			//find meta box
			BoxLocationInfo bli=new BoxLocationInfo();
			bli=this.FindBox( dcfPathName, BoxType.Meta);
			
			File  file=new  File(dcfPathName);
			FileInputStream fis=new FileInputStream(file);
			
			//locate
			fis.skip( bli.boxOffset);
			
			//size
			byte[] bytes=new byte[4];
			fis.read( bytes);
			bli.boxSize = ArrayUtils.ByteArrayToInt(bytes);
			metaBox.size=bli.boxSize;
			
			//type
			bytes=new byte[4];
			fis.read(bytes);
			String str=ArrayUtils.ByteArrayToString(bytes);
			str.equals((Object)BoxType.Meta);
			
			//version
			bytes=new byte[1];
			fis.read(bytes);
			metaBox.version=bytes[0];
			
			//flag
			bytes=new byte[3];
			fis.read(bytes);
			for(int i=0; i<3; i++)
			{
				metaBox.flags[i]=bytes[i];
			}
			
			//handler box
			bytes=new byte[4];
			fis.read(bytes);
			int boxSize=ArrayUtils.ByteArrayToInt(bytes);
			fis.skip(-4);
			bytes=new byte[boxSize];
			fis.read(bytes);
			HandlerBox handlerBox=GetHandlerBox(bytes);
			metaBox.handlerBox=handlerBox;
			
			//iloc box
			bytes=new byte[4];
			fis.read(bytes);
			boxSize=ArrayUtils.ByteArrayToInt(bytes);
			fis.skip(-4);
			bytes=new byte[boxSize];
			fis.read(bytes);
			ItemLocationBox ilocBox=GetItemLocationBox(bytes);
			metaBox.itemLocationBox=ilocBox;
			
			//bxml box
//			bytes=new byte[4];
//			fis.read(bytes);
//			boxSize=ArrayUtils.ByteArrayToInt(bytes);
//			fis.skip(-4);
//			bytes=new byte[boxSize];
//			fis.read(bytes);
//			BXMLBox bxmlBox=this.GetBXMLBox(bytes);
//			metaBox.bxmlBox=bxmlBox;
		
			//xml box
			bytes=new byte[4];
			fis.read(bytes);
			boxSize=ArrayUtils.ByteArrayToInt(bytes);
			fis.skip(-4);
			bytes=new byte[boxSize];
			fis.read(bytes);
			XMLBox xmlBox=this.GetXMLBox(bytes);
			metaBox.xmlBox=xmlBox;
		}
		catch(Exception ex)
		{
			System.out.println("GetMetaBox:" + ex.toString() );
		}
		return metaBox;
	}
	
//	/**
//	 * Get BXML box from a data buffer
//	 * @param data bxml data f
//	 * @return bxml box
//	 */
//	private BXMLBox GetBXMLBox(byte[] data)
//	{
//		
//		byte[] bytes;
//		BXMLBox bxmlBox=new BXMLBox();
//		
//		try
//		{
//			int n=0;
//			
//			//size
//			bytes=new byte[4];
//			for(int i=0; i<4; i++)
//			{
//				bytes[i]=data[n];
//				n++;
//			}
//			bxmlBox.size=ArrayUtils.ByteArrayToInt(bytes);
//			
//			//type
//			bytes=new byte[4];
//			for(int i=0; i<4; i++)
//			{
//				bytes[i]=data[n];
//				n++;
//			}
//			String str=ArrayUtils.ByteArrayToString(bytes);
//			str.equals((Object)BoxType.BXML);
//			
//			//version
//			bxmlBox.version=data[n];
//			n++;
//			
//			//flag
//			bytes=new byte[3];
//			for(int i=0; i<3; i++)
//			{
//				bxmlBox.flags[i]=data[n];
//				n++;
//			}
//			
//			//bxml data
//			bxmlBox.BXMLData=new byte[data.length-12];
//			for(int i=0; i<data.length-12; i++)
//			{
//				bxmlBox.BXMLData[i]=data[n];
//				n++;
//			}
//		}
//		catch(Exception ex)
//		{
//			System.out.println("GetBXMLBox1:" + ex.toString() );
//		}
//		
//		return bxmlBox;
//	}
	
	/**
	 * Get XML box from a data buffer
	 * @param data xml data f
	 * @return xml box
	 */
	private XMLBox GetXMLBox(byte[] data)
	{
		
		byte[] bytes;
		XMLBox xmlBox=new XMLBox();
		
		try
		{
			int n=0;
			
			//size
			bytes=new byte[4];
			for(int i=0; i<4; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			xmlBox.size=ArrayUtils.ByteArrayToInt(bytes);
			
			//type
			bytes=new byte[4];
			for(int i=0; i<4; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			String str=ArrayUtils.ByteArrayToString(bytes);
			str.equals((Object)BoxType.XML);
			
			//version
			xmlBox.version=data[n];
			n++;
			
			//flag
			bytes=new byte[3];
			for(int i=0; i<3; i++)
			{
				xmlBox.flags[i]=data[n];
				n++;
			}
			
			//xml data
			xmlBox.XMLData=new byte[data.length-12];
			for(int i=0; i<data.length-12; i++)
			{
				xmlBox.XMLData[i]=data[n];
				n++;
			}
		}
		catch(Exception ex)
		{
			System.out.println("GetXMLBox:" + ex.toString() );
		}
		
		return xmlBox;
	}

	/**
	 * Get item location box from a data buffer
	 * @param data item location box data
	 * @return item location box object
	 */
	private ItemLocationBox GetItemLocationBox(byte[] data)
	{
		ItemLocationBox ilocBox=new ItemLocationBox();
		
		try
		{
			int n=0;
			
			//size 4
			byte[] bytes=new byte[4];
			for(int i=0; i<4; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			ilocBox.size=ArrayUtils.ByteArrayToInt(bytes);
			
			//type 4
			bytes=new byte[4];
			for(int i=0; i<4; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			String str=ArrayUtils.ByteArrayToString(bytes);
			str.equals((Object)BoxType.ItemLocation);
			
			//version 1
			ilocBox.version=data[n];
			n++;
			
			//flag 3
			for(int i=0; i<3; i++)
			{
				ilocBox.flags[i]=data[n];
				n++;
			}
			
			//offset Size & length size 1
			byte temp=data[n];
			n++;
			
			ilocBox.offsetSize=(byte)(((int)temp & 0xf0)>>4);
			ilocBox.lengthSize=(byte)(((int)temp)& 0x0f);
			
			//base offset size & reserved 1
			temp=data[n];
			n++;
			ilocBox.baseOffsetSize=(byte)(((int)temp & 0xf0)>>4);
			ilocBox.reserved=(byte)(((int)temp)& 0x0f);
			
			//item count 2
			bytes=new byte[2];
			for(int i=0; i<2; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			ilocBox.itemCount=ArrayUtils.ByteArrayToInt(bytes);
			
			//item infos 
			bytes=new byte[data.length-16];
			for(int i=0; i<data.length-16; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			ilocBox.itemInfos=this.GetItemInfos(ilocBox.offsetSize,
								  			    ilocBox.lengthSize,
								  			    ilocBox.baseOffsetSize,
								  			    ilocBox.itemCount,
								  			    bytes);
		}
		catch(Exception ex)
		{
			System.out.println("GetItemLocationBox:" + ex.toString() );
		}
		
		return ilocBox;
	}
	
	/**
	 * Get item info from a data buffer
	 * @param offsetSize
	 * @param lengthSize
	 * @param baseOffsetSize
	 * @param itemCount
	 * @param data
	 * @return itemInfos
	 */
	private ArrayList<ItemInfo> GetItemInfos(int offsetSize, 
							  	  int lengthSize, 
								  int baseOffsetSize, 
								  int itemCount, 
								  byte[] data)
	{
		ArrayList<ItemInfo> itemInfos = new ArrayList<ItemInfo>();
		int n=0;
		
		try
		{
			for(int i=0; i<itemCount; i++)
			{
				ItemInfo itemInfo=new ItemInfo(offsetSize, lengthSize,baseOffsetSize);
				
				//item ID
				byte[] bytes=new byte[2];
				for(int j=0; j<2; j++)
				{
					bytes[j]=data[n];
					n++;
				}
				/*int temp_itemID=ArrayUtils.ByteArrayToInt(bytes);
				itemInfo.itemID=TypeUtils.IntToString(temp_itemID);*/
				itemInfo.itemID=TypeUtils.HexToString_m(bytes);
				
			
				//data reference index
				bytes=new byte[2];
				for(int j=0; j<2; j++)
				{
					bytes[j]=data[n];
					n++;
				}
				itemInfo.dataReferenceIndex=ArrayUtils.ByteArrayToInt(bytes);
			
				//base offset
				bytes=new byte[baseOffsetSize];
				for(int j=0; j<baseOffsetSize; j++)
				{
					bytes[j]=data[n];
					n++;
				}
				itemInfo.baseOffset=ArrayUtils.ByteArrayToLong(bytes);
			
				//extent count
				bytes=new byte[2];
				for(int j=0; j<2; j++)
				{
					bytes[j]=data[n];
					n++;
				}
				itemInfo.extentCount=ArrayUtils.ByteArrayToInt(bytes);
			
				//extent infos
				byte[] temp_extentData=new byte[(offsetSize + lengthSize)* itemInfo.extentCount];
				for(int j=0; j<(offsetSize + lengthSize)* itemInfo.extentCount; j++)
				{
					temp_extentData[j]=data[n];
					n++;
				}
				itemInfo.extentInfos=GetExtentInfos(offsetSize,
												   lengthSize,
												   itemInfo.extentCount,
						                           temp_extentData);
				
				itemInfos.add(itemInfo);
			}
		}
		catch(Exception ex)
		{
			System.out.println("GetItemInfos:" + ex.toString());
		}
		
		return itemInfos;
	}
	
	
	/**
	 * Get entent infos from a data buffer
	 * @param extentCount
	 * @param data
	 * @return extent infos
	 */
	private ArrayList<ExtentInfo> GetExtentInfos(int offsetSize,
								    int lengthSize,
								    int extentCount,
							  	    byte[] data)
	{
		
		ArrayList<ExtentInfo> extentInfos=new ArrayList<ExtentInfo>();
		int n=0;
		
		try
		{
			for(int i=0; i<extentCount; i++)
			{
				ExtentInfo extentInfo=new ExtentInfo(offsetSize, lengthSize);
				
				//extent offset
				byte[] bytes=new byte[offsetSize];
				for(int j=0; j<offsetSize; j++)
				{
					bytes[j]=data[n];
					n++;
				}
				extentInfo.extentOffset = ArrayUtils.ByteArrayToLong(bytes);
			
				//extent length
				bytes=new byte[lengthSize];
				for(int j=0; j<lengthSize; j++)
				{
					bytes[j]=data[n];
					n++;
				}
				extentInfo.extentLength = ArrayUtils.ByteArrayToLong(bytes);
			
				//add to array list
				extentInfos.add(extentInfo);
			}
		}
		catch(Exception ex)
		{
			System.out.println("GetExtentInfo:" + ex.toString());
		}

		return extentInfos;
	}
	
	
	
	/**
	 * Get media data box from a dcf
	 * @param dcfPathName DCF full path name
	 * @param offset offset of  the media data box
	 * @param mediaDataFile Media data file which contains media data get from DCF
	 * @return media data box object
	 * @throws IOException 
	 * @throws MPEG21FileParsingException 
	 * @throws DCFGenerationException 
	 */
	private MediaDataBox GetMediaDataBox(String dcfPathName,
										 long baseOffset, 
										 String mediaDataFileName) throws IOException, MPEG21FileParsingException {
		byte[] bytes;
		MediaDataBox mediaDataBox=new MediaDataBox();
		
		File file=new  File(dcfPathName);
		FileInputStream fis=new FileInputStream(file);
		
		//locate
		fis.skip(baseOffset);
		
		//box size
		bytes=new byte[4];
		fis.read( bytes);
		int boxSize = ArrayUtils.ByteArrayToInt(bytes);
		mediaDataBox.size=boxSize;
		
		//box type
		bytes=new byte[4];
		fis.read( bytes);
		String str=ArrayUtils.ByteArrayToString(bytes);
		str.compareTo(BoxType.MediaData);
		
		//media data.
		//put the media data which is parsed out from the dcf into a media data file,
		//becuase the media data could be possibly very large,
		//it's hard to put media data into a byte buffer due to the limited memory size
		mediaDataBox.setMediaDataFileName(mediaDataFileName);
		File mediaFile=new File(mediaDataFileName);
		FileOutputStream fos=new FileOutputStream(mediaFile);
		int mediaSize=boxSize-8;
		int blockSize=1024;
		int n=(int)mediaSize/blockSize;
		byte[] mediaData=new byte[blockSize];
		for(int i=0; i<n; i++)
		{
			fis.read(mediaData);
			fos.write(mediaData);
		}
		if(mediaSize > n* blockSize)
		{
			mediaData=new byte[mediaSize-n*blockSize];
			fis.read(mediaData);
			fos.write(mediaData);
		}
		
		fis.close();
		fos.close();
		
		if (mediaFile.length() == 0)
			throw new MPEG21FileParsingException("The Media Data Box did not contain any data.");

		return mediaDataBox;
	}
	

	/**
	 * Get handler box from a data buffer
	 * @param dcfPathName DCF full path name
	 * @param data handler box data  f
	 * @return handler box
	 */
	private HandlerBox GetHandlerBox(byte[] data)
	{
		byte[] bytes;
		HandlerBox handlerBox=new HandlerBox();
		int n=0;
		
		try
		{
			//box size
			bytes=new byte[4];
 			for(int i=0; i<4; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			int boxSize = ArrayUtils.ByteArrayToInt(bytes);
			handlerBox.size=boxSize;
			
			//box type
			bytes=new byte[4];
			for(int i=0; i<4; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			String str=ArrayUtils.ByteArrayToString(bytes);
			str.compareTo(BoxType.Handler);
			
			//version
			handlerBox.version=data[n];
			n++;
			
			//flag
			for(int i=0; i<3; i++)
			{
				handlerBox.flags[i]=data[n];
				n++;
			}
			
			//pre defined
			bytes=new byte[4];
			for(int i=0; i<4; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			handlerBox.preDefined=ArrayUtils.ByteArrayToInt(bytes);
			
			//handler type
			bytes=new byte[4];
			for(int i=0; i<4; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			handlerBox.handlerType=ArrayUtils.ByteArrayToString(bytes);
			
			//reserved
			for(int i=0; i<3; i++)
			{
				byte[] temp=new byte[4];
				for(int j=0; j<4; j++)
				{
					temp[j]=data[n];
					n++;
				}
				handlerBox.reserved[i]=ArrayUtils.ByteArrayToInt(temp);
			}

			//name
			bytes=new byte[boxSize -32];
			for(int i=0; i<boxSize-32; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			handlerBox.name=ArrayUtils.ByteArrayToString(bytes);
			
		}
		catch(Exception ex)
		{
			System.out.println("GetHandlerBox:" + ex.toString());
		}
		
		return handlerBox;
	}

	/**
	 * Get license container box from dcf f
	 * @param dcfPathName DCF full path name
	 * @return license container box
	 */
	private LicenseContainerBox GetLicenseContainerBox(String dcfPathName) 	
	{

		byte[] bytes;
		LicenseContainerBox licenseContainerBox=new LicenseContainerBox();
		
		try
		{
			BoxLocationInfo bli=new BoxLocationInfo();
			bli=this.FindBox( dcfPathName, BoxType.LicenseContainer);
			
			File  file=new  File(dcfPathName);
			FileInputStream fis=new FileInputStream(file);
			
			//locate
			fis.skip( bli.boxOffset);
			
			//size
			bytes=new byte[4];
			fis.read( bytes);
			bli.boxSize = ArrayUtils.ByteArrayToInt(bytes);
			licenseContainerBox.size=bli.boxSize;
			
			//type
			bytes=new byte[4];
			fis.read(bytes);
			String str=ArrayUtils.ByteArrayToString(bytes);
			str.equals((Object)BoxType.LicenseContainer);
			
			//version
			bytes=new byte[1];
			fis.read(bytes);
			licenseContainerBox.version=bytes[0];
			
			//flag
			bytes=new byte[3];
			fis.read(bytes);
			for(int i=0; i<3; i++)
			{
				licenseContainerBox.flags[i]=bytes[i];
			}
			
			//item count
			bytes=new byte[2];
			fis.read(bytes);
			licenseContainerBox.itemCount=ArrayUtils.ByteArrayToInt(bytes);
			
			//licenses arraylist
			bytes=new byte[bli.boxSize -14];
			fis.read(bytes);
			
			ArrayList<License> licenses = GetLicenses(licenseContainerBox.itemCount, bytes);
			licenseContainerBox.licenses=licenses;
			
			fis.close();
		}
		catch(Exception ex)
		{
			System.out.println("GetLicenseContainerBox:" + ex.toString() );
		}
		
		return licenseContainerBox;
	}

	/**
	 * Get licenses arraylist from a data buffer
	 * @param itemCount license item count
	 * @param data data buffer which contains licenses data
	 * @return
	 */
	public ArrayList<License> GetLicenses(int itemCount,
								 byte[] data)
	{
		
		ArrayList<License> licenses=new ArrayList<License>();
		int n=0;
		
		try
		{
			for(int i=0; i<itemCount; i++)
			{
				License license = new License();
				
				//license ID
				byte[] bytes=new byte[4];
				for(int j=0; j<4; j++)
				{
					bytes[j]=data[n];
					n++;
				}
				license.licenseID=ArrayUtils.ByteArrayToString(bytes);
			
				//License box
				bytes=new byte[4];
				for(int j=0; j<4; j++)
				{
					bytes[j]=data[n];
					n++;
				}
				n=n-4;
				int licenseBoxSize=ArrayUtils.ByteArrayToInt(bytes);
				bytes = new byte[licenseBoxSize];
				for(int j=0; j<licenseBoxSize; j++)
				{
					bytes[j]=data[n];
					n++;
				}
				LicenseBox licenseBox=new LicenseBox();
				licenseBox=GetLicenseBox(bytes);
				license.licenseBox=licenseBox;
				
				//add license
				licenses.add(license);
			}
		}
		catch(Exception ex)
		{
			System.out.println("GetLicenses:" + ex.toString());
		}
		
		return licenses;
		
	}
	/**
	 * Get license box
	 * @param data license box data f
	 * @return license box
	 */
	private LicenseBox GetLicenseBox(byte[] data)
	{
		byte[] bytes;
		LicenseBox licenseBox=new LicenseBox();
		
		try
		{
			int n=0;
			
			//size
			bytes=new byte[4];
			for(int i=0; i<4; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			licenseBox.size=ArrayUtils.ByteArrayToInt(bytes);
			
			//type
			bytes=new byte[4];
			for(int i=0; i<4; i++)
			{
				bytes[i]=data[n];
				n++;
			}
			String str=ArrayUtils.ByteArrayToString(bytes);
			str.equals((Object)BoxType.License);
			
			//version
			licenseBox.version=data[n];
			n++;
			
			//flag
			bytes=new byte[3];
			for(int i=0; i<3; i++)
			{
				licenseBox.flags[i]=data[n];
				n++;
			}
			
			//license data
			for(int i=0; i<data.length-12; i++)
			{
				licenseBox.licenseData=new byte[data.length-12]; 
				licenseBox.licenseData[i]=data[n];
				n++;
			}
			
		}
		catch(Exception ex)
		{
			System.out.println("GetLicenseBox:" + ex.toString() );
		}
		
		return licenseBox;
	}
	
	/**
	 * Get signature box
	 * @param dcfPathName DCF full path name
	 * @return signature box
	 */
	private SignatureBox GetSignatureBox(String dcfPathName)
	{
		byte[] bytes;
		SignatureBox signatureBox=new SignatureBox();
		
		try
		{
			BoxLocationInfo bli=new BoxLocationInfo();
			bli=this.FindBox( dcfPathName, BoxType.Signature);
			
			File file=new  File(dcfPathName);
			FileInputStream fis=new FileInputStream(file);
			
			//locate
			fis.skip( bli.boxOffset);
			
			//size
			bytes=new byte[4];
			fis.read( bytes);
			bli.boxSize = ArrayUtils.ByteArrayToInt(bytes);
			signatureBox.size=bli.boxSize;
			
			//type
			bytes=new byte[4];
			fis.read(bytes);
			String str=ArrayUtils.ByteArrayToString(bytes);
			str.equals((Object)BoxType.Signature);
			
			//signature box data
			bytes=new byte[bli.boxSize -8];
			fis.read( bytes);
			signatureBox.signatureData=new byte[bli.boxSize-8];
			for(int i=0; i <bli.boxSize-8; i++)
			{
				signatureBox.signatureData[i]=bytes[i];
			}
			
			fis.close();
		}
		catch(Exception ex)
		{
			System.out.println("GetSignatureBox:" + ex.toString() );
		}
		
		return signatureBox;
	}

	/**
	 * Get free box
	 * @param dcfPathName DCF full path name
	 * @return free box
	 */
	private FreeBox GetFreeBox(String dcfPathName) 
	{
		byte[] bytes;
		FreeBox freeBox=new FreeBox();
		
		try
		{
			BoxLocationInfo bli=new BoxLocationInfo();
			bli=this.FindBox( dcfPathName, BoxType.Free);
			
			File file=new  File(dcfPathName);
			FileInputStream fis=new FileInputStream(file);
			
			//locate
			fis.skip( bli.boxOffset);
			
			//size
			bytes=new byte[4];
			fis.read( bytes);
			bli.boxSize = ArrayUtils.ByteArrayToInt(bytes);
			freeBox.size=bli.boxSize;
			
			//type
			bytes=new byte[4];
			fis.read(bytes);
			String str=ArrayUtils.ByteArrayToString(bytes);
			str.equals((Object)BoxType.Free);
			
			//free data
			bytes=new byte[bli.boxSize-8];
			fis.read( bytes);
			freeBox.freeData= bytes;
			fis.close();
		}
		catch(Exception ex)
		{
			System.out.println("GetFreeBox:" + ex.toString() );
		}
		
		return freeBox;
	}
		
	/**
	 * Get box offset and box size with the specified box type from dcf
	 * @param dcfPathName DCF full path name
	 * @param boxType box type to be founded
	 * @return box location info 
	 */
	private BoxLocationInfo FindBox(String dcfPathName, String boxType)
	{
		BoxLocationInfo bli=new BoxLocationInfo();
		
		int offset=0;
		int boxSize=0;
		try
		{
			File file=new File(dcfPathName);
			FileInputStream fis=new FileInputStream(file);
			
			while(offset < file.length() )
			{
				//box size
				byte[] sizearray=new byte[4];
				fis.read(sizearray,0,4);
				boxSize= ArrayUtils.ByteArrayToInt(sizearray);
				
				//box type
				byte[] typearray=new byte[4];
				fis.read(typearray,0,4);
				String type=ArrayUtils.ByteArrayToString(typearray);
				
				if(type.equals(boxType))
				{
					break;
				}
				else
				{
					fis.skip(boxSize-8);
					offset = offset + boxSize;
				}
			}
			fis.close();
		}
		catch(Exception ex)
		{
			System.out.println("FindBox:" + ex.toString() );
		}
		
		bli.boxOffset=offset;
		bli.boxSize=boxSize;
		
		return bli;
	}

}
