/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.mpeg21ffTE.boxes;
import java.util.ArrayList;

import org.iso.mpeg.mxm.mpeg21ffTE.constants.BoxType;
import org.iso.mpeg.mxm.mpeg21ffTE.utils.ArrayUtils;


/**
 * This class provides user methods to perform operations on FileType Box. 
 * It is extended from Box
 * 
 * @author Xueqing Liu
 *	
 */
public class FileTypeBox extends Box
{
	/**
	 * Reports the box type
	 */
	 public final String type = BoxType.FileType;
	 
	 /**
	  * Specifies the major brand.
	  */
	 public String majorBrand = "dmpf";
	 
	 /**
	  * Specifies the minor version.
	  */
	 public String minorVersion = "0000";
	 
	 //*************************************************************
	 
	 /**
	  * Specifies the compatible brands.
	  */
	 public ArrayList<String> compatibleBrands =new ArrayList<String>() ;

	 /**
	  * Caculates the Box's size in bytes.
	  */
     public int GetBoxSize()
     {
    	 boxSize = 4 + //size
 		   		   type.length() + 
 		           majorBrand.length() +
 		           minorVersion.length();
    	 
    	 for(int i=0; i<compatibleBrands.size() ; i++)
    	 {
    		 String temp = compatibleBrands.get( i);
    		 boxSize = boxSize + temp.length() ;
    	 }
    	 return boxSize;
     }
     
     /**
      * Put all data of this box into a byte array
      */
     public int Serialize(byte[] bytes)
     {
    	 int n=0;
         
         try
         {
        	 //size
        	 byte[] temp=new byte[4];
         	 ArrayUtils.IntToByteArray(size, temp);
         	 for (int i = 0; i < temp.length ; i++)
             {
         		bytes[n] = (byte)temp[i];
         		n++;
             }

         	 //type
    	 	 for (int i = 0; i < type.length() ; i++)
    	 	 {
    	 		bytes[n] = (byte)type.charAt(i);
    	 		n++;
    	 	 }
    	 
    	 	 //major brand
    		 for (int i = 0; i < majorBrand.length() ; i++)
    		 {
    			 bytes[n] =(byte) majorBrand.charAt( i);
    			 n++;
    		 }

    		 //minor version
    		 for (int i = 0; i < minorVersion.length() ; i++)
    		 {
    			 bytes[n] = (byte) (minorVersion.charAt( i) - '0');
    			 n++;
    		 }

    		 //compatible brands
    		 for(int i=0; i<compatibleBrands.size() ; i++)
    		 {
    			 String str=(String)compatibleBrands.get( i);
    			 for (int j = 0; j < str.length(); j++)
    			 {
    				 bytes[n] = (byte)str.charAt(j);
    				 n++;
    			 }
    		 }
    	 }
    	 catch(Exception ex)
    	 {
    		 System.out.println("FileType:" + ex.toString());
    	 }
    	 
    	 return n;
     }
     
     /**
      * Get box type
      * @return Box type
      */
     public String GetBoxType()
     {
     	return this.type;
     }
}
