/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.mpeg21ffTE.boxes;
import java.io.File;
import java.io.FileInputStream;

import org.iso.mpeg.mxm.mpeg21ffTE.constants.BoxType;
import org.iso.mpeg.mxm.mpeg21ffTE.utils.ArrayUtils;


/**
 * This class provides user methods to perform operations on FreeBox. 
 * It extends from Box
 * 
 * @author Xueqing Liu
 *	
 */
public class FreeBox extends Box
{
	/**
	 * Reports the box type.
	 */
	public final String type = BoxType.Free;
	
	/**
	 * A byte array which contains free data.
	 */
    public byte[] freeData;
    
    //*************************************************************
    
    /**
     * Get free data from a freedata file
     * @param fileName the path of the freedata file which contains the free data
     */
    public void GetFreeDataFromFile(String fileName)
    {
    	try
     	{
     		File file =new File(fileName);
     		FileInputStream fis=new FileInputStream(fileName);
     		freeData=new byte[(int)file.length()];
     		fis.read(freeData,0,(int)file.length());
     		fis.close();
     	}
     	catch (Exception ex)
     	{
     		System.out.println("FreeBox1:" + ex.toString() );
     	}
    }
    
    /**
     * Get free data from a byte array
     * @param data free data
     */
    public void GetFreeData(byte[] data)
    {
    	try
     	{
     		freeData=new byte[data.length];
     		for(int i=0; i<data.length; i++)
     		{
     			freeData[i]=data[i];
     		}
     	}
     	catch (Exception ex)
     	{
     		System.out.println("FreeBox2:" + ex.toString() );
     	}
    }
     
    /**
     * Caculates the Box's size in bytes.
     */
    public int GetBoxSize()
    {
    	 boxSize = 4 + //size
		   		   type.length() + 
		           freeData.length;

    	return boxSize;
    }
    
    /**
     * Put data of this box into a byte array
     */
    public int Serialize(byte[] bytes)
    {
    	int n=0;
        
        try
        {
        	//size
        	byte[] temp=new byte[4];
        	ArrayUtils.IntToByteArray(size, temp);
        	for (int i = 0; i < temp.length ; i++)
            {
        		bytes[n] = (byte)temp[i];
        		n++;
            }

        	//type
       	    for (int i = 0; i < 4; i++)
        	{
       		    bytes[n] = (byte)type.charAt(i);
       		    n++;
        	}

       	    //free data
       	    for(int i=0; i<freeData.length ; i++)
    	    {
    		    bytes[n]=freeData[i];
    		    n++;
    	    }
    	}
        catch(Exception ex)
        {
        	System.out.println("FreeBox3:" + ex.toString() );
        }
        
        return n;
    }
    
    /**
     * Get box type
     * @return Box type
     */
    public String GetBoxType()
    {
    	return this.type;
    	
    }
}
