/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.mpeg21ffTE.boxes;
import java.util.ArrayList;

import org.iso.mpeg.mxm.mpeg21ffTE.utils.TypeUtils;


/**
 * This class specifies the attributes of the ItemInfo.
 * This class is used by the class ItemLocationBox.
 * 
 * @author Xueqing Liu
 *
 */
public class ItemInfo 
{
	/**
	 * Offset size
	 */
	protected int offsetSize;
	
	/**
	 * Length size
	 */
	protected int lengthSize;
	
	/**
	 * Base offset size
	 */
	protected int baseOffsetSize;
	
	/**
	 * Item ID.
	 */
	public String itemID;
	
	/**
	 * Data reference index
	 */
	public int dataReferenceIndex=0;
	
	/**
	 * Base offset.
	 */
	public long baseOffset;
	
	/**
	 * Extent count
	 */
	public int extentCount=0;
	
	/**
	 * An arraylist to contain the ExtentInfos,
	 * the number of the ExtentInfo object in  the arraylist 
	 * depends on the extentCount attribute. 
	 */
	public ArrayList<ExtentInfo> extentInfos=new ArrayList<ExtentInfo>();
	
	/**
	 * Constructor method
	 * @param offsetSize offset size 
	 * @param lengthSize length size
	 * @param baseOffsetSize base offset size
	 */
	public ItemInfo(int offsetSize, int lengthSize, int baseOffsetSize )
	{
		this.baseOffset=offsetSize;
		this.lengthSize=lengthSize;
		this.baseOffsetSize=baseOffsetSize;
	}
	
	/**
	 * Calculates all fields' size in byte
	 * @return the size of this ItemInfo class in byte
	 */
	public int GetItemInfoSize()
	{
		int size=0;
		for(int i=0; i<extentCount; i++)
		{
			ExtentInfo extentInfo = extentInfos.get(i);
			size = size + extentInfo.GetExtentInfoSize();
		}
		
		size = 2 + //ItemID;
			   2 + //dataReferenceIndex;
			   2 + //extent count
			   baseOffsetSize + //baseOffse
			   size; //extent infos
				 
		return size;
	}
	
	/**
	 * Put the all data of this class into a byte array
	 * @param bytes a byte array which will contain all data in this box
	 * @return the bytes number which has been put into the array
	 */
	public int Serialize(byte[] bytes)
	{
		int n=0;
		
		try
		{
			//itemID
			/*int temp_itemID=TypeUtils.StringToInt(itemID);
			for(int i=0; i<2; i++)
			{	
				bytes[n]=(byte)(temp_itemID >>(1-i)*8);
				n++;
			}*/
			
			byte[] temp_itemID=new byte[2];
			temp_itemID=TypeUtils.StringToHex_m(itemID);
			for(int i=0; i<2; i++)
			{
				bytes[n]=temp_itemID[i];
				n++;
			}
			
		
			//dataReferenceIndex
			for(int i=0; i<2; i++)
			{
				bytes[n]=(byte)(dataReferenceIndex >>(1-i)*8);
				n++;
			}
		
			//baseOffset
			for(int i=0; i<baseOffsetSize; i++)
			{
				bytes[n]=(byte)(baseOffset >>(baseOffsetSize-1-i)*8);
				n++;
			}
		
			//extnetCount
			for(int i=0; i<2; i++)
			{
				bytes[n]=(byte)(extentCount >>(1-i)*8);
				n++;
			}
		
			//extentInfos
			for(int i=0; i<extentCount; i++)
			{
				ExtentInfo extentInfo=(ExtentInfo)extentInfos.get( i);
				int extentInfoSize=extentInfo.GetExtentInfoSize();
				byte[] temp_extent=new byte[extentInfoSize];
				extentInfo.Serialize(temp_extent);
				
				for(int j=0; j<extentInfoSize; j++)
				{
					bytes[n]=temp_extent[j];
					n++;
				}
			}
		}
		catch(Exception ex)
		{
			System.out.println("ItemInfo:" + ex.toString() );
		}
		
		return n;
	}
}
