/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.mpeg21ffTE.boxes;
import java.util.ArrayList;

import org.iso.mpeg.mxm.mpeg21ffTE.constants.BoxType;
import org.iso.mpeg.mxm.mpeg21ffTE.utils.ArrayUtils;


/**
 * This class provides user methods to perform operations on ItemLocationBox. 
 * It is extended from FullBox
 * 
 * @author Xueqing Liu
 *	
 */
public class ItemLocationBox extends FullBox
{
	/**
	 * Reports the box type.
	 */
	public final String type = BoxType.ItemLocation;
	
	/**
	 * Specifies the offset size.
	 */
	public int offsetSize;
	
	/**
	 * Specifies the length size.
	 */
	public int lengthSize;
	
	/**
	 * Specifies the base offset size.
	 */
	public int baseOffsetSize;
	
	/**
	 * Reserved field.
	 */
	public int reserved=0;
	
	/**
	 * Item count in this box.
	 */
	public int itemCount=0;
	
	/**
	 * An arraylist which contains the ItemInfos in this box.
	 */
	public ArrayList<ItemInfo> itemInfos = new ArrayList<ItemInfo>();
	
    //*************************************************************
	
	/**
	 * Add ItemInfo object into an arraylist
	 * @param itemInfo the ItemInfo object which to be added
	 */
	public void AddItemInfo(ItemInfo itemInfo)
	{
		itemInfos.add(itemInfo);
	}
	
	/**
	 * Caculates the full size of this box in bytes
	 */
	public int GetBoxSize()
    {
		boxSize=0;
		for (int i=0; i<itemCount; i++)
		{
			ItemInfo itemInfo=(ItemInfo)itemInfos.get( i);
			boxSize=boxSize+itemInfo.GetItemInfoSize() ;
		}
	   
	    boxSize = boxSize + // ItemInfos' size
	    		  4 + //size
				  type.length() + 
				  1 + //version
		   		  3 + //flag
				  1 + //offsetSize + lengthSize
				  1 + //baseOffsetSize + reserved
				  2;  //itemCount

	    return boxSize;
    }	
	
	/**
	 * Put all data of this box into a byte array
	 */
	public int Serialize(byte[] bytes)
	{
		int n=0;

		try
		{
			//size
			byte[] temp=new byte[4];
        	ArrayUtils.IntToByteArray(size, temp);
        	for (int i = 0; i < temp.length ; i++)
            {
        		bytes[n] = (byte)temp[i];
        		n++;
            }

        	//type
			for (int i = 0; i < type.length() ; i++)
			{
				bytes[n] = (byte)type.charAt(i);
				n++;
			}

			//version
        	bytes[n]=version;
        	n++;
        	
        	//flag
        	for(int i=0; i<3; i++)
        	{
        		bytes[n]=flags[i];
        		n++;
        	}
        	
			//offsetSize + lengthSize
			bytes[n]= (byte)(offsetSize<<4 | lengthSize );
			n++;
		
			//baseOffsetSize + reserved
			bytes[n]=(byte)(baseOffsetSize <<4 | reserved);
			n++;
		
			//itemCount	
			for(int i=0; i<2; i++)
			{
				bytes[n]=(byte)(itemCount>>(1-i)*8);
				n++;
			}
		
			//itemInfo
			for(int i=0; i<itemCount; i++)
			{
				ItemInfo itemInfo=(ItemInfo)itemInfos.get(i);
				temp=new byte[itemInfo.GetItemInfoSize()];
				itemInfo.Serialize(temp);
			
				for(int j=0; j<itemInfo.GetItemInfoSize(); j++)
				{
					bytes[n]=temp[j];
					n++;
				}
			}
		}
		catch(Exception ex)
		{
			System.out.println("ItemLocationBox:" + ex.toString() );
		}
	 
		return n;
	}
	
	/**
     * Get box type
     * @return Box type
     */
    public String GetBoxType()
    {
    	return this.type;
    }
}
