/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.mpeg21ffTE.boxes;
import java.io.File;
import java.io.FileInputStream;

import org.iso.mpeg.mxm.mpeg21ffTE.constants.BoxType;
import org.iso.mpeg.mxm.mpeg21ffTE.utils.ArrayUtils;


/**
 * This class provides user methods to perform operations on MediaBox. 
 * It is extended from Box
 * 
 * @author Xueqing Liu
 *	
 */
public class MediaDataBox extends Box
{
	/**
	 * Reports the box type.
	 */
	 public final String  type = BoxType.MediaData;
	 
	 /**
	  * A byte array which contains the media data 
	  */
     private byte[] mediaData;
     
     /**
      * Media data file name
      */
     private String mediaDataFileName;
     
     /**
      * Media data file length
      */
     private int mediaDataSize;

     //*************************************************************
     
     /**
 	 * Calculates the box's size in byte
 	 */
     public int GetBoxSize()
     {
     	 boxSize = 4 + //size
 		   		   type.length();
     	
     	return boxSize;
     }
    
     public int GetBoxFullSize()
     {
    	 boxSize = 4 + //size
    	 		   type.length() +
    	 		   mediaDataSize;

    	 return boxSize;
     }
     
     
     /**
      * Get the media data from a file
      * @param fileName the name of the media file which contains the media data 
      */
     public void GetMediaDataFromFile(String fileName)
     {
    	 try
     	{
     		File file =new File(fileName);
     		FileInputStream fis=new FileInputStream(fileName);
     		mediaData=new byte[(int)file.length()];
     		fis.read(mediaData,0,(int)file.length());
     		fis.close();
     	}
     	catch (Exception ex)
     	{
     		System.out.println("MediaDataBox:GetMediaDataFromFile:" + ex.toString() );
     	}
     }
     
     /**
      * Put all data of this box into a byte array
      */
     public int Serialize(byte[] bytes)
     {
    	int n=0;
        
        try
        {
        	//size
        	byte[] temp=new byte[4];
        	ArrayUtils.IntToByteArray(size, temp);
        	for (int i = 0; i < temp.length ; i++)
            {
        		bytes[n] = (byte)temp[i];
        		n++;
            }

        	//type
        	for (int i = 0; i < type.length(); i++)
        	{
        		bytes[n] = (byte)type.charAt(i);
        		n++;
        	}
        	
        	/*       	
        	//mediaData
        	for(int i=0; i<mediaData.length; i++)
        	{
        		bytes[n]=mediaData[i];
        		n++;
        	}
        	*/
        }
        catch (Exception ex)
        {
        	System.out.println("MediaDataBox:" + ex.toString() );
        }

        return n;
    }

	public void setMediaDataFileName(String mediaDataFileName) {
		this.mediaDataFileName = mediaDataFileName;
		File  file=new File( mediaDataFileName);
		
		this.setMediaDataSize((int)file.length());
	}

	public String getMediaDataFileName() 
	{
		return mediaDataFileName;
	}
	
	public String GetBoxType()
	{
		return this.type;
	}

	private void setMediaDataSize(int mediaDataSize) 
	{
		this.mediaDataSize = mediaDataSize;
	}

	public int getMediaDataSize() {
		return mediaDataSize;
	}
	
	
}
