/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Xueqing Liu 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.mpeg21ffTE.utils;

public class TypeUtils 
{
	/**
	 * This method converts a integer string
	 * (i.e. this string only contains 1~9 ASCII chars)
	 * to a real integer type.
	 * @param str
	 * @return
	 */
	public static int StringToInt(String str)
	{
		int n=0;
		try
		{
			for(int i=str.length()-1; i>=0; i--)
			{
				char temp=str.charAt(i);
				n=n +  ( temp & 0x0f) *  (int)Math.pow(10,(str.length()-1-i));
			}
		}
		catch(Exception ex)
		{
			System.out .println("StringToInt:" + ex.toString());
		}
			
		return n;
	}
	
	/**
	 * This method converts an int to string
	 * (this string only contains 1~9 ASCII chars)
	 * @param str
	 * @return
	 */
	public static String IntToString(int value)
	{
		String str="";
		try
		{
			byte[] bytes=new byte[100];
			int quotient = value / 10;
			int remainder= value % 10;
			int n=0;
			bytes[n]=(byte)remainder;
			n++;
			while(quotient!=0)
			{
				remainder=quotient % 10;
				quotient =quotient / 10;
				bytes[n]=(byte)remainder;
				n++;
			}
			for(int i=n-1; i>=0; i--)
			{
				str = str + bytes[i];
			}
		}
		catch(Exception ex)
		{
			System.out .println("IntToString:" + ex.toString());
		}
			
		return str;
	}
	
	public static byte[] StringToHex(String str, int hexLength)
	{
		byte bytes[]=new byte[hexLength];
		byte temp[]=new byte[2];
		byte btemp=0;
		int k=0;
		for (int i=0; i<hexLength*2; i++)
		{
			btemp=(byte)str.charAt(i);
			if(btemp<='9' && btemp>='0') btemp=(byte)((int)((byte)btemp) - 0x30);
			else if(btemp<='F' && btemp>='A') btemp=(byte)((int)((byte)btemp) - 0x37);
			else if(btemp<='f' && btemp>='a') btemp=(byte)((int)((byte)btemp)- 0x57);

			if((i % 2)==0) 
			{
				temp[0]=(byte)((int)btemp << 4);//first byte
			}
			else
			{
				temp[1]=(byte)btemp;//second byte
				bytes[k]=(byte)(temp[0]+temp[1]);//combine two bytes into one byte
				k++;
			}
		}
		return bytes;
	}
	
	public static String HexToString(byte[] bytes, int hexLength)
	{
		String str="";
		byte btemp=0;
		byte btemp1=0;
		byte btemp2=0;
		for(int i=0; i<hexLength; i++)
		{
			btemp=bytes[i];
			btemp1=(byte)((int)(btemp & 0xf0)>>4);
			btemp2=(byte)((int)btemp & 0x0f);
		
			if((btemp1<=0x9) && (btemp1>=0x0))
			{
				btemp1=(byte)((int)btemp1 + 0x30);
			}
			else if(btemp1<=0xf && btemp1>=0xa) 
			{
				btemp1=(byte)((int)btemp1 + 0x37);
			}
			
			if(btemp2<=0x9 && btemp2>=0x0)
			{
				btemp2=(byte)((int)btemp2 + 0x30);
			}
			else if(btemp2<=0xf && btemp2>=0xa) 
			{
				btemp2=(byte)((int)btemp2 + 0x37);
			}

			str=str+(char)btemp1;
			str=str+(char)btemp2;
		}
		return str;
	}
	
	public static byte[] StringToHex_m(String str)
	{
		byte bytes[]=new byte[str.length()];
		byte btemp=0;
		int k=0;
		for (int i=0; i<str.length(); i++)
		{
			btemp=(byte)str.charAt(i);
			if(btemp<='9' && btemp>='0') btemp=(byte)((int)((byte)btemp) - 0x30);
			else if(btemp<='F' && btemp>='A') btemp=(byte)((int)((byte)btemp) - 0x37);
			else if(btemp<='f' && btemp>='a') btemp=(byte)((int)((byte)btemp)- 0x57);

			bytes[i]=btemp;
		}
		return bytes;
	}
	
	public static String HexToString_m(byte[] bytes)
	{
		String str="";
		char btemp=0;
		for(int i=0; i<bytes.length; i++)
		{
			btemp=(char)bytes[i];
			
			if((btemp<=0x9) && (btemp>=0x0))
			{
				btemp=(char)((int)btemp + 0x30);
			}
			else if(btemp<=0xf && btemp>=0xa) 
			{
				btemp=(char)((int)btemp + 0x37);
			}
			str=str + btemp;
			
		}
		return str;
	}
}
