/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.mpeg21ffTE.test;


import java.io.File;

import junit.framework.TestCase;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.tEngine.mp21ffTE.MPEG21FileCreator;
import org.iso.mpeg.mxm.tEngine.mp21ffTE.MPEG21FileParser;
import org.iso.mpeg.mxm.tEngine.mp21ffTE.exceptions.MPEG21FileEngineException;
import org.iso.mpeg.mxm.mpeg21ffTE.MPEG21FileEngine;
import org.junit.Assert;

public class MPEG21FileEngineTest extends TestCase {

	private Logger logger = Logger.getLogger(MPEG21FileEngineTest.class);
	
	private final String OUTPUT_DCF_FILE_NAME = System.getProperty("java.io.tmpdir") + File.separator + "testDCF.mp21";
	private final String OUTPUT_DCI_FILE_NAME = System.getProperty("java.io.tmpdir") + File.separator + "testDCI.xml";
	private final String OUTPUT_MEDIA_RESOURCE_FILE_NAME = System.getProperty("java.io.tmpdir") + File.separator + "chillout_small.jpg";

	private final String SOURCE_DCI_FILE_NAME = "testDCI.xml";
	private final String SOURCE_MEDIA_RESOURCE_FILE_NAME = "chillout_small.jpg";
	private final String MEDIA_RESOURCE_ITEM_ID = "B1";
	
	
	public void testDCFCreation() throws Exception {
		logger.info("\n\n---------- testDCFCreation ----------");
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		MPEG21FileEngine fileEngine = (MPEG21FileEngine)mxm.getDefaultEngine(MXMEngineName.MPEG21FileTE.toString());
		MPEG21FileCreator dcfCreator = fileEngine.getMPEG21FileCreator();
		
		dcfCreator.setDIPathName(getClass().getClassLoader().getResource(SOURCE_DCI_FILE_NAME).getFile());
		dcfCreator.addResource(getClass().getClassLoader().getResource(SOURCE_MEDIA_RESOURCE_FILE_NAME).getFile(), MEDIA_RESOURCE_ITEM_ID);
		dcfCreator.createMPEG21File(OUTPUT_DCF_FILE_NAME);
		
		Assert.assertEquals(new Long(new File(OUTPUT_DCF_FILE_NAME).length()), new Long(9479));

		logger.info("The DCF file was successfully created in file: " + OUTPUT_DCF_FILE_NAME);
	}
	
	public void testDCFParsing() throws Exception {
		logger.info("\n\n---------- testDCFParsing ----------");
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		MPEG21FileEngine fileEngine = (MPEG21FileEngine)mxm.getDefaultEngine(MXMEngineName.MPEG21FileTE.toString());
		MPEG21FileParser dcfParser = fileEngine.getMPEG21FileParser();
		
		//Extract the DCI and compare it to the original
		dcfParser.getDCI(OUTPUT_DCF_FILE_NAME, OUTPUT_DCI_FILE_NAME);
		File sourceDCI = new File(getClass().getClassLoader().getResource(SOURCE_DCI_FILE_NAME).getFile());
		File outputDCI = new File(OUTPUT_DCI_FILE_NAME);
		
		if (FileUtils.contentEquals(sourceDCI, outputDCI))
			logger.info("The DCI: " + outputDCI.getAbsolutePath() + " was correctly extracted from the DCF");
		else
			throw new MPEG21FileEngineException("The source and output DCIs are different!");

		//Extract the Resource and compare it to the original
		dcfParser.getMediaData(MEDIA_RESOURCE_ITEM_ID, OUTPUT_MEDIA_RESOURCE_FILE_NAME);
		File sourceResource = new File(getClass().getClassLoader().getResource(SOURCE_MEDIA_RESOURCE_FILE_NAME).getFile());
		File outputResource = new File(OUTPUT_MEDIA_RESOURCE_FILE_NAME);
		
		if (FileUtils.contentEquals(sourceResource, outputResource))
			logger.info("The Media Resource: " + outputDCI.getAbsolutePath() + " was correctly extracted from the DCF");
		else
			throw new MPEG21FileEngineException("The source and output resources are different!");
		
		logger.info("The DCF file was successfully parsed");
	}


}
