/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.orchestrator;


import org.iso.mpeg.mxm.core.MXMAbstractEngine;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.orchestratorEngine.authenticatecontent.AuthenticateContentEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.authenticateuser.AuthenticateUserEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.authorizeuser.AuthorizeUserEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.checkwithlicense.CheckWithLicenseEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.createcontent.CreateContentEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.createlicense.CreateLicenseEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.describecontent.DescribeContentEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.identifycontent.IdentifyContentEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.packagecontent.PackageContentEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.unpublishcontent.UnpublishContentEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.identifydevice.IdentifyDeviceEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.identifyuser.IdentifyUserEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.postcontent.PostContentEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.requestevent.RequestEventEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.revokelicense.RevokeLicenseEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.searchlicense.SearchLicenseEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.storeevent.StoreEventEngineOrchestrator;
import org.iso.mpeg.mxm.orchestratorEngine.storelicense.StoreLicenseEngineOrchestrator;
import org.iso.mpeg.mxm.pEngine.authenticatecontentPE.AuthenticateContentEngine;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.AuthenticateUserEngine;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.AuthorizeUserEngine;
import org.iso.mpeg.mxm.pEngine.checkwithlicensePE.CheckWithLicenseEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine;
import org.iso.mpeg.mxm.pEngine.createlicensePE.CreateLicenseEngine;
import org.iso.mpeg.mxm.pEngine.describecontentPE.DescribeContentEngine;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.IdentifyContentPE;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.PackageContentPE;
import org.iso.mpeg.mxm.pEngine.unpublishcontentPE.UnpublishContentPE;
import org.iso.mpeg.mxm.pEngine.identifydevicePE.IdentifyDeviceEngine;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.IdentifyUserEngine;
import org.iso.mpeg.mxm.pEngine.postcontentPE.PostContentEngine;
import org.iso.mpeg.mxm.pEngine.requesteventPE.RequestEventEngine;
import org.iso.mpeg.mxm.pEngine.revokelicensePE.RevokeLicenseEngine;
import org.iso.mpeg.mxm.pEngine.searchlicensePE.SearchLicenseEngine;
import org.iso.mpeg.mxm.pEngine.storeeventPE.StoreEventEngine;
import org.iso.mpeg.mxm.pEngine.storelicensePE.StoreLicenseEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

import java.util.List;


/**
 * @author Angelos-Christos Anadiotis
 */
public class OrchestratorEngine extends MXMAbstractEngine implements
    org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine
{
    // PEs
    protected CreateContentEngine       createContentEngine;
    protected IdentifyUserEngine        identifyUserEngine;
    protected RequestEventEngine        requestEventEngine;
    protected PostContentEngine         postContentEngine;
    protected CreateLicenseEngine       createLicenseEngine;
    protected StoreEventEngine          storeEventEngine;
    protected IdentifyContentPE         identifyContentEngine;
    protected IdentifyDeviceEngine      identifyDeviceEngine;
    protected AuthenticateContentEngine authenticateContentEngine;
    protected AuthorizeUserEngine       authorizeUserEngine;
    protected CheckWithLicenseEngine    checkWithLicenseEngine;
    protected DescribeContentEngine     describeContentEngine;
    protected AuthenticateUserEngine    authenticateUserEngine;
    protected StoreLicenseEngine 		storeLicenseEngine;
    protected RevokeLicenseEngine		revokeLicenseEngine;
    protected SearchLicenseEngine		searchLicenseEngine;
    protected PackageContentPE          packageContentEngine;
    protected UnpublishContentPE        unpublishContentEngine;
    
    // TEs
	protected RELEngine relEngine;
	protected EventReportEngine erEngine;
	protected DIEngine diEngine;
	protected MetadataEngine metadataEngine;
	protected IPMPEngine ipmpEngine;
	protected SecurityEngine securityEngine;
    
	/*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#getPostContentEngineOrchestrator
     * ()
     */
    @Override
    public PostContentEngineOrchestrator getPostContentEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.postcontent.PostContentEngineOrchestrator(
            postContentEngine);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#
     * getRequestEventEngineOrchestrator()
     */
    @Override
    public RequestEventEngineOrchestrator getRequestEventEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.requestevent.RequestEventEngineOrchestrator(
            requestEventEngine, erEngine);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#
     * getCreateLicenseEngineOrchestrator()
     */
    @Override
    public CreateLicenseEngineOrchestrator getCreateLicenseEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.createlicense.CreateLicenseEngineOrchestrator(
            createLicenseEngine, relEngine);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.core.MXMEngine#getEngineName()
     */
    @Override
    public String getEngineName()
    {
        return MXMEngineName.Orchestrator.value();
    }

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMEngine#setDependencies(java.util.List)
	 */
	@Override
	public void setDependencies(List<MXMEngine> engines)
			throws MXMEngineException
	{
		if (engines != null)
		{
			for (MXMEngine engine : engines)
			{
				String engineName = engine.getEngineName();
				
				// Protocol Engines
				if (engineName.equals(MXMEngineName.CreateContentPE.value()))
				{
					this.createContentEngine = (CreateContentEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.IdentifyUserPE.value()))
				{
					this.identifyUserEngine = (IdentifyUserEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.RequestEventPE.value()))
				{
					this.requestEventEngine = (RequestEventEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.PostContentPE.value()))
				{
					this.postContentEngine = (PostContentEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.CreateLicensePE.value()))
				{
					this.createLicenseEngine = (CreateLicenseEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.StoreEventPE.value()))
				{
					this.storeEventEngine = (StoreEventEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.IdentifyContentPE.value()))
				{
					this.identifyContentEngine=(IdentifyContentPE)engine;
				}
				else if (engineName.equals(MXMEngineName.AuthenticateContentPE.value()))
				{
					this.authenticateContentEngine=(AuthenticateContentEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.CheckWithLicensePE.value()))
				{
					this.checkWithLicenseEngine = (CheckWithLicenseEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.DescribeContentPE.value()))
				{
					this.describeContentEngine = (DescribeContentEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.AuthenticateUserPE.value()))
                {
                    this.authenticateUserEngine = (AuthenticateUserEngine)engine;
                }
				else if (engineName.equals(MXMEngineName.StoreLicensePE.value())) 
				{
					this.storeLicenseEngine = (StoreLicenseEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.RevokeLicensePE.value()))
				{
					this.revokeLicenseEngine = (RevokeLicenseEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.SearchLicensePE.value())) 
				{
					this.searchLicenseEngine = (SearchLicenseEngine)engine;
				}				


				// Technology Engines
				else if (engineName.equals(MXMEngineName.RELTE.value()))
				{
					this.relEngine = (RELEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.EventReportTE.value()))
				{
					this.erEngine = (EventReportEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.DigitalItemTE.value()))
				{
					this.diEngine = (DIEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.MetadataTE.value()))
				{
					this.metadataEngine = (MetadataEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.IPMPTE.value()))
				{
					this.ipmpEngine = (IPMPEngine)engine;
				}
				else if (engineName.equals(MXMEngineName.SecurityTE.value()))
				{
					this.securityEngine = (SecurityEngine)engine;
				}
            }
        }
    }

    @Override
    public IdentifyContentEngineOrchestrator getIdentifyContentEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.identifycontent.IdentifyContentEngineOrchestrator(
            identifyContentEngine);
    }

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#getCreateContentEngineOrchestrator()
	 */
	@Override
	public CreateContentEngineOrchestrator getCreateContentEngineOrchestrator()
	{
		return new org.iso.mpeg.mxm.orchestrator.createcontent.CreateContentEngineOrchestrator(createContentEngine, diEngine);
	}

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#
     * getIdentifyUserEngineOrchestrator()
     */
    @Override
    public IdentifyUserEngineOrchestrator getIdentifyUserEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.identifyuser.IdentifyUserEngineOrchestrator(
            identifyUserEngine, securityEngine, this.metadataEngine);
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#getStoreEventEngineOrchestrator
     * ()
     */
    @Override
    public StoreEventEngineOrchestrator getStoreEventEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.storeevent.StoreEventEngineOrchestrator(
            storeEventEngine, erEngine);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#
     * getIdentifyDeviceEngineOrchestrator()
     */
    @Override
    public IdentifyDeviceEngineOrchestrator getIdentifyDeviceEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.identifydevice.IdentifyDeviceEngineOrchestrator(
            identifyDeviceEngine);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#
     * getAuthenticateContentEngineOrchestrator()
     */
    @Override
    public AuthenticateContentEngineOrchestrator getAuthenticateContentEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.authenticatecontent.AuthenticateContentEngineOrchestrator(
            authenticateContentEngine, securityEngine);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#
     * getAuthorizeUserEngineOrchestrator()
     */
    @Override
    public AuthorizeUserEngineOrchestrator getAuthorizeUserEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.authorizeuser.AuthorizeUserEngineOrchestrator(
            authorizeUserEngine, relEngine);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#
     * getCheckWithLicenseEngineOrchestrator()
     */
    @Override
    public CheckWithLicenseEngineOrchestrator getCheckWithLicenseEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.checkwithlicense.CheckWithLicenseEngineOrchestrator(
            checkWithLicenseEngine);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#
     * getDescribeContentEngineOrchestrator()
     */
    @Override
    public DescribeContentEngineOrchestrator getDescribeContentEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.describecontent.DescribeContentEngineOrchestrator(describeContentEngine, metadataEngine);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine#
     * getAuthenticateUserEngineOrchestrator()
     */
    @Override
    public AuthenticateUserEngineOrchestrator getAuthenticateUserEngineOrchestrator()
    {
        return new org.iso.mpeg.mxm.orchestrator.authenticateuser.AuthenticateUserEngineOrchestrator(
            authenticateUserEngine, securityEngine);
    }

	@Override
	public StoreLicenseEngineOrchestrator getStoreLicenseEngineOrchestrator() 
	{
		return new	org.iso.mpeg.mxm.orchestrator.storelicense.StoreLicenseEngineOrchestrator(storeLicenseEngine);
	}
	
	@Override
	public RevokeLicenseEngineOrchestrator getRevokeLicenseEngineOrchestrator()
	{
		return new org.iso.mpeg.mxm.orchestrator.revokelicense.RevokeLicenseEngineOrchestrator(revokeLicenseEngine);
	}

	@Override
	public SearchLicenseEngineOrchestrator getSearchLicenseEngineOrchestrator()
	{
		return new org.iso.mpeg.mxm.orchestrator.searchlicense.SearchLicenseEngineOrchestrator(searchLicenseEngine);
	}

	@Override
    public PackageContentEngineOrchestrator getPackageContentEngineOrchestrator()
    {
        //return new org.iso.mpeg.mxm.orchestrator.identifycontent.IdentifyContentEngineOrchestrator(identifyContentEngine);
		//return new org.iso.mpeg.mxm.orchestrator.packagecontent.PackageContentEngineOrchestrator(packageContentEngine);
		return null;
    }
	
	@Override
    public UnpublishContentEngineOrchestrator getUnpublishContentEngineOrchestrator()
    {
        //return new org.iso.mpeg.mxm.orchestrator.identifycontent.IdentifyContentEngineOrchestrator(identifyContentEngine);
		//return new org.iso.mpeg.mxm.orchestrator.packagecontent.PackageContentEngineOrchestrator(packageContentEngine);
		return null;
    }
	
}
