/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.orchestrator.authenticateuser;


import java.io.IOException;
import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPublicKeySpec;
import java.util.Date;
import java.util.Dictionary;
import java.util.HashMap;

import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.xerces.impl.dv.util.Base64;
import org.hamcrest.core.IsInstanceOf;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.AuthenticateUserEngine;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserRequest;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserResponse;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ApplicationSpecificData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Assertion;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnContext;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnStatement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Conditions;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.NameID;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.SubjectConfirmation;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AuthnRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.Response;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.UserDataBaseManager;
import org.w3c.dom.Document;


/**
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 */
public class AuthenticateUserEngineOrchestrator
    implements
    org.iso.mpeg.mxm.orchestratorEngine.authenticateuser.AuthenticateUserEngineOrchestrator
{
    private AuthenticateUserEngine         authenticateUserEngine;
    private SecurityEngine                 securityTE;
    private static HashMap<String, String> map = new HashMap<String, String>();

    public AuthenticateUserEngineOrchestrator(
        AuthenticateUserEngine authenticateUserEngine, SecurityEngine securityTE)
    {
        this.authenticateUserEngine = authenticateUserEngine;
        this.securityTE = securityTE;
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.orchestratorEngine.authenticateuser.AuthenticateUserEngineOrchestrator
     * #getAuthenticateUserEngine()
     */
    @Override
    public AuthenticateUserEngine getAuthenticateUserEngine()
    {
        return authenticateUserEngine;
    }

    @Override
    public String getChallangeFor(PublicKey key)
    {
        String hashcodeForPublicKey = null;
        try
        {
            hashcodeForPublicKey = this.securityTE.getSecurityEngineTechnologyHandler()
                .getFileAndObjectHandler().getString(key).hashCode()
                + "";
        }
        catch (SecurityEngineException e)
        {
            e.printStackTrace();
        }
        catch (MXMEngineException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        int random = (int)Math.round(Math.random() * 9999);
        String challange = random + "000000000000";
        challange = challange + challange;
        // challange = challange.substring(0, challange.length() - 2);
        map.put(hashcodeForPublicKey, challange);
        return challange;
    }

    

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.orchestratorEngine.authenticateuser.AuthenticateUserEngineOrchestrator
     * #authenticateUser(org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.
     * AuthenticateUserRequest)
     */
    @Override
    public AuthenticateUserResponse authenticateUser(
        AuthenticateUserRequest authenticateUserRequest) throws MXMEngineException
    {
        //recive request from browser
        //get assertion from the request
        Assertion assertion = authenticateUserRequest.getAssertion();
        AuthenticateUserResponse authenticateUserResponse = authenticateUserEngine
                .getAuthenticateUserSchemaHandler().getAuthenticateUserResponse();
       
        try
        {
            //vaidate if assertion is really from the identify provider
            boolean valid = this.isValidAssertion(assertion);
            if(valid) {
                String userId = assertion.getID();
                ApplicationSpecificData data = authenticateUserEngine.getBaseProtocolSchemaHandler().getApplicationSpecificData();
                data.setAny("SOME DUMMY DATA REQUESTED BY CLIENT :"+userId);
                authenticateUserResponse.addApplicationSpecificData(data);
            }
        }
        catch (MXMException e1)
        {
            e1.printStackTrace();
        }
        //return resposne
        return authenticateUserResponse;
        
    }
    
    /**
     * Service provider method
     * Returns true if the SAML response is valid, which mean following request is allowed to acess the resource
     * @param response
     * @return
     * @throws MXMException 
     */
    @Override
    public boolean isAuthorised(AuthenticateUserResponse response) throws MXMException {
        if(response.getResponse().getAssertionOrEncryptedAssertion().size() == 0) {
            return false;
        }
        Assertion assertion = null;
        
        for(int i = 0;i< response.getResponse().getAssertionOrEncryptedAssertion().size();i++) {
            Object object = response.getResponse().getAssertionOrEncryptedAssertion().get(i);
            if(object.getClass().getCanonicalName().equals(org.iso.mpeg.mxm.securityTE.schemahandler.saml.Assertion.class.getCanonicalName()) ) {
                assertion = (Assertion)object;
                if(isValidAssertion(assertion)) {
                    return true;
                }
            }
        }
     
        return false;
    }
    
    
    private boolean isValidAssertion(Assertion assertion) throws MXMException {
        //verofy signature
        Signature signature = assertion.getSignature();
        boolean verified = false;
        if(signature != null) {
          //  todo:
           // Document doc = (Document)assertion.createObject(Document.class.getCanonicalName()).getContent();
            //verify against the publci key of the identity provider
            int hashcode = assertion.getIssueInstant().getMillisecond();
            verified = this.securityTE.getSecurityEngineTechnologyHandler().getSignatureManager().verifySignature((hashcode+"").getBytes(), signature);
        }
        return verified;
    }
    

}
