/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *      this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *      this list of conditions and the following disclaimer in the documentation 
 *      and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *      may be used to endorse or promote products derived from this software 
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.orchestrator.authorizeuser;

import java.util.Date;

import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.AuthorizeUserEngine;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.schemahandler.AuthorizeUserRequest;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.schemahandler.AuthorizeUserResponse;
import org.iso.mpeg.mxm.pEngine.authorizeuserPE.schemahandler.AuthorizeUserSuccess;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Principal;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Right;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.AuthorisationManager;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.AuthorisationResult;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.RELRight;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class AuthorizeUserEngineOrchestrator implements org.iso.mpeg.mxm.orchestratorEngine.authorizeuser.AuthorizeUserEngineOrchestrator
{
    private AuthorizeUserEngine authorizeUserEngine;
    private RELEngine relEngine;
    
    public AuthorizeUserEngineOrchestrator(AuthorizeUserEngine authorizeUserEngine, RELEngine relEngine)
    {
        this.authorizeUserEngine = authorizeUserEngine;
        this.relEngine = relEngine;
    }

    /* (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.authorizeuser.AuthorizeUserEngineOrchestrator#getAuthorizeUserEngine()
     */
    @Override
    public AuthorizeUserEngine getAuthorizeUserEngine()
    {
        return authorizeUserEngine;
    }

    /* (non-Javadoc)
     * @see org.iso.mpeg.mxm.orchestratorEngine.authorizeuser.AuthorizeUserEngineOrchestrator#authorizeUser(org.iso.mpeg.mxm.pEngine.authorizeuserPE.schemahandler.AuthorizeUserRequest)
     */
    @Override
    public AuthorizeUserResponse authorizeUser(AuthorizeUserRequest request) throws MXMEngineException
    {
        AuthorisationManager authorisationManager = relEngine.getRELEngineTechnologyHandler().getAuthorisationManager();
        
        LicenseEntity licenseEntity = request.getLicenseEntity();
        Principal user = request.getPrincipal();
        Right right = request.getRights().get(0);
        Resource resource = request.getResources().get(0);
        
        AuthorisationResult authorisationResult = authorisationManager.authoriseUser(
        		licenseEntity.getLicense(), user, RELRight.valueOf(right.getClass().getSimpleName().toLowerCase()), resource);
        
        AuthorizeUserResponse authorizeUserResponse = 
        		authorizeUserEngine.getAuthorizeUserSchemaHandler().getAuthorizeUserResponse();
        
        authorizeUserResponse.setTimestamp(new Date());
        authorizeUserResponse.setTransactionIdentifier(request.getTransactionIdentifier());
        
        if (authorisationResult.equals(AuthorisationResult.AUTHORISATION_SUCCEEDED))
        {
        	AuthorizeUserSuccess authorizeUserSuccess = 
        			authorizeUserEngine.getAuthorizeUserSchemaHandler().getAuthorizeUserSuccess();
        	try
			{
				authorizeUserResponse.setAuthorizeUserSuccess(authorizeUserSuccess);
			} catch (MXMException e)
			{
				e.printStackTrace();
			}
        }
        else
        {
        	ProtocolFailure protocolFailure = 
        			authorizeUserEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
        	protocolFailure.setDisplayString(authorisationResult.toString());
        	try
			{
				authorizeUserResponse.setProtocolFailure(protocolFailure);
			} catch (MXMException e)
			{
				e.printStackTrace();
			}
        }
        
        return authorizeUserResponse;
    }

}
