/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.orchestrator.createcontent;


import java.util.UUID;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.exceptions.CreateContentEngineException;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSuccess;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;

/**
 * @author Angelos-Christos Anadiotis
 * @author Angelo Difino 
 *
 */
public class CreateContentEngineOrchestrator implements org.iso.mpeg.mxm.orchestratorEngine.createcontent.CreateContentEngineOrchestrator
{
	private Logger logger = Logger.getLogger(this.getClass());//
	
	private CreateContentEngine createContentEngine;
	private DIEngine diengine;
	
	public CreateContentEngineOrchestrator(CreateContentEngine createContentEngine, DIEngine diengine)
	{
		this.createContentEngine = createContentEngine;
		this.diengine=diengine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.createcontent.CreateContentEngineOrchestrator#getCreateContentEngine()
	 */
	@Override
	public CreateContentEngine getCreateContentEngine()
	{
		return createContentEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.createcontent.CreateContentEngineOrchestrator#createContent(org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest)
	 */
	@Override
	public CreateContentResponse createContent(CreateContentRequest request) throws CreateContentEngineException
	{
		try{
			logger.info("Create Content orchestrator invoked");
			logger.info("Transaction ID is:"+request.getTransactionIdentifier());		
			//a content entity to return is made of:
			//-signature
			//-top level item (id=A0)										v
			//-----------signature
			//-----------identifier											v
			//-----------meta												v
			//-----------inner item (id=Bx)									v
			//--------------------signature
			//--------------------ev report									v
			//--------------------id										v
			//--------------------meta										v
			//--------------------ipmp										v
			//--------------------appl-specific
			//--------------------resource									v
			
			
			CreateContentResponse response=createContentEngine.getCreateContentSchemaHandler().getCreateContentResponse();
			CreateContentSuccess success=createContentEngine.getCreateContentSchemaHandler().getCreateContentSuccess();
			DIDL didl=diengine.getDIEngineSchemaHandler().getDIDL();
			Item topLevelItem=diengine.getDIEngineSchemaHandler().getItem();
			topLevelItem.setId("A0");
			topLevelItem.setIdentifier("identifier:content:temp:" + UUID.randomUUID());	
			
			if (request.getMetadata()!=null)
			{
				topLevelItem.addMetadataDescriptor(request.getMetadata());
			}
			
			int i=0;
			for (Asset a:request.getAsset()){
				Item innerItem=diengine.getDIEngineSchemaHandler().getItem();
				innerItem.setId("B"+i);
				innerItem.setIdentifier("identifier:contentelement:temp:" + UUID.randomUUID());
				
				for (ERR err:a.getERRs()){
					innerItem.addERR(err);
				}
				
				if (a.getMetadata()!=null)
				{
					innerItem.addMetadataDescriptor(a.getMetadata());
				}
				
				if (a.getIPMPGeneralInfoDescriptor()!=null)
				{
					innerItem.addIPMPGeneralInfoDescriptor(a.getIPMPGeneralInfoDescriptor());
				}
				
				try
				{
					Component c=diengine.getDIEngineSchemaHandler().getComponent();
					c.addResource((org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource)a.getResource());
					innerItem.addComponent(c);
				}
				catch(Exception ex)
				{
					logger.error("Either the resource is incompatible or non existent", ex);
				}
				
				topLevelItem.addItem(innerItem);
			}
			
			didl.setMPEGMItem(topLevelItem);
			
			ContentEntity ce=createContentEngine.getBaseProtocolSchemaHandler().getContentEntity();
			ce.setDIDL(didl);
			success.setContentEntity(ce);
	
			response.setTransactionIdentifier(request.getTransactionIdentifier());
			response.setCreateContentSuccess(success);
			logger.info("Create Content orchestrator ended");
			return response;
		}
		catch (Exception e){
			e.printStackTrace();
			throw new CreateContentEngineException(e.getMessage(), e);
		}
	}

}
