/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.orchestrator.createlicense;

import java.util.UUID;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.pEngine.createlicensePE.CreateLicenseEngine;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseResponse;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseSuccess;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CustomLicense;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.RightGroup;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.ForAll;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Grant;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Issuer;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Principal;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.w3c.dom.Document;


public class CreateLicenseEngineOrchestrator implements org.iso.mpeg.mxm.orchestratorEngine.createlicense.CreateLicenseEngineOrchestrator
{
	private CreateLicenseEngine createLicenseEngine;
	private RELEngine relEngine;
	private SecurityEngine securityEngine;
	
	public CreateLicenseEngineOrchestrator(CreateLicenseEngine createLicenseEngine, RELEngine relEngine)
	{
		this.createLicenseEngine = createLicenseEngine;
		this.relEngine = relEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.createlicense.CreateLicenseEngineOrchestrator#getCreateLicenseEngine()
	 */
	@Override
	public CreateLicenseEngine getCreateLicenseEngine()
	{
		return createLicenseEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.createlicense.CreateLicenseEngineOrchestrator#createLicense(org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest)
	 */
	@Override
	public CreateLicenseResponse createLicense(CreateLicenseRequest createLicenseRequest)
	{
		CreateLicenseResponse createLicenseResponse = createLicenseEngine.getCreateLicenseSchemaHandler().getCreateLicenseResponse();
		
		try
		{
//			System.out.println(createLicenseRequest.createObject(String.class.getCanonicalName()).getContent());
			
			if (createLicenseRequest.getCustomLicense()!=null)
			{
				CustomLicense customLicense = createLicenseRequest.getCustomLicense();
				
				// input data
				ForAll forAll = null;
				Principal principal;
				
				// the output license
				License license = relEngine.getRELEngineSchemaHandler().getLicense();
				
				// creating principal if forAll is passed
				if (customLicense.getForAll()!=null)
				{
					forAll = customLicense.getForAll();
					
					principal = relEngine.getRELEngineSchemaHandler().getPrincipal();
					
					if (forAll.getVarName()!=null)
					{
						principal.setVarRef(forAll.getVarName());
					}
					else
					{
						String varStr = UUID.randomUUID() +"";
						
						forAll.setVarName(varStr);
						principal.setVarRef(varStr);
					}
				}
				else // it has a principal
				{
					principal = customLicense.getPrincipal();
				}
				
				// must create a grant for each right group
				for (RightGroup rightGroup : customLicense.getRightGroup())
				{
					// creating the grant
					Grant grant = relEngine.getRELEngineSchemaHandler().getGrant();
					
					// setting principal and forall
					grant.setPrincipal(principal);
					
					if (forAll!=null)
					{
						grant.addForAll(forAll);
					}
					
					// setting the condition
					if (rightGroup.getCondition()!=null)
					{
						grant.setCondition(rightGroup.getCondition());
					}
					
					// setting the right
					grant.setRight(rightGroup.getRight());
					
					// setting the resource
					grant.setResource(customLicense.getResource());
					
					// add the grant
					license.addGrant(grant);
				}
				
				// add license info
				// TODO UniqueIDType licenseInfo = customLicense.getLicenseIdentifier();
//				OtherInfo otherInfo = new org.iso.mpeg.mxm.engine.relTE.schema.relr.License.OtherInfo();
//				otherInfo.getAny().add(licenseInfo);
				
				// add remote issuer if passed
				if (customLicense.getIssuer()!=null)
				{
					license.addIssuer(customLicense.getIssuer());
				}
				
				// create local issuer TODO not required, a license could have no issuers
				Signature signature = null;
//				signature = securityEngine.getSecurityEngineTechnologyHandler().getSignatureManager().
//							generateSignature(license.createObject(Document.class.getCanonicalName()).getContent(), "DUMMY_ALGORITHM", null, "DUMMY_PASSWORD");
				
				Issuer localIssuer = relEngine.getRELEngineSchemaHandler().getIssuer();
				localIssuer.setSignature(signature);
				
				license.addIssuer(localIssuer);
				
				CreateLicenseSuccess createLicenseSuccess = createLicenseEngine.getCreateLicenseSchemaHandler().getCreateLicenseSuccess();
				
				LicenseEntity licenseEntity = createLicenseEngine.getBaseProtocolSchemaHandler().getLicenseEntity();
				licenseEntity.setLicense(license);
				
				createLicenseSuccess.setLicenseEntity(licenseEntity);
				
				createLicenseResponse.setCreateLicenseSuccess(createLicenseSuccess);
				
//				System.out.println(createLicenseResponse.createObject(String.class.getCanonicalName()).getContent());
				
				return createLicenseResponse;
			}
			else if (createLicenseRequest.getTemplateLicense()!=null)
			{
				// TODO
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		try
		{
			// TODO create Protocol Failure
			ProtocolFailure protocolFailure = null;
			
			createLicenseResponse.setCreateLicenseFailure(protocolFailure);
			
			// TODO may add a signature to the request
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return createLicenseResponse;
	}

}
