/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.orchestrator.describecontent;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.pEngine.describecontentPE.DescribeContentEngine;
import org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.GetDescribeContentRequest;
import org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.GetDescribeContentResponse;
import org.iso.mpeg.mxm.pEngine.describeentityPE.schemahandler.GetDescribeEntitySuccess;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.fru.Query;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.AccessUnit;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.FragmentUpdateContext;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.FragmentUpdatePayload;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.FragmentUpdateUnit;
import org.iso.mpeg.mxm.tEngine.metadataTE.technologyhandler.MetadataRepository;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * @author Az Mousas <azmousas@icbnet.ece.ntua.gr>
 *
 */
public class GetDescribeContentEngineOrchestrator implements org.iso.mpeg.mxm.orchestratorEngine.describecontent.GetDescribeContentEngineOrchestrator
{
	private DescribeContentEngine describeContentEngine;
	private MetadataEngine metadataEngine;
	
	public GetDescribeContentEngineOrchestrator(DescribeContentEngine describeContentEngine, MetadataEngine metadataEngine)
	{
		this.describeContentEngine = describeContentEngine;
		this.metadataEngine = metadataEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.describecontent.GetDescribeContentEngineOrchestrator#getDescribeContent(org.iso.mpeg.mxm.pEngine.describecontentPE.schemahandler.GetDescribeContentRequest)
	 */
	@Override
	public GetDescribeContentResponse getDescribeContent(GetDescribeContentRequest describeContentRequest)
	{
		GetDescribeContentResponse getDescribeContentResponse = (GetDescribeContentResponse)
				describeContentEngine.getDescribeContentSchemaHandler().getGetDescribeEntityResponse();
		
		getDescribeContentResponse.setTransactionIdentifier(describeContentRequest.getTransactionIdentifier());
		
		try
		{
			String contentId = describeContentRequest.getEntityIdentifier();
	
			AccessUnit accessUnit = metadataEngine.getMetadataEngineSchemaHandler().getAccessUnit(); // this contains responses for each query
			
			// xPath queries
			List<Query> queryList = new ArrayList<Query>();
			
			if (describeContentRequest.getFRU().getQuery().size()==0) // neither '/' nor '//' work for selecting the root node. '/*' works though
			{
				Query rootQuery = metadataEngine.getMetadataEngineSchemaHandler().getQuery();
				rootQuery.setValue("");
				
				queryList.add(rootQuery);
			}
			else
			{
				queryList.addAll(describeContentRequest.getFRU().getQuery());
			}
			
			for (Query query : queryList)
			{
//				fru.getXMLPull(); // TODO The query could have this instead of xPath queries
								  // 	  but queries can effectively substitute it. Also noted in the standard
//				describeContentRequest.getSupportedMetadataSchema(); // TODO multiple descriptions per schema could exist 
				
				String xPath = query.getValue();
//				BigInteger depth = query.getLevelDepth(); // TODO the depth of the received descriptions could be also used

				MetadataRepository metadataRepository = metadataEngine.getMetadataEngineTechnologyHandler().getMetadataRepository();
				
				Map<String, String> context2description = metadataRepository.getXmlDescription(contentId, xPath);
				
				for (String context : context2description.keySet())
				{
					String fragmentDescription = context2description.get(context);
					
					DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
					
					// TODO actually the received description could be just a string, and not xml. In that case
					//		the standard suggests the addition of the parent node and modification of the context
					Document bufferDoc = docBuilder.parse(new ByteArrayInputStream(fragmentDescription.getBytes()));
					
					Element xmlfragment = bufferDoc.getDocumentElement();
							
					FragmentUpdatePayload fup = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdatePayload();
					fup.setAny(xmlfragment);
					
					FragmentUpdateContext fuc = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateContext();
					fuc.setValue(context);
					
					FragmentUpdateUnit fuu = metadataEngine.getMetadataEngineSchemaHandler().getFragmentUpdateUnit();
					fuu.setFUCommand("addNode"); // the client should add the node to the description
					fuu.setFUPayload(fup);
					fuu.setFUContext(fuc);
					
					accessUnit.addFragmentUpdateUnit(fuu);
				}
			}
			
			GetDescribeEntitySuccess getDescribeContentSuccess = describeContentEngine.getDescribeContentSchemaHandler().getGetDescribeEntitySuccess();
			getDescribeContentSuccess.setAccessUnit(accessUnit);
	
			getDescribeContentResponse.setGetDescribeEntitySuccess(getDescribeContentSuccess);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			
			try
			{
				ProtocolFailure getDescribeContentFailure = describeContentEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
				getDescribeContentResponse.setGetDescribeEntityFailure(getDescribeContentFailure);
			}
			catch (MXMException e)
			{
				e.printStackTrace();
			}
		}
		
		return getDescribeContentResponse;
	}

}
