/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, Cedeo.net
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the Cedeo.net nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.orchestrator.identifycontent;

import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.IdentifyContentPE;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.exceptions.IdentifyContentEngineException;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.schemahandler.IdentifyContentRequest;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.schemahandler.IdentifyContentResponse;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.schemahandler.IdentifyContentSuccess;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.exceptions.DIEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.KeyValueData;

/**
 * @author Angelo Difino
 *
 */
public class IdentifyContentEngineOrchestrator implements org.iso.mpeg.mxm.orchestratorEngine.identifycontent.IdentifyContentEngineOrchestrator
{
	private Logger logger = Logger.getLogger(this.getClass());
	private IdentifyContentPE identifyContentEngine;
	
	public IdentifyContentEngineOrchestrator(IdentifyContentPE identifyContentEngine)
	{
		this.identifyContentEngine = identifyContentEngine;
	}

	@Override
	public IdentifyContentPE getIdentifyContentEngine() {
		return identifyContentEngine;
	}

	@Override
	public IdentifyContentResponse identifyContent(
			IdentifyContentRequest request) throws IdentifyContentEngineException {
		
		logger.info("Identify Content orchestrator invoked");
		logger.info("Transaction ID is:"+request.getTransactionIdentifier());		
		
		//TODO CHECK ME
		//System.out.println("!!!DEBUG ME!!!");
		
		//return identifyContentEngine.getTechnologyHandler().getRemoteIdentifyContentProtocol().identifyContent(request);
		//System.out.println("In IDENTIFY CONTENT local ES");
		
		try {	
			IdentifyContentResponse iContentResponse=identifyContentEngine.getSchemaHandler().getIdentifyContentResponse();
//			IdentifyContentSuccess iContentSuccess=identifyContentEngine.getSchemaHandler().getIdentifyContentSuccess(OrchestratorTE.getBaseEngine()/*,OrchestratorTE.getDidEngine()*/);
			IdentifyContentSuccess iContentSuccess=identifyContentEngine.getSchemaHandler().getIdentifyContentSuccess();
			org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity cet=request.getContentEntity();
			List<KeyValueData> kvds= request.getEntry();
				
			//FIXME check here if the DIDL instance is coming from mxm or from mp21
			
			//SUPERFIXME manage a DIParser
			//org.iso.mpeg.mxm.engine.digitalitemTE.DIParser dip=diengine.getDIParser();
			//DIDL dt=(DIDL)cet.getDIDL();
			DIDL dip_new=(DIDL)cet.getDIDL(); //forced to mpegm profile
			
			//DIDL didl_identified = this.addContentID(dt, ic.createIdentifier());
			DIDL new_didl=this.addContentID(dip_new, createIdentifier(kvds));

			/*Signature signature = signDCI(didl);

			DIDLInfo didlInfo = new DIDLInfo();
			didlInfo.setDISignature(signature);			
			didl.addDIDLInfo(didlInfo);*/

			//ContentEntityType cet_identified=new ContentEntityType();
			//org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity cet_new=baseE.getBaseProtocolSchemaHandler().getContentEntity();
			//cet_identified.setDIDL((DIDLType)didl_identified.getLevel0());
			cet.setDIDL(new_didl);												//superfixme
			//cet_identified .setDIDL(new_didl.)
			//ce_response.parse(new MXMAdapter(cet_identified));
			iContentSuccess.setIdentifiedContent(cet);		
			
			iContentResponse.setTransactionIdentifier(request.getTransactionIdentifier());
			iContentResponse.setTimestamp(new Date());
			iContentResponse.setIdentifyContentSuccess(iContentSuccess);
			logger.info("Identify Content orchestrator ended");
			return iContentResponse;
		} 
		catch (MXMException e) 
		{
			e.printStackTrace();
			throw new IdentifyContentEngineException(e.getMessage());
		}
		
	}
	
	public String createIdentifier(List<KeyValueData> kvds){
		//String myserverid = CIDContext.getInstance().getContentIdentifierURNPrefix();
		String identifierPrefix = "urn:eu:convergence:server1";
		if (kvds!=null) for (int i =0;i<kvds.size();i++)
		{
			if (kvds.get(i).getKey().equals("PRINCIPAL"))
				identifierPrefix=kvds.get(i).getValue();
		}
		String randomID = UUID.randomUUID().toString();
		String contentID = identifierPrefix + File.separator + randomID;
		return contentID;
	}
	
	private DIDL addContentID (DIDL didl, String contentID) throws MXMException {
	if (didl == null)
		throw new DIEngineException("No DIDL in IdentifyContentRequest");
	
	Item topLevelItem = ((DIDL)didl).getMPEGMItem();			//superfixme
	
	if (topLevelItem.getIdentifier()!=null){
		//if any identifiers were already present, we override it

			logger.info("BEWARE: The DI in IdentifyContentRequest already contained a temp Content ID: "+(topLevelItem.getIdentifier()));

			//((DIDL)didl).setMPEGMItem(topLevelItem);			//superfixme	needed it????	
			//return didl;				//break here
		}
	
	topLevelItem.setIdentifier(contentID);
	logger.info("Releasing a new Content ID: "+contentID);
	
//	Descriptor identifierDescriptor = new Descriptor();
//	identifierDescriptor.setIdentifier(contentID);
//	topLevelItem.addDescriptor(0, identifierDescriptor);
//	((DIDL)didl).setTopLevelItem(topLevelItem);	//superfixme
	return didl;
}
}
