/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.orchestrator.identifyuser;


import java.security.PublicKey;
import java.sql.Date;

import javax.security.cert.X509Certificate;
import javax.xml.bind.JAXBElement;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.RSAKeyValueType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.X509DataType;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.UserEntity;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.IdentifyUserEngine;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserResponse;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Person;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.PersonName;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.UniqueID;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.UserDataBaseManager;


/**
 * @author Angelos-Christos Anadiotis
 * @author Az Mousas <azmousas@icbnet.ece.ntua.gr>
 */
public class IdentifyUserEngineOrchestrator implements
    org.iso.mpeg.mxm.orchestratorEngine.identifyuser.IdentifyUserEngineOrchestrator
{
    // Converts to javax.security
    public static javax.security.cert.X509Certificate convert(final byte[] encoded)
    {
        try
        {
            return javax.security.cert.X509Certificate.getInstance(encoded);
        }
        catch (final javax.security.cert.CertificateEncodingException e)
        {
        }
        catch (final javax.security.cert.CertificateException e)
        {
        }
        return null;
    }

    private final IdentifyUserEngine identifyUserEngine;
    private final MetadataEngine     metaDataEngine;
    private SecurityEngine           securityTE = null;

    public IdentifyUserEngineOrchestrator(final IdentifyUserEngine identifyUserEngine,
        final SecurityEngine securityTE, final MetadataEngine metaDataEngine)
    {
        this.identifyUserEngine = identifyUserEngine;
        this.securityTE = securityTE;
        this.metaDataEngine = metaDataEngine;
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.orchestratorEngine.identifyuser.IdentifyUserEngineOrchestrator
     * #getIdentifyUserEngine()
     */
    @Override
    public IdentifyUserEngine getIdentifyUserEngine()
    {
        return this.identifyUserEngine;
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.orchestratorEngine.identifyuser.IdentifyUserEngineOrchestrator
     * #identifyUser
     * (org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserRequest)
     */
    @Override
    public IdentifyUserResponse identifyUser(final IdentifyUserRequest request)
        throws MXMEngineException
    {
        /*
         * TODO: move it to authorize user
         * org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Assertion
         * assertion = null; assertion = request.getUserIdentificationAssertion(); if
         * (assertion != null) {
         * org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature =
         * assertion .getSignature(); XMLGregorianCalendar date =
         * assertion.getIssueInstant(); Conditions conditions = assertion.getConditions();
         * List<StatementAbstract> listOfStatements = assertion.getStatements();
         * org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Subject
         * subject = assertion .getSubject(); // * TODO:check the asssertion if not valid
         * throw exception for (int i = 0; i < listOfStatements.size(); i++) { // TODO
         * dont use types here AuthnStatementType statement =
         * (AuthnStatementType)listOfStatements .get(i); AuthnContextType authContex =
         * statement.getAuthnContext(); List<JAXBElement<?>> contents =
         * authContex.getContent(); for (int j = 0; j < contents.size(); j++) {
         * JAXBElement<?> ele = contents.get(i); // * TODO: what to do with qualified anme
         * and value QName qname = ele.getName(); Object value = ele.getValue(); } } //
         * TODO print out assertion }
         */
        org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity contentEntity = null;
        try
        {
            contentEntity = request.getContentEntity();
        }
        catch (final MXMEngineException e2)
        {
            // TODO Auto-generated catch block
            e2.printStackTrace();
        }
        ;
        try
        {
            final UserEntity userEntity = request.getUserEntity();
            contentEntity.getDIDL();
            final String generatedId = this
                .storeUserCredential(contentEntity, userEntity);
            // get user entity add identifier and sign it
            final UniqueID id = this.metaDataEngine.getMetadataEngineSchemaHandler()
                .getUniqueID();
            id.setValue(generatedId);
            userEntity.addUniqueID(id);
            userEntity.getUser().setId(generatedId);
            final org.iso.mpeg.mxm.pEngine.identifyuserPE.schemahandler.IdentifyUserSuccess identifyUserSuccess = this.identifyUserEngine
                .getIdentifyUserSchemaHandler().getIdentifyUserSuccess();
            identifyUserSuccess.setIdentifiedUser(userEntity);
            final IdentifyUserResponse identifyUserResponse = this.identifyUserEngine
                .getIdentifyUserSchemaHandler().getIdentifyUserResponse();
            identifyUserResponse.setIdentifyUserSuccess(identifyUserSuccess);
            // * TODO: sign the response (but leave it out right now)
            return identifyUserResponse;
        }
        catch (final MXMEngineException e1)
        {
            e1.printStackTrace();
        }
        catch (final MXMException e1)
        {
            e1.printStackTrace();
        }
        // TODO parse the identifyUserRequest
        // - check the info of the assertion if it exists
        // - get the contentEntity and store user credentials
        // - get the userEntity
        // - add an identifier and sign it
        // TODO create a identifyUserSuccess object which has inside it the above
        // userEntity
        // TODO create a identifyUserResponse with the identifyUserSuccess
        // IdentifyUserResponse identifyUserResponse =
        // identifyUserEngine.getIdentifyUserSchemaHandler().getIdentifyUserResponse();
        // identifyUserResponse.setIdentifyUserSuccess(identifyUserSuccess);
        // TODO printout the identifyUserResponse
        return null; // TODO return the identifyUserResponse
    }

    private String identifyUserToIP(
        final javax.security.cert.X509Certificate userCertificate,
        final String firstName, final String lastName, final Date birthday,
        final String country, final String locality, final String organization,
        final String PIN)
    {
        int userID = -1;
        try
        {
            // Store everything to the data base.
            userID = this.securityTE
                .getSecurityEngineTechnologyHandler().getUserDataBaseManager()
                .registerUser(userCertificate, firstName, lastName, birthday, country,
                    locality, organization);
        }
        catch (final Exception e)
        {
            e.printStackTrace();
        }
        return userID + "";
    }

    private String identifyUserToIP(final PublicKey key, final String firstName,
        final String lastName, final Date birthday, final String country,
        final String locality, final String organization, final String PIN)
    {
        int userID = -1;
        try
        {
            // Store everything to the data base.
            userID = ((UserDataBaseManager)this.securityTE
                .getSecurityEngineTechnologyHandler().getUserDataBaseManager())
                .registerUser(key, firstName, lastName, birthday, country, locality,
                    organization);
        }
        catch (final Exception e)
        {
            e.printStackTrace();
        }
        return userID + "";
    }

    // TODO certificates as inputparameters
    public int identifyUserToSP(final X509Certificate iPUserCertificate,
        final String userIdFromAssertion, final PublicKey publicCiphKey,
        final PublicKey publicSignKey, final PublicKey publicAuthKey)
    {
        final int userID = -1;
        return userID;
        /*
         * if(userIdFromAssertion != null || !userIdFromAssertion.equals("")) { //store
         * userid } // TODO store the designated application name inside the certificate
         * // (organizationalUnit entry?) // TODO check the iPUserCertificate against the
         * IP root certificate. String userReference; try { // Concatenate the user
         * information into one string. // TODO put the userPseudonym also into the
         * certificate (reference) at the // proper location userReference =
         * NameService.get( NameService.create(null, null, null, null, "applicationName",
         * null, null, null,userIdFromAssertion)).toString(); // Create the user key pair.
         * // TODO create key pairs on card and retrieve only its public key final KeyPair
         * authKeyPair =
         * this.securityTE.getSecurityEngineTechnologyHandler().getCertificateManager()
         * .generateRSAKeyPair(2048, new BigInteger("65537")); final KeyPair signKeyPair =
         * this.securityTE.getSecurityEngineTechnologyHandler().getCertificateManager()
         * .generateRSAKeyPair(2048, new BigInteger("65537")); final KeyPair ciphKeyPair =
         * this.securityTE.getSecurityEngineTechnologyHandler().getCertificateManager()
         * .generateRSAKeyPair(2048, new BigInteger("65537")); // Create the
         * authentication certificate. final X509Certificate authCertificate =
         * X509Certificate
         * .getInstance(this.securityTE.getSecurityEngineTechnologyHandler()
         * .getCertificateManager() .generateCVCertificate(userReference,
         * this.ISSUER_REFERENCE, this.HOLDER_CERT_EXPIRY_DAYS, authKeyPair.getPublic(),
         * this.ISSUER_KEY_PAIR.getPrivate(), "SHA256withRSA") .getX509Certificate()); //
         * Create the sign certificate. final X509Certificate signCertificate =
         * X509Certificate
         * .getInstance(this.securityTE.getSecurityEngineTechnologyHandler()
         * .getCertificateManager() .generateCVCertificate(userReference,
         * this.ISSUER_REFERENCE, this.HOLDER_CERT_EXPIRY_DAYS, signKeyPair.getPublic(),
         * this.ISSUER_KEY_PAIR.getPrivate(), "SHA256withRSA") .getX509Certificate()); //
         * Create the encryption certificate. final X509Certificate ciphCertificate =
         * X509Certificate
         * .getInstance(this.securityTE.getSecurityEngineTechnologyHandler()
         * .getCertificateManager() .generateCVCertificate(userReference,
         * this.ISSUER_REFERENCE, this.HOLDER_CERT_EXPIRY_DAYS, ciphKeyPair.getPublic(),
         * this.ISSUER_KEY_PAIR.getPrivate(), "SHA256withRSA") .getX509Certificate()); //
         * Store everything to the data base. // TODO only store the pseudonym to the
         * certificates userID =
         * ((SPUserDataBaseManager)this.USERDATABASE).storeUser(null, authCertificate,
         * signCertificate, ciphCertificate, iPUserCertificate); // Store the certificates
         * and key pairs to the smart card. // TODO do that on client side } catch (final
         * Exception e) { e.printStackTrace(); } return userID;
         */
    }

    private String storeUserCredential(
        final org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity contentEntity,
        final UserEntity userEntity) throws MXMEngineException,MXMException
    {
        final org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL didl = (org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL)contentEntity
            .getDIDL();
        // * TODO: store content entity with registration tool
        final Item item = didl.getMPEGMItem();
        final org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component component = item
            .getComponent().get(0);
        final Resource resource = component.getResource().get(0);
        final JAXBElement jaxbElement = (JAXBElement)resource.getContent().get(0);
        X509DataType certificateType = null;
        RSAKeyValueType keyValueType = null;
        if (this.securityTE.getPropertyValue("enableSmartCard").equals("false"))
        {
            certificateType = (X509DataType)jaxbElement.getValue();
            final X509Data data = this.securityTE.getSecurityEngineSchemaHandler()
                .getX509Data();
            data.parse(new MXMAdapter(certificateType));
            // TODO: FIX AT org.iso.mpeg.mxm.securityTE.schemahandler.X509Data so we do'nt
            // have data.getX509Certificate() null
            final byte[] X509CertificateBytes = data.getX509Certificate();
            final javax.security.cert.X509Certificate certificate = convert(X509CertificateBytes);
            // get person name
            final Person person = (Person)userEntity.getUser();
            final PersonName personName = (PersonName)person.getName().get(0);
            final org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.NameComponent nameComponent = (personName
                .getGivenName().size() > 0) ? personName.getGivenName().get(0) : null;
            // get family name
            final org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.NameComponent familyName = (personName
                .getFamilyName().size() > 0) ? personName.getFamilyName().get(0) : null;
            final String firstName = (nameComponent == null) ? "unknown" : nameComponent
                .getValue();
            final String lastName = (nameComponent == null) ? "unknown" : familyName
                .getValue();
            ((Person)userEntity.getUser()).getNationality();
            final org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Place place = ((Person)userEntity
                .getUser()).getAddress();
            return this.identifyUserToIP(certificate, firstName, lastName, null, "", "",
                "", "PIN NUMBER");
        }
        else
        {
            keyValueType = (RSAKeyValueType)jaxbElement.getValue();
            final RSAKeyValue data = this.securityTE.getSecurityEngineSchemaHandler()
                .getRSAKeyValue();
            data.parse(new MXMAdapter(keyValueType));
            // get person name
            final Person person = (Person)userEntity.getUser();
            final PersonName personName = (PersonName)person.getName().get(0);
            final org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.NameComponent nameComponent = (personName
                .getGivenName().size() > 0) ? personName.getGivenName().get(0) : null;
            // get family name
            final org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.NameComponent familyName = (personName
                .getFamilyName().size() > 0) ? personName.getFamilyName().get(0) : null;
            final String firstName = (nameComponent == null) ? "unknown" : nameComponent
                .getValue();
            final String lastName = (nameComponent == null) ? "unknown" : familyName
                .getValue();
            ((Person)userEntity.getUser()).getNationality();
            final org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Place place = ((Person)userEntity
                .getUser()).getAddress();
            return this.identifyUserToIP(this.securityTE.getSecurityEngineTechnologyHandler().getFileAndObjectHandler().publicKeyAdapter(data),
                firstName, lastName, null, "", "", "", "PIN NUMBER");
        }
    }
}
