/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, Cedeo.net
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the Cedeo.net nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.orchestrator.packagecontent;


import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.UUID;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import org.xml.sax.*;
import javax.xml.parsers.*;
import javax.xml.transform.*; 
import javax.xml.transform.dom.*; 
import javax.xml.transform.stream.*;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.PackageContentPE;
import org.iso.mpeg.mxm.pEngine.createcontentPE.exceptions.CreateContentEngineException;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSuccess;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.exceptions.PackageContentEngineException;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.schemahandler.PackageContentRequest;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.schemahandler.PackageContentResponse;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.schemahandler.PackageContentSuccess;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * @author Angelos-Christos Anadiotis
 * @author Angelo Difino 
 *
 */
public class PackageContentEngineOrchestrator implements org.iso.mpeg.mxm.orchestratorEngine.packagecontent.PackageContentEngineOrchestrator
{
	private Logger logger = Logger.getLogger(this.getClass());
	
	private PackageContentPE packageContentEngine;
	private DIEngine diengine;
	
	public PackageContentEngineOrchestrator(PackageContentPE packageContentEngine, DIEngine diengine)
	{
		this.packageContentEngine = packageContentEngine;
		this.diengine=diengine;
	}
	
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.createcontent.CreateContentEngineOrchestrator#getCreateContentEngine()
	 */
	/**
	 * Method to get the used Package Content Engine
	 *
	 */
	@Override
	public PackageContentPE getPackageContentEngine()
	{
		return packageContentEngine;
	}
	
	/**
	 * Method to get the format Type adopted for the Package Content
	 *
	 */
	public String getformatType(Document node)
	{
		System.out.println("PA1 - Executing getFormatType() method.");	
		NodeList list=node.getElementsByTagName("PackageFormatType");
		Node firstChild = list.item(0);
		String formatType = firstChild.getTextContent();
		System.out.println("PA2 - Returning format type.");	
		return formatType;
	}
	
	/**
	 * Method that packages the content in a Stream
	 *
	 */
	public void packageStream(String did, String dirName) throws PackageContentEngineException
	{	
		try {
		System.out.println("PA1 - Executing packageStream() method.");
		//Creation a new inside directory
		File f = new File(dirName);
        f.mkdir();
		
        //Creation of a new stream
        System.out.println("PA2 - Creation of a new stream.");	
        String fileName = "stream.xml";
		File dir = new File (dirName);
		File file = new File (dir, fileName);
		
				
		//Check the existence of the stream
		System.out.println("PA3 - Checking the existence of the stream.");	
 	    boolean blnCreated = false;
	    try
	    {
		       blnCreated = file.createNewFile();
	    }
	    catch(IOException ioe)
	    {
	    	   System.out.println("Error while creating a new empty file :" + ioe);
	    }

	    System.out.println("PA4 - Was file " + file.getPath() + " created ? : " + blnCreated);
	        
	    	    
	    //Transform the XML destination stream
	    System.out.println("PA5 - Transforming the XML destination stream.");	
	    TransformerFactory tranFact = 
		    TransformerFactory.newInstance(); 
		Transformer transfor = tranFact.newTransformer();
		
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		// Use document builder factory	
		DocumentBuilder builder = factory.newDocumentBuilder();
			
		Document doc = builder.parse(did); 
		NodeList node2 =doc.getElementsByTagName("Package");
		
		System.out.println("PA6 - Extracting the node element.");	
		Node node3 = node2.item(0);
		
		Source src = new DOMSource(node3); 
		//Result dest = new StreamResult(System.out);
	
		System.out.println("PA7 - Placing in the right folder.");
		String TEMP_FOLDER2 = getClass().getClassLoader().getResource(".").getPath();
		String filePathInit = TEMP_FOLDER2 + "/package";
		String filePath = filePathInit + "/stream.xml";
		System.out.println("PA8 - Path is " + filePath);
		Result dest = new StreamResult(filePath);
		transfor.transform(src, dest);
		
		System.out.println("PA9 - Process Done");
		}
		catch (Exception e){
			e.printStackTrace();
		throw new PackageContentEngineException(e.getMessage(), e);
		}
	}
	
	/**
	 * Method to extract the number of fragments
	 *
	 */
	public int getFragmentSize(Document node) {
		System.out.println("PA1 - Executing the getFragmentSize() method.");	
		int size = 0;
		NodeList list=node.getElementsByTagName("Fragment");
		Node firstChild = list.item(0);
		System.out.println("PA2 - Childs were found.");
		size++;
		for(int i=1;i<list.getLength();++i)
		{
			Node child=list.item(i);  // a child node is still a node!!	
			size++;
		}
		System.out.println("PA3 - Fragment size is: " + size);	
		System.out.println("PA4 - Returning fragment size.");	
		return size;
	}
	
	
	/**
	 * Method that packages the content in a File
	 *
	 */
	public void packageFile(String did, String dirName, Document node) throws PackageContentEngineException
	{					
		try {
		System.out.println("PA1 - Executing packageFile() method.");
		//Creation a new inside directory
		File f = new File(dirName);
        f.mkdir();
		
        //Creation of a new file
        System.out.println("PA2 - Creation of a new file for fragments.");
        String fileName = "fragment1.xml";
		File dir = new File (dirName);
		File file = new File (dir, fileName);
		
				
		//Check the existence of the file
		System.out.println("PA3 - Checking the existence of the file.");
 	    boolean blnCreated = false;
	    try
	    {
		       blnCreated = file.createNewFile();
	    }
	    catch(IOException ioe)
	    {
	    	   System.out.println("Error while creating a new empty file :" + ioe);
	    }

	    System.out.println("PA4 - Was file " + file.getPath() + " created ? : " + blnCreated);
		    
	      
	    //Transform the XML destination file
	    System.out.println("PA5 - Transforming the XML destination file.");
	    TransformerFactory tranFact = 
		    TransformerFactory.newInstance(); 
		Transformer transfor = tranFact.newTransformer();
		
		
		DocumentBuilderFactory factory = 
		    DocumentBuilderFactory.newInstance();
		// Use document builder factory
		DocumentBuilder builder = factory.newDocumentBuilder();
		
		System.out.println("PA6 - Parsing the XML file.");
		Document doc = builder.parse(did); 
		NodeList node2 =doc.getElementsByTagName("Fragment");
				
		//Getting the first node element
		Node node3 = node2.item(0);
		
		Source src = new DOMSource(node3); 
		//Result dest = new StreamResult(System.out);
	
		String filePathInit = dirName;
		String filePath = filePathInit + "/fragment1.xml";
		System.out.println("Path is " + filePath);
		Result dest = new StreamResult(filePath);
		transfor.transform(src, dest);
		
		System.out.println("PA7 - Applying to the childs.");
		String name="";
		int fragmentsSize = getFragmentSize(node); 
		for (int i=1;i<fragmentsSize;i++)
		{
			fileName="";
			System.out.println("PA8 - Creation of new files.");
			//Creation a new inside directory
			f = new File(dirName);
	        f.mkdir();
			
	        //Creation of a new file
	        int j = i+1;
	        name = "/fragment" + j;
	        fileName = name +".xml";
			dir = new File (dirName);
			file = new File (dir, fileName);
			
			System.out.println("PA9 - Extracting the node element.");
			node3 = node2.item(i);
		
			src = new DOMSource(node3); 
			filePath = filePathInit + fileName;
			System.out.println("PA10 - Path is " + filePath);
			dest = new StreamResult(filePath);
			transfor.transform(src, dest);
		}
		
		System.out.println("Process Done"); 
		}
		catch (Exception e){
			e.printStackTrace();
			throw new PackageContentEngineException(e.getMessage(), e);
		}
	}
		
	
	

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.createcontent.CreateContentEngineOrchestrator#createContent(org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest)
	 */
	/**
	 * Method to deliver the Response of a Package Content invocation
	 *
	 */
	@Override
	public PackageContentResponse packageContent(PackageContentRequest request) throws PackageContentEngineException
	{
		try{
			logger.info("PG1 - Package Content  orchestrator invoked");
			logger.info("PG2 - Transaction ID is:"+request.getTransactionIdentifier());
			
			String TEMP_FOLDER2 = getClass().getClassLoader().getResource(".").getPath();
			String SOURCE_DCI_FILE_NAME2 = TEMP_FOLDER2 + "testPKG.xml";
			
			System.out.println("Path 1: " + TEMP_FOLDER2);
			System.out.println("Path 2: " + SOURCE_DCI_FILE_NAME2);
			String did = SOURCE_DCI_FILE_NAME2;
						
			PackageContentResponse response=packageContentEngine.getSchemaHandler().getPackageContentResponse();
			PackageContentSuccess success=packageContentEngine.getSchemaHandler().getPackageContentSuccess();
			DIDL didl=diengine.getDIEngineSchemaHandler().getDIDL();
			Item topLevelItem=diengine.getDIEngineSchemaHandler().getItem();
			
			/*
			topLevelItem.setId("A0");
			topLevelItem.setIdentifier("identifier:content:temp:" + UUID.randomUUID());
			*/
			
			DocumentBuilderFactory factory=javax.xml.parsers.DocumentBuilderFactory.newInstance();
			DocumentBuilder builder;
			Document node = null;
						
			System.out.println("PG3 - Assignment of the DID file.");
			try {
				builder = factory.newDocumentBuilder();
				org.w3c.dom.Document doc=builder.parse(new FileInputStream(did) );
				node=doc;  // A Document is a node
			} catch (ParserConfigurationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SAXException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			//Getting format type
			String formatType = getformatType(node);
			System.out.println("PG4 - Format type is: " + formatType);
							
			// Move to the right temp directory
			String dirName = TEMP_FOLDER2+"/package";
			
			System.out.println("PG4 - Checking the format type");
			if (formatType.equals("STREAM"))
			{
				System.out.println("PG5A - Stream type");
				packageStream(did, dirName);
			}
			else if (formatType.equals("FILE"))
			{
				System.out.println("PG5B - File type");
				packageFile(did, dirName, node);
			}
					
			
			didl.setMPEGMItem(topLevelItem);
			
			ContentEntity ce=packageContentEngine.getBaseProtocolSchemaHandler().getContentEntity();
			ce.setDIDL(didl);
			success.setIdentifiedContent(ce);
	
			response.setTransactionIdentifier(request.getTransactionIdentifier());
			response.setPackageContentSuccess(success);
			logger.info("PG6 - Package Content orchestrator ended.");
			
			return response;
		}
		catch (Exception e){
			e.printStackTrace();
			throw new PackageContentEngineException(e.getMessage(), e);
		}
		
	}

}
