/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.orchestrator.storelicense;

import java.io.File;
import java.util.List;

import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.BasicFailureCode;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.BasicSuccessCode;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.pEngine.storelicensePE.StoreLicenseEngine;
import org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseRequest;
import org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseResponse;
import org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseSuccess;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_mp21.Container;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_mp21.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_mp21.Item;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;

/**
 * @author Jonathan Florido, DMAG-UPC <jflorido@ac.upc.edu>
 *
 */
public class StoreLicenseEngineOrchestrator implements org.iso.mpeg.mxm.orchestratorEngine.storelicense.StoreLicenseEngineOrchestrator
{
	private StoreLicenseEngine storeLicenseEngine;
	
	public StoreLicenseEngineOrchestrator(StoreLicenseEngine storeLicenseEngine)
	{
		this.storeLicenseEngine = storeLicenseEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.storelicense.StoreLicenseEngineOrchestrator#getStoreLicenseEngine()
	 */
	@Override
	public StoreLicenseEngine getStoreLicenseEngine()
	{
		return storeLicenseEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.storelicense.StoreLicenseEngineOrchestrator#storeLicense(org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseRequest)
	 */
	@Override
	public StoreLicenseResponse storeLicense(StoreLicenseRequest storeLicenseRequest)
	{
		StoreLicenseResponse storeLicenseResponse = storeLicenseEngine.getStoreLicenseSchemaHandler().getStoreLicenseResponse();
		try {
			// There are no licenses to store
			if (storeLicenseRequest.getLicenseEntity() == null && storeLicenseRequest.getContentEntity() == null) {
				ProtocolFailure storeLicenseFailure = storeLicenseEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
			    storeLicenseFailure.addResultCode(BasicFailureCode.UNABLE_TO_PROCESS.value());
			    storeLicenseFailure.setDisplayString("Bad storeLicenseRequest!");
			    storeLicenseResponse.setStoreLicenseFailure(storeLicenseFailure);
			}
			// There is only one license to store
			else if (storeLicenseRequest.getLicenseEntity() != null) {
				License license = storeLicenseRequest.getLicenseEntity().getLicense();
				storeLicense(license, storeLicenseResponse);
				StoreLicenseSuccess success = storeLicenseEngine.getStoreLicenseSchemaHandler().getStoreLicenseSuccess();
	            success.addSuccessCode(BasicSuccessCode.OK.value());
	            success.setDisplayString("The License with ID: " + license.getLicenseId() + " has been successfully stored");
	            success.setLicenseEntity(storeLicenseRequest.getLicenseEntity());
	            storeLicenseResponse.setStoreLicenseSuccess(success);
			}
			// There are two or more licenses to store
			else if (storeLicenseRequest.getContentEntity() != null) {
				// DIDL, 
				ContentEntity ce = storeLicenseRequest.getContentEntity();
				DIDL didl = ce.getDIDL();
				Container container = didl.getContainer();
				List<Item> items = container.getItem();
				for (Item item: items) {
					License license = (License)item.createObject(License.class.getCanonicalName());
					storeLicense(license, storeLicenseResponse);
				}
				StoreLicenseSuccess success = storeLicenseEngine.getStoreLicenseSchemaHandler().getStoreLicenseSuccess();
	            success.addSuccessCode(BasicSuccessCode.OK.value());
	            success.setDisplayString("All the licenses have been stored successfully");
	            storeLicenseResponse.setStoreLicenseSuccess(success);
			}
		} catch (Exception e) {
			e.printStackTrace();
			try {
				ProtocolFailure storeLicenseFailure = storeLicenseEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
	            storeLicenseFailure.addResultCode(BasicFailureCode.UNKNOWN_ERROR.value());
	            storeLicenseFailure.setDisplayString("Cannot perform operation. Details: " + e.getMessage());
	            storeLicenseResponse.setStoreLicenseFailure(storeLicenseFailure);
			}
			catch (Exception e2) {
				e2.printStackTrace();
			}
		} 
		storeLicenseResponse.setTransactionIdentifier(storeLicenseRequest.getTransactionIdentifier());
		return storeLicenseResponse;
	}
	
	public void storeLicense(License license, StoreLicenseResponse storeLicenseResponse) throws MXMException {
		try {
			license.createFile(new File(license.getLicenseId() + ".xml"));
		} catch (Exception e) {
			e.printStackTrace();
			ProtocolFailure storeLicenseFailure = storeLicenseEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
            storeLicenseFailure.addResultCode(BasicFailureCode.UNKNOWN_ERROR.value());
            storeLicenseFailure.setDisplayString("Cannot store the license. Details: " + e.getMessage());
            storeLicenseResponse.setStoreLicenseFailure(storeLicenseFailure);
		}
		
	}

}
