/*
*** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, Cedeo.net
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the Cedeo.net nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.orchestrator.unpublishcontent;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.UUID;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import org.xml.sax.*;
import javax.xml.parsers.*;
import javax.xml.transform.*; 
import javax.xml.transform.dom.*; 
import javax.xml.transform.stream.*;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.PackageContentPE;
import org.iso.mpeg.mxm.pEngine.createcontentPE.exceptions.CreateContentEngineException;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentRequest;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentResponse;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.CreateContentSuccess;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.exceptions.PackageContentEngineException;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.schemahandler.PackageContentRequest;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.schemahandler.PackageContentResponse;
import org.iso.mpeg.mxm.pEngine.packagecontentPE.schemahandler.PackageContentSuccess;

import org.iso.mpeg.mxm.pEngine.unpublishcontentPE.UnpublishContentPE;
import org.iso.mpeg.mxm.pEngine.unpublishcontentPE.exceptions.UnpublishContentEngineException;
import org.iso.mpeg.mxm.pEngine.unpublishcontentPE.schemahandler.UnpublishContentRequest;
import org.iso.mpeg.mxm.pEngine.unpublishcontentPE.schemahandler.UnpublishContentResponse;
import org.iso.mpeg.mxm.pEngine.unpublishcontentPE.schemahandler.UnpublishContentSuccess;

import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;

import javax.xml.parsers.*;
import javax.xml.transform.*; 
import javax.xml.transform.dom.DOMSource; 
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.*;
import org.xml.sax.*;

import java.util.Date;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.io.*;

/**
 * @author Angelos-Christos Anadiotis
 * @author Angelo Difino 
 *
 */
public class UnpublishContentEngineOrchestrator implements org.iso.mpeg.mxm.orchestratorEngine.unpublishcontent.UnpublishContentEngineOrchestrator
{
	private Logger logger = Logger.getLogger(this.getClass());
	
	private UnpublishContentPE unpublishContentEngine;
	private DIEngine diengine;
	
	String text="";
	
	public UnpublishContentEngineOrchestrator(UnpublishContentPE unpublishContentEngine, DIEngine diengine)
	{
		this.unpublishContentEngine = unpublishContentEngine;
		this.diengine=diengine;
	}
	
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.orchestratorEngine.createcontent.CreateContentEngineOrchestrator#getCreateContentEngine()
	 */
	/**
	 * Method to get the used Package Content Engine
	 *
	 */
	@Override
	public UnpublishContentPE getUnpublishContentEngine()
	{
		return unpublishContentEngine;
	}
	
	/**
	 * <HL>
	 * Extracts the Fractals elements
	 * @param node is the document of the DID
	 */
	public String[] getFractals(Document node) {
		System.out.println("PA1 - Executing getFractals() method.");
		String [] tempFractals = new String[10];
		NodeList list=node.getElementsByTagName("ns21:Fractal");
		Node firstChild = list.item(0);
		System.out.println("PA2 - Identifier is: "+firstChild.getTextContent() );
		tempFractals[0] = firstChild.getTextContent();
		
		for(int i=1;i<list.getLength();++i)
		{
			Node child=list.item(i);  // a child node is still a node!!
			System.out.println("Node Content: "+child.getTextContent() );
			tempFractals[i] = child.getTextContent();
		}
		
		System.out.println("PA3 - Returning the identifier.");	
		return tempFractals;
	}
	
	/* 
	 * filter all elements whose tag name = filter 
	 */  
	public void filterElements( Node parent, String filter ){  
		NodeList children = parent.getChildNodes();  
	   
		for( int i=0; i < children.getLength(); i++ ){  
			Node child = children.item( i );  
	   
			// only interested in elements  
			if( child.getNodeType() == Node.ELEMENT_NODE ){  
	   
				// remove elements whose tag name  = filter  
				// otherwise check its children for filtering with a recursive call  
				if( child.getNodeName().equals(filter) ){  
					parent.removeChild( child );  
				} else {  
					filterElements( child, filter );  
				}  
			}  
		}  
	}

	   
	/* 
	 * print a nodes elements and children 
	 */  
	public void printNodeElements( Node node){  
		System.out.println( node.getNodeName() );
                
		NodeList children = node.getChildNodes();  
	   
		for ( int i = 0; i < children.getLength(); i++ ){  
			Node child = children.item( i );  
	   
			if( child.getNodeType() == Node.ELEMENT_NODE ){  
				printNodeElements( child );  
			}  
		}  		
	}
	
	
	/*
	 * Method to store and advertise from CONET
	 */
	public boolean storeAdverstiseConet (String VDI)
	{
		//ToDo
		return true;
	}
	
	
	/**
	 * Method that revokes from CONET
	 *
	 */
	public UnpublishContentResponse revokeFromConet (String VDI) throws UnpublishContentEngineException, MXMException
	{					
		UnpublishContentResponse response=unpublishContentEngine.getSchemaHandler().getUnpublishContentResponse();
		UnpublishContentSuccess success=unpublishContentEngine.getSchemaHandler().getUnpublishContentSuccess();
		
		DocumentBuilderFactory factory=javax.xml.parsers.DocumentBuilderFactory.newInstance();
		DocumentBuilder builder;
		Document node = null;
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		System.out.println("BCG1");
		MXM mxm = MXM.getInstance(f);
		System.out.println("BCG2");
		mxm.listEnginesAndProperties();
		
		System.out.println("PG1 - Creation of a DIDL.");
		DIEngine didEngine = (DIEngine) mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
		
		DIDL di = null;
        try
        {
            di = didEngine.getDIEngineSchemaHandler().getDIDL();
        }
        catch (MXMException e1)
        {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
		
		
		System.out.println("PG1 - Executing revokeFromConet() method.");
		
		//1. Information regarding inputs
		System.out.println("PG2 - VDI location is: " + VDI);
		
		//2. Assignment of DID file
		System.out.println("PG3 - Assignment of the DID file.");
		try {
			builder = factory.newDocumentBuilder();
			org.w3c.dom.Document doc=builder.parse(new FileInputStream(VDI) );
			node=doc;  // A Document is a node
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		//3. Test the current date
		System.out.println("PG4 - Caaling the CONET method");
		Boolean responseConet = storeAdverstiseConet (VDI);
		
		logger.info("PG6 - Setting Unpublish Content.");
		response.setUnpublishContentSuccess(success);
		logger.info("PG6 - Unpublish Content orchestrator ended.");
				
		return response;
}
	

	
	/**
	 * Method that unpublishes from COMID
	 *
	 */
	public UnpublishContentResponse unpublishFromComid (String VDI) throws UnpublishContentEngineException, MXMException
	{					
		UnpublishContentResponse response=unpublishContentEngine.getSchemaHandler().getUnpublishContentResponse();
		UnpublishContentSuccess success=unpublishContentEngine.getSchemaHandler().getUnpublishContentSuccess();
		
		DocumentBuilderFactory factory=javax.xml.parsers.DocumentBuilderFactory.newInstance();
		DocumentBuilder builder;
		Document node = null;
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		System.out.println("BCG1");
		MXM mxm = MXM.getInstance(f);
		System.out.println("BCG2");
		mxm.listEnginesAndProperties();
		
		System.out.println("PG1 - Creation of a DIDL.");
		DIEngine didEngine = (DIEngine) mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
		
		DIDL di = null;
        try
        {
            di = didEngine.getDIEngineSchemaHandler().getDIDL();
        }
        catch (MXMException e1)
        {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
		
		
		System.out.println("PG1 - Executing unpublishFromComid() method.");
		
		//1. Information regarding inputs
		System.out.println("PG2 - VDI location is: " + VDI);
		
		//2. Assignment of DID file
		System.out.println("PG3 - Assignment of the DID file.");
		try {
			builder = factory.newDocumentBuilder();
			org.w3c.dom.Document doc=builder.parse(new FileInputStream(VDI) );
			node=doc;  // A Document is a node
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		//3. Test the current date
		System.out.println("PG4 - Verify the current data against expiry date");
		
		String revocationDate = "";
		System.out.println("PG4A - Extrating the Revocation Date");
		NodeList list2 = node.getElementsByTagName("ns21:ExpiryDate");
		Node firstChild2 = list2.item(0);
				
		System.out.println("PG4C - Revocation Date is: "+firstChild2.getTextContent() );
		String dateDIDStr = firstChild2.getTextContent();
		
		//Converting string to date	
		Date dateDID = null;
		try {
			dateDID = new SimpleDateFormat("yyyy/MM/dd").parse(dateDIDStr);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println(dateDID);
		
		//Getting current date
		DateFormat df1 = new SimpleDateFormat("yyyy/MM/dd");
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd");
		Date currentDate = null;
		try {
			currentDate = df1.parse(df1.format(new Date()));
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Boolean expiry = false;
				
		if (dateDID.compareTo(currentDate) > 0)
		        System.out.println("PG5 - The ITEM is still active.");
		else if (dateDID.compareTo(currentDate) <= 0)
		{
		    	System.out.println("PG5 - The ITEM is expired.");
		    	expiry = true;
		}
		
		if (expiry == true)
		{
			//4. Creation of a new instance of Engine Overlay
			System.out.println("PG6 - Creation of a new OverlayEngine instance.");
			//OverlayEngineOrchestrator overlayEngine = (OverlayEngineOrchestrator)mxm.getDefaultEngine(MXMEngineName.OverlayTE.toString());
						
			//Extracting fractals
			String fractals[] = new String[10];			
			System.out.println("PG7 - Get fractals content.");
			fractals = getFractals(node);
			
			// Use some test fractals if the information is not available in VDI
			if (fractals[0]=="")
			{
				System.out.println("PG8 - Using dummy fractals.");
				fractals[0]="teste1";
				fractals[1]="teste2";
			}
				
			System.out.println("PG8 - Calling the handleRevocationVDI() method.");
						
			//MXMEngineResponse test2 = overlayEngine.handleRevocationVDI(VDI, fractals);
			
			System.out.println("PG9 - Eliminating local Item.");
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer tFormer = null;
			DocumentBuilderFactory factory2 = DocumentBuilderFactory.newInstance();  
			DocumentBuilder builder2 = null;  
			Document document = null;  
			
			System.out.println("Point1");
			try{  
				builder = factory.newDocumentBuilder();  
				document = builder.parse( new File( VDI ) );
				System.out.println("Point2");
				String tagFilter = "mpegm-didl:Item";
				System.out.println("Point3");
				filterElements( document, tagFilter );
				System.out.println("Point4");
				printNodeElements(document);
			} catch ( Exception e ){  
			e.printStackTrace();  
			}    
		}
		
		logger.info("PG6 - Setting Unpublish Content.");
		response.setUnpublishContentSuccess(success);
		logger.info("PG6 - Unpublish Content orchestrator ended.");
		
		return response;
}


	@Override
	public UnpublishContentResponse unpublishContent( UnpublishContentRequest request)
			throws UnpublishContentEngineException
	{
		logger.info("PG1 - Running unpublishContent() method.");
		logger.info("PG2 - Transaction ID is: "+request.getTransactionIdentifier());
		
		UnpublishContentResponse response = null;
		try {
			response=unpublishContentEngine.getSchemaHandler().getUnpublishContentResponse();
			UnpublishContentSuccess success=unpublishContentEngine.getSchemaHandler().getUnpublishContentSuccess();
		} catch (MXMEngineException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		DocumentBuilderFactory factory=javax.xml.parsers.DocumentBuilderFactory.newInstance();
		DocumentBuilder builder;
		Document node = null;
		
		//1. Get the VDI file
		logger.info("PG3 - Getting the VDI file.");
		String TEMP_FOLDER2 = getClass().getClassLoader().getResource(".").getPath();
		String SOURCE_DCI_FILE_NAME2 = TEMP_FOLDER2 + "testUnpublish.xml";
		
		System.out.println("Path 1: " + TEMP_FOLDER2);
		System.out.println("Path 2: " + SOURCE_DCI_FILE_NAME2);
		String VDI = SOURCE_DCI_FILE_NAME2;
		
		//2. Get the DID information
		logger.info("PG4 - Getting the DID information.");
		String TEMP_FOLDER3 = getClass().getClassLoader().getResource(".").getPath();
		String SOURCE_DCI_FILE_NAME3 = TEMP_FOLDER3 + "MXMConfiguration.xml";
		
		org.w3c.dom.Document doc = null;
		
		try {
			builder = factory.newDocumentBuilder();
			doc=builder.parse(new FileInputStream(VDI) );
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		node=doc;  // A Document is a node
		
		logger.info("PG5A - Getting the DIDUnpublish element name.");
		NodeList list=node.getElementsByTagName("DIDUnpublish");
		Node firstChild = list.item(0);
		
		
		//3. Calling the revokeFromConet() method
		logger.info("PG6 - Calling the revokeFromConet() method.");
		try {
			response = revokeFromConet (VDI);
		} catch (MXMException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger.info("PG7 - Response received from CONET.");
		
		
		//4. Calling the unpublishFromComid() method
		logger.info("PG6 - Calling the unpublishFromComid() method.");
		try {
			response = unpublishFromComid (VDI);
		} catch (MXMException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		logger.info("PG7 - Response received from COMID.");
		return response;
	}	
}
