import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.keeper.IDGenerator;
import org.iso.mpeg.mxm.orchestrator.requestevent.RequestEventEngineOrchestrator;
import org.iso.mpeg.mxm.orchestrator.storeevent.StoreEventEngineOrchestrator;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolSuccess;
import org.iso.mpeg.mxm.pEngine.requesteventPE.RequestEventEngine;
import org.iso.mpeg.mxm.pEngine.storeeventPE.StoreEventEngine;
import org.iso.mpeg.mxm.pEngine.storeeventPE.schemahandler.StoreEventRequest;
import org.iso.mpeg.mxm.pEngine.storeeventPE.schemahandler.StoreEventResponse;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;
import org.testng.Assert;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Test;

import java.io.InputStream;

/**
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 4/30/12
 * Time: 6:45 PM
 * <p/>
 * UTI Systems, TIC
 */
@Test
public class StoreAndRequestEventEngineOrchTest {

    private RequestEventEngine requestEventEngine;

    private StoreEventEngine storeEventEngine;

    private EventReportEngine erEngine;

    private RequestEventEngineOrchestrator reqEventOrchestrator;

    private StoreEventEngineOrchestrator storeEventOrchestrator;


    @BeforeTest
    public void beforeTest() throws Exception {
        InputStream is = this.getClass().getClassLoader().getResourceAsStream("MXMConfiguration_ER.xml");
        MXM mxm = MXM.getInstance(is);
        mxm.listEnginesAndProperties();

        System.out.println("PG1 - Introduction......................................");

        requestEventEngine = (RequestEventEngine) mxm.getDefaultEngine(MXMEngineName.RequestEventPE.toString());
        storeEventEngine = (StoreEventEngine) mxm.getDefaultEngine(MXMEngineName.StoreEventPE.toString());

        erEngine = (EventReportEngine) mxm.getDefaultEngine(MXMEngineName.EventReportTE.toString());

        reqEventOrchestrator = new RequestEventEngineOrchestrator(requestEventEngine, erEngine);
        storeEventOrchestrator = new StoreEventEngineOrchestrator(storeEventEngine, erEngine);
    }

    public void testStoreAndRequestER() throws Exception {
        String errId = "err_" + IDGenerator.generateUsingUUID(IDGenerator.ER_IDENTIFIER_PREFIX);
        String DIDLId = "didl_" + IDGenerator.generateUsingUUID(IDGenerator.ER_IDENTIFIER_PREFIX);

        System.out.println("PG1 - Introduction......................................");

        // create dummy ER
        ER er = erEngine.getEventReportEngineSchemaHandler().getER();
        er.setDIIId(DIDLId);
        er.setERRIdentifier(errId);

        // create request (ER)
        StoreEventRequest request = storeEventEngine.getStoreEventSchemaHandler().getStoreEventRequest();
        request.addER(er);

        // store ER
        StoreEventResponse response = storeEventOrchestrator.storeEvent(request);

        // check the response
        if (null != response) {
            if (null != response.getStoreEventSuccess()) {
                ProtocolSuccess success = response.getStoreEventSuccess();
                System.out.println("StoreEvent for ER: response success: " + success.getDisplayString());
            } else if (null != response.getStoreEventFailure()) {
                Assert.fail("StoreEvent for ER: response failure. Details: " + response.getStoreEventFailure().getDisplayString());
            }
        } else {
            Assert.fail("StoreEvent for ER: response is null");
        }

        System.out.println("PG1 - Introduction......................................");


    }
}
