/**
 * ***** BEGIN LICENSE BLOCK ***** Version: MPL 1.1
 * 
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * 
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 * 
 * The Initial Developer of the Original Code is CEDEO.net Portions created by
 * the Initial Developer are Copyright (C) 2006-2007 All Rights Reserved.
 * 
 * The Initial Developer: Andrea Pignatiello
 * 
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK *****
 */

package org.iso.mpeg.mxm.orchestratorTE;


import java.io.File;
import java.util.UUID;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.exceptions.DIEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.IPMPGeneralInfoDescriptor;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.LicenseCollection;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.RightsDescriptor;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngineSchemaHandler;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.CompleteDescription;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Creation;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.CreationDescription;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.CreationInformation;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Mpeg7;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.TextAnnotation;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Textual;
import org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Title;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.ConditionBuilder;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Condition;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Grant;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.ValidityInterval;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.RELRight;


public class DIDEngineOrchestrator {
	
	private final String MPEG7_METADATA_URI = "urn:mpeg:mpeg7:schema:2004";
	
	private Logger logger = Logger.getLogger(DIDEngineOrchestrator.class);
	
	private org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine didEngine;
	private org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine metadataEngine;
	private org.iso.mpeg.mxm.tEngine.relTE.RELEngine relEngine;
	private org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine ipmpEngine;
	private static DIDEngineOrchestrator didEngineOrchestrator;
	
	//private static String OUT_RESOURCE_URL=null;
	public static String OUT_TITLE=null;
	public static String OUT_ABS=null;
	public static String OUT_NV_BEFORE=null;
	public static String OUT_NV_AFTER=null;
	
	private DIDEngineOrchestrator(org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine didEngine, org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine metadataEngine, org.iso.mpeg.mxm.tEngine.relTE.RELEngine relEngine, org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine ipmpEngine) {
		this.didEngine = didEngine;
		this.metadataEngine = metadataEngine;
		this.relEngine = relEngine;
		this.ipmpEngine = ipmpEngine;		
	}

	public static DIDEngineOrchestrator getInstance(org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine didEngine, org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine metadataEngine, org.iso.mpeg.mxm.tEngine.relTE.RELEngine relEngine, org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine ipmpEngine) {
		if (didEngineOrchestrator == null)
			didEngineOrchestrator = new DIDEngineOrchestrator(didEngine, metadataEngine, relEngine, ipmpEngine);
		return didEngineOrchestrator;
	}
	
	public DIDL buildSimpleDI(String MIMETYPE, String RESOURCE_URL, String TITLE, String ABS, String NV_BF, String NV_AF) throws DIEngineException{
		try {

			//resource
			Resource resource=didEngine.getDIEngineSchemaHandler().getResource();
			resource.setMime(MIMETYPE);
			resource.setRef(RESOURCE_URL);
			Component c=didEngine.getDIEngineSchemaHandler().getComponent();
			c.addResource(resource);
			Item innerItem=didEngine.getDIEngineSchemaHandler().getItem();
			innerItem.setId("B0");
			innerItem.setIdentifier("identifier:contentelement:temp:" + UUID.randomUUID());	
			innerItem.addComponent(c);
			
			//ipmp
			IPMPGeneralInfoDescriptor gen_desc=ipmpEngine.getIPMPEngineSchemaHandler().getIPMPGeneralInfoDescriptor();
			LicenseCollection lc=ipmpEngine.getIPMPEngineSchemaHandler().getLicenseCollection();
			RightsDescriptor rd=ipmpEngine.getIPMPEngineSchemaHandler().getRightsDescriptor();
			org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.License ipmp_l =ipmpEngine.getIPMPEngineSchemaHandler().getIPMPLicense();
			
			License l =relEngine.getRELEngineSchemaHandler().getLicense();
			l.setLicenseId("identifier:licence:temp:" + UUID.randomUUID());	
			
			//licence
			ConditionBuilder c2 = relEngine.getRELEngineSchemaHandler().getConditionBuilder();
			DatatypeFactory dateTypeFactory=DatatypeFactory.newInstance();
	
			XMLGregorianCalendar notBefore = null;
			if ((NV_BF!=null)&(!NV_BF.isEmpty()))
			{
				notBefore = dateTypeFactory.newXMLGregorianCalendar(NV_BF);
			}
			XMLGregorianCalendar notAfter = null;
			if ((NV_AF!=null)&(!NV_AF.isEmpty()))
			{
				notAfter = dateTypeFactory.newXMLGregorianCalendar(NV_AF);
			}
			Grant grant = relEngine.getRELEngineSchemaHandler().getGrant();
			grant.setRELRight(RELRight.play);
			Condition valInt = relEngine.getRELEngineSchemaHandler().getConditionBuilder().buildValidityInterval(notBefore, notAfter);
			if (valInt!= null)
			{
				valInt.setLicensePartIdRef("oneYear");
				grant.setCondition(valInt);
			}
			l.addGrant(grant);
			
			//ipmp
			ipmp_l.addLicense(l);
			rd.setLicense(ipmp_l);
			lc.setRightsDescriptor(rd);
			gen_desc.setLicenseCollection(lc);
			innerItem.addIPMPGeneralInfoDescriptor(gen_desc);
			
			
			DIDL di=didEngine.getDIEngineSchemaHandler().getDIDL();
			Item toplevelitem=didEngine.getDIEngineSchemaHandler().getItem();
			toplevelitem.setId("A0");
			toplevelitem.setIdentifier("identifier:content:temp:" + UUID.randomUUID());	
			toplevelitem.addItem(innerItem);
			
			//metadata
			MetadataEngineSchemaHandler metadataEngineSchemaHandler = metadataEngine.getMetadataEngineSchemaHandler();
			CreationDescription cd= metadataEngineSchemaHandler.getCreationDescription();
			CreationInformation creationInformation = metadataEngineSchemaHandler.getCreationInformation();
			Creation creation = metadataEngineSchemaHandler.getCreation();
			TextAnnotation ta = metadataEngineSchemaHandler.getTextAnnotation();
			Title title = metadataEngineSchemaHandler.getTitle();
			title.setValue(TITLE);
			creation.addTitle(title);
			if (ABS!=null)	
			{
				Textual abs = metadataEngineSchemaHandler.getTextual();
				abs.setValue(ABS);
				ta.addFreeTextAnnotation(abs);
				creation.addAbstract(ta);
			}
			creationInformation.setCreation(creation);
			cd.addCreationInformation(creationInformation);
			Mpeg7 mpeg7smp = metadataEngineSchemaHandler.getMpeg7();
			mpeg7smp.addDescription(cd);

			StructuredData sd=didEngine.getDIEngineSchemaHandler().getStructuredData();
			sd.setRef(MPEG7_METADATA_URI);			
			sd.setAny(mpeg7smp);
			toplevelitem.addMetadata(sd);

			di.setMPEGMItem(toplevelitem);
			return di;
		} catch (DatatypeConfigurationException e) {
			logger.error(e.getMessage());
			throw new DIEngineException(e.getMessage(),e);
		} catch (MXMEngineException e) {
			logger.error(e.getMessage());
			throw new DIEngineException(e.getMessage(),e);
		} catch (MXMException e) {
			logger.error(e.getMessage());
			throw new DIEngineException(e.getMessage(),e);
		}			
	}

	public DIDL buildDI(String MIMETYPE, String RESOURCE_URL, String TITLE, String ACTOR_GIVENNAME, String ACTOR_FAMILYNAME) throws DIEngineException
	{	
		try 
		{	
			// Creating the Resource element
			Resource resource1=didEngine.getDIEngineSchemaHandler().getResource();
			resource1.setMime(MIMETYPE);
			resource1.setRef(RESOURCE_URL);
			
			// Creating the Item element containing the Resource element
			Component c=didEngine.getDIEngineSchemaHandler().getComponent();
			c.addResource(resource1);
			Item innerItem=didEngine.getDIEngineSchemaHandler().getItem();
			innerItem.setIdentifier("identifier:contentelement:temp:" + UUID.randomUUID());
			innerItem.addComponent(c);
			
			//TODO setERR
			//TODO setLicense
			
			// Creating the topLevel Item element
			Item topLevelItem = didEngine.getDIEngineSchemaHandler().getItem();
			topLevelItem.setIdentifier("identifier:contentitem:temp:"+ UUID.randomUUID());
			topLevelItem.addItem(innerItem);
			
			// Create Metadata Element
			org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Mpeg7 m7=metadataEngine.getMetadataEngineSchemaHandler().getMpeg7();	
//			org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.DescriptionMetadata dm=metadataEngine.getMetadataEngineSchemaHandler().getDescriptionMetadata();
//			org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.Creator cr=metadataEngine.getMetadataEngineSchemaHandler().getCreator();
//			org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.PersonName pn=metadataEngine.getMetadataEngineSchemaHandler().getPersonName();
//			org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.NameComponent fn_nc=metadataEngine.getMetadataEngineSchemaHandler().getNameComponent();
//			org.iso.mpeg.mxm.tEngine.metadataTE.schemahandler.mpeg7_v3.NameComponent fn_gn=metadataEngine.getMetadataEngineSchemaHandler().getNameComponent();
//			fn_nc.setValue(ACTOR_FAMILYNAME);
//			fn_nc.setValue(ACTOR_GIVENNAME);
//			pn.addFamilyName(fn_nc);
//			pn.addGivenName(fn_gn);
//			cr.getCharacter().add(pn);
//			dm.getCreator().add(cr);
//			m7.setDescriptionMetadata(dm);
			StructuredData sd=didEngine.getDIEngineSchemaHandler().getStructuredData();
			sd.setRef(MPEG7_METADATA_URI);			
			sd.setAny(m7);
			topLevelItem.addMetadata(sd);
			
			// Create DI
			DIDL dci1=didEngine.getDIEngineSchemaHandler().getDIDL();
			dci1.setMPEGMItem(topLevelItem);
			return dci1;
		} 
		catch (MXMEngineException e) 
		{
			logger.error(e.getMessage());
			throw new DIEngineException(e.getMessage(),e);
		}
		catch (MXMException e) 
		{
			logger.error(e.getMessage());
			throw new DIEngineException(e.getMessage(),e);
		}		
	}
	
	//return url of the media
	public String readDI(String SOURCE_DCI_FILE_NAME) throws MXMEngineException
	{
		
		//System.out.println("The path of the VDI is: "+SOURCE_DCI_FILE_NAME);
		
		OUT_NV_AFTER=null;
		OUT_NV_BEFORE=null;
		
		String url_toreturn=null;

		try 
		{

			DIDL dci1=didEngine.getDIEngineSchemaHandler().getDIDL();
			Mpeg7 mp7=metadataEngine.getMetadataEngineSchemaHandler().getMpeg7();
			
			MXMEngineResponse response = dci1.parse(new MXMAdapter(new File(SOURCE_DCI_FILE_NAME)));
			if (response.equals(MXMEngineResponse.SUCCESS)) {
				Item topLevelItem=(Item)dci1.getMPEGMItem();
			
				logger.info("Content ID: " + topLevelItem.getId());
				logger.info("Identifier: " + topLevelItem.getIdentifier());

				if (topLevelItem.getMetadata()!=null)
				{
					logger.info("there are some metdata for top level item");
					logger.info("WITH NS:"+topLevelItem.getMetadata().getRef());
					
					if (topLevelItem.getMetadata().getRef().equals(MPEG7_METADATA_URI))
					{
						Object o= topLevelItem.getMetadata().getAny();
						mp7.parse(new MXMAdapter(o));
						logger.info("time unit is:"+mp7.getTimeUnit());
			
				
				
						for (CompleteDescription completeDescription : mp7.getDescription()) {			
							if (completeDescription != null) {
								if( completeDescription instanceof CreationDescription ){
									CreationDescription cd = (CreationDescription) completeDescription;
									
										for (CreationInformation ci:cd.getCreationInformation()){
									
											for (Title title : ci.getCreation().getTitle()) {
												logger.info("Title: " + title.getValue());
												OUT_TITLE=title.getValue();
											}
											
											for (TextAnnotation ta : ci.getCreation().getAbstract()) {
												for (Textual t:ta.getFreeTextAnnotation()){
													logger.info("ABS: " + t.getValue());
													OUT_ABS=t.getValue();
												}
											}
										}
								}
							}
						}
					}
				}
				

			
				
//				
//				//let's cycle  through the content elements
				for (Item item:dci1.getMPEGMItem().getItem()){
//				
					logger.info("Content element ID: " + item.getId());
					
					if (item.getIPMPGeneralInfoDescriptor()!=null){
						if (item.getIPMPGeneralInfoDescriptor().getLicenseCollection()!=null){
							if (!item.getIPMPGeneralInfoDescriptor().getLicenseCollection().getRightsDescriptor().isEmpty()){
								for (RightsDescriptor rd:item.getIPMPGeneralInfoDescriptor().getLicenseCollection().getRightsDescriptor()){
									if ((rd.getLicense()!=null)&&(!rd.getLicense().getLicense().isEmpty()))
										for (License l: rd.getLicense().getLicense()) {
											logger.info("LICENSE id is: "+l.getLicenseId());
																				
											for (Grant g : l.getGrants())
											{
												for (Condition c: g.getConditions()){
																	
													logger.info("--- Condition: " + (c.getClass().getCanonicalName()));
													if (c instanceof ValidityInterval)
														{
															ValidityInterval validityInterval = (ValidityInterval) c;
															
															if (validityInterval.getNotAfter()!=null)
															{
																logger.info("Not After: " + validityInterval.getNotAfter().toString());
																OUT_NV_AFTER=validityInterval.getNotAfter().toString();
															}
															else
															{
																logger.info("Not After is NULL");

															}
															if (validityInterval.getNotBefore()!=null)
															{
																logger.info("Not Before: " + validityInterval.getNotBefore().toString());
																OUT_NV_BEFORE=validityInterval.getNotBefore().toString();
															}
															else
															{
																logger.info("Not Before is NULL");
															}
														}
												}

											}
										}
								}
							}
						}
					}
//					
//					//TODO manage Metadata
//
//					//TODO manage Signature
//					
					for (Component c:item.getComponent()){
						for (Resource r:c.getResource()){
							logger.info("resource mime>"+r.getMime());
							logger.info("resource ref>"+r.getRef());
							url_toreturn=r.getRef();
							//OUT_RESOURCE_URL=url_toreturn;
							logger.info("resource contencoding>"+r.getContentEncoding());
							logger.info("resource encoding>"+r.getEncoding());
						}	
					}
				}
			}
		}
		catch (MXMException e) 
		{
			System.err.println(e.getMessage());
			throw new DIEngineException(e.getMessage(),e);
		}
		return url_toreturn;
	}
	
	public void updateDI(String SOURCE_DCI_FILE_NAME, String output, String newReference, String newmimetype){


			DIDL dci1;
			try {
				dci1 = didEngine.getDIEngineSchemaHandler().getDIDL();
				
			
				MXMEngineResponse response = dci1.parse(new MXMAdapter(new File(SOURCE_DCI_FILE_NAME)));
				if (response.equals(MXMEngineResponse.SUCCESS)) 
				{
					Item topLevelItem=(Item)dci1.getMPEGMItem();	
					for (Item item:dci1.getMPEGMItem().getItem()){
						for (Component c:item.getComponent()){
							for (Resource r:c.getResource()){
								logger.info("resource mime>"+r.getMime());
								logger.info("... updating");
								r.setMime(newmimetype);
								logger.info("resource mime>"+r.getMime());								
								logger.info("resource ref>"+r.getRef());
								logger.info("... updating");
								r.setRef(newReference);
								logger.info("resource ref>"+r.getRef());
							}	
						}
					}
					dci1.createFile(new File(output));
				}
			} catch (MXMEngineException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (MXMException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
	}
}
