package org.iso.mpeg.mxm.orchestratorTE;

import java.awt.Canvas;
import java.io.InputStream;
import java.util.List;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAbstractEngine;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.IdentifyContentPE;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.exceptions.IdentifyContentEngineException;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.IdentifyUserEngine;
import org.iso.mpeg.mxm.pEngine.identifyuserPE.exception.IdentifyUserEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.exceptions.DIEngineException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class OrchestratorTE extends MXMAbstractEngine implements MXMEngine {
	
	Logger logger = LoggerFactory.getLogger(this.getClass());
	
	private static org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine metadataEngine = null;
	private static org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine didEngine = null;
	private static org.iso.mpeg.mxm.tEngine.relTE.RELEngine relEngine = null;
	private static org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine ipmpEngine = null;	
	private static org.iso.mpeg.mxm.pEngine.identifycontentPE.IdentifyContentPE icontentProtocolEngine = null;
	private static org.iso.mpeg.mxm.tEngine.mediaframeworkTE.MediaFrameworkEngine MFEngine = null;
	private static org.iso.mpeg.mxm.pEngine.identifyuserPE.IdentifyUserEngine identifyUserEngine = null;
	private static org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine securityEngine = null;
    private static org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine erEngine = null;
	//private static org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine baseE = null;
	private static org.iso.mpeg.mxm.pEngine.createlicensePE.CreateLicenseEngine createLicenseEngine = null;
	private static org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine createContentEngine = null;
	private static org.iso.mpeg.mxm.tEngine.mp21ffTE.MPEG21FileEngine fileEngine = null;
    
	private static org.iso.mpeg.mxm.pEngine.authenticatecontentPE.AuthenticateContentEngine authenticateContentEngine = null;
	
	private static OrchestratorTE orchestratorEngine = null;
	
	private static MXM mxm = null;
	
	private static void getInstance(InputStream is) throws MXMException
	{
		if(orchestratorEngine == null)
			orchestratorEngine = new OrchestratorTE(is);
	}
	
	private static void getInstance() throws MXMException
	{
		if(orchestratorEngine == null)
			orchestratorEngine = new OrchestratorTE();
	}
	
	private OrchestratorTE(InputStream is) throws MXMException
	{
		try
		{
			this.initialise(is);
		} 
		catch (MXMConfigurationException e) 
		{
			logger.error("Unable to initialise",e);
			throw e;
		}
	}
	
	private OrchestratorTE() throws MXMException
	{
		try
		{
			this.initialise(getClass().getClassLoader().getResourceAsStream("MXMConfiguration.xml"));
		} 
		catch (MXMConfigurationException e) 
		{
			logger.error("Unable to initialise",e);
			throw e;
		}
	}
	
	public static MXM getMXM() throws MXMException{
		getInstance();
		return mxm;
	}
	
	public static MXM getMXM(InputStream is) throws MXMException{
		getInstance(is);
		return mxm;
	}
	
	private void initialise(InputStream is) throws MXMConfigurationException, MXMException
	{
		
		mxm = MXM.getInstance(is);

		try 
		{
			metadataEngine = (org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine) mxm.getDefaultEngine(MXMEngineName.MetadataTE.value());
			didEngine = (org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine) mxm.getDefaultEngine(MXMEngineName.DigitalItemTE.value());
			icontentProtocolEngine = (org.iso.mpeg.mxm.pEngine.identifycontentPE.IdentifyContentPE) mxm.getDefaultEngine(MXMEngineName.IdentifyContentPE.value());
			MFEngine = (org.iso.mpeg.mxm.tEngine.mediaframeworkTE.MediaFrameworkEngine) mxm.getDefaultEngine(MXMEngineName.MediaFrameworkTE.value());
			identifyUserEngine = (org.iso.mpeg.mxm.pEngine.identifyuserPE.IdentifyUserEngine) mxm.getDefaultEngine(MXMEngineName.IdentifyUserPE.value());
			securityEngine = (org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine) mxm.getDefaultEngine(MXMEngineName.SecurityTE.value());
			erEngine = (org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine) mxm.getDefaultEngine(MXMEngineName.EventReportTE.value());
			//authenticateContentEngine = (org.iso.mpeg.mxm.pEngine.authenticatecontentPE.AuthenticateContentEngine) mxm.getDefaultEngine(MXMEngineName.AuthenticateContentPE.value());
			createLicenseEngine = (org.iso.mpeg.mxm.pEngine.createlicensePE.CreateLicenseEngine) mxm.getDefaultEngine(MXMEngineName.CreateLicensePE.value());
			createContentEngine = (org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine) mxm.getDefaultEngine(MXMEngineName.CreateContentPE.value());
			fileEngine = (org.iso.mpeg.mxm.tEngine.mp21ffTE.MPEG21FileEngine)mxm.getDefaultEngine(MXMEngineName.MPEG21FileTE.toString());
			relEngine = (org.iso.mpeg.mxm.tEngine.relTE.RELEngine)mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
			ipmpEngine = (org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine)mxm.getDefaultEngine(MXMEngineName.IPMPTE.toString());
			
		} catch (MXMEngineNotFoundException e) {
			throw new MXMConfigurationException("Unable to getDefaultEngine",e);
		}

	}
	
	public String getEngineName() {
		return MXMEngineName.OrchestratorTE.value();
	}
	
	public static DIDEngineOrchestrator dealWithDIDEngine() throws DIEngineException {
		try {
			getInstance();
			return DIDEngineOrchestrator.getInstance(didEngine, metadataEngine, relEngine, ipmpEngine);
		} catch (MXMException e) {
			throw new DIEngineException(e.getMessage(),e);
		}
	}

	public static IdentifyContentEngineOrchestrator dealWithIdentifyContentEngine() throws IdentifyContentEngineException {
		try {
			getInstance();
			return IdentifyContentEngineOrchestrator.getInstance(icontentProtocolEngine, didEngine);
		} catch (MXMException e) {
			throw new IdentifyContentEngineException(e.getMessage(),e);
		}
	}

	public static MFOrchestrator dealWithMFEngine(Canvas canvas) throws MXMEngineException {
		try {
			getInstance();
			return MFOrchestrator.getInstance(MFEngine, canvas);
		} catch (MXMException e) {
			throw new MXMEngineException(e.getMessage(),e);
		}
	}
	
	public static MFOrchestrator dealWithMFEngine() throws MXMEngineException {
		try {
			getInstance();
			return MFOrchestrator.getInstance(MFEngine,null);
		} catch (MXMException e) {
			throw new MXMEngineException(e.getMessage(),e);
		}
	}
	
	public static IdentifyUserEngineOrchestrator dealWithIdentifyUserEngine() throws IdentifyUserEngineException
	{
		try
		{
			getInstance();
			return IdentifyUserEngineOrchestrator.getInstance(identifyUserEngine);
		} catch (MXMException e)
		{
			throw new IdentifyUserEngineException(e.getMessage(), e);
		}
	}
	
	public static org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine getMetadataEngine() throws MXMException {
		getInstance();
		return metadataEngine;
	}
	
//	public static BaseProtocolEngine getBaseEngine() throws MXMException {
//		getInstance();
//		return baseE;
//	}
	
	/*public static IPMPTE getIpmpEngine() throws MXMException {
		getInstance();
		return ipmpEngine;
	}*/
	
	public static org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine getDidEngine() throws MXMException {
		getInstance();
		return didEngine;
	}

    public static org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine getEREngine() throws MXMException {
		getInstance();
		return erEngine;
	}
    
    public static org.iso.mpeg.mxm.tEngine.mp21ffTE.MPEG21FileEngine getMP21FileEngine() throws MXMException {
		getInstance();
		return fileEngine;
	}
	
	public static IdentifyContentPE getIdentifyContentProtocolEngine() throws MXMException {
		getInstance();
		return icontentProtocolEngine;
	}
	
	public static IdentifyUserEngine getIdentifyUserEngine() throws MXMException
	{
		getInstance();
		return identifyUserEngine;
	}
	
	public static org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine getSecurityEngine() throws MXMException
	{
		getInstance();
		return securityEngine;
	}
	
	public static org.iso.mpeg.mxm.pEngine.authenticatecontentPE.AuthenticateContentEngine getAuthenticateContentEngine() throws MXMException
	{
		getInstance();
		return authenticateContentEngine;
	}
	
	public static org.iso.mpeg.mxm.pEngine.createlicensePE.CreateLicenseEngine getCreateLicenseEngine() throws MXMException
	{
		getInstance();
		return createLicenseEngine;
	}
	
	public static org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine getCreateContentEngine() throws MXMException
	{
		getInstance();
		return createContentEngine;
	}

	@Override
	public void setDependencies(List<MXMEngine> engines)
			throws MXMEngineException {
		// TODO Auto-generated method stub
	}
	
	public static void reset(){
		mxm=null;
		orchestratorEngine=null;
	}
}
