package org.iso.mpeg.mxm.relTE.schemahandler;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.relTE.schema.SuperRelAbstractLevel1;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.EncryptedContent;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Grant;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.GrantGroup;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Inventory;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Issuer;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.LicenseOtherInfo;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.LicensePart;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.LinguisticString;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

public class License implements org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License 
{
	protected org.iso.mpeg.mxm.engine.relTE.schema.relr.License licenseType;
	private SecurityEngine securityEngine;
	
	public License(SecurityEngine securityEngine) 
	{
		this(new org.iso.mpeg.mxm.engine.relTE.schema.relr.License(), securityEngine);
	}
	
	public License(org.iso.mpeg.mxm.engine.relTE.schema.relr.License licenseType, SecurityEngine securityEngine)
	{
		this.licenseType = licenseType;
		this.securityEngine = securityEngine;
	}

	@Override
	public Map<QName, String> getOtherAttributes()
	{
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Inventory getInventory()
	{
		if (licenseType.getInventory()!=null)
		{
			return new org.iso.mpeg.mxm.relTE.schemahandler.Inventory(licenseType.getInventory());
		}
		else
		{
			return new org.iso.mpeg.mxm.relTE.schemahandler.Inventory();
		}
	}

	@Override
	public void setInventory(Inventory inventory)
	{
		try
		{
			licenseType.setInventory((org.iso.mpeg.mxm.engine.relTE.schema.relr.Inventory) inventory.createObject(
					org.iso.mpeg.mxm.engine.relTE.schema.relr.Inventory.class.getCanonicalName()).getContent());
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}

	public List<Grant> getGrants()
	{
		List<Grant> grantList = new ArrayList<Grant>();
		
		for (org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource grantResource : licenseType.getGrantOrGrantGroup())
		{
			if (grantResource instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.Grant)
			{
				org.iso.mpeg.mxm.relTE.schemahandler.resource.Grant grant = 
						new org.iso.mpeg.mxm.relTE.schemahandler.resource.Grant(
						(org.iso.mpeg.mxm.engine.relTE.schema.relr.Grant) grantResource, securityEngine);
				
				if (grant.getLicensePartIdRef()!=null && getInventory()!=null)
				{
					for (LicensePart licensePart : getInventory().getLicenseParts())
					{
						if (grant.getLicensePartIdRef().equals(licensePart.getLicensePartIdRef()) &&
								licensePart instanceof Grant)
						{
							((Grant) licensePart).setInventory(getInventory());
							
							grantList.add((Grant) licensePart);
						}
					}
				}
				else
				{
					grant.setInventory(getInventory());
					
					grantList.add(grant);
				}
			}
		}
		
		return grantList;
	}
	
	public void addGrant(Grant grant)
	{
		try
		{
			licenseType.getGrantOrGrantGroup().add(((org.iso.mpeg.mxm.engine.relTE.schema.relr.Grant) grant.createObject(
					org.iso.mpeg.mxm.engine.relTE.schema.relr.Grant.class.getCanonicalName()).getContent()));
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}
	
	public List<GrantGroup> getGrantGroups()
	{
		List<GrantGroup> grantGroupList = new ArrayList<GrantGroup>();
		// TODO Auto-generated method stub
		return grantGroupList;
	}
	
	public void addGrantGroup(GrantGroup grantGroup)
	{
		try
		{
			licenseType.getGrantOrGrantGroup().add(((org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup) grantGroup.createObject(
					org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup.class.getCanonicalName()).getContent()));
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public List<Issuer> getIssuers()
	{
		List<Issuer> issuers = new ArrayList<Issuer>();
		
		for (org.iso.mpeg.mxm.engine.relTE.schema.relr.Issuer issuerType : licenseType.getIssuer())
		{
			issuers.add(new org.iso.mpeg.mxm.relTE.schemahandler.Issuer(issuerType, securityEngine));
		}
		return issuers;
	}
	
	@Override
	public void addIssuer(Issuer issuer)
	{
		try
		{
			licenseType.getIssuer().add((org.iso.mpeg.mxm.engine.relTE.schema.relr.Issuer) 
					issuer.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.Issuer.class.getCanonicalName()).getContent());
		}
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public LicenseOtherInfo getOtherInfo()
	{
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void setOtherInfo(LicenseOtherInfo otherInfo)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public EncryptedContent getEncryptedLicense()
	{
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void setEncryptedLicense(EncryptedContent encryptedContent)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public String getLicenseId()
	{
		return licenseType.getLicenseId();
	}

	@Override
	public void setLicenseId(String string)
	{
		licenseType.setLicenseId(string);
	}

	@Override
	public List<LinguisticString> getTitle()
	{
		// TODO Auto-generated method stub
		return null;
	}
	
	// MXM CREATOR AND PARSER METHODS

	@Override
	public void createFile(File f) throws MXMException
	{
		new SuperRelAbstractLevel1(licenseType).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		return new SuperRelAbstractLevel1(licenseType).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperRelAbstractLevel1 superAbstract = new SuperRelAbstractLevel1(licenseType);

		MXMEngineResponse repsonse = superAbstract.parse(object);

		licenseType = (org.iso.mpeg.mxm.engine.relTE.schema.relr.License) 
				superAbstract.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.License.class.getCanonicalName()).getContent();

		return repsonse;
	}
	
}