/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006
 * All Rights Reserved.S
 *
 * The Initial Developer: Filippo Chiariglione
 * Contributor(s): Edoardo Radica
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.relTE.schemahandler.condition;

import java.math.BigInteger;
import java.util.List;

import javax.xml.datatype.Duration;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm2x.SecuritySystem;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm3x.CopyrightNotice;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm3x.NonCommercialUse;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.AllConditions;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Condition;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.AccountPayable;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.ExerciseLimit;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.FeeFlat;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.FeeMetered;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.FeePerInterval;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.FeePerUse;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.FeePerUsePrePay;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.Rate;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.Territory;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.ValidityTimeMetered;

public class ConditionBuilder implements org.iso.mpeg.mxm.tEngine.relTE.schemahandler.ConditionBuilder
{
	private Logger logger = Logger.getLogger(ConditionBuilder.class);

	@Override
	public AllConditions buildAllConditions(List<Condition> conditions)
	{
		AllConditions allConditions = new org.iso.mpeg.mxm.relTE.schemahandler.condition.AllConditions();
		
		for (Condition condition : conditions)
		{
			allConditions.addCondition(condition);
		}
		
		return allConditions;
	}
	
	/**
	 * Builds the exercise limit condition.
	 * 
	 * @param conditionCountLimit java.math.BigInteger exercise limit
	 * 
	 * @return the JAXB element< exercise limit>
	 */
	public ExerciseLimit buildCountLimit(BigInteger conditionCountLimit) // TODO
	{
//		if(conditionCountLimit != null)
//		{
//			ExerciseLimit limit = factory.createExerciseLimit();
//			limit.setCount(conditionCountLimit);
//		}
		
		return null;
	}

	/**
	 * Builds the Territory location condition.
	 * 
	 * @param conditionCountry javax.xml.namespace.QName ISO code of a certain country
	 * @param conditionRegion javax.xml.namespace.QName ISO code of a certain region
	 * 
	 * @return the JAXB element< territory>
	 */
	public Territory buildTerritoryLocation(QName conditionCountry, QName conditionRegion) // TODO
	{
//		if (conditionCountry == null)
//			return null;
//		
//		Territory territory = factory.createTerritory();
//		Location location = factory.createTerritoryLocation();
//		location.setCountry(conditionCountry);
//		if (conditionRegion != null) {
//			location.setRegion(conditionRegion);
//		}
//		territory.getLocationOrDomain().add(location);
		
		return null;
	}

	/**
	 * Builds the Territory domain condition.
	 * 
	 * @param domainURI The URI of a certain Domain
	 * 
	 * @return the JAXB element< territory>
	 */
	public Territory buildTerritoryDomain(String domainURI) // TODO
	{
//		if (domainURI == null)
//			return null;
//		
//		Territory territory = factory.createTerritory();
//		Domain domain = factory.createTerritoryDomain();
//		domain.setUri(domainURI);
//		territory.getLocationOrDomain().add(domain);
		
		return null;
	}

	/**
	 * Builds the ValidityInterval condition.
	 * 
	 * @param conditionNotBeforeDate javax.xml.datatype.XMLGregorianCalendar
	 * NotBeforeDate with the format YYYY-MM-DDTHH:MM:SS
	 * @param conditionNotAfterDate javax.xml.datatype.XMLGregorianCalendar
	 * NotAfterDate with the format YYYY-MM-DDTHH:MM:SS
	 * 
	 * @return the JAXB element< validity interval>
	 */
	public ValidityInterval buildValidityInterval(XMLGregorianCalendar conditionNotBeforeDate, XMLGregorianCalendar conditionNotAfterDate)
	{
		if(conditionNotBeforeDate != null || conditionNotAfterDate != null)
		{
			ValidityInterval validityInterval = new org.iso.mpeg.mxm.relTE.schemahandler.condition.ValidityInterval();
			if(conditionNotBeforeDate != null)
				validityInterval.setNotBefore(conditionNotBeforeDate);
			if(conditionNotAfterDate != null)
				validityInterval.setNotAfter(conditionNotAfterDate);
			
			return validityInterval;
		}
		
		return null;
	}
	
	/**
	 * Builds the ValidityTimeMetered condition.
	 * 
	 * @param validTimeMeteredDuration the Duration expressing for how long a right can be exercised
	 * 
	 * @return the JAXB element< validity time metered>
	 */
	public ValidityTimeMetered buildValidityTimeMeteredDuration(Duration validTimeMeteredDuration) // TODOs
	{
//		if(validTimeMeteredDuration == null)
//			return null;
//
//		ValidityTimeMetered validityTimeMetered = factory.createValidityTimeMetered();
//		validityTimeMetered.setDuration(validTimeMeteredDuration);
		
		return null;
	}

	/**
	 * Builds the ValidityTimeMetered condition.
	 * 
	 * @param validTimeMeteredQuantum the valid time metered quantum
	 * 
	 * @return the JAXB element< validity time metered>
	 */
	public ValidityTimeMetered buildValidityTimeMeteredQuantum(Duration validTimeMeteredQuantum) // TODO
	{
//		if(validTimeMeteredQuantum == null)
//			return null;
//
//		ValidityTimeMetered validityTimeMetered = factory.createValidityTimeMetered();
//		validityTimeMetered.setQuantum(validTimeMeteredQuantum);
		
		return null;
	}

	/**
	 * Builds the SecuritySystem condition.
	 * 
	 * @param securityIdentifier The Identifier of the Security System targeted
	 * @param securityLevel The value in the Security System specified
	 * 
	 * @return the JAXB element< security system>
	 */
	public SecuritySystem buildSecuritySystem(String securityIdentifier, String securityLevel) // TODO
	{
//		if(securityIdentifier == null || securityLevel == null)
//			return null;
//
//		SecuritySystem securitySystem = factory.createSecuritySystem();
//		securitySystem.setIdentifier(securityIdentifier);
//		securitySystem.setLevel(securityLevel);
		
		return null;
	}

	/**
	 * Builds the copyright notice.
	 * 
	 * @param copyrightString the copyright string
	 * @param noticeType the notice type
	 * 
	 * @return the jAXB element< copyright notice>
	 */
	public CopyrightNotice buildCopyrightNotice(String copyrightString, String noticeType) // TODO
	{
//		if (copyrightString == null || copyrightString.equals(""))
//			return null;
//		CopyrightNotice copyNotice = factory.createCopyrightNotice();
//		copyNotice.setCopyrightString(copyrightString);
//		if (noticeType != null)
//			copyNotice.setNoticeType(new QName("urn:mpeg:mpeg21:2006:01-REL-M3X-NS", noticeType));
		
		return null;
	}

	/**
	 * Builds the non commercial use.
	 * 
	 * @return the jAXB element< non commercial use>
	 */
	public NonCommercialUse buildNonCommercialUse() // TODO
	{
//		NonCommercialUse nonComm = factory.createNonCommercialUse();
		
		return null;
	}

	/**
	 * Builds the payment condition.
	 * 
	 * @param conditionName the condition name
	 * @param rate the rate
	 * @param to the to
	 * 
	 * @return the JAXB element<? extends condition>
	 * 
	 * @throws RELCreationException the REL creation exception
	 */
	@Override
	public FeeFlat buildFeeFlatPaymentCondition(Rate rate, AccountPayable to) // TODO
	{
//		FeeFlat condition = factory.createFeeFlat();
//		condition.setRate(rate);
//		condition.setTo(to);
		
		return null;
	}

	/**
	 * Builds the payment condition.
	 * 
	 * @param conditionName the condition name
	 * @param rate the rate
	 * @param to the to
	 * 
	 * @return the JAXB element<? extends condition>
	 * 
	 * @throws RELCreationException the REL creation exception
	 */
	@Override
	public FeeMetered buildFeeMeteredPaymentCondition(Rate rate, AccountPayable to) // TODO
	{
//		FeeMetered condition = factory.createFeeMetered();
//		condition.setRate(rate);
//		condition.setTo(to);
		
		return null;
	}
	
	/**
	 * Builds the payment condition.
	 * 
	 * @param conditionName the condition name
	 * @param rate the rate
	 * @param to the to
	 * 
	 * @return the JAXB element<? extends condition>
	 * 
	 * @throws RELCreationException the REL creation exception
	 */
	@Override
	public FeePerInterval buildFeePerIntervalPaymentCondition(Rate rate, AccountPayable to) // TODO
	{
//		FeePerInterval condition = factory.createFeePerInterval();
//		condition.setRate(rate);
//		condition.setTo(to);
		
		return null;
	}

	/**
	 * Builds the payment condition.
	 * 
	 * @param conditionName the condition name
	 * @param rate the rate
	 * @param to the to
	 * 
	 * @return the JAXB element<? extends condition>
	 * 
	 * @throws RELCreationException the REL creation exception
	 */
	@Override
	public FeePerUse buildFeePerUsePaymentCondition(Rate rate, AccountPayable to) // TODO
	{
//		FeePerUse condition = factory.createFeePerUse();
//		condition.setRate(rate);
//		condition.setTo(to);

		return null;
	}

	/**
	 * Builds the payment condition.
	 * 
	 * @param conditionName the condition name
	 * @param rate the rate
	 * @param to the to
	 * 
	 * @return the JAXB element<? extends condition>
	 * 
	 * @throws RELCreationException the REL creation exception
	 */
	@Override
	public FeePerUsePrePay buildFeePerUsePrePayPaymentCondition(Rate rate, AccountPayable to) // TODO
	{
//		FeePerUsePrePay condition = factory.createFeePerUsePrePay();
//		condition.setRate(rate);
//		condition.setTo(to);

		return null;
	}

}
