package org.iso.mpeg.mxm.relTE.schemahandler.resource;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBElement;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.relTE.schema.SuperRelAbstractLevel1;
import org.iso.mpeg.mxm.engine.relTE.schema.relr.KeyHolder;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Condition;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.DelegationControl;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.EncryptedContent;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.ForAll;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Grant;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.LicensePart;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Principal;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

public class GrantGroup extends Resource implements org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.GrantGroup
{
	private org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup grantGroupType;
	private List<LicensePart> inventory;
	
	private SecurityEngine securityEngine;
	
	public GrantGroup(SecurityEngine securityEngine)
	{
		this(new org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup(), securityEngine);
	}
	
	public GrantGroup(org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup grantGroup, SecurityEngine securityEngine)
	{
		super(grantGroup);
		this.grantGroupType = grantGroup;
		this.securityEngine = securityEngine;
	}
	
	public void setInventory(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Inventory inventory)
	{
		this.inventory = inventory.getLicenseParts();
	}
	
	@Override
	public List<Grant> getGrants()
	{
		List<Grant> grantList = new ArrayList<Grant>();
		
		List<org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource> resourceList = grantGroupType.getGrantOrGrantGroup();
		
		for (org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource resource : resourceList)
		{
			if (resource instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.Grant)
			{
				org.iso.mpeg.mxm.relTE.schemahandler.resource.Grant grantResource = 
						new org.iso.mpeg.mxm.relTE.schemahandler.resource.Grant((org.iso.mpeg.mxm.engine.relTE.schema.relr.Grant) resource, securityEngine);
				
				if (grantResource.getLicensePartIdRef()!=null && inventory!=null)
				{
					for (LicensePart licensePart : inventory)
					{
						if (grantResource.getLicensePartIdRef().equals(licensePart.getLicensePartIdRef())
								&& licensePart instanceof org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Grant)
						{
							grantList.add((Grant) licensePart);
						}
					}
				}
				else
				{
					grantList.add(grantResource);
				}
			}
		}
		
		return grantList;
	}

	@Override
	public void addGrant(Grant grant)
	{
		try
		{
			grantGroupType.getGrantOrGrantGroup().add((org.iso.mpeg.mxm.engine.relTE.schema.relr.Grant)
					grant.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.Grant.class.getCanonicalName()).getContent());
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}

	@Override
	public List<org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.GrantGroup> getGrantGroups()
	{
		List<org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.GrantGroup> grantGroupList = 
				new ArrayList<org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.GrantGroup>();
		
		List<org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource> resourceList = grantGroupType.getGrantOrGrantGroup();
		
		for (org.iso.mpeg.mxm.engine.relTE.schema.relr.Resource resource : resourceList)
		{
			if (resource instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup)
			{
				GrantGroup grantGroupResource = new GrantGroup((org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup) resource, securityEngine);
				
				if (grantGroupResource.getLicensePartIdRef()!=null && inventory!=null)
				{
					for (LicensePart licensePart : inventory)
					{
						if (grantGroupResource.getLicensePartIdRef().equals(licensePart.getLicensePartIdRef())
								&& licensePart instanceof org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Grant)
						{
							grantGroupList.add((GrantGroup) licensePart);
						}
					}
				}
				else
				{
					grantGroupList.add(grantGroupResource);
				}
			}
		}
		
		return grantGroupList;
	}

	@Override
	public void addGrantGroup(org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.GrantGroup grantGroup)
	{
		try
		{
			grantGroupType.getGrantOrGrantGroup().add((org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup)
					grantGroup.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup.class.getCanonicalName()).getContent());
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}

	@Override
	public List<ForAll> getForAll()
	{
		List<ForAll> forAllList = new ArrayList<ForAll>();
		
		for (org.iso.mpeg.mxm.engine.relTE.schema.relr.ForAll forAll : grantGroupType.getForAll())
		{
			forAllList.add(new org.iso.mpeg.mxm.relTE.schemahandler.ForAll(forAll));
		}
		
		return forAllList;
	}
	
	@Override
	public void addForAll(ForAll forAll)
	{
		try
		{
			grantGroupType.getForAll().add((org.iso.mpeg.mxm.engine.relTE.schema.relr.ForAll)
					forAll.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.ForAll.class.getCanonicalName()).getContent());
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}

	@Override
	public DelegationControl getDelegationControl()
	{
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void setDelegationControl(DelegationControl delegationControl)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public Principal getPrincipal()
	{
		org.iso.mpeg.mxm.engine.relTE.schema.relr.Principal principal = grantGroupType.getPrincipal().getValue();
		
		if (principal instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.KeyHolder)
		{
			org.iso.mpeg.mxm.relTE.schemahandler.resource.principal.KeyHolder keyHolder = 
					new org.iso.mpeg.mxm.relTE.schemahandler.resource.principal.KeyHolder((KeyHolder) principal, securityEngine);
			
			if (keyHolder.getLicensePartIdRef()!=null && inventory!=null)
			{
				for (LicensePart licensePart : inventory)
				{
					if (keyHolder.getLicensePartIdRef().equals(licensePart.getLicensePartIdRef())
							&& licensePart instanceof org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.KeyHolder)
					{
						return (Principal) licensePart;
					}
				}
			}
			else
			{
				return keyHolder;
			}
		}
		else if (principal instanceof org.iso.mpeg.mxm.engine.relTE.schema.relm1x.IdentityHolder)
		{
			org.iso.mpeg.mxm.relTE.schemahandler.resource.principal.IdentityHolder identityHolder = 
					new org.iso.mpeg.mxm.relTE.schemahandler.resource.principal.IdentityHolder(
							(org.iso.mpeg.mxm.engine.relTE.schema.relm1x.IdentityHolder) principal);
			
			if (identityHolder.getLicensePartIdRef()!=null && inventory!=null)
			{
				for (LicensePart licensePart : inventory)
				{
					if (identityHolder.getLicensePartIdRef().equals(licensePart.getLicensePartIdRef())
							&& licensePart instanceof org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm1x.IdentityHolder)
					{
						return (Principal) licensePart;
					}
				}
			}
			else
			{
				return identityHolder;
			}
		}
		else if (principal instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.Principal)
		{
			return new org.iso.mpeg.mxm.relTE.schemahandler.resource.principal.Principal(principal);
		}
		
		return null;
	}

	@Override
	public void setPrincipal(Principal principal)
	{
		try
		{
			grantGroupType.setPrincipal((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Principal>)
					principal.createObject(JAXBElement.class.getCanonicalName()).getContent());
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public List<Condition> getConditions()
	{
		List<Condition> conditionsList = new ArrayList<Condition>();
		
		List<org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition> conditionTypes = 
				new ArrayList<org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition>();
		
		if (grantGroupType.getCondition()!=null)
		{
			org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition grantCondition = grantGroupType.getCondition().getValue();
			
			if (grantCondition instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.AllConditions)
			{
				for (JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition> innerConditionJaxbElement : 
					((org.iso.mpeg.mxm.engine.relTE.schema.relr.AllConditions) grantCondition).getCondition())
				{
					conditionTypes.add(innerConditionJaxbElement.getValue());
				}
			}
			else
			{
				conditionTypes.add(grantCondition);
			}
			
			for (org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition condition : conditionTypes)
			{
				// TODO make this return direct implementations
	//			if (condition instanceof org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition)
	//			{
	//				
	//			}
				
				conditionsList.add(new org.iso.mpeg.mxm.relTE.schemahandler.condition.Condition(condition));
			}
		}
		
		return conditionsList;
	}

	@Override
	public void setCondition(Condition condition)
	{
		try
		{
			grantGroupType.setCondition((JAXBElement<? extends org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition>)
					condition.createObject(JAXBElement.class.getCanonicalName()).getContent());
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public EncryptedContent getEncryptedGrantGroup()
	{
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void setEncryptedGrantGroup(EncryptedContent encryptedContent)
	{
		// TODO Auto-generated method stub
		
	}
	
	// MXM CREATOR AND PARSER METHODS

	@Override
	public void createFile(File f) throws MXMException
	{
		new SuperRelAbstractLevel1(grantGroupType).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		return new SuperRelAbstractLevel1(grantGroupType).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperRelAbstractLevel1 superAbstract = new SuperRelAbstractLevel1(grantGroupType);

		MXMEngineResponse repsonse = superAbstract.parse(object);

		grantGroupType = (org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup) 
				superAbstract.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.GrantGroup.class.getCanonicalName()).getContent();

		return repsonse;
	}

}
