package org.iso.mpeg.mxm.relTE.schemahandler.resource;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.relTE.schema.SuperRelAbstractLevel1;
import org.iso.mpeg.mxm.engine.securityTE.schema.xenc.EncryptedKeyType;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.DigitalResource;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedData;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey;

public class ProtectedResource extends Resource implements org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm1x.ProtectedResource
{
	private org.iso.mpeg.mxm.engine.relTE.schema.relm1x.ProtectedResource protectedResourceType;
	private SecurityEngine securityEngine;
	
	public ProtectedResource(SecurityEngine securityEngine)
	{
		this(new org.iso.mpeg.mxm.engine.relTE.schema.relm1x.ProtectedResource(), securityEngine);
	}
	
	public ProtectedResource(org.iso.mpeg.mxm.engine.relTE.schema.relm1x.ProtectedResource protectedResource, SecurityEngine securityEngine)
	{
		super(protectedResource);
		this.protectedResourceType = protectedResource;
		this.securityEngine = securityEngine;
	}
	
	@Override
	public DigitalResource getDigitalResource()
	{
		if (protectedResourceType.getDigitalResource()!=null)
		{
			return new org.iso.mpeg.mxm.relTE.schemahandler.resource.DigitalResource(protectedResourceType.getDigitalResource());
		}
		
		return null;
	}

	@Override
	public void setDigitalResource(DigitalResource digitalResource)
	{
		try
		{
			protectedResourceType.setDigitalResource((org.iso.mpeg.mxm.engine.relTE.schema.relr.DigitalResource)
					digitalResource.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relr.DigitalResource.class.getCanonicalName()).getContent());
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}
	
	@Override
	public void addEncryptedKey(EncryptedKey encryptedKey)
	{
		try
		{
			protectedResourceType.getEncryptedDataOrEncryptedKey().add((org.iso.mpeg.mxm.engine.securityTE.schema.xenc.EncryptedType)
					encryptedKey.createObject(EncryptedKeyType.class.getCanonicalName()).getContent());
		} 
		catch (MXMException e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public void addEncryptedData(EncryptedData encrypteddata)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public List<EncryptedData> getEncryptedDataList()
	{
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<EncryptedKey> getEncryptedKeyList()
	{
		List<EncryptedKey> encryptedKeyList = new ArrayList<EncryptedKey>();
		
		for (org.iso.mpeg.mxm.engine.securityTE.schema.xenc.EncryptedType encryptedType : protectedResourceType.getEncryptedDataOrEncryptedKey())
		{
			if (encryptedType instanceof EncryptedKeyType)
			{
				try
				{
					EncryptedKey encryptedKey = securityEngine.getSecurityEngineSchemaHandler().getEncryptedKey();
					encryptedKey.parse(new MXMAdapter(encryptedType));
					
					encryptedKeyList.add(encryptedKey);
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
				}
			}
		}
		
		return encryptedKeyList;
	}

	@Override
	public String getResourceLocator()
	{
		return protectedResourceType.getResourceLocator();
	}

	@Override
	public void setResourceLocator(String string)
	{
		protectedResourceType.setResourceLocator(string);
	}
	
	// MXM CREATOR AND PARSER METHODS

	@Override
	public void createFile(File f) throws MXMException
	{
		new SuperRelAbstractLevel1(protectedResourceType).createFile(f);
	}

	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		return new SuperRelAbstractLevel1(protectedResourceType).createObject(objectType);
	}

	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperRelAbstractLevel1 superAbstract = new SuperRelAbstractLevel1(protectedResourceType);

		MXMEngineResponse repsonse = superAbstract.parse(object);

		protectedResourceType = (org.iso.mpeg.mxm.engine.relTE.schema.relm1x.ProtectedResource) 
				superAbstract.createObject(org.iso.mpeg.mxm.engine.relTE.schema.relm1x.ProtectedResource.class.getCanonicalName()).getContent();

		return repsonse;
	}

}
