/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2007
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.relTE.technologyhandler;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBElement;

import org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.EvaluatedConditionResult;


public class ConditionResult 
{
	private EvaluatedConditionResult result = EvaluatedConditionResult.SUCCESSFULLY_MET;

	private List<EvaluatedCondition> conditions = null;

	public ConditionResult ()
	{
		conditions = new ArrayList<EvaluatedCondition>();
	}
	
	public EvaluatedConditionResult getResult()
	{
		return result;
	}

	public List<EvaluatedCondition> getEvaluatedConditions()
	{
		return conditions;
	}

	public void addEvaluatedCondition(EvaluatedCondition condition)
	{
		this.conditions.add(condition);

		EvaluatedConditionResult newConditionType = condition.getConditionType();
		
		recalculateResult(newConditionType);		
	}

	public void setEvaluatedConditions(List<EvaluatedCondition> conditions)
	{
		this.conditions = conditions;

		for (EvaluatedCondition condition : conditions)
		{
			recalculateResult(condition.getConditionType());
		}
	}

	public List<JAXBElement<? extends Condition>> getConditions() 
	{
		List<JAXBElement<? extends Condition>> relConditions = new ArrayList<JAXBElement<? extends Condition>>();
//		for (EvaluatedCondition condition : conditions)
//FIXME-other			
//			relConditions.add(condition.getCondition());
		return relConditions;
	}

	/**
	 * This method calculates whether a new added condition is "more far away from being met"
	 * than the previous "most far away from being met" condition result stored in the 
	 * <code>result</code> member variable.
	 * 
	 * @param newConditionType
	 */
	private void recalculateResult(EvaluatedConditionResult newConditionType) 
	{
		if (newConditionType.compareTo(result) < 0 )
			result = newConditionType;		
	}
	
}
