package org.iso.mpeg.relTE.test;

import java.io.File;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
import org.iso.mpeg.mxm.engine.relTE.schema.RELEngineContext;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm1x.ProtectedResource;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.DigitalResource;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.KeyHolder;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.NonSecureReference;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.PropertyPossessor;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.PropertyUri;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.AuthorisationManager;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.AuthorisationResult;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.RELRight;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SecurityEngineKeys;

public class LicenseAuthTest extends TestCase
{
	private JAXBContext jc = null;
	private Marshaller m = null;	

	private Logger logger = Logger.getLogger(LicenseAuthTest.class);

	private static final String LICENSE_CRE = "license_Creator.xml";
	private static final String LICENSE_PROD = "license_Producer.xml";
	private static final String LICENSE_AGG = "license_Aggregator.xml";

	private SecurityEngine securityEngine;
	private RELEngine relEngine;

	public LicenseAuthTest(String arg0)
	{
		super(arg0);
	}

	protected void setUp() throws Exception 
	{
		super.setUp();	
		jc = RELEngineContext.getInstance().getJAXBContext();
		m = jc.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		m.setProperty("com.sun.xml.bind.namespacePrefixMapper", SchemaNamespacePrefixMapperImpl.getInstance());

		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = null;
		try 
		{
			mxm = MXM.getInstance(f);
		}
		catch (MXMConfigurationException e)
		{
			e.printStackTrace();
			throw new MXMConfigurationException("Unable to getInstance");
		}
		
		mxm.listEnginesAndProperties();

		relEngine = null;
		try 
		{
			relEngine = (RELEngine)mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
		} 
		catch (MXMEngineNotFoundException e)
		{
			e.printStackTrace();
			throw new MXMEngineNotFoundException("Unable to getDefaultEngine");
		}

		//let's set up security engine
		securityEngine = (SecurityEngine)mxm.getDefaultEngine(MXMEngineName.SecurityTE.toString());
		
//		initialiseSecureRepository(securityEngine);
	}

	protected void tearDown() throws Exception
	{
		super.tearDown();
	}

	public void testAuthorisation() throws Exception
	{
		File licenseFile = new File(getClass().getClassLoader().getResource(LICENSE_CRE).getFile());
		MXMObject licenseObject = new MXMAdapter(licenseFile);
		License license = relEngine.getRELEngineSchemaHandler().getLicense();
		license.parse(licenseObject);

		logger.info("----- Producer -----");

		KeyHolder producer = getKeyHolder("RSA_modulus_Producer", "RSA_exponent_Producer", "Producer");
		KeyHolder creator = getKeyHolder("RSA_modulus_Creator", "RSA_exponent_Creator", "Creator");

		RELRight play = RELRight.play;
		RELRight govCopy = RELRight.governedCopy;
		RELRight issue = RELRight.issue;

		// let's build the resource 
		ProtectedResource protectedResource = relEngine.getRELEngineSchemaHandler().getProtectedResource();
		DigitalResource dr = relEngine.getRELEngineSchemaHandler().getDigitalResource();
		NonSecureReference nonSecureUrl = relEngine.getRELEngineSchemaHandler().getNonSecureReference();
		nonSecureUrl.setURI("ServerCEDEO:digitalResource");
		dr.setNonSecureIndirect(nonSecureUrl);
		protectedResource.setDigitalResource(dr);
		String idResource = "protectedResource";
		protectedResource.setLicensePartId(idResource);

		AuthorisationManager am = relEngine.getRELEngineTechnologyHandler().getAuthorisationManager();
		AuthorisationResult ar = am.authoriseUser(license, producer, play, protectedResource);

		logger.info("(Play) Authorisation result: " + ar);

		ar = am.authoriseUser(license, producer, govCopy, protectedResource);

		logger.info("(GovernedCopy) Authorisation result: " + ar);

//		PropertyPossessor pp = relEngine.getRELEngineSchemaHandler().getPropertyPossessor();
//		pp.setPropertyUri("Producer:Agg_Dom");
//		pp.addPrincipals(new MXMAdapter(producer));
//
//		// TODO na metatrapei se resource tipou grant
//		ar = am.authoriseIssue(license, producer, issue, new MXMAdapter(protectedResource), new MXMAdapter(pp));
//
//		logger.info("(Issue) Authorisation result: " + ar);

		licenseFile = new File(getClass().getClassLoader().getResource(LICENSE_PROD).getFile());
		license = relEngine.getRELEngineSchemaHandler().getLicense();
		license.parse(new MXMAdapter(licenseFile));

		logger.info("----- Aggregator -----");

		KeyHolder aggregator = getKeyHolder("RSA_modulus_Aggregator", "RSA_exponent_Aggregator", "Aggregator");

		ar = am.authoriseUser(license, aggregator, play, protectedResource);

		logger.info("(Play) Authorisation result: " + ar);

		ar = am.authoriseUser(license, aggregator, govCopy, protectedResource);

		logger.info("(GovernedCopy) Authorisation result: " + ar);

		PropertyPossessor pp = relEngine.getRELEngineSchemaHandler().getPropertyPossessor();
		PropertyUri property = relEngine.getRELEngineSchemaHandler().getPropertyUri();
		property.setDefinition("Aggregator:USERS");
		pp.setPropertyAbstract(property);
//		pp.addPrincipals(new MXMAdapter(aggregator));

//		// TODO na metatrapei se resource tipou grant
//		ar = am.authoriseUser(license, aggregator, issue, protectedResource, pp);
//
//		logger.info("(Issue) Authorisation result: " + ar);

		licenseFile = new File(getClass().getClassLoader().getResource(LICENSE_AGG).getFile());
		license = relEngine.getRELEngineSchemaHandler().getLicense();
		license.parse(new MXMAdapter(licenseFile));

		logger.info("----- End User -----");

		KeyHolder endUser = getKeyHolder("RSA_modulus_U1", "RSA_exponent_U1", "U1");

		// TODO auto mallon prepei na bgei false dld na tsekarei poios mou to leei kai na faei akiro
		ar = am.authoriseUser(license, endUser, play, protectedResource);

		logger.info("(Play) Authorisation result: " + ar);
	}

	private KeyHolder getKeyHolder(String modulus,String exponent,String licensePartId) throws MXMException
	{
		RSAKeyValue rsa = securityEngine.getSecurityEngineSchemaHandler().getRSAKeyValue();
		rsa.setModulus(modulus.getBytes());
		rsa.setExponent(exponent.getBytes());
		
		KeyValue kvc = securityEngine.getSecurityEngineSchemaHandler().getKeyValue();
		kvc.setRSAKeyValue(rsa);
		
		KeyInfo kic = securityEngine.getSecurityEngineSchemaHandler().getKeyInfo();
		kic.setKeyValue(kvc);

		KeyHolder kh = relEngine.getRELEngineSchemaHandler().getKeyHolder();
		kh.setKeyInfo(kic);
		
		if(licensePartId != null)
		{
			kh.setLicensePartId(licensePartId);
		}
		
		return kh;
	}

//	/**
//	 * This method is used to add some 
//	 * @param securityEngine
//	 * @throws SecureRepositoryManagerException
//	 */
//	private void initialiseSecureRepository(SecurityEngine securityEngine) 
//	{
//		String secureInfoStoragePath = securityEngine.getPropertyValue(SecurityEngineKeys.SECURE_INFO_STORAGE_PATH);
//		File secureRepoFile = new File(secureInfoStoragePath);
//		if (secureRepoFile.exists())
//			secureRepoFile.delete();
//
//		SecureRepositoryManager srm = securityEngine.getSecureRepositoryManager();
//
//		srm.addConfidentialInfo(SecurityEngineKeys.COUNTRY, new MXMAdapter("IT"), new MXMAdapter(secureInfoStoragePath));
//		srm.addConfidentialInfo(SecurityEngineKeys.COMMERCIAL_ENTITY, new MXMAdapter("false"), new MXMAdapter(secureInfoStoragePath));
//	}

}
