/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.relTE.test;

//usefull import
import java.io.File;
import java.io.FileNotFoundException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import junit.framework.TestCase;
import org.apache.log4j.Logger;

//mxm structures
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;


import org.iso.mpeg.mxm.engine.relTE.schema.RELEngineContext;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Issuer;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

public class RelEngineTest extends TestCase {

	private JAXBContext jc = null;
	private Marshaller m = null;	
	
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		jc = RELEngineContext.getInstance().getJAXBContext();
		m = jc.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);	
		m.setProperty("com.sun.xml.bind.namespacePrefixMapper", SchemaNamespacePrefixMapperImpl.getInstance());
	}

	private Logger logger = Logger.getLogger(RelEngineTest.class);
	
	private final String TEMP_FOLDER = System.getProperty("java.io.tmpdir") + File.separator;
	
	private final String OUTPUTREL_FILE_NAME = TEMP_FOLDER + "mytestLicense.xml";
	private final String SOURCEREL_FILE_NAME = "testLicense.xml";

		
	public void testRelParsing() throws MXMConfigurationException, FileNotFoundException, JAXBException, MXMEngineException {
		logger.info("\n\n---------- testRELParsing ----------");
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		RELEngine relEngine = (RELEngine) mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
		
		SecurityEngine secEngine = (SecurityEngine) mxm.getDefaultEngine(MXMEngineName.SecurityTE.toString());
		
		org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License li = relEngine.getRELEngineSchemaHandler().getLicense();
		
		MXMAdapter ad=new MXMAdapter(new File(getClass().getClassLoader().getResource(SOURCEREL_FILE_NAME).getFile()));
		
		MXMEngineResponse response = li.parse(ad);

		System.out.println("name is:"+li.getLicenseId());
		
		for (Issuer issuer : li.getIssuers()) 
		{
			if (issuer.getSignature()!=null)
			{
				System.out.println("sig_id is:"+issuer.getSignature().getId());
			}
		}
	}

	public void testRelCreation() throws MXMConfigurationException, MXMEngineException {
		logger.info("\n\n---------- testRelParsing ----------");
		
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		RELEngine relEngine = (RELEngine) mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
		
		SecurityEngine secEngine = (SecurityEngine) mxm.getDefaultEngine(MXMEngineName.SecurityTE.toString());
		
		org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature sig= secEngine.getSecurityEngineSchemaHandler().getSignature();

		sig.setId("signature_id_1");
				
		org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License li = relEngine.getRELEngineSchemaHandler().getLicense();
		org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Issuer is = relEngine.getRELEngineSchemaHandler().getIssuer();

		is.setSignature(sig);

		li.setLicenseId("1234");
		li.addIssuer(is);
	
		try {
			li.createFile(new File(OUTPUTREL_FILE_NAME));
		} catch (MXMException e) {
			e.printStackTrace();
		}
	}	
}