/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.requesteventPE.schema;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolRequest;
import org.iso.mpeg.mxm.baseprotocol.schema.ContentEntityType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.eventreportTE.schema.er.ERRType;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.requesteventPE.schema.requesteventprotocol.RequestEventRequestType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
public class RequestEventRequest extends ProtocolRequest implements org.iso.mpeg.mxm.pEngine.requesteventPE.schemahandler.RequestEventRequest
{
	private BaseProtocolEngine baseProtocolEngine;
	private EventReportEngine eventReportEngine;
	
	public RequestEventRequest(BaseProtocolEngine baseProtocolEngine, SecurityEngine securityEngine,
			EventReportEngine eventReportEngine)
	{
		super(new RequestEventRequestType(), securityEngine);
		this.baseProtocolEngine = baseProtocolEngine;
		this.eventReportEngine = eventReportEngine;
	}
	
	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((RequestEventRequestType)protocolBaseType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#createObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((RequestEventRequestType)protocolBaseType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((RequestEventRequestType)protocolBaseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolBaseType = 
					(RequestEventRequestType)(level1.createObject(RequestEventRequestType.class.getName()).getContent());
		}
		return response;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.requesteventPE.schemahandler.RequestEventRequest#getERR()
	 */
	@Override
	public ERR getERR() throws MXMEngineException
	{
		ERRType errType = ((RequestEventRequestType)protocolBaseType).getERR();
		ERR err = eventReportEngine.getEventReportEngineSchemaHandler().getERR();
		err.parse(new MXMAdapter(errType));
		return err;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.requesteventPE.schemahandler.RequestEventRequest#setERR(org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR)
	 */
	@Override
	public void setERR(ERR eRR) throws MXMException
	{
		ERRType errType = (ERRType)(eRR.createObject(ERRType.class.getName()).getContent());
		((RequestEventRequestType)protocolBaseType).setERR(errType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.requesteventPE.schemahandler.RequestEventRequest#getContentEntity()
	 */
	@Override
	public ContentEntity getContentEntity() throws MXMEngineException
	{
		ContentEntityType contentEntityType = ((RequestEventRequestType)protocolBaseType).getContentEntity();
		ContentEntity contentEntity = baseProtocolEngine.getBaseProtocolSchemaHandler().getContentEntity();
		contentEntity.parse(new MXMAdapter(contentEntityType));
		return contentEntity;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.requesteventPE.schemahandler.RequestEventRequest#setContentEntity(org.iso.mpeg.mxm.pEngine.basePE.schemahandler.baseprotocol.ContentEntity)
	 */
	@Override
	public void setContentEntity(ContentEntity contentEntity) throws MXMException
	{
		ContentEntityType contentEntityType = (ContentEntityType)
													(contentEntity.createObject(ContentEntityType.class.getName()).getContent());
		((RequestEventRequestType)protocolBaseType).setContentEntity(contentEntityType);
	}

}
