package org.iso.mpeg.mxm.securityTE;


import java.util.HashMap;

import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;
import org.iso.mpeg.mxm.securityTE.technologyhandler.AppUserDataBaseManager;
import org.iso.mpeg.mxm.securityTE.technologyhandler.CryptoModuleManagerInterface;
import org.iso.mpeg.mxm.securityTE.technologyhandler.IPUserDataBaseManager;
import org.iso.mpeg.mxm.securityTE.technologyhandler.OnCardCryptoModuleManager;
import org.iso.mpeg.mxm.securityTE.technologyhandler.SPUserDataBaseManager;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.AuthenticationManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.FileAndObjectHandler;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.KeyManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SecureDeviceManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SecurityEngineKeys;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.UserDataBaseManager;


public class SecurityEngineTechnologyHandler implements
    org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineTechnologyHandler
{
    private AuthenticationManager             authenticationManager       = null;
    private String                            cardProtocol                = null;
    private String                            cardTerminalName            = null;
    private CertificateManager                certificateManager          = null;
    private CryptoModuleManagerInterface      cryptoModuleManager         = null;
    private String                            dataBaseName                = null;
    private String                            engineFocus                 = null;
    private FileAndObjectHandler              fileAndObjectHandler        = null;
    private KeyManager                        keyManager                  = null;
    private String                            mySqlPassword               = null;
    private String                            mySqlUrl                    = null;
    private String                            mySqlUsername               = null;
    private org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler securityEngineSchemaHandler = null;
    private SignatureManager                  signatureManager            = null;
    private UserDataBaseManager               userDataBaseManager         = null;
    private boolean                           useSmartCard                = false;

    @Override
    public AuthenticationManager getAuthenticationManager()
        throws SecurityEngineException
    {
        if (this.authenticationManager == null)
        {
            return this.authenticationManager = new org.iso.mpeg.mxm.securityTE.technologyhandler.AuthenticationManager(
                this.cryptoModuleManager);
        }
        else
        {
            return this.authenticationManager;
        }
    }

    @Override
    public CertificateManager getCertificateManager() throws SecurityEngineException
    {
        if (this.certificateManager == null)
        {
            return this.certificateManager = new org.iso.mpeg.mxm.securityTE.technologyhandler.CertificateManager(
                this.cryptoModuleManager);
        }
        else
        {
            return this.certificateManager;
        }
    }

    @Override
    public FileAndObjectHandler getFileAndObjectHandler() throws SecurityEngineException
    {
        if (this.fileAndObjectHandler == null)
        {
            this.fileAndObjectHandler = new org.iso.mpeg.mxm.securityTE.technologyhandler.FileAndObjectHandler();
        }
        return this.fileAndObjectHandler;
    }

    @Override
    public KeyManager getKeyManager() throws SecurityEngineException
    {
        if (this.keyManager == null)
        {
            return this.keyManager = new org.iso.mpeg.mxm.securityTE.technologyhandler.KeyManager(
                this.cryptoModuleManager);
        }
        else
        {
            return this.keyManager;
        }
    }

    @Override
    public SecureDeviceManager getSecureDeviceManager() throws SecurityEngineException
    {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public SignatureManager getSignatureManager() throws SecurityEngineException
    {
        if (this.signatureManager == null)
        {
            return this.signatureManager = new org.iso.mpeg.mxm.securityTE.technologyhandler.SignatureManager(
                this.cryptoModuleManager, this.securityEngineSchemaHandler);
        }
        else
        {
            return this.signatureManager;
        }
    }

    @Override
    public UserDataBaseManager getUserDataBaseManager() throws SecurityEngineException
    {
        if (this.userDataBaseManager == null)
        {
            if (this.engineFocus.equalsIgnoreCase("Application"))
            {
                this.userDataBaseManager = new AppUserDataBaseManager(
                    this.cryptoModuleManager, this.mySqlUrl, this.mySqlUsername,
                    this.mySqlPassword, this.dataBaseName);
            }
            else if (this.engineFocus.equalsIgnoreCase("ServiceProvider"))
            {
                this.userDataBaseManager = new SPUserDataBaseManager(
                    this.cryptoModuleManager, this.mySqlUrl, this.mySqlUsername,
                    this.mySqlPassword, this.dataBaseName);
            }
            else if (this.engineFocus.equalsIgnoreCase("IdentityProvider"))
            {
                this.userDataBaseManager = new IPUserDataBaseManager(
                    this.cryptoModuleManager, this.mySqlUrl, this.mySqlUsername,
                    this.mySqlPassword, this.dataBaseName,this.securityEngineSchemaHandler,getAuthenticationManager(), getSignatureManager());
            }
        }
        return this.userDataBaseManager;
    }

    public void initialize(final MXMGenericParameters mxmParameters,
        final CryptoModuleManagerInterface cryptoModuleManager, org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler securityEngineSchemaHandler)
        throws SecurityEngineException
    {
      
        this.securityEngineSchemaHandler = securityEngineSchemaHandler;
        this.cryptoModuleManager = cryptoModuleManager;
        this.useSmartCard = Boolean.parseBoolean(mxmParameters
            .getProperty("enableSmartCard"));
        this.cardProtocol = mxmParameters.getProperty("cardProtocol");
        this.cardTerminalName = mxmParameters.getProperty("cardTerminalName");
        this.engineFocus = mxmParameters.getProperty("engineFocus");
        this.mySqlUrl = mxmParameters.getProperty("mySqlUrl");
        this.mySqlUsername = mxmParameters.getProperty("mySqlUsername");
        this.mySqlPassword = mxmParameters.getProperty("mySqlPassword");
        this.dataBaseName = mxmParameters.getProperty("dataBaseName");
        
        if (this.cryptoModuleManager == null)
        {
            if (this.useSmartCard)
            {
                this.cryptoModuleManager = new OnCardCryptoModuleManager(
                    this.cardProtocol, this.cardTerminalName,mxmParameters );
            }
            else
            {
                this.cryptoModuleManager = new org.iso.mpeg.mxm.securityTE.technologyhandler.OffCardCryptoModuleManager(mxmParameters);
            }
        }
    }
}
