package org.iso.mpeg.mxm.securityTE;


import java.security.Security;
import java.util.ArrayList;
import java.util.List;

import org.iso.mpeg.mxm.core.MXMAbstractEngine;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.securityTE.technologyhandler.CryptoModuleManagerInterface;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineTechnologyHandler;


public class SecurityTE extends MXMAbstractEngine implements
    org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine
{
    private String                             cardProtocol        = null;
    private String                             cardTerminalName    = null;
    private final CryptoModuleManagerInterface cryptoModuleManager = null;
    private String                             dataBaseName        = null;
    private String                             engineFocus         = null;
    private List<MXMEngine>                    engines;
    private String                             mySqlPassword       = null;
    private String                             mySqlUrl            = null;
    private String                             mySqlUsername       = null;
    private boolean                            useSmartCard        = false;

    public SecurityTE()
    {
        this.engines = new ArrayList<MXMEngine>();
        Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
    }

    @Override
    public String getEngineName()
    {
        return MXMEngineName.SecurityTE.toString();
    }

    @Override
    public SecurityEngineSchemaHandler getSecurityEngineSchemaHandler()
        throws MXMEngineException
    {
        return new org.iso.mpeg.mxm.securityTE.SecurityEngineSchemaHandler(this.engines);
    }

    @Override
    public SecurityEngineTechnologyHandler getSecurityEngineTechnologyHandler()
        throws MXMEngineException
    {
        final SecurityEngineTechnologyHandler handler = new org.iso.mpeg.mxm.securityTE.SecurityEngineTechnologyHandler();
        ((org.iso.mpeg.mxm.securityTE.SecurityEngineTechnologyHandler)handler)
            .initialize(this.mxmParameters, this.cryptoModuleManager,this.getSecurityEngineSchemaHandler());
        return handler;
    }
    
    
    private void initialise()
    {
        this.useSmartCard = Boolean
            .parseBoolean(this.getPropertyValue("enableSmartCard"));
        this.cardProtocol = this.getPropertyValue("cardProtocol");
        this.cardTerminalName = this.getPropertyValue("cardTerminalName");
        this.engineFocus = this.getPropertyValue("engineFocus");
        this.mySqlUrl = this.getPropertyValue("mySqlUrl");
        this.mySqlUsername = this.getPropertyValue("mySqlUsername");
        this.mySqlPassword = this.getPropertyValue("mySqlPassword");
        this.dataBaseName = this.getPropertyValue("dataBaseName");
        if (this.useSmartCard)
        {
            if (!(this.cardProtocol.equals("0") || this.cardProtocol.equals("1")))
            {
                throw new RuntimeException(
                    "Card Protocol is not properly configured. Either set it to '0' or '1'");
            }
            if ((this.cardTerminalName == null) || this.cardTerminalName.equals(""))
            {
                throw new RuntimeException(
                    "The name of the smart card terminal is not properly configured");
            }
        }
        if (!(this.engineFocus.equals("Application")
            || this.engineFocus.equals("ServiceProvider") || this.engineFocus
                .equals("IdentityProvider")))
        {
            throw new RuntimeException(
                "The focus of the security engine is not properly set. Set it either to 'Application', 'ServiceProvider' or 'IdentityProvider'");
        }
        if ((this.mySqlUrl == null) || this.mySqlUrl.equals(""))
        {
            throw new RuntimeException("The url of the MySql server is not properly set");
        }
        if ((this.mySqlUsername == null) || this.mySqlUsername.equals(""))
        {
            throw new RuntimeException(
                "The username for the MySql server is not properly set");
        }
        if (this.mySqlPassword == null)
        {
            this.mySqlPassword = "";
        }
        if ((this.dataBaseName == null) || this.dataBaseName.equals(""))
        {
            throw new RuntimeException(
                "The name for the MySql database is not properly set");
        }
    }

    @Override
    public void initialise(final String engineID,
        final MXMGenericParameters engineParameters, final boolean isTheDefaultEngine)
    {
        super.initialise(engineID, engineParameters, isTheDefaultEngine);
        this.initialise();
    }

    @Override
    public void setDependencies(final List<MXMEngine> engines) throws MXMEngineException
    {
        this.engines = engines;
    }
}
