package org.iso.mpeg.mxm.securityTE.schemahandler;


import java.io.File;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.securityTE.schema.xenc.EncryptedDataType;
import org.iso.mpeg.mxm.engine.securityTE.schema.xenc.SuperXencAbstractLevel1;


public class EncryptedData extends Encrypted implements
    org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedData
{
    private EncryptedDataType encryptedDataType;

    public EncryptedData()
    {
        this(new EncryptedDataType());
    }

    public EncryptedData(final EncryptedDataType encrypted)
    {
        super(encrypted);
        this.encryptedDataType = encrypted;
    }

    // MXM CREATOR AND PARSER METHODS
    @Override
    public void createFile(final File f) throws MXMException
    {
        new SuperXencAbstractLevel1(this.encryptedDataType).createFile(f);
    }

    @Override
    public MXMObject createObject(final String objectType) throws MXMException
    {
        return new SuperXencAbstractLevel1(this.encryptedDataType)
            .createObject(objectType);
    }

    public byte[] getEncryptedData()
    {
        if (this.encryptedDataType.getCipherData() != null)
        {
            return this.encryptedDataType.getCipherData().getCipherValue();
        }
        else
        {
            return null;
        }
    }

    @Override
    public MXMEngineResponse parse(final MXMObject object) throws MXMEngineException
    {
        final SuperXencAbstractLevel1 level1 = new SuperXencAbstractLevel1(
            this.encryptedDataType);
        final MXMEngineResponse response = level1.parse(object);
        this.encryptedDataType = (EncryptedDataType)level1.createObject(
            EncryptedDataType.class.getCanonicalName()).getContent();
        return response;
    }
}
