package org.iso.mpeg.mxm.securityTE.schemahandler;


import java.io.File;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.RSAKeyValueType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SuperDsigAbstractLevel1;


public class RSAKeyValue implements
    org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue
{
    private RSAKeyValueType rsaKeyValueType;

    public RSAKeyValue()
    {
        this(new RSAKeyValueType());
    }

    public RSAKeyValue(final byte[] modulus, final byte[] exponent)
    {
        this.rsaKeyValueType = new RSAKeyValueType();
        this.setModulus(modulus);
        this.setExponent(exponent);
    }

    public RSAKeyValue(final RSAKeyValueType rsaKeyValue)
    {
        this.rsaKeyValueType = rsaKeyValue;
    }

    @Override
    public void createFile(final File f) throws MXMException
    {
        new SuperDsigAbstractLevel1(this.rsaKeyValueType).createFile(f);
    }

    @Override
    public MXMObject createObject(final String objectType) throws MXMException
    {
        return new SuperDsigAbstractLevel1(this.rsaKeyValueType).createObject(objectType);
    }

    @Override
    public byte[] getExponent()
    {
        return this.rsaKeyValueType.getExponent();
    }

    @Override
    public byte[] getModulus()
    {
        return this.rsaKeyValueType.getModulus();
    }

    // MXM CREATOR AND PARSER METHODS
    @Override
    public MXMEngineResponse parse(final MXMObject object) throws MXMEngineException
    {
        final SuperDsigAbstractLevel1 level1 = new SuperDsigAbstractLevel1(
            this.rsaKeyValueType);
        final MXMEngineResponse response = level1.parse(object);
        this.rsaKeyValueType = (RSAKeyValueType)level1.createObject(
            RSAKeyValueType.class.getCanonicalName()).getContent();
        return response;
    }

    @Override
    public void setExponent(final byte[] b)
    {
        this.rsaKeyValueType.setExponent(b);
    }

    @Override
    public void setModulus(final byte[] b)
    {
        this.rsaKeyValueType.setModulus(b);
    }
}
