package org.iso.mpeg.mxm.securityTE.schemahandler.dsig;


import java.io.File;
import java.util.List;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.KeyInfoType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureValueType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignedInfoType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SuperDsigAbstractLevel1;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Object;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignatureValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignedInfo;


public class Signature implements
    org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature
{
    private final org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler securityEngineSchemaHandler;
    private SignatureType                                                         signatureType;

    public Signature(
        final org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler securityEngineSchemaHandler)
    {
        this.signatureType = new SignatureType();
        this.securityEngineSchemaHandler = securityEngineSchemaHandler;
    }

    @Override
    public void createFile(final File f) throws MXMException
    {
        new SuperDsigAbstractLevel1(this.signatureType).createFile(f);
    }

    @Override
    public MXMObject createObject(final String objectType) throws MXMException
    {
        return new SuperDsigAbstractLevel1(this.signatureType).createObject(objectType);
    }

    @Override
    public String getId()
    {
        return this.signatureType.getId();
    }

    @Override
    public KeyInfo getKeyInfo() throws MXMEngineException
    {
        if (this.signatureType.getKeyInfo() != null)
        {
            return new org.iso.mpeg.mxm.securityTE.schemahandler.dsig.KeyInfo(
                this.signatureType.getKeyInfo());
        }
        return null;
    }

    @Override
    public List<Object> getObject()
    {
        // TODO: implement later
        return null;
    }

    @Override
    public SignatureValue getSignatureValue() throws MXMEngineException
    {
        return new org.iso.mpeg.mxm.securityTE.schemahandler.SignatureValue(
            this.signatureType.getSignatureValue());
    }

    @Override
    public SignedInfo getSignedInfo() throws MXMEngineException
    {
        new org.iso.mpeg.mxm.securityTE.schemahandler.SignatureValue(
            this.signatureType.getSignatureValue());
        final SignedInfo keyInfo = this.securityEngineSchemaHandler.getSignedInfo();
        keyInfo.parse(new MXMAdapter(this.signatureType.getSignedInfo()));
        return keyInfo;
    }

    @Override
    public MXMEngineResponse parse(final MXMObject object) throws MXMEngineException
    {
        final SuperDsigAbstractLevel1 level1 = new SuperDsigAbstractLevel1(
            this.signatureType);
        final MXMEngineResponse response = level1.parse(object);
        this.signatureType = (SignatureType)level1.createObject(
            SignatureType.class.getCanonicalName()).getContent();
        return response;
    }

    @Override
    public void setId(final String string)
    {
        this.signatureType.setId(string);
    }

    @Override
    public void setKeyInfo(final KeyInfo keyInfo) throws MXMException
    {
        KeyInfoType keyInfoType = null;
        keyInfoType = (KeyInfoType)(keyInfo.createObject(KeyInfoType.class.getName()))
            .getContent();
        this.signatureType.setKeyInfo(keyInfoType);
    }

    @Override
    public void setSignatureValue(final String id, final byte[] value)
    {
        try
        {
            final SignatureValue signatureValue = new org.iso.mpeg.mxm.securityTE.schemahandler.SignatureValue();
            signatureValue.setId(id);
            signatureValue.setValue(value);
            this.signatureType.setSignatureValue((SignatureValueType)signatureValue
                .createObject(SignatureValueType.class.getCanonicalName()).getContent());
        }
        catch (final MXMException e)
        {
            e.printStackTrace();
        }
    }

    @Override
    public void setSignedInfo(final SignedInfo signedInfo) throws MXMException
    {
        SignedInfoType signedInfoType = null;
        signedInfoType = (SignedInfoType)(signedInfo.createObject(SignedInfoType.class
            .getName())).getContent();
        this.signatureType.setSignedInfo(signedInfoType);
    }
}
