package org.iso.mpeg.mxm.securityTE.schemahandler.saml;


import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.xml.datatype.XMLGregorianCalendar;

import org.iso.mpeg.mxm.baseprotocol.schema.SuperBaseProtocolAbstractLevel1;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AdviceType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AssertionType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AttributeStatementType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AuthnStatementType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AuthzDecisionStatementType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.ConditionsType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.NameIDType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.ObjectFactory;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.StatementAbstractType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.SubjectType;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Advice;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AttributeStatement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnStatement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthzDecisionStatement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Conditions;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.NameID;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.StatementAbstract;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Subject;


public class Assertion implements
    org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Assertion
{
    private final ObjectFactory                                                   assertionObjectFactory;
    private AssertionType                                                         assertionType;
    private final org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler securityEngineSchemaHandler;

    public Assertion(
        final org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler securityEngineSchemaHandler)
    {
        this.assertionType = new AssertionType();
        this.securityEngineSchemaHandler = securityEngineSchemaHandler;
        this.assertionObjectFactory = new ObjectFactory();
    }

    @Override
    public void addAttributeStatement(final AttributeStatement authnStatement)
        throws MXMException
    {
        final StatementAbstractType statement = (AttributeStatementType)authnStatement
            .createObject(AttributeStatementType.class.getCanonicalName()).getContent();
        this.assertionType.getStatementOrAuthnStatementOrAuthzDecisionStatement().add(
            statement);
    }

    @Override
    public void addAuthnStatement(final AuthnStatement authnStatement)
        throws MXMException
    {
        final AuthnStatementType statement = (AuthnStatementType)authnStatement
            .createObject(AuthnStatementType.class.getCanonicalName()).getContent();
        this.assertionType.getStatementOrAuthnStatementOrAuthzDecisionStatement().add(
            statement);
    }

    @Override
    public void addAuthzDecisionStatement(final AuthzDecisionStatement authnStatement)
        throws MXMException
    {
        final AuthzDecisionStatementType statementType = (AuthzDecisionStatementType)authnStatement
            .createObject(AuthzDecisionStatementType.class.getCanonicalName())
            .getContent();
        this.assertionType.getStatementOrAuthnStatementOrAuthzDecisionStatement().add(
            statementType);
    }

    // MXM CREATOR AND PARSER
    @Override
    public void createFile(final File f) throws MXMException
    {
        new SuperBaseProtocolAbstractLevel1(this.assertionType).createFile(f);
    }

    @Override
    public MXMObject createObject(final String objectType) throws MXMException
    {
        return new SuperBaseProtocolAbstractLevel1(this.assertionType)
        .createObject(objectType);
      //  SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((AssertionType)assertionType);
       // return level1.createObject(objectType);
    }

    @Override
    public Advice getAdvice() throws MXMEngineException
    {
        final Advice advice = this.securityEngineSchemaHandler.getAdvice();
        advice.parse(new MXMAdapter(this.assertionType.getAdvice()));
        return advice;
    }

    @Override
    public List<AttributeStatement> getAttributeStatements()
    {
        for (final StatementAbstractType element : this.assertionType
            .getStatementOrAuthnStatementOrAuthzDecisionStatement())
        {
            if (element.getClass().getName().equals("AttributeStatementType"))
            {
                // return new Attributesta;
            }
        }
        return null;
    }

    @Override
    public List<AuthnStatement> getAuthnStatements() throws MXMEngineException
    {
        final List<AuthnStatement> statements = new ArrayList<AuthnStatement>();
        for (final StatementAbstractType element : this.assertionType
            .getStatementOrAuthnStatementOrAuthzDecisionStatement())
        {
            if (element.getClass().getName().equals("AuthnStatementType"))
            {
                // return new Attributesta;
                final AuthnStatement statement = this.securityEngineSchemaHandler
                    .getAuthnStatement();
                statement.parse(new MXMAdapter(element));
                statements.add(statement);
            }
        }
        return statements;
    }

    @Override
    public List<AuthzDecisionStatement> getAuthzDecisionStatements()
        throws MXMEngineException
    {
        final List<AuthzDecisionStatement> statements = new ArrayList<AuthzDecisionStatement>();
        for (final StatementAbstractType element : this.assertionType
            .getStatementOrAuthnStatementOrAuthzDecisionStatement())
        {
            if (element.getClass().getName().equals("AuthzDecisionStatementType"))
            {
                // return new Attributesta;
                final AuthzDecisionStatement statement = this.securityEngineSchemaHandler
                    .getAuthzDecisionStatement();
                statement.parse(new MXMAdapter(element));
                statements.add(statement);
            }
        }
        return statements;
    }

    @Override
    public Conditions getConditions() throws MXMEngineException
    {
        final Conditions conditions = this.securityEngineSchemaHandler.getConditions();
        conditions.parse(new MXMAdapter(this.assertionType.getConditions()));
        return conditions;
    }

    @Override
    public String getID()
    {
        return this.assertionType.getID();
    }

    @Override
    public XMLGregorianCalendar getIssueInstant()
    {
        return this.assertionType.getIssueInstant();
    }

    @Override
    public NameID getIssuer() throws MXMEngineException
    {
        final NameID nameId = this.securityEngineSchemaHandler.getNameID();
        nameId.parse(new MXMAdapter(this.assertionType.getIssuer()));
        return nameId;
    }

    @Override
    public Signature getSignature() throws MXMEngineException
    {
        final Signature signature = this.securityEngineSchemaHandler.getSignature();
        signature.parse(new MXMAdapter(this.assertionType.getSignature()));
        return signature;
    }

    @Override
    public List<StatementAbstract> getStatements() throws MXMEngineException
    {
        final List<StatementAbstract> statements = new ArrayList<StatementAbstract>();
        for (final StatementAbstractType element : this.assertionType
            .getStatementOrAuthnStatementOrAuthzDecisionStatement())
        {
            if (element.getClass().getName().equals("StatementAbstractType"))
            {
                // return new Attributesta;
                final AuthzDecisionStatement statement = this.securityEngineSchemaHandler
                    .getAuthzDecisionStatement();
                statement.parse(new MXMAdapter(element));
                statements.add(statement);
            }
        }
        return statements;
    }

    @Override
    public Subject getSubject() throws MXMEngineException
    {
        final Subject subject = this.securityEngineSchemaHandler.getSubject();
        subject.parse(new MXMAdapter(this.assertionType.getSubject()));
        return subject;
    }

    @Override
    public String getVersion()
    {
        return this.assertionType.getVersion();
    }

    @Override
    public MXMEngineResponse parse(final MXMObject object) throws MXMEngineException
    {
        final SuperBaseProtocolAbstractLevel1 superAbstract = new SuperBaseProtocolAbstractLevel1(
            this.assertionType);
        final MXMEngineResponse repsonse = superAbstract.parse(object);
        this.assertionType = (AssertionType)superAbstract.createObject(
            AssertionType.class.getCanonicalName()).getContent();
        return repsonse;
    }

    @Override
    public void setAdvice(final Advice advice) throws MXMException
    {
        final AdviceType adviceType = (AdviceType)advice.createObject(
            AdviceType.class.getCanonicalName()).getContent();
        this.assertionType.setAdvice(adviceType);
    }

    @Override
    public void setConditions(final Conditions conditions) throws MXMException
    {
        final ConditionsType conditionsType = (ConditionsType)conditions.createObject(
            ConditionsType.class.getCanonicalName()).getContent();
        this.assertionType.setConditions(conditionsType);
    }

    @Override
    public void setID(final String string)
    {
        this.assertionType.setID(string);
    }

    @Override
    public void setIssueInstant(final XMLGregorianCalendar xMLGregorianCalendar)
    {
        this.assertionType.setIssueInstant(xMLGregorianCalendar);
    }

    @Override
    public void setIssuer(final NameID nameID) throws MXMException
    {
        final NameIDType nameIdType = (NameIDType)nameID.createObject(
            NameIDType.class.getCanonicalName()).getContent();
        this.assertionType.setIssuer(nameIdType);
    }

    @Override
    public void setSignature(final Signature signature) throws MXMException
    {
        final SignatureType sigType = (SignatureType)signature.createObject(
            SignatureType.class.getCanonicalName()).getContent();
        this.assertionType.setSignature(sigType);
    }

    @Override
    public void setSubject(final Subject subject) throws MXMException
    {
        SubjectType subjectType = null;
        subjectType = (SubjectType)(subject.createObject(SubjectType.class.getName()))
            .getContent();
        this.assertionType.setSubject(subjectType);
    }

    @Override
    public void setVersion(final String string)
    {
        this.assertionType.setVersion(string);
    }
}
