package org.iso.mpeg.mxm.securityTE.schemahandler.saml;


import java.io.File;

import javax.xml.datatype.XMLGregorianCalendar;

import org.iso.mpeg.mxm.baseprotocol.schema.SuperBaseProtocolAbstractLevel1;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AuthnContextType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.AuthnStatementType;
import org.iso.mpeg.mxm.engine.securityTE.schema.saml.assertion.SubjectLocalityType;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnContext;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.SubjectLocality;


public class AuthnStatement implements
    org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnStatement
{
    private AuthnStatementType   authStatementType;
    private final SecurityEngine securityEngine;

    public AuthnStatement(final SecurityEngine securityEngine)
    {
        this.authStatementType = new AuthnStatementType();
        this.securityEngine = securityEngine;
    }

    @Override
    public void createFile(final File f) throws MXMException
    {
        new SuperBaseProtocolAbstractLevel1(this.authStatementType).createFile(f);
    }

    @Override
    public MXMObject createObject(final String objectType) throws MXMException
    {
        return new SuperBaseProtocolAbstractLevel1(this.authStatementType)
            .createObject(objectType);
    }

    @Override
    public AuthnContext getAuthnContext() throws MXMEngineException
    {
        final AuthnContext authContex = this.securityEngine
            .getSecurityEngineSchemaHandler().getAuthnContext();
        authContex.parse(new MXMAdapter(this.authStatementType.getAuthnContext()));
        return authContex;
    }

    @Override
    public XMLGregorianCalendar getAuthnInstant()
    {
        return this.authStatementType.getAuthnInstant();
    }

    @Override
    public String getSessionIndex()
    {
        return this.authStatementType.getSessionIndex();
    }

    @Override
    public XMLGregorianCalendar getSessionNotOnOrAfter()
    {
        return this.authStatementType.getSessionNotOnOrAfter();
    }

    @Override
    public SubjectLocality getSubjectLocality() throws MXMEngineException
    {
        final SubjectLocality subject = this.securityEngine
            .getSecurityEngineSchemaHandler().getSubjectLocality();
        subject.parse(new MXMAdapter(this.authStatementType.getSubjectLocality()));
        return subject;
    }

    @Override
    public MXMEngineResponse parse(final MXMObject object) throws MXMEngineException
    {
        final SuperBaseProtocolAbstractLevel1 superAbstract = new SuperBaseProtocolAbstractLevel1(
            this.authStatementType);
        final MXMEngineResponse repsonse = superAbstract.parse(object);
        this.authStatementType = (AuthnStatementType)superAbstract.createObject(
            AuthnStatementType.class.getCanonicalName()).getContent();
        return repsonse;
    }

    @Override
    public void setAuthnContext(final AuthnContext authnContext) throws MXMException
    {
        AuthnContextType authContexType = null;
        authContexType = (AuthnContextType)(authnContext
            .createObject(AuthnContextType.class.getName())).getContent();
        this.authStatementType.setAuthnContext(authContexType);
    }

    @Override
    public void setAuthnInstant(final XMLGregorianCalendar xMLGregorianCalendar)
    {
        this.authStatementType.setAuthnInstant(xMLGregorianCalendar);
    }

    @Override
    public void setSessionIndex(final String string)
    {
        this.authStatementType.setSessionIndex(string);
    }

    @Override
    public void setSessionNotOnOrAfter(final XMLGregorianCalendar xMLGregorianCalendar)
    {
        this.authStatementType.setSessionNotOnOrAfter(xMLGregorianCalendar);
    }

    @Override
    public void setSubjectLocality(final SubjectLocality subjectLocality)
        throws MXMException
    {
        SubjectLocalityType subjectLocType = null;
        subjectLocType = (SubjectLocalityType)(subjectLocality
            .createObject(SubjectLocalityType.class.getName())).getContent();
        this.authStatementType.setSubjectLocality(subjectLocType);
    }
}
