package org.iso.mpeg.mxm.securityTE.technologyhandler;


import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.interfaces.RSAPublicKey;
import java.util.Random;

import javax.security.cert.CertificateException;
import javax.security.cert.X509Certificate;

import morpho.util.Converter;

import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;


/**
 * This class provided methods for client authentication needed in challange response fashion
 * 
 * @author Andreas Kohlos
 */
public class AuthenticationManager implements
    org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.AuthenticationManager
{
    CryptoModuleManagerInterface cryptoModuleManager;

    public AuthenticationManager(final CryptoModuleManagerInterface cryptoModuleManager)
    {
        this.cryptoModuleManager = cryptoModuleManager;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.AuthenticationManager#clientAuth(java
     * .lang.String , java.lang.String, java.lang.String)
     */
    @Override
    public String clientAuth(final String serverChallenge, final String appAlias,
        final String PIN) throws SecurityEngineException
    {
        // throw new SecurityEngineException("METHOD_NOT_IMPLEMENTED");
        byte[] result;
        byte[] resultHash = null;
        MessageDigest md;
        try
        {
            md = MessageDigest.getInstance("SHA-256", "BC");
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        md.update(Converter.toByteArray(serverChallenge));
        resultHash = md.digest();
        result = this.cryptoModuleManager.clientAuth(resultHash, appAlias);
        return Converter.toHex(result);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.AuthenticationManager#clientTokenVerify
     * (java .lang.String, java.lang.String)
     */
    @Override
    public boolean clientTokenVerify(final String challenge, final String cardAuthToken,
        final String algorithm, final PublicKey pubKey) throws SecurityEngineException
    {
        // throw new SecurityEngineException("METHOD_NOT_IMPLEMENTED");
        boolean result = this.cryptoModuleManager.clientTokenVerify(challenge, cardAuthToken, algorithm, pubKey);;
        return result;
    }

    
    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.AuthenticationManager#getCardChallenge ()
     */
    @Override
    public String getCardChallenge() throws SecurityEngineException
    {
        throw new SecurityEngineException("METHOD_NOT_IMPLEMENTED");
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.AuthenticationManager#getCertificate
     * (java.lang .String)
     */
    @Override
    public String getCertificate(final String appAlias, final String PIN)
        throws SecurityEngineException
    {
        PublicKey result = null;
        result = this.cryptoModuleManager.getPublicKey("auth", appAlias);
        // final RSAPublicKeySpec pubKeySpec = new RSAPublicKeySpec(new BigInteger(
        // modulus, 16), new BigInteger("10001", 16));
        // KeyFactory keyFactory;
        // keyFactory = KeyFactory.getInstance("RSA", "BC");
        // final RSAPublicKey rsaPublicKey = (RSAPublicKey)keyFactory
        // .generatePublic(pubKeySpec);
        // result = Converter.toHex(rsaPublicKey.getEncoded());
        return result.toString();
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.AuthenticationManager#getChallenge()
     */
    @Override
    public String getChallenge() throws SecurityEngineException
    {
        // throw new SecurityEngineException("METHOD_NOT_IMPLEMENTED");
        final Random random = new Random();
        final StringBuilder sb = new StringBuilder();
        while (sb.length() < 65)
        {
            sb.append(Integer.toHexString(random.nextInt()));
        }
        return sb.toString().substring(0, 64);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.AuthenticationManager#getCertificate
     * (java.lang .String)
     */
    @Override
    public RSAPublicKey getPublicKey(final String appAlias, final String PIN)
        throws SecurityEngineException
    {
        PublicKey publicKey = null;
        publicKey = this.cryptoModuleManager.getPublicKey("auth", appAlias);
        return (RSAPublicKey)publicKey;
    }
    
    @Override
    public RSAPublicKey getPublicKey(final String appAlias,String keyIdentifier,final String PIN)
        throws SecurityEngineException
    {
        PublicKey publicKey = null;
        publicKey = this.cryptoModuleManager.getPublicKey(keyIdentifier, appAlias);
        return (RSAPublicKey)publicKey;
    }
    
    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.AuthenticationManager#serverAuth(java
     * .lang.String , java.lang.String, java.lang.String)
     */
    @Override
    public String serverAuth(final String cardChallenge, final String keyAlias,
        final String password) throws SecurityEngineException
    {
        throw new SecurityEngineException("METHOD_NOT_IMPLEMENTED");
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.AuthenticationManager#verifyCertificate
     * (java .security.cert.X509Certificate, java.security.PublicKey)
     */
    @Override
    public boolean verifyCertificate(final X509Certificate certificate,
        final PublicKey pubKey) throws SecurityEngineException
    {
        boolean result = true;
        try
        {
            certificate.verify(pubKey);
        }
        catch (final InvalidKeyException e)
        {
            result = false;
        }
        catch (final CertificateException e)
        {
            result = false;
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final SignatureException e)
        {
            result = false;
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.AuthenticationManager#verifyCVCertificate
     * (java .security.cert.X509Certificate, java.security.PublicKey)
     */
    @Override
    public boolean verifyCVCertificate(final X509Certificate CVcertificate,
        final PublicKey pubKey) throws SecurityEngineException
    {
        boolean result = true;
        try
        {
            CVcertificate.verify(pubKey);
        }
        catch (final InvalidKeyException e)
        {
            result = false;
        }
        catch (final CertificateException e)
        {
            result = false;
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final SignatureException e)
        {
            result = false;
        }
        return result;
    }
}
