package org.iso.mpeg.mxm.securityTE.technologyhandler;


import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.util.Date;

import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.cert.X509v3CertificateBuilder;
import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
import org.bouncycastle.cert.jcajce.JcaX509v3CertificateBuilder;
import org.bouncycastle.operator.ContentSigner;
import org.bouncycastle.operator.OperatorCreationException;
import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
import org.iso.mpeg.mxm.securityTE.technologyhandler.util.FileSystemStorage;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data;


/**
 * This implementation of the CertificateManager uses/requires the Bouncy Castle
 * cryptographic libraries and the Morpho CryptoModuleManager library.
 * 
 * @author Andreas Kohlos
 */
public class CertificateManager implements
    org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager
{
    private final CryptoModuleManagerInterface cryptoModuleManager;

    public CertificateManager(final CryptoModuleManagerInterface cryptoModuleManager)
    {
        this.cryptoModuleManager = cryptoModuleManager;
    }

    // TODO create a method/modify the method into one which generates a
    // certificate on
    // the card.
    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.CertificateManager#generateCertificate
     * (java. lang.String, java.lang.String, int, java.security.PublicKey,
     * java.security.PrivateKey, byte, java.lang.String)
     */
    @Override
    // TODO add global unique user ID as input parameter
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data generateCertificate(
        final BigInteger serialNumber, final String holderReference,
        final String issuerReference, final int expiryDays, final PublicKey pubKey,
        final PrivateKey privKey, final String SignOID) throws SecurityEngineException
    {
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data x509Data = null;
        X509Certificate resultCertificate = null;
        ContentSigner contentSigner = null;
        try
        {
            final X500Name issuer = new X500Name(issuerReference);
            final X500Name subject = new X500Name(holderReference);
            final Date currentTime = new Date(System.currentTimeMillis());
            final Date expiryTime = new Date(currentTime.getTime()
                + (1000L * 60 * 60 * 24 * expiryDays));
            final X509v3CertificateBuilder v3CertGen = new JcaX509v3CertificateBuilder(
                issuer, serialNumber, currentTime, expiryTime, subject, pubKey);
            contentSigner = new JcaContentSignerBuilder("SHA256withRSAEncryption")
                .setProvider("BC").build(privKey);
            resultCertificate = new JcaX509CertificateConverter().setProvider("BC")
                .getCertificate(v3CertGen.build(contentSigner));
            x509Data = new org.iso.mpeg.mxm.securityTE.schemahandler.X509Data();
            x509Data.setX509Certificate(resultCertificate.getEncoded());
        }
        catch (final OperatorCreationException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final CertificateException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return x509Data;
    }

    @Override
    // TODO add global unique user ID as input parameter
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data generateCertificate(
        final BigInteger serialNumber, final String holderReference,
        final String issuerReference, final int expiryDays, final PublicKey pubKey,
        final String SignOID) throws SecurityEngineException
    {
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data x509Data = null;
        X509Certificate resultCertificate = null;
        ContentSigner contentSigner = null;
        try
        {
            final X500Name issuer = new X500Name(issuerReference);
            final X500Name subject = new X500Name(holderReference);
            final Date currentTime = new Date(System.currentTimeMillis());
            final Date expiryTime = new Date(currentTime.getTime()
                + (1000L * 60 * 60 * 24 * expiryDays));
            final X509v3CertificateBuilder v3CertGen = new JcaX509v3CertificateBuilder(
                issuer, serialNumber, currentTime, expiryTime, subject, pubKey);
            contentSigner = new JcaContentSignerBuilder("SHA256withRSAEncryption")
            .setProvider("BC").build(null);//TO DO HOW TO GET PRIVATE KEY
            resultCertificate = new JcaX509CertificateConverter().setProvider("BC")
                .getCertificate(v3CertGen.build(contentSigner));
            x509Data = new org.iso.mpeg.mxm.securityTE.schemahandler.X509Data();
            x509Data.setX509Certificate(resultCertificate.getEncoded());
        }
        catch (final CertificateException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final OperatorCreationException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return x509Data;
    }
    
    @Override
    public X509Data getUserCertificate(final String keyIdentifier,
        final String appAlias) throws SecurityEngineException
    {
        X509Data x509Data = new org.iso.mpeg.mxm.securityTE.schemahandler.X509Data();
        try
        {
        	if (this.cryptoModuleManager.getUserCertificate(keyIdentifier, appAlias)!=null)
        		x509Data.setX509Certificate(this.cryptoModuleManager.getUserCertificate(keyIdentifier, appAlias).getEncoded());
        }
        catch (CertificateEncodingException e)
        {
            e.printStackTrace();
        }
        return x509Data;
    }
    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.CertificateManager#generateCVCertificate
     * (java .lang.String, java.lang.String, int, java.security.PublicKey,
     * java.security.PrivateKey, java.lang.String)
     */
    @Override
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data generateCVCertificate(
        final String holderReference, final String issuerReference, final int expiryDays,
        final PublicKey pubKey, final PrivateKey privKey, final String SignOID)
        throws SecurityEngineException
    {
        // TODO create a CV certificate Data Type and use it as return value
        throw new SecurityEngineException("METHOD_NOT_IMPLEMENTED");
    }

    @Override
    public PublicKey generateRSAKeyPair(final String appAlias,
        final String keyIdentifier, final int keyLength)
        throws SecurityEngineException
    {
        return this.cryptoModuleManager.generateRSAKeyPair(keyIdentifier, appAlias,
            keyLength);
    }

    @Override
    // TODO check
    public RSAKeyValue toPrivateRSAKeyValue(final PrivateKey key)
        throws NoSuchAlgorithmException,InvalidKeySpecException
    {
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue keyValue = new org.iso.mpeg.mxm.securityTE.schemahandler.RSAKeyValue();
        final KeyFactory fact = KeyFactory.getInstance(key.getAlgorithm());
        final RSAPrivateKeySpec priv = fact.getKeySpec(key, RSAPrivateKeySpec.class);
        keyValue.setModulus(priv.getModulus().toByteArray());
        keyValue.setExponent(priv.getPrivateExponent().toByteArray());
        return keyValue;
    }

    @Override
    // TODO check
    public RSAKeyValue toPublicRSAKeyValue(final PublicKey key)
        throws NoSuchAlgorithmException,InvalidKeySpecException
    {
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue keyValue = new org.iso.mpeg.mxm.securityTE.schemahandler.RSAKeyValue();
        final KeyFactory fact = KeyFactory.getInstance(key.getAlgorithm());
        final RSAPublicKeySpec pub = fact.getKeySpec(key, RSAPublicKeySpec.class);
        keyValue.setModulus(pub.getModulus().toByteArray());
        keyValue.setExponent(pub.getPublicExponent().toByteArray());
        return keyValue;
    }
}
