package org.iso.mpeg.mxm.securityTE.technologyhandler;


import java.security.PublicKey;
import java.security.cert.X509Certificate;

import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;


/**
 * Classes implementing this interface are used to store, retrieve and manage confidential
 * information in the cryptomodule manager. MXMConfiguration.xml files allow to choose
 * between different implementations (smart card, filesystem implementation,...).
 * 
 * @author Amit Shrestha, Andreas Kohlos
 */
public interface CryptoModuleManagerInterface
{
    /**
     * sign the challanege with auth
     * @param intChallenge
     * @param appAlias
     * @param PIN
     * @return
     * @throws SecurityEngineException
     */
    public byte[] clientAuth(byte[] intChallenge, String appAlias)
        throws SecurityEngineException;
    /**
     * 
     * @param keyIdentifier
     * @param appAlias
     * @param keyLength
     * @param PIN
     * @return
     * @throws SecurityEngineException
     */
    public PublicKey generateRSAKeyPair(String keyIdentifier, String appAlias,
        int keyLength) throws SecurityEngineException;
    
    /**
     * sign the challanege with sig
     * @param message
     * @param appAlias
     * @param PIN
     * @return
     * @throws SecurityEngineException
     */
    public byte[] generateSignature(byte[] message, String appAlias)
        throws SecurityEngineException;

    public PublicKey getPublicKey(String keyIdentifier, String appAlias)
        throws SecurityEngineException;

    public X509Certificate getUserCertificate(final String keyIdentifier,
        final String appAlias) throws SecurityEngineException;
    /**
     * 
     * @param appAlias
     * @param PIN
     * @return
     * @throws SecurityEngineException
     */
    public boolean initStreamUnwrap(String appAlias)
        throws SecurityEngineException;

    public boolean storeCertificate(X509Certificate certificate, String keyIdentifier,
        String appAlias) throws SecurityEngineException;
    
    /**
     * wrapped key decrypted with conf [private key]
     * @param cryptogram
     * @return
     * @throws SecurityEngineException
     */
    public byte[] streamUnwrapAfterInit(final byte[] cryptogram)
        throws SecurityEngineException;

    public byte[] unwrapKeyOnce(byte[] cryptogram, String appAlias)
        throws SecurityEngineException;
    public boolean clientTokenVerify(String challenge, String cardAuthToken, String algorithm,
        PublicKey pubKey) throws SecurityEngineException;
    
    public boolean verifySignature(byte[] message, byte[] signature, String algorithm,
        PublicKey pubKey) throws SecurityEngineException;
}
