package org.iso.mpeg.mxm.securityTE.technologyhandler;


import java.io.IOException;
import java.security.Key;
import java.security.KeyPair;
import java.security.PublicKey;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.security.cert.X509Certificate;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.xerces.impl.dv.util.Base64;
import org.bouncycastle.cert.X509v1CertificateBuilder;
import org.bouncycastle.jce.provider.JCERSAPublicKey;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserRequest;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserResponse;
import org.iso.mpeg.mxm.securityTE.SecurityEngineSchemaHandler;
import org.iso.mpeg.mxm.securityTE.SecurityTE;
import org.iso.mpeg.mxm.securityTE.technologyhandler.util.FileAndObjectHandler;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Assertion;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnContext;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.AuthnStatement;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Conditions;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.NameID;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.SubjectConfirmation;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AuthnRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.Response;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.AuthenticationManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.UserDataBaseManager;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.support.AbstractLobCreatingPreparedStatementCallback;
import org.springframework.jdbc.support.lob.DefaultLobHandler;
import org.springframework.jdbc.support.lob.LobCreator;
import org.w3c.dom.Document;


/**
 * Implements a MySQL_Gateway for the Identity_Provider Repository Manager.
 * 
 * @author Andreas Kohlos, Christoph Topmoeller
 */
public class IPUserDataBaseManager implements
    org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.UserDataBaseManager
{
    private X509Certificate   cert          = null;
    private String            idHandler     = "";
    private boolean           isRevoked     = false;
    private boolean           isUserCreated = false;
    private JdbcTemplate      jdbcTemplate  = null;
    private KeyPair           keyPair       = null;
    private DefaultLobHandler lobhandler    = null;
    private PublicKey         pubKey        = null;
    private org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler        securityEngineSchemaHandler = null;
    private AuthenticationManager authenticationManager = null;
    private SignatureManager signatureManager = null;

    /**
     * Constructor - Adds a CryptoModuleManager, creates the jdbcTemplate and the MySql
     * Database. Creates the database and the tables "users" and "ip_credentials" if they
     * do not exist already.
     * 
     * @param cryptoModuleManager
     * @param mySqlUrl
     * @param mySqlUsername
     * @param mySqlPassword
     * @param dataBaseName
     * @param authenticationManager 
     * @param signatureManager 
     */
    public IPUserDataBaseManager(final CryptoModuleManagerInterface cryptoModuleManager,
        final String mySqlUrl, final String mySqlUsername, final String mySqlPassword,
        final String dataBaseName, final org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler securityEngineSchemaHandler, AuthenticationManager authenticationManager, SignatureManager signatureManager)
    {
        System.out.println("SELECTING " + dataBaseName);
        if (this.jdbcTemplate == null)
        {
            final BasicDataSource dataSource = new BasicDataSource();
            dataSource.setDriverClassName("com.mysql.jdbc.Driver");
            dataSource.setUrl("jdbc:mysql://" + mySqlUrl);
            dataSource.setUsername(mySqlUsername);
            dataSource.setPassword(mySqlPassword);
            this.jdbcTemplate = new JdbcTemplate(dataSource);
        }
        this.jdbcTemplate.execute("CREATE DATABASE if not exists " + dataBaseName
            + " DEFAULT CHARSET=utf8");
        this.jdbcTemplate.execute("USE " + dataBaseName);
        this.jdbcTemplate.execute("CREATE TABLE if not exists users ( " + "id Integer,"
            + " first_name Varchar(80)," + " last_name Varchar(80)," + " birthday Date,"
            + " country Varchar(80)," + " city Varchar(80)," + " callback Boolean,"
            + " callback_date Date," + " callback_reason Varchar(80),"
            + " user_certificate BLOB," + " user_public_key_hash Varchar(255),"
            + " UNIQUE KEY `ID` (`ID`) ) ENGINE=InnoDB DEFAULT CHARSET=utf8");
        this.jdbcTemplate.execute("CREATE TABLE if not exists ip_credentials ( "
            + "ip_root_certificate BLOB," + " ip_keypair BLOB)"
            + " ENGINE=InnoDB DEFAULT CHARSET=utf8");
        this.securityEngineSchemaHandler = securityEngineSchemaHandler;
        this.authenticationManager = authenticationManager;
        this.signatureManager = signatureManager;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#deleteUser(java.
     * lang.String)
     */
    @Override
    public void deleteUser(final String userID)
    {
        this.jdbcTemplate.update("DELETE FROM users WHERE id = ?", new Object[] {userID});
    }

    // /**
    // * This method returns the x509 certificate from the database which is
    // used for user
    // * authorization.
    // *
    // * @param userID - the ID of a user in the database
    // * @return java.security.cert.X509Certificate
    // * @throws IOException
    // */
    // public X509Certificate getUserCertificate(final int userID)
    // throws SecurityEngineException
    // {
    // this.lobhandler = new DefaultLobHandler();
    // this.jdbcTemplate.query("SELECT user_certificate FROM users WHERE id = ?",
    // new RowMapper() {
    // @Override
    // public String mapRow(final ResultSet rs, final int i) throws SQLException
    // {
    // final byte[] blobBytes = IPRepositoryManager.this.lobhandler
    // .getBlobAsBytes(rs, "user_certificate");
    // final ByteArrayInputStream bais = new ByteArrayInputStream(blobBytes);
    // ObjectInputStream ois;
    // try
    // {
    // ois = new ObjectInputStream(bais);
    // IPRepositoryManager.this.cert = (X509Certificate)ois.readObject();
    // }
    // catch (final IOException e)
    // {
    // throw new SQLException(e.getMessage());
    // }
    // catch (final ClassNotFoundException e)
    // {
    // throw new SQLException(e.getMessage());
    // }
    // return null;
    // }
    // });
    // return this.cert;
    // }
    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#getIPRootCertificate ()
     */
    @Override
    public X509Certificate getIPRootCertificate() throws SecurityEngineException
    {
        this.cert = null;
        this.lobhandler = new DefaultLobHandler();
        this.jdbcTemplate.query("SELECT ip_root_certificate FROM ip_credentials",
            new RowMapper() {
                @Override
                public String mapRow(final ResultSet rs, final int i) throws SQLException
                {
                    final byte[] blobBytes = IPUserDataBaseManager.this.lobhandler
                        .getBlobAsBytes(rs, "ip_root_certificate");
                    try
                    {
                        IPUserDataBaseManager.this.cert = (X509Certificate)FileAndObjectHandler
                            .blobToObject(blobBytes);
                    }
                    catch (final SecurityEngineException e)
                    {
                        throw new SQLException(e.getMessage());
                    }
                    return null;
                }
            });
        return this.cert;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#getIssuerKeyPair()
     */
    @Override
    public KeyPair getIssuerKeyPair() throws SecurityEngineException
    {
        this.keyPair = null;
        this.lobhandler = new DefaultLobHandler();
        this.jdbcTemplate.query("SELECT ip_keypair FROM ip_credentials", new RowMapper() {
            @Override
            public String mapRow(final ResultSet rs, final int i) throws SQLException
            {
                final byte[] blobBytes = IPUserDataBaseManager.this.lobhandler
                    .getBlobAsBytes(rs, "ip_keypair");
                try
                {
                    IPUserDataBaseManager.this.keyPair = (KeyPair)FileAndObjectHandler
                        .blobToObject(blobBytes);
                }
                catch (final SecurityEngineException e)
                {
                    throw new SQLException(e.getMessage());
                }
                return null;
            }
        });
        return this.keyPair;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#getIssuerRootCertificate
     * ()
     */
    @Override
    public X509Certificate getIssuerRootCertificate() throws SecurityEngineException
    {
        this.cert = null;
        this.lobhandler = new DefaultLobHandler();
        this.jdbcTemplate.query("SELECT ip_root_certificate FROM ip_credentials",
            new RowMapper() {
                @Override
                public String mapRow(final ResultSet rs, final int i) throws SQLException
                {
                    final byte[] blobBytes = IPUserDataBaseManager.this.lobhandler
                        .getBlobAsBytes(rs, "ip_root_certificate");
                    try
                    {
                        IPUserDataBaseManager.this.cert = (X509Certificate)FileAndObjectHandler
                            .blobToObject(blobBytes);
                    }
                    catch (final SecurityEngineException e)
                    {
                        throw new SQLException(e.getMessage());
                    }
                    return null;
                }
            });
        return this.cert;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#getNextUserID()
     */
    @Override
    public int getNextUserID()
    {
        int result = -1;
        result = this.jdbcTemplate.queryForInt("SELECT MAX(id) FROM users;");
        result++;
        return result;
    }

    @Override
    public PublicKey getPublicKey(final String hashCode)
    {
        this.cert = null;
        this.pubKey = null;
        this.lobhandler = new DefaultLobHandler();
        this.jdbcTemplate.query(
            "SELECT user_certificate FROM users where user_public_key_hash = '"
                + hashCode + "'", new RowMapper() {
                @Override
                public String mapRow(final ResultSet rs, final int i) throws SQLException
                {
                    final byte[] blobBytes = IPUserDataBaseManager.this.lobhandler
                        .getBlobAsBytes(rs, "user_certificate");
                    try
                    {
                        final Object obj = FileAndObjectHandler.blobToObject(blobBytes);
                        ;
                        if (obj.getClass().getCanonicalName()
                            .equals(JCERSAPublicKey.class.getCanonicalName()))
                        {
                            IPUserDataBaseManager.this.pubKey = (JCERSAPublicKey)FileAndObjectHandler
                                .blobToObject(blobBytes);
                        }
                        else
                        {
                            IPUserDataBaseManager.this.cert = (X509Certificate)FileAndObjectHandler
                                .blobToObject(blobBytes);
                        }
                    }
                    catch (final SecurityEngineException e)
                    {
                        throw new SQLException(e.getMessage());
                    }
                    return null;
                }
            });
        if (this.pubKey != null)
        {
            return this.pubKey;
        }
        if(this.cert != null) {
            return this.cert.getPublicKey();
        }
       return null;
    }

    @Override
    public String getUserId(final PublicKey publicKey)
    {
        String hashCode = "";
        try
        {
            hashCode = this.toBase64String(publicKey).hashCode() + "";
        }
        catch (final IOException e)
        {
            e.printStackTrace();
        }
        this.jdbcTemplate.query("SELECT id FROM users WHERE user_public_key_hash='"
            + hashCode + "'", new RowMapper() {
            @Override
            public String mapRow(final ResultSet result, final int arg1)
                throws SQLException
            {
                IPUserDataBaseManager.this.idHandler = result.getString("id");
                return IPUserDataBaseManager.this.idHandler;
            }
        });
        return this.idHandler;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#grantUser(java.lang
     * .String)
     */
    @Override
    public void grantUser(final String userID)
    {
        this.jdbcTemplate.update("UPDATE users SET callback = ? WHERE id = ?",
            new Object[] {false, userID});
        this.jdbcTemplate.update("UPDATE users SET callback_date = ? WHERE id = ?",
            new Object[] {null, userID});
        this.jdbcTemplate.update("UPDATE users SET callback_reason = ? WHERE id = ?",
            new Object[] {null, userID});
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#isUserCreated(java
     * .lang.String )
     */
    @Override
    public boolean isUserCreated(final String userID)
    {
        this.jdbcTemplate.query("SELECT * FROM users WHERE id=" + userID,
            new RowMapper() {
                @Override
                public String mapRow(final ResultSet result, final int arg1)
                    throws SQLException
                {
                    if (result != null)
                    {
                        IPUserDataBaseManager.this.isUserCreated = true;
                    }
                    return null;
                }
            });
        return this.isUserCreated;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#isUserRevoked(java
     * .lang.String )
     */
    @Override
    public boolean isUserRevoked(final String userID)
    {
        this.jdbcTemplate.query("SELECT callback FROM users WHERE id=" + userID,
            new RowMapper() {
                @Override
                public String mapRow(final ResultSet result, final int arg1)
                    throws SQLException
                {
                    IPUserDataBaseManager.this.isRevoked = result.getBoolean("callback");
                    return null;
                }
            });
        return this.isRevoked;
    }

    @Override
    public int registerUser(final PublicKey key, final String firstName,
        final String lastName, final Date birthday, final String country,
        final String locality, final String organization) throws SecurityEngineException
    {
        return this.storeUser((Object)key, firstName, lastName, birthday, country,
            locality, organization,false);
    }

    /**
     * @param userCertificate
     * @param firstName
     * @param lastName
     * @param birthday
     * @param country
     * @param locality
     * @param organization
     * @return
     * @throws SecurityEngineException
     */
    @Override
    public int registerUser(final X509Certificate userCertificate,
        final String firstName, final String lastName, final Date birthday,
        final String country, final String locality, final String organization)
        throws SecurityEngineException
    {
        return this.storeUser((Object)userCertificate, firstName, lastName, birthday,
            country, locality, organization,true);
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#revokeUser(java.
     * lang.String, java.lang.String)
     */
    @Override
    public void revokeUser(final String userID, final String callbackReason)
    {
        this.jdbcTemplate.update("UPDATE users SET callback = ? WHERE id = ?",
            new Object[] {true, userID});
        this.jdbcTemplate.update("UPDATE users SET callback_date = ? WHERE id = ?",
            new Object[] {new Date(System.currentTimeMillis()), userID});
        this.jdbcTemplate.update("UPDATE users SET callback_reason = ? WHERE id= ?",
            new Object[] {callbackReason, userID});
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#storeIssuerCredentials
     * (javax .security.cert.X509Certificate, javax.security.cert.X509Certificate,
     * java.security.KeyPair)
     */
    @Override
    public void storeIssuerCredentials(final X509Certificate iPRootCertificate,
        final X509Certificate sPRootCertificate, final KeyPair issuerKeyPair)
        throws SecurityEngineException
    {
        this.lobhandler = new DefaultLobHandler();
        // Convert the root certificate to a byte array.
        final byte[] iPRootCertificateAsBlob = FileAndObjectHandler
            .objectToBlob(iPRootCertificate);
        // Convert the key pair to a byte array.
        final byte[] iPKeyPairAsBlob = FileAndObjectHandler.objectToBlob(issuerKeyPair);
        this.jdbcTemplate.execute(
            "insert into ip_credentials(ip_root_certificate, ip_keypair) values (?, ?)",
            new AbstractLobCreatingPreparedStatementCallback(this.lobhandler) {
                @Override
                protected void setValues(final PreparedStatement ps,
                    final LobCreator lobCreator) throws SQLException
                {
                    lobCreator.setBlobAsBytes(ps, 1, iPRootCertificateAsBlob);
                    lobCreator.setBlobAsBytes(ps, 2, iPKeyPairAsBlob);
                }
            });
    }

    private int storeUser(final Object object, final String firstName,
        final String lastName, final Date birthday, final String country,
        final String locality, final String organization,boolean isCertificate) throws SecurityEngineException
    {
        this.lobhandler = new DefaultLobHandler();
        // Convert the certificate to a byte array.
        final byte[] userCertificateAsBlob = FileAndObjectHandler.objectToBlob(object);
        final int createdId = this.getNextUserID();
        PublicKey pubKey = null;
        String s = object.getClass().getCanonicalName();
        System.out.println(s);
        if (isCertificate)
        {
            pubKey = ((X509Certificate)object).getPublicKey();
        }
        else
        {
            pubKey = (PublicKey)object;
            final String id = this.getUserId(pubKey);
            if (!id.equals(""))
            {
                return -1;
            }
        }
        String keyCode = null;
        try
        {
            keyCode = this.toBase64String(pubKey).hashCode() + "";
        }
        catch (final IOException e)
        {
            e.printStackTrace();
        }
        final String publicKeyEncoded = keyCode;
        this.jdbcTemplate
            .execute(
                "insert into users(id, first_name, last_name, birthday, country, city, callback, callback_date, callback_reason, user_certificate, user_public_key_hash) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
                new AbstractLobCreatingPreparedStatementCallback(this.lobhandler) {
                    @Override
                    protected void setValues(final PreparedStatement ps,
                        final LobCreator lobCreator) throws SQLException
                    {
                        ps.setInt(1, createdId);
                        ps.setString(2, firstName);
                        ps.setString(3, lastName);
                        ps.setDate(4, birthday);
                        ps.setString(5, country);
                        ps.setString(6, locality);
                        ps.setBoolean(7, false);
                        ps.setDate(8, null);
                        ps.setString(9, null);
                        lobCreator.setBlobAsBytes(ps, 10, userCertificateAsBlob);
                        ps.setString(11, publicKeyEncoded);
                    }
                });
        return createdId;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#storeUserCertsToCard
     * (java. lang.String, javax.security.cert.X509Certificate[])
     */
    @Override
    public void storeUserCertsToCard(final String PIN,
        final X509Certificate[] userCertificates) throws SecurityEngineException
    {
        // TODO use real card command here
        // this.cryptoModuleManager.exampleCardCommand();
    }

    private String toBase64String(final Key key) throws IOException
    {
        final byte array[] = key.getEncoded();
        return Base64.encode(array);
    }
    
    /* (non-Javadoc)
    * @see
    * org.iso.mpeg.mxm.orchestratorEngine.authenticateuser.AuthenticateUserEngineOrchestrator
    * #authenticateUser(org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.
    * AuthenticateUserRequest)
    */
   @Override
   public Response authenticateUser(
       AuthnRequest authenticateUserRequest) throws MXMEngineException
   {
       //Assertion assertion = authenticateUserRequest.getAssertion();
       String publickKeyHash = authenticateUserRequest.getID();
       PublicKey storedKey = getPublicKey(publickKeyHash);
       Response authenticateUserResponse = this.securityEngineSchemaHandler.getResponse();
       if(storedKey == null) {
           Response response = this.securityEngineSchemaHandler.getResponse();
               response.setID("-1");
               response.getAssertionOrEncryptedAssertion().add(null);
           System.out.println("user do not exist");
           return authenticateUserResponse;
       }
       String stored = Base64.encode(storedKey.getEncoded());
      
       if (publickKeyHash.equals(stored.hashCode() + ""))
       {
           System.out.println("User exist");
           // generate challange and send to client
           String challange =  this.authenticationManager.getChallenge();
           // client sign the challange
           String sign = this.signTheChallange(challange);
           // send the signature to server
           // server validates the signature
           boolean verified =  this.authenticationManager.clientTokenVerify(challange, sign, "RSA", storedKey);
           if (verified)
           {
               System.out.println("verified");
               // if sucessfull create assertion
               Assertion assertion = this.securityEngineSchemaHandler.getAssertion();
               String userId = getUserId(storedKey);
               assertion.setID(userId);
               NameID nameId = this.securityEngineSchemaHandler.getNameID();
               nameId.setValue("https://www.convergence.com");
               try
               {
                   assertion.setIssuer(nameId);
                   assertion.setIssueInstant(FileAndObjectHandler.getCurrentInstant());
                   assertion.setConditions(getConditions(FileAndObjectHandler
                       .toXMLGregorianCalendar(new java.util.Date().getTime()), FileAndObjectHandler
                       .toXMLGregorianCalendar(new java.util.Date().getTime() + 10000000)));
                  
                   assertion.setSubject(getSubject(
                       "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress", userId,
                       "urn:oasis:names:tc:SAML:2.0:cm:bearer"));
                   AuthnStatement statement = this.securityEngineSchemaHandler.getAuthnStatement();
                   statement.setAuthnContext(getAuthContex(
                       "urn:oasis:names:tc:SAML:2.0:ac", "Password"));
                   assertion.addAuthnStatement(statement);
                   //generate signature
                   
                  
                   //todo: insted of generating signature of  current instant generate signature of whole assertion it self so it is safe from modificaiton in the middle
                  // Document doc = (Document)assertion.createObject(Document.class.getCanonicalName()).getContent();
                   int hashCode = assertion.getIssueInstant().getMillisecond();
                   Signature signature = this.signatureManager.generateSignature((hashCode+"").getBytes(), "1");
                   assertion.setSignature(signature);
           
                   Response response = this.securityEngineSchemaHandler.getResponse();
                   response.setID(userId);
                   response.getAssertionOrEncryptedAssertion().add(assertion);
                   return response;
                   
               }
               catch (MXMException e)
               {
                   e.printStackTrace();
               }
               return authenticateUserResponse;
           }
           else
           {
               Response response = this.securityEngineSchemaHandler.getResponse();
                   response.setID("-1");
                   response.getAssertionOrEncryptedAssertion().add(null);
               System.out.println("User do not exist");
           }
       }
      
       return authenticateUserResponse;
   }
    
   
   /**
    * this method is means to send request to client and get the signed message
    * 
    * @param challange
    * @return
    * @throws MXMEngineException
    * @throws SecurityEngineException
    */
   private String signTheChallange(String challange) throws SecurityEngineException,
       MXMEngineException
   {
       // to do: get app alias
       return this.authenticationManager.clientAuth(challange, "1", "");
   }
   
   private AuthnContext getAuthContex(String namespace, String localPart)
           throws MXMEngineException
       {
           AuthnContext contex = this.securityEngineSchemaHandler.getAuthnContext();
           QName qname = new QName(namespace, localPart);
           // contex.getContent().add(qname);
           return contex;
       }

       private Conditions getConditions(XMLGregorianCalendar noBeforeDate,
           XMLGregorianCalendar notOnOrAfterDate) throws MXMEngineException
       {
           Conditions conditions = this.securityEngineSchemaHandler.getConditions();
           conditions.setNotBefore(noBeforeDate);
           return conditions;
       }

       private org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Subject getSubject(
           String format, String name, String method) throws MXMException
       {
           SubjectConfirmation subjectConfirmation = this.securityEngineSchemaHandler.getSubjectConfirmation();
           subjectConfirmation.setMethod(method);
           NameID nameId = this.securityEngineSchemaHandler.getNameID();
           nameId.setFormat(format);
           nameId.setValue(name);
           org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.assertion.Subject subject = this.securityEngineSchemaHandler.getSubject();
           subject.addSubjectConfirmation(subjectConfirmation);
           // subject.getContent().add(nameId);
           return subject;
       }
}
