package org.iso.mpeg.mxm.securityTE.technologyhandler;


import java.math.BigInteger;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.spec.AlgorithmParameterSpec;
import java.util.Date;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import javax.security.cert.CertificateException;
import javax.security.cert.X509Certificate;

import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.cert.X509v3CertificateBuilder;
import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
import org.bouncycastle.cert.jcajce.JcaX509v3CertificateBuilder;
import org.bouncycastle.operator.ContentSigner;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;


/**
 * This implementation of the KeyManager uses/requires the Bouncy Castle cryptographic
 * libraries.
 * 
 * @author Andreas Kohlos
 */
public class KeyManager implements
    org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.KeyManager
{
    private final CryptoModuleManagerInterface cryptoModuleManager;

    public KeyManager(final CryptoModuleManagerInterface cryptoModuleManager)
    {
        this.cryptoModuleManager = cryptoModuleManager;
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.KeyManager#decrypt(byte[],
     * javax.crypto.SecretKey, java.lang.String)
     */
    @Override
    public byte[] decrypt(final byte[] data, final SecretKey decryptionKey,
        final String algorithm) throws SecurityEngineException
    {
        byte[] result = null;
        Cipher cipher = null;
        try
        {
            // Using zero IV.
            final byte[] iv = new byte[] {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
            final AlgorithmParameterSpec paramSpec = new IvParameterSpec(iv);
            cipher = Cipher.getInstance("AES/CBC/PKCS7Padding", "BC");
            cipher.init(Cipher.DECRYPT_MODE, decryptionKey, paramSpec);
            result = cipher.doFinal(data);
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IllegalBlockSizeException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final BadPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidAlgorithmParameterException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.KeyManager#decrypt(org.iso.mpeg.mxm
     * .engine. securityTE.data.EncryptedData, javax.crypto.SecretKey, java.lang.String)
     */
    @Override
    public byte[] decrypt(
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedData data,
        final SecretKey decryptionKey, final String algorithm)
        throws SecurityEngineException
    {
        byte[] result = null;
        Cipher cipher = null;
        try
        {
            // Using zero IV.
            final byte[] iv = new byte[] {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
            final AlgorithmParameterSpec paramSpec = new IvParameterSpec(iv);
            cipher = Cipher.getInstance("AES/CBC/PKCS7Padding", "BC");
            cipher.init(Cipher.DECRYPT_MODE, decryptionKey, paramSpec);
            result = cipher
                .doFinal(((org.iso.mpeg.mxm.securityTE.schemahandler.EncryptedData)data)
                    .getEncryptedData());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IllegalBlockSizeException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final BadPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidAlgorithmParameterException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.KeyManager#encrypt(byte[],
     * javax.crypto.SecretKey, java.lang.String)
     */
    @Override
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedData encrypt(
        final byte[] data, final SecretKey encryptionKey, final String algorithm)
        throws SecurityEngineException
    {
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedData encryptedData = null;
        org.iso.mpeg.mxm.engine.securityTE.schema.xenc.CipherDataType cipherDataType = null;
        org.iso.mpeg.mxm.engine.securityTE.schema.xenc.EncryptionMethodType encryptionMethodType = null;
        byte[] result = null;
        Cipher cipher = null;
        try
        {
            // Using zero IV.
            final byte[] iv = new byte[] {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
            final AlgorithmParameterSpec paramSpec = new IvParameterSpec(iv);
            cipher = Cipher.getInstance("AES/CBC/PKCS7Padding", "BC");
            cipher.init(Cipher.ENCRYPT_MODE, encryptionKey, paramSpec);
            result = cipher.doFinal(data);
            encryptionMethodType = new org.iso.mpeg.mxm.engine.securityTE.schema.xenc.EncryptionMethodType();
            encryptionMethodType.setAlgorithm("AES/CBC/PKCS7Padding");
            final org.iso.mpeg.mxm.engine.securityTE.schema.xenc.EncryptedDataType encryptedDataType = new org.iso.mpeg.mxm.engine.securityTE.schema.xenc.EncryptedDataType();
            encryptedDataType.setEncryptionMethod(encryptionMethodType);
            cipherDataType = new org.iso.mpeg.mxm.engine.securityTE.schema.xenc.CipherDataType();
            cipherDataType.setCipherValue(result);
            encryptedDataType.setCipherData(cipherDataType);
            encryptedData = new org.iso.mpeg.mxm.securityTE.schemahandler.EncryptedData(
                encryptedDataType);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IllegalBlockSizeException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final BadPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidAlgorithmParameterException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return encryptedData;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.KeyManager#generateRandomKey(java.
     * lang.String, int)
     */
    @Override
    public SecretKey generateRandomKey(final String algorithm, final int keyLength)
        throws SecurityEngineException
    {
        SecretKey result = null;
        try
        {
            final KeyGenerator kgen = KeyGenerator.getInstance("AES", "BC");
            kgen.init(keyLength);
            result = kgen.generateKey();
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    @Override
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data getCertificate(
        final String appAlias, final String PIN) throws SecurityEngineException
    {
        PublicKey publicKey = null;
        // TODO switch to getCertificate
        publicKey = this.cryptoModuleManager.getPublicKey("conf", appAlias);
        // Dummy creation of a certificate object.
        final String issuerReference = "";
        final String holderReference = "";
        final int expiryDays = 30;
        final BigInteger serialNumber = new BigInteger("256");
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data x509Data = null;
        java.security.cert.X509Certificate resultCertificate = null;
        final ContentSigner contentSigner = null;
        final X500Name issuer = new X500Name(issuerReference);
        final X500Name subject = new X500Name(holderReference);
        final Date currentTime = new Date(System.currentTimeMillis());
        final Date expiryTime = new Date(currentTime.getTime()
            + (1000L * 60 * 60 * 24 * expiryDays));
        final X509v3CertificateBuilder v3CertGen = new JcaX509v3CertificateBuilder(
            issuer, serialNumber, currentTime, expiryTime, subject, publicKey);
        try
        {
            resultCertificate = new JcaX509CertificateConverter().setProvider("BC")
                .getCertificate(v3CertGen.build(contentSigner));
            x509Data = new org.iso.mpeg.mxm.securityTE.schemahandler.X509Data();
            x509Data.setX509Certificate(resultCertificate.getEncoded());
        }
        catch (final java.security.cert.CertificateException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return x509Data;
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.KeyManager#initStreamUnwrap
     * (java.lang.String, java.lang.String, java.lang.String)
     */
    @Override
    public boolean initStreamUnwrap(final String algorithm, final String appAlias,
        final String PIN) throws SecurityEngineException
    {
        boolean result = false;
        result = this.cryptoModuleManager.initStreamUnwrap(appAlias);
        return result;
    }

    @Override
    public SecretKey streamUnwrapAfterInit(
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrappedSymmKey)
        throws SecurityEngineException
    {
        byte[] plainTextKey = null;
        SecretKeySpec unWrappedKey = null;
        plainTextKey = this.cryptoModuleManager.streamUnwrapAfterInit(wrappedSymmKey
            .getCipherData().getCipherValue());
        unWrappedKey = new SecretKeySpec(plainTextKey, "AES");
        return unWrappedKey;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.KeyManager#unwrap(org.iso.mpeg.mxm
     * .engine.securityTE .data.EncryptedKey, java.lang.String, java.lang.String)
     */
    @Override
    public SecretKey unwrap(
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrappedSymmKey,
        final String appAlias, final String PIN) throws SecurityEngineException
    {
        byte[] plainTextKey = null;
        SecretKeySpec unWrappedKey = null;
        plainTextKey = this.cryptoModuleManager.unwrapKeyOnce(
            ((org.iso.mpeg.mxm.securityTE.schemahandler.EncryptedKey)wrappedSymmKey)
                .getCipherData().getCipherValue(), appAlias);
        unWrappedKey = new SecretKeySpec(plainTextKey, "AES");
        return unWrappedKey;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.KeyManager#wrap(javax.crypto.SecretKey ,
     * java.lang.String, org.iso.mpeg.mxm.engine.securityTE.data.X509Data)
     */
    @Override
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrap(
        final SecretKey symmKey, final String algorithm,
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data x509Data)
        throws SecurityEngineException
    {
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey encryptedKey = null;
        byte[] result = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.CipherData cipherData = null;
        try
        {
            final X509Certificate x509Cert = X509Certificate.getInstance(x509Data
                .getX509Certificate());
            // TODO verify certificate here
            final Cipher c = Cipher.getInstance("RSA/NONE/OAEPWithSHA256AndMGF1Padding",
                "BC");
            c.init(Cipher.WRAP_MODE, x509Cert.getPublicKey());
            result = c.wrap(symmKey);
            encryptedKey = new org.iso.mpeg.mxm.securityTE.schemahandler.EncryptedKey();
            // encryptedKey.setEncryptedKey(result,
            // "RSA/NONE/OAEPWithSHA256AndMGF1Padding");
            // ((org.iso.mpeg.mxm.securityTE.schemahandler.EncryptedKey)encryptedKey)
            // .setEncryptedKey(result);
            cipherData = new org.iso.mpeg.mxm.securityTE.schemahandler.CipherData();
            cipherData.setCipherValue(result);
            encryptedKey.setCipherData(cipherData);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IllegalBlockSizeException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final CertificateException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return encryptedKey;
    }

    @Override
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrap(
        final SecretKey symmKey, final String algorithm, final PublicKey publicKey)
        throws SecurityEngineException
    {
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey encryptedKey = null;
        byte[] result = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.CipherData cipherData = null;
        try
        {
            final Cipher c = Cipher.getInstance("RSA/NONE/OAEPWithSHA256AndMGF1Padding",
                "BC");
            c.init(Cipher.WRAP_MODE, publicKey);
            result = c.wrap(symmKey);
            encryptedKey = new org.iso.mpeg.mxm.securityTE.schemahandler.EncryptedKey();
            
            // encryptedKey.setEncryptedKey(result,
            // "RSA/NONE/OAEPWithSHA256AndMGF1Padding");
            // ((org.iso.mpeg.mxm.securityTE.schemahandler.EncryptedKey)encryptedKey)
            // .setEncryptedKey(result);
            cipherData = new org.iso.mpeg.mxm.securityTE.schemahandler.CipherData();
            cipherData.setCipherValue(result);
            encryptedKey.setCipherData(cipherData);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchPaddingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IllegalBlockSizeException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return encryptedKey;
    }
    
    public PublicKey getPublicKey(final String keyIdentifier, final String appAlias,
        final String PIN) throws SecurityEngineException
        {
        return this.cryptoModuleManager.getPublicKey(keyIdentifier, appAlias);
    }
}
