package org.iso.mpeg.mxm.securityTE.technologyhandler;


import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Array;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.xml.crypto.dsig.keyinfo.X509Data;

import morpho.util.Converter;

import org.apache.log4j.Logger;
import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.cert.X509v3CertificateBuilder;
import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
import org.bouncycastle.cert.jcajce.JcaX509v3CertificateBuilder;
import org.bouncycastle.operator.ContentSigner;
import org.bouncycastle.operator.OperatorCreationException;
import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
import org.bouncycastle.util.Arrays;
import org.bouncycastle.util.encoders.Base64;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;
import org.iso.mpeg.mxm.securityTE.technologyhandler.obj.CertificateStoreDataObject;
import org.iso.mpeg.mxm.securityTE.technologyhandler.obj.KeyStoreDataObject;
import org.iso.mpeg.mxm.securityTE.technologyhandler.util.FileAndObjectHandler;
import org.iso.mpeg.mxm.securityTE.technologyhandler.util.FileSystemStorage;
import org.iso.mpeg.mxm.securityTE.technologyhandler.util.SecureInfoStorageTool;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SecurityEngineKeys;


public class OffCardCryptoModuleManager extends CryptoModuleManagerBase
{
    private final Logger            logger                = Logger
                                                              .getLogger(OffCardCryptoModuleManager.class);
    private MXMGenericParameters    mxmParameters         = null;
    private SecureInfoStorageTool   storegeTool;
    private String                  secureInfoAlias       = "infoAlias";
    private String                  defautAppAlias        = "defaulAppAlias";
    private String                  secureInfoStoragePath = null;
    private String                  initStreamUnwrapAppAlias;
    private String                  initStreamUnwrapAppPIN;
    private final String       holderReference = "CN=Holder,L=London,ST=State,OU=Organizational Unit,C=GB,O=Organization,STREET=Street,DC=lmu,UID=HolderID";
    private final String       issuerReference = "CN=Issuer,L=London,ST=State,OU=Organizational Unit,C=GB,O=Organization,STREET=Street,DC=lmu,UID=IssuerID";
    private final int          expiryDays      = 10;
    
    public OffCardCryptoModuleManager(MXMGenericParameters mxmParameter)
    {
        this.mxmParameters = mxmParameter;
        // initialzie KeyStoreDataObject
        this.secureInfoStoragePath = this.mxmParameters
            .getProperty(SecurityEngineKeys.SECURE_INFO_STORAGE_PATH);
        try
        {
            if (FileSystemStorage.getInstance(mxmParameter).getKeyStoreDataObject()
                .getAuthKeyPair(defautAppAlias) == null)
            {
                this.generateDefaults();
            }
        }
        catch (SecurityEngineException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }
    
    private    X509Certificate generateCertificate(
        final BigInteger serialNumber, final String holderReference,
        final String issuerReference, final int expiryDays, final PublicKey pubKey, final PrivateKey privKey ,
        final String SignOID) throws SecurityEngineException
    {
        X509Certificate resultCertificate = null;
        ContentSigner contentSigner = null;
        try
        {
            final X500Name issuer = new X500Name(issuerReference);
            final X500Name subject = new X500Name(holderReference);
            final Date currentTime = new Date(System.currentTimeMillis());
            final Date expiryTime = new Date(currentTime.getTime()
                + (1000L * 60 * 60 * 24 * expiryDays));
            final X509v3CertificateBuilder v3CertGen = new JcaX509v3CertificateBuilder(
                issuer, serialNumber, currentTime, expiryTime, subject, pubKey);
            contentSigner = new JcaContentSignerBuilder("SHA256withRSAEncryption")
            .setProvider("BC").build(privKey);
            resultCertificate = new JcaX509CertificateConverter().setProvider("BC")
                .getCertificate(v3CertGen.build(contentSigner));
            return resultCertificate;
        }
        catch (final CertificateException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final OperatorCreationException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
    }
    
    private void generateDefaults() throws SecurityEngineException,IOException
    {
        KeyStoreDataObject dataObject = FileSystemStorage.getInstance(mxmParameters)
            .getKeyStoreDataObject();
        CertificateStoreDataObject certificateObject = FileSystemStorage.getInstance(mxmParameters).getCertificateStoreDataObject();
        KeyPair authKeyPair = this.generateKeyPair(2048);
        KeyPair confKeyPair = this.generateKeyPair(2048);
        KeyPair sigKeyPair = this.generateKeyPair(2048);
        dataObject.setAuthKeyPair(defautAppAlias, authKeyPair);
        dataObject.setConfKeyPair(defautAppAlias, confKeyPair);
        dataObject.setSigKeyPair(defautAppAlias, sigKeyPair);
        
        
        X509Certificate authCert =  this.generateCertificate(new BigInteger("1"), holderReference, issuerReference, expiryDays, authKeyPair.getPublic(), authKeyPair.getPrivate(), "SHA256withRSAEncryption");
        X509Certificate confCert =  this.generateCertificate(new BigInteger("1"), holderReference, issuerReference, expiryDays, confKeyPair.getPublic(), confKeyPair.getPrivate(), "SHA256withRSAEncryption");
        X509Certificate sigCert =  this.generateCertificate(new BigInteger("1"), holderReference, issuerReference, expiryDays, sigKeyPair.getPublic(), sigKeyPair.getPrivate(), "SHA256withRSAEncryption");
        certificateObject.setAuthCertificate(defautAppAlias, authCert);
        certificateObject.setAuthCertificate(defautAppAlias, confCert);
        certificateObject.setAuthCertificate(defautAppAlias, sigCert);
        FileSystemStorage.getInstance(mxmParameters).save();
    }

    private KeyPair generateKeyPair(int length)
    {
        KeyPairGenerator kpg = null;
        try
        {
            kpg = KeyPairGenerator.getInstance("RSA");
        }
        catch (NoSuchAlgorithmException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        kpg.initialize(length);
        KeyPair kp = kpg.genKeyPair();
        Key publicKey = kp.getPublic();
        Key privateKey = kp.getPrivate();
        try
        {
            KeyFactory fact = KeyFactory.getInstance("RSA");
            RSAPublicKeySpec pub = fact.getKeySpec(publicKey, RSAPublicKeySpec.class);
            RSAPrivateKeySpec priv = fact.getKeySpec(privateKey, RSAPrivateKeySpec.class);
            return kp;
        }
        catch (Exception e)
        {
            System.out.println(e.getMessage());
        }
        return null;
    }

    @Override
    public byte[] clientAuth(final byte[] intChallenge, final String appAlias)
        throws SecurityEngineException
    {
        KeyPair sigKeyPair = FileSystemStorage.getInstance(mxmParameters).getKeyStoreDataObject()
            .getAuthKeyPair(appAlias);
        return this.rsaEncrypt(intChallenge, sigKeyPair.getPrivate());
    }
    
    @Override
    public boolean clientTokenVerify(final String challenge, final String cardAuthToken,
        final String algorithm, final PublicKey pubKey) throws SecurityEngineException
    {
        // throw new SecurityEngineException("METHOD_NOT_IMPLEMENTED");
        boolean result = false;
        Signature sign;
      
        byte [] bytes = Converter.toByteArray(cardAuthToken);
        result = this.verifyClientToken(Converter.toByteArray(challenge), bytes, "", pubKey);
     
        return result;
    }

    
    /**
     * @param message
     * @param signature
     * @param algorithm
     * @param pubKey
     * @return
     * @throws SecurityEngineException
     */
    private boolean verifyClientToken(final byte[] message, final byte[] signature,
        final String algorithm, final PublicKey pubKey) throws SecurityEngineException
    {
        MessageDigest md = null;
        try
        {
            md = MessageDigest.getInstance("SHA-256", "BC");
        }
        catch (NoSuchAlgorithmException e1)
        {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        catch (NoSuchProviderException e1)
        {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        md.update(message);
        byte[] messageHash = md.digest();
        boolean result = false;
        Signature sign;
        try
        {
            sign = Signature.getInstance("RSA", "BC");
            sign.initVerify(pubKey);
            sign.update(messageHash);
            result = sign.verify(signature);
            
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final SignatureException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
        /* try {
            Cipher cipher = Cipher.getInstance("RSA");
            cipher.init(Cipher.DECRYPT_MODE, pubKey);
            byte[] cipherData = cipher.doFinal(signature);
            if(Arrays.equals(message, cipherData)) {
                return true;
            }
        } catch (NoSuchPaddingException e) {
            e.printStackTrace();
        }
        catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        catch (InvalidKeyException e) {
            e.printStackTrace();
        }
        catch (BadPaddingException e) {
            e.printStackTrace();
        }
        catch (IllegalBlockSizeException e)
        {
            e.printStackTrace();
        }
        return false;*/
    }
    
    
    @Override
    public PublicKey generateRSAKeyPair(final String keyIdentifier,
        final String appAlias, final int keyLength) throws SecurityEngineException
    {
        KeyPair pair = this.generateKeyPair(keyLength);
        X509Certificate cert = generateCertificate(new BigInteger("1"), holderReference, issuerReference, expiryDays, pair.getPublic(), pair.getPrivate(), "SHA256withRSAEncryption");
        if (keyIdentifier.equals(AUTH))
        {
            FileSystemStorage.getInstance(mxmParameters).getKeyStoreDataObject()
                .setAuthKeyPair(appAlias, pair);
            FileSystemStorage.getInstance(mxmParameters).getCertificateStoreDataObject()
                .setAuthCertificate(appAlias, cert);
        }
        else if (keyIdentifier.equals(CONF))
        {
            FileSystemStorage.getInstance(mxmParameters).getKeyStoreDataObject()
                .setConfKeyPair(appAlias, pair);
            FileSystemStorage.getInstance(mxmParameters).getCertificateStoreDataObject()
                .setConfCertificate(appAlias, cert);
        }
        else if (keyIdentifier.equals(SIG))
        {
            FileSystemStorage.getInstance(mxmParameters).getKeyStoreDataObject()
                .setSigKeyPair(appAlias, pair);
            FileSystemStorage.getInstance(mxmParameters).getCertificateStoreDataObject()
                .setSigCertificate(appAlias, cert);
        }
        FileSystemStorage.getInstance(mxmParameters).save();
        return pair.getPublic();
    }

    @Override
    public byte[] generateSignature(final byte[] message, final String appAlias)
        throws SecurityEngineException
    {
        KeyPair sigKeyPair = FileSystemStorage.getInstance(mxmParameters).getKeyStoreDataObject()
            .getSigKeyPair(appAlias);
        MessageDigest md = null;
        try
        {
            md = MessageDigest.getInstance("SHA-256", "BC");
        }
        catch (NoSuchAlgorithmException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (NoSuchProviderException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        md.update(message);
        byte[] messageHash = md.digest();
        byte[] sig = this.rsaEncrypt(messageHash, sigKeyPair.getPrivate());
        return sig;
    }

    @Override
    public boolean verifySignature(final byte[] message, final byte[] signature,
        final String algorithm, final PublicKey pubKey) throws SecurityEngineException
    {
        MessageDigest md = null;
        try
        {
            md = MessageDigest.getInstance("SHA-256", "BC");
        }
        catch (NoSuchAlgorithmException e1)
        {
            e1.printStackTrace();
        }
        catch (NoSuchProviderException e1)
        {
            e1.printStackTrace();
        }
        md.update(message);
        byte[] messageHash = md.digest();
        boolean result = false;
        Signature sign;
        try
        {
            sign = Signature.getInstance("RSA", "BC");
            sign.initVerify(pubKey);
            sign.update(messageHash);
            result = sign.verify(signature);
            
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final SignatureException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
        /* try {
            Cipher cipher = Cipher.getInstance("RSA");
            cipher.init(Cipher.DECRYPT_MODE, pubKey);
            byte[] cipherData = cipher.doFinal(signature);
            if(Arrays.equals(message, cipherData)) {
                return true;
            }
        } catch (NoSuchPaddingException e) {
            e.printStackTrace();
        }
        catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        catch (InvalidKeyException e) {
            e.printStackTrace();
        }
        catch (BadPaddingException e) {
            e.printStackTrace();
        }
        catch (IllegalBlockSizeException e)
        {
            e.printStackTrace();
        }
        return false;*/
    }
    

    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data generateSigCertificate(
        final BigInteger serialNumber, final String holderReference,
        final String issuerReference, final int expiryDays, final String appAlias,
        final String SignOID) throws SecurityEngineException
    {
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data x509Data = null;
        X509Certificate resultCertificate = null;
        ContentSigner contentSigner = null;
        try
        {
            final X500Name issuer = new X500Name(issuerReference);
            final X500Name subject = new X500Name(holderReference);
            final Date currentTime = new Date(System.currentTimeMillis());
            final Date expiryTime = new Date(currentTime.getTime()
                + (1000L * 60 * 60 * 24 * expiryDays));
            final X509v3CertificateBuilder v3CertGen = new JcaX509v3CertificateBuilder(
                issuer, serialNumber, currentTime, expiryTime, subject, FileSystemStorage
                    .getInstance(mxmParameters).getKeyStoreDataObject().getSigKeyPair(appAlias)
                    .getPublic());
            contentSigner = new JcaContentSignerBuilder("SHA256withRSAEncryption")
                .setProvider("BC").build(
                    FileSystemStorage.getInstance(mxmParameters).getKeyStoreDataObject()
                        .getSigKeyPair(appAlias).getPrivate());
            resultCertificate = new JcaX509CertificateConverter().setProvider("BC")
                .getCertificate(v3CertGen.build(contentSigner));
            x509Data = new org.iso.mpeg.mxm.securityTE.schemahandler.X509Data();
            x509Data.setX509Certificate(resultCertificate.getEncoded());
        }
        catch (final CertificateException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final OperatorCreationException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return x509Data;
    }

    private byte[] concat  (byte[]a,byte[]b){
        if (a == null) return b;
        if (b == null) return a;
        byte[] r = new byte[a.length+b.length];
        System.arraycopy(a, 0, r, 0, a.length);
        System.arraycopy(b, 0, r, a.length, b.length);
        return r;

    }

    /**
     * encrypts the data with stored private key
     * 
     * @param data
     * @return
     */
    private byte[] rsaEncrypt(byte[] data, PrivateKey pubKey)
    {
        int length = 2048;//pubKey.getEncoded().length;
        int maxChars = length/8 - 11;
        byte [] array = null;
        byte[] part = new byte[maxChars];
        for(int i = 0;i< data.length; i=i+maxChars) {
            try
            {
                Cipher cipher = Cipher.getInstance("RSA");
                cipher.init(Cipher.ENCRYPT_MODE, pubKey);
                byte[] cipherData = null;
                if(i+part.length > data.length) {
                    part = new byte[data.length - i];
                    System.arraycopy(data, i, part, 0, part.length);
                    cipherData = cipher.doFinal(part);
                    
                }else {
                    System.arraycopy(data, i, part, 0, part.length);
                    cipherData = cipher.doFinal(part);
                }
                
                array = this.concat(array, cipherData);
            }
            catch (NoSuchPaddingException e)
            {
                e.printStackTrace();
            }
            catch (NoSuchAlgorithmException e)
            {
                e.printStackTrace();
            }
            catch (InvalidKeyException e)
            {
                e.printStackTrace();
            }
            catch (BadPaddingException e)
            {
                e.printStackTrace();
            }
            catch (IllegalBlockSizeException e)
            {
                e.printStackTrace();
            }
        }
        
        return array;
    }

    @Override
    public PublicKey getPublicKey(final String keyIdentifier, final String appAlias)
        throws SecurityEngineException
    {
        KeyPair pair = null;
        String alias = appAlias;
        if (appAlias == null)
        {
            alias = defautAppAlias;
        }
        if (keyIdentifier.equals(AUTH))
        {
            pair = FileSystemStorage.getInstance(mxmParameters).getKeyStoreDataObject()
                .getAuthKeyPair(alias);
        }
        else if (keyIdentifier.equals(CONF))
        {
            pair = FileSystemStorage.getInstance(mxmParameters).getKeyStoreDataObject()
                .getConfKeyPair(alias);
        }
        else if (keyIdentifier.equals(SIG))
        {
            pair = FileSystemStorage.getInstance(mxmParameters).getKeyStoreDataObject()
                .getSigKeyPair(alias);
        }
        if(pair == null) {
            return null;
        }
        return pair.getPublic();
    }

    @Override
    public X509Certificate getUserCertificate(final String keyIdentifier,
        final String appAlias) throws SecurityEngineException
    {
        if (keyIdentifier.equals(AUTH))
        {
            return FileSystemStorage.getInstance(mxmParameters).getCertificateStoreDataObject()
                .getAuthCertificate(appAlias);
        }
        else if (keyIdentifier.equals(SIG))
        {
            return FileSystemStorage.getInstance(mxmParameters).getCertificateStoreDataObject()
                .getSigCertificate(appAlias);
        }
        else if (keyIdentifier.equals(CONF))
        {
            return FileSystemStorage.getInstance(mxmParameters).getCertificateStoreDataObject()
                .getConfCertificate(appAlias);
        }
        return null;
    }

    @Override
    public boolean initStreamUnwrap(final String appAlias) throws SecurityEngineException
    {
        this.initStreamUnwrapAppAlias = appAlias;
        this.initStreamUnwrapAppPIN = this.mxmParameters.getProperty(CONF);
        // to do check pin
        return true;
    }

    @Override
    public boolean storeCertificate(final X509Certificate certificate,
        final String keyIdentifier, final String appAlias) throws SecurityEngineException
    {
        if (keyIdentifier.equals(AUTH))
        {
            FileSystemStorage.getInstance(mxmParameters).getCertificateStoreDataObject()
                .setAuthCertificate(appAlias, certificate);
        }
        else if (keyIdentifier.equals(SIG))
        {
            FileSystemStorage.getInstance(mxmParameters).getCertificateStoreDataObject()
                .setSigCertificate(appAlias, certificate);
        }
        else if (keyIdentifier.equals(CONF))
        {
            FileSystemStorage.getInstance(mxmParameters).getCertificateStoreDataObject()
                .setConfCertificate(appAlias, certificate);
        }
        return true;
    }

    @Override
    public byte[] streamUnwrapAfterInit(final byte[] cryptogram)
        throws SecurityEngineException
    {
        if (this.initStreamUnwrapAppAlias != null && this.initStreamUnwrapAppPIN != null)
        {
            return this.unwrapKeyOnce(cryptogram, this.initStreamUnwrapAppAlias);
        }
        return null;
    }

    @Override
    public byte[] unwrapKeyOnce(final byte[] cryptogram, final String appAlias)
        throws SecurityEngineException
    {
        KeyPair pair = FileSystemStorage.getInstance(mxmParameters).getKeyStoreDataObject()
            .getConfKeyPair(appAlias);
        ;
        try
        {
            Cipher cipher = Cipher.getInstance("RSA/NONE/OAEPWithSHA256AndMGF1Padding",
                "BC");
            cipher.init(Cipher.UNWRAP_MODE, pair.getPrivate());
            Key key = cipher.unwrap(cryptogram, "AES", Cipher.SECRET_KEY);
            return key.getEncoded();
        }
        catch (NoSuchPaddingException e)
        {
            e.printStackTrace();
        }
        catch (NoSuchAlgorithmException e)
        {
            e.printStackTrace();
        }
        catch (InvalidKeyException e)
        {
            e.printStackTrace();
        }
        catch (NoSuchProviderException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
       
        return null;
    }

    /**
     * Decrpts data from stored privatekey
     * 
     * @param data
     * @return
     */
    private byte[] rsaDecrypt(byte[] data, Key pubKey)
    {
        try
        {
            Cipher cipher = Cipher.getInstance("RSA");
            cipher.init(Cipher.DECRYPT_MODE, pubKey);
            byte[] cipherData = cipher.doFinal(data);
            return cipherData;
        }
        catch (NoSuchPaddingException e)
        {
            e.printStackTrace();
        }
        catch (NoSuchAlgorithmException e)
        {
            e.printStackTrace();
        }
        catch (InvalidKeyException e)
        {
            e.printStackTrace();
        }
        catch (BadPaddingException e)
        {
            e.printStackTrace();
        }
        catch (IllegalBlockSizeException e)
        {
            e.printStackTrace();
        }
        return null;
    }
}
