package org.iso.mpeg.mxm.securityTE.technologyhandler;


import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPublicKeySpec;
import java.util.Arrays;
import java.util.HashMap;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import morpho.cryptomodule.CryptoModuleException;
import morpho.util.Converter;

import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;


/**
 * Implements the CryptoModuleManagerInterface by using Morpho's CryptoModuleManager.jar.
 * It enables the use of the CONVERGENCE smart card as high secure crypto module and
 * credential repository.
 */
public class OnCardCryptoModuleManager extends CryptoModuleManagerBase
{
    private morpho.cryptomodule.convergence.CommandDispatcher morphoCryptoModuleManager = null;
    private MXMGenericParameters                              parms                     = null;

    public OnCardCryptoModuleManager(final String cardProtocol,
        final String cardTerminalName, final MXMGenericParameters mxmParameters)
        throws SecurityEngineException
    {
        this.parms = mxmParameters;
        try
        {
            this.morphoCryptoModuleManager = new morpho.cryptomodule.convergence.CommandDispatcher(
                cardProtocol, cardTerminalName);
        }
        catch (final CryptoModuleException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
    }

    @Override
    public byte[] clientAuth(final byte[] intChallenge, final String appAlias)
        throws SecurityEngineException
    {
        try
        {
            String pin = this.parms.getProperty(AUTH);
            byte[] encChallange = this.morphoCryptoModuleManager.clientAuth(intChallenge,
                appAlias, pin);
            return encChallange;
        }
        catch (final CryptoModuleException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
    }

    @Override
    public boolean clientTokenVerify(final String challenge, final String cardAuthToken,
        final String algorithm, final PublicKey pubKey) throws SecurityEngineException
    {
        // byte []encchallange = Converter.toByteArray(cardAuthToken);
        // byte[] challange = Converter.toByteArray(challenge);
        // try {
        // Cipher cipher = Cipher.getInstance("SHA256withRSA/PSS", "BC");
        // cipher.init(Cipher.DECRYPT_MODE, pubKey);
        // byte[] cipherData = cipher.doFinal(encchallange);
        // if(Arrays.equals(challange, cipherData)) {
        // return true;
        // }
        // }
        // catch (NoSuchAlgorithmException e) {
        // e.printStackTrace();
        // }
        // catch (InvalidKeyException e) {
        // e.printStackTrace();
        // } catch (BadPaddingException e) {
        // e.printStackTrace();
        // }
        // catch (IllegalBlockSizeException e)
        // {
        // e.printStackTrace();
        // }
        // catch (NoSuchPaddingException e)
        // {
        // e.printStackTrace();
        // }
        // catch (NoSuchProviderException e)
        // {
        // // TODO Auto-generated catch block
        // e.printStackTrace();
        // }
        //
        // return false;
        // // throw new SecurityEngineException("METHOD_NOT_IMPLEMENTED");
        boolean result = false;
        Signature sign;
        try
        {
            sign = Signature.getInstance("SHA256withRSA/PSS", "BC");
            sign.initVerify(pubKey);
            byte[] challange = Converter.toByteArray(challenge);
            sign.update(challange);
            byte[] encchallange = Converter.toByteArray(cardAuthToken);
            result = sign.verify(encchallange);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final SignatureException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    @Override
    public boolean verifySignature(final byte[] message, final byte[] signature,
        final String algorithm, final PublicKey pubKey) throws SecurityEngineException
    {
        boolean result = false;
        Signature sign;
        try
        {
            sign = Signature.getInstance("SHA256withRSA/PSS", "BC");
            sign.initVerify(pubKey);
            sign.update(message);
            result = sign.verify(signature);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final SignatureException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
        /*
         * try { Cipher cipher = Cipher.getInstance("RSA");
         * cipher.init(Cipher.DECRYPT_MODE, pubKey); byte[] cipherData =
         * cipher.doFinal(signature); if(Arrays.equals(message, cipherData)) { return
         * true; } } catch (NoSuchPaddingException e) { e.printStackTrace(); } catch
         * (NoSuchAlgorithmException e) { e.printStackTrace(); } catch
         * (InvalidKeyException e) { e.printStackTrace(); } catch (BadPaddingException e)
         * { e.printStackTrace(); } catch (IllegalBlockSizeException e) {
         * e.printStackTrace(); } return false;
         */
    }

    @Override
    public PublicKey generateRSAKeyPair(final String keyIdentifier,
        final String appAlias, final int keyLength) throws SecurityEngineException
    {
        PublicKey publicKey = null;
        byte[] cardModulus = null;
        try
        {
            String pin = this.parms.getProperty(PIN_GAKP);
            System.out.println("inpute pin" + pin);
            cardModulus = this.morphoCryptoModuleManager.generateRSAKeyPair(
                keyIdentifier, appAlias, keyLength, pin);
            final RSAPublicKeySpec pubKeySpec = new RSAPublicKeySpec(new BigInteger(
                Converter.toHex(cardModulus), 16), new BigInteger("10001", 16));
            KeyFactory keyFactory;
            keyFactory = KeyFactory.getInstance("RSA", "BC");
            publicKey = keyFactory.generatePublic(pubKeySpec);
        }
        catch (final CryptoModuleException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeySpecException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return publicKey;
    }

    @Override
    public byte[] generateSignature(final byte[] message, final String appAlias)
        throws SecurityEngineException
    {
        byte[] signature = null;
        byte[] messageHash = null;
        MessageDigest md = null;
        try
        {
            // Generate a hash of the input message - then sign it.
            md = MessageDigest.getInstance("SHA-256", "BC");
            md.update(message);
            messageHash = md.digest();
            String pin = this.parms.getProperty(SIG);
            signature = this.morphoCryptoModuleManager.generateSignature(messageHash,
                appAlias, pin);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final CryptoModuleException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return signature;
    }

    @Override
    public PublicKey getPublicKey(final String keyIdentifier, final String appAlias)
        throws SecurityEngineException
    {
        PublicKey publicKey = null;
        byte[] modulus = null;
        try
        {
            String pin = this.parms.getProperty(PIN_GAKP);
            modulus = this.morphoCryptoModuleManager.getPublicKey(keyIdentifier,
                appAlias, pin);
            final RSAPublicKeySpec pubKeySpec = new RSAPublicKeySpec(new BigInteger(
                Converter.toHex(modulus), 16), new BigInteger("10001", 16));
            KeyFactory keyFactory;
            keyFactory = KeyFactory.getInstance("RSA", "BC");
            publicKey = keyFactory.generatePublic(pubKeySpec);
        }
        catch (final CryptoModuleException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeySpecException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return publicKey;
    }

    @Override
    public X509Certificate getUserCertificate(final String keyIdentifier,
        final String appAlias) throws SecurityEngineException
    {
        X509Certificate x509Certificate = null;
        byte[] byteEncodedCert = null;
        try
        {
            byteEncodedCert = this.morphoCryptoModuleManager.readCertificate(
                keyIdentifier, appAlias);
            final InputStream in = new ByteArrayInputStream(byteEncodedCert);
            final CertificateFactory certFactory = CertificateFactory
                .getInstance("X.509");
            x509Certificate = (X509Certificate)certFactory.generateCertificate(in);
        }
        catch (final CryptoModuleException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final CertificateException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return x509Certificate;
    }

    //
    // /**
    // * @param mxmParameters
    // */
    // public void initialise(final MXMGenericParameters mxmParameters)
    // {
    // this.mxmParameters = mxmParameters;
    // }
    @Override
    public boolean initStreamUnwrap(final String appAlias) throws SecurityEngineException
    {
        boolean result = false;
        try
        {
            result = this.morphoCryptoModuleManager.initStreamUnwrap("", appAlias,
                this.parms.getProperty(CONF));
        }
        catch (final CryptoModuleException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    @Override
    public boolean storeCertificate(final X509Certificate certificate,
        final String keyIdentifier, final String appAlias) throws SecurityEngineException
    {
        boolean result = false;
        try
        {
            result = this.morphoCryptoModuleManager.storeCertificate(
                certificate.getEncoded(), keyIdentifier, appAlias,
                this.parms.getProperty(keyIdentifier));
        }
        catch (final CryptoModuleException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final CertificateEncodingException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    @Override
    public byte[] streamUnwrapAfterInit(final byte[] cryptogram)
        throws SecurityEngineException
    {
        byte[] symmetricKey = null;
        try
        {
            symmetricKey = this.morphoCryptoModuleManager
                .streamUnwrapAfterInit(cryptogram);
        }
        catch (final CryptoModuleException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return symmetricKey;
    }

    @Override
    public byte[] unwrapKeyOnce(final byte[] cryptogram, final String appAlias)
        throws SecurityEngineException
    {
        byte[] symmetricKey = null;
        try
        {
            symmetricKey = this.morphoCryptoModuleManager.unwrapKeyOnce(cryptogram, "",
                appAlias, this.parms.getProperty(CONF));
        }
        catch (final CryptoModuleException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return symmetricKey;
    }
}
