package org.iso.mpeg.mxm.securityTE.technologyhandler;


import java.io.FileInputStream;
import java.io.IOException;
import java.io.StringWriter;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPublicKeySpec;
import java.util.Arrays;
import java.util.UUID;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.bouncycastle.util.encoders.Base64;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.securityTE.SecurityEngineSchemaHandler;
import org.iso.mpeg.mxm.securityTE.technologyhandler.util.FileSystemStorage;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 * This implementation of the SignatureManager uses/requires the Bouncy Castle
 * cryptographic libraries and the Morpho CryptoModuleManager library.
 * 
 * @author Andreas Kohlos
 */
public class SignatureManager implements
    org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager
{
    private DocumentBuilder                    builder;
    private final CryptoModuleManagerInterface cryptoModuleManager;
    private final DocumentBuilderFactory       domFactory;
    private final org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler  schemaHandler;
    private final XPath                        xpath;
    private static boolean                     sigIDfound = false;

    /**
     * @param cryptoModuleManager
     * @param secureRepoManager
     */
    public SignatureManager(final CryptoModuleManagerInterface cryptoModuleManager,
        final org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngineSchemaHandler schemaHandler)
    {
        this.cryptoModuleManager = cryptoModuleManager;
        this.domFactory = DocumentBuilderFactory.newInstance();
        this.xpath = XPathFactory.newInstance().newXPath();
        this.domFactory.setNamespaceAware(true);
        this.schemaHandler = schemaHandler;
        try
        {
            this.builder = this.domFactory.newDocumentBuilder();
        }
        catch (final ParserConfigurationException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * @param doc
     * @return
     */
    private String docToString(final Document doc)
    {
        final StringWriter out = new StringWriter();
        final XMLSerializer serializer = new XMLSerializer(out, new OutputFormat(doc));
        try
        {
            serializer.serialize(doc);
        }
        catch (final IOException e)
        {
            return null;
        }
        return out.toString();
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager#
     * generateCardSignature(byte[], java.lang.String, java.lang.String, java.lang.String)
     */
    @Override
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature generateCardSignature(
        final byte[] message, final String algorithm, final String appAlias,
        final String PIN) throws MXMException
    {
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo keyInfo = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignedInfo signedInfo = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.CanonicalizationMethod canonicalizationMethod = null;
        byte[] hashValue = null;
        try
        {
            hashValue = this.generateHash(message, "SHA-256");
            // final RSAPublicKey rsaPubKey = (RSAPublicKey)keyPair.getPublic();
            // keyValue = new
            // org.iso.mpeg.mxm.engine.securityTE.schema.KeyValue();
            // keyValue.setRSA(rsaPubKey.getModulus().toByteArray(), rsaPubKey
            // .getPublicExponent().toByteArray());
            keyInfo = new org.iso.mpeg.mxm.securityTE.schemahandler.dsig.KeyInfo(
                this.schemaHandler);
            keyInfo.setKeyName("RSAKey");
            // keyInfo.setKeyValue(keyValue);
            canonicalizationMethod = new org.iso.mpeg.mxm.securityTE.schemahandler.CanonicalizationMethod();
            ((org.iso.mpeg.mxm.securityTE.schemahandler.CanonicalizationMethod)canonicalizationMethod)
                .setAlgorithm("SHA256withRSA/PSS");
            signedInfo = new org.iso.mpeg.mxm.securityTE.schemahandler.SignedInfo();
            ((org.iso.mpeg.mxm.securityTE.schemahandler.SignedInfo)signedInfo)
                .setCanonicalizationMethod(canonicalizationMethod);
            signature = new org.iso.mpeg.mxm.securityTE.schemahandler.dsig.Signature(
                this.schemaHandler);
            signature.setSignedInfo(signedInfo);
            signature.setKeyInfo(keyInfo);
            signature.setSignatureValue("0",
                this.cryptoModuleManager.clientAuth(hashValue, appAlias));
        }
        catch (final MXMEngineException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return signature;
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager#generateHash
     * (byte[], java.lang.String)
     */
    @Override
    public byte[] generateHash(final byte[] data, final String algorithm)
        throws SecurityEngineException
    {
        byte[] resultHash = null;
        MessageDigest md;
        try
        {
            md = MessageDigest.getInstance("SHA-256", "BC");
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        md.update(data);
        resultHash = md.digest();
        return resultHash;
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager#generateHash
     * (java.io.FileInputStream, java.lang.String)
     */
    @Override
    public byte[] generateHash(final FileInputStream fis, final String algorithm)
        throws SecurityEngineException
    {
        byte[] resultHash = null;
        MessageDigest md;
        final byte streamContent[] = null;
        try
        {
            md = MessageDigest.getInstance(algorithm, "BC");
            fis.read(streamContent);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        md.update(streamContent);
        resultHash = md.digest();
        return resultHash;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager#
     * generateSignature(byte[],
     * org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue,
     * java.lang.String)
     */
    @Override
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature generateSignature(
        final byte[] message, final String appAlias) throws SecurityEngineException
    {
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyValue keyValue = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo keyInfo = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignedInfo signedInfo = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignatureMethod signatureMethod = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue rsaKeyValue = null;
        String signatureID = null;
        try
        {
            // Generate a new signature ID.
            signatureID = UUID.randomUUID().toString();
            final byte[] sig = this.cryptoModuleManager.generateSignature(message,
                appAlias);
            
            final PublicKey publicKey = this.cryptoModuleManager.getPublicKey("sig",
                appAlias);
            final RSAPublicKey rsaPubKey = (RSAPublicKey)publicKey;
            rsaKeyValue = new org.iso.mpeg.mxm.securityTE.schemahandler.RSAKeyValue(
                rsaPubKey.getModulus().toByteArray(), rsaPubKey.getPublicExponent()
                    .toByteArray());
            keyValue = new org.iso.mpeg.mxm.securityTE.schemahandler.KeyValue();
            keyValue.setRSAKeyValue(rsaKeyValue);
            keyInfo = new org.iso.mpeg.mxm.securityTE.schemahandler.dsig.KeyInfo(
                this.schemaHandler);
            keyInfo.setKeyName("RSAKey");
            keyInfo.setKeyValue(keyValue);
            signedInfo = new org.iso.mpeg.mxm.securityTE.schemahandler.SignedInfo();
            signatureMethod = new org.iso.mpeg.mxm.securityTE.schemahandler.SignatureMethod();
            signatureMethod.setAlgorithm("SHA256withRSAandMGF1");
            signedInfo.setSignatureMethod(signatureMethod);
            signature = new org.iso.mpeg.mxm.securityTE.schemahandler.dsig.Signature(
                this.schemaHandler);
            signature.setSignedInfo(signedInfo);
            signature.setKeyInfo(keyInfo);
            signature.setId(signatureID);
            signature.setSignatureValue(null, sig);
        }
        catch (final MXMException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final Exception e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return signature;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager#
     * generateSignature(org.w3c.dom.Document,
     * org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue,
     * java.lang.String)
     */
    @Override
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature generateSignature(
        final Document inputDomObject, final String appAlias) throws SecurityEngineException
    {
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyValue keyValue = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo keyInfo = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignedInfo signedInfo = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.SignatureMethod signatureMethod = null;
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue rsaKeyValue = null;
        String signatureID = null;
        try
        {
            // Generate a new signature ID.
            signatureID = UUID.randomUUID().toString();
            final byte[] sig = this.cryptoModuleManager.generateSignature(this
                .docToString(inputDomObject).getBytes(), appAlias);
            
            final PublicKey publicKey = this.cryptoModuleManager.getPublicKey("sig",
                appAlias);
            final RSAPublicKey rsaPubKey = (RSAPublicKey)publicKey;
            rsaKeyValue = new org.iso.mpeg.mxm.securityTE.schemahandler.RSAKeyValue(
                rsaPubKey.getModulus().toByteArray(), rsaPubKey.getPublicExponent()
                    .toByteArray());
            keyValue = new org.iso.mpeg.mxm.securityTE.schemahandler.KeyValue();
            keyValue.setRSAKeyValue(rsaKeyValue);
            keyInfo = new org.iso.mpeg.mxm.securityTE.schemahandler.dsig.KeyInfo(
                this.schemaHandler);
            keyInfo.setKeyName("RSAKey");
            keyInfo.setKeyValue(keyValue);
            signedInfo = new org.iso.mpeg.mxm.securityTE.schemahandler.SignedInfo();
            signatureMethod = new org.iso.mpeg.mxm.securityTE.schemahandler.SignatureMethod();
            signatureMethod.setAlgorithm("SHA256withRSAandMGF1");
            signedInfo.setSignatureMethod(signatureMethod);
            signature = new org.iso.mpeg.mxm.securityTE.schemahandler.dsig.Signature(
                this.schemaHandler);
            signature.setSignedInfo(signedInfo);
            signature.setKeyInfo(keyInfo);
            signature.setId(signatureID);
            signature.setSignatureValue(null, sig);
        }
        catch (final MXMException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return signature;
    }

    /**
     * @param doc
     * @param signatureID
     * @return
     */
    private Document stripSignatureFromDoc(final Document doc, final String signatureID)
    {
        Node node = doc.getFirstChild();
        stripIDfromNode(node, signatureID);
        return doc;
    }

    private static void stripIDfromNode(Node node, String ID)
    {
        Element currentElement = (Element)node;
        if (currentElement.getAttribute("Id").equals(ID))
        {
            node.getParentNode().getParentNode().removeChild(node.getParentNode());
            sigIDfound = true;
        }
        else
        {
            // Visit the children
            NodeList childs = node.getChildNodes();
            int i = 0;
            while (!sigIDfound && i < childs.getLength())
            {
                if(childs.item(i).hasChildNodes())
                stripIDfromNode(childs.item(i), ID);
                i++;
            }
        }
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager#verifyHash
     * (byte[], byte[], java.lang.String)
     */
    @Override
    public boolean verifyHash(final byte[] data, final byte[] hashValue,
        final String algorithm) throws SecurityEngineException
    {
        boolean result = false;
        MessageDigest md;
        try
        {
            md = MessageDigest.getInstance("SHA-256", "BC");
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        md.update(data);
        if (MessageDigest.isEqual(md.digest(), hashValue))
        {
            result = true;
        }
        return result;
    }

    // /*
    // * (non-Javadoc)
    // * @see org.iso.mpeg.mxm.engine.securityTE.SignatureManager#verifySignature(byte [],
    // * org.iso.mpeg.mxm.engine.securityTE.data.Signature, java.lang.String,
    // * org.iso.mpeg.mxm.engine.securityTE.data.X509Data)
    // */
    // @Override
    // public boolean verifySignature(final byte[] message,
    // final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature,
    // final String algorithm,
    // final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data x509Data)
    // throws SecurityEngineException
    // {
    // Signature sign = null;
    // X509Certificate x509Cert = null;
    // try
    // {
    // x509Cert = X509Certificate.getInstance(x509Data.getX509Certificate());
    // // TODO verify cert here
    // }
    // catch (final CertificateException e)
    // {
    // throw new SecurityEngineException(e.getMessage());
    // }
    // return this.verifySignature(message, signature.getSignatureValue().getValue(),
    // algorithm, x509Cert.getPublicKey());
    // }
    /**
     * @param message
     * @param signature
     * @param algorithm
     * @param pubKey
     * @return
     * @throws SecurityEngineException
     */
    private boolean verifySignature(final byte[] message, final byte[] signature,
        final String algorithm, final PublicKey pubKey) throws SecurityEngineException
    {
       return this.cryptoModuleManager.verifySignature(message,signature,algorithm,pubKey);
    }
    
    
    
    
    
    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager#verifySignature
     * (byte[], org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature)
     */
    @Override
    public boolean verifySignature(final byte[] message,
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature)
        throws MXMEngineException
    {
        KeyFactory keyFactory = null;
        RSAPublicKey rsaPublicKey = null;
        final RSAPublicKeySpec rsaPubKeySpec = new RSAPublicKeySpec(new BigInteger(
            signature.getKeyInfo().getKeyValue().getRSAKeyValue().getModulus()),
            new BigInteger(signature.getKeyInfo().getKeyValue().getRSAKeyValue()
                .getExponent()));
        try
        {
            keyFactory = KeyFactory.getInstance("RSA", "BC");
            rsaPublicKey = (RSAPublicKey)keyFactory.generatePublic(rsaPubKeySpec);
        }
        catch (final InvalidKeySpecException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return this.verifySignature(message, signature.getSignatureValue().getValue(),
            "RSA", rsaPublicKey);
    }

    /*
     * (non-Javadoc)
     * @see
     * org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager#verifySignature
     * (org.w3c.dom.Document,
     * org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature)
     */
    @Override
    public boolean verifySignature(final Document inputDomObject,
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature)
        throws MXMEngineException
    {
        final Document strippedDocument = this.stripSignatureFromDoc(inputDomObject,
            signature.getId());
//        StringWriter out = new StringWriter();
//        XMLSerializer serializer = new XMLSerializer(out,
//            new OutputFormat(inputDomObject));
//        try
//        {
//            serializer.serialize(inputDomObject);
//        }
//        catch (IOException e)
//        {
//            throw new SecurityEngineException(e.getMessage());
//        }
//        System.out.println("strippedDocument: " + out.toString());
        return this.verifySignature(this.docToString(strippedDocument).getBytes(),
            signature);
    }

   
}
