package org.iso.mpeg.mxm.securityTE.technologyhandler.obj;


import java.util.HashMap;

import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedData;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey;


public class EncryptedDataObj
{
    private final EncryptedData                  cipherText;
    // HashMap which associated userIds to wrappedKeys
    private final HashMap<Integer, EncryptedKey> wrappedKeys;

    /**
     * This constructor initialize the Hashmap for the wrapped keys and add the ciphertext
     * 
     * @param cipherText
     */
    public EncryptedDataObj(final EncryptedData cipherText)
    {
        super();
        this.cipherText = cipherText;
        this.wrappedKeys = new HashMap<Integer, EncryptedKey>(); // <UserId, wrappedKey>
    }

    /**
     * This method returns the CipherText of the object
     * 
     * @return
     */
    public byte[] getCipherText()
    {
        return this.cipherText.getCipherData().getCipherValue();
    }

    /**
     * This method returns the wrappedkey if it exists
     * 
     * @param userID
     * @return The wrappedKey to the associated userID
     */
    public EncryptedKey getWrappedKey(final int userID)
    {
        if (this.wrappedKeys.get(userID) != null)
        {
            return this.wrappedKeys.get(userID);
        }
        return null;
    }

    /**
     * This method insert a wrappedKey to the object
     * 
     * @param id
     * @param wrappedKey
     */
    public void insertWrappedKey(final int id, final EncryptedKey wrappedKey)
    {
        this.wrappedKeys.put(id, wrappedKey);
    }
}
