package org.iso.mpeg.mxm.securityTE.technologyhandler.util;


import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.math.BigInteger;
import java.security.Key;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.xerces.impl.dv.util.Base64;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue;


public class FileAndObjectHandler
{
    public static Object blobToObject(final byte[] blob) throws SecurityEngineException
    {
        Object result = null;
        final ByteArrayInputStream bais = new ByteArrayInputStream(blob);
        ObjectInputStream ois;
        try
        {
            ois = new ObjectInputStream(bais);
            result = ois.readObject();
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final ClassNotFoundException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    public static void deleteFile(final String filePath) throws SecurityEngineException
    {
        final File file = new File(filePath);
        // Make sure the file or directory exists and isn't write protected.
        if (!file.exists())
        {
            throw new SecurityEngineException("Delete: no such file or directory");
        }
        if (!file.canWrite())
        {
            throw new SecurityEngineException("Delete: write protected");
        }
        // If it is a directory, make sure it is empty
        if (file.isDirectory())
        {
            final String[] files = file.list();
            if (files.length > 0)
            {
                throw new SecurityEngineException("Delete: directory not empty");
            }
        }
        // Attempt to delete it.
        final boolean success = file.delete();
        if (!success)
        {
            throw new SecurityEngineException("Delete: deletion failed");
        }
    }

    /**
     * @return
     */
    public static XMLGregorianCalendar getCurrentInstant()
    {
        final Date date = new Date();
        final GregorianCalendar gc = new GregorianCalendar();
        gc.setTimeInMillis(date.getTime());
        XMLGregorianCalendar date2 = null;
        try
        {
            date2 = DatatypeFactory.newInstance().newXMLGregorianCalendar(gc);
        }
        catch (final DatatypeConfigurationException e)
        {
            e.printStackTrace();
        }
        return date2;
    }

    /**
     * Returns base64 value of the given key
     * 
     * @param key
     * @return
     * @throws IOException
     */
    public static String getString(final Key key) throws IOException
    {
        final byte array[] = key.getEncoded();
        return Base64.encode(array);
    }

    public static byte[] objectToBlob(final Object object) throws SecurityEngineException
    {
        byte[] result = null;
        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ObjectOutputStream oos = null;
        try
        {
            oos = new ObjectOutputStream(baos);
            oos.writeObject(object);
            oos.flush();
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        finally
        {
            if (oos != null)
            {
                try
                {
                    oos.close();
                }
                catch (final IOException e)
                {
                    throw new SecurityEngineException(e.getMessage());
                }
            }
            result = baos.toByteArray();
            if (baos != null)
            {
                try
                {
                    baos.close();
                }
                catch (final IOException e)
                {
                    throw new SecurityEngineException(e.getMessage());
                }
            }
        }
        return result;
    }

    /**
     * @param privateKey
     * @return
     * @throws SecurityEngineException
     */
    public static PrivateKey privateKeyAdapter(final RSAKeyValue privateKey)
        throws SecurityEngineException
    {
        final RSAPrivateKeySpec privKeySpec = new RSAPrivateKeySpec(new BigInteger(
            privateKey.getModulus()), new BigInteger(privateKey.getExponent()));
        KeyFactory keyFactory;
        PrivateKey privKey = null;
        try
        {
            keyFactory = KeyFactory.getInstance("RSA", "BC");
            privKey = (PrivateKey)keyFactory.generatePrivate(privKeySpec);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeySpecException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return privKey;
    }

    /**
     * @param publicKey
     * @return
     * @throws SecurityEngineException
     */
    public static PublicKey publicKeyAdapter(final RSAKeyValue publicKey)
        throws SecurityEngineException
    {
        final RSAPublicKeySpec pubKeySpec = new RSAPublicKeySpec(new BigInteger(
            publicKey.getModulus()), new BigInteger(publicKey.getExponent()));
        KeyFactory keyFactory;
        PublicKey pubKey = null;
        try
        {
            keyFactory = KeyFactory.getInstance("RSA", "BC");
            pubKey = (PublicKey)keyFactory.generatePublic(pubKeySpec);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeySpecException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return pubKey;
    }

    public static byte[] readBytesFromFile(final String filePath)
        throws SecurityEngineException
    {
        final File file = new File(filePath);
        try
        {
            if (file.exists())
            {
                final FileInputStream in = new FileInputStream(file);
                final byte[] result = new byte[in.available()];
                in.read(result);
                in.close();
                return result;
            }
            else
            {
                throw new SecurityEngineException("File: '" + filePath + "' - not found");
            }
        }
        catch (final FileNotFoundException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
    }

    public static Object readObjectFromFile(final String filePath)
        throws SecurityEngineException
    {
        Object result = null;
        ObjectInputStream ois = null;
        FileInputStream fis = null;
        final File file = new File(filePath);
        if (file.exists())
        {
            try
            {
                fis = new FileInputStream(file);
                ois = new ObjectInputStream(fis);
                result = ois.readObject();
            }
            catch (final IOException e)
            {
                throw new SecurityEngineException("Error while reading the file: '"
                    + filePath + "'.");
            }
            catch (final ClassNotFoundException e)
            {
                throw new SecurityEngineException("Error while reading the file: '"
                    + filePath + "'.");
            }
            finally
            {
                if (ois != null)
                {
                    try
                    {
                        ois.close();
                    }
                    catch (final IOException e)
                    {
                        throw new SecurityEngineException(e.getMessage());
                    }
                }
                if (fis != null)
                {
                    try
                    {
                        fis.close();
                    }
                    catch (final IOException e)
                    {
                        throw new SecurityEngineException(e.getMessage());
                    }
                }
            }
        }
        else
        {
            throw new SecurityEngineException("File: '" + filePath + "' - not found");
        }
        return result;
    }

    /**
     * @param string
     * @param algorithmForKey
     * @return
     * @throws IOException
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeySpecException
     */
    public static PublicKey toPublicKey(final String string, final String algorithmForKey)
        throws IOException,NoSuchAlgorithmException,InvalidKeySpecException
    {
        final byte array[] = Base64.decode(string);
        final X509EncodedKeySpec keySpec = new X509EncodedKeySpec(array);
        final KeyFactory fact = KeyFactory.getInstance(algorithmForKey);
        final PublicKey key = fact.generatePublic(keySpec);
        return key;
    }

    /**
     * @param timeInMilliSecond
     * @return
     */
    public static XMLGregorianCalendar toXMLGregorianCalendar(final long timeInMilliSecond)
    {
        final GregorianCalendar gc = new GregorianCalendar();
        gc.setTimeInMillis(timeInMilliSecond);
        XMLGregorianCalendar date2 = null;
        try
        {
            date2 = DatatypeFactory.newInstance().newXMLGregorianCalendar(gc);
        }
        catch (final DatatypeConfigurationException e)
        {
            e.printStackTrace();
        }
        return date2;
    }

    public static void writeBytesToFile(final byte[] data, final String filePath)
        throws SecurityEngineException
    {
        final File file = new File(filePath);
        if (file.exists())
        {
            FileAndObjectHandler.deleteFile(filePath);
        }
        FileOutputStream out = null;
        try
        {
            out = new FileOutputStream(file);
            out.write(data);
            out.flush();
            out.close();
        }
        catch (final FileNotFoundException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
    }

    public static void writeObjectToFile(final String filePath, final Object object)
        throws SecurityEngineException
    {
        FileOutputStream fos = null;
        ObjectOutputStream oos = null;
        final File file = new File(filePath);
        if (file.exists())
        {
            FileAndObjectHandler.deleteFile(filePath);
        }
        try
        {
            fos = new FileOutputStream(file);
            oos = new ObjectOutputStream(fos);
            oos.writeObject(object);
            oos.flush();
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        finally
        {
            if (oos != null)
            {
                try
                {
                    oos.close();
                }
                catch (final IOException e)
                {
                    throw new SecurityEngineException(e.getMessage());
                }
            }
            if (fos != null)
            {
                try
                {
                    fos.close();
                }
                catch (final IOException e)
                {
                    throw new SecurityEngineException(e.getMessage());
                }
            }
        }
    }
    
    public static Object fromString(String s) throws IOException,ClassNotFoundException
    {
        byte[] data = Base64.decode(s);
        ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(data));
        Object o = ois.readObject();
        ois.close();
        return o;
    }

    /** Write the object to a Base64 string. */
    public static String toString(Serializable o) throws IOException
    {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ObjectOutputStream oos = new ObjectOutputStream(baos);
        oos.writeObject(o);
        oos.close();
        return new String(Base64.encode(baos.toByteArray()));
    }
}
