package org.iso.mpeg.securityTE.test;


import java.io.File;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.SignatureException;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.util.Date;

import javax.security.cert.CertificateException;
import javax.security.cert.X509Certificate;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;

import junit.framework.TestCase;
import morpho.util.Converter;

import org.apache.log4j.Logger;
import org.bouncycastle.util.encoders.Base64;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.securityTE.schema.SecurityEngineContext;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager;


public class CertificateManagerTest extends TestCase
{
    private CertificateManager certMngr        = null;
    private final int          expiryDays      = 10;
    private final String       holderReference = "CN=Holder,L=London,ST=State,OU=Organizational Unit,C=GB,O=Organization,STREET=Street,DC=lmu,UID=HolderID";
    private final String       issuerReference = "CN=Issuer,L=London,ST=State,OU=Organizational Unit,C=GB,O=Organization,STREET=Street,DC=lmu,UID=IssuerID";
    private JAXBContext        jc              = null;
    private final int          keyLength       = 2048;
    private final Logger       logger          = Logger
                                                   .getLogger(CertificateManagerTest.class);
    private Marshaller         m               = null;
    private final BigInteger   PubExp          = new BigInteger("10001", 16);
    private SecurityEngine     securityTE      = null;
    private final BigInteger   serialNumber    = new BigInteger("1");
    private String APP_ALIAS   = "1";
    private String APP_ALIAS1   = "2";

    @Override
    protected void setUp() throws Exception
    {
        super.setUp();
        // Initialize JAXB Context.
        this.jc = SecurityEngineContext.getInstance().getJAXBContext();
        this.m = this.jc.createMarshaller();
        this.m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
        this.m.setProperty("com.sun.xml.bind.namespacePrefixMapper",
            SchemaNamespacePrefixMapperImpl.getInstance());
        // Initialize MXM.
        this.logger.info("\n--- Setup MXMConfiguration ---");
        final File f = new File(this.getClass().getClassLoader()
            .getResource("MXMConfiguration.xml").getFile());
        final MXM mxm = org.iso.mpeg.mxm.core.MXM.getInstance(f);
        mxm.listEnginesAndProperties();
        this.logger.info("\n--- Init SecurityEngine ---");
        this.securityTE = (SecurityEngine)mxm.getDefaultEngine(MXMEngineName.SecurityTE
            .toString());
        this.securityTE.setProperty("conf", "223456");
        this.securityTE.setProperty("auth", "323456");
        this.securityTE.setProperty("sig", "12345678");
        this.securityTE.setProperty("gakp", "12345678");
        this.logger.info("\n--- Done! ---");
        // TODO replace the Mngr with hard coded input values from testSec.xml
        this.certMngr = this.securityTE.getSecurityEngineTechnologyHandler()
            .getCertificateManager();
    }

    private KeyPair generateKeyPair(int length)
    {
        KeyPairGenerator kpg = null;
        try
        {
            kpg = KeyPairGenerator.getInstance("RSA");
        }
        catch (NoSuchAlgorithmException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        kpg.initialize(length);
        KeyPair kp = kpg.genKeyPair();
        Key publicKey = kp.getPublic();
        Key privateKey = kp.getPrivate();
        try
        {
            KeyFactory fact = KeyFactory.getInstance("RSA");
            RSAPublicKeySpec pub = fact.getKeySpec(publicKey, RSAPublicKeySpec.class);
            RSAPrivateKeySpec priv = fact.getKeySpec(privateKey, RSAPrivateKeySpec.class);
            return kp;
        }
        catch (Exception e)
        {
            System.out.println(e.getMessage());
        }
        return null;
    }

    
    public void testCertificateManager() throws MXMConfigurationException,
        MXMEngineException
    {
        System.out.println("----------TESTING DEFAULT CERTIFICATES--------------");
        //todo: check the default certificates
        System.out.println("------TEST GENERATE KEY PAIR-------------------");
        PublicKey authKey = this.certMngr.generateRSAKeyPair(APP_ALIAS, "auth", 2048);
        PublicKey authKey1 = this.certMngr.generateRSAKeyPair(APP_ALIAS1, "auth", 2048);
        org.iso.mpeg.mxm.securityTE.technologyhandler.KeyManager keyManager= (org.iso.mpeg.mxm.securityTE.technologyhandler.KeyManager)this.securityTE.getSecurityEngineTechnologyHandler().getKeyManager();
        
        PublicKey key = keyManager.getPublicKey("auth", APP_ALIAS, "PIN");
        PublicKey key1 = keyManager.getPublicKey("auth", APP_ALIAS1, "PIN");
        
        System.out.println(new String(Base64.encode(key.getEncoded())));
        System.out.println(new String(Base64.encode(authKey.getEncoded())));
        assertEquals(new String(Base64.encode(key.getEncoded())), new String(Base64.encode(authKey.getEncoded())));
        
        System.out.println(new String(Base64.encode(key1.getEncoded())));
        System.out.println(new String(Base64.encode(authKey1.getEncoded())));
        assertEquals(new String(Base64.encode(key1.getEncoded())), new String(Base64.encode(authKey1.getEncoded())));
        
        try
        {
            this.logger.info("\n---------- testCertificateManager ----------");
            this.logger.info("\n--- GenerateRSAKeyPair ---");
            
            final KeyPair cardPair = this.generateKeyPair(this.keyLength);
            this.logger.info("\nkeyLength: " + this.keyLength + "\nPubExp: "
                + this.PubExp);
             //TODO use card for the following, if enabled in the settings.
            final X509Data x509Data = this.certMngr.generateCertificate(serialNumber, holderReference, issuerReference, expiryDays, cardPair.getPublic(),cardPair.getPrivate(), "SHA256withRSAEncryption");
            final X509Certificate x509Cert = X509Certificate.getInstance(x509Data
                .getX509Certificate());
            this.logger.info("\nGenerated X509Certificate String Representation:"
                + x509Cert.toString());
            x509Cert.checkValidity(new Date());
            x509Cert.verify(cardPair.getPublic());
            
        }
        catch (final CertificateException e)
        {
            this.logger.error(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            this.logger.error(e.getMessage());
        }
        catch (final NoSuchAlgorithmException e)
        {
            this.logger.error(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            this.logger.error(e.getMessage());
        }
        catch (final SignatureException e)
        {
            this.logger.error(e.getMessage());
        }
    }
}
