/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.securityTE.test;


import java.io.File;
import java.io.FileNotFoundException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.securityTE.schema.SecurityEngineContext;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.KeyInfo;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;


public class SecurityEngineTest extends TestCase
{
    private JAXBContext  jc                   = null;
    private final Logger logger               = Logger
                                                  .getLogger(SecurityEngineTest.class);
    private Marshaller   m                    = null;
    private final String OUTPUT_SEC_FILE_NAME = this.TEMP_FOLDER + "mytestSecurity.xml";
    private final String SOURCE_SEC_FILE_NAME = "testSec.xml";
    private final String TEMP_FOLDER          = System.getProperty("java.io.tmpdir")
                                                  + File.separator;

    @Override
    protected void setUp() throws Exception
    {
        super.setUp();
        this.jc = SecurityEngineContext.getInstance().getJAXBContext();
        this.m = this.jc.createMarshaller();
        this.m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
        this.m.setProperty("com.sun.xml.bind.namespacePrefixMapper",
            SchemaNamespacePrefixMapperImpl.getInstance());
    }

    public void testDCICreation() throws MXMConfigurationException,MXMEngineException
    {
        this.logger.info("\n\n---------- testDCICreation ----------");
        final File f = new File(this.getClass().getClassLoader()
            .getResource("MXMConfiguration.xml").getFile());
        final MXM mxm = MXM.getInstance(f);
        mxm.listEnginesAndProperties();
        SecurityEngine secEngine = (SecurityEngine)mxm
            .getDefaultEngine(MXMEngineName.SecurityTE.toString());
        secEngine.setProperty("conf", "223456");
        secEngine.setProperty("auth", "323456");
        secEngine.setProperty("sig", "123456");
        secEngine.setProperty("gakp", "12345678");
        secEngine.getSecurityEngineSchemaHandler().getKeyInfo();
        secEngine.getSecurityEngineSchemaHandler().getKeyValue();
        // KeyValue kv= null;
        // kv.setRSA("1234".getBytes(), "1234".getBytes());
        final Signature s = secEngine.getSecurityEngineSchemaHandler().getSignature(); // ki.setKeyName("myname");
        // ki.setKeyValue(kv);
        try
        {
            s.createFile(new File(this.OUTPUT_SEC_FILE_NAME));
        }
        catch (final MXMException e)
        {
            this.logger.error(e.getMessage());
        }
    }

    public void testSecurityParsing() throws MXMConfigurationException,
        FileNotFoundException,JAXBException,MXMEngineException
    {
        this.logger.info("\n\n---------- testSecParsing ----------");
        final File f = new File(this.getClass().getClassLoader()
            .getResource("MXMConfiguration.xml").getFile());
        final MXM mxm = MXM.getInstance(f);
        mxm.listEnginesAndProperties();
        final SecurityEngine secEngine = (SecurityEngine)mxm
            .getDefaultEngine(MXMEngineName.SecurityTE.toString());
        final KeyInfo ki = secEngine.getSecurityEngineSchemaHandler().getKeyInfo();
        // KeyInfo ki = null;
        final MXMAdapter ad = new MXMAdapter(new File(this.getClass().getClassLoader()
            .getResource(this.SOURCE_SEC_FILE_NAME).getFile()));
        ki.parse(ad);
        // System.out.println("name is:" + ki.getKeyName());
        // System.out.println("value is (exp):" + ki.getKeyValue().getRSAExponent());
        // System.out.println("value is (mod):" + ki.getKeyValue().getRSAModulus());
    }
}