package org.iso.mpeg.securityTE.test;


import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPublicKeySpec;
import java.util.Date;
import java.util.List;
import java.util.Random;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;
import morpho.util.Converter;

import org.apache.log4j.Logger;
import org.apache.xerces.impl.dv.util.Base64;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.securityTE.schema.SecurityEngineContext;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.DigestMethod;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Reference;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Transforms;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager;
import org.xml.sax.SAXException;

import com.sun.xml.txw2.Document;


public class SignatureManagerTest extends TestCase
{
    public class TextReference implements
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Reference
    {
        private String uri;

        @Override
        public void createFile(final File f) throws MXMException
        {
        }

        @Override
        public MXMObject createObject(final String objectType) throws MXMException
        {
            // TODO Auto-generated method stub
            return null;
        }

        @Override
        public String get()
        {
            return null;
        }

        @Override
        public DigestMethod getDigestMethod()
        {
            return null;
        }

        @Override
        public byte[] getDigestValue()
        {
            return null;
        }

        @Override
        public String getId()
        {
            return null;
        }

        @Override
        public Transforms getTransforms()
        {
            return null;
        }

        @Override
        public String getURI()
        {
            return this.uri;
        }

        @Override
        public MXMEngineResponse parse(final MXMObject object) throws MXMEngineException
        {
            return null;
        }

        @Override
        public void set(final String string)
        {
        }

        @Override
        public void setDigestMethod(final DigestMethod digestMethod)
        {
        }

        @Override
        public void setDigestValue(final byte[] b)
        {
        }

        @Override
        public void setId(final String string)
        {
        }

        @Override
        public void setTransforms(final Transforms transforms)
        {
        }

        @Override
        public void setURI(final String string)
        {
            this.uri = string;
        }
    }

    private CertificateManager                                                            certMngr        = null;
    private final String                                                                  hashAlgorithm   = "SHA-256";
    private JAXBContext                                                                   jc              = null;
    private final int                                                                     keyLength       = 2048;
    private final Logger                                                                  logger          = Logger
                                                                                                              .getLogger(CertificateManagerTest.class);
    private Marshaller                                                                    m               = null;
    private final BigInteger                                                              PubExp          = new BigInteger(
                                                                                                              "10001",
                                                                                                              16);
    private SecurityEngine                                                                securityTE      = null;
    private final String                                                                  signAlgorithm   = "SHA256withRSAandMGF1";
    private SignatureManager                                                              signMngr        = null;
    private final String                                                                  verifyAlgortihm = "SHA256withRSA/PSS";
    
    private String APP_ALIAS = "1";
    private String getString(final Key key) throws IOException
    {
        final byte array[] = key.getEncoded();
        return Base64.encode(array);
    }

    @Override
    protected void setUp() throws Exception
    {
        super.setUp();
        // initialize the repomanager by adding certificates
        // Initialize JAXB Context.
        this.jc = SecurityEngineContext.getInstance().getJAXBContext();
        this.m = this.jc.createMarshaller();
        this.m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
        this.m.setProperty("com.sun.xml.bind.namespacePrefixMapper",
            SchemaNamespacePrefixMapperImpl.getInstance());
        // Initialize MXM.
        this.logger.info("\n--- Setup MXMConfiguration ---");
        final File f = new File(this.getClass().getClassLoader()
            .getResource("MXMConfiguration.xml").getFile());
        final MXM mxm = org.iso.mpeg.mxm.core.MXM.getInstance(f);
        mxm.listEnginesAndProperties();
        this.logger.info("\n--- Init SecurityEngine ---");
        this.securityTE = (SecurityEngine)mxm.getDefaultEngine(MXMEngineName.SecurityTE
            .toString());
        this.securityTE.setProperty("conf", "223456");
        this.securityTE.setProperty("auth", "323456");
        this.securityTE.setProperty("sig", "123456");
        this.securityTE.setProperty("gakp", "12345678");
        this.logger.info("\n--- Done! ---");
        // TODO replace the Mngr with hard coded input values from testSec.xml
        this.certMngr = this.securityTE.getSecurityEngineTechnologyHandler()
            .getCertificateManager();
        this.signMngr = this.securityTE.getSecurityEngineTechnologyHandler()
            .getSignatureManager();
    }

    public void testSignatureManager() throws MXMConfigurationException,
        MXMEngineException
    {
        this.securityTE.getSecurityEngineTechnologyHandler().getSignatureManager();
        org.iso.mpeg.mxm.securityTE.technologyhandler.KeyManager keyManager= (org.iso.mpeg.mxm.securityTE.technologyhandler.KeyManager)this.securityTE.getSecurityEngineTechnologyHandler().getKeyManager();
        if (this.securityTE.getPropertyValue("enableSmartCard").equals("false"))
        {
            System.out.println("CHECKING IF WE HAVE DEFAULT 3 KEY PARIS ");
            
            PublicKey authKey = keyManager.getPublicKey("auth", null, "PIN");
            PublicKey confKey = keyManager.getPublicKey("conf", null, "PIN");
            PublicKey sigKey = keyManager.getPublicKey("sig", null, "PIN");
            assertNotNull(authKey);
            assertNotNull(confKey);
            assertNotNull(sigKey);
            PublicKey sigKey1 = keyManager.getPublicKey("sig", APP_ALIAS, "PIN");
            if(sigKey1 == null) {
                sigKey1 = this.certMngr.generateRSAKeyPair(APP_ALIAS, "sig", 2048);
            }
        }
       this.logger.info("\n---------- testSignatureManager ----------");
        this.logger.info("\n--- Generating a random plaintext message ---");
        final Date date = new Date();
        final long time = date.getTime();
        final Random r = new Random(time);
        int random = (int)Math.round(Math.random() * 9999);
        String challange = random + "111111110000fdgdsfgsdgsdgsdf";
        challange = challange + challange;
        final byte[] plainText = challange.getBytes();
       ;
        this.logger.info("\nGenerated plaintext: " + Converter.toHex(plainText));
        //final PublicKey publicKey = this.certMngr
        //    .generateRSAKeyPair("myalias1", "sig", 2048, "PIN");
       // final PublicKey pubKey = this.signMngr.ge
        this.logger.info("\n--- generateSignature ---");
    
    
        Signature signedMessage = null;
        MessageDigest md =  null;
        try
        {
            md = MessageDigest.getInstance("SHA-256", "BC");
        }
        catch (NoSuchAlgorithmException e1)
        {
            e1.printStackTrace();
        }
        catch (NoSuchProviderException e1)
        {
            e1.printStackTrace();
        }
        md.update(plainText);
        byte[] messageHash = md.digest();
        signedMessage = this.signMngr.generateSignature(plainText,"1");
        String str = this.securityTE.getSecurityEngineTechnologyHandler().getAuthenticationManager().clientAuth(challange,"1","");
        this.logger.info("\n--- ckeck public key of signature ---");
        RSAPublicKeySpec rsaPubKeySpec = new RSAPublicKeySpec(new BigInteger(
            signedMessage.getKeyInfo().getKeyValue().getRSAKeyValue().getModulus()),
            new BigInteger(signedMessage.getKeyInfo().getKeyValue().getRSAKeyValue()
                .getExponent()));
        try
        {
            KeyFactory keyFactory = KeyFactory.getInstance("RSA", "BC");
            RSAPublicKey rsaPublicKey = (RSAPublicKey)keyFactory.generatePublic(rsaPubKeySpec);
            //getting key from device
            PublicKey keyFromDevice = keyManager.getPublicKey("sig", "1", "PIN");
            System.out.println(new String(Base64.encode(rsaPublicKey.getEncoded())));
            System.out.println(new String(Base64.encode(keyFromDevice.getEncoded())));
            assertEquals(new String(Base64.encode(rsaPublicKey.getEncoded())), new String(Base64.encode(keyFromDevice.getEncoded())));
        }
        catch (final InvalidKeySpecException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        
        
        
        this.logger.info("\nGENERATED SIGNATURE: "
            + Converter.toHex(signedMessage.getSignatureValue().getValue()));
        this.logger.info("\n--- verifySignature ---");
        System.out.println("VERIFED:"+this.signMngr.verifySignature(plainText, signedMessage));
        assertTrue(this.signMngr.verifySignature(plainText, signedMessage));
        this.logger.info("\nSuccess!");
        this.logger.info("\n--- generateHash ---");
        final byte[] hash = this.signMngr.generateHash(plainText, this.hashAlgorithm);
        this.logger.info("\nHash: " + Converter.toHex(hash));
        this.logger.info("\n--- verifyHash ---");
        assertTrue(this.signMngr.verifyHash(plainText, hash, this.hashAlgorithm));
        this.logger.info("\nSuccess!");
    }

    public void testSignatureManagerViaDoc() {
        this.logger.info("\n--- signing and verifying  xml document ---");
        final File f = new File(this.getClass().getClassLoader()
          .getResource("testSec.xml").getFile());
        DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder;
        try
        {
            docBuilder = dbfac.newDocumentBuilder();
            org.w3c.dom.Document doc = docBuilder.parse(f.getAbsoluteFile());
            Signature sign = this.signMngr.generateSignature(doc, "1");
            boolean verified = this.signMngr.verifySignature(doc, sign);
            assertTrue(verified);
        }
        catch (ParserConfigurationException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (SAXException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (SecurityEngineException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (MXMEngineException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    
        
    }
    
    public void testSignatureManagerViaURI() throws MXMConfigurationException,
        MXMEngineException
    {
//        this.logger.info("\n---------- testSignatureManager ----------");
//        this.logger.info("\n--- Generating a random plaintext message ---");
//        new Date();
//        final File f = new File(this.getClass().getClassLoader()
//            .getResource("testSignatureManager.xml").getFile());
//        final String uri = f.getAbsolutePath();
//        final String ref = "//person/*/text()";
//        final Reference inputUri = new TextReference();
//        inputUri.setURI(uri);
//        final Reference inputRef = new TextReference();
//        inputRef.setURI(ref);
//        this.certMngr.generateRSAKeyPair(this.keyLength, this.PubExp);
//        this.logger.info("\n--- generateSignature ---");
//        // final Signature signedMessage = this.signMngr.generateSignature(
//        // inputUri, inputRef,this.signAlgorithm, pair, "");
//        // this.logger
//        // .info("\nGenerated Signature: "
//        // + Converter.toHex(signedMessage.getSignatureValue()
//        // .getValue()));
//        // this.logger.info("\n--- verifySignature ---");
//        // assertTrue(this.signMngr.verifySignature(inputUri, inputRef, signedMessage,
//        // this.verifyAlgortihm, pair.getPublic()));
//        this.logger.info("\nSuccess!");
    }
}
