/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.storelicensePE.schema;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolRequest;
import org.iso.mpeg.mxm.baseprotocol.schema.ContentEntityType;
import org.iso.mpeg.mxm.baseprotocol.schema.LicenseEntityType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.storelicensePE.schema.storelicenseprotocol.StoreLicenseRequestType;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;

/**
 * @author Jonathan Florido, DMAG-UPC <jflorido@ac.upc.edu>
 *
 */
public class StoreLicenseRequest extends ProtocolRequest implements org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseRequest
{
	private BaseProtocolEngine baseProtocolEngine;
	private RELEngine relEngine;
	private MetadataEngine metadataEngine;
	
	public StoreLicenseRequest(BaseProtocolEngine baseProtocolEngine, SecurityEngine securityEngine, RELEngine relEngine, MetadataEngine metadataEngine)
	{
		super(new StoreLicenseRequestType(), securityEngine);
		this.baseProtocolEngine = baseProtocolEngine;
		this.relEngine = relEngine;
		this.metadataEngine = metadataEngine;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#storeFile(java.io.File)
	 */
	@Override
	public void createFile(File f) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((StoreLicenseRequestType)protocolBaseType);
		level1.createFile(f);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMCreator#storeObject(java.lang.String)
	 */
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((StoreLicenseRequestType)protocolBaseType);
		return level1.createObject(objectType);
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.core.MXMParser#parse(org.iso.mpeg.mxm.core.MXMObject)
	 */
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((StoreLicenseRequestType)protocolBaseType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolBaseType = 
					(StoreLicenseRequestType)(level1.createObject(StoreLicenseRequestType.class.getName()).getContent());
		}
		return response;
	}

	@Override
	public LicenseEntity getLicenseEntity() throws MXMEngineException {
		LicenseEntityType licenseEntityType = ((StoreLicenseRequestType)protocolBaseType).getLicenseEntity();
		LicenseEntity licenseEntity = new org.iso.mpeg.mxm.basePE.LicenseEntity(relEngine, metadataEngine);
		licenseEntity.parse(new MXMAdapter(licenseEntityType));
		return licenseEntity;
	}

	@Override
	public void setLicenseEntity(LicenseEntity licenseEntity) throws MXMException {
		LicenseEntityType licenseEntityType = (LicenseEntityType)(licenseEntity.createObject(LicenseEntityType.class.getName()).getContent());
		((StoreLicenseRequestType)protocolBaseType).setLicenseEntity(licenseEntityType);
	}

	@Override
	public ContentEntity getContentEntity() throws MXMEngineException {
		ContentEntityType contentEntityType = ((StoreLicenseRequestType)protocolBaseType).getContentEntity();
		ContentEntity contentEntity = baseProtocolEngine.getBaseProtocolSchemaHandler().getContentEntity();
		contentEntity.parse(new MXMAdapter(contentEntityType));
		return contentEntity;
	}

	@Override
	public void setContentEntity(ContentEntity contentEntity) throws MXMException {
		ContentEntityType contentEntityType = (ContentEntityType)(contentEntity.createObject(ContentEntityType.class.getName()).getContent());
		((StoreLicenseRequestType)protocolBaseType).setContentEntity(contentEntityType);
	}
}
