/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The licenses of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Angelo Difino
 * Contributor(s): 
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.storelicensePE.schema;

import java.io.File;

import org.iso.mpeg.mxm.basePE.ProtocolSuccess;
import org.iso.mpeg.mxm.baseprotocol.schema.LicenseEntityType;
import org.iso.mpeg.mxm.baseprotocol.schema.SuperBaseProtocolAbstractLevel1;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.storelicensePE.schema.storelicenseprotocol.StoreLicenseSuccessType;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.LicenseEntity;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;

/**
 * @author Jonathan Florido, DMAG-UPC <jflorido@ac.upc.edu>
 *
 */
public class StoreLicenseSuccess extends ProtocolSuccess implements org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseSuccess
{
	//private BaseProtocolEngine baseProtocolEngine;
	private RELEngine relEngine;
	private MetadataEngine metadataEngine;
	
	public StoreLicenseSuccess(RELEngine relEngine, MetadataEngine metadataEngine) {
		super(new StoreLicenseSuccessType());
		this.relEngine = relEngine;
		this.metadataEngine = metadataEngine;
	}

	@Override
	public void createFile(File f) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1((StoreLicenseSuccessType)protocolSuccessType);
		level1.createFile(f);
	}
	
	@Override
	public MXMObject createObject(String objectType) throws MXMException
	{
		SuperBaseProtocolAbstractLevel1 level1 = new SuperBaseProtocolAbstractLevel1((StoreLicenseSuccessType)protocolSuccessType);
		return level1.createObject(objectType);
	}
	
	@Override
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException
	{
		SuperProtocolEngineAbstractLevel1 level1 = new SuperProtocolEngineAbstractLevel1((StoreLicenseSuccessType)protocolSuccessType);
		MXMEngineResponse response = level1.parse(object);
		if (response.equals(MXMEngineResponse.SUCCESS))
		{
			protocolSuccessType = 
					(StoreLicenseSuccessType)(level1.createObject(StoreLicenseSuccessType.class.getName()).getContent());
		}
		return response;
	}

	@Override
	public LicenseEntity getLicenseEntity() throws MXMException {
		LicenseEntityType licenseEntityType = ((StoreLicenseSuccessType)protocolSuccessType).getLicenseEntity();
		LicenseEntity licenseEntity = new org.iso.mpeg.mxm.basePE.LicenseEntity(relEngine, metadataEngine);
		licenseEntity.parse(new MXMAdapter(licenseEntityType));
		return licenseEntity;
	}

	@Override
	public void setLicenseEntity(LicenseEntity licenseEntity)
			throws MXMException {
		LicenseEntityType licenseEntityType = (LicenseEntityType)(licenseEntity.createObject(LicenseEntityType.class.getName()).getContent());
		((StoreLicenseSuccessType)protocolSuccessType).setLicenseEntity(licenseEntityType);		
	}
}
