/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.storelicensePE.technology.remote;

import java.net.URL;

import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.ServiceFactory;

import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.es.storelicenseES.StoreLicenseProtocolRemote;
import org.iso.mpeg.mxm.pEngine.basePE.BaseProtocolEngine;
import org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseRequest;
import org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseResponse;
import org.iso.mpeg.mxm.tEngine.metadataTE.MetadataEngine;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Jonathan Florido, DMAG-UPC <jflorido@ac.upc.edu>
 *
 */
public class StoreLicenseProtocol implements org.iso.mpeg.mxm.pEngine.storelicensePE.technologyhandler.StoreLicenseProtocol
{
	private URL serviceURL;
	private QName serviceName;
	
	private Logger logger;
	
	private BaseProtocolEngine baseProtocolEngine;
	private SecurityEngine securityEngine;
	private RELEngine relEngine;
	private MetadataEngine metadataEngine;
	
	public StoreLicenseProtocol(BaseProtocolEngine baseProtocolEngine, SecurityEngine securityEngine, RELEngine relEngine, MetadataEngine metadataEngine)
	{
		this.baseProtocolEngine = baseProtocolEngine;
		this.securityEngine = securityEngine;
		this.relEngine = relEngine;
		this.metadataEngine = metadataEngine;
		
		logger = LoggerFactory.getLogger(this.getClass());
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.storelicensePE.technologyhandler.StoreLicenseProtocol#initServiceParams(java.net.URL, javax.xml.namespace.QName)
	 */
	@Override
	public MXMEngineResponse initServiceParams(URL serviceURL, QName serviceName)
	{
		this.serviceName = serviceName;
		this.serviceURL = serviceURL;
		return MXMEngineResponse.SUCCESS;
	}

	/* (non-Javadoc)
	 * @see org.iso.mpeg.mxm.pEngine.storelicensePE.technologyhandler.StoreLicenseProtocol#storeLicense(org.iso.mpeg.mxm.pEngine.storelicensePE.schemahandler.StoreLicenseRequest)
	 */
	@Override
	public StoreLicenseResponse storeLicense(StoreLicenseRequest request)
	{
		if (serviceURL == null)
		{
			logger.warn("Service URL has not been set");
		}
		if (serviceName == null)
		{
			logger.warn("Service Name has not been set");
		}
		ServiceFactory serviceFactory = null;
		
		try
		{
			serviceFactory = ServiceFactory.newInstance();
			Service service = serviceFactory.createService(serviceURL, serviceName);
			
			String serviceRequest = (String)(request.createObject(String.class.getCanonicalName()).getContent());
			StoreLicenseProtocolRemote storeLicenseProtocolRemote = (StoreLicenseProtocolRemote)
					service.getPort(StoreLicenseProtocolRemote.class);
			String serviceResponse = storeLicenseProtocolRemote.storeLicense(serviceRequest);
			StoreLicenseResponse storeLicenseResponse = 
					new org.iso.mpeg.mxm.storelicensePE.schema.StoreLicenseResponse(baseProtocolEngine, securityEngine, relEngine, metadataEngine);
			storeLicenseResponse.parse(new MXMAdapter(serviceResponse));
			
			return storeLicenseResponse;
		} catch (ServiceException e)
		{
			e.printStackTrace();
		} catch (MXMException e)
		{
			e.printStackTrace();
		}
		return null;
	}

}
