/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.es.CreateLicenseES.service;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateless;
import javax.jws.WebService;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine;
import org.iso.mpeg.mxm.pEngine.createlicensePE.CreateLicenseEngine;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseRequest;
import org.iso.mpeg.mxm.pEngine.createlicensePE.schemahandler.CreateLicenseResponse;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Angelos-Christos Anadiotis
 *
 */
@Stateless
@WebService(endpointInterface = "org.iso.mpeg.mxm.es.createlicenseES.CreateLicenseProtocolRemote")
public class CreateLicenseES
{
	public String createLicense(String request) throws MXMConfigurationException, MXMEngineException
	{
		InputStream is = getClass().getClassLoader().getResourceAsStream("MXMConfiguration.xml");
		MXM mxm = MXM.getInstance(is);
		CreateLicenseEngine createLicenseEngine = 
				(org.iso.mpeg.mxm.pEngine.createlicensePE.CreateLicenseEngine) mxm.getDefaultEngine(MXMEngineName.CreateLicensePE.value());
		RELEngine relEngine = 
				(org.iso.mpeg.mxm.tEngine.relTE.RELEngine) mxm.getDefaultEngine(MXMEngineName.RELTE.value());
		
		ApplicationContext applicationContext = new ClassPathXmlApplicationContext("beans.xml");
		OrchestratorEngine orchestratorEngine = (OrchestratorEngine) applicationContext.getBean("OrchestratorEngine");
		
		List<MXMEngine> engines = new ArrayList<MXMEngine>();
		engines.add(relEngine);
		engines.add(orchestratorEngine);
		createLicenseEngine.setDependencies(engines);
		engines.clear();
		
		engines.add(relEngine);
		engines.add(createLicenseEngine);
		orchestratorEngine.setDependencies(engines);
		CreateLicenseRequest createLicenseRequest = createLicenseEngine.getCreateLicenseSchemaHandler().getCreateLicenseRequest();
		
		try
		{
			createLicenseRequest.parse(new MXMAdapter(request));
//			CreateLicenseProtocol createLicenseProtocol = createLicenseEngine.getCreateLicenseTechnologyHandler().getLocalCreateLicenseProtocol();
			CreateLicenseResponse createLicenseResponse = orchestratorEngine.getCreateLicenseEngineOrchestrator().createLicense(createLicenseRequest);
			
			String response = (String)(createLicenseResponse.createObject(String.class.getCanonicalName()).getContent());
			
			return response;
		} catch (MXMEngineException e)
		{
			e.printStackTrace();
		} catch (MXMException e)
		{
			e.printStackTrace();
		}
		return null;
	}
}
