/** ***** BEGIN LICENSE BLOCK *****
 * 
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, Cedeo.net
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the Cedeo.net nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
***** END LICENSE BLOCK ***** */

package org.iso.mpeg.es.IdentifyContentES.service;

import java.io.InputStream;

import javax.ejb.Stateless;
import javax.jws.WebService;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
import org.iso.mpeg.mxm.identifycontentPE.IdentifyContentEngine;
import org.iso.mpeg.mxm.orchestratorEngine.OrchestratorEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.exceptions.CreateContentEngineException;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.exceptions.IdentifyContentEngineException;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.schemahandler.IdentifyContentRequest;
import org.iso.mpeg.mxm.pEngine.identifycontentPE.schemahandler.IdentifyContentResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Angelo Difino <angelo@cedeo.net>
 *
 */
@Stateless						 
@WebService(endpointInterface = "org.iso.mpeg.mxm.es.identifycontentES.IdentifyContentProtocolRemote")
public class IdentifyContentES 
{
	Logger logger = LoggerFactory.getLogger(this.getClass());
	
	IdentifyContentEngine identifyContentEngine;
	OrchestratorEngine orchestratorEngine;
	MXM mxm=null;
	
	private void init()
	{
		logger.info("identifyContentES created");
		InputStream is = getClass().getClassLoader().getResourceAsStream("MXMConfiguration.xml");
		try {
			mxm = MXM.getInstance(is);
			 identifyContentEngine = (IdentifyContentEngine) mxm.getDefaultEngine(MXMEngineName.IdentifyContentPE.value());
			 orchestratorEngine = (OrchestratorEngine)mxm.getDefaultEngine(MXMEngineName.Orchestrator.value());
		} catch (MXMConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MXMEngineException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public String identifyContent(String request) throws IdentifyContentEngineException 
	{
		if (mxm==null) init();
		
		try
		{		
			logger.info("identifyContentES called");
			logger.info("------------------------------------------------------------------------");
			logger.info("REQUEST is:"+request);
			logger.info("------------------------------------------------------------------------");
			
			IdentifyContentRequest icRequest = identifyContentEngine.getSchemaHandler().getIdentifyContentRequest();
			
			icRequest.parse(new MXMAdapter(request));
			IdentifyContentResponse icResponse =
					orchestratorEngine.getIdentifyContentEngineOrchestrator().identifyContent(icRequest);
				
			String response = (String) (icResponse.createObject(String.class.getCanonicalName()).getContent());
			
			logger.info("------------------------------------------------------------------------");
			logger.info("RESPONSE is:"+response);
			logger.info("------------------------------------------------------------------------");
			logger.info("identifyContentES ended");			
			
			return response;
		}
		catch (MXMConfigurationException e)
		{
			throw new IdentifyContentEngineException(e.getMessage(),e);
		} 
		catch (MXMEngineNotFoundException e)
		{
			throw new IdentifyContentEngineException(e.getMessage(), e);
		} 		
		catch (MXMEngineException e)
		{
			throw new IdentifyContentEngineException(e.getMessage(), e);
		}
		catch (MXMException e)
		{
			throw new IdentifyContentEngineException(e.getMessage(), e);			
		}
		catch (Exception e)
		{
			throw new IdentifyContentEngineException(e.getMessage(), e);
		}
	}
}